'use strict';function ba(a){var b=0;return function(){return b<a.length?{done:!1,value:a[b++]}:{done:!0}}}var da="function"==typeof Object.defineProperties?Object.defineProperty:function(a,b,c){if(a==Array.prototype||a==Object.prototype)return a;a[b]=c.value;return a};
function fa(a){a=["object"==typeof globalThis&&globalThis,a,"object"==typeof window&&window,"object"==typeof self&&self,"object"==typeof global&&global];for(var b=0;b<a.length;++b){var c=a[b];if(c&&c.Math==Math)return c}throw Error("Cannot find global object");}var ja=fa(this);function ma(a,b){if(b)a:{var c=ja;a=a.split(".");for(var d=0;d<a.length-1;d++){var e=a[d];if(!(e in c))break a;c=c[e]}a=a[a.length-1];d=c[a];b=b(d);b!=d&&null!=b&&da(c,a,{configurable:!0,writable:!0,value:b})}}
ma("Symbol.asyncIterator",function(a){return a?a:Symbol("Symbol.asyncIterator")});function ra(a){function b(d){return a.next(d)}function c(d){return a.throw(d)}return new Promise(function(d,e){function f(g){g.done?d(g.value):Promise.resolve(g.value).then(b,c).then(f,e)}f(a.next())})}function sa(a){return ra(a())}
function ta(a){this[Symbol.asyncIterator]=function(){return this};this[Symbol.iterator]=function(){return a};this.next=function(b){return Promise.resolve(a.next(b))};void 0!==a["throw"]&&(this["throw"]=function(b){return Promise.resolve(a["throw"](b))});void 0!==a["return"]&&(this["return"]=function(b){return Promise.resolve(a["return"](b))})}ma("Object.values",function(a){return a?a:function(b){var c=[],d;for(d in b)Object.prototype.hasOwnProperty.call(b,d)&&c.push(b[d]);return c}});
ma("Array.prototype.includes",function(a){return a?a:function(b,c){var d=this;d instanceof String&&(d=String(d));var e=d.length;c=c||0;for(0>c&&(c=Math.max(c+e,0));c<e;c++){var f=d[c];if(f===b||Object.is(f,b))return!0}return!1}});function ua(a,b){a instanceof String&&(a+="");var c=0,d=!1,e={next:function(){if(!d&&c<a.length){var f=c++;return{value:b(f,a[f]),done:!1}}d=!0;return{done:!0,value:void 0}}};e[Symbol.iterator]=function(){return e};return e}
ma("Array.prototype.values",function(a){return a?a:function(){return ua(this,function(b,c){return c})}});ma("globalThis",function(a){return a||ja});ma("Array.prototype.flat",function(a){return a?a:function(b){b=void 0===b?1:b;var c=[];Array.prototype.forEach.call(this,function(d){Array.isArray(d)&&0<b?(d=Array.prototype.flat.call(d,b-1),c.push.apply(c,d)):c.push(d)});return c}});
ma("Object.entries",function(a){return a?a:function(b){var c=[],d;for(d in b)Object.prototype.hasOwnProperty.call(b,d)&&c.push([d,b[d]]);return c}});function va(a){a=Math.trunc(a)||0;0>a&&(a+=this.length);if(!(0>a||a>=this.length))return this[a]}ma("Array.prototype.at",function(a){return a?a:va});function wa(a){return a?a:va}ma("Int8Array.prototype.at",wa);ma("Uint8Array.prototype.at",wa);ma("Uint8ClampedArray.prototype.at",wa);ma("Int16Array.prototype.at",wa);ma("Uint16Array.prototype.at",wa);
ma("Int32Array.prototype.at",wa);ma("Uint32Array.prototype.at",wa);ma("Float32Array.prototype.at",wa);ma("Float64Array.prototype.at",wa);ma("String.prototype.at",function(a){return a?a:va});ma("String.prototype.padStart",function(a){return a?a:function(b,c){if(null==this)throw new TypeError("The 'this' value for String.prototype.padStart must not be null or undefined");b-=this.length;c=void 0!==c?String(c):" ";return(0<b&&c?c.repeat(Math.ceil(b/c.length)).substring(0,b):"")+this}});
ma("Object.fromEntries",function(a){return a?a:function(b){var c={};if(!(Symbol.iterator in b))throw new TypeError(""+b+" is not iterable");b=b[Symbol.iterator].call(b);for(var d=b.next();!d.done;d=b.next()){d=d.value;if(Object(d)!==d)throw new TypeError("iterable for fromEntries should yield objects");c[d[0]]=d[1]}return c}});/*

 Copyright The Closure Library Authors.
 SPDX-License-Identifier: Apache-2.0
*/
var xa=this||self;function ya(a){var b=typeof a;b="object"!=b?b:a?Array.isArray(a)?"array":b:"null";return"array"==b||"object"==b&&"number"==typeof a.length}function Ca(a){var b=typeof a;return"object"==b&&null!=a||"function"==b}function Da(a){return a};var Ea,Ka;a:{for(var Na=["CLOSURE_FLAGS"],Oa=xa,Ra=0;Ra<Na.length;Ra++)if(Oa=Oa[Na[Ra]],null==Oa){Ka=null;break a}Ka=Oa}var Sa=Ka&&Ka[610401301];Ea=null!=Sa?Sa:!1;var Wa;const Xa=xa.navigator;Wa=Xa?Xa.userAgentData||null:null;function Za(a){return Ea?Wa?Wa.brands.some(({brand:b})=>b&&-1!=b.indexOf(a)):!1:!1}function $a(a){var b;a:{if(b=xa.navigator)if(b=b.userAgent)break a;b=""}return-1!=b.indexOf(a)};function ab(){return Ea?!!Wa&&0<Wa.brands.length:!1}function bb(){return ab()?Za("Chromium"):($a("Chrome")||$a("CriOS"))&&!(ab()?0:$a("Edge"))||$a("Silk")};const db=Array.prototype.map?function(a,b,c){return Array.prototype.map.call(a,b,c)}:function(a,b,c){const d=a.length,e=Array(d),f="string"===typeof a?a.split(""):a;for(let g=0;g<d;g++)g in f&&(e[g]=b.call(c,f[g],g,a));return e};!$a("Android")||bb();bb();$a("Safari")&&(bb()||(ab()?0:$a("Coast"))||(ab()?0:$a("Opera"))||(ab()?0:$a("Edge"))||(ab()?Za("Microsoft Edge"):$a("Edg/"))||ab()&&Za("Opera"));class eb{}class fb{}Object.freeze(new eb);Object.freeze(new fb);/*

 Copyright Google LLC All Rights Reserved.

 Use of this source code is governed by an MIT-style license that can be
 found in the LICENSE file at https://angular.io/license
 @param {!tsickle_zone_impl_1.ZonePrivate} api
 @param {?} target
 @param {string} targetName
 @param {string} method
 @param {!Array<string>} callbacks
 @return {void}
*/
function gb(a,b,c,d,e){const f=Zone.__symbol__(d);if(!b[f]){var g=b[f]=b[d];b[d]=function(k,m,p){m&&m.prototype&&e.forEach(function(t){const w=`${c}.${d}::`+t,B=m.prototype;try{if(B.hasOwnProperty(t)){const G=a.ObjectGetOwnPropertyDescriptor(B,t);G&&G.value?(G.value=a.wrapWithCurrentZone(G.value,w),a._redefineProperty(m.prototype,t,G)):B[t]&&(B[t]=a.wrapWithCurrentZone(B[t],w))}else B[t]&&(B[t]=a.wrapWithCurrentZone(B[t],w))}catch(G){}});return g.call(b,k,m,p)};a.attachOriginToPatched(b[d],g)}};/*

 Copyright Google LLC All Rights Reserved.

 Use of this source code is governed by an MIT-style license that can be
 found in the LICENSE file at https://angular.io/license
*/
const hb=globalThis;function ib(a){return(hb.__Zone_symbol_prefix||"__zone_symbol__")+a}
function jb(){function a(F){g&&g.mark&&g.mark(F)}function b(F,M){g&&g.measure&&g.measure(F,M)}function c(F){aa||hb[B]&&(aa=hb[B].resolve(0));if(aa){let M=aa[G];M||(M=aa.then);M.call(aa,F)}else hb[w](F,0)}function d(F){0===la&&0===J.length&&c(e);F&&J.push(F)}function e(){if(!N){for(N=!0;J.length;){const F=J;J=[];for(let M=0;M<F.length;M++){const W=F[M];try{W.zone.runTask(W,null,null)}catch(Aa){O.onUnhandledError(Aa)}}}O.microtaskDrainDone();N=!1}}function f(){}const g=hb.performance;a("Zone");class k{static assertZonePatched(){if(hb.Promise!==
ia.ZoneAwarePromise)throw Error("Zone.js has detected that ZoneAwarePromise `(window|global).Promise` has been overwritten.\nMost likely cause is that a Promise polyfill has been loaded after Zone.js (Polyfilling Promise api is not necessary when zone.js is loaded. If you must load one, do so before loading zone.js.)");}static get root(){let F=k.current;for(;F.parent;)F=F.parent;return F}static get current(){return T.zone}static get currentTask(){return Q}static __load_patch(F,M,W=!1){if(ia.hasOwnProperty(F)){if(M=
!0===hb[ib("forceDuplicateZoneCheck")],!W&&M)throw Error("Already loaded patch: "+F);}else hb["__Zone_disable_"+F]||(W="Zone:"+F,a(W),ia[F]=M(hb,k,O),b(W,W))}get parent(){return this.Qa}get name(){return this.qh}constructor(F,M){this.Qa=F;this.qh=M?M.name||"unnamed":"<root>";this.D=M&&M.properties||{};this.C=new p(this,this.Qa&&this.Qa.C,M)}get(F){const M=this.getZoneWith(F);if(M)return M.D[F]}getZoneWith(F){let M=this;for(;M;){if(M.D.hasOwnProperty(F))return M;M=M.Qa}return null}fork(F){if(!F)throw Error("ZoneSpec required!");
return this.C.fork(this,F)}wrap(F,M){if("function"!==typeof F)throw Error("Expecting function got: "+F);const W=this.C.intercept(this,F,M),Aa=this;return function(){return Aa.runGuarded(W,this,arguments,M)}}run(F,M,W,Aa){T={parent:T,zone:this};try{return this.C.invoke(this,F,M,W,Aa)}finally{T=T.parent}}runGuarded(F,M=null,W,Aa){T={parent:T,zone:this};try{try{return this.C.invoke(this,F,M,W,Aa)}catch(pb){if(this.C.handleError(this,pb))throw pb;}}finally{T=T.parent}}runTask(F,M,W){if(F.zone!=this)throw Error("A task can only be run in the zone of creation! (Creation: "+
(F.zone||ca).name+"; Execution: "+this.name+")");if("notScheduled"!==F.state||"eventTask"!==F.type&&"macroTask"!==F.type){var Aa="running"!=F.state;Aa&&F.wd("running","scheduled");F.runCount++;var pb=Q;Q=F;T={parent:T,zone:this};try{"macroTask"==F.type&&F.data&&!F.data.isPeriodic&&(F.cancelFn=void 0);try{return this.C.invokeTask(this,F,M,W)}catch(qb){if(this.C.handleError(this,qb))throw qb;}}finally{"notScheduled"!==F.state&&"unknown"!==F.state&&("eventTask"==F.type||F.data&&F.data.isPeriodic?Aa&&
F.wd("scheduled","running"):(F.runCount=0,this.Ek(F,-1),Aa&&F.wd("notScheduled","running","notScheduled"))),T=T.parent,Q=pb}}}scheduleTask(F){if(F.zone&&F.zone!==this)for(var M=this;M;){if(M===F.zone)throw Error(`can not reschedule task to ${this.name} which is descendants of the original zone ${F.zone.name}`);M=M.parent}F.wd("scheduling","notScheduled");M=[];F.Qi=M;F.yb=this;try{F=this.C.scheduleTask(this,F)}catch(W){throw F.wd("unknown","scheduling","notScheduled"),this.C.handleError(this,W),W;
}F.Qi===M&&this.Ek(F,1);"scheduling"==F.state&&F.wd("scheduled","scheduling");return F}scheduleMicroTask(F,M,W,Aa){return this.scheduleTask(new t("microTask",F,M,W,Aa,void 0))}scheduleMacroTask(F,M,W,Aa,pb){return this.scheduleTask(new t("macroTask",F,M,W,Aa,pb))}scheduleEventTask(F,M,W,Aa,pb){return this.scheduleTask(new t("eventTask",F,M,W,Aa,pb))}cancelTask(F){if(F.zone!=this)throw Error("A task can only be cancelled in the zone of creation! (Creation: "+(F.zone||ca).name+"; Execution: "+this.name+
")");if("scheduled"===F.state||"running"===F.state){F.wd("canceling","scheduled","running");try{this.C.cancelTask(this,F)}catch(M){throw F.wd("unknown","canceling"),this.C.handleError(this,M),M;}this.Ek(F,-1);F.wd("notScheduled","canceling");F.runCount=0;return F}}Ek(F,M){const W=F.Qi;-1==M&&(F.Qi=null);for(let Aa=0;Aa<W.length;Aa++)W[Aa].Ek(F.type,M)}}k.__symbol__=ib;const m={name:"",onHasTask:(F,M,W,Aa)=>F.hasTask(W,Aa),onScheduleTask:(F,M,W,Aa)=>F.scheduleTask(W,Aa),onInvokeTask:(F,M,W,Aa,pb,qb)=>
F.invokeTask(W,Aa,pb,qb),onCancelTask:(F,M,W,Aa)=>F.cancelTask(W,Aa)};class p{get zone(){return this.yb}constructor(F,M,W){this.LA={microTask:0,macroTask:0,eventTask:0};this.yb=F;this.vF=M;this.pp=W&&(W&&W.onFork?W:M.pp);this.Bt=W&&(W.onFork?M:M.Bt);this.Pz=W&&(W.onFork?this.yb:M.Pz);this.xp=W&&(W.onIntercept?W:M.xp);this.Ut=W&&(W.onIntercept?M:M.Ut);this.Tt=W&&(W.onIntercept?this.yb:M.Tt);this.Ap=W&&(W.onInvoke?W:M.Ap);this.Xt=W&&(W.onInvoke?M:M.Xt);this.Vt=W&&(W.onInvoke?this.yb:M.Vt);this.rp=W&&
(W.onHandleError?W:M.rp);this.It=W&&(W.onHandleError?M:M.It);this.Ht=W&&(W.onHandleError?this.yb:M.Ht);this.Lm=W&&(W.onScheduleTask?W:M.Lm);this.Jp=W&&(W.onScheduleTask?M:M.Jp);this.Ip=W&&(W.onScheduleTask?this.yb:M.Ip);this.Bm=W&&(W.onInvokeTask?W:M.Bm);this.zp=W&&(W.onInvokeTask?M:M.zp);this.yp=W&&(W.onInvokeTask?this.yb:M.yp);this.sm=W&&(W.onCancelTask?W:M.sm);this.ip=W&&(W.onCancelTask?M:M.ip);this.hp=W&&(W.onCancelTask?this.yb:M.hp);this.Kt=this.Mt=this.Lt=this.qk=null;F=W&&W.onHasTask;const Aa=
M&&M.qk;if(F||Aa)this.qk=F?W:m,this.Lt=M,this.Mt=this,this.Kt=this.yb,W.onScheduleTask||(this.Lm=m,this.Jp=M,this.Ip=this.yb),W.onInvokeTask||(this.Bm=m,this.zp=M,this.yp=this.yb),W.onCancelTask||(this.sm=m,this.ip=M,this.hp=this.yb)}fork(F,M){return this.pp?this.pp.onFork(this.Bt,this.zone,F,M):new k(F,M)}intercept(F,M,W){return this.xp?this.xp.onIntercept(this.Ut,this.Tt,F,M,W):M}invoke(F,M,W,Aa,pb){return this.Ap?this.Ap.onInvoke(this.Xt,this.Vt,F,M,W,Aa,pb):M.apply(W,Aa)}handleError(F,M){return this.rp?
this.rp.onHandleError(this.It,this.Ht,F,M):!0}scheduleTask(F,M){let W=M;if(this.Lm)this.qk&&W.Qi.push(this.Mt),(W=this.Lm.onScheduleTask(this.Jp,this.Ip,F,M))||(W=M);else if(M.scheduleFn)M.scheduleFn(M);else if("microTask"==M.type)d(M);else throw Error("Task is missing scheduleFn.");return W}invokeTask(F,M,W,Aa){return this.Bm?this.Bm.onInvokeTask(this.zp,this.yp,F,M,W,Aa):M.callback.apply(W,Aa)}cancelTask(F,M){if(this.sm)F=this.sm.onCancelTask(this.ip,this.hp,F,M);else{if(!M.cancelFn)throw Error("Task is not cancelable");
F=M.cancelFn(M)}return F}hasTask(F,M){try{this.qk&&this.qk.onHasTask(this.Lt,this.Kt,F,M)}catch(W){this.handleError(F,W)}}Ek(F,M){const W=this.LA,Aa=W[F];M=W[F]=Aa+M;if(0>M)throw Error("More tasks executed then were scheduled.");0!=Aa&&0!=M||this.hasTask(this.yb,{OC:0<W.microTask,JC:0<W.macroTask,MF:0<W.eventTask,wb:F})}}class t{constructor(F,M,W,Aa,pb,qb){this.yb=null;this.runCount=0;this.Qi=null;this.pc="notScheduled";this.type=F;this.source=M;this.data=Aa;this.scheduleFn=pb;this.cancelFn=qb;if(!W)throw Error("callback is not defined");
this.callback=W;const Ia=this;this.invoke="eventTask"===F&&Aa&&Aa.Ls?t.invokeTask:function(){return t.invokeTask.call(hb,Ia,this,arguments)}}static invokeTask(F,M,W){F||(F=this);la++;try{return F.runCount++,F.zone.runTask(F,M,W)}finally{1==la&&e(),la--}}get zone(){return this.yb}get state(){return this.pc}cancelScheduleRequest(){this.wd("notScheduled","scheduling")}wd(F,M,W){if(this.pc===M||this.pc===W)this.pc=F,"notScheduled"==F&&(this.Qi=null);else throw Error(`${this.type} '${this.source}': can not transition to '${F}', expecting state '${M}'${W?
" or '"+W+"'":""}, was '${this.pc}'.`);}toString(){return this.data&&"undefined"!==typeof this.data.handleId?this.data.handleId.toString():Object.prototype.toString.call(this)}toJSON(){return{type:this.type,state:this.state,source:this.source,zone:this.zone.name,runCount:this.runCount}}}const w=ib("setTimeout"),B=ib("Promise"),G=ib("then");let J=[],N=!1,aa;const ca={name:"NO ZONE"},ia={},O={symbol:ib,currentZoneFrame:()=>T,onUnhandledError:f,microtaskDrainDone:f,scheduleMicroTask:d,showUncaughtError:()=>
!k[ib("ignoreConsoleErrorUncaughtError")],patchEventTarget:()=>[],patchOnProperties:f,patchMethod:()=>f,bindArguments:()=>[],patchThen:()=>f,patchMacroTask:()=>f,patchEventPrototype:()=>f,isIEOrEdge:()=>!1,getGlobalObjects:()=>{},ObjectDefineProperty:()=>f,ObjectGetOwnPropertyDescriptor:()=>{},ObjectCreate:()=>{},ArraySlice:()=>[],patchClass:()=>f,wrapWithCurrentZone:()=>f,filterProperties:()=>[],attachOriginToPatched:()=>f,_redefineProperty:()=>f,patchCallbacks:()=>f,nativeScheduleMicroTask:c};let T=
{parent:null,zone:new k(null,null)},Q=null,la=0;b("Zone","Zone");return k};var lb=Object.getOwnPropertyDescriptor,mb=Object.defineProperty,nb=Object.getPrototypeOf,rb=Object.create,sb=Array.prototype.slice,yb=ib("addEventListener"),zb=ib("removeEventListener"),Ab=ib("");function Bb(a,b){return Zone.current.wrap(a,b)}const Cb="undefined"!==typeof window,Db=Cb?window:void 0,Fb=Cb&&Db||globalThis;function Gb(a,b){for(let c=a.length-1;0<=c;c--)"function"===typeof a[c]&&(a[c]=Bb(a[c],b+"_"+c));return a}
function Ib(a){var b=["getCurrentPosition","watchPosition"];const c=a.constructor.name;for(let d=0;d<b.length;d++){const e=b[d],f=a[e];if(f){const g=lb(a,e);Jb(g)&&(a[e]=(k=>{function m(){return k.apply(this,Gb(arguments,c+"."+e))}Kb(m,k);return m})(f))}}}function Jb(a){return a?!1===a.writable?!1:!("function"===typeof a.get&&"undefined"===typeof a.set):!0}
var Lb="undefined"!==typeof WorkerGlobalScope&&self instanceof WorkerGlobalScope,Mb=!("nw"in Fb)&&"undefined"!==typeof Fb.process&&"[object process]"===Fb.process.toString(),Nb=!Mb&&!Lb&&!(!Cb||!Db.HTMLElement),Ob="undefined"!==typeof Fb.process&&"[object process]"===Fb.process.toString()&&!Lb&&!(!Cb||!Db.HTMLElement);const Pb={};
function Qb(a){if(a=a||Fb.event){var b=Pb[a.type];b||(b=Pb[a.type]=ib("ON_PROPERTY"+a.type));var c=this||a.target||Fb;b=c[b];Nb&&c===Db&&"error"===a.type?(c=a,c=b&&b.call(this,c.message,c.filename,c.lineno,c.colno,c.error),!0===c&&a.preventDefault()):(c=b&&b.apply(this,arguments),void 0==c||c||a.preventDefault());return c}}
function Tb(a,b,c){let d=lb(a,b);!d&&c&&lb(c,b)&&(d={enumerable:!0,configurable:!0});if(d&&d.configurable&&(c=ib("on"+b+"patched"),!a.hasOwnProperty(c)||!a[c])){delete d.writable;delete d.value;var e=d.get,f=d.set,g=b.slice(2),k=Pb[g];k||(k=Pb[g]=ib("ON_PROPERTY"+g));d.set=function(m){let p=this;p||a!==Fb||(p=Fb);p&&("function"===typeof p[k]&&p.removeEventListener(g,Qb),f&&f.call(p,null),p[k]=m,"function"===typeof m&&p.addEventListener(g,Qb,!1))};d.get=function(){let m=this;m||a!==Fb||(m=Fb);if(!m)return null;
var p=m[k];return p?p:e&&(p=e.call(this))?(d.set.call(this,p),"function"===typeof m.removeAttribute&&m.removeAttribute(b),p):null};mb(a,b,d);a[c]=!0}}function Ub(a,b,c){if(b)for(var d=0;d<b.length;d++)Tb(a,"on"+b[d],c);else{b=[];for(d in a)"on"==d.slice(0,2)&&b.push(d);for(d=0;d<b.length;d++)Tb(a,b[d],c)}}const Yb=ib("originalInstance");
function Zb(a){const b=Fb[a];if(b){Fb[ib(a)]=b;Fb[a]=function(){const e=Gb(arguments,a);switch(e.length){case 0:this[Yb]=new b;break;case 1:this[Yb]=new b(e[0]);break;case 2:this[Yb]=new b(e[0],e[1]);break;case 3:this[Yb]=new b(e[0],e[1],e[2]);break;case 4:this[Yb]=new b(e[0],e[1],e[2],e[3]);break;default:throw Error("Arg list too long.");}};Kb(Fb[a],b);var c=new b(function(){}),d;for(d in c)"XMLHttpRequest"===a&&"responseBlob"===d||function(e){"function"===typeof c[e]?Fb[a].prototype[e]=function(){return this[Yb][e].apply(this[Yb],
arguments)}:mb(Fb[a].prototype,e,{set:function(f){"function"===typeof f?(this[Yb][e]=Bb(f,a+"."+e),Kb(this[Yb][e],f)):this[Yb][e]=f},get:function(){return this[Yb][e]}})}(d);for(d in b)"prototype"!==d&&b.hasOwnProperty(d)&&(Fb[a][d]=b[d])}}
function $b(a,b,c){let d=a;for(;d&&!d.hasOwnProperty(b);)d=nb(d);!d&&a[b]&&(d=a);a=ib(b);let e=null;if(d&&(!(e=d[a])||!d.hasOwnProperty(a))){e=d[a]=d[b];const f=d&&lb(d,b);if(Jb(f)){const g=c(e,a,b);d[b]=function(){return g(this,arguments)};Kb(d[b],e)}}return e}
function bc(a,b,c){function d(f){const g=f.data;g.Ye[g.Zm]=function(){f.invoke.apply(this,arguments)};e.apply(g.target,g.Ye);return f}let e=null;e=$b(a,b,f=>function(g,k){const m=c(g,k);return 0<=m.Zm&&"function"===typeof k[m.Zm]?Zone.current.scheduleMacroTask(m.name,k[m.Zm],m,d,void 0):f.apply(g,k)})}function Kb(a,b){a[ib("OriginalDelegate")]=b}let cc=!1,dc=!1;
function ec(){if(cc)return dc;cc=!0;try{const a=Db.navigator.userAgent;if(-1!==a.indexOf("MSIE ")||-1!==a.indexOf("Trident/")||-1!==a.indexOf("Edge/"))dc=!0}catch(a){}return dc};function fc(a,b,c){if(!c||0===c.length)return b;c=c.filter(e=>e.target===a);if(!c||0===c.length)return b;const d=c[0].lC;return b.filter(e=>-1===d.indexOf(e))}function gc(a){return Object.getOwnPropertyNames(a).filter(b=>b.startsWith("on")&&2<b.length).map(b=>b.substring(2))};let hc=!1;if("undefined"!==typeof window)try{const a=Object.defineProperty({},"passive",{get:function(){hc=!0}});window.addEventListener("test",a,a);window.removeEventListener("test",a,a)}catch(a){hc=!1}const ic={Ls:!0};var jc={},mc={};const nc=new RegExp("^"+Ab+"(\\w+)(true|false)$"),oc=ib("propagationStopped");function pc(a,b){var c=(b?b(a):a)+"false";b=(b?b(a):a)+"true";c=Ab+c;b=Ab+b;jc[a]={};jc[a]["false"]=c;jc[a]["true"]=b}
function qc(a,b,c,d){function e(aa){return k(this,aa,!0)}function f(aa){return k(this,aa,!1)}function g(aa,ca,ia){if(!aa.hr){var O=aa.callback;"object"===typeof O&&O.handleEvent&&(aa.callback=la=>O.handleEvent(la),aa.jf=O);try{aa.invoke(aa,ca,[ia])}catch(la){var T=la}var Q=aa.options;Q&&"object"===typeof Q&&Q.once&&ca[t].call(ca,ia.type,aa.jf?aa.jf:aa.callback,Q);return T}}function k(aa,ca,ia){if(ca=ca||a.event){aa=aa||ca.target||a;var O=aa[jc[ca.type][ia?"true":"false"]];if(O){ia=[];if(1===O.length)(ca=
g(O[0],aa,ca))&&ia.push(ca);else{O=O.slice();for(let T=0;T<O.length&&(!ca||!0!==ca[oc]);T++){const Q=g(O[T],aa,ca);Q&&ia.push(Q)}}if(1===ia.length)throw ia[0];for(ca=0;ca<ia.length;ca++){const T=ia[ca];b.nativeScheduleMicroTask(()=>{throw T;})}}}}function m(aa,ca){function ia(pa,Ha,Ua,kb,xb=!1,Tc=!1){return function(){const ac=this||a;let Xb=arguments[0];ca&&ca.Xg&&(Xb=ca.Xg(Xb));let Jc=arguments[1];if(!Jc||Mb&&"uncaughtException"===Xb)return pa.apply(this,arguments);let Hf=!1;if("function"!==typeof Jc){if(!Jc.handleEvent)return pa.apply(this,
arguments);Hf=!0}if(!Aa||Aa(pa,Jc,ac,arguments)){var hd=hc&&!!ub&&-1!==ub.indexOf(Xb),Ac=M(arguments[2],hd),Vd=null==Ac?void 0:Ac.signal;if(null==Vd||!Vd.aborted){if(Pa)for(var If=0;If<Pa.length;If++)if(Xb===Pa[If])return hd?pa.call(ac,Xb,Jc,Ac):pa.apply(this,arguments);hd=Ac?"boolean"===typeof Ac?!0:Ac.capture:!1;If=Ac&&"object"===typeof Ac?Ac.once:!1;var sr=Zone.current,Kc=jc[Xb];Kc||(pc(Xb,Vb),Kc=jc[Xb]);var cd=Kc[hd?"true":"false"];Kc=ac[cd];var Bc=!1;if(Kc){if(Bc=!0,pb)for(cd=0;cd<Kc.length;cd++)if(cb(Kc[cd],
Jc))return}else Kc=ac[cd]=[];var Dd;cd=ac.constructor.name;var Pk=mc[cd];Pk&&(Dd=Pk[Xb]);Dd||(Dd=cd+Ha+(Vb?Vb(Xb):Xb));Ma.options=Ac;If&&(Ma.options.once=!1);Ma.target=ac;Ma.capture=hd;Ma.Ih=Xb;Ma.wC=Bc;if(Bc=W?ic:void 0)Bc.vE=Ma;Vd&&(Ma.options.signal=void 0);var qd=sr.scheduleEventTask(Dd,Jc,Bc,Ua,kb);if(Vd){Ma.options.signal=Vd;const ei=()=>qd.zone.cancelTask(qd);pa.call(Vd,"abort",ei,{once:!0});Bc&&(Bc.Yr=()=>Vd.removeEventListener("abort",ei))}Ma.target=null;Bc&&(Bc.vE=null);If&&(Ac.once=!0);
if(hc||"boolean"!==typeof qd.options)qd.options=Ac;qd.target=ac;qd.capture=hd;qd.Ih=Xb;Hf&&(qd.jf=Jc);Tc?Kc.unshift(qd):Kc.push(qd);if(xb)return ac}}}}function O(pa,Ha){const Ua=typeof Ha;return"function"===Ua&&pa.callback===Ha||"object"===Ua&&pa.jf===Ha}function T(pa){return U.call(pa.target,pa.Ih,pa.invoke,pa.options)}function Q(pa){return na.call(Ma.target,Ma.Ih,pa.invoke,Ma.options)}function la(pa){return qa.call(Ma.target,Ma.Ih,pa.invoke,Ma.options)}function F(pa){if(!pa.hr){var Ha=jc[pa.Ih];
let Ua;Ha&&(Ua=Ha[pa.capture?"true":"false"]);if(Ha=Ua&&pa.target[Ua])for(let kb=0;kb<Ha.length;kb++)if(Ha[kb]===pa){Ha.splice(kb,1);pa.hr=!0;0===Ha.length&&(pa.Cu=!0,pa.target[Ua]=null);break}}if(pa.Cu)return U.call(pa.target,pa.Ih,pa.capture?e:f,pa.options)}function M(pa,Ha){return!hc&&"object"===typeof pa&&pa?!!pa.capture:hc&&Ha?"boolean"===typeof pa?{capture:pa,passive:!0}:pa?"object"===typeof pa&&!1!==pa.passive?Object.assign({},pa,{passive:!0}):pa:{passive:!0}:pa}if(!aa)return!1;let W=!0;ca&&
void 0!==ca.Ls&&(W=ca.Ls);const Aa=ca&&ca.SE;let pb=!0;ca&&void 0!==ca.fB&&(pb=ca.fB);let qb=!1;ca&&void 0!==ca.ND&&(qb=ca.ND);let Ia=aa;for(;Ia&&!Ia.hasOwnProperty(p);)Ia=nb(Ia);!Ia&&aa[p]&&(Ia=aa);if(!Ia||Ia[G])return!1;const Vb=ca&&ca.LF,Ma={},qa=Ia[G]=Ia[p],U=Ia[ib(t)]=Ia[t];aa=Ia[ib(w)]=Ia[w];const ea=Ia[ib(B)]=Ia[B];let na;ca&&ca.prepend&&(na=Ia[ib(ca.prepend)]=Ia[ca.prepend]);const Ja=W?F:T,cb=ca&&ca.mn?ca.mn:O,Pa=Zone[ib("UNPATCHED_EVENTS")],ub=a[ib("PASSIVE_EVENTS")];Ia[p]=ia(qa,J,W?function(){if(!Ma.wC)return qa.call(Ma.target,
Ma.Ih,Ma.capture?e:f,Ma.options)}:la,Ja,qb);na&&(Ia.prependListener=ia(na,".prependListener:",Q,Ja,qb,!0));Ia[t]=function(){const pa=this||a;var Ha=arguments[0];ca&&ca.Xg&&(Ha=ca.Xg(Ha));var Ua=arguments[2];Ua=Ua?"boolean"===typeof Ua?!0:Ua.capture:!1;const kb=arguments[1];if(!kb)return U.apply(this,arguments);if(!Aa||Aa(U,kb,pa,arguments)){var xb=jc[Ha],Tc;xb&&(Tc=xb[Ua?"true":"false"]);var ac=Tc&&pa[Tc];if(ac)for(let Xb=0;Xb<ac.length;Xb++)if(xb=ac[Xb],cb(xb,kb)){ac.splice(Xb,1);xb.hr=!0;0===ac.length&&
(xb.Cu=!0,pa[Tc]=null,Ua||"string"!==typeof Ha||(pa[Ab+"ON_PROPERTY"+Ha]=null));Ha=xb.data;let Jc;if(null==(Jc=Ha)?0:Jc.Yr)Ha.Yr(),Ha.Yr=null;xb.zone.cancelTask(xb);if(qb)return pa;return}return U.apply(this,arguments)}};Ia[w]=function(pa){var Ha=this||a,Ua=pa;ca&&ca.Xg&&(Ua=ca.Xg(Ua));pa=[];Ha=rc(Ha,Vb?Vb(Ua):Ua);for(Ua=0;Ua<Ha.length;Ua++){const kb=Ha[Ua];pa.push(kb.jf?kb.jf:kb.callback)}return pa};Ia[B]=function(pa){var Ha=this||a;if(pa){ca&&ca.Xg&&(pa=ca.Xg(pa));var Ua=jc[pa];if(Ua){var kb=Ha[Ua["false"]];
Ha=Ha[Ua["true"]];if(kb)for(kb=kb.slice(),Ua=0;Ua<kb.length;Ua++){const xb=kb[Ua];this[t].call(this,pa,xb.jf?xb.jf:xb.callback,xb.options)}if(Ha)for(Ha=Ha.slice(),kb=0;kb<Ha.length;kb++)Ua=Ha[kb],this[t].call(this,pa,Ua.jf?Ua.jf:Ua.callback,Ua.options)}}else{pa=Object.keys(Ha);for(Ha=0;Ha<pa.length;Ha++)(kb=(kb=nc.exec(pa[Ha]))&&kb[1])&&"removeListener"!==kb&&this[B].call(this,kb);this[B].call(this,"removeListener")}if(qb)return this};Kb(Ia[p],qa);Kb(Ia[t],U);ea&&Kb(Ia[B],ea);aa&&Kb(Ia[w],aa);return!0}
const p=d&&d.add||"addEventListener",t=d&&d.rG||"removeEventListener",w=d&&d.WF||"eventListeners",B=d&&d.sG||"removeAllListeners",G=ib(p),J="."+p+":";let N=[];for(let aa=0;aa<c.length;aa++)N[aa]=m(c[aa],d);return N}function rc(a,b){if(!b){var c=[];for(let e in a){var d=nc.exec(e);if((d=d&&d[1])&&(!b||d===b)&&(d=a[e]))for(let f=0;f<d.length;f++)c.push(d[f])}return c}c=jc[b];c||(pc(b),c=jc[b]);b=a[c["false"]];a=a[c["true"]];return b?a?b.concat(a):b.slice():a?a.slice():[]}
function sc(a,b){(a=a.Event)&&a.prototype&&b.patchMethod(a.prototype,"stopImmediatePropagation",c=>function(d,e){d[oc]=!0;c&&c.apply(d,e)})};function uc(){vc.__load_patch("util",(a,b,c)=>{const d=gc(a);c.patchOnProperties=Ub;c.patchMethod=$b;c.bindArguments=Gb;c.patchMacroTask=bc;const e=b.__symbol__("BLACK_LISTED_EVENTS"),f=b.__symbol__("UNPATCHED_EVENTS");a[f]&&(a[e]=a[f]);a[e]&&(b[e]=b[f]=a[e]);c.patchEventPrototype=sc;c.patchEventTarget=qc;c.isIEOrEdge=ec;c.ObjectDefineProperty=mb;c.ObjectGetOwnPropertyDescriptor=lb;c.ObjectCreate=rb;c.ArraySlice=sb;c.patchClass=Zb;c.wrapWithCurrentZone=Bb;c.filterProperties=fc;c.attachOriginToPatched=
Kb;c._redefineProperty=Object.defineProperty;c.patchCallbacks=gb;c.getGlobalObjects=()=>({globalSources:mc,zoneSymbolEventNames:jc,eventNames:d,isBrowser:Nb,isMix:Ob,isNode:Mb,TRUE_STR:"true",FALSE_STR:"false",ZONE_SYMBOL_PREFIX:Ab,ADD_EVENT_LISTENER_STR:"addEventListener",REMOVE_EVENT_LISTENER_STR:"removeEventListener"})})};let wc,xc,yc,zc,Cc;
function Dc(){wc=Zone.__symbol__;xc=Object[wc("defineProperty")]=Object.defineProperty;yc=Object[wc("getOwnPropertyDescriptor")]=Object.getOwnPropertyDescriptor;zc=Object.create;Cc=wc("unconfigurables");Object.defineProperty=function(a,b,c){if(a&&a[Cc]&&a[Cc][b])throw new TypeError("Cannot assign to read only property '"+b+"' of "+a);const d=c.configurable;"prototype"!==b&&(c=Ec(a,b,c));return Fc(a,b,c,d)};Object.defineProperties=function(a,b){Object.keys(b).forEach(function(c){Object.defineProperty(a,c,
b[c])});for(const c of Object.getOwnPropertySymbols(b)){let d;(null==(d=Object.getOwnPropertyDescriptor(b,c))?0:d.enumerable)&&Object.defineProperty(a,c,b[c])}return a};Object.create=function(a,b){"object"!==typeof b||Object.isFrozen(b)||Object.keys(b).forEach(function(c){b[c]=Ec(a,c,b[c])});return zc(a,b)};Object.getOwnPropertyDescriptor=function(a,b){const c=yc(a,b);c&&a&&a[Cc]&&a[Cc][b]&&(c.configurable=!1);return c}}function Gc(a,b,c){const d=c.configurable;c=Ec(a,b,c);return Fc(a,b,c,d)}
function Ec(a,b,c){Object.isFrozen(c)||(c.configurable=!0);c.configurable||(a[Cc]||Object.isFrozen(a)||xc(a,Cc,{writable:!0,value:{}}),a[Cc]&&(a[Cc][b]=!0));return c}
function Fc(a,b,c,d){try{return xc(a,b,c)}catch(e){if(c.configurable){"undefined"==typeof d?delete c.configurable:c.configurable=d;try{return xc(a,b,c)}catch(f){d=!1;if("createdCallback"===b||"attachedCallback"===b||"detachedCallback"===b||"attributeChangedCallback"===b)d=!0;if(!d)throw f;d=null;try{d=JSON.stringify(c)}catch(g){d=c.toString()}console.log(`Attempting to configure '${b}' with descriptor '${d}' on object '${a}' and got error, giving up: ${f}`)}}else throw e;}};function Lc(a,b){const {eventNames:c,globalSources:d,zoneSymbolEventNames:e,TRUE_STR:f,FALSE_STR:g,ZONE_SYMBOL_PREFIX:k}=b.getGlobalObjects();var m="ApplicationCache EventSource FileReader InputMethodContext MediaController MessagePort Node Performance SVGElementInstance SharedWorker TextTrack TextTrackCue TextTrackList WebKitNamedFlow Window Worker WorkerGlobalScope XMLHttpRequest XMLHttpRequestEventTarget XMLHttpRequestUpload IDBRequest IDBOpenDBRequest IDBDatabase IDBTransaction IDBCursor DBIndex WebSocket".split(" ");
let p=[];var t="Anchor Area Audio BR Base BaseFont Body Button Canvas Content DList Directory Div Embed FieldSet Font Form Frame FrameSet HR Head Heading Html IFrame Image Input Keygen LI Label Legend Link Map Marquee Media Menu Meta Meter Mod OList Object OptGroup Option Output Paragraph Pre Progress Quote Script Select Source Span Style TableCaption TableCell TableCol Table TableRow TableSection TextArea Title Track UList Unknown Video".split(" ");a.wtf?p=t.map(ia=>"HTML"+ia+"Element").concat(m):
a.EventTarget?p.push("EventTarget"):p=m;const w=a.__Zone_disable_IE_check||!1,B=a.__Zone_enable_cross_context_check||!1,G=b.isIEOrEdge(),J={MSPointerCancel:"pointercancel",MSPointerDown:"pointerdown",MSPointerEnter:"pointerenter",MSPointerHover:"pointerhover",MSPointerLeave:"pointerleave",MSPointerMove:"pointermove",MSPointerOut:"pointerout",MSPointerOver:"pointerover",MSPointerUp:"pointerup"};for(m=0;m<c.length;m++){var N=c[m],aa=k+(N+g),ca=k+(N+f);e[N]={};e[N][g]=aa;e[N][f]=ca}for(m=0;m<t.length;m++)for(N=
t[m],aa=d[N]={},ca=0;ca<c.length;ca++){const ia=c[ca];aa[ia]=N+".addEventListener:"+ia}t=[];for(m=0;m<p.length;m++)N=a[p[m]],t.push(N&&N.prototype);b.patchEventTarget(a,b,t,{SE:function(ia,O,T,Q){if(!w&&G)if(B)try{const la=O.toString();if("[object FunctionWrapper]"===la||"function __BROWSERTOOLS_CONSOLE_SAFEFUNC() { [native code] }"==la)return ia.apply(T,Q),!1}catch(la){return ia.apply(T,Q),!1}else{if(O=O.toString(),"[object FunctionWrapper]"===O||"function __BROWSERTOOLS_CONSOLE_SAFEFUNC() { [native code] }"==
O)return ia.apply(T,Q),!1}else if(B)try{O.toString()}catch(la){return ia.apply(T,Q),!1}return!0},Xg:ia=>J[ia]||ia});Zone[b.symbol("patchEventTarget")]=!!a.EventTarget};function Mc(a,b){const {ADD_EVENT_LISTENER_STR:c,REMOVE_EVENT_LISTENER_STR:d}=a.getGlobalObjects(),e=b.WebSocket;b.EventTarget||a.patchEventTarget(b,a,[e.prototype]);b.WebSocket=function(g,k){const m=1<arguments.length?new e(g,k):new e(g);let p,t;const w=a.ObjectGetOwnPropertyDescriptor(m,"onmessage");w&&!1===w.configurable?(p=a.ObjectCreate(m),t=m,[c,d,"send","close"].forEach(function(B){p[B]=function(){const G=a.ArraySlice.call(arguments);if(B===c||B===d){var J=0<G.length?G[0]:void 0;J&&(J=Zone.__symbol__("ON_PROPERTY"+
J),m[J]=p[J])}return m[B].apply(m,G)}})):p=m;a.patchOnProperties(p,["close","error","message","open"],t);return p};const f=b.WebSocket;for(const g in e)f[g]=e[g]};function Nc(a,b){const {isBrowser:c,isMix:d}=a.getGlobalObjects();if((c||d)&&!a.ObjectGetOwnPropertyDescriptor(HTMLElement.prototype,"onclick")&&"undefined"!==typeof Element){var e=a.ObjectGetOwnPropertyDescriptor(Element.prototype,"onclick");if(e&&!e.configurable)return!1;if(e){a.ObjectDefineProperty(Element.prototype,"onclick",{enumerable:!0,configurable:!0,get:function(){return!0}});var f=!!document.createElement("div").onclick;a.ObjectDefineProperty(Element.prototype,"onclick",e);return f}}var g=
b.XMLHttpRequest;if(!g)return!1;e=g.prototype;if(f=a.ObjectGetOwnPropertyDescriptor(e,"onreadystatechange"))return a.ObjectDefineProperty(e,"onreadystatechange",{enumerable:!0,configurable:!0,get:function(){return!0}}),g=!!(new g).onreadystatechange,a.ObjectDefineProperty(e,"onreadystatechange",f||{}),g;const k=a.symbol("fake");a.ObjectDefineProperty(e,"onreadystatechange",{enumerable:!0,configurable:!0,get:function(){return this[k]},set:function(m){this[k]=m}});a=new g;e=()=>{};a.onreadystatechange=
e;e=a[k]===e;a.onreadystatechange=null;return e}const Oc="abort animationcancel animationend animationiteration auxclick beforeinput blur cancel canplay canplaythrough change compositionstart compositionupdate compositionend cuechange click close contextmenu curechange dblclick drag dragend dragenter dragexit dragleave dragover drop durationchange emptied ended error focus focusin focusout gotpointercapture input invalid keydown keypress keyup load loadstart loadeddata loadedmetadata lostpointercapture mousedown mouseenter mouseleave mousemove mouseout mouseover mouseup mousewheel orientationchange pause play playing pointercancel pointerdown pointerenter pointerleave pointerlockchange mozpointerlockchange webkitpointerlockerchange pointerlockerror mozpointerlockerror webkitpointerlockerror pointermove pointout pointerover pointerup progress ratechange reset resize scroll seeked seeking select selectionchange selectstart show sort stalled submit suspend timeupdate volumechange touchcancel touchmove touchstart touchend transitioncancel transitionend waiting wheel webglcontextrestored webglcontextlost webglcontextcreationerror autocomplete autocompleteerror toggle afterscriptexecute beforescriptexecute DOMContentLoaded freeze fullscreenchange mozfullscreenchange webkitfullscreenchange msfullscreenchange fullscreenerror mozfullscreenerror webkitfullscreenerror msfullscreenerror readystatechange visibilitychange resume absolutedeviceorientation afterinput afterprint appinstalled beforeinstallprompt beforeprint beforeunload devicelight devicemotion deviceorientation deviceorientationabsolute deviceproximity hashchange languagechange message mozbeforepaint offline online paint pageshow pagehide popstate rejectionhandled storage unhandledrejection unload userproximity vrdisplayconnected vrdisplaydisconnected vrdisplaypresentchange beforecopy beforecut beforepaste copy cut paste dragstart loadend animationstart search transitionrun transitionstart webkitanimationend webkitanimationiteration webkitanimationstart webkittransitionend activate afterupdate ariarequest beforeactivate beforedeactivate beforeeditfocus beforeupdate cellchange controlselect dataavailable datasetchanged datasetcomplete errorupdate filterchange layoutcomplete losecapture move moveend movestart propertychange resizeend resizestart rowenter rowexit rowsdelete rowsinserted command compassneedscalibration deactivate help mscontentzoom msmanipulationstatechanged msgesturechange msgesturedoubletap msgestureend msgesturehold msgesturestart msgesturetap msgotpointercapture msinertiastart mslostpointercapture mspointercancel mspointerdown mspointerenter mspointerhover mspointerleave mspointermove mspointerout mspointerover mspointerup pointerout mssitemodejumplistitemremoved msthumbnailclick stop storagecommit".split(" ");
function Pc(a){const b=a.symbol("unbound");for(let c=0;c<Oc.length;c++){const d=Oc[c],e="on"+d;self.addEventListener(d,function(f){f=f.target;let g,k;for(k=f?f.constructor.name+"."+e:"unknown."+e;f;)f[e]&&!f[e][b]&&(g=a.wrapWithCurrentZone(f[e],k),g[b]=f[e],f[e]=g),f=f.parentElement},!0)}};function Qc(){const a="undefined"!==typeof window?window:"undefined"!==typeof global?global:"undefined"!==typeof self?self:{};a[(a.__Zone_symbol_prefix||"__zone_symbol__")+"legacyPatch"]=function(){const b=a.Zone;b.__load_patch("defineProperty",(c,d,e)=>{e._redefineProperty=Gc;Dc()});b.__load_patch("registerElement",(c,d,e)=>{const {isBrowser:f,isMix:g}=e.getGlobalObjects();(f||g)&&"registerElement"in c.document&&e.patchCallbacks(e,document,"Document","registerElement",["createdCallback","attachedCallback",
"detachedCallback","attributeChangedCallback"])});b.__load_patch("EventTargetLegacy",(c,d,e)=>{Lc(c,e);const {isNode:f,isMix:g}=e.getGlobalObjects();f&&!g||Nc(e,c)||(d="undefined"!==typeof WebSocket,Pc(e),e.patchClass("XMLHttpRequest"),d&&Mc(e,c),Zone[e.symbol("patchEvents")]=!0)})}};function Uc(a,b){b.patchMethod(a,"queueMicrotask",()=>function(c,d){Zone.current.scheduleMicroTask("queueMicrotask",d[0])})};const Vc=ib("zoneTask");
function Wc(a,b,c,d){function e(p){const t=p.data;t.Ye[0]=function(){return p.invoke.apply(this,arguments)};t.handleId=g.apply(a,t.Ye);return p}function f(p){return k.call(a,p.data.handleId)}let g=null,k=null;b+=d;c+=d;const m={};g=$b(a,b,p=>function(t,w){if("function"===typeof w[0]){const B={isPeriodic:"Interval"===d,delay:"Timeout"===d||"Interval"===d?w[1]||0:void 0,Ye:w},G=w[0];w[0]=function(){try{return G.apply(this,arguments)}finally{B.isPeriodic||("number"===typeof B.handleId?delete m[B.handleId]:
B.handleId&&(B.handleId[Vc]=null))}};t=Zone.current.scheduleMacroTask(b,w[0],B,e,f);if(!t)return t;w=t.data.handleId;"number"===typeof w?m[w]=t:w&&(w[Vc]=t);w&&w.Hj&&w.Js&&"function"===typeof w.Hj&&"function"===typeof w.Js&&(t.Hj=w.Hj.bind(w),t.Js=w.Js.bind(w));return"number"===typeof w||w?w:t}return p.apply(a,w)});k=$b(a,c,p=>function(t,w){t=w[0];let B;"number"===typeof t?B=m[t]:(B=t&&t[Vc])||(B=t);B&&"string"===typeof B.type?"notScheduled"!==B.state&&(B.cancelFn&&B.data.isPeriodic||0===B.runCount)&&
("number"===typeof t?delete m[t]:t&&(t[Vc]=null),B.zone.cancelTask(B)):p.apply(a,w)})};function Xc(){var a=vc;a.__load_patch("legacy",b=>{(b=b[a.__symbol__("legacyPatch")])&&b()});a.__load_patch("timers",b=>{Wc(b,"set","clear","Timeout");Wc(b,"set","clear","Interval");Wc(b,"set","clear","Immediate")});a.__load_patch("requestAnimationFrame",b=>{Wc(b,"request","cancel","AnimationFrame");Wc(b,"mozRequest","mozCancel","AnimationFrame");Wc(b,"webkitRequest","webkitCancel","AnimationFrame")});a.__load_patch("blocking",(b,c)=>{const d=["alert","prompt","confirm"];for(let e=0;e<d.length;e++)$b(b,
d[e],(f,g,k)=>function(m,p){return c.current.run(f,b,p,k)})});a.__load_patch("EventTarget",(b,c,d)=>{d.patchEventPrototype(b,d);if(!Zone[d.symbol("patchEventTarget")]){var {eventNames:e,zoneSymbolEventNames:f,TRUE_STR:g,FALSE_STR:k,ZONE_SYMBOL_PREFIX:m}=d.getGlobalObjects();for(c=0;c<e.length;c++){const p=e[c],t=m+(p+k),w=m+(p+g);f[p]={};f[p][k]=t;f[p][g]=w}(c=b.EventTarget)&&c.prototype&&d.patchEventTarget(b,d,[c&&c.prototype])}(c=b.XMLHttpRequestEventTarget)&&c.prototype&&d.patchEventTarget(b,d,
[c.prototype])});a.__load_patch("MutationObserver",()=>{Zb("MutationObserver");Zb("WebKitMutationObserver")});a.__load_patch("IntersectionObserver",()=>{Zb("IntersectionObserver")});a.__load_patch("FileReader",()=>{Zb("FileReader")});a.__load_patch("on_property",(b,c,d)=>{if((!Mb||Ob)&&!Zone[d.symbol("patchEvents")]){c=b.__Zone_ignore_on_properties;d=[];if(Nb){var e=window;d=d.concat("Document SVGElement Element HTMLElement HTMLBodyElement HTMLMediaElement HTMLFrameSetElement HTMLFrameElement HTMLIFrameElement HTMLMarqueeElement Worker".split(" "));
a:{try{var f=Db.navigator.userAgent;if(-1!==f.indexOf("MSIE ")||-1!==f.indexOf("Trident/")){var g=!0;break a}}catch(m){}g=!1}var k=g?[{target:e,lC:["error"]}]:[];g=e;f=gc(e);k=c?c.concat(k):c;e=nb(e);g&&(f=fc(g,f,k),Ub(g,f,e))}d=d.concat("XMLHttpRequest XMLHttpRequestEventTarget IDBIndex IDBRequest IDBOpenDBRequest IDBDatabase IDBTransaction IDBCursor WebSocket".split(" "));for(e=0;e<d.length;e++)(f=b[d[e]])&&f.prototype&&(g=f.prototype,f=gc(f.prototype),g&&(f=fc(g,f,c),Ub(g,f,void 0)))}});a.__load_patch("customElements",
(b,c,d)=>{const {isBrowser:e,isMix:f}=d.getGlobalObjects();(e||f)&&b.customElements&&"customElements"in b&&d.patchCallbacks(d,b.customElements,"customElements","define","connectedCallback disconnectedCallback adoptedCallback attributeChangedCallback formAssociatedCallback formDisabledCallback formResetCallback formStateRestoreCallback".split(" "))});a.__load_patch("XHR",(b,c)=>{(function(p){function t(Q){const la=Q.data,F=la.target;F[g]=!1;F[m]=!1;var M=F[f];J||(J=F[yb],N=F[zb]);M&&N.call(F,"readystatechange",
M);M=F[f]=()=>{if(F.readyState===F.DONE)if(!la.aborted&&F[g]&&"scheduled"===Q.state){const W=F[c.__symbol__("loadfalse")];if(0!==F.status&&W&&0<W.length){const Aa=Q.invoke;Q.invoke=function(){const pb=F[c.__symbol__("loadfalse")];for(let qb=0;qb<pb.length;qb++)pb[qb]===Q&&pb.splice(qb,1);la.aborted||"scheduled"!==Q.state||Aa.call(Q)};W.push(Q)}else Q.invoke()}else la.aborted||!1!==F[g]||(F[m]=!0)};J.call(F,"readystatechange",M);F[d]||(F[d]=Q);O.apply(F,la.Ye);F[g]=!0;return Q}function w(){}function B(Q){Q=
Q.data;Q.aborted=!0;return T.apply(Q.target,Q.Ye)}var G=p.XMLHttpRequest;if(G){G=G.prototype;var J=G[yb],N=G[zb];!J&&(p=p.XMLHttpRequestEventTarget)&&(p=p.prototype,J=p[yb],N=p[zb]);var aa=$b(G,"open",()=>function(Q,la){Q[e]=0==la[2];Q[k]=la[1];return aa.apply(Q,la)}),ca=ib("fetchTaskAborting"),ia=ib("fetchTaskScheduling"),O=$b(G,"send",()=>function(Q,la){if(!0===c.current[ia]||Q[e])return O.apply(Q,la);la={target:Q,url:Q[k],isPeriodic:!1,Ye:la,aborted:!1};const F=Zone.current.scheduleMacroTask("XMLHttpRequest.send",
w,la,t,B);Q&&!0===Q[m]&&!la.aborted&&"scheduled"===F.state&&F.invoke()}),T=$b(G,"abort",()=>function(Q,la){const F=Q[d];if(F&&"string"==typeof F.type)null==F.cancelFn||F.data&&F.data.aborted||F.zone.cancelTask(F);else if(!0===c.current[ca])return T.apply(Q,la)})}})(b);const d=ib("xhrTask"),e=ib("xhrSync"),f=ib("xhrListener"),g=ib("xhrScheduled"),k=ib("xhrURL"),m=ib("xhrErrorBeforeScheduled")});a.__load_patch("geolocation",b=>{b.navigator&&b.navigator.geolocation&&Ib(b.navigator.geolocation)});a.__load_patch("PromiseRejectionEvent",
(b,c)=>{function d(e){return function(f){rc(b,e).forEach(g=>{var k=b.PromiseRejectionEvent;k&&(k=new k(e,{promise:f.promise,reason:f.rejection}),g.invoke(k))})}}b.PromiseRejectionEvent&&(c[ib("unhandledPromiseRejectionHandler")]=d("unhandledrejection"),c[ib("rejectionHandledHandler")]=d("rejectionhandled"))});a.__load_patch("queueMicrotask",(b,c,d)=>{Uc(b,d)})};function Yc(){vc.__load_patch("canvas",(a,b,c)=>{a=a.HTMLCanvasElement;"undefined"!==typeof a&&a.prototype&&a.prototype.toBlob&&c.patchMacroTask(a.prototype,"toBlob",(d,e)=>({name:"HTMLCanvasElement.toBlob",target:d,Zm:0,Ye:e}))})};function Zc(a,b,c){const d={};for(const e in a)d[e]=b.call(c,a[e],e,a);return d};var $c;var bd=class{constructor(a){this.C=a}toString(){return this.C+""}};function dd(a){return a instanceof bd&&a.constructor===bd?a.C:"type_error:TrustedResourceUrl"}var ed={};function id(a){if(void 0===$c){var b=null;var c=xa.trustedTypes;if(c&&c.createPolicy){try{b=c.createPolicy("goog#html",{createHTML:Da,createScript:Da,createScriptURL:Da})}catch(d){xa.console&&xa.console.error(d.message)}$c=b}else $c=b}a=(b=$c)?b.createScriptURL(a):a;return new bd(a,ed)};/*

 SPDX-License-Identifier: Apache-2.0
*/
var jd=class{constructor(a){this.C=a}toString(){return this.C}};function kd(a){if(a instanceof jd)return a.C;throw Error("");};class ld{constructor(a){this.ir=a}}function md(a){return new ld(b=>b.substr(0,a.length+1).toLowerCase()===a+":")}const nd=[md("data"),md("http"),md("https"),md("mailto"),md("ftp"),new ld(a=>/^[^:]*([/?#]|$)/.test(a))];function rd(a){if(a instanceof jd)return a;for(let b=0;b<nd.length;++b){const c=nd[b];if(c instanceof ld&&c.ir(a))return new jd(a)}}var sd=/^\s*(?!javascript:)(?:[\w+.-]+:|[^:/?#]*(?:[/?#]|$))/i;function td(a){a instanceof jd?a=kd(a):a=sd.test(a)?a:void 0;return a};function ud(a,b){b=td(b);void 0!==b&&(a.href=b)};function vd(a,b){a.src=dd(b);var c;let d;(c=(b=null==(d=(c=(a.ownerDocument&&a.ownerDocument.defaultView||window).document).querySelector)?void 0:d.call(c,"script[nonce]"))?b.nonce||b.getAttribute("nonce")||"":"")&&a.setAttribute("nonce",c)};var wd=String.prototype.repeat?function(a,b){return a.repeat(b)}:function(a,b){return Array(b+1).join(a)};function xd(a){throw Error("Data "+a.D+" not defined.");}function yd(){var a=Ad;if(null==a.C)return null;if("boolean"===typeof a.C)return a.C;if("string"===typeof a.C){const b=a.C.toLowerCase();if("true"===b||"1"===b)return!0;if("false"===b||"0"===b)return!1}throw new TypeError(`Invalid boolean data <${a.D}>: ${a.C} (typeof ${typeof a.C})`);}function Bd(a,b){return db(b,function(c,d){return new Cd(this.D+"["+d+"]",c)},a)}function Ed(a){a=a.F();return""==a.trim()?[]:a.split(",").map(b=>b.trim())}
class Cd{constructor(a,b){this.D=a;this.C=b}F(a){if(null==this.C)return 0==arguments.length&&xd(this),a;if("string"===typeof this.C)return this.C;throw new TypeError("Invalid string data <"+this.D+">: "+this.C+"-"+typeof this.C);}number(a){if(null==this.C)return 0==arguments.length&&xd(this),a;if("number"===typeof this.C)return this.C;if("string"===typeof this.C){const b=Number(this.C);if(!isNaN(b)&&!/^[\s\xa0]*$/.test(this.C))return b}throw new TypeError("Invalid numeric data <"+this.D+">: "+this.C+
"-"+typeof this.C);}toString(){if(null==this.C)var a=null;else if("string"===typeof this.C)a=this.C;else throw new TypeError(`Invalid string data <${this.D}>: ${this.C} (typeof ${typeof this.C})`);null===a&&xd(this);return a}array(a){if(null==this.C){if(0==arguments.length)throw Error("Data "+this.D+" not defined.");return a}return Bd(this,ya(this.C)?this.C:"string"!==typeof this.C?[this.C]:Ed(this))}object(a){if(null==this.C){if(0==arguments.length)throw Error("Data "+this.D+" not defined.");return a}if(!ya(this.C)&&
Ca(this.C))return Zc(this.C,function(b,c){return new Cd(this.D+"."+c,b)},this);throw new TypeError("Invalid object data <"+this.D+">: "+this.C+"-"+typeof this.C);}};function Fd(a=window){return a.WIZ_global_data}function Gd(a=window){return(a=Fd(a))&&"mXaIFf"in a?a.mXaIFf:null};function Hd(){vc.__load_patch("Error",(a,b,c)=>{function d(O){let T={Ry:O.zone.name},Q=T;for(;O.parent;){O=O.parent;const la={Ry:O.zone.name};T=T.parent=la}return Q}function e(O,T,Q=!0){O=O.split("\n");let la=0;for(;O[la]!==t&&O[la]!==w&&O[la]!==B&&O[la]!==G&&O[la]!==J&&la<O.length;)la++;for(;la<O.length&&T;la++){let F=O[la];if(F.trim())switch(p[F]){case g.Ts:O.splice(la,1);la--;break;case g.transition:T.parent?T=T.parent:T=null;O.splice(la,1);la--;break;default:O[la]+=Q?` [${T.zone.name}]`:` [${T.Ry}]`}}return O.join("\n")}
function f(){let O=m.apply(this,arguments);const T=O.originalStack=O.stack;if(f.stackRewrite&&T){let Q=c.currentZoneFrame();if("lazy"===N)O[c.symbol("zoneFrameNames")]=d(Q);else if("default"===N)try{O.stack=O.zoneAwareStack=e(T,Q)}catch(la){}}return this instanceof m&&this.constructor!=m?(Object.keys(O).concat("stack","message").forEach(Q=>{const la=O[Q];if(void 0!==la)try{this[Q]=la}catch(F){}}),this):O}const g={Ts:0,transition:1},k=c.symbol("zoneJsInternalStackFrames"),m=a[c.symbol("Error")]=a.Error,
p={};let t,w,B,G,J;a.Error=f;const N=a.__Zone_Error_BlacklistedStackFrames_policy||a.__Zone_Error_ZoneJsInternalStackFrames_policy||"default";f.prototype=m.prototype;f[k]=p;f.stackRewrite=!1;const aa=c.symbol("zoneAwareStack");"lazy"===N&&Object.defineProperty(f.prototype,"zoneAwareStack",{configurable:!0,enumerable:!0,get:function(){this[aa]||(this[aa]=e(this.originalStack,this[c.symbol("zoneFrameNames")],!1));return this[aa]},set:function(O){this.originalStack=O;this[aa]=e(this.originalStack,this[c.symbol("zoneFrameNames")],
!1)}});const ca=["stackTraceLimit","captureStackTrace","prepareStackTrace"];(a=Object.keys(m))&&a.forEach(O=>{0===ca.filter(T=>T===O).length&&Object.defineProperty(f,O,{get:function(){return m[O]},set:function(T){m[O]=T}})});m.hasOwnProperty("stackTraceLimit")&&(m.stackTraceLimit=Math.max(m.stackTraceLimit,15),Object.defineProperty(f,"stackTraceLimit",{get:function(){return m.stackTraceLimit},set:function(O){return m.stackTraceLimit=O}}));m.hasOwnProperty("captureStackTrace")&&Object.defineProperty(f,
"captureStackTrace",{value:function(O,T){m.captureStackTrace(O,T)}});Object.defineProperty(f,"prepareStackTrace",{get:function(){return m.hx},set:function(O){return O&&"function"===typeof O?m.hx=function(T,Q){if(Q)for(let la=0;la<Q.length;la++)if("zoneCaptureStackTrace"===Q[la].getFunctionName()){Q.splice(la,1);break}return O.call(this,T,Q)}:m.hx=O}});if("disable"!==N){var ia=b.current.fork({name:"detect",onHandleError:function(O,T,Q,la){if(la.originalStack&&Error===f)for(O=la.originalStack.split(/\n/),
la=Q=T=!1;O.length;){let F=O.shift();if(/:\d+:\d+/.test(F)||"ZoneAwareError"===F){let M=F.split("(")[0].split("@")[0],W=g.transition;-1!==M.indexOf("ZoneAwareError")&&(-1!==M.indexOf("new ZoneAwareError")?(t=F,w=F.replace("new ZoneAwareError","new Error.ZoneAwareError")):(B=F,G=F.replace("Error.",""),-1===F.indexOf("Error.ZoneAwareError")&&(J=F.replace("ZoneAwareError","Error.ZoneAwareError"))),p[w]=g.Ts);-1!==M.indexOf("runGuarded")?Q=!0:-1!==M.indexOf("runTask")?la=!0:-1!==M.indexOf("run")?T=!0:
W=g.Ts;p[F]=W;if(T&&Q&&la){f.stackRewrite=!0;break}}}return!1}}).fork({name:"child",onScheduleTask:function(O,T,Q,la){return O.scheduleTask(Q,la)},onInvokeTask:function(O,T,Q,la,F,M){return O.invokeTask(Q,la,F,M)},onCancelTask:function(O,T,Q,la){return O.cancelTask(Q,la)},onInvoke:function(O,T,Q,la,F,M,W){return O.invoke(Q,la,F,M,W)}});b=Error.stackTraceLimit;Error.stackTraceLimit=100;ia.run(()=>{ia.runGuarded(()=>{const O=()=>{};ia.scheduleEventTask(k,()=>{ia.scheduleMacroTask(k,()=>{ia.scheduleMicroTask(k,
()=>{throw Error();},void 0,T=>{T.wd=O;T.invoke()});ia.scheduleMicroTask(k,()=>{throw Error();},void 0,T=>{T.wd=O;T.invoke()})},void 0,T=>{T.wd=O;T.invoke()},()=>{})},void 0,T=>{T.wd=O;T.invoke()},()=>{})})});Error.stackTraceLimit=b}})};function Id(){vc.__load_patch("ZoneAwarePromise",(a,b,c)=>{function d(){}function e(){let U=!1;return function(ea){return function(){U||(U=!0,ea.apply(null,arguments))}}}function f(U){if(U&&U.toString===Object.prototype.toString){const ea=U.constructor&&U.constructor.name;return(ea?ea:"")+": "+JSON.stringify(U)}return U?U.toString():Object.prototype.toString.call(U)}function g(U){c.onUnhandledError(U);try{const ea=b[la];"function"===typeof ea&&ea.call(this,U)}catch(ea){}}function k(U){return U}function m(U){return Ma.reject(U)}
function p(U,ea){return na=>{try{t(U,ea,na)}catch(Ja){t(U,!1,Ja)}}}function t(U,ea,na){var Ja=e();if(U===na)throw new TypeError("Promise resolved with itself");if(null===U[F]){var cb=null;try{if("object"===typeof na||"function"===typeof na)cb=na&&na.then}catch(Pa){return Ja(()=>{t(U,!1,Pa)})(),U}if(!1!==ea&&na instanceof Ma&&na.hasOwnProperty(F)&&na.hasOwnProperty(M)&&null!==na[F])w(na),t(U,na[F],na[M]);else if(!1!==ea&&"function"===typeof cb)try{cb.call(na,Ja(p(U,ea)),Ja(p(U,!1)))}catch(Pa){Ja(()=>
{t(U,!1,Pa)})()}else{U[F]=ea;Ja=U[M];U[M]=na;U[W]===W&&!0===ea&&(U[F]=U[pb],U[M]=U[Aa]);!1===ea&&na instanceof Error&&(cb=b.currentTask&&b.currentTask.data&&b.currentTask.data.__creationTrace__)&&aa(na,qb,{configurable:!0,enumerable:!1,writable:!0,value:cb});for(cb=0;cb<Ja.length;)B(U,Ja[cb++],Ja[cb++],Ja[cb++],Ja[cb++]);if(0==Ja.length&&0==ea){U[F]=0;ea=na;try{throw Error("Uncaught (in promise): "+f(na)+(na&&na.stack?"\n"+na.stack:""));}catch(Pa){ea=Pa}O&&(ea.throwOriginal=!0);ea.rejection=na;ea.promise=
U;ea.zone=b.current;ea.task=b.currentTask;ia.push(ea);c.scheduleMicroTask()}}}return U}function w(U){if(0===U[F]){try{var ea=b[Ia];ea&&"function"===typeof ea&&ea.call(this,{rejection:U[M],promise:U})}catch(na){}U[F]=!1;for(ea=0;ea<ia.length;ea++)U===ia[ea].promise&&ia.splice(ea,1)}}function B(U,ea,na,Ja,cb){w(U);const Pa=U[F],ub=Pa?"function"===typeof Ja?Ja:k:"function"===typeof cb?cb:m;ea.scheduleMicroTask("Promise.then",()=>{try{const pa=U[M],Ha=!!na&&W===na[W];Ha&&(na[Aa]=pa,na[pb]=Pa);const Ua=
ea.run(ub,void 0,Ha&&ub!==m&&ub!==k?[]:[pa]);t(na,!0,Ua)}catch(pa){t(na,!1,pa)}},na)}function G(U){const ea=U.prototype,na=N(ea,"then");if(!na||!1!==na.writable&&na.configurable){var Ja=ea.then;ea[Q]=Ja;U.prototype.then=function(cb,Pa){return(new Ma((ub,pa)=>{Ja.call(this,ub,pa)})).then(cb,Pa)};U[qa]=!0}}function J(U){return function(ea,na){ea=U.apply(ea,na);if(ea instanceof Ma)return ea;na=ea.constructor;na[qa]||G(na);return ea}}const N=Object.getOwnPropertyDescriptor,aa=Object.defineProperty,ca=
c.symbol,ia=[],O=!1!==a[ca("DISABLE_WRAPPING_UNCAUGHT_PROMISE_REJECTION")];var T=ca("Promise");const Q=ca("then");c.onUnhandledError=U=>{if(c.showUncaughtError()){const ea=U&&U.rejection;ea?console.error("Unhandled Promise rejection:",ea instanceof Error?ea.message:ea,"; Zone:",U.zone.name,"; Task:",U.task&&U.task.source,"; Value:",ea,ea instanceof Error?ea.stack:void 0):console.error(U)}};c.microtaskDrainDone=()=>{for(;ia.length;){const U=ia.shift();try{U.zone.runGuarded(()=>{if(U.throwOriginal)throw U.rejection;
throw U;})}catch(ea){g(ea)}}};const la=ca("unhandledPromiseRejectionHandler"),F=ca("state"),M=ca("value"),W=ca("finally"),Aa=ca("parentPromiseValue"),pb=ca("parentPromiseState"),qb=ca("currentTaskTrace"),Ia=ca("rejectionHandledHandler"),Vb=a.AggregateError;class Ma{static toString(){return"function ZoneAwarePromise() { [native code] }"}static resolve(U){return U instanceof Ma?U:t(new this(null),!0,U)}static reject(U){return t(new this(null),!1,U)}static any(U){if(!U||"function"!==typeof U[Symbol.iterator])return Promise.reject(new Vb([],
"All promises were rejected"));const ea=[];let na=0;try{for(let Pa of U)na++,ea.push(Ma.resolve(Pa))}catch(Pa){return Promise.reject(new Vb([],"All promises were rejected"))}if(0===na)return Promise.reject(new Vb([],"All promises were rejected"));let Ja=!1;const cb=[];return new Ma((Pa,ub)=>{for(let pa=0;pa<ea.length;pa++)ea[pa].then(Ha=>{Ja||(Ja=!0,Pa(Ha))},Ha=>{cb.push(Ha);na--;0===na&&(Ja=!0,ub(new Vb(cb,"All promises were rejected")))})})}static race(U){function ea(ub){Ja(ub)}function na(ub){cb(ub)}
let Ja,cb,Pa=new this((ub,pa)=>{Ja=ub;cb=pa});for(let ub of U)ub&&ub.then||(ub=this.resolve(ub)),ub.then(ea,na);return Pa}static all(U){return Ma.Fu(U)}static allSettled(U){return(this&&this.prototype instanceof Ma?this:Ma).Fu(U,{wE:ea=>({status:"fulfilled",value:ea}),KB:ea=>({status:"rejected",reason:ea})})}static Fu(U,ea){let na,Ja,cb=new this((Ha,Ua)=>{na=Ha;Ja=Ua}),Pa=2,ub=0;const pa=[];for(let Ha of U){Ha&&Ha.then||(Ha=this.resolve(Ha));const Ua=ub;try{Ha.then(kb=>{pa[Ua]=ea?ea.wE(kb):kb;Pa--;
0===Pa&&na(pa)},kb=>{ea?(pa[Ua]=ea.KB(kb),Pa--,0===Pa&&na(pa)):Ja(kb)})}catch(kb){Ja(kb)}Pa++;ub++}Pa-=2;0===Pa&&na(pa);return cb}constructor(U){if(!(this instanceof Ma))throw Error("Must be an instanceof Promise.");this[F]=null;this[M]=[];try{const ea=e();U&&U(ea(p(this,!0)),ea(p(this,!1)))}catch(ea){t(this,!1,ea)}}get [Symbol.toStringTag](){return"Promise"}get [Symbol.species](){return Ma}then(U,ea){var na,Ja=null==(na=this.constructor)?void 0:na[Symbol.species];Ja&&"function"===typeof Ja||(Ja=
this.constructor||Ma);na=new Ja(d);Ja=b.current;null==this[F]?this[M].push(Ja,na,U,ea):B(this,Ja,na,U,ea);return na}catch(U){return this.then(null,U)}finally(U){var ea,na=null==(ea=this.constructor)?void 0:ea[Symbol.species];na&&"function"===typeof na||(na=Ma);ea=new na(d);ea[W]=W;na=b.current;null==this[F]?this[M].push(na,ea,U,U):B(this,na,ea,U,U);return ea}}Ma.resolve=Ma.resolve;Ma.reject=Ma.reject;Ma.race=Ma.race;Ma.all=Ma.all;T=a[T]=a.Promise;a.Promise=Ma;const qa=ca("thenPatched");c.patchThen=
G;T&&(G(T),$b(a,"fetch",U=>J(U)));Promise[b.__symbol__("uncaughtPromiseErrors")]=ia;return Ma})};function Jd(){vc.__load_patch("toString",a=>{function b(){if("function"===typeof this){var k=this[d];if(k)return"function"===typeof k?c.call(k):Object.prototype.toString.call(k);if(this===Promise&&(k=a[e])||this===Error&&(k=a[f]))return c.call(k)}return c.call(this)}const c=Function.prototype.toString,d=ib("OriginalDelegate"),e=ib("Promise"),f=ib("Error");b[d]=c;Function.prototype.toString=b;const g=Object.prototype.toString;Object.prototype.toString=function(){return"function"===typeof Promise&&
this instanceof Promise?"[object Promise]":g.call(this)}})};var Kd;if(!(Kd="undefined"===typeof window)){var Ad=new Cd("mXaIFf",Gd(window));const a=yd();Kd=null===a?!0:a}if(Kd){var vc;const a=globalThis,b=!0===a[ib("forceDuplicateZoneCheck")];if(a.Zone&&(b||"function"!==typeof a.Zone.__symbol__))throw Error("Zone already loaded.");null!=a.Zone||(a.Zone=jb());vc=a.Zone;Hd();Id();Jd();uc();Qc();Xc();Yc()};function Ld(a){for(let b in a)if(a[b]===Ld)return b;throw Error("Could not find renamed property on target object.");};function Md(a){return{ma:a.ma,ea:a.ea||null,ba:a.ba,value:void 0}}function Nd(a){return{Wa:a.Wa||[],imports:a.imports||[]}}function Od(a){var b;(b=a.hasOwnProperty(Pd)?a[Pd]:null)||(b=a.hasOwnProperty(Td)?a[Td]:null);return b}function Ud(a){return(a=a&&(a[Pd]||a[Td]))?a:null}function Wd(a){return a&&(a.hasOwnProperty(Xd)||a.hasOwnProperty(Yd))?a[Xd]:null}var Pd=Ld({pa:Ld}),Xd=Ld({Xa:Ld}),Td=Ld({ZF:Ld}),Yd=Ld({aG:Ld});var Zd=class extends Error{constructor(a,b){super(`${`NG0${Math.abs(a)}`}${b?": "+b:""}`);this.code=a}};var $d=Ld({Da:Ld}),ae=Ld({Fa:Ld}),be=Ld({IG:Ld}),ce=Ld({Ya:Ld}),de=Ld({K:Ld}),ee=Ld({wf:Ld}),fe=Ld({wz:Ld});function ge(a){return"string"===typeof a?a:null==a?"":String(a)};function he(a){if("string"===typeof a)return a;if(Array.isArray(a))return"["+a.map(he).join(", ")+"]";if(null==a)return""+a;if(a.qD)return`${a.qD}`;if(a.name)return`${a.name}`;a=a.toString();if(null==a)return""+a;const b=a.indexOf("\n");return-1===b?a:a.substring(0,b)}function ie(a,b){return null==a||""===a?null===b?"":b:null==b||""===b?a:a+" "+b};/*

 The MIT License

 Copyright (c) 2014-2016 Google, Inc. http://angular.io

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
*/
function ke(){throw new Zd(-201,!1);};function le(a,b){return Object.is(a,b)};let me=null,ne=!1,oe=1;var pe=Symbol("SIGNAL");function qe(a){const b=me;me=a;return b}var re={version:0,yw:0,rc:!1,nc:void 0,bi:void 0,ke:void 0,eo:0,jd:void 0,Cg:void 0,hq:!1,iq:!1,kx:()=>!1,lx:()=>{},jq:()=>{},hv:()=>{}};function se(a){if(ne)throw Error("");if(null!==me){me.hv(a);var b=me.eo++;te(me);b<me.nc.length&&me.nc[b]!==a&&ue(me)&&ve(me.nc[b],me.ke[b]);me.nc[b]!==a&&(me.nc[b]=a,me.ke[b]=ue(me)?we(a,me,b):0);me.bi[b]=a.version}}
function xe(a){if(!ue(a)||a.rc)if(a.rc||a.yw!==oe)(a.kx(a)||ye(a))&&a.lx(a),a.rc=!1,a.yw=oe}function ze(a){if(void 0!==a.jd){var b=ne;ne=!0;try{for(const c of a.jd)c.rc||Ae(c)}finally{ne=b}}}function Be(){let a;return!1!==(null==(a=me)?void 0:a.hq)}function Ae(a){a.rc=!0;ze(a);let b;null==(b=a.jq)||b.call(a,a)}function Ce(a){a&&(a.eo=0);return qe(a)}
function De(a,b){qe(b);if(a&&void 0!==a.nc&&void 0!==a.ke&&void 0!==a.bi){if(ue(a))for(b=a.eo;b<a.nc.length;b++)ve(a.nc[b],a.ke[b]);for(;a.nc.length>a.eo;)a.nc.pop(),a.bi.pop(),a.ke.pop()}}function ye(a){te(a);for(let b=0;b<a.nc.length;b++){const c=a.nc[b],d=a.bi[b];if(d!==c.version)return!0;xe(c);if(d!==c.version)return!0}return!1}function Ee(a){te(a);if(ue(a))for(let b=0;b<a.nc.length;b++)ve(a.nc[b],a.ke[b]);a.nc.length=a.bi.length=a.ke.length=0;a.jd&&(a.jd.length=a.Cg.length=0)}
function we(a,b,c){Fe(a);te(a);if(0===a.jd.length)for(let d=0;d<a.nc.length;d++)a.ke[d]=we(a.nc[d],a,d);a.Cg.push(c);return a.jd.push(b)-1}function ve(a,b){Fe(a);te(a);if(1===a.jd.length)for(var c=0;c<a.nc.length;c++)ve(a.nc[c],a.ke[c]);c=a.jd.length-1;a.jd[b]=a.jd[c];a.Cg[b]=a.Cg[c];a.jd.length--;a.Cg.length--;b<a.jd.length&&(c=a.Cg[b],a=a.jd[b],te(a),a.ke[c]=b)}function ue(a){let b,c;return a.iq||0<(null!=(c=null==a?void 0:null==(b=a.jd)?void 0:b.length)?c:0)}
function te(a){null!=a.nc||(a.nc=[]);null!=a.ke||(a.ke=[]);null!=a.bi||(a.bi=[])}function Fe(a){null!=a.jd||(a.jd=[]);null!=a.Cg||(a.Cg=[])};function Ge(a){const b=Object.create(He);b.lB=a;a=()=>{xe(b);se(b);if(b.value===Ie)throw b.error;return b.value};a[pe]=b;return a}
const Je=Symbol("UNSET"),Ke=Symbol("COMPUTING"),Ie=Symbol("ERRORED"),He=Object.assign({},re,{value:Je,rc:!0,error:null,Cq:le,kx(a){return a.value===Je||a.value===Ke},lx(a){if(a.value===Ke)throw Error("Detected cycle in computations.");const b=a.value;a.value=Ke;const c=Ce(a);let d;try{d=a.lB()}catch(e){d=Ie,a.error=e}finally{De(a,c)}b!==Je&&b!==Ie&&d!==Ie&&a.Cq(b,d)?a.value=b:(a.value=d,a.version++)}});function Le(){throw Error();};function Me(a){const b=Object.create(Se);b.value=a;a=()=>{se(b);return b.value};a[pe]=b;return a}function Te(a,b){Be()||Le();a.Cq(a.value,b)||(a.value=b,a.version++,oe++,ze(a))}var Se=Object.assign({},re,{Cq:le,value:void 0});function Ue(a,b,c){const d=Object.create(Ve);c&&(d.hq=!0);d.yn=a;d.Oc=b;const e=f=>{d.Zi=f};d.Hj={notify:()=>Ae(d),run:()=>{if(null!==d.yn){if(ne)throw Error("Schedulers cannot synchronously execute watches while scheduling.");d.rc=!1;if(!d.cw||ye(d)){d.cw=!0;var f=Ce(d);try{d.Zi(),d.Zi=We,d.yn(e)}finally{De(d,f)}}}},be:()=>d.Zi(),destroy:()=>{if(null!==d.yn||null!==d.Oc)Ee(d),d.Zi(),d.yn=null,d.Oc=null,d.Zi=We},[pe]:d};return d.Hj}
const We=()=>{},Ve=Object.assign({},re,{iq:!0,hq:!1,jq:a=>{null!==a.Oc&&a.Oc(a.Hj)},cw:!1,Zi:We});let Xe;function Ye(a){const b=Xe;Xe=a;return b}function Ze(a,b,c){if((a=Od(a))&&"root"==a.ea)return void 0===a.value?a.value=a.ba():a.value;if(c&8)return null;if(void 0!==b)return b;ke()};const $e=Ld({gt:Ld});function af(a){a.gt=af;a.toString=function(){return he(this())};return a}function bf(a){return cf(a)?a():a}function cf(a){return"function"===typeof a&&a.hasOwnProperty($e)&&a.gt===af};var df=class{constructor(a,b){this.C=a;this.pa=void 0;"number"==typeof b?this.wf=b:void 0!==b&&(this.pa=Md({ma:this,ea:b.ea||"root",ba:b.ba}))}get Eb(){return this}toString(){return`InjectionToken ${this.C}`}};var ef=globalThis;var ff={};const gf=/\n/gm;let hf=void 0;function jf(a){const b=hf;hf=a;return b}function kf(a,b=0){if(void 0===hf)throw new Zd(-203,!1);return null===hf?Ze(a,void 0,b):hf.get(a,b&8?null:void 0,b)}function r(a,b=0){return(Xe||kf)(bf(a),b)}function lf(a,b=0){return r(a,mf(b))}function mf(a){return"undefined"===typeof a||"number"===typeof a?a:0|(a.optional&&8)|(a.host&&1)|(a.self&&2)|(a.wG&&4)}
function nf(a){const b=[];for(let c=0;c<a.length;c++){const d=bf(a[c]);if(Array.isArray(d)){if(0===d.length)throw new Zd(900,!1);let e=void 0,f=0;for(let g=0;g<d.length;g++){const k=d[g],m=k.__NG_DI_FLAG__;"number"===typeof m?-1===m?e=k.ma:f|=m:e=k}b.push(r(e,f))}else b.push(r(d))}return b}function of(a,b){a.__NG_DI_FLAG__=b;a.prototype.__NG_DI_FLAG__=b;return a}
function pf(a,b,c=null){a=a&&"\n"===a.charAt(0)&&"\u0275"==a.charAt(1)?a.slice(2):a;var d=he(b);if(Array.isArray(b))d=b.map(he).join(" -> ");else if("object"===typeof b){d=[];for(let e in b)if(b.hasOwnProperty(e)){let f=b[e];d.push(e+":"+("string"===typeof f?JSON.stringify(f):he(f)))}d=`{${d.join(", ")}}`}return`${"R3InjectorError"}${c?"("+c+")":""}[${d}]: ${a.replace(gf,"\n  ")}`};var qf=new df("");var rf=new df("",-1);var sf=new df("");var tf=class{get(a,b=ff){if(b===ff)throw a=Error(`NullInjectorError: No provider for ${he(a)}!`),a.name="NullInjectorError",a;return b}};function uf(a,b,c){let d=0;for(;d<c.length;){var e=c[d];if("number"===typeof e){if(0!==e)break;d++;e=c[d++];var f=c[d++];const g=c[d++];a.setAttribute(b,f,g,e)}else f=c[++d],64===e.charCodeAt(0)?a.setProperty(b,e,f):a.setAttribute(b,e,f),d++}}function vf(a,b){if(null!==b&&0!==b.length)if(null===a||0===a.length)a=b.slice();else{let c=-1;for(let d=0;d<b.length;d++){const e=b[d];"number"===typeof e?c=e:0!==c&&(-1===c||2===c?wf(a,c,e,b[++d]):wf(a,c,e,null))}}return a}
function wf(a,b,c,d){let e=0,f=a.length;if(-1===b)f=-1;else for(;e<a.length;){var g=a[e++];if("number"===typeof g)if(g===b){f=-1;break}else if(g>b){f=e-1;break}}for(;e<a.length;){g=a[e];if("number"===typeof g)break;else if(g===c){null!==d&&(a[e+1]=d);return}e++;null!==d&&e++}-1!==f&&(a.splice(f,0,b),e=f+1);a.splice(e++,0,c);null!==d&&a.splice(e++,0,d)};function xf(a){return 4===a.type&&"ng-template"!==a.value}
function yf(a,b,c){let d=4;const e=a.attrs;if(null!==e)a:{for(var f=0;f<e.length;f++){var g=e[f];if(3===g||4===g||6===g){g=f;break a}}g=e.length}else g=0;f=!1;for(let G=0;G<b.length;G++){var k=b[G];if("number"===typeof k){if(!f&&0!==(d&1)&&0!==(k&1))return!1;f&&0===(k&1)||(f=!1,d=k|d&1)}else if(!f)if(d&4){if(d=2|d&1,""!==k&&k!==(4!==a.type||c?a.value:"ng-template")||""===k&&1===b.length){if(0===(d&1))return!1;f=!0}}else if(d&8){var m;if(!(m=null===e)){a:{m=void 0;var p=a,t=0;if(c)for(;t<e.length&&
"string"===typeof e[t];t+=2){if(p="class"===e[t]){b:{p=e[t+1].toLowerCase();for(var w=0,B=p.length;;){w=p.indexOf(k,w);if(-1===w){p=w;break b}if(0===w||32>=p.charCodeAt(w-1)){const J=k.length;if(w+J===B||32>=p.charCodeAt(w+J)){p=w;break b}}w+=1}}p=-1!==p}if(p){k=!0;break a}}else if(xf(p)){k=!1;break a}t=e.indexOf(1,t);if(-1<t)for(;++t<e.length&&"string"===typeof(m=e[t]);)if(m.toLowerCase()===k){k=!0;break a}k=!1}m=!k}if(m){if(0===(d&1))return!1;f=!0}}else{m=b[++G];a:if(null===e)k=-1;else if(t=0,c||
!xf(a)){for(p=!1;t<e.length;){B=e[t];if(B===k){k=t;break a}if(3===B||6===B)p=!0;else if(1===B||2===B){for(B=e[++t];"string"===typeof B;)B=e[++t];continue}else if(4===B)break;else if(0===B){t+=4;continue}t+=p?1:2}k=-1}else b:{t=e.indexOf(4);if(-1<t)for(t++;t<e.length;){p=e[t];if("number"===typeof p)break;if(p===k){k=t;break b}t++}k=-1}if(-1===k){if(0===(d&1))return!1;f=!0}else if(""!==m&&(k=k>g?"":e[k+1].toLowerCase(),d&2&&m!==k)){if(0===(d&1))return!1;f=!0}}}return 0===(d&1)||f}
function zf(a,b,c=!1){for(let d=0;d<b.length;d++)if(yf(a,b[d],c))return!0;return!1}function Bf(a,b){return a?":not("+b.trim()+")":b}function Cf(a){let b=a[0],c=1,d=2,e="",f=!1;for(;c<a.length;){let g=a[c];if("string"===typeof g)if(d&2){const k=a[++c];e+="["+g+(0<k.length?'="'+k+'"':"")+"]"}else d&8?e+="."+g:d&4&&(e+=" "+g);else""!==e&&0!==(g&1)&&(b+=Bf(f,e),e=""),d=g,f=f||0!==(d&1);c++}""!==e&&(b+=Bf(f,e));return b};function Df(a){return{toString:a}.toString()};var Ef={},Ff=[];function Gf(a){return Df(()=>{var b=Jf(a),c;b=Object.assign({},b,{za:a.za,Aa:a.Aa,ta:a.ta,Ca:a.Ca||null,Zb:a.Zb,Br:0===a.Ra,Dv:null,ax:null,Ha:b.ka&&a.Ha||null,Iq:null,ii:null!=(c=a.ii)?c:!1,data:a.data||{},pb:a.pb||0,styles:a.styles||Ff,Gb:null,Pj:a.Pj||null,ub:null,id:""});Kf(b);c=a.Ha;b.Dv=Lf(c,!1);b.ax=Lf(c,!0);c=0;const d=[b.ha,b.Zb,b.Za,b.Ka,b.Ca,b.Aa,b.za,b.pb,b.ka,b.ii,b.ib,JSON.stringify(b.inputs),JSON.stringify(b.outputs),Object.getOwnPropertyNames(b.type.prototype),!!b.ec,!!b.fb].join("|");
for(const e of d)c=Math.imul(31,c)+e.charCodeAt(0)<<0;b.id="c"+(c+2147483648);return b})}function Mf(a){return a[$d]||a[ae]||null}function Nf(a){return null!==a}function Of(a){return Df(()=>({type:a.type,Kk:a.Kk||Ff,xB:a.xB||Ff,imports:a.imports||Ff,exports:a.exports||Ff,CG:null,Pj:a.Pj||null,id:a.id||null}))}
function Pf(a,b){if(null==a)return Ef;const c={};for(const e in a)if(a.hasOwnProperty(e)){var d=a[e];let f,g=0;if(Array.isArray(d)){g=d[0];f=d[1];let k;d=null!=(k=d[2])?k:f}else f=d;b?(c[f]=0!==g?[e,g]:e,b[f]=d):c[f]=e}return c}function Qf(a){return Df(()=>{const b=Jf(a);Kf(b);return b})}function Rf(a){return a[be]||null}
function Jf(a){const b={};return{type:a.type,Vr:null,ba:null,Ia:a.Ia||null,Za:a.Za||0,Ka:a.Ka||null,ec:a.ec||null,ej:b,De:null,rC:a.inputs||Ef,ib:a.ib||null,ka:!0===a.ka,ii:!0===a.ii,ha:a.ha||Ff,fb:a.fb||null,features:a.features||null,Ao:null,nj:null,ol:null,inputs:Pf(a.inputs,b),outputs:Pf(a.outputs),BF:null}}function Kf(a){let b;null==(b=a.features)||b.forEach(c=>c(a))}function Lf(a,b){if(!a)return null;const c=b?Rf:Mf;return()=>("function"===typeof a?a():a).map(d=>c(d)).filter(Nf)};function Sf(a){return a.hasOwnProperty(de)?a[de]:null};function Tf(a,b){a.forEach(c=>Array.isArray(c)?Tf(c,b):b(c))}function Uf(a,b){return b>=a.length-1?a.pop():a.splice(b,1)[0]}function Vf(a,b,c){let d=Wf(a,b);if(0<=d)a[d|1]=c;else{var e=d=~d;let f=a.length;if(f==e)a.push(b,c);else if(1===f)a.push(c,a[0]),a[0]=b;else{f--;for(a.push(a[f-1],a[f]);f>e;)a[f]=a[f-2],f--;a[e]=b;a[e+1]=c}}return d}function Xf(a,b){b=Wf(a,b);if(0<=b)return a[b|1]}
function Wf(a,b){let c=0,d=a.length>>1;for(;d!==c;){const e=c+(d-c>>1),f=a[e<<1];if(b===f)return e<<1;f>b?d=e:c=e+1}return~(d<<1)};function Yf(...a){return{Zj:Zf(a),HG:!0}}function Zf(...a){const b=[],c=new Set;let d;const e=f=>{b.push(f)};Tf(a,f=>{$f(f,e,[],c)&&(d||(d=[]),d.push(f))});void 0!==d&&ag(d,e);return b}function ag(a,b){for(let c=0;c<a.length;c++){const {Aj:d,Wa:e}=a[c];bg(e,f=>{b(f,d)})}}
function $f(a,b,c,d){a=bf(a);if(!a)return!1;let e=null;var f=Wd(a);const g=!f&&(a[$d]||null);if(f||g){if(g&&!g.ka)return!1;e=a}else{var k=a.Aj;if(f=Wd(k))e=k;else return!1}k=d.has(e);if(g){if(k)return!1;d.add(e);if(g.Ha){f="function"===typeof g.Ha?g.Ha():g.Ha;for(var m of f)$f(m,b,c,d)}}else if(f){if(null!=f.imports&&!k){d.add(e);let p;Tf(f.imports,t=>{$f(t,b,c,d)&&(p||(p=[]),p.push(t))});void 0!==p&&ag(p,b)}k||(m=Sf(e)||(()=>new e),b({oa:e,Fc:m,Ac:Ff},e),b({oa:sf,nb:e,Eb:!0},e),b({oa:qf,nb:()=>r(e),
Eb:!0},e));f=f.Wa;if(null!=f&&!k){const p=a;bg(f,t=>{b(t,p)})}}else return!1;return e!==a&&void 0!==a.Wa}function bg(a,b){for(let c of a)c&&c.Zj&&(c=c.Zj),Array.isArray(c)?bg(c,b):b(c)}var cg=Ld({oa:String,nb:Ld});function dg(a){return null!==a&&"object"==typeof a&&cg in a}function eg(a){return"function"===typeof a};var fg=new df("");const gg={},hg={};let ig=void 0;function jg(){void 0===ig&&(ig=new tf);return ig}var kg=class{};function lg(a,b){b=bf(b);let c=eg(b)?b:bf(b&&b.oa);const d=mg(b);if(!eg(b)&&!0===b.Eb){let e=a.D.get(c);e||(e=ng(void 0,gg,!0),e.ba=()=>nf(e.Eb),a.D.set(c,e));c=b;e.Eb.push(b)}a.D.set(c,d)}function og(a){if(a.Ga)throw new Zd(205,!1);}function pg(a){const b=qe(null),c=jf(a),d=Ye();try{const e=a.get(qf,Ff,2);for(const f of e)f()}finally{jf(c),Ye(d),qe(b)}}
var vg=class extends kg{get C(){return this.Ga}constructor(a,b,c,d){super();this.parent=b;this.source=c;this.N=d;this.D=new Map;this.G=new Set;this.F=[];this.Ga=!1;qg(a,e=>lg(this,e));this.D.set(rf,ng(void 0,this));d.has("environment")&&this.D.set(kg,ng(void 0,this));a=this.D.get(fg);null!=a&&"string"===typeof a.value&&this.N.add(a.value);this.J=new Set(this.get(sf,Ff,2))}destroy(){og(this);this.Ga=!0;const a=qe(null);try{for(const c of this.G)c.va();const b=this.F;this.F=[];for(const c of b)c()}finally{this.D.clear(),
this.G.clear(),this.J.clear(),qe(a)}}ob(a){og(this);this.F.push(a);return()=>{const b=this.F.indexOf(a);-1!==b&&this.F.splice(b,1)}}get(a,b=ff,c=0){og(this);if(a.hasOwnProperty(fe))return a[fe](this);c=mf(c);const d=jf(this),e=Ye();try{if(!(c&4)){let B=this.D.get(a);if(void 0===B){const G=("function"===typeof a||"object"===typeof a&&a instanceof df)&&Od(a);var f;if(f=G)if(G.ea){var g=bf(G.ea);f="string"===typeof g?"any"===g||this.N.has(g):this.J.has(g)}else f=!1;f?B=ng(ug(a),gg):B=null;this.D.set(a,
B)}if(null!=B){a:{b=B;const G=qe(null);try{b.value===gg&&(b.value=hg,b.value=b.ba());var k;if(k="object"===typeof b.value&&b.value){var m=b.value;k=null!==m&&"object"===typeof m&&"function"===typeof m.va}k&&this.G.add(b.value);var p=b.value;break a}finally{qe(G)}p=void 0}return p}}const w=c&2?jg():this.parent;b=c&8&&b===ff?null:b;return w.get(a,b)}catch(w){if("NullInjectorError"===w.name){(w.ngTempTokenPath=w.ngTempTokenPath||[]).unshift(he(a));if(d)throw w;p=w;var t=this.source;const B=p.ngTempTokenPath;
a.__source&&B.unshift(a.__source);p.message=pf("\n"+p.message,B,t);p.ngTokenPath=B;p.ngTempTokenPath=null;throw p;}throw w;}finally{Ye(e),jf(d)}}toString(){const a=[],b=this.D;for(const c of b.keys())a.push(he(c));return`R3Injector[${a.join(", ")}]`}};function ug(a){var b=Od(a);b=null!==b?b.ba:Sf(a);if(null!==b)return b;if(a instanceof df)throw new Zd(204,!1);if(a instanceof Function)return wg(a);throw new Zd(204,!1);}
function wg(a){if(0<a.length)throw new Zd(204,!1);const b=Ud(a);return null!==b?()=>b.ba(a):()=>new a}function mg(a){if(dg(a))return ng(void 0,a.nb);a=xg(a);return ng(a,gg)}function xg(a){var b=void 0;if(eg(a))return b=bf(a),Sf(b)||ug(b);if(dg(a))b=()=>bf(a.nb);else if(a&&a.Fc)b=()=>a.Fc(...nf(a.Ac||[]));else if(a&&a.Ab)b=()=>r(bf(a.Ab));else{const c=bf(a&&(a.Ec||a.oa));if(a.Ac)b=()=>new c(...nf(a.Ac));else return Sf(c)||ug(c)}return b}function ng(a,b,c=!1){return{ba:a,value:b,Eb:c?[]:void 0}}
function qg(a,b){for(const c of a)Array.isArray(c)?qg(c,b):c&&c&&c.Zj?qg(c.Zj,b):b(c)};function yg(a,b){a instanceof vg&&og(a);a=jf(a);const c=Ye();try{return b()}finally{jf(a),Ye(c)}}function zg(){if(void 0===Xe&&null==hf)throw new Zd(-203,!1);};function Ag(a){return Array.isArray(a)&&"object"===typeof a[1]}function Bg(a){return Array.isArray(a)&&!0===a[1]};var Cg=class{constructor(a,b,c){this.jx=a;this.pq=b;this.wn=c}};function Dg(){return Eg}function Eg(a){a.type.prototype.tc&&(a.Ao=Fg);return Gg}Dg.Mw=!0;function Gg(){const a=this.__ngSimpleChanges__||null,b=null==a?void 0:a.current;if(b){const c=a.Rr;if(c===Ef)a.Rr=b;else for(let d in b)c[d]=b[d];a.current=null;this.tc(b)}}function Fg(a,b,c,d,e){d=this.ej[d];var f;(f=a.__ngSimpleChanges__||null)||(f=a.__ngSimpleChanges__={Rr:Ef,current:null});var g=f;f=g.current||(g.current={});g=g.Rr;const k=g[d];f[d]=new Cg(k&&k.pq,c,g===Ef);null!==b?b.Iu(b,c):a[e]=c};function Hg(a){for(;Array.isArray(a);)a=a[0];return a}function Ig(a,b){return Hg(b[a.index])}function Jg(a,b){a=b[a];return Ag(a)?a:a[0]}function Kg(a,b){return null===b||void 0===b?null:a[b]}function Lg(a){let b;return!!(a[2]&9216||(null==(b=a[23])?0:b.rc))}function Mg(a){let b;null==(b=a[10].Xi)||b.notify(7);a[2]&64&&(a[2]|=1024);Lg(a)&&Ng(a)}function Ng(a){let b;null==(b=a[10].Xi)||b.notify(0);for(a=Og(a);null!==a&&!(a[2]&8192);){a[2]|=8192;if(128!==(a[2]&128))break;a=Og(a)}}
function Pg(a,b){if(256===(a[2]&256))throw new Zd(911,!1);null===a[21]&&(a[21]=[]);a[21].push(b)}function Og(a){a=a[3];return Bg(a)?a[3]:a};var Rg=Qg(null),Sg=null;let Tg=!1;function Ug(){return Rg.Sf}function u(a){Rg.Bh=a;return a[8]}function x(a){Rg.Bh=null;return a}function Vg(){let a=Rg.Eh;for(;null!==a&&64===a.type;)a=a.parent;return a}function Wg(a,b){const c=Rg;c.Eh=a;c.Rf=b}function Xg(){const a=Rg;let b=a.Wm;-1===b&&(b=a.Wm=a.ub.aq);return b}function Yg(a){const b=Rg,c=b.ed;b.ed+=a;return c}function Zg(a){const b=Rg.dj;return-1===b?null:a[b]}
function $g(a,b,c){if(c&4){for(;!(b=b.parent,null!==b||c&1);){b=a[1];b=2===b.type?b.zv:1===b.type?a[5]:null;if(null===b)break;a=a[14];if(b.type&10)break}if(null===b)return!1}c=Rg=ah();c.Eh=b;c.Sf=a;return!0}function bh(a){const b=ah(),c=a[1];Rg=b;b.Eh=c.firstChild;b.Sf=a;b.ub=c;b.Bh=a;b.ed=c.aq;b.fw=!1}function ah(){const a=Rg,b=null===a?null:a.Hf;return null===b?Qg(a):b}
function Qg(a){const b={Eh:null,Rf:!0,Sf:null,ub:null,selectedIndex:-1,Bh:null,qn:0,Vk:null,dj:-1,Wm:-1,ed:-1,ce:0,parent:a,Hf:null,fw:!1};null!==a&&(a.Hf=b);return b}function ch(){const a=Rg;Rg=a.parent;a.Eh=null;a.Sf=null;return a}function dh(){const a=ch();a.Rf=!0;a.ub=null;a.selectedIndex=-1;a.Bh=null;a.qn=0;a.dj=-1;a.Vk=null;a.Wm=-1;a.ed=-1;a.ce=0}function eh(){const a=Rg;return a.ub.data[a.selectedIndex]}function fh(){Rg.Vk="svg"}let gh=!0;function hh(a,b){for(let d=b.nd,e=b.we;d<e;d++){var c=a.data[d].type.prototype;b=c.yj;const f=c.Kw,g=c.pd,k=c.Lw;c=c.va;if(b){let m,p;(null!=(p=(m=a).Sk)?p:m.Sk=[]).push(-d,b)}if(f){let m,p;(null!=(p=(m=a).Sk)?p:m.Sk=[]).push(d,f);let t,w;(null!=(w=(t=a).kq)?w:t.kq=[]).push(d,f)}if(g){let m,p;(null!=(p=(m=a).lm)?p:m.lm=[]).push(-d,g)}if(k){let m,p;(null!=(p=(m=a).lm)?p:m.lm=[]).push(d,k);let t,w;(null!=(w=(t=a).Ns)?w:t.Ns=[]).push(d,k)}if(null!=c){let m,p;(null!=(p=(m=a).Wk)?p:m.Wk=[]).push(d,c)}}}
function ih(a,b,c,d){(a[2]&3)===c&&jh(a,b,c,d)}function kh(a,b){let c=a[2];(c&3)===b&&(a[2]=(c&16383)+1)}function jh(a,b,c,d){var e=void 0!==d?a[17]&65535:0;const f=null!=d?d:-1,g=b.length-1;let k=0;for(;e<g;e++)if("number"===typeof b[e+1]){if(k=b[e],null!=d&&k>=d)break}else{0>b[e]&&(a[17]+=65536);if(k<f||-1==f){const m=0>b[e],p=b[e+1],t=a[m?-b[e]:b[e]];m?a[2]>>14<a[17]>>16&&(a[2]&3)===c&&(a[2]+=16384,lh(t,p)):lh(t,p);a[17]=(a[17]&4294901760)+e+2}e++}}
function lh(a,b){const c=qe(null);try{b.call(a)}finally{qe(c)}};var mh=class{constructor(a,b){var c=z;this.ba=a;this.F=!1;this.J=b;this.G=c}};function nh(a,b){for(a>>=16;0<a;)b=b[14],a--;return b};let oh=!0;function ph(a){const b=oh;oh=a;return b}let qh=0;const rh={};function sh(a,b,c){let d;"string"===typeof c?d=c.charCodeAt(0)||0:c.hasOwnProperty(ee)&&(d=c[ee]);null==d&&(d=c[ee]=qh++);c=d&255;b.data[a+(c>>5)]|=1<<c}function th(a,b){var c=uh(a,b);if(-1!==c)return c;c=b[1];c.Bc&&(a.zd=b.length,vh(c.data,a),vh(b,null),vh(c.qg,null));c=wh(a,b);a=a.zd;if(-1!==c){const d=c&32767,e=nh(c,b),f=e[1].data;for(let g=0;8>g;g++)b[a+g]=e[d+g]|f[d+g]}b[a+8]=c;return a}
function vh(a,b){a.push(0,0,0,0,0,0,0,0,b)}function uh(a,b){return-1===a.zd||a.parent&&a.parent.zd===a.zd||null===b[a.zd+8]?-1:a.zd}function wh(a,b){if(a.parent&&-1!==a.parent.zd)return a.parent.zd;a=0;let c=b;for(;null!==c;){b=xh(c);if(null===b)break;a++;c=c[14];if(-1!==b.zd)return b.zd|a<<16}return-1}
function yh(a){var b=Vg();if("class"===a)return b.Yi;if("style"===a)return b.styles;if(b=b.attrs){const c=b.length;let d=0;for(;d<c;){const e=b[d];if(3===e||4===e||6===e)break;if(0===e)d+=2;else if("number"===typeof e)for(d++;d<c&&"string"===typeof b[d];)d++;else{if(e===a)return b[d+1];d+=2}}}return null}function zh(a,b,c,d){c&8&&void 0===d&&(d=null);if(0===(c&3)){a=a[9];const e=Ye();try{return a?a.get(b,d,c&8):Ze(b,d,c&8)}finally{Ye(e)}}c&8||void 0!==d?b=d:(ke(),b=void 0);return b}
function Ah(a,b,c,d=0,e){if(null!==a){if(b[2]&2048&&!(d&2)){a:{var f=rh;var g=a;let k=b;for(;null!==g&&null!==k&&k[2]&2048&&!(k[2]&512);){const m=Bh(g,k,c,d|2);if(m!==rh){f=m;break a}g=g.parent;if(!g){if(g=k[20])if(g=g.get(c,rh,d),g!==rh){f=g;break a}g=xh(k);k=k[14]}}}if(f!==rh)return f}a=Bh(a,b,c,d);if(a!==rh)return a}return zh(b,c,d,e)}
function Bh(a,b,c,d){var e=rh;if("string"===typeof c)var f=c.charCodeAt(0)||0;else f=c.hasOwnProperty(ee)?c[ee]:void 0,f="number"===typeof f?0<=f?f&255:Ch:f;if("function"===typeof f){if(!$g(b,a,d))return d&1?d&8||void 0!==e?b=e:(ke(),b=void 0):b=zh(b,c,d,e),b;try{var g=f(d);if(null!=g||d&8)return g;ke()}finally{ch()}}else if("number"===typeof f){g=null;let m=uh(a,b);let p=d&1?b[15][5]:null;if(-1===m||d&4){var k=-1===m?wh(a,b):b[m+8];-1===k||d&2?m=-1:(g=b[1],m=k&32767,b=nh(k,b))}for(;-1!==m;){a=b[1];
if(a.data[m+(f>>5)]&1<<f){k=b[1];const t=k.data[m+8],w=Dh(t,k,c,null==g?-1<t.Jf&&oh:g!=k&&0!==(t.type&3),d&1&&p===t);k=null!==w?Eh(b,k,w,t):rh;if(k!==rh)return k}k=b[m+8];-1!==k&&!(d&2||d&1&&b[1].data[m+8]===p)&&b[m+(f>>5)]&1<<f?(g=a,m=k&32767,b=nh(k,b)):m=-1}}return e}function Dh(a,b,c,d,e){var f=a.Jg;const g=b.data;var k=f&1048575;b=a.nd;a=a.we;f>>=20;d=d?k:k+f;for(k=e?k+f:a;d<k;d++)if(f=g[d],d<b&&c===f||d>=b&&f.type===c)return d;return e&&(e=g[b])&&e.ta&&e.type===c?b:null}
function Eh(a,b,c,d){let e=a[c];const f=b.data;if(e instanceof mh){const k=e;if(k.F)throw a=f[c],new Zd(-200,"function"===typeof a?a.name||a.toString():"object"===typeof a&&null!=a&&"function"===typeof a.type?a.type.name||a.type.toString():ge(a));const m=ph(k.J);k.F=!0;const p=k.G?Ye(k.G):null;$g(a,d,0);try{if(e=a[c]=k.ba(void 0,f,a,d),b.Bc&&c>=d.nd){var g=f[c];const t=g.type.prototype.Fb,w=g.type.prototype.zj;if(g.type.prototype.tc){const B=Eg(g);let G;(null!=(G=b.lf)?G:b.lf=[]).push(c,B);let J;
(null!=(J=b.Ig)?J:b.Ig=[]).push(c,B)}if(t){let B;(null!=(B=b.lf)?B:b.lf=[]).push(-c,t)}if(w){let B;(null!=(B=b.lf)?B:b.lf=[]).push(c,w);let G;(null!=(G=b.Ig)?G:b.Ig=[]).push(c,w)}}}finally{null!==p&&Ye(p),ph(m),k.F=!1,ch()}}return e}var Fh=class{constructor(a,b){this.F=a;this.D=b}get(a,b,c){return Ah(this.F,this.D,a,mf(c),b)}};function Ch(){return new Fh(Vg(),Ug())}
function Mh(a){return Df(()=>{var b=a.prototype.constructor;b=b[de]||Nh(b);const c=Object.prototype;let d=Object.getPrototypeOf(a.prototype).constructor;for(;d&&d!==c;){const e=d[de]||Nh(d);if(e&&e!==b)return e;d=Object.getPrototypeOf(d)}return e=>new e})}function Nh(a){return cf(a)?()=>{const b=Nh(bf(a));return b&&b()}:Sf(a)}function xh(a){const b=a[1],c=b.type;return 2===c?b.zv:1===c?a[5]:null};(new df("")).wf=a=>{const b=Vg();if(null===b)throw new Zd(204,!1);if(b.type&2)return b.value;if(a&8)return null;throw new Zd(204,!1);};function Oh(a){return function(...b){if(a){b=a(...b);for(const c in b)this[c]=b[c]}}}function Ph(a){return Df(()=>{function b(...d){if(this instanceof b)return c.apply(this,d),this;const e=new b(...d);return function(f,g,k){for(g=f.hasOwnProperty("__parameters__")?f.__parameters__:Object.defineProperty(f,"__parameters__",{value:[]}).__parameters__;g.length<=k;)g.push(null);(g[k]=g[k]||[]).push(e);return f}}const c=Oh(a);return b})};var Qh=of(Ph(a=>({ma:a})),-1),Rh=of(Ph(),8);of(Ph(),2);var Sh=of(Ph(),4);of(Ph(),1);Ld({oa:String,nb:Ld});function Th(a,b=null,c=null,d){a=Uh(a,b,c,d);pg(a);return a}function Uh(a,b=null,c=null,d,e=new Set){c=[c||Ff,Yf(a)];d=d||("object"===typeof a?void 0:he(a));return new vg(c,b||jg(),d||null,e)};function Vh(a){if(Array.isArray(a))return Th({name:""},void 0,a,"");let b;const c=null!=(b=a.name)?b:"";return Th({name:c},a.parent,a.Wa,c)}var Wh=class{},Xh=new tf;Wh.pa=Md({ma:Wh,ea:"any",ba:()=>r(rf)});Wh.wf=-1;var Yh=class{};var Zh=class{},$h=class{};var ai=class{},bi=new df("",{ea:"root",ba:()=>!1}),ci=new df("");let di=void 0;const fi=new Map;let gi=0;function hi(a,b){Ag(b)?(a.__ngContext__=b[19],fi.set(b[19],b)):a.__ngContext__=b};function ii(a){for(;null!==a&&!Bg(a);)a=a[4];return a};var ji=new df("",{ea:"root",ba:()=>"ng"}),ki=new df(""),li=new df("",{ea:"platform",ba:()=>"unknown"});new df("");var mi=new df(""),ni=new df("",{ea:"root",ba:()=>{let a,b;if(void 0!==di)var c=di;else if("undefined"!==typeof document)c=document;else throw new Zd(210,!1);return(null==(a=c.body)?void 0:null==(b=a.querySelector("[ngCspNonce]"))?void 0:b.getAttribute("ngCspNonce"))||null}}),oi={Nu:[16,32,48,64,96,128,256,384,640,750,828,1080,1200,1920,2048,3840],kG:30,EF:!1,DF:!1};
new df("",{ea:"root",ba:()=>oi});var pi=class{},qi=class{};function ri(a,b){return new si(Ig(a,b))}var si=class{constructor(a){this.R=a}};si.wf=function(){return ri(Vg(),Ug())};function ti(a){return a instanceof si?a.R:a};var ui=class{},vi=class{constructor(){this.Xk=null}};vi.wf=()=>{const a=Ug(),b=Jg(Vg().index,a);return(Ag(b)?b:a)[11]};var wi=class{constructor(){this.st=console}handleError(a){const b=this.Oz(a);this.st.error("ERROR",a);b&&this.st.error("ORIGINAL ERROR",b)}Oz(a){for(a=a&&a.ngOriginalError;a&&a.ngOriginalError;)a=a.ngOriginalError;return a||null}},xi=new df("",{ea:"root",ba:()=>lf(wi).handleError.bind(this)});var yi=class{};yi.wf=zi;yi.wz=a=>a;class Ai extends yi{constructor(){var a=Ug();super();this.D=a}ob(a){Pg(this.D,a);return()=>{var b=this.D;if(null!==b[21]){var c=b[21].indexOf(a);-1!==c&&b[21].splice(c,1)}}}}function zi(){return new Ai};function Bi(a){return a.ownerDocument.defaultView}function Ci(a){return a.ownerDocument};const Di=new Set;function Ei(a){if(!Di.has(a)){Di.add(a);var b,c;null==(b=performance)||null==(c=b.mark)||c.call(b,"mark_feature_usage",{detail:{OF:a}})}};/*


 Copyright (c) 2015-2018 Google, Inc., Netflix, Inc., Microsoft Corp. and contributors
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at
     http://www.apache.org/licenses/LICENSE-2.0
 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
*/
let Fi=!1,Gi=!1;
var Hi={set Xj(a){this.DD||(a?console.warn("DEPRECATED! RxJS was set to use deprecated synchronous error handling behavior by code at: \n"+Error().stack):Fi&&console.log("RxJS: Back to a better error behavior. Thank you. <3"));Fi=a},get Xj(){return Fi},set zy(a){this.DD||(a?console.warn("DEPRECATED! RxJS was set to use deprecated next context. This will result in deoptimizations when creating any new subscription. \n"+Error().stack):Gi&&console.log("RxJS: back to more optimized subscription creation. Thank you. <3"));Gi=
a},get zy(){return Gi}};function Ii(a){setTimeout(()=>{throw a;},0)};var Ji={closed:!0,next(){},error(a){if(Hi.Xj)throw a;Ii(a)},complete(){}};function Ki(a){a=a(b=>{b.name=b.constructor.name;b.stack=Error().stack});a.prototype=Object.create(Error.prototype);return a.prototype.constructor=a};var Li=Ki(a=>function(b){a(this);this.message=b?`${b.length} errors occurred during unsubscription:
${b.map((c,d)=>`${d+1}) ${c.toString()}`).join("\n  ")}`:"";this.name="UnsubscriptionError";this.errors=b});function Mi(a){return"function"===typeof a};var Ni=class{constructor(){this.qC=void 0;this.closed=!1;this.Oi=this.cc=this.lg=null}unsubscribe(){let a;if(!this.closed){this.closed=!0;const c=this.lg;let d;if(c)this.lg=null,c.remove(this);else if(d=this.cc){this.cc=null;for(var b of d)b.remove(this)}b=this.qC;if(Mi(b))try{b()}catch(f){a=f instanceof Li?f.errors:[f]}const e=this.Oi;this.Oi=null;if(e)for(const f of e)try{"function"===typeof f?f():f.unsubscribe()}catch(g){let k;a=null!=(k=a)?k:[];g instanceof Li?a=[...a,...g.errors]:a.push(g)}if(a)throw new Li(a);
}}add(a){if(a&&a!==this)if(this.closed)"function"===typeof a?a():a.unsubscribe();else{if(a instanceof Ni){if(a.closed||a.dA(this))return;a.yz(this)}let b;this.Oi=null!=(b=this.Oi)?b:[];this.Oi.push(a)}}dA(a){let b;return this.lg===a||(null==(b=this.cc)?void 0:b.includes(a))||!1}yz(a){const b=this.lg;let c;b?(this.cc=[b,a],this.lg=null):(c=this.cc)?c.push(a):this.lg=a}BA(a){const b=this.lg;let c;if(b)b===a&&(this.lg=null);else if(c=this.cc)a=c.indexOf(a),0<=a&&c.splice(a,1)}remove(a){const b=this.Oi;
if(b){const c=b.indexOf(a);0<=c&&b.splice(c,1)}a instanceof Ni&&a.BA(this)}},Oi=new Ni;Oi.closed=!0;Ni.EMPTY=Oi;function Pi(a){return a instanceof Ni||a&&"closed"in a&&"function"===typeof a.remove&&"function"===typeof a.add&&"function"===typeof a.unsubscribe};var Qi=class extends Ni{static create(a,b,c){a=new Qi(a,b,c);a.ag=!1;return a}constructor(a,b,c){super();this.Es=null;this.Rb=this.ag=this.Ds=!1;switch(arguments.length){case 0:this.destination=Ji;break;case 1:if(!a){this.destination=Ji;break}if("object"===typeof a){a instanceof Qi?(this.ag=a.ag,this.destination=a,a.add(this)):(this.ag=!0,this.destination=new Ri(this,a));break}default:this.ag=!0,this.destination=new Ri(this,a,b,c)}}next(a){this.Rb||this.Oa(a)}error(a){this.Rb||(this.Rb=!0,this.pe(a))}complete(){this.Rb||
(this.Rb=!0,this.oc())}unsubscribe(){this.closed||(this.Rb=!0,super.unsubscribe())}Oa(a){this.destination.next(a)}pe(a){this.destination.error(a);this.unsubscribe()}oc(){this.destination.complete();this.unsubscribe()}},Ri=class extends Qi{constructor(a,b,c,d){super();this.D=a;let e;if(Mi(b))e=b;else if(b&&(e=b.next,c=b.error,d=b.complete,b!==Ji)){let f;Hi.zy?(f=Object.create(b),f.unsubscribe=this.unsubscribe.bind(this)):f=b;e=e&&e.bind(f);c=c&&c.bind(f);d=d&&d.bind(f);Pi(b)&&b.add(this.unsubscribe.bind(this))}this.Oa=
e;this.pe=c;this.oc=d}next(a){if(!this.Rb&&this.Oa)try{this.Oa(a)}catch(b){this.C(b)}}error(a){if(!this.Rb)if(this.pe){try{this.pe(a)}catch(b){this.C(b);return}this.unsubscribe()}else this.C(a)}C(a){this.unsubscribe();if(Hi.Xj){const b=this.D;if(null==b?0:b.ag)b.Es=a,b.Ds=!0;else throw a;}else Ii(a)}complete(){if(!this.Rb){if(this.oc)try{this.oc()}catch(a){this.C(a);return}this.unsubscribe()}}unsubscribe(){if(!this.closed){const a=this.D;this.D=null;a.unsubscribe();super.unsubscribe()}}};var Si="function"===typeof Symbol&&Symbol.observable||"@@observable";function Ti(a){return a};function Ui(a){return 0===a.length?Ti:1===a.length?a[0]:function(b){return a.reduce((c,d)=>d(c),b)}};function Vi(a){return a&&"function"===typeof a.next&&"function"===typeof a.error&&"function"===typeof a.complete}class Wi extends Qi{constructor(a){super();this.destination=a}};function C(a,...b){return 0===b.length?a:Ui(b)(a)}
var Xi=class{constructor(a){a&&(this.se=a)}Wn(a){const b=new Xi;b.source=this;b.operator=a;return b}subscribe(a,b,c){const d=this.operator;a:{if(a){if(a instanceof Qi||Vi(a)&&Pi(a))break a;if(Vi(a)){a=new Wi(a);break a}}a=a||b||c?new Qi(a,b,c):new Qi(Ji)}d?a.add(d.call(a,this.source)):a.add(this.source||Hi.Xj&&!a.ag?this.se(a):this.D(a));if(Hi.Xj&&a.ag&&(a.ag=!1,a.Ds))throw a.Es;return a}D(a){try{return this.se(a)}catch(c){if(Hi.Xj)a.Ds=!0,a.Es=c;else{var b;a:{for(b=a;b;){const d=b.destination,e=
b.Rb;if(b.closed||e){b=!1;break a}d&&d instanceof Qi?b=d:b=null}b=!0}b?a.error(c):console.warn(c)}}}forEach(a,b){b=Yi(b);return new b((c,d)=>{let e;e=this.subscribe(f=>{try{a(f)}catch(g){d(g),e&&e.unsubscribe()}},d,c)})}se(a){const b=this.source;return b&&b.subscribe(a)}[Si](){return this}};Xi.create=a=>new Xi(a);function Yi(a){a||(a=Promise);if(!a)throw Error("no Promise impl found");return a};var Zi=class extends Ni{constructor(a,b){super();this.Tg=a;this.Bs=b;this.closed=!1}unsubscribe(){if(!this.closed){this.closed=!0;var a=this.Tg,b=a.mc;this.Tg=null;!b||0===b.length||a.Rb||a.closed||(a=b.indexOf(this.Bs),-1!==a&&b.splice(a,1))}}};var $i=Ki(a=>function(){a(this);this.message="object unsubscribed"});function aj(a){const b=new Xi;b.source=a;return b}
var cj=class extends Xi{constructor(){super();this.mc=[];this.ze=this.Rb=this.closed=!1;this.C=null}Wn(a){const b=new bj(this,this);b.operator=a;return b}next(a){if(this.closed)throw new $i;if(!this.Rb){var b=this.mc;const c=b.length;b=b.slice();for(let d=0;d<c;d++)b[d].next(a)}}error(a){if(this.closed)throw new $i;this.ze=!0;this.C=a;this.Rb=!0;var b=this.mc;const c=b.length;b=b.slice();for(let d=0;d<c;d++)b[d].error(a);this.mc.length=0}complete(){if(this.closed)throw new $i;this.Rb=!0;var a=this.mc;
const b=a.length;a=a.slice();for(let c=0;c<b;c++)a[c].complete();this.mc.length=0}unsubscribe(){this.closed=this.Rb=!0;this.mc=null}D(a){if(this.closed)throw new $i;return super.D(a)}se(a){if(this.closed)throw new $i;if(this.ze)return a.error(this.C),Ni.EMPTY;if(this.Rb)return a.complete(),Ni.EMPTY;this.mc.push(a);return new Zi(this,a)}};cj.create=(a,b)=>new bj(a,b);
var bj=class extends cj{constructor(a,b){super();this.destination=a;this.source=b}next(a){const b=this.destination;b&&b.next&&b.next(a)}error(a){this.destination&&this.destination.error(a)}complete(){this.destination&&this.destination.complete()}se(a){return this.source?this.source.subscribe(a):Ni.EMPTY}};function dj(a){if(a.ze)throw a.C;if(a.closed)throw new $i;return a.Lc}var ej=class extends cj{constructor(a){super();this.Lc=a}get value(){return dj(this)}se(a){const b=super.se(a);b&&!b.closed&&a.next(this.Lc);return b}next(a){super.next(this.Lc=a)}};var fj=new Xi(a=>a.complete());function gj(a,b){return new Xi(c=>{const d=new Ni;let e=0;d.add(b.Oc(function(){e===a.length?c.complete():(c.next(a[e++]),c.closed||d.add(this.Oc()))}));return d})};var hj=a=>b=>{for(let c=0,d=a.length;c<d&&!b.closed;c++)b.next(a[c]);b.complete()};function ij(a){return a&&"function"===typeof a.Oc};function jj(...a){let b=a[a.length-1];return ij(b)?(a.pop(),gj(a,b)):new Xi(hj(a))};function kj(a){return new Xi(b=>b.error("function"===typeof a?a():a))};var lj={now(){return(lj.rb||Date).now()},rb:void 0};var mj=class extends cj{constructor(a=Infinity,b=Infinity,c=lj){super();this.aa=c;this.Bb=[];this.J=!1;this.F=1>a?1:a;this.O=1>b?1:b;Infinity===b?(this.J=!0,this.next=this.P):this.next=this.T}P(a){if(!this.Rb){const b=this.Bb;b.push(a);b.length>this.F&&b.shift()}super.next(a)}T(a){this.Rb||(this.Bb.push({time:this.G(),value:a}),this.N());super.next(a)}se(a){const b=this.J,c=b?this.Bb:this.N(),d=c.length;let e;if(this.closed)throw new $i;this.Rb||this.ze?e=Ni.EMPTY:(this.mc.push(a),e=new Zi(this,a));
if(b)for(var f=0;f<d&&!a.closed;f++)a.next(c[f]);else for(f=0;f<d&&!a.closed;f++)a.next(c[f].value);this.ze?a.error(this.C):this.Rb&&a.complete();return e}G(){const a=this.aa;return a?a.now():lj.now()}N(){const a=this.G(),b=this.F,c=this.O,d=this.Bb,e=d.length;let f=0;for(;f<e&&!(a-d[f].time<c);)f++;e>b&&(f=Math.max(f,e-b));0<f&&d.splice(0,f);return d}};var oj=class{constructor(a,b=nj){this.D=a;this.now=b}Oc(a,b=0,c){return(new this.D(this,a)).Oc(c,b)}},nj=lj.now;var pj=Ki(a=>function(){a(this);this.message="no elements in sequence"});var qj=class extends Qi{constructor(a,b,c,d){super(a);this.C=d;b&&(this.Oa=function(e){try{null==b||b(e)}catch(f){this.error(f)}});c&&(this.oc=function(){try{c()}catch(e){this.destination.error(e)}this.unsubscribe()})}unsubscribe(){let a;this.closed||null!=(a=this.C)&&a.call(this);super.unsubscribe()}};function rj(a,b){if(a&&"function"===typeof a.Wn)return a.Wn(b);throw new TypeError("Unable to lift unknown Observable type");};function sj(){return a=>rj(a,function(b){const c=this;let d=null;b.zk++;const e=new qj(c,void 0,void 0,()=>{if(!b||0>=b.zk||0<--b.zk)d=null;else{var g=b.Ci,k=d;d=null;!g||k&&g!==k||g.unsubscribe();c.unsubscribe()}}),f=b.subscribe(e);e.closed||(d=b.connect());return f})};var uj=class extends Xi{constructor(a,b){super();this.source=a;this.hy=b;this.zk=0;this.Cm=!1}se(a){return this.En().subscribe(a)}En(){const a=this.wh;if(!a||a.Rb)this.wh=this.hy();return this.wh}connect(){let a=this.Ci;a||(this.Cm=!1,a=this.Ci=new Ni,a.add(this.source.subscribe(new tj(this.En(),this))),a.closed&&(this.Ci=null,a=Ni.EMPTY));return a}Rl(){return sj()(this)}},vj;const wj=uj.prototype;
vj={operator:{value:null},zk:{value:0,writable:!0},wh:{value:null,writable:!0},Ci:{value:null,writable:!0},se:{value:wj.se},Cm:{value:wj.Cm,writable:!0},En:{value:wj.En},connect:{value:wj.connect},Rl:{value:wj.Rl}};
class tj extends Qi{constructor(a,b){super();this.destination=a;this.D=b}pe(a){this.C();super.pe(a)}oc(){this.D.Cm=!0;this.C();super.oc()}C(){const a=this.D;if(a){this.D=null;const b=a.Ci;a.zk=0;a.wh=null;a.Ci=null;b&&b.unsubscribe()}}unsubscribe(){this.closed||(this.C(),super.unsubscribe())}};function xj(a){return function(b){if("function"!==typeof a)throw new TypeError("argument is not a function. Are you looking for `mapTo()`?");return rj(b,new yj(a))}}var yj=class{constructor(a){this.project=a}call(a,b){return b.subscribe(new zj(a,this.project))}};class zj extends Qi{constructor(a,b){super(a);this.project=b;this.count=0;this.C=this}Oa(a){let b;try{b=this.project.call(this.C,a,this.count++)}catch(c){this.destination.error(c);return}this.destination.next(b)}};var Aj=Array.isArray||(a=>a&&"number"===typeof a.length);var Bj="function"===typeof Symbol&&Symbol.iterator?Symbol.iterator:"@@iterator";function Cj(a){return null!==a&&"object"===typeof a};function Dj(a){return b=>{Ej(a,b).catch(c=>b.error(c))}}
function Ej(a,b){return sa(function*(){var c;try{var d,e=a[Symbol.asyncIterator];if(void 0!==e)var f=e.call(a);else{var g="undefined"!=typeof Symbol&&Symbol.iterator&&a[Symbol.iterator];if(g)var k=g.call(a);else if("number"==typeof a.length)k={next:ba(a)};else throw Error(String(a)+" is not an iterable or ArrayLike");f=new ta(k)}for(d=f;;){var m=yield d.next();if(m.done)break;b.next(m.value)}}catch(t){var p={error:t}}finally{try{m&&!m.done&&(c=d.return)&&(yield c.call(d))}finally{if(p)throw p.error;
}}b.complete()})};var Fj=a=>b=>{const c=a[Bj]();do{let d;try{d=c.next()}catch(e){b.error(e);return}if(d.done){b.complete();break}b.next(d.value);if(b.closed)break}while(1);"function"===typeof c.return&&b.add(()=>{c.return&&c.return()});return b};var Gj=a=>b=>{const c=a[Si]();if("function"!==typeof c.subscribe)throw new TypeError("Provided object does not correctly implement Symbol.observable");return c.subscribe(b)};var Hj=a=>b=>{a.then(c=>{b.closed||(b.next(c),b.complete())},c=>b.error(c)).then(null,Ii);return b};var Ij=a=>{if(a&&"function"===typeof a[Si])return Gj(a);if(a&&"number"===typeof a.length&&"function"!==typeof a)return hj(a);if(a&&"function"!==typeof a.subscribe&&"function"===typeof a.then)return Hj(a);if(a&&"function"===typeof a[Bj])return Fj(a);if(Symbol&&Symbol.asyncIterator&&a&&"function"===typeof a[Symbol.asyncIterator])return Dj(a);throw new TypeError(`You provided ${Cj(a)?"an invalid object":`'${a}'`} where a stream was expected.`+" You can provide an Observable, Promise, Array, or Iterable.");
};var Jj=class extends Qi{constructor(a){super();this.parent=a}Oa(a){this.parent.Eg(a)}pe(a){this.parent.zr(a);this.unsubscribe()}oc(){this.parent.Dg();this.unsubscribe()}},Kj=class extends Qi{constructor(a,b){super();this.parent=a;this.C=b}Oa(a){this.parent.Eg(null,a,this.C)}pe(a){this.parent.zr(a);this.unsubscribe()}oc(){this.parent.Dg();this.unsubscribe()}},Lj=class extends Qi{Eg(a){this.destination.next(a)}zr(a){this.destination.error(a)}Dg(){this.destination.complete()}},Mj=class extends Qi{Eg(a,
b){this.destination.next(b)}zr(a){this.destination.error(a)}Dg(){this.destination.complete()}};function Nj(a,b){if(!b.closed)return a instanceof Xi?a.subscribe(b):Ij(a)(b)};const Oj={};function Pj(...a){let b=void 0,c=void 0,d=void 0;ij(a[a.length-1])&&(c=a.pop());"function"===typeof a[a.length-1]&&(b=a.pop());if(1===a.length){const e=a[0];Aj(e)&&(a=e);Cj(e)&&Object.getPrototypeOf(e)===Object.prototype&&(d=Object.keys(e),a=d.map(f=>e[f]))}return rj(c?gj(a,c):new Xi(hj(a)),new Qj(b,d))}
var Qj=class{constructor(a,b){this.C=a;this.keys=b}call(a,b){return b.subscribe(new Rj(a,this.C,this.keys))}},Rj=class extends Mj{constructor(a,b,c){super(a);this.D=b;this.keys=c;this.active=0;this.values=[];this.F=[]}Oa(a){this.values.push(Oj);this.F.push(a)}oc(){const a=this.F,b=a.length;if(0===b)this.destination.complete();else{this.C=this.active=b;for(let c=0;c<b;c++)this.add(Nj(a[c],new Kj(this,c)))}}Dg(){0===--this.active&&this.destination.complete()}Eg(a,b,c){const d=this.values;var e=d[c];
e=this.C?e===Oj?--this.C:this.C:0;d[c]=b;0===e&&(this.D?this.G(d):this.destination.next(this.keys?this.keys.reduce((f,g,k)=>(f[g]=d[k],f),{}):d.slice()))}G(a){let b;try{b=this.D.apply(this,a)}catch(c){this.destination.error(c);return}this.destination.next(b)}};function Sj(a){return a instanceof Xi?a:new Xi(Ij(a))};function Tj(a,b){var c=Infinity;if("function"===typeof b)return d=>C(d,Tj((e,f)=>C(Sj(a(e,f)),xj((g,k)=>b(e,g,f,k))),c));"number"===typeof b&&(c=b);return d=>rj(d,new Uj(a,c))}
var Uj=class{constructor(a,b=Infinity){this.project=a;this.C=b}call(a,b){return b.subscribe(new Vj(a,this.project,this.C))}},Vj=class extends Lj{constructor(a,b,c=Infinity){super(a);this.destination=a;this.project=b;this.F=c;this.D=!1;this.C=[];this.index=this.active=0}Oa(a){if(this.active<this.F){let b;const c=this.index++;try{b=this.project(a,c)}catch(d){this.destination.error(d);return}this.active++;a=new Jj(this);this.destination.add(a);Nj(b,a)}else this.C.push(a)}oc(){this.D=!0;0===this.active&&
0===this.C.length&&this.destination.complete();this.unsubscribe()}Eg(a){this.destination.next(a)}Dg(){const a=this.C;this.active--;0<a.length?this.Oa(a.shift()):0===this.active&&this.D&&this.destination.complete()}};function Wj(a=Infinity){return Tj(Ti,a)};function hk(...a){return Wj(1)(jj(...a))};function ik(a){return new Xi(b=>{let c;try{c=a()}catch(d){b.error(d);return}return Sj(c).subscribe(b)})};function jk(...a){if(1===a.length){const b=a[0];if(Aj(b))return kk(b,null);if(Cj(b)&&Object.getPrototypeOf(b)===Object.prototype)return a=Object.keys(b),kk(a.map(c=>b[c]),a)}if("function"===typeof a[a.length-1]){const b=a.pop();a=1===a.length&&Aj(a[0])?a[0]:a;return C(kk(a,null),xj(c=>b(...c)))}return kk(a,null)}
function kk(a,b){return new Xi(c=>{const d=a.length;if(0===d)c.complete();else{var e=Array(d),f=0,g=0;for(let k=0;k<d;k++){const m=Sj(a[k]);let p=!1;c.add(m.subscribe({next:t=>{p||(p=!0,g++);e[k]=t},error:t=>c.error(t),complete:()=>{f++;f!==d&&p||(g===d&&c.next(b?b.reduce((t,w,B)=>(t[w]=e[B],t),{}):e),c.complete())}}))}}})};function lk(a,b,c){if(Mi(c)){var d=c;c=void 0}return d?C(lk(a,b,c),xj(e=>Aj(e)?d(...e):d(e))):new Xi(e=>{mk(a,b,function(f){1<arguments.length?e.next(Array.prototype.slice.call(arguments)):e.next(f)},e,c)})}
function mk(a,b,c,d,e){let f;if(a&&"function"===typeof a.addEventListener&&"function"===typeof a.removeEventListener)a.addEventListener(b,c,e),f=()=>a.removeEventListener(b,c,e);else if(a&&"function"===typeof a.on&&"function"===typeof a.off)a.on(b,c),f=()=>a.off(b,c);else if(a&&"function"===typeof a.addListener&&"function"===typeof a.removeListener)a.addListener(b,c),f=()=>a.removeListener(b,c);else if(a&&a.length)for(let g=0,k=a.length;g<k;g++)mk(a[g],b,c,d,e);else throw new TypeError("Invalid event target");
d.add(f)};var nk=class extends Ni{Oc(){return this}};var ok=class extends nk{constructor(a,b){super();this.scheduler=a;this.F=b;this.pending=!1}Oc(a,b=0){if(this.closed)return this;this.state=a;a=this.id;const c=this.scheduler;null!=a&&(this.id=this.C(c,a,b));this.pending=!0;this.delay=b;this.id=this.id||this.D(c,this.id,b);return this}D(a,b,c=0){a=a.flush.bind(a,this);return setInterval(a,c)}C(a,b,c=0){if(null!=c&&this.delay===c&&!1===this.pending)return b;clearInterval(b)}lj(a,b){if(this.closed)return Error("executing a cancelled action");this.pending=
!1;if(a=this.G(a,b))return a;!1===this.pending&&null!=this.id&&(this.id=this.C(this.scheduler,this.id,null))}G(a){let b=!1,c=void 0;try{this.F(a)}catch(d){b=!0,c=!!d&&d||Error(d)}if(b)return this.unsubscribe(),c}unsubscribe(){if(!this.closed){const a=this.id,b=this.scheduler,c=b.actions,d=c.indexOf(this);this.state=this.F=null;this.pending=!1;this.scheduler=null;-1!==d&&c.splice(d,1);null!=a&&(this.id=this.C(b,a,null));this.delay=null;super.unsubscribe()}}};var pk=class extends oj{constructor(a,b=nj){super(a,b);this.actions=[];this.active=!1;this.C=void 0}flush(a){const b=this.actions;if(this.active)b.push(a);else{var c;this.active=!0;do if(c=a.lj(a.state,a.delay))break;while(a=b.shift());this.active=!1;if(c){for(;a=b.shift();)a.unsubscribe();throw c;}}}};var qk=new pk(ok);function rk(...a){let b=Infinity,c=void 0,d=a[a.length-1];ij(d)?(c=a.pop(),1<a.length&&"number"===typeof a[a.length-1]&&(b=a.pop())):"number"===typeof d&&(b=a.pop());return!c&&1===a.length&&a[0]instanceof Xi?a[0]:Wj(b)(c?gj(a,c):new Xi(hj(a)))};function sk(){};function tk(a){return function(b){return rj(b,new uk(a))}}class uk{constructor(a){this.mf=a}call(a,b){return b.subscribe(new vk(a,this.mf))}}class vk extends Qi{constructor(a,b){super(a);this.mf=b;this.count=0}Oa(a){let b;try{b=this.mf.call(void 0,a,this.count++)}catch(c){this.destination.error(c);return}b&&this.destination.next(a)}};function wk(a=0){var b;let c=-1;null!=qk&&(ij(qk)?b=qk:c=qk);ij(b)||(b=qk);return new Xi(d=>{const e=Math.max(0,a instanceof Date&&!isNaN(a)?+a-b.now():a);return b.Oc(xk,e,{counter:0,sD:c,Bs:d})})}function xk(a){const b=a.sD,c=a.Bs,d=a.counter++;c.next(d);if(!c.closed){if(0>b)return c.complete();this.Oc(a,b)}};Ki(a=>function(b=null){a(this);this.message="Timeout has occurred";this.name="TimeoutError";this.info=b});let yk=1,zk;const Ak={};function Bk(a){return a in Ak?(delete Ak[a],!0):!1}function Ck(a){const b=yk++;Ak[b]=!0;zk||(zk=Promise.resolve());zk.then(()=>Bk(b)&&a());return b};var Dk=new class extends pk{flush(a){this.active=!0;this.C=void 0;const b=this.actions;let c,d=-1;a=a||b.shift();let e=b.length;do if(c=a.lj(a.state,a.delay))break;while(++d<e&&(a=b.shift()));this.active=!1;if(c){for(;++d<e&&(a=b.shift());)a.unsubscribe();throw c;}}}(class extends ok{constructor(a,b){super(a,b);this.scheduler=a;this.F=b}D(a,b,c=0){if(null!==c&&0<c)return super.D(a,b,c);a.actions.push(this);return a.C||(a.C=Ck(a.flush.bind(a,void 0)))}C(a,b,c=0){if(null!=c&&0<c||null==c&&0<this.delay)return super.C(a,
b,c);0===a.actions.length&&(Bk(b),a.C=void 0)}});Ki(a=>function(){a(this);this.message="argument out of range"});Ki(a=>function(b){a(this);this.message=b});Ki(a=>function(b){a(this);this.message=b});function Ek(a){return!!a&&(a instanceof Xi||"function"===typeof a.Wn&&"function"===typeof a.subscribe)};class Fk extends cj{constructor(a=!1){super();this.G=void 0;this.F=a;if(void 0!==Xe||null!=hf){let b;this.G=null!=(b=lf(yi,{optional:!0}))?b:void 0}}emit(a){const b=qe(null);try{super.next(a)}finally{qe(b)}}subscribe(a,b,c){var d=a;b=b||(()=>null);if(a&&"object"===typeof a){let e;d=null==(e=a.next)?void 0:e.bind(a);let f;b=null==(f=a.error)?void 0:f.bind(a);let g;c=null==(g=a.complete)?void 0:g.bind(a)}this.F&&(b=Gk(b),d&&(d=Gk(d)),c&&(c=Gk(c)));d=super.subscribe({next:d,error:b,complete:c});a instanceof
Ni&&a.add(d);return d}}function Gk(a){return b=>{setTimeout(a,void 0,b)}};function Hk(a){let b=!0;setTimeout(()=>{b&&(b=!1,a())});"function"===typeof ef.requestAnimationFrame&&ef.requestAnimationFrame(()=>{b&&(b=!1,a())});return()=>{b=!1}}function Ik(a){let b=!0;queueMicrotask(()=>{b&&a()});return()=>{b=!1}};function Jk(){};function Kk(){return"undefined"!==typeof Zone&&!0===Zone.current.get("isAngularZone")}function Lk(){if(Kk())throw new Zd(909,!1);}function Mk(a,b){return a.uA.run(b)}
var Qk=class{constructor({IB:a=!1,Bo:b=!1,Xl:c=!1}){this.kl=this.In=!1;this.isStable=!0;this.ko=new Fk(!1);this.Ow=new Fk(!1);this.Jl=new Fk(!1);this.onError=new Fk(!1);if("undefined"==typeof Zone)throw new Zd(908,!1);Zone.assertZonePatched();this.Ji=0;this.uA=this.Bf=Zone.current;Zone.TaskTrackingZoneSpec&&(this.Bf=this.Bf.fork(new Zone.TaskTrackingZoneSpec));a&&Zone.longStackTraceZoneSpec&&(this.Bf=this.Bf.fork(Zone.longStackTraceZoneSpec));this.Bo=!c&&b;this.Xl=c;this.Lk=!1;Nk(this)}run(a,b,c){return this.Bf.run(a,
b,c)}runTask(a,b,c,d){const e=this.Bf;a=e.scheduleEventTask("NgZoneEvent: "+d,a,Ok,Jk,Jk);try{return e.runTask(a,b,c)}finally{e.cancelTask(a)}}runGuarded(a,b,c){return this.Bf.runGuarded(a,b,c)}};const Ok={};function Rk(a){if(0==a.Ji&&!a.kl&&!a.isStable)try{a.Ji++,a.Ow.emit(null)}finally{if(a.Ji--,!a.kl)try{Mk(a,()=>a.Jl.emit(null))}finally{a.isStable=!0}}}function Sk(a){a.pw||a.Lk||(a.Lk=!0,Zone.root.run(()=>{Hk(()=>{a.Lk=!1;Tk(a);a.pw=!0;Rk(a);a.pw=!1})}),Tk(a))}
function Nk(a){a.Bf=a.Bf.fork({name:"angular",properties:{isAngularZone:!0},onInvokeTask:(b,c,d,e,f,g)=>{if(Uk(g,"__ignore_ng_zone__"))return b.invokeTask(d,e,f,g);try{return Vk(a),b.invokeTask(d,e,f,g)}finally{(a.Bo&&"eventTask"===e.type||a.Xl)&&Sk(a),a.Ji--,Rk(a)}},onInvoke:(b,c,d,e,f,g,k)=>{try{return Vk(a),b.invoke(d,e,f,g,k)}finally{!a.Xl||a.Lk||Uk(g,"__scheduler_tick__")||Sk(a),a.Ji--,Rk(a)}},onHasTask:(b,c,d,e)=>{b.hasTask(d,e);c===d&&("microTask"==e.wb?(a.eA=e.OC,Tk(a),Rk(a)):"macroTask"==
e.wb&&(a.In=e.JC))},onHandleError:(b,c,d,e)=>{b.handleError(d,e);Mk(a,()=>a.onError.emit(e));return!1}})}function Tk(a){a.kl=a.eA||(a.Bo||a.Xl)&&!0===a.Lk?!0:!1}function Vk(a){a.Ji++;a.isStable&&(a.isStable=!1,a.ko.emit(null))}function Uk(a,b){if(!Array.isArray(a)||1!==a.length)return!1;let c,d;return!0===(null==(c=a[0])?void 0:null==(d=c.data)?void 0:d[b])};const Wk={destroy(){}};function Xk(a,b){!b&&zg();let c;const d=null!=(c=null==b?void 0:b.xa)?c:lf(Wh);if("browser"!==(null!=d?d:lf(Wh)).get(li))return Wk;Ei("NgAfterRender");let e,f;const g=null!=(f=(e=d.get(Yk)).handler)?f:e.handler=new Zk;let k;const m=null!=(k=null==b?void 0:b.Xh)?k:2;b=()=>{g.unregister(t);p()};const p=d.get(yi).ob(b),t=yg(d,()=>new $k(m,a));g.register(t);return{destroy:b}}
function al(a,b){!b&&zg();let c;const d=null!=(c=null==b?void 0:b.xa)?c:lf(Wh);if("browser"!==(null!=d?d:lf(Wh)).get(li))return Wk;Ei("NgAfterNextRender");let e,f;const g=null!=(f=(e=d.get(Yk)).handler)?f:e.handler=new Zk;let k;const m=null!=(k=null==b?void 0:b.Xh)?k:2,p=()=>{g.unregister(w);t()},t=d.get(yi).ob(p),w=yg(d,()=>new $k(m,()=>{p();a()}));g.register(w);return{destroy:p}}
class $k{constructor(a,b){this.Xh=a;this.C=b;this.zone=lf(Qk);this.tn=lf(wi,{optional:!0});let c;null==(c=lf(ai,{optional:!0}))||c.notify(6)}invoke(){try{Mk(this.zone,this.C)}catch(a){let b;null==(b=this.tn)||b.handleError(a)}}}
class Zk{constructor(){this.F=!1;this.C={[0]:new Set,[1]:new Set,[2]:new Set,[3]:new Set};this.D=new Set}register(a){(this.F?this.D:this.C[a.Xh]).add(a)}unregister(a){this.C[a.Xh].delete(a);this.D.delete(a)}lj(){this.F=!0;for(const a of Object.values(this.C))for(const b of a)b.invoke();this.F=!1;for(const a of this.D)this.C[a.Xh].add(a);this.D.clear()}destroy(){for(const a of Object.values(this.C))a.clear();this.D.clear()}}function bl(a){const b=[...a.C];a.C.length=0;for(const c of b)c()}
var Yk=class{constructor(){this.handler=null;this.C=[]}lj(){bl(this);let a;null==(a=this.handler)||a.lj()}va(){let a;null==(a=this.handler)||a.destroy();this.handler=null;this.C.length=0}};Yk.pa=Md({ma:Yk,ea:"root",ba:()=>new Yk});function cl(a,b){const c=Tg?64:1088;let d;for(null==(d=a[10].Xi)||d.notify(b);a;){a[2]|=c;b=Og(a);if(0!==(a[2]&512)&&!b)break;a=b}};new df("");var dl=new df("",{ea:"root",ba:()=>!1});new df("");new df("");function D(a=1){el(Rg.ub,Ug(),Rg.selectedIndex+a)}function el(a,b,c){3===(b[2]&3)?(a=a.Ig,null!==a&&jh(b,a,3,c)):(a=a.lf,null!==a&&ih(b,a,0,c));Rg.selectedIndex=c};function z(a,b=0){const c=Ug();return null===c?r(a,b):Ah(Vg(),c,bf(a),b)};function fl(a,b,c,d,e,f){const g=qe(null);try{let k=null;0!==(e&1)&&(k=b[d][pe]);null!==k&&void 0!==k.py&&(f=k.py(f));0!==(e&2)&&(f=a.De[d].call(b,f));null!==a.Ao?a.Ao(b,k,f,c,d):(a=k,null!==a?a.Iu(a,f):b[d]=f)}finally{qe(g)}};const gl=/^>|^->|\x3c!--|--\x3e|--!>|<!-$/g,hl=/(<|>)/g;function il(){return"".replace(gl,a=>a.replace(hl,"\u200b$1\u200b"))};function jl(a,b,c,d,e){if(null!=d){var f=!1;if(Bg(d))var g=d;else Ag(d)&&(f=!0,d=d[0]);d=Hg(d);if(0===a&&null!==c)null==e?b.appendChild(c,d):b.insertBefore(c,d,e||null,!0);else if(1===a&&null!==c)b.insertBefore(c,d,e||null,!0);else if(2===a){const k=b.parentNode(d);k&&b.removeChild(k,d,f)}else 3===a&&b.Xk(d);if(null!=g)for(d=g[7],d!==Hg(g)&&jl(a,b,c,d,e),e=10;e<g.length;e++)f=g[e],kl(b,a,f[1].firstChild,f,c,d,!1)}}
function ll(a,b){let c;null==(c=b[10].Xi)||c.notify(8);kl(b[11],2,a.firstChild,b,null,null,!1)}function ml(a,b){const c=a[9],d=b[3];Ag(d)?a[2]|=2:b[15]!==d[3][15]&&(a[2]|=2);null===c?a[9]=[b]:c.push(b)}function nl(a,b){a=a[9];a.splice(a.indexOf(b),1)}function ol(a,b){if(!(10>=a.length)){var c=10+b,d=a[c];if(d){const e=d[16];null!==e&&e!==a&&nl(e,d);0<b&&(a[c-1][4]=d[4]);a=Uf(a,10+b);ll(d[1],d);d[0]=null;d[5]=null;b=a[18];null!==b&&b.fj(a[1]);d[3]=null;d[4]=null;d[2]&=-129}return d}}
function pl(a,b){if(!(b[2]&256)){var c=b[11];c.Xk&&kl(c,3,a.firstChild,b,null,null,!1);if(a=b[12])for(;a;){c=null;if(Ag(a))c=a[12];else{const d=a[10];d&&(c=d)}if(!c){for(;a&&!a[4]&&a!==b;)Ag(a)&&ql(a[1],a),a=a[3];null===a&&(a=b);Ag(a)&&ql(a[1],a);c=a&&a[4]}a=c}else ql(b[1],b)}}
function ql(a,b){if(!(b[2]&256)){var c=qe(null);try{b[2]&=-129;b[2]|=256;b[23]&&Ee(b[23]);var d;if(null!=a&&null!=(d=a.Wk))for(let m=0;m<d.length;m+=2){const p=b[d[m]];if(!(p instanceof mh)){const t=d[m+1];if(Array.isArray(t))for(let w=0;w<t.length;w+=2)t[w+1].call(p[t[w]]);else t.call(p)}}var e=a.be;const f=b[7];if(null!==e)for(d=0;d<e.length-1;d+=2)if("string"===typeof e[d]){const m=e[d+3];if(0<=m)f[m]();else f[-m].unsubscribe();d+=2}else e[d].call(f[e[d+1]]);null!==f&&(b[7]=null);const g=b[21];
if(null!==g)for(b[21]=null,e=0;e<g.length;e++)(0,g[e])();1===b[1].type&&b[11].destroy();const k=b[16];if(null!==k&&Bg(b[3])){k!==b[3]&&nl(k,b);const m=b[18];null!==m&&m.fj(a)}fi.delete(b[19])}finally{qe(c)}}}function rl(a,b,c){for(;null!==b&&b.type&40;)b=b.parent;if(null===b)return c[0];const d=b.Jf;return-1<d&&({pb:a}=a.data[b.nd+d],2===a||0===a)?null:Ig(b,c)}
function sl(a,b,c,d){a=rl(a,d.parent,b);const e=b[11];d=d.parent||b[5];b=d.type&40?Ig(d,b):null;if(null!=a)if(Array.isArray(c))for(let k=0;k<c.length;k++){d=e;var f=a,g=c[k];null!==b?d.insertBefore(f,g,b,!1):d.appendChild(f,g)}else null!==b?e.insertBefore(a,c,b,!1):e.appendChild(a,c)}
function tl(a,b){if(null!==b){var c=b.type;if(c&3)return Ig(b,a);if(c&4)return ul(-1,a[b.index]);if(c&8){c=b.Hf;if(null!==c)return tl(a,c);a=a[b.index];return Bg(a)?ul(-1,a):Hg(a)}if(c&32)return(void 0)(b,a)()||Hg(a[b.index]);c=null!==b?a[15][5].Je[b.Je]:null;if(null!==c){if(Array.isArray(c))return c[0];a=Og(a[15]);return tl(a,c)}return tl(a,b.next)}return null}function ul(a,b){a+=11;if(a<b.length){a=b[a];const c=a[1].firstChild;if(null!==c)return tl(a,c)}return b[7]}
function kl(a,b,c,d,e,f,g){for(;null!=c;){const m=d[c.index];var k=c.type;g&&0===b&&(m&&hi(Hg(m),d),c.flags|=2);if(32!==(c.flags&32))if(k&8)kl(a,b,c.Hf,d,e,f,!1),jl(b,a,e,m,f);else if(k&32){k=(void 0)(c,d);let p;for(;p=k();)jl(b,a,e,p,f);jl(b,a,e,m,f)}else k&16?vl(a,b,d,c,e,f):jl(b,a,e,m,f);c=g?c.Ur:c.next}}function vl(a,b,c,d,e,f){var g=c[15];c=g[5].Je[d.Je];if(Array.isArray(c))for(d=0;d<c.length;d++)jl(b,a,e,c[d],f);else g=g[3],128===(d.flags&128)&&(c.flags|=128),kl(a,b,c,g,e,f,!0)}
function wl(a,b,c){const d=c.Ge,e=c.Yi;c=c.styles;null!==d&&uf(a,b,d);null!==e&&(""===e?a.removeAttribute(b,"class"):a.setAttribute(b,"class",e));null!==c&&a.setAttribute(b,"style",c)};var xl={};class yl{constructor(a){this.C=a}toString(){return`SafeValue must use [property]=binding: ${this.C}`+" (see https://g.co/ng/security#xss)"}}class zl extends yl{Ys(){return"URL"}}class Al extends yl{Ys(){return"ResourceURL"}}function Bl(a){return a instanceof yl?a.C:a}function Cl(a,b){a=a instanceof yl&&a.Ys()||null;if(null!=a&&a!==b){if("ResourceURL"===a&&"URL"===b)return!0;throw Error(`Required a safe ${b}, got a ${a} (see ${"https://g.co/ng/security#xss"})`);}return a===b};let Dl;function El(){if(void 0===Dl&&(Dl=null,ef.trustedTypes))try{Dl=ef.trustedTypes.createPolicy("angular",{createHTML:a=>a,createScript:a=>a,createScriptURL:a=>a})}catch(a){}return Dl}function Fl(a){let b;return(null==(b=El())?void 0:b.createHTML(a))||a};class Gl{constructor(a){this.C=a}Dn(a){a="<body><remove></remove>"+a;try{const b=(new window.DOMParser).parseFromString(Fl(a),"text/html").body;if(null===b)return this.C.Dn(a);b.removeChild(b.firstChild);return b}catch(b){return null}}}class Hl{constructor(a){this.C=a;this.D=this.C.implementation.createHTMLDocument("sanitization-inert")}Dn(a){const b=this.D.createElement("template");b.innerHTML=Fl(a);return b}};const Il=/^(?!javascript:)(?:[a-z0-9+.-]+:|[^&:\/?#]*(?:[\/?#]|$))/i;function Jl(a){a=String(a);return a.match(Il)?a:"unsafe:"+a};function Kl(a){const b={};for(const c of a.split(","))b[c]=!0;return b}function Ll(...a){const b={};for(const c of a)for(const d in c)c.hasOwnProperty(d)&&(b[d]=!0);return b}
const Ml=Kl("area,br,col,hr,img,wbr"),Nl=Kl("colgroup,dd,dt,li,p,tbody,td,tfoot,th,thead,tr"),Ol=Kl("rp,rt"),Pl=Ll(Ol,Nl),Ql=Ll(Nl,Kl("address,article,aside,blockquote,caption,center,del,details,dialog,dir,div,dl,figure,figcaption,footer,h1,h2,h3,h4,h5,h6,header,hgroup,hr,ins,main,map,menu,nav,ol,pre,section,summary,table,ul")),Rl=Ll(Ol,Kl("a,abbr,acronym,audio,b,bdi,bdo,big,br,cite,code,del,dfn,em,font,i,img,ins,kbd,label,map,mark,picture,q,ruby,rp,rt,s,samp,small,source,span,strike,strong,sub,sup,time,track,tt,u,var,video"));
var Sl=Ll(Ml,Ql,Rl,Pl),Tl=Kl("background,cite,href,itemtype,longdesc,poster,src,xlink:href");
const Ul=Kl("abbr,accesskey,align,alt,autoplay,axis,bgcolor,border,cellpadding,cellspacing,class,clear,color,cols,colspan,compact,controls,coords,datetime,default,dir,download,face,headers,height,hidden,hreflang,hspace,ismap,itemscope,itemprop,kind,label,lang,language,loop,media,muted,nohref,nowrap,open,preload,rel,rev,role,rows,rowspan,rules,scope,scrolling,shape,size,sizes,span,srclang,srcset,start,summary,tabindex,target,title,translate,type,usemap,valign,value,vspace,width"),Vl=Kl("aria-activedescendant,aria-atomic,aria-autocomplete,aria-busy,aria-checked,aria-colcount,aria-colindex,aria-colspan,aria-controls,aria-current,aria-describedby,aria-details,aria-disabled,aria-dropeffect,aria-errormessage,aria-expanded,aria-flowto,aria-grabbed,aria-haspopup,aria-hidden,aria-invalid,aria-keyshortcuts,aria-label,aria-labelledby,aria-level,aria-live,aria-modal,aria-multiline,aria-multiselectable,aria-orientation,aria-owns,aria-placeholder,aria-posinset,aria-pressed,aria-readonly,aria-relevant,aria-required,aria-roledescription,aria-rowcount,aria-rowindex,aria-rowspan,aria-selected,aria-setsize,aria-sort,aria-valuemax,aria-valuemin,aria-valuenow,aria-valuetext");
var Wl=Ll(Tl,Ul,Vl);const Xl=Kl("script,style,template");class Yl{constructor(){this.C=[]}chars(a){this.C.push(Zl(a))}}function $l(a){a=a.nodeName;return"string"===typeof a?a:"FORM"}function am(a){return Error(`Failed to sanitize html because the element is clobbered: ${a.outerHTML}`)}const bm=/[\uD800-\uDBFF][\uDC00-\uDFFF]/g,cm=/([^#-~ |!])/g;
function Zl(a){return a.replace(/&/g,"&amp;").replace(bm,function(b){return"&#"+(1024*(b.charCodeAt(0)-55296)+(b.charCodeAt(1)-56320)+65536)+";"}).replace(cm,function(b){return"&#"+b.charCodeAt(0)+";"}).replace(/</g,"&lt;").replace(/>/g,"&gt;")}let sm;function tm(a){return"content"in a&&a.nodeType===Node.ELEMENT_NODE&&"TEMPLATE"===a.nodeName?a.content:null};var um={NONE:0,cF:1,kF:2,jF:3,URL:4,hF:5,0:"NONE",1:"HTML",2:"STYLE",3:"SCRIPT",4:"URL",5:"RESOURCE_URL"};function vm(a,b,c,d,e,f,g,k,m,p,t){const w=b.qg.slice();w[0]=e;w[2]=d|204;if(null!==p||a&&a[2]&2048)w[2]|=2048;w[17]=0;w[3]=w[14]=a;w[8]=c;w[10]=g||a&&a[10];w[11]=k||a&&a[11];w[9]=m||a&&a[9]||null;w[5]=f;w[19]=gi++;w[6]=t;w[20]=p;w[15]=2==b.type?a[15]:w;return w}
function wm(a,b,c,d,e){var f=a.data[b];if(null===f){f=Rg.Eh;const m=Rg.Rf;var g=a.data,k=m?f:f&&f.parent;let p=k?k.zd:-1,t=0;null!==Sg&&(t|=128);b=g[b]={type:c,index:b,VF:null,zd:p,nd:-1,we:-1,nn:-1,Jf:-1,mG:null,flags:t,Jg:0,value:d,attrs:e,Ge:null,kr:null,iw:void 0,inputs:null,outputs:null,ub:null,next:null,AD:null,Ur:null,Hf:null,parent:k,Je:null,styles:null,zs:null,yo:void 0,Yi:null,dn:null,xo:void 0,Ok:0,cm:0};null===a.firstChild&&(a.firstChild=b);null!==f&&(m?null==f.Hf&&null!==b.parent&&(f.Hf=
b):null===f.next&&(f.next=b,b.AD=f));f=b;Rg.fw&&(f.flags|=32)}else f.type&64&&(f.type=c,f.value=d,f.attrs=e,a=Rg,b=a.Eh,a=a.Rf?b:b.parent,f.zd=null===a?-1:a.zd);Wg(f,!0);return f}function xm(a,b,c,d){if(0===c)return-1;const e=b.length;for(let f=0;f<c;f++)b.push(d),a.qg.push(d),a.data.push(null);return e}function ym(a,b,c,d,e){const f=Rg.selectedIndex;try{Rg.selectedIndex=-1,d&2&&25<b.length&&el(a,b,25),c(d,e)}finally{Rg.selectedIndex=f}}
function zm(a,b,c){if(0!==(b.flags&4)){const d=qe(null);try{const e=b.nd,f=b.we;for(b=e;b<f;b++){const g=a.data[b];g.ec&&g.ec(1,c[b],b)}}finally{qe(d)}}}
function Am(a,b,c){var d=Ig(c,b);const e=c.nd,f=c.we;if(-1<c.Jf){var g=a.data[e+c.Jf],k=Ig(c,b),m=Bm(g),p=b[10].ds,t=16;g.ii?t=4096:g.Br&&(t=64);g=Cm(b,vm(b,m,null,t,k,c,null,p.Uk(k,g),null,null,null));b[c.index]=g}a.Bc||th(c,b);hi(d,b);d=c.iw;for(g=e;g<f;g++){k=a.data[g];m=Eh(b,a,g,c);hi(m,b);if(null!==d&&(p=d[g-e],null!==p))for(t=0;t<p.length;){const w=p[t++],B=p[t++],G=p[t++],J=p[t++];fl(k,m,w,B,G,J)}k.ta&&(Jg(c.index,b)[8]=Eh(b,a,g,c))}64===(c.flags&64)&&Dm(a,b,c)}
function Em(a,b,c=Ig){const d=b.kr;if(null!==d){let f=b.index+1;for(let g=0;g<d.length;g+=2){var e=d[g+1];e=-1===e?c(b,a):a[e];a[f++]=e}}}function Bm(a){const b=a.ub;return null===b||b.gw?a.ub=Fm(1,null,a.ta,a.za,a.Aa,a.Dv,a.ax,a.fb,a.Pj,a.Ca,a.id):b}
function Fm(a,b,c,d,e,f,g,k,m,p,t){d=25+d;e=d+e;const w=[];for(let B=0;B<e;B++)w.push(B<d?null:xl);p="function"===typeof p?p():p;return w[1]={type:a,qg:w,ta:c,queries:null,fb:k,zv:b,data:w.slice().fill(null,d),aq:d,Pv:e,Sq:null,Bc:!0,xn:!0,by:!1,ay:!1,lf:null,Ig:null,Sk:null,kq:null,lm:null,Ns:null,Wk:null,be:null,ec:null,Kf:null,Ev:"function"===typeof f?f():f,vD:"function"===typeof g?g():g,firstChild:null,Pj:m,Ca:p,gw:!1,Zx:t}}
function Gm(a,b,c,d,e){for(let g in b){if(!b.hasOwnProperty(g))continue;var f=b[g];if(void 0===f)continue;null!=d||(d={});let k,m=0;Array.isArray(f)?(k=f[0],m=f[1]):k=f;f=g;if(null!==e){if(!e.hasOwnProperty(g))continue;f=e[g]}0===a?Hm(d,c,f,k,m):Hm(d,c,f,k)}return d}function Hm(a,b,c,d,e){let f;a.hasOwnProperty(c)?(f=a[c]).push(b,d):f=a[c]=[b,d];void 0!==e&&f.push(e)}
function Im(a,b,c,d,e,f,g,k){const m=Ig(b,c);let p=b.inputs,t;!k&&null!=p&&(t=p[d])?(Jm(a,c,t,d,e),-1<b.Jf&&(a=Jg(b.index,c),a[2]&16||(a[2]|=64))):b.type&3&&(d="class"===d?"className":"for"===d?"htmlFor":"formaction"===d?"formAction":"innerHtml"===d?"innerHTML":"readonly"===d?"readOnly":"tabindex"===d?"tabIndex":d,e=null!=g?g(e,b.value||"",d):e,f.setProperty(m,d,e))}
function Km(a,b,c,d){const e=null===d?null:{"":-1};var f=a.Ev;let g=null,k=null;if(f)for(let t=0;t<f.length;t++){const w=f[t];if(zf(c,w.ha,!1))if(g||(g=[]),w.ta)if(null!==w.nj){const B=[];k=k||new Map;w.nj(w,B,k);g.unshift(...B,w);Lm(a,c,B.length)}else g.unshift(w),Lm(a,c,0);else{k=k||new Map;let B,G;null==(G=(B=w).nj)||G.call(B,w,g,k);g.push(w)}}f=null===g?null:[g,k];var m;let p;null===f?m=p=null:[m,p]=f;null!==m&&Mm(a,b,c,m,e,p);if(e&&d)for(a=c.kr=[],b=0;b<d.length;b+=2){m=e[d[b+1]];if(null==m)throw new Zd(-301,
!1);a.push(d[b],m)}c.Ge=vf(c.Ge,c.attrs)}
function Mm(a,b,c,d,e,f){for(var g=0;g<d.length;g++){var k=th(c,b);sh(k,a,d[g].type)}k=a.data.length;g=d.length;c.flags|=1;c.nd=k;c.we=k+g;c.Jg=k;for(k=0;k<d.length;k++)g=d[k],g.Vr&&g.Vr(g);g=k=!1;var m=xm(a,b,d.length,null);for(var p=0;p<d.length;p++){var t=d[p];c.Ge=vf(c.Ge,t.Ka);var w=a,B=c,G=b,J=m;w.data[J]=t;var N=t.ba||(t.ba=Sf(t.type));N=new mh(N,!!t.ta);w.qg[J]=N;G[J]=N;N=w;var aa=B;w=xm(w,G,t.Za,xl);if(G=t.Ia){B=N.Sq;null===B&&(B=N.Sq=[]);N=~aa.index;a:{for(aa=B.length;0<aa;){const Q=B[--aa];
if("number"===typeof Q&&0>Q){aa=Q;break a}}aa=0}aa!=N&&B.push(N);B.push(J,w,G)}J=m;N=t;if(w=e){if(N.ib)for(G=0;G<N.ib.length;G++)w[N.ib[G]]=J;N.ta&&(w[""]=J)}null!==t.ec&&(c.flags|=4);if(null!==t.Ia||null!==t.Ka||0!==t.Za)c.flags|=64;t=t.type.prototype;if(!k&&(t.tc||t.Fb||t.zj)){var ca=void 0,ia=void 0;(null!=(ia=(ca=a).lf)?ia:ca.lf=[]).push(c.index);k=!0}if(!g&&(t.tc||t.zj)){var O=void 0,T=void 0;(null!=(T=(O=a).Ig)?T:O.Ig=[]).push(c.index);g=!0}m++}O=c.nd;b=c.we;a=a.data;d=c.attrs;e=[];for(ia=ca=
null;O<b;O++){T=a[O];g=(k=f?f.get(T):null)?k.outputs:null;ca=Gm(0,T.inputs,O,ca,k?k.inputs:null);ia=Gm(1,T.outputs,O,ia,g);if(null===ca||null===d||xf(c))T=null;else{T=ca;k=O;g=null;for(m=0;m<d.length;)if(p=d[m],0===p)m+=4;else{if(5!==p){if("number"===typeof p)break;if(T.hasOwnProperty(p))for(null===g&&(g=[]),t=T[p],J=0;J<t.length;J+=3)if(t[J]===k){g.push(p,t[J+1],t[J+2],d[m+1]);break}}m+=2}T=g}e.push(T)}null!==ca&&(ca.hasOwnProperty("class")&&(c.flags|=8),ca.hasOwnProperty("style")&&(c.flags|=16));
c.iw=e;c.inputs=ca;c.outputs=ia}function Dm(a,b,c){var d=c.nd;const e=c.we,f=c.index;c=Rg.dj;try{for(Rg.selectedIndex=f;d<e;d++){const g=a.data[d],k=b[d];Rg.dj=d;null===g.Ia&&0===g.Za&&null===g.Ka||null===g.Ia||g.Ia(1,k)}}finally{Rg.selectedIndex=-1,Rg.dj=c}}function Lm(a,b,c){b.Jf=c;let d;(null!=(d=a.Kf)?d:a.Kf=[]).push(b.index)}
function Nm(a,b){const c=a.ec;if(null!==c){const d=qe(null);try{for(let e=0;e<c.length;e+=2){const f=c[e],g=c[e+1];if(-1!==g){const k=a.data[g];Rg.ce=f;k.ec(2,b[g],g)}}}finally{qe(d)}}}function Cm(a,b){a[12]?a[13][4]=b:a[12]=b;return a[13]=b}function Om(a,b,c){Rg.ce=0;const d=qe(null);try{b(a,c)}finally{qe(d)}}function Pm(a,b,c){if(null===a||a.ta)a:{for(a=c[b.index];Array.isArray(a);){if("object"===typeof a[1]){c=a;break a}a=a[0]}c=null}return c[11]}
function Qm(a,b){(a=(a=a[9])?a.get(wi,null):null)&&a.handleError(b)}function Jm(a,b,c,d,e){for(let f=0;f<c.length;){const g=c[f++],k=c[f++],m=c[f++];fl(a.data[g],b[g],d,k,m,e)}}function Rm(a,b){a[11].setValue(Hg(a[Rg.selectedIndex]),b)};function Sm(a,b,c){bh(b);try{var d=a.fb;null!==d&&Om(1,d,c);var e=a.ta;null!==e&&ym(a,b,e,1,c);a.Bc&&(a.Bc=!1);let f;null!=(f=b[18])&&Tm(f,a);a.ay&&Nm(a,b);a.by&&Om(2,a.fb,c);const g=a.Kf;if(null!==g)for(e=0;e<g.length;e++){const k=Jg(g[e],b),m=k[1];c=m;d=k;for(let p=d.length;p<c.qg.length;p++)d.push(c.qg[p]);null!==k[0]&&null===k[6]&&(k[6]=null);Sm(m,k,k[8])}}catch(f){throw a.Bc&&(a.gw=!0,a.Bc=!1),f;}finally{b[2]&=-5,dh()}};function Um(a,b,c){let d=c?a.styles:null,e=c?a.Yi:null,f=0;if(null!==b)for(let k=0;k<b.length;k++){var g=b[k];if("number"===typeof g)f=g;else if(1==f)e=ie(e,g);else if(2==f){const m=b[++k];d=ie(d,g+": "+m+";")}}c?a.styles=d:a.zs=d;c?a.Yi=e:a.dn=e};function Vm(a,b,c,d,e=!1){for(;null!==c;){var f=b[c.index];null!==f&&d.push(Hg(f));if(Bg(f)){var g=d;for(let k=10;k<f.length;k++){const m=f[k],p=m[1].firstChild;null!==p&&Vm(m[1],m,p,g)}f[7]!==f[0]&&g.push(f[7])}f=c.type;if(f&8)Vm(a,b,c.Hf,d);else if(f&32)for(f=(void 0)(c,b);g=f();)d.push(g);else f&16&&(f=null!==c?b[15][5].Je[c.Je]:null,Array.isArray(f)?d.push(...f):(g=Og(b[15]),Vm(g[1],g,f,d,!0)));c=e?c.Ur:c.next}return d};let Wm=[];const Xm=Object.assign({},re,{iq:!0,jq:a=>{Ng(a.Sf)},hv(){this.Sf[23]=this}});function Ym(a,b=!0,c=0){const d=a[10],e=d.ds;var f;null==(f=e.Vm)||f.call(e);try{f=Tg;try{for(Tg=!0,Zm(a,c),c=0;Lg(a);){if(100===c)throw new Zd(103,!1);c++;Zm(a,1)}}finally{Tg=f}}catch(g){throw b&&Qm(a,g),g;}finally{let g;null==(g=e.end)||g.call(e);let k;null==(k=d.jw)||k.flush()}}function $m(a,b){for(let c=ii(a[12]);null!==c;c=ii(c[4]))for(let d=10;d<c.length;d++)a=c[d],128===(a[2]&128)&&Zm(a,b)}
function Zm(a,b){var c=a[1],d=a[2],e=a[23],f=!!(0===b&&d&16);f||(f=!!(d&64&&0===b));f||(f=!!(d&1024));f||(f=!!((null==e?0:e.rc)&&ye(e)));f||(f=!1);e&&(e.rc=!1);a[2]&=-9217;if(f){if(e=c.ta,b=a[8],f=a[2],256!==(f&256)){var g;null!=(g=a[10].jw)&&g.flush();bh(a);d=g=null;if(2!==c.type){if(null!=(g=a[23]))var k=g;else g=null!=(k=Wm.pop())?k:Object.create(Xm),g.Sf=a,k=g;d=k;g=Ce(d)}try{a[17]=0;Rg.ed=c.aq;null!==e&&ym(c,a,e,2,b);if(k=3===(f&3)){var m=c.Ig;null!==m&&jh(a,m,3,null)}else{const N=c.lf;null!==
N&&ih(a,N,0,null);kh(a,0)}for(var p=ii(a[12]);null!==p;p=ii(p[4]))if(p[2]&2){var t=p[9];for(m=0;m<t.length;m++){var w=t[m];w[2]&1024||(w[2]|=1024,128===(w[2]&128)&&Ng(w))}}$m(a,0);null!==c.ec&&Nm(c,a);if(k){const N=c.kq;null!==N&&jh(a,N,3)}else{const N=c.Sk;null!==N&&ih(a,N,1);kh(a,1)}const B=c.Sq;if(null!==B)try{for(w=0;w<B.length;w++){const N=B[w];if(0>N)Rg.selectedIndex=~N;else{p=N;const aa=B[++w],ca=B[++w];t=Rg;t.ed=t.Wm=aa;Rg.dj=p;ca(2,a[p])}}}finally{Rg.selectedIndex=-1}const G=c.Kf;null!==
G&&an(a,G,0);const J=c.fb;null!==J&&Om(2,J,b);if(k){const N=c.Ns;null!==N&&jh(a,N,3)}else{const N=c.lm;null!==N&&ih(a,N,2);kh(a,2)}!0===c.xn&&(c.xn=!1);if(a[22]){for(const N of a[22])N();a[22]=null}a[2]&=-73}catch(B){throw Ng(a),B;}finally{null!==d&&(De(d,g),a=d,a.Sf[23]!==a&&(a.Sf=null,Wm.push(a))),dh()}}}else d&8192&&($m(a,1),c=c.Kf,null!==c&&an(a,c,1))}function an(a,b,c){for(let e=0;e<b.length;e++){var d=c;const f=Jg(b[e],a);128===(f[2]&128)&&Zm(f,d)}};function bn(a){cl(a.N||a.D,4)}function cn(a){a.D[2]|=1024;Ym(a.D,a.J)}
var dn=class{get rootNodes(){const a=this.D,b=a[1];return Vm(b,a,b.firstChild,[])}constructor(a,b,c=!0){this.D=a;this.N=b;this.J=c;this.F=null;this.G=!1}get context(){return this.D[8]}set context(a){this.D[8]=a}get C(){return 256===(this.D[2]&256)}destroy(){if(this.F)this.F.fj(this);else if(this.G){const a=this.D[3];if(Bg(a)){const b=a[8],c=b?b.indexOf(this):-1;-1<c&&(ol(a,c),Uf(b,c))}this.G=!1}pl(this.D[1],this.D)}ob(a){Pg(this.D,a)}detach(){this.D[2]&=-129}};var en=class{};en.pa=Md({ma:en,ea:"root",ba:()=>null});var fn={};function gn(a,b){return new hn(b[$d]||null,a.Aj)}var jn=class extends Yh{constructor(a){super();this.Aj=a}};function kn(a){const b=[];for(const c in a){if(!a.hasOwnProperty(c))continue;const d=a[c];void 0!==d&&b.push({mx:Array.isArray(d)?d[0]:d,yG:c})}return b}
var ln=class{constructor(a,b){this.xa=a;this.Cj=b}get(a,b,c){c=mf(c);const d=this.xa.get(a,fn,c);return d!==fn||b===fn?d:this.Cj.get(a,b,c)}},hn=class extends qi{get inputs(){var a=this.C;const b=a.De;a=kn(a.inputs);if(null!==b)for(const c of a)b.hasOwnProperty(c.mx)&&(c.transform=b[c.mx]);return a}get outputs(){return kn(this.C.outputs)}constructor(a,b){super();this.C=a;this.Aj=b;this.fq=a.type;this.Wl=a.ha.map(Cf).join(",");this.Zb=a.Zb?a.Zb:[];this.tC=!!b}create(a,b,c,d){const e=qe(null);try{d=
d||this.Aj;let la,F=d instanceof kg?d:null==(la=d)?void 0:la.xa;F&&null!==this.C.Iq&&(F=this.C.Iq(F)||F);var f=F?new ln(a,F):a,g=f.get(ui,null);if(null===g)throw new Zd(407,!1);const M=f.get(en,null),W=f.get(Yk,null),Aa=f.get(ai,null);a={ds:g,uG:M,jw:null,xF:W,Xi:Aa};var k=g.Uk(null,this.C);const pb=this.C.ha[0][0]||"div";if(c){g=k;var m=this.C.pb,p=f.get(dl,!1)||3===m;var t=g.ns(c,p)}else{const qa=pb.toLowerCase();t=k.createElement(pb,"svg"===qa?"svg":"math"===qa?"math":null)}m=512;this.C.ii?m|=
4096:this.C.Br||(m|=16);p=null;null!==t&&(p=null);const qb=Fm(0,null,null,1,0,null,null,null,null,null,null),Ia=vm(null,qb,null,m,null,null,a,k,f,null,p);bh(Ia);let Vb,Ma;try{var w=this.C;f=null;if(w.nj){var B=[];f=new Map;w.nj(w,B,f);B.push(w)}else B=[w];var G=Ia[1];Ia[25]=t;var J=wm(G,25,2,"#host",null);p=t;G=w;m=B;var N=Ia[1];for(var aa of m)J.Ge=vf(J.Ge,aa.Ka);null!==J.Ge&&(Um(J,J.Ge,!0),null!==p&&wl(k,p,J));aa=null;null!==p&&(aa=null);var ca=a.ds.Uk(p,G);p=16;G.ii?p=4096:G.Br&&(p=64);const qa=
vm(Ia,Bm(G),null,p,Ia[J.index],J,a,ca,null,null,aa);N.Bc&&Lm(N,J,m.length-1);Cm(Ia,qa);var ia=Ia[J.index]=qa;Ma=qb.data[25];if(t)if(c)uf(k,t,["ng-version","0.0.0-PLACEHOLDER"]);else{var O=w.ha[0];c=[];J=[];N=1;for(ca=2;N<O.length;){let ub=O[N];if("string"===typeof ub)2===ca?""!==ub&&c.push(ub,O[++N]):8===ca&&J.push(ub);else{if(0!==(ca&1))break;ca=ub}N++}const {attrs:cb,Yi:Pa}={attrs:c,Yi:J};cb&&uf(k,t,cb);if(Pa&&0<Pa.length){O=k;k=t;var T=Pa.join(" ");""===T?O.removeAttribute(k,"class"):O.setAttribute(k,
"class",T)}}if(void 0!==b){var Q=this.Zb;const cb=Ma.Je=[];for(T=0;T<Q.length;T++){const Pa=b[T];cb.push(null!=Pa?Array.from(Pa):null)}}b=w;w=f;Q=[mn];const U=Vg(),ea=Ia[1],na=Ig(U,Ia);Mm(ea,Ia,U,B,null,w);for(w=0;w<B.length;w++){const cb=Eh(Ia,ea,U.nd+w,U);hi(cb,Ia)}Dm(ea,Ia,U);na&&hi(na,Ia);const Ja=Eh(Ia,ea,U.nd+U.Jf,U);ia[8]=Ia[8]=Ja;if(null!==Q)for(const cb of Q)cb(Ja,b);zm(ea,U,Ia);Vb=Ja;Sm(qb,Ia,null)}finally{dh()}return new nn(this.fq,Vb,ri(Ma,Ia),Ia,Ma)}finally{qe(e)}}},nn=class extends pi{constructor(a,
b,c,d,e){super();this.location=c;this.G=d;this.F=e;this.D=null;this.instance=b;this.C=new dn(d,void 0,!1);this.fq=a}Ao(a,b){var c=this.F.inputs;let d;null!==c&&(d=c[a])&&(null!=this.D||(this.D=new Map),this.D.has(a)&&Object.is(this.D.get(a),b)||(c=this.G,Jm(c[1],c,d,a,b),this.D.set(a,b),a=Jg(this.F.index,c),cl(a,1)))}get xa(){return new Fh(this.F,this.G)}destroy(){this.C.destroy()}ob(a){this.C.ob(a)}};function mn(){hh(Ug()[1],Vg())};var on=class extends Zh{constructor(a,b){super();this.Qa=b;this.fp=[];this.F=[];this.C=new jn(this);var c=(a[ce]||null).Kk;this.fp=c instanceof Function?c():c;this.D=Uh(a,b,[{oa:Zh,nb:this},{oa:Yh,nb:this.C}],he(a),new Set(["environment"]));pg(this.D);this.instance=this.D.get(a)}get xa(){return this.D}destroy(){const a=this.D;!a.C&&a.destroy();this.F.forEach(b=>b());this.F=null}ob(a){this.F.push(a)}},pn=class extends $h{constructor(a){super();this.C=a}create(a){return new on(this.C,a)}},qn=class extends Zh{constructor(a){super();
this.C=new jn(this);this.instance=null;const b=new vg([...a.Wa,{oa:Zh,nb:this},{oa:Yh,nb:this.C}],a.parent||jg(),a.yv,new Set(["environment"]));this.xa=b;a.zx&&pg(b)}destroy(){this.xa.destroy()}ob(a){this.xa.ob(a)}};function rn(a,b,c=null){return(new qn({Wa:a,parent:b,yv:c,zx:!0})).xa};function sn(a){return!!a&&"function"===typeof a.then};var tn=new df("");function un(a){if(!a.C){var b=[];for(const d of a.D){const e=d();if(sn(e))b.push(e);else if(e&&"function"===typeof e.subscribe){const f=new Promise((g,k)=>{e.subscribe({complete:g,error:k})});b.push(f)}}var c=()=>{a.done=!0;a.resolve()};Promise.all(b).then(()=>{c()}).catch(d=>{a.reject(d)});0===b.length&&c();a.C=!0}}var vn=class{constructor(){this.done=this.C=!1;this.F=new Promise((b,c)=>{this.resolve=b;this.reject=c});let a;this.D=null!=(a=lf(tn,{optional:!0}))?a:[]}};
vn.K=function(a){return new (a||vn)};vn.pa=Md({ma:vn,ba:vn.K,ea:"root"});var wn=class{log(a){console.log(a)}warn(a){console.warn(a)}};wn.K=function(a){return new (a||wn)};wn.pa=Md({ma:wn,ba:wn.K,ea:"platform"});var xn=class{constructor(){this.F=0;this.C=new Set;this.ll=new ej(!1)}get D(){return this.ll.value}add(){this.D||this.ll.next(!0);const a=this.F++;this.C.add(a);return a}remove(a){this.C.delete(a);0===this.C.size&&this.D&&this.ll.next(!1)}va(){this.C.clear();this.D&&this.ll.next(!1)}};xn.K=function(a){return new (a||xn)};xn.pa=Md({ma:xn,ba:xn.K,ea:"root"});function yn(a){return"function"===typeof a&&void 0!==a[pe]};var zn=new df(""),An=new df("");function Bn(a){return a.D?a.D.XF.map(b=>({source:b.source,oB:b.oB,data:b.data})):[]}function Cn(a,b,c,d){let e=-1;c&&0<c&&(e=setTimeout(()=>{a.C=a.C.filter(f=>f.Gs!==e);b()},c));a.C.push({FB:b,Gs:e,yy:d})}
var Fn=class{constructor(a,b,c){this.qa=a;this.G=b;this.F=!0;this.C=[];this.D=null;Dn||(Dn=c,En(b));this.N();a.run(()=>{this.D="undefined"==typeof Zone?null:Zone.current.get("TaskTrackingZone")})}N(){this.qa.ko.subscribe({next:()=>{this.F=!1}});Mk(this.qa,()=>{this.qa.Jl.subscribe({next:()=>{Lk();queueMicrotask(()=>{this.F=!0;this.J()})}})})}isStable(){return this.F&&!this.qa.In}J(){if(this.isStable())queueMicrotask(()=>{for(;0!==this.C.length;){let a=this.C.pop();clearTimeout(a.Gs);a.FB()}});else{let a=
Bn(this);this.C=this.C.filter(b=>b.yy&&b.yy(a)?(clearTimeout(b.Gs),!1):!0)}}whenStable(a,b,c){if(c&&!this.D)throw Error('Task tracking zone is required when passing an update callback to whenStable(). Is "zone.js/plugins/task-tracking" loaded?');Cn(this,a,b,c);this.J()}findProviders(){return[]}};Fn.K=function(a){return new (a||Fn)(r(Qk),r(Gn),r(An))};Fn.pa=Md({ma:Fn,ba:Fn.K});var Gn=class{constructor(){this.C=new Map}fl(a,b=!0){let c,d;return null!=(d=null==(c=Dn)?void 0:c.fl(this,a,b))?d:null}};
Gn.K=function(a){return new (a||Gn)};Gn.pa=Md({ma:Gn,ba:Gn.K,ea:"platform"});let Dn;function Hn(a){return function(b){return rj(b,new In(a))}}class In{constructor(a){this.C=a}call(a,b){return b.subscribe(new Jn(a,this.C))}}function Kn(a){const b=a.value,c=a.D,d=a.C;d&&(a.remove(d),a.C=null,d.unsubscribe());c&&(a.value=null,a.D=!1,a.destination.next(b))}
class Jn extends Lj{constructor(a,b){super(a);this.F=b;this.value=null;this.D=!1;this.C=null}Oa(a){this.value=a;this.D=!0;if(!this.C){let b;try{const c=this.F;b=c(a)}catch(c){return this.destination.error(c)}a=Nj(b,new Jj(this));!a||a.closed?Kn(this):this.add(this.C=a)}}Eg(){Kn(this)}Dg(){Kn(this)}};function Ln(a){return Hn(()=>wk(a))};function Mn(a){return b=>rj(b,function(c){const d=this,e=new Ni;let f=null,g=!1,k;f=c.subscribe(new Nn(d,m=>{try{k=Sj(a(m,Mn(a)(c)))}catch(p){d.error(p)}k&&(f?(f.unsubscribe(),f=null,e.add(k.subscribe(d))):g=!0)}));g?(f.unsubscribe(),f=null,e.add(k.subscribe(d))):e.add(f);return e})}class Nn extends Qi{constructor(a,b){super(a);this.onError=b}pe(a){this.onError(a);this.unsubscribe()}};function On(a){return b=>rj(b,new Pn(a))}class Pn{constructor(a){this.C=a;this.scheduler=qk}call(a,b){return b.subscribe(new Qn(a,this.C,this.scheduler))}}function Rn(a){const b=a.D;null!==b&&(a.remove(b),b.unsubscribe(),a.D=null)}function Sn(a){Rn(a);if(a.C){const b=a.F;a.F=null;a.C=!1;a.destination.next(b)}}
class Qn extends Qi{constructor(a,b,c){super(a);this.G=b;this.scheduler=c;this.F=this.D=null;this.C=!1}Oa(a){Rn(this);this.F=a;this.C=!0;this.add(this.D=this.scheduler.Oc(Tn,this.G,this))}oc(){Sn(this);this.destination.complete()}}function Tn(a){Sn(a)};function Un(a=null){return b=>rj(b,new Vn(a))}class Vn{constructor(a){this.defaultValue=a}call(a,b){return b.subscribe(new Wn(a,this.defaultValue))}}class Wn extends Qi{constructor(a,b){super(a);this.defaultValue=b;this.C=!0}Oa(a){this.C=!1;this.destination.next(a)}oc(){this.C&&this.destination.next(this.defaultValue);this.destination.complete()}};function Xn(a=qk){return b=>rj(b,new Yn(a))}class Yn{constructor(a){this.delay=0;this.scheduler=a}call(a,b){return b.subscribe(new Zn(a,this.delay,this.scheduler))}}function $n(a){var b=a.source;const c=b.C,d=a.scheduler,e=a.destination;for(;0<c.length&&0>=c[0].time-d.now();)e.next(c.shift().value);0<c.length?(b=Math.max(0,c[0].time-d.now()),this.Oc(a,b)):(b.Rb?b.destination.complete():this.unsubscribe(),b.active=!1)}
class Zn extends Qi{constructor(a,b,c){super(a);this.destination=a;this.delay=b;this.scheduler=c;this.C=[];this.active=!1}D(a){this.active=!0;const b=this.destination;b.add(a.Oc($n,this.delay,{source:this,destination:b,scheduler:a}))}Oa(a){const b=this.scheduler;a=new ao(b.now()+this.delay,a);this.C.push(a);!1===this.active&&this.D(b)}pe(a){this.C.length=0;this.destination.error(a);this.unsubscribe()}oc(){0===this.C.length&&this.destination.complete();this.unsubscribe()}}
class ao{constructor(a,b){this.time=a;this.value=b}};function bo(){return a=>rj(a,new co)}class co{call(a,b){return b.subscribe(new eo(a))}}class eo extends Qi{constructor(a){super(a);this.C=!1}D(a,b){return a===b}Oa(a){let b;try{b=a}catch(d){return this.destination.error(d)}let c=!1;if(this.C)try{const d=this.D;c=d(this.key,b)}catch(d){return this.destination.error(d)}else this.C=!0;c||(this.key=b,this.destination.next(a))}};function fo(){if(isNaN(1))throw new TypeError("'count' is not a number");return a=>rj(a,new go)}class go{constructor(){this.count=1}call(a,b){return b.subscribe(new ho(a,this.count))}}class ho extends Qi{constructor(a,b){super(a);this.count=b;this.C=0}Oa(a){const b=this.count,c=++this.C;c<=b&&(this.destination.next(a),c===b&&(this.destination.complete(),this.unsubscribe()))}};function io(a=jo){return b=>rj(b,new ko(a))}class ko{constructor(a){this.C=a}call(a,b){return b.subscribe(new lo(a,this.C))}}class lo extends Qi{constructor(a,b){super(a);this.D=b;this.C=!1}Oa(a){this.C=!0;this.destination.next(a)}oc(){if(this.C)return this.destination.complete();let a;try{a=this.D()}catch(b){a=b}this.destination.error(a)}}function jo(){return new pj};function mo(a){return b=>rj(b,new no(a))}class no{constructor(a){this.callback=a}call(a,b){a=b.subscribe(a);a.add(this.callback);return a}};function oo(a,b){const c=2<=arguments.length;return d=>C(d,a?tk((e,f)=>a(e,f,d)):Ti,fo(),c?Un(b):io(()=>new pj))};function po(){if(isNaN(1))throw new TypeError("'count' is not a number");return function(a){return rj(a,new qo)}}class qo{constructor(){this.total=1}call(a,b){return b.subscribe(new ro(a,this.total))}}
class ro extends Qi{constructor(a,b){super(a);this.total=b;this.C=[];this.count=0}Oa(a){const b=this.C,c=this.total,d=this.count++;b.length<c?b.push(a):b[d%c]=a}oc(){const a=this.destination;let b=this.count;if(0<b){const c=this.count>=this.total?this.total:this.count,d=this.C;for(let e=0;e<c;e++){const f=b++%c;a.next(d[f])}}a.complete()}};function so(a,b){const c=2<=arguments.length;return d=>C(d,a?tk((e,f)=>a(e,f,d)):Ti,po(),c?Un(b):io(()=>new pj))};function to(a){return b=>rj(b,new uo(a))}class uo{constructor(a){this.value=a}call(a,b){return b.subscribe(new vo(a,this.value))}}class vo extends Qi{constructor(a,b){super(a);this.value=b}Oa(){this.destination.next(this.value)}};function wo(a,b){let c=!1;2<=arguments.length&&(c=!0);return function(d){return rj(d,new xo(a,b,c))}}class xo{constructor(a,b,c=!1){this.C=a;this.D=b;this.F=c}call(a,b){return b.subscribe(new yo(a,this.C,this.D,this.F))}}class yo extends Qi{constructor(a,b,c,d){super(a);this.D=b;this.pc=c;this.C=d;this.index=0}Oa(a){const b=this.destination;if(this.C){const c=this.index++;let d;try{d=this.D(this.pc,a,c)}catch(e){b.error(e);return}this.pc=d;b.next(d)}else this.pc=a,this.C=!0,b.next(a)}};function zo(){var a=Ao;return function(b){let c;c="function"===typeof a?a:function(){return a};const d=Object.create(b,vj);d.source=b;d.hy=c;return d}};function Ao(){return new cj}function Bo(){return a=>sj()(zo()(a))};function Co(){var a={bufferSize:1,Rl:!0};let b;a&&"object"===typeof a?b=a:b={bufferSize:a,UE:void 0,Rl:!1,scheduler:void 0};return c=>rj(c,Do(b))}
function Do({bufferSize:a=Infinity,UE:b=Infinity,Rl:c,scheduler:d}){let e,f=0,g;return function(k){f++;let m;e?m=e.subscribe(this):(e=new mj(a,b,d),m=e.subscribe(this),g=k.subscribe({next(p){e.next(p)},error(p){const t=e;e=g=void 0;t.error(p)},complete(){g=void 0;e.complete()}}),g.closed&&(g=void 0));this.add(()=>{f--;m.unsubscribe();c&&0===f&&g&&(g.unsubscribe(),e=g=void 0)})}};function Eo(){return a=>rj(a,new Fo)}class Fo{constructor(){this.total=1}call(a,b){return b.subscribe(new Go(a,this.total))}}class Go extends Qi{constructor(a,b){super(a);this.total=b;this.count=0}Oa(a){++this.count>this.total&&this.destination.next(a)}};function Ho(...a){const b=a[a.length-1];return ij(b)?(a.pop(),c=>hk(a,c,b)):c=>hk(a,c)};function Io(a){return b=>rj(b,new Jo(a))}class Jo{constructor(a){this.project=a}call(a,b){return b.subscribe(new Ko(a,this.project))}}
class Ko extends Lj{constructor(a,b){super(a);this.destination=a;this.project=b;this.index=0}Oa(a){let b;const c=this.index++;try{b=this.project(a,c)}catch(d){this.destination.error(d);return}(a=this.C)&&a.unsubscribe();a=new Jj(this);this.destination.add(a);this.C=a;Nj(b,a)}oc(){const a=this.C;a&&!a.closed||super.oc();this.C=void 0;this.unsubscribe()}Dg(){this.C=void 0;this.Rb&&super.oc()}Eg(a){this.destination.next(a)}};function Lo(a){return b=>rj(b,function(c){const d=this;Sj(a).subscribe(new qj(d,()=>d.complete(),sk));!d.closed&&c.subscribe(d)})};function Mo(a){return b=>rj(b,new No(a))}class No{constructor(a){this.mf=a}call(a,b){return b.subscribe(new Oo(a,this.mf))}}class Oo extends Qi{constructor(a,b){super(a);this.mf=b;this.index=0}Oa(a){var b=this.destination;let c;try{c=this.mf(a,this.index++)}catch(d){b.error(d);return}b=this.destination;c?b.next(a):b.complete()}};function Po(a){return function(b){return rj(b,new Qo(a))}}class Qo{constructor(a){this.C=a;this.complete=this.error=void 0}call(a,b){return b.subscribe(new Ro(a,this.C,this.error,this.complete))}}
class Ro extends Qi{constructor(a,b,c,d){super(a);this.C=this.D=this.F=sk;this.D=c||sk;this.C=d||sk;Mi(b)?(this.Db=this,this.F=b):b&&(this.Db=b,this.F=b.next||sk,this.D=b.error||sk,this.C=b.complete||sk)}Oa(a){try{this.F.call(this.Db,a)}catch(b){this.destination.error(b);return}this.destination.next(a)}pe(a){try{this.D.call(this.Db,a)}catch(b){this.destination.error(b);return}this.destination.error(a)}oc(){try{this.C.call(this.Db)}catch(a){this.destination.error(a);return}return this.destination.complete()}}
;var So={jr:!0,Is:!1};function To(a){return b=>rj(b,new Uo(a))}class Uo{constructor(a){var b=!!So.jr,c=!!So.Is;this.C=a;this.jr=b;this.Is=c}call(a,b){return b.subscribe(new Vo(a,this.C,this.jr,this.Is))}}function Wo(a){const b=a.D;a.C&&(a.destination.next(b),Xo(a,b));a.C=!1;a.D=null}function Xo(a,b){let c;try{c=a.G(b)}catch(d){a.destination.error(d);return}a.add(a.F=Nj(c,new Jj(a)))}function Yo(a){const b=a.F,c=a.N;b&&b.unsubscribe();a.F=null;c&&Wo(a)}
class Vo extends Lj{constructor(a,b,c,d){super(a);this.destination=a;this.G=b;this.J=c;this.N=d;this.D=null;this.C=!1}Oa(a){this.C=!0;this.D=a;this.F||(this.J?Wo(this):Xo(this,a))}Eg(){Yo(this)}Dg(){Yo(this)}};var Zo=new df("");function $o(){Le=()=>{throw new Zd(600,!1);}}function ap(a,b,c){try{const d=c();return sn(d)?d.catch(e=>{Mk(b,()=>a.handleError(e));throw e;}):d}catch(d){throw Mk(b,()=>a.handleError(d)),d;}}
function bp(a,b){let c=null;a.Ua.C||(c=a.Ua.get(ui,null,{optional:!0}));let d=0;const e=a.fa;for(;10>d;){const f=0===d;if(b||!f){a.ja.next(f);for(let {D:g,J:k}of a.D)cp(g,k,f,a.aa)}else{let g,k,m;null==(g=c)||null==(m=(k=g).Vm)||m.call(k);let p,t,w;null==(p=c)||null==(w=(t=p).end)||w.call(t)}d++;bl(e);if(!a.P.some(({D:g})=>Lg(g))&&(e.lj(),!a.P.some(({D:g})=>Lg(g))))break}}
function dp(a,b){a.D.push(b);if(b.G)throw new Zd(902,!1);b.F=a;a=0!==(b.D[2]&512);const c=b.D[16];null===c||a||ml(c,b.D);Mg(b.D)}
var fp=class{constructor(){this.N=[];this.Ga=this.G=!1;this.F=[];this.D=[];this.sa=lf(xi);this.fa=lf(Yk);this.aa=lf(bi);this.na=new Set;this.ja=new cj;this.O=new cj;this.T=[];this.Kf=[];this.isStable=C(lf(xn).ll,xj(a=>!a));this.Ua=lf(kg)}get P(){return[...this.na.keys(),...this.D]}get C(){return this.Ga}get xa(){return this.Ua}Kk(a,b){const c=a instanceof qi;if(!this.Ua.get(vn).done)throw new Zd(405,!1);let d;c?d=a:d=gn(this.Ua.get(Yh),a);this.T.push(d.fq);a=d.tC?void 0:this.Ua.get(Zh);const e=d.create(Xh,
[],b||d.Wl,a),f=e.location.R,g=e.xa.get(zn,null);null!=g&&g.G.C.set(f,g);e.ob(()=>{this.fj(e.C);ep(this.Kf,e);null!=g&&g.G.C.delete(f)});this.ca(e);return e}J(a){if(this.G)throw new Zd(101,!1);const b=qe(null);try{this.G=!0,bp(this,a)}catch(c){this.sa(c)}finally{this.G=!1,qe(b),this.O.next()}}fj(a){ep(this.D,a);a.F=null;const b=0!==(a.D[2]&512),c=a.D[16];null===c||b||nl(c,a.D);ll(a.D[1],a.D)}ca(a){dp(this,a.C);this.J(!0);this.Kf.push(a);const b=this.Ua.get(Zo,[]);[...this.N,...b].forEach(c=>c(a))}va(){if(!this.Ga)try{this.F.forEach(a=>
a()),this.D.slice().forEach(a=>a.destroy())}finally{this.Ga=!0,this.D=[],this.N=[],this.F=[]}}ob(a){this.F.push(a);return()=>ep(this.F,a)}destroy(){if(this.Ga)throw new Zd(406,!1);const a=this.Ua;a.destroy&&!a.C&&a.destroy()}get ua(){return this.D.length}};fp.K=function(a){return new (a||fp)};fp.pa=Md({ma:fp,ba:fp.K,ea:"root"});function ep(a,b){b=a.indexOf(b);-1<b&&a.splice(b,1)}function cp(a,b,c,d){(c||Lg(a))&&Ym(a,b,c&&!d?0:1)};function gp(a,b,c,d){const e=qe(null);try{const f=b.ub;let g,k,m;const p=vm(a,f,c,a[2]&4096?4096:16,null,b,null,null,null!=(g=null==d?void 0:d.xa)?g:null,null!=(k=null==d?void 0:d.HB)?k:null,null!=(m=null==d?void 0:d.qq)?m:null);p[16]=a[b.index];const t=a[18];null!==t&&(p[18]=t.xd(f));Sm(f,p,c);return p}finally{qe(e)}}function hp(a,b){b=10+b;if(b<a.length)return a[b]}function ip(a,b){return!b||null===b.firstChild||128===(a.flags&128)}
function jp(a,b,c,d=!0){const e=b[1];var f=10+c;const g=a.length;0<c&&(a[f-1][4]=b);c<g-10?(b[4]=a[f],f=10+c,f>=a.length?a.push(b):a.splice(f,0,b)):(a.push(b),b[4]=null);b[3]=a;f=b[16];null!==f&&a!==f&&ml(f,b);f=b[18];null!==f&&Tm(f,e);Mg(b);b[2]|=128;d&&(c=ul(c,a),f=b[11],d=f.parentNode(a[7]),null!==d&&(a=a[5],b[0]=d,b[5]=a,kl(f,1,e.firstChild,b,d,c,!1)));b=b[6];null!==b&&null!==b.firstChild&&(b.firstChild=null)};var kp=class{};kp.wf=lp;function lp(){return mp(Vg(),Ug())}function np(a,b,c,d){const e=b.D;if(Bg(e[3])){var f=a.indexOf(b);-1!==f?a.detach(f):(f=e[3],f=new op(f,f[5],f[3]),f.detach(f.indexOf(b)))}c=a.Zo(c);a=a.jg;jp(a,e,c,d);if(b.F)throw new Zd(902,!1);b.G=!0;d=pp(a);c>=d.length?d.push(b):d.splice(c,0,b);return b}
function qp(a,b,c,d,e){var f=b&&"function"!==typeof b;if(!f){var g=c||{};c=g.index;d=g.xa;e=g.uo;g=g.Of||g.bG}b=f?b:new hn(b[$d]||null);d=d||a.Cj;g||null!=b.Aj||(f=(f?d:a.Cj).get(kg,null))&&(g=f);e=b.create(d,e,null,g);np(a,e.C,c,ip(a.tk,null));return e}
const op=class extends kp{constructor(a,b,c){super();this.jg=a;this.tk=b;this.sk=c}get element(){return ri(this.tk,this.sk)}get xa(){return new Fh(this.tk,this.sk)}get Cj(){const a=wh(this.tk,this.sk);if(-1!==a){const b=nh(a,this.sk);return new Fh(b[1].data[(a&32767)+8],b)}return new Fh(null,this.sk)}clear(){for(;0<this.length;)this.remove(this.length-1)}get(a){const b=this.jg[8];return null!==b&&b[a]||null}get length(){return this.jg.length-10}xd(a,b,c){let d,e;"number"===typeof c?d=c:null!=c&&(d=
c.index,e=c.xa);a=rp(a,b||{},e,null);np(this,a,d,ip(this.tk,null));return a}ar(a,b){return np(this,a,b,!0)}move(a,b){return this.ar(a,b)}indexOf(a){const b=this.jg[8];return null!==b?b.indexOf(a):-1}remove(a){a=this.Zo(a,-1);const b=ol(this.jg,a);b&&(Uf(pp(this.jg),a),pl(b[1],b))}detach(a){a=this.Zo(a,-1);const b=ol(this.jg,a);return b&&null!=Uf(pp(this.jg),a)?new dn(b):null}Zo(a,b=0){return null==a?this.length+b:a}};function pp(a){return a[8]||(a[8]=[])}
function mp(a,b){let c;const d=b[a.index];Bg(d)?c=d:(c=[d,!0,0,b,null,a,null,null,null,null],b[a.index]=c,Cm(b,c));sp(c,b,a,d);return new op(c,a,b)}let sp=tp;function tp(a,b,c,d){if(!a[7]){if(c.type&8)d=Hg(d);else{var e=b[11];d=e.createComment("");const g=Ig(c,b);c=e.parentNode(g);b=e;var f=d;e=e.nextSibling(g);b.insertBefore(c,f,e,!1)}a[7]=d}};function up(a){return null===a||"function"!==typeof a&&"object"!==typeof a?!1:Array.isArray(a)||!(a instanceof Map)&&Symbol.iterator in a}function vp(a,b){if(Array.isArray(a))for(var c=0;c<a.length;c++)b(a[c]);else for(a=a[Symbol.iterator]();!(c=a.next()).done;)b(c.value)};function wp(a,b,c){if(Object.is(a[b],c))return!1;a[b]=c;return!0}function xp(a,b,c,d){c=wp(a,b,c);return wp(a,b+1,d)||c};function yp(a,b,c,d,e,f,g,k,m,p){const t=c+25;if(b.Bc){const w=b.Ca;g=wm(b,t,4,g||null,k||null);Km(b,a,g,Kg(w,m));hh(b,g);d=g.ub=Fm(2,g,d,e,f,b.Ev,b.vD,null,b.Pj,w,null);null!==b.queries&&(b.queries.ta(b,g),d.queries=b.queries.Aq(g));d=g}else d=b.data[t];Wg(d,!1);c=zp(b,a,d,c);gh&&sl(b,a,c,d);hi(c,a);c=[c,!0,0,a,null,d,null,c,null,null];a[t]=c;Cm(a,c);1===(d.flags&1)&&Am(b,a,d);null!=m&&Em(a,d,p)}function Ap(a,b,c,d,e,f,g,k){const m=Ug(),p=Rg.ub;f=Kg(p.Ca,f);yp(m,p,a,b,c,d,e,f,g,k);return Ap}
let zp=Bp;function Bp(a,b){gh=!0;return b[11].createComment("")};new df("DEFER_BLOCK_DEPENDENCY_INTERCEPTOR");new df("");function Cp(a){var b=Object.getPrototypeOf(a.type.prototype).constructor,c=!0;const d=[a];for(;b;){var e=void 0;if(a.ta)e=b.Da||b.Fa;else{if(b.Da)throw new Zd(903,!1);e=b.Fa}if(e){if(c){d.push(e);var f=a;f.inputs=Dp(a.inputs);f.De=Dp(a.De);f.ej=Dp(a.ej);f.outputs=Dp(a.outputs);(f=e.Ia)&&Ep(a,f);f=e.fb;var g=e.ec;f&&Fp(a,f);g&&Gp(a,g);f=a;g=e;for(var k in g.inputs)if(g.inputs.hasOwnProperty(k)&&!f.inputs.hasOwnProperty(k)){var m=g.inputs[k];if(void 0!==m&&(f.inputs[k]=m,f.ej[k]=g.ej[k],null!==g.De)){m=
Array.isArray(m)?m[0]:m;if(!g.De.hasOwnProperty(m))continue;let p;null!=(p=f).De||(p.De={});f.De[m]=g.De[m]}}f=a.outputs;g=e.outputs;for(const p in g)g.hasOwnProperty(p)&&!f.hasOwnProperty(p)&&(f[p]=g[p]);e.ta&&e.data.animation&&(f=a.data,f.animation=(f.animation||[]).concat(e.data.animation))}if(e=e.features)for(f=0;f<e.length;f++)(g=e[f])&&g.Mw&&g(a),g===Cp&&(c=!1)}b=Object.getPrototypeOf(b)}a=0;b=null;for(c=d.length-1;0<=c;c--)k=d[c],k.Za=a+=k.Za,k.Ka=vf(k.Ka,b=vf(b,k.Ka))}
function Dp(a){return a===Ef?{}:a===Ff?[]:a}function Fp(a,b){const c=a.fb;a.fb=c?(d,e)=>{b(d,e);c(d,e)}:b}function Gp(a,b){const c=a.ec;a.ec=c?(d,e,f)=>{b(d,e,f);c(d,e,f)}:b}function Ep(a,b){const c=a.Ia;a.Ia=c?(d,e)=>{b(d,e);c(d,e)}:b};function Hp(a,b,c){if(null!==a.ol)for(const e of a.ol){const f=e.rq[ae]||null;a=f.ej;var d=e.inputs;for(const g in d)d.hasOwnProperty(g)&&(a[d[g]]=a[g]);Hp(f,b,c);c.set(f,e);b.push(f)}}function Ip(a){if(void 0===a||0===a.length)return Ef;const b={};for(let c=0;c<a.length;c+=2)b[a[c]]=a[c+1];return b};function Jp(a){const b=a.rC,c={};for(const d in b)if(b.hasOwnProperty(d)){const e=b[d];Array.isArray(e)&&e[3]&&(c[d]=e[3])}a.De=c};function Kp(a,b,c,d){var e=Ug(),f=Rg.ed++;if(wp(e,f,b)){f=eh();var g=Ig(f,e);e=e[11];f=f.value;null==b?e.removeAttribute(g,a,d):(b=null==c?ge(b):c(b,f||"",a),e.setAttribute(g,a,b,d))}return Kp};function H(a,b,c){const d=Ug();var e=Rg.ed++;if(wp(d,e,b)){e=Rg.ub;const f=eh();Im(e,f,d,a,b,d[11],c,!1)}return H}function Lp(a,b,c,d,e){e=e?"class":"style";Jm(a,c,b.inputs[e],e,d)};function Mp(a,b,c,d){const e=a[c+1],f=null===b;let g=d?e>>17&32767:(e&131068)>>2,k=!1;for(;0!==g&&(!1===k||f);){const p=a[g+1];var m=a[g];if(null===m||null==b||(Array.isArray(m)?m[1]:m)===b||Array.isArray(m)&&"string"===typeof b&&0<=Wf(m,b))k=!0,a[g+1]=d?p|1:p|2;g=d?p>>17&32767:(p&131068)>>2}k&&(a[c+1]=d?e|2:e|1)};var gq=0,hq=0,iq=0;function jq(a,b){const c=gq;if(c===b)return-1;for(b=hq=b;b<c&&32<a.charCodeAt(b);)b++;b=iq=b;return kq(a,b,c)}function kq(a,b,c){for(;b<c&&32>=a.charCodeAt(b);)b++;return b};function lq(a,b,c){mq(a,b,c,!1);return lq}function nq(a,b){mq(a,b,null,!0);return nq}function mq(a,b,c,d){const e=Ug(),f=Rg.ub,g=Yg(2);f.xn&&oq(f,a,g,d);if(b!==xl&&wp(e,g,b)){var k=f.data[Rg.selectedIndex],m=e[11];null!=b&&""!==b&&("string"===typeof c?b+=c:"object"===typeof b&&(b=he(Bl(b))));pq(f,k,e,m,a,e[g+1]=b,d,g)}}
function qq(a,b){const c=Rg.ub;var d=Yg(2);c.xn&&oq(c,null,d,!0);const e=Ug();if(b!==xl&&wp(e,d,b)){var f=c.data[Rg.selectedIndex];if(0===(f.flags&8)||d>=c.Pv){var g=e[11],k=e[d+1];if(null==b||""===b)var m=Ff;else if(m=[],b=Bl(b),Array.isArray(b))for(var p=0;p<b.length;p++)a(m,b[p],!0);else if("object"===typeof b)for(p in b)b.hasOwnProperty(p)&&a(m,p,b[p]);else if("string"===typeof b)for(iq=hq=0,gq=b.length,a=jq(b,kq(b,0,gq));0<=a;a=jq(b,a))Vf(m,b.substring(hq,iq),!0);m=e[d+1]=m;k===xl&&(k=Ff);a=
b=0;p=0<k.length?k[0]:null;let t=0<m.length?m[0]:null;for(;null!==p||null!==t;){const w=b<k.length?k[b+1]:void 0,B=a<m.length?m[a+1]:void 0;let G=null,J=void 0;p===t?(b+=2,a+=2,w!==B&&(G=t,J=B)):null===t||null!==p&&p<t?(b+=2,G=p):(a+=2,G=t,J=B);null!==G&&pq(c,f,e,g,G,J,!0,d);p=b<k.length?k[b]:null;t=a<m.length?m[a]:null}}else d=f.dn,null!==d&&(b=ie(d,b?b:"")),Lp(c,f,e,b,!0)}}
function oq(a,b,c,d){const e=a.data;if(null===e[c+1]){var f=e[Rg.selectedIndex];a=c>=a.Pv;0!==(f.flags&(d?8:16))&&null===b&&!a&&(b=!1);var g=f,k=Zg(e),m=d?g.xo:g.yo;if(null===k)0===(d?g.Ok:g.cm)&&(b=rq(null,e,g,b,d),b=sq(b,g.attrs,d),m=null);else{var p=g.nn;if(-1===p||e[p]!==k)if(b=rq(k,e,g,b,d),null===m)k=d?g.Ok:g.cm,k=0!==(k&131068)>>2?e[k>>17&32767]:void 0,void 0!==k&&Array.isArray(k)&&(k=rq(null,e,g,k[1],d),k=sq(k,g.attrs,d),e[(d?g.Ok:g.cm)>>17&32767]=k);else{m=void 0;k=g.we;for(p=1+g.nn;p<k;p++)m=
sq(m,e[p].Ka,d);m=sq(m,g.attrs,d)}}void 0!==m&&(d?g.xo=m:g.yo=m);m=a;g=d?f.Ok:f.cm;a=g>>17&32767;g=(g&131068)>>2;e[c]=b;k=!1;if(Array.isArray(b)){if(p=b[1],null===p||0<Wf(b,p))k=!0}else p=b;m?0!==g?(b=e[a+1]>>17&32767,e[c+1]=b<<17|a<<2,0!==b&&(e[b+1]=e[b+1]&-131069|c<<2),e[a+1]=e[a+1]&131071|c<<17):(e[c+1]=a<<17|0,0!==a&&(e[a+1]=e[a+1]&-131069|c<<2),a=c):(e[c+1]=g<<17|0,0===a?a=c:e[g+1]=e[g+1]&-131069|c<<2,g=c);k&&(e[c+1]|=2);Mp(e,p,c,!0);Mp(e,p,c,!1);b=p;m=d?f.xo:f.yo;null!=m&&"string"==typeof b&&
0<=Wf(m,b)&&(e[c+1]|=1);g=a<<17|g<<2;d?f.Ok=g:f.cm=g}}function rq(a,b,c,d,e){let f;const g=c.we;let k=c.nn;for(-1===k?k=c.nd:k++;k<g;){f=b[k];d=sq(d,f.Ka,e);if(f===a)break;k++}null!==a&&(c.nn=k);return d}function sq(a,b,c){const d=c?1:2;let e=-1;if(null!==b)for(let f=0;f<b.length;f++){const g=b[f];"number"===typeof g?e=g:e===d&&(Array.isArray(a)||(a=void 0===a?[]:["",a]),Vf(a,g,c?!0:b[++f]))}return void 0===a?null:a}function tq(a,b,c){b=String(b);""===b||b.includes(" ")||Vf(a,b,c)}
function pq(a,b,c,d,e,f,g,k){if(b.type&3){a=a.data;var m=a[k+1];void 0===(1===(m&1)?uq(a,b,c,e,(m&131068)>>2,g):void 0)&&(void 0!==f||2==(m&2)&&(f=uq(a,null,c,e,k,g)),b=Hg(c[Rg.selectedIndex]),g?f?d.Ri(b,e):d.wo(b,e):(g=-1===e.indexOf("-")?void 0:2,null==f?d.Zr(b,e,g):("string"===typeof f&&f.endsWith("!important")&&(f=f.slice(0,-10),g|=1),d.setStyle(b,e,f,g))))}}
function uq(a,b,c,d,e,f){const g=null===b;let k=void 0;for(;0<e;){const p=a[e],t=Array.isArray(p);var m=t?p[1]:p;const w=null===m;let B=c[e+1];B===xl&&(B=w?Ff:void 0);m=w?Xf(B,d):m===d?B:void 0;t&&void 0===m&&(m=Xf(p,d));if(void 0!==m&&(k=m,g))return k;e=a[e+1];e=g?e>>17&32767:(e&131068)>>2}null!==b&&(a=f?b.xo:b.yo,null!=a&&(k=Xf(a,d)));return k};function vq(a){a=wp(Ug(),Rg.ed++,a)?"mat-mdc-select-panel mdc-menu-surface mdc-menu-surface--open "+ge(a):xl;qq(Vf,a)};function wq(a,b,c){const d=Math.min(b,c);b=Math.max(b,c);c=a.detach(b);if(1<b-d){const e=a.detach(d);a.attach(d,c);a.attach(b,e)}else a.attach(d,c)}var xq=class{destroy(){}C(){}move(a,b){this.attach(b,this.detach(a))}};function yq(a,b,c,d,e){return a===c&&Object.is(b,d)?1:Object.is(e(a,b),e(c,d))?-1:0}
function zq(a,b,c){let d=void 0,e=void 0,f=0,g=a.length-1;if(Array.isArray(b)){for(var k=b.length-1;f<=g&&f<=k;){var m=a.at(f),p=b[f],t=yq(f,m,f,p,c);if(0!==t)0>t&&a.C(f,p),f++;else{var w=a.at(g);t=b[k];var B=yq(g,w,k,t,c);0!==B?(0>B&&a.C(g,t),g--,k--):(m=c(f,m),w=c(g,w),B=c(f,p),Object.is(B,w)?(w=c(k,t),Object.is(w,m)?(wq(a,f,g),a.C(g,t),k--,g--):a.move(g,f),a.C(f,p),f++):(null!=d||(d=new Aq),null!=e||(e=Bq(a,f,g,c)),Cq(a,d,f,B)?(a.C(f,p),f++,g++):e.has(B)?(d.set(m,a.detach(f)),g--):(p=a.create(f,
b[f]),a.attach(f,p),f++,g++)))}}for(;f<=k;)Dq(a,d,c,f,b[f]),f++}else if(null!=b){b=b[Symbol.iterator]();for(k=b.next();!k.done&&f<=g;)p=a.at(f),t=k.value,m=yq(f,p,f,t,c),0!==m?(0>m&&a.C(f,t),f++,k=b.next()):(null!=d||(d=new Aq),null!=e||(e=Bq(a,f,g,c)),m=c(f,t),Cq(a,d,f,m)?(a.C(f,t),f++,g++,k=b.next()):e.has(m)?(p=c(f,p),d.set(p,a.detach(f)),g--):(a.attach(f,a.create(f,t)),f++,g++,k=b.next()));for(;!k.done;)Dq(a,d,c,a.length,k.value),k=b.next()}for(;f<=g;)a.destroy(a.detach(g--));let G;null==(G=d)||
G.forEach(J=>{a.destroy(J)})}function Cq(a,b,c,d){return void 0!==b&&b.has(d)?(a.attach(c,b.get(d)),b.delete(d),!0):!1}function Dq(a,b,c,d,e){Cq(a,b,d,c(d,e))?a.C(d,e):(b=a.create(d,e),a.attach(d,b))}function Bq(a,b,c,d){const e=new Set;for(;b<=c;b++)e.add(d(b,a.at(b)));return e}
var Aq=class{constructor(){this.D=new Map;this.C=void 0}has(a){return this.D.has(a)}delete(a){if(!this.has(a))return!1;const b=this.D.get(a);void 0!==this.C&&this.C.has(b)?(this.D.set(a,this.C.get(b)),this.C.delete(b)):this.D.delete(a);return!0}get(a){return this.D.get(a)}set(a,b){if(this.D.has(a)){a=this.D.get(a);void 0===this.C&&(this.C=new Map);const c=this.C;for(;c.has(a);)a=c.get(a);c.set(a,b)}else this.D.set(a,b)}forEach(a){for(let [b,c]of this.D){let d=b,e=c;a(e,d);if(void 0!==this.C){const f=
this.C;for(;f.has(e);)e=f.get(e),a(e,d)}}}};function Eq(a){Ei("NgControlFlow");var b=Ug(),c=Rg.ed++,d=b[c]!==xl?b[c]:-1;d=-1!==d?b[25+d]:void 0;if(wp(b,c,a)){c=qe(null);try{if(void 0!==d){const e=ol(d,0);void 0!==e&&pl(e[1],e)}if(-1!==a){a=25+a;const e=b[a],f=b[1].data[a],g=gp(b,f,void 0,{qq:null});jp(e,g,0,ip(f,null))}}finally{qe(c)}}else void 0!==d&&(b=hp(d,0),void 0!==b&&(b[8]=void 0))}var Fq=class{constructor(a,b,c){this.C=a;this.da=b;this.Cb=c}get Ty(){return this.C.length-10}};function Gq(a){return a}function Hq(a,b){return b}
class Iq{constructor(a){this.D=a;this.C=void 0}}function Jq(a,b,c,d,e,f,g){Ei("NgControlFlow");const k=Ug(),m=Rg.ub;Ug()[25+a]=new Iq(g);yp(k,m,a+1,b,c,d,e,Kg(m.Ca,f))}
class Kq extends xq{constructor(a,b,c){super();this.D=a;this.N=b;this.J=c;this.G=void 0;this.F=!1}get length(){return this.D.length-10}at(a){return hp(this.D,a)[8].da}attach(a,b){const c=b[6];this.F||(this.F=a!==this.length);jp(this.D,b,a,ip(this.J,c))}detach(a){this.F||(this.F=a!==this.length-1);return ol(this.D,a)}create(a,b){a=gp(this.N,this.J,new Fq(this.D,b,a),{qq:null});let c;null==(c=this.G)||c.C();return a}destroy(a){pl(a[1],a);let b;null==(b=this.G)||b.D()}C(a,b){hp(this.D,a)[8].da=b}reset(){this.F=
!1;let a;null==(a=this.G)||a.reset()}}function Lq(a){const b=qe(null);var c=Rg.selectedIndex;try{const d=Ug(),e=d[1],f=d[c];c+=1;const g=d[c];void 0===f.C?f.C=new Kq(g,d,e.data[c]):f.C.reset();const k=f.C;zq(k,a,f.D);if(k.F)for(a=0;a<k.length;a++)hp(k.D,a)[8].Cb=a}finally{qe(b)}};function R(a,b,c,d){const e=Ug(),f=Rg.ub;var g=25+a;a=e[11];if(f.Bc){var k=f.Ca;c=Kg(k,c);c=wm(f,g,2,b,c);Km(f,e,c,Kg(k,d));null!==c.attrs&&Um(c,c.attrs,!1);null!==c.Ge&&Um(c,c.Ge,!0);null!==f.queries&&f.queries.bl(f,c);k=c}else k=f.data[g];gh=!0;b=a.createElement(b,Rg.Vk);e[g]=b;g=1===(k.flags&1);Wg(k,!0);wl(a,b,k);32!==(k.flags&32)&&gh&&sl(f,e,b,k);0===Rg.qn&&hi(b,e);Rg.qn++;g&&(Am(f,e,k),zm(f,k,e));null!==d&&Em(e,k);return R}
function S(){let a=Vg();Rg.Rf?Rg.Rf=!1:(a=a.parent,Wg(a,!1));const b=a;Sg===b&&(Sg=null);Rg.qn--;const c=Rg.ub;c.Bc&&(hh(c,a),0!==(a.flags&4)&&c.queries.rn(a));null!=b.dn&&0!==(b.flags&8)&&Lp(c,b,Ug(),b.dn,!0);null!=b.zs&&0!==(b.flags&16)&&Lp(c,b,Ug(),b.zs,!1);return S}function Mq(a,b,c,d){R(a,b,c,d);S();return Mq};function Nq(a,b,c){const d=Ug(),e=Rg.ub;a+=25;if(e.Bc){var f=e.Ca;const g=Kg(f,b);b=wm(e,a,8,"ng-container",g);null!==g&&Um(b,g,!0);f=Kg(f,c);Km(e,d,b,f);null!==e.queries&&e.queries.bl(e,b);f=b}else f=e.data[a];Wg(f,!0);gh=!0;b=d[11].createComment(il());d[a]=b;gh&&sl(e,d,b,f);hi(b,d);1===(f.flags&1)&&(Am(e,d,f),zm(e,f,d));null!=c&&Em(d,f);return Nq}function Oq(){let a=Vg();const b=Rg.ub;Rg.Rf?Rg.Rf=!1:(a=a.parent,Wg(a,!1));b.Bc&&(hh(b,a),0!==(a.flags&4)&&b.queries.rn(a));return Oq}
function Pq(a,b,c){Nq(a,b,c);Oq();return Pq};function Qq(a,b,c){const d=Ug();var e=Rg.ed++;if(wp(d,e,b)){e=Rg.ub;const f=eh();Im(e,f,d,a,b,d[11],c,!0)}return Qq}function Rq(a,b,c){const d=Ug();var e=Rg.ed++;if(wp(d,e,b)){e=Rg.ub;const f=eh();Im(e,f,d,a,b,Pm(Zg(e.data),f,d),c,!0)}return Rq};function V(a,b,c,d){c=Ug();Sq(Rg.ub,c,c[11],Vg(),a,b,d);return V}function Tq(a,b){const c=Vg(),d=Ug(),e=Rg.ub;Sq(e,d,Pm(Zg(e.data),c,d),c,a,b);return Tq}function Uq(a,b,c,d){a=a.be;if(null!=a)for(let e=0;e<a.length-1;e+=2){const f=a[e];if(f===c&&a[e+1]===d)return b=b[7],c=a[e+2],b.length>c?b[c]:null;"string"===typeof f&&(e+=2)}return null}
function Sq(a,b,c,d,e,f,g){var k=1===(d.flags&1);const m=a.Bc&&(a.be||(a.be=[])),p=b[7]||(b[7]=[]);var t=!0;if(d.type&3||g){var w=Ig(d,b);const G=g?g(w):w;w=p.length;const J=g?aa=>g(Hg(aa[d.index])):d.index;let N=null;!g&&k&&(N=Uq(a,b,e,d.index));null!==N?((N.xz||N).ht=f,N.xz=f,t=!1):(f=Vq(d,b,f),a=c.Gd(G,e,f),p.push(f,a),m&&m.push(e,J,w,w+1))}else f=Vq(d,b,f);a=d.outputs;let B;if(t&&null!==a&&(B=a[e])&&(t=B.length))for(a=0;a<t;a+=2)c=b[B[a]][B[a+1]].subscribe(f),k=p.length,p.push(f,c),m&&m.push(e,
d.index,k,-(k+1))}function Wq(a,b,c){const d=qe(null);try{return!1!==b(c)}catch(e){return Qm(a,e),!1}finally{qe(d)}}function Vq(a,b,c){return function f(e){if(e===Function)return c;var g=-1<a.Jf?Jg(a.index,b):b;cl(g,5);g=Wq(b,c,e);let k=f.ht;for(;k;)g=Wq(b,k,e)&&g,k=k.ht;return g}};function Y(a=1){for(var b=Rg,c=Rg.Bh;0<a;)c=c[14],a--;return(b.Bh=c)[8]};function Xq(a){var b=Ug()[15][5];if(!b.Je){var c=a?a.length:1,d=[];for(var e=0;e<c;e++)d.push(null);c=b.Je=d;d=c.slice();for(b=b.Hf;null!==b;){if(a)a:{e=b;var f=a;let p=null;b:{var g=e.attrs;if(null!=g){var k=g.indexOf(5);if(0===(k&1)){g=g[k+1];break b}}g=null}for(k=0;k<f.length;k++){var m=f[k];if("*"===m)p=k;else{if(null===g)m=zf(e,m,!0);else b:{c:for(let t=0;t<m.length;t++){const w=m[t];if(g.length===w.length){for(let B=0;B<g.length;B++)if(g[B]!==w[B])continue c;m=!0;break b}}m=!1}if(m){e=k;break a}}}e=
p}else e=0;null!==e&&(d[e]?d[e].Ur=b:c[e]=b,d[e]=b);b=b.next}}}function Yq(a,b=0){const c=Ug();var d=Rg.ub;a=wm(d,25+a,16,null,null);null===a.Je&&(a.Je=b);Rg.Rf=!1;if((!c[6]||null!==Sg)&&32!==(a.flags&32)){b=c[11];d=rl(d,a.parent,c);var e=a.parent||c[5];e=e.type&40?Ig(e,c):null;vl(b,0,c,a,d,e)}};function Zq(){return this.D[Symbol.iterator]()}function $q(a){void 0===a.F||!a.G&&a.J||a.F.emit(a)}
var ar=class{get C(){let a;return null!=(a=this.F)?a:this.F=new Fk}constructor(a=!1){this.J=a;this.rc=!0;this.N=void 0;this.D=[];this.G=!1;this.F=void 0;this.length=0;this.first=void 0;const b=ar.prototype;b[Symbol.iterator]||(b[Symbol.iterator]=Zq)}get(a){return this.D[a]}map(a){return this.D.map(a)}filter(a){return this.D.filter(a)}find(a){return this.D.find(a)}reduce(a,b){return this.D.reduce(a,b)}forEach(a){this.D.forEach(a)}some(a){return this.D.some(a)}toArray(){return this.D.slice()}toString(){return this.D.toString()}reset(a,b){this.rc=
!1;a=a.flat(Number.POSITIVE_INFINITY);a:{var c=this.D;if(c.length!==a.length)b=!1;else{for(let d=0;d<c.length;d++){let e=c[d],f=a[d];b&&(e=b(e),f=b(f));if(f!==e){b=!1;break a}}b=!0}}if(this.G=!b)this.D=a,this.length=a.length,this.first=a[0]}rs(){this.rc=!0;let a;null==(a=this.N)||a.call(this)}destroy(){void 0!==this.F&&(this.F.complete(),this.F.unsubscribe())}};Symbol.iterator;var br=class{};br.wf=cr;function rp(a,b,c,d){a=gp(a.Jz,a.ut,b,{HB:c,qq:d});return new dn(a)}const dr=class extends br{constructor(a,b,c){super();this.Jz=a;this.ut=b;this.fc=c}get Zx(){let a;return(null==(a=this.ut.ub)?void 0:a.Zx)||null}xd(a,b){return rp(this,a,b)}};function cr(){return er(Vg(),Ug())}function er(a,b){return a.type&4?new dr(b,a,ri(a,b)):null};class fr{constructor(a){this.Wr=a;this.matches=null}clone(){return new fr(this.Wr)}rs(){this.Wr.rs()}}function Tm(a,b){for(let c=0;c<a.queries.length;c++)null!==b.queries.queries[c].matches&&a.queries[c].rs()}class gr{constructor(a=[]){this.queries=a}xd(a){const b=a.queries;if(null!==b){a=null!==a.ec?a.ec[0]:b.length;const c=[];for(let d=0;d<a;d++)c.push(this.queries[b.queries[d].hw].clone());return new gr(c)}return null}fj(a){Tm(this,a)}}
var ir=class{constructor(a,b){this.flags=b;this.read=null;"string"===typeof a?this.mf=hr(a):this.mf=a}};
class jr{constructor(a=[]){this.queries=a}bl(a,b){for(let c=0;c<this.queries.length;c++)this.queries[c].bl(a,b)}rn(a){for(let b=0;b<this.queries.length;b++)this.queries[b].rn(a)}Aq(a){let b=null;for(let c=0;c<this.length;c++){const d=this.queries[c].Aq(a,null!==b?b.length:0);d&&(d.hw=c,null!==b?b.push(d):b=[d])}return null!==b?new jr(b):null}ta(a,b){for(let c=0;c<this.queries.length;c++)this.queries[c].ta(a,b)}get length(){return this.queries.length}track(a){this.queries.push(a)}}
function kr(a,b){if(a.C&&1!==(a.metadata.flags&1)){a=a.D;for(b=b.parent;null!==b&&b.type&8&&b.index!==a;)b=b.parent;return a===(null!==b?b.index:-1)}return a.C}function lr(a,b,c){null===a.matches?a.matches=[b,c]:a.matches.push(b,c)}function mr(a,b,c,d){if(null!==d){const e=a.metadata.read;null!==e?e===si||e===kp||e===br&&c.type&4?lr(a,c.index,-2):(b=Dh(c,b,e,!1,!1),null!==b&&lr(a,c.index,b)):lr(a,c.index,d)}}
class nr{constructor(a,b=-1){this.metadata=a;this.matches=null;this.hw=-1;this.kv=!1;this.C=!0;this.D=b}bl(a,b){if(kr(this,b)){const d=this.metadata.mf;if(Array.isArray(d))for(let e=0;e<d.length;e++){const f=d[e];a:{var c=b.kr;if(null!==c)for(let g=0;g<c.length;g+=2)if(c[g]===f){c=c[g+1];break a}c=null}mr(this,a,b,c);mr(this,a,b,Dh(b,a,f,!1,!1))}else d===br?b.type&4&&mr(this,a,b,-1):mr(this,a,b,Dh(b,a,d,!1,!1))}}rn(a){this.D===a.index&&(this.C=!1)}ta(a,b){this.bl(a,b)}Aq(a,b){return kr(this,a)?(this.kv=
!0,lr(this,-a.index,b),new nr(this.metadata)):null}}function or(a,b,c,d){d=b[18].queries[d];if(null===d.matches){const p=a.data,t=c.matches,w=[];for(let B=0;null!==t&&B<t.length;B+=2){var e=t[B];if(0>e)w.push(null);else{a=w;var f=a.push,g=b,k=p[e],m=t[B+1];e=c.metadata.read;-1===m?(e=k,g=e.type&11?ri(e,g):e.type&4?er(e,g):null):-2===m?g=e===si?ri(k,g):e===br?er(k,g):e===kp?mp(k,g):void 0:g=Eh(g,g[1],m,k);f.call(a,g)}}d.matches=w}return d.matches}
function pr(a,b,c,d){var e=a.queries.queries[c];const f=e.matches;if(null!==f)for(a=or(a,b,e,c),c=0;c<f.length;c+=2){var g=f[c];if(0<g)d.push(a[c/2]);else{e=f[c+1];g=b[-g];for(var k=10;k<g.length;k++){var m=g[k];m[16]===m[3]&&pr(m[1],m,e,d)}if(null!==g[9])for(g=g[9],k=0;k<g.length;k++)m=g[k],pr(m[1],m,e,d)}}return d}
function qr(a,b){var c=Ug();b=new ar(4===(b&4));var d=b.destroy;const e=c[7]||(c[7]=[]);e.push(b);a.Bc&&(a.be||(a.be=[])).push(d,e.length-1);let f;(null!=(f=c[18])?f:c[18]=new gr).queries.push(new fr(b))}function hr(a){return a.split(",").map(b=>b.trim())}function rr(a,b,c){null===a.queries&&(a.queries=new jr);a.queries.track(new nr(b,c))};function tr(a,b,c){const d=Rg.ub;if(d.Bc){const e=Vg();rr(d,new ir(b,c),e.index);b=d.ec||(d.ec=[]);a!==(b.length?b[b.length-1]:-1)&&b.push(d.queries.length-1,a);2===(c&2)&&(d.ay=!0)}qr(d,c)}function ur(a,b){const c=Rg.ub;c.Bc&&(rr(c,new ir(a,b),-1),2===(b&2)&&(c.by=!0));qr(c,b)}
function vr(a){var b=Ug(),c=Rg.ub;const d=Rg.ce;Rg.ce=d+1;c=c.queries.queries[d];if(a.rc&&4===(b[2]&4)===(2===(c.metadata.flags&2))){if(null===c.matches)a.reset([]);else{c=b[1];const e=c.queries.queries[d];b=e.kv?pr(c,b,d,[]):or(c,b,e,d);a.reset(b,ti);$q(a)}return!0}return!1}function wr(){return Ug()[18].queries[Rg.ce].Wr};function xr(a){Ei("NgSignals");a=Me(a);const b=a[pe];a.set=c=>Te(b,c);a.update=c=>{Be()||Le();Te(b,c(b.value))};a.yF=yr.bind(a);return a}function yr(){const a=this[pe];if(void 0===a.sx){const b=()=>this();b[pe]=a;a.sx=b}return a.sx};function zr(a){return Rg.Bh[25+a]};function Z(a,b=""){const c=Ug(),d=Rg.ub;a+=25;const e=d.Bc?wm(d,a,1,b,null):d.data[a];gh=!0;b=c[11].lq(b);c[a]=b;gh&&sl(d,c,b,e);Wg(e,!1)};function Ar(a){Br("",a,"");return Ar}function Br(a,b,c){const d=Ug();a=wp(d,Rg.ed++,b)?a+ge(b)+c:xl;a!==xl&&Rm(d,a);return Br}function Cr(a,b,c,d,e){const f=Ug(),g=xp(f,Rg.ed,b,d);Yg(2);a=g?a+ge(b)+c+ge(d)+e:xl;a!==xl&&Rm(f,a);return Cr}function Dr(a,b,c,d,e,f,g,k,m){const p=Ug();var t=Rg.ed;const w=xp(p,t,b,d);t=xp(p,t+2,f,k)||w;Yg(4);a=t?a+ge(b)+c+ge(d)+e+ge(f)+g+ge(k)+m:xl;a!==xl&&Rm(p,a);return Dr};function Er(a,b,c,d,e){a=bf(a);if(Array.isArray(a))for(var f=0;f<a.length;f++)Er(a[f],b,c,d,e);else{f=Rg.ub;const w=Ug(),B=Vg();let G=eg(a)?a:bf(a.oa);var g=xg(a),k=B.Jg&1048575,m=B.nd,p=B.Jg>>20;if(eg(a)||!a.Eb)d=new mh(g,e),g=Fr(G,b,e?k:k+p,m),-1===g?(g=th(B,w),sh(g,f,G),Gr(f,a,b.length),b.push(G),B.nd++,B.we++,e&&(B.Jg+=1048576),c.push(d),w.push(d)):(c[g]=d,w[g]=d);else{m=Fr(G,b,k+p,m);k=Fr(G,b,k,k+p);var t=0<=m&&c[m];p=0<=k&&c[k];e&&!p||!e&&!t?(m=th(B,w),sh(m,f,G),m=c.length,t=new mh(e?Hr:Ir,
e),t.Eb=[],t.index=m,t.C=0,d&&!e&&t.C++,t.Eb.push(g),g=t,!e&&p&&(c[k].D=g),Gr(f,a,b.length,0),b.push(G),B.nd++,B.we++,e&&(B.Jg+=1048576),c.push(g),w.push(g)):(b=c[e?k:m],!e&&d&&b.C++,b=b.Eb.push(g)-1,Gr(f,a,-1<m?m:k,b));!e&&d&&p&&c[k].C++}}}function Gr(a,b,c,d){const e=eg(b);var f=!!b.Ec;if(e||f)if(f=(f?bf(b.Ec):b).prototype.va)a=a.Wk||(a.Wk=[]),!e&&b.Eb?(b=a.indexOf(c),-1===b?a.push(c,[d,f]):a[b+1].push(d,f)):a.push(c,f)}function Fr(a,b,c,d){for(;c<d;c++)if(b[c]===a)return c;return-1}
function Ir(){return Jr(this.Eb,[])}function Hr(a,b,c,d){b=this.Eb;if(this.D){const e=this.D.C;d=Eh(c,c[1],this.D.index,d);c=d.slice(0,e);Jr(b,c);for(b=e;b<d.length;b++)c.push(d[b])}else c=[],Jr(b,c);return c}function Jr(a,b){for(let c=0;c<a.length;c++)b.push((0,a[c])());return b};function Kr(a){var b=[];return c=>{c.Vr=(d,e)=>{e=e?e(a):a;const f=Rg.ub;f.Bc&&(d=!!d.ta,Er(b,f.data,f.qg,d,!0),Er(e,f.data,f.qg,d,!1))}}};class Lr{constructor(a){this.Ua=a;this.C=new Map}va(){try{for(const a of this.C.values())null!==a&&a.destroy()}finally{this.C.clear()}}}Lr.pa=Md({ma:Lr,ea:"environment",ba:()=>new Lr(r(kg))});function Mr(a){Ei("NgStandalone");a.Iq=b=>{b=b.get(Lr);if(a.ka){if(!b.C.has(a)){var c=Zf(a.type);c=0<c.length?rn([c],b.Ua,`Standalone[${a.type.name}]`):null;b.C.set(a,c)}b=b.C.get(a)}else b=null;return b}};function Nr(){var a=Or,b=Xg()+6;const c=Ug();c[b]===xl?(a=a(),b=c[b]=a):b=c[b];return b}function Pr(a,b){var c=Ug(),d=Xg()+2;wp(c,d,b)?(d+=1,a=a(b),c=c[d]=a):(c=c[d+1],c=c===xl?void 0:c);return c};function Qr(a,b){return er(a,b)};var Rr=class{};Rr.K=function(a){return new (a||Rr)(r(fp))};Rr.Ya=Of({type:Rr});Rr.Xa=Nd({});Object.assign({},Se,{py:void 0,Iu(a,b){Te(a,b)}});var Sr=class{};Sr.wf=Tr;function Tr(a){var b=Vg(),c=Ug();-1<b.Jf&&16!==(a&16)?(a=Jg(b.index,c),a=new dn(a,a)):a=b.type&47?new dn(c[15],c):null;return a};var Vr=class{supports(a){return up(a)}create(a){return new Ur(a)}};const Wr=(a,b)=>b;function Xr(a,b){let c=a.N;a=a.P;let d=0,e=null;for(;c||a;){const f=!a||c&&c.fd<Yr(a,d,e)?c:a,g=Yr(f,d,e),k=f.fd;if(f===a)d--,a=a.D;else if(c=c.Oa,null==f.Ie)d++;else{e||(e=[]);const m=g-d,p=k-d;if(m!=p){for(let t=0;t<m;t++){const w=t<e.length?e[t]:e[t]=0,B=w+t;p<=B&&B<m&&(e[t]=w+1)}e[f.Ie]=p-m}}g!==k&&b(f,g,k)}}function Zr(a,b){for(a=a.fa;null!==a;a=a.P)b(a)}
function $r(a,b){as(a);let c=a.N,d=!1,e,f,g;if(Array.isArray(b)){a.length=b.length;for(let k=0;k<a.length;k++)f=b[k],g=a.wa(k,f),null!==c&&Object.is(c.G,g)?(d&&(c=a.Sa(c,f,g,k)),Object.is(c.item,f)||a.ca(c,f)):(c=a.Ja(c,f,g,k),d=!0),c=c.Oa}else e=0,vp(b,k=>{g=a.wa(e,k);null!==c&&Object.is(c.G,g)?(d&&(c=a.Sa(c,k,g,e)),Object.is(c.item,k)||a.ca(c,k)):(c=a.Ja(c,k,g,e),d=!0);c=c.Oa;e++}),a.length=e;a.sb(c);a.collection=b;return a.Tn}
function as(a){if(a.Tn){let b;for(b=a.gb=a.N;null!==b;b=b.Oa)b.aa=b.Oa;for(b=a.aa;null!==b;b=b.O)b.Ie=b.fd;a.aa=a.G=null;for(b=a.ja;null!==b;b=b.T)b.Ie=b.fd;a.ja=a.O=null;a.P=a.D=null;a.fa=a.J=null}}
var Ur=class{constructor(a){this.length=0;this.J=this.fa=this.D=this.P=this.O=this.ja=this.G=this.aa=this.T=this.N=this.gb=this.C=this.F=null;this.wa=a||Wr}mn(a){null==a&&(a=[]);if(!up(a))throw new Zd(900,!1);return $r(this,a)?this:null}ob(){}get Tn(){return null!==this.aa||null!==this.ja||null!==this.P||null!==this.fa}Ja(a,b,c,d){let e;null===a?e=this.T:(e=a.F,this.lb(a));a=null===this.C?null:this.C.get(c,null);null!==a?(Object.is(a.item,b)||this.ca(a,b),this.Pa(a,e,d)):(a=null===this.F?null:this.F.get(c,
d),null!==a?(Object.is(a.item,b)||this.ca(a,b),this.bb(a,e,d)):a=this.Va(new bs(b,c),e,d));return a}Sa(a,b,c,d){b=null===this.C?null:this.C.get(c,null);null!==b?a=this.Pa(b,a.F,d):a.fd!=d&&(a.fd=d,this.na(a,d));return a}sb(a){for(;null!==a;){const b=a.Oa;this.Ea(this.ua(a));a=b}null!==this.C&&this.C.clear();null!==this.G&&(this.G.O=null);null!==this.O&&(this.O.T=null);null!==this.T&&(this.T.Oa=null);null!==this.D&&(this.D.D=null);null!==this.J&&(this.J.P=null)}Pa(a,b,c){null!==this.C&&this.C.remove(a);
const d=a.N,e=a.D;null===d?this.P=e:d.D=e;null===e?this.D=d:e.N=d;this.sa(a,b,c);this.na(a,c);return a}bb(a,b,c){this.ua(a);this.sa(a,b,c);this.na(a,c);return a}Va(a,b,c){this.sa(a,b,c);this.G=null===this.G?this.aa=a:this.G.O=a;return a}sa(a,b,c){const d=null===b?this.N:b.Oa;a.Oa=d;a.F=b;null===d?this.T=a:d.F=a;null===b?this.N=a:b.Oa=a;null===this.F&&(this.F=new cs);ds(this.F,a);a.fd=c;return a}lb(a){return this.Ea(this.ua(a))}ua(a){null!==this.F&&this.F.remove(a);const b=a.F,c=a.Oa;null===b?this.N=
c:b.Oa=c;null===c?this.T=b:c.F=b;return a}na(a,b){if(a.Ie===b)return a;this.O=null===this.O?this.ja=a:this.O.T=a;return a}Ea(a){null===this.C&&(this.C=new cs);ds(this.C,a);a.fd=null;a.D=null;null===this.D?(this.D=this.P=a,a.N=null):(a.N=this.D,this.D=this.D.D=a);return a}ca(a,b){a.item=b;this.J=null===this.J?this.fa=a:this.J.P=a;return a}},bs=class{constructor(a,b){this.item=a;this.G=b;this.P=this.T=this.O=this.D=this.N=this.C=this.J=this.Oa=this.F=this.aa=this.Ie=this.fd=null}};
class es{constructor(){this.D=this.C=null}add(a){null===this.C?(this.C=this.D=a,a.C=null,a.J=null):(this.D.C=a,a.J=this.D,a.C=null,this.D=a)}get(a,b){let c;for(c=this.C;null!==c;c=c.C)if((null===b||b<=c.fd)&&Object.is(c.G,a))return c;return null}remove(a){const b=a.J;a=a.C;null===b?this.C=a:b.C=a;null===a?this.D=b:a.J=b;return null===this.C}}function ds(a,b){const c=b.G;let d=a.map.get(c);d||(d=new es,a.map.set(c,d));d.add(b)}
class cs{constructor(){this.map=new Map}get(a,b){const c=this.map.get(a);return c?c.get(a,b):null}remove(a){const b=a.G;this.map.get(b).remove(a)&&this.map.delete(b);return a}clear(){this.map.clear()}}function Yr(a,b,c){a=a.Ie;if(null===a)return a;let d=0;c&&a<c.length&&(d=c[a]);return a+b+d};var fs=class{constructor(a){this.C=a}find(a){const b=this.C.find(c=>c.supports(a));if(null!=b)return b;throw new Zd(901,!1);}};fs.pa=Md({ma:fs,ea:"root",ba:function(){return new fs([new Vr])}});function gs(a,b){if(!a.C&&!a.G.C){var c=a.F.add();try{a.fa.run(()=>{a.C=!0;a.G.J(b)},void 0,a.ja)}catch(d){throw a.F.remove(c),d;}finally{a.be()}a.aa=!0;Ik(()=>{a.aa=!1;a.F.remove(c)})}}
var hs=class{constructor(){this.G=lf(fp);this.F=lf(xn);this.fa=lf(Qk);this.O=lf(bi);let a;this.P=null!=(a=lf(ci,{optional:!0}))?a:!1;this.ca="undefined"!==typeof Zone&&!!Zone.root.run;this.ja=[{data:{__scheduler_tick__:!0}}];this.T=new Ni;this.J=null;this.C=this.aa=this.N=!1;this.D=null;this.T.add(this.G.O.subscribe(()=>{this.C||this.be()}));this.T.add(this.fa.ko.subscribe(()=>{this.C||this.be()}));this.P||(this.P=!this.O&&!this.ca)}notify(a){if(this.O||5!==a){switch(a){case 3:case 2:case 0:case 4:case 5:case 1:this.N=
!0}if(!(this.P||null!==this.D||this.C||this.G.G||!this.O&&this.ca&&Kk())){var b=this.aa?Ik:Hk;this.D=this.F.add();this.ca?Zone.root.run(()=>{this.J=b(()=>{gs(this,this.N)})}):this.J=b(()=>{gs(this,this.N)})}}}va(){this.T.unsubscribe();this.be()}be(){this.C=this.N=!1;var a;null==(a=this.J)||a.call(this);this.J=null;null!==this.D&&(a=this.D,this.D=null,this.F.remove(a))}};hs.K=function(a){return new (a||hs)};hs.pa=Md({ma:hs,ba:hs.K,ea:"root"});function is(a){a.C||(a.C=a.zone.Ow.subscribe({next:()=>{let b;(null==(b=a.Xi)?0:b.C)||a.zone.run(()=>{a.D.J(!0)})}}))}var js=class{constructor(){this.zone=lf(Qk);this.Xi=lf(ai,{optional:!0});this.D=lf(fp);lf(bi)}va(){let a;null==(a=this.C)||a.unsubscribe()}};js.K=function(a){return new (a||js)};js.pa=Md({ma:js,ba:js.K,ea:"root"});new df("");
function ks(){var {cG:a,TF:b}={};null!=a||(a=()=>new Qk({IB:!1,Bo:!1,Xl:!1}));return[{oa:Qk,Fc:a},{oa:qf,Eb:!0,Fc:()=>{const c=lf(js,{optional:!0});return()=>is(c)}},{oa:qf,Eb:!0,Fc:()=>{const c=lf(ls);return()=>{ms(c)}}},{oa:xi,Fc:ns},!0===b?{oa:ci,nb:!0}:[],{oa:ai,Ab:hs}]}function ns(){const a=lf(Qk),b=lf(wi);return c=>Mk(a,()=>b.handleError(c))}
function ms(a){if(!a.F){a.F=!0;var b=null;a.zone.isStable||a.zone.In||a.zone.kl||(b=a.C.add());Mk(a.zone,()=>{a.D.add(a.zone.Jl.subscribe(()=>{Lk();queueMicrotask(()=>{null===b||a.zone.In||a.zone.kl||(a.C.remove(b),b=null)})}))});a.D.add(a.zone.ko.subscribe(()=>{if(!Kk())throw new Zd(909,!1);null!=b||(b=a.C.add())}))}}var ls=class{constructor(){this.D=new Ni;this.F=!1;this.zone=lf(Qk);this.C=lf(xn)}va(){this.D.unsubscribe()}};ls.K=function(a){return new (a||ls)};ls.pa=Md({ma:ls,ba:ls.K,ea:"root"});var os=new df("",{ea:"root",ba:()=>lf(os,12)||"undefined"!==typeof $localize&&$localize.locale||"en-US"});new df("",{ea:"root",ba:()=>"USD"});new df("");new df("");var ps=class{};ps.K=function(a){return new (a||ps)};ps.pa=Md({ma:ps,ba:ps.K,ea:"root"});new df("");var qs=new df(""),rs=class{constructor(a){this.Ua=a;this.F=[];this.D=[];this.Ga=!1}G(a){const b=a.xa.get(fp);if(0<a.fp.length)a.fp.forEach(c=>b.Kk(c));else if(a.instance.WC)a.instance.WC(b);else throw new Zd(-403,!1);this.F.push(a)}ob(a){this.D.push(a)}get xa(){return this.Ua}destroy(){if(this.Ga)throw new Zd(404,!1);this.F.slice().forEach(b=>b.destroy());this.D.forEach(b=>b());const a=this.Ua.get(qs,null);a&&(a.forEach(b=>b()),a.clear());this.Ga=!0}get C(){return this.Ga}};
rs.K=function(a){return new (a||rs)(r(Wh))};rs.pa=Md({ma:rs,ba:rs.K,ea:"platform"});let ss=null;var ts=new df("");function us(a,b,c=[]){const d=`Platform: ${b}`,e=new df(d);return(f=[])=>{let g=vs();if(!g||g.xa.get(ts,!1))if(f=[...c,...f,{oa:e,nb:!0}],a)a(f);else{f=ws(f,d);if(ss&&!ss.get(ts,!1))throw new Zd(400,!1);$o();ss=f;f.get(rs);xs(f)}f=vs();if(!f)throw new Zd(401,!1);return f}}function ws(a=[],b){return Vh({name:b,Wa:[{oa:fg,nb:"platform"},{oa:qs,nb:new Set([()=>ss=null])},...a]})}function vs(){let a,b;return null!=(b=null==(a=ss)?void 0:a.get(rs))?b:null}
function ys(a=[]){if(ss)return ss;ss=a=ws(a);$o();xs(a);return a}function xs(a){let b;null==(b=a.get(ki,null))||b.forEach(c=>c())};function zs(a){return"boolean"===typeof a?a:null!=a&&"false"!==a}function As(a,b=NaN){return isNaN(parseFloat(a))||isNaN(Number(a))?b:Number(a)};function Bs(a){Ei("NgSignals");return Ge(a)};var Ds=new df("",{ea:"root",ba:()=>lf(Cs)}),Cs=class{};Cs.pa=Md({ma:Cs,ea:"root",ba:()=>new Es});function Fs(a,b){var c=b.pB;a.C.has(c)||a.C.set(c,new Set);c=a.C.get(c);c.has(b)||(a.F++,c.add(b))}function Gs(a,b){Fs(a,b);if(null===a.D){const c=a.D=a.G.add();queueMicrotask(()=>{a.flush();a.G.remove(c);a.D=null})}}function Hs(a,b){for(const c of b)b.delete(c),a.F--,c.run()}
var Es=class{constructor(){this.F=0;this.C=new Map;this.G=lf(xn);this.D=null}flush(){for(;0<this.F;)for(const [a,b]of this.C){const c=a,d=b;null===c?Hs(this,d):c.run(()=>Hs(this,d))}}};
class Is{constructor(a,b,c,d,e,f){this.scheduler=a;this.D=b;this.pB=c;this.xa=e;this.C=Ue(g=>{try{this.D(g)}catch(k){let m;null==(m=this.xa.get(wi,null,{optional:!0}))||m.handleError(k)}},()=>this.Oc(),f);this.F=null==d?void 0:d.ob(()=>this.destroy())}run(){this.C.run()}Oc(){Gs(this.scheduler,this)}destroy(){this.C.destroy();let a;null==(a=this.F)||a.call(this)}}
function Js(a,b){Ei("NgSignals");(null==b?0:b.xa)||zg();var c,d=null!=(c=null==b?void 0:b.xa)?c:lf(Wh);c=!0!==(null==b?void 0:b.LC)?d.get(yi):null;let e;a=new Is(d.get(Ds),a,"undefined"===typeof Zone?null:Zone.current,c,d,null!=(e=null==b?void 0:b.Wp)?e:!1);if((d=d.get(Sr,null,{optional:!0}))&&d.D[2]&8){let f,g;(null!=(g=(f=d.D)[22])?g:f[22]=[]).push(a.C.notify)}else a.C.notify();return a};function Ks(a){const b=qe(null);try{return a()}finally{qe(b)}};var Ls=us(null,"core",[]);function Ms(a){const b=Ns[$d]||null,c=a.IF||jg();return(new hn(b)).create(c,a.uo,a.df,a.Of)};var Os=new df("GMAT_AUTOCOMPLETE_OPTIONS",{ea:"root",ba:()=>({sc:!1})});function Ps(a){return Array.isArray(a)?a:[a]};function Qs(a){return null!=a&&"false"!==`${a}`};function Rs(a){return null==a?"":"string"===typeof a?a:`${a}px`};function Ss(a){return a instanceof si?a.R:a};function Ts(a){return isNaN(parseFloat(a))||isNaN(Number(a))?0:Number(a)};const Us=/\s+/,Vs=[];
var Ws=class{constructor(a,b){this.F=a;this.G=b;this.N=Vs;this.C=new Map}set GC(a){this.N=null!=a?a.trim().split(Us):Vs}set VC(a){this.P="string"===typeof a?a.trim().split(Us):a}zj(){for(var a of this.N)this.D(a,!0);a=this.P;if(Array.isArray(a)||a instanceof Set)for(const b of a)this.D(b,!0);else if(null!=a)for(const b of Object.keys(a))this.D(b,!!a[b]);this.O()}D(a,b){const c=this.C.get(a);void 0!==c?(c.enabled!==b&&(c.Ah=!0,c.enabled=b),c.Vc=!0):this.C.set(a,{enabled:b,Ah:!0,Vc:!0})}O(){for(const a of this.C){const b=
a[0],c=a[1];c.Ah?(this.J(b,c.enabled),c.Ah=!1):c.Vc||(c.enabled&&this.J(b,!1),this.C.delete(b));c.Vc=!1}}J(a,b){a=a.trim();0<a.length&&a.split(Us).forEach(c=>{b?this.G.Ri(this.F.R,c):this.G.wo(this.F.R,c)})}};Ws.K=function(a){return new (a||Ws)(z(si),z(vi))};Ws.Fa=Qf({type:Ws,ha:[["","ngClass",""]],inputs:{GC:[0,"class","klass"],VC:"ngClass"},ka:!0});var Xs=class{constructor(a,b){this.da=a;this.rr=b;this.count=this.index=-1}get first(){return 0===this.index}};function Ys(a,b){const c=a.O;Xr(b,(d,e,f)=>{null==d.Ie?c.xd(a.G,new Xs(d.item,a.C),null===f?void 0:f):null==f?c.remove(null===e?void 0:e):null!==e&&(e=c.get(e),c.move(e,f),e.context.da=d.item)});for(let d=0,e=c.length;d<e;d++){const f=c.get(d).context;f.index=d;f.count=e;f.rr=a.C}Zr(b,d=>{c.get(d.fd).context.da=d.item})}
var Zs=class{set rr(a){this.C=a;this.F=!0}set sr(a){this.N=a}get sr(){return this.N}constructor(a,b,c){this.O=a;this.G=b;this.J=c;this.C=null;this.F=!0;this.D=null}set XC(a){a&&(this.G=a)}zj(){if(this.F){this.F=!1;var a=this.C;!this.D&&a&&(this.D=this.J.find(a).create(this.sr))}this.D&&(a=this.D.mn(this.C))&&Ys(this,a)}};Zs.K=function(a){return new (a||Zs)(z(kp),z(br),z(fs))};Zs.Fa=Qf({type:Zs,ha:[["","ngFor","","ngForOf",""]],inputs:{rr:"ngForOf",sr:"ngForTrackBy",XC:"ngForTemplate"},ka:!0});function $s(a){a.Db.da?a.F||(a.C.clear(),a.D=null,a.J&&(a.F=a.C.xd(a.J,a.Db))):a.D||(a.C.clear(),a.F=null,a.G&&(a.D=a.C.xd(a.G,a.Db)))}var ct=class{constructor(a,b){this.C=a;this.Db=new at;this.D=this.F=this.G=null;this.J=b}set tr(a){this.Db.da=this.Db.tr=a;$s(this)}set ZC(a){bt("ngIfThen",a);this.J=a;this.F=null;$s(this)}set YC(a){bt("ngIfElse",a);this.G=a;this.D=null;$s(this)}};ct.K=function(a){return new (a||ct)(z(kp),z(br))};
ct.Fa=Qf({type:ct,ha:[["","ngIf",""]],inputs:{tr:"ngIf",ZC:"ngIfThen",YC:"ngIfElse"},ka:!0});var at=class{constructor(){this.tr=this.da=null}};function bt(a,b){if(b&&!b.xd)throw Error(`${a} must be a TemplateRef, but received '${he(b)}'.`);};var dt=class{constructor(a,b){this.D=a;this.F=b;this.C=!1}create(){this.C=!0;this.D.xd(this.F)}destroy(){this.C=!1;this.D.clear()}},et=class{constructor(){this.F=[];this.J=!1;this.G=this.D=0;this.C=!1}set ur(a){this.T=a;0===this.D&&this.N(!0)}O(){return this.D++}aa(a){this.F.push(a)}P(a){a=a==this.T;this.C||(this.C=a);this.G++;this.G===this.D&&(this.N(!this.C),this.G=0,this.C=!1);return a}N(a){if(0<this.F.length&&a!==this.J){this.J=a;for(const d of this.F){var b=d,c=a;c&&!b.C?b.create():!c&&b.C&&
b.destroy()}}}};et.K=function(a){return new (a||et)};et.Fa=Qf({type:et,ha:[["","ngSwitch",""]],inputs:{ur:"ngSwitch"},ka:!0});var ft=class{constructor(a,b,c){this.ur=c;c.O();this.C=new dt(a,b)}zj(){var a=this.C,b=this.ur.P(this.aD);b&&!a.C?a.create():!b&&a.C&&a.destroy()}};ft.K=function(a){return new (a||ft)(z(kp),z(br),z(et,9))};ft.Fa=Qf({type:ft,ha:[["","ngSwitchCase",""]],inputs:{aD:"ngSwitchCase"},ka:!0});var gt=class{constructor(a){this.D=a;this.Nw=this.wr=this.Il=this.C=null}tc(a){if(this.G(a)){const c=this.D;this.C&&c.remove(c.indexOf(this.C));if(this.wr){a=this.F();var b;this.C=c.xd(this.wr,a,{xa:null!=(b=this.Nw)?b:void 0})}else this.C=null}}G(a){return!!a.ngTemplateOutlet||!!a.ngTemplateOutletInjector}F(){return new Proxy({},{set:(a,b,c)=>this.Il?Reflect.set(this.Il,b,c):!1,get:(a,b,c)=>{if(this.Il)return Reflect.get(this.Il,b,c)}})}};gt.K=function(a){return new (a||gt)(z(kp))};
gt.Fa=Qf({type:gt,ha:[["","ngTemplateOutlet",""]],inputs:{Il:"ngTemplateOutletContext",wr:"ngTemplateOutlet",Nw:"ngTemplateOutletInjector"},ka:!0,features:[Dg]});new df("");new df("");var ht=class{};ht.K=function(a){return new (a||ht)};ht.Ya=Of({type:ht});ht.Xa=Nd({});var it=a=>a.src;new df("",{ea:"root",ba:()=>it});var jt=new df("");new df("");new df("NG_OPTIMIZED_PRELOADED_IMAGES",{ea:"root",ba:()=>new Set});let kt=null;var lt=class{};var mt=class{F(){throw Error("");}};mt.K=function(a){return new (a||mt)};mt.pa=Md({ma:mt,ba:()=>lf(nt),ea:"platform"});new df("");function ot(a,b){const c=pt(a.J,"window");c.addEventListener("popstate",b,!1);return()=>c.removeEventListener("popstate",b)}function qt(a,b){const c=pt(a.J,"window");c.addEventListener("hashchange",b,!1);return()=>c.removeEventListener("hashchange",b)}
var nt=class extends mt{constructor(){super();this.J=lf(jt);this.C=window.location;this.D=window.history}get port(){return this.C.port}get G(){return this.C.pathname}get N(){return this.C.search}get hash(){return this.C.hash}set G(a){this.C.pathname=a}F(a=0){this.D.go(a)}getState(){return this.D.state}};nt.K=function(a){return new (a||nt)};nt.pa=Md({ma:nt,ba:()=>new nt,ea:"platform"});function rt(a,b){if(0==a.length)return b;if(0==b.length)return a;let c=0;a.endsWith("/")&&c++;b.startsWith("/")&&c++;return 2==c?a+b.substring(1):1==c?a+b:a+"/"+b}function st(a){var b=a.match(/#|\?|$/);b=b&&b.index||a.length;return a.slice(0,b-("/"===a[b-1]?1:0))+a.slice(b)}function tt(a){return a&&"?"!==a[0]?"?"+a:a};var ut=class{G(){throw Error("");}};ut.K=function(a){return new (a||ut)};ut.pa=Md({ma:ut,ba:()=>lf(vt),ea:"root"});var wt=new df("");function xt(a,b){a.F.push(ot(a.C,b),qt(a.C,b))}
var vt=class extends ut{constructor(a,b){super();this.C=a;this.F=[];let c,d,e;null==b&&(b=(yt=yt||document.querySelector("base"))?yt.getAttribute("href"):null,b=null==b?null:(new URL(b,document.baseURI)).pathname);this.D=null!=(e=null!=(d=b)?d:null==(c=lf(jt).location)?void 0:c.origin)?e:""}va(){for(;this.F.length;)this.F.pop()()}path(a=!1){const b=this.C.G+tt(this.C.N),c=this.C.hash;return c&&a?`${b}${c}`:b}getState(){return this.C.getState()}G(a=0){let b,c;null==(c=(b=this.C).F)||c.call(b,a)}};
vt.K=function(a){return new (a||vt)(r(mt),r(wt,8))};vt.pa=Md({ma:vt,ba:vt.K,ea:"root"});function zt(a,b){b&&"/"!==b[0]&&(b="/"+b);return rt(a.C.D,b)}function At(a,b,c=null){var d=a.C;d.C.D.pushState(c,"",rt(d.D,b+tt("")));a.D(zt(a,b+tt("")),c)}function Bt(a,b,c=null){var d=a.C;d.C.D.replaceState(c,"",rt(d.D,b+tt("")));a.D(zt(a,b+tt("")),c)}function Ct(a,b=0){let c,d;null==(d=(c=a.C).G)||d.call(c,b)}
var Et=class{constructor(a){this.wh=new Fk;this.F=[];this.J=null;this.C=a;this.G=Dt(st(this.C.D.replace(/\/index.html$/,"")));xt(this.C,b=>{this.wh.emit({url:this.path(!0),pop:!0,state:b.state,type:b.type})})}va(){let a;null==(a=this.J)||a.unsubscribe();this.F=[]}path(a=!1){return this.normalize(this.C.path(a))}getState(){return this.C.getState()}normalize(a){var b=this.G;a=a.replace(/\/index.html$/,"");b&&a.startsWith(b)?(b=a.substring(b.length),b=""===b||["/",";","?","#"].includes(b[0])?b:a):b=
a;return st(b)}D(a="",b){this.F.forEach(c=>c(a,b))}subscribe(a,b,c){return this.wh.subscribe({next:a,error:b,complete:c})}};Et.K=function(a){return new (a||Et)(r(ut))};Et.pa=Md({ma:Et,ba:()=>new Et(r(ut)),ea:"root"});function Dt(a){return RegExp("^(https?:)?//").test(a)?([,a]=a.split(/\/\/[^\/]+/),a):a};var Ft=class{};var Gt=class{constructor(a){this.D=[];this.ia=a}va(){this.detach()}add(a){this.remove(a);this.D.push(a)}remove(a){a=this.D.indexOf(a);-1<a&&this.D.splice(a,1);0===this.D.length&&this.detach()}};Gt.K=function(a){return new (a||Gt)(r(jt))};Gt.pa=Md({ma:Gt,ba:Gt.K,ea:"root"});var Ht=class extends Gt{constructor(a,b){super(a);this.qa=b;this.F=c=>{const d=this.D;for(let e=d.length-1;-1<e;e--)if(0<d[e].aa.mc.length){const f=d[e].aa;this.qa?this.qa.run(()=>f.next(c)):f.next(c);break}}}add(a){super.add(a);this.C||(this.qa?Mk(this.qa,()=>this.ia.body.addEventListener("keydown",this.F)):this.ia.body.addEventListener("keydown",this.F),this.C=!0)}detach(){this.C&&(this.ia.body.removeEventListener("keydown",this.F),this.C=!1)}};Ht.K=function(a){return new (a||Ht)(r(jt),r(Qk,8))};
Ht.pa=Md({ma:Ht,ba:Ht.K,ea:"root"});let It;function Jt(){if(null==It&&"undefined"!==typeof window)try{window.addEventListener("test",null,Object.defineProperty({},"passive",{get:()=>It=!0}))}finally{It=It||!1}return It}function Kt(a){return Jt()?a:!!a.capture};let Lt,Mt;function Nt(){if(null==Mt){if("object"!==typeof document||!document||"function"!==typeof Element||!Element)return Mt=!1;if("scrollBehavior"in document.documentElement.style)Mt=!0;else{const a=Element.prototype.scrollTo;Mt=a?!/\{\s*\[native code\]\s*\}/.test(a.toString()):!1}}return Mt}
function Ot(){if("object"!==typeof document||!document)return 0;if(null==Lt){const b=document.createElement("div");var a=b.style;b.dir="rtl";a.width="1px";a.overflow="auto";a.visibility="hidden";a.pointerEvents="none";a.position="absolute";a=document.createElement("div");const c=a.style;c.width="2px";c.height="1px";b.appendChild(a);document.body.appendChild(b);Lt=0;0===b.scrollLeft&&(b.scrollLeft=1,Lt=0===b.scrollLeft?1:2);b.remove()}return Lt};let Pt;function Qt(a){if(null==Pt){const b="undefined"!==typeof document?document.head:null;Pt=!(!b||!b.createShadowRoot&&!b.attachShadow)}return Pt&&(a=a.getRootNode?a.getRootNode():null,"undefined"!==typeof ShadowRoot&&ShadowRoot&&a instanceof ShadowRoot)?a:null}function Rt(){let a="undefined"!==typeof document&&document?document.activeElement:null;for(;a&&a.shadowRoot;){const b=a.shadowRoot.activeElement;if(b===a)break;else a=b}return a}
function St(a){return a.composedPath?a.composedPath()[0]:a.target};function Tt(){return"undefined"!==typeof __karma__&&!!__karma__||"undefined"!==typeof jasmine&&!!jasmine||"undefined"!==typeof jest&&!!jest||"undefined"!==typeof Mocha&&!!Mocha};let Ut;try{Ut="undefined"!==typeof Intl&&Intl.v8BreakIterator}catch(a){Ut=!1}
var Vt=class{constructor(a){this.J=(this.isBrowser=(this.O=a)?"browser"===this.O:"object"===typeof document&&!!document)&&/(edge)/i.test(navigator.userAgent);this.D=this.isBrowser&&/(msie|trident)/i.test(navigator.userAgent);this.G=this.isBrowser&&!(!window.chrome&&!Ut)&&"undefined"!==typeof CSS&&!this.J&&!this.D;this.F=this.isBrowser&&/AppleWebKit/i.test(navigator.userAgent)&&!this.G&&!this.J&&!this.D;this.C=this.isBrowser&&/iPad|iPhone|iPod/.test(navigator.userAgent)&&!("MSStream"in window);this.N=
this.isBrowser&&/(firefox|minefield)/i.test(navigator.userAgent);this.P=this.isBrowser&&/android/i.test(navigator.userAgent)&&!this.D;this.T=this.isBrowser&&/safari/i.test(navigator.userAgent)&&this.F}};Vt.K=function(a){return new (a||Vt)(r(li))};Vt.pa=Md({ma:Vt,ba:Vt.K,ea:"root"});var Wt=class extends Gt{constructor(a,b,c){super(a);this.Ma=b;this.qa=c;this.G=!1;this.O=d=>{this.J=St(d)};this.F=d=>{const e=St(d),f="click"===d.type&&this.J?this.J:e;this.J=null;const g=this.D.slice();for(let k=g.length-1;-1<k;k--){const m=g[k];if(1>m.ca.mc.length||!m.ic())continue;if(m.N.contains(e)||m.N.contains(f))break;const p=m.ca;this.qa?this.qa.run(()=>p.next(d)):p.next(d)}}}add(a){super.add(a);if(!this.C){const b=this.ia.body;this.qa?Mk(this.qa,()=>this.N(b)):this.N(b);this.Ma.C&&!this.G&&
(this.P=b.style.cursor,b.style.cursor="pointer",this.G=!0);this.C=!0}}detach(){if(this.C){const a=this.ia.body;a.removeEventListener("pointerdown",this.O,!0);a.removeEventListener("click",this.F,!0);a.removeEventListener("auxclick",this.F,!0);a.removeEventListener("contextmenu",this.F,!0);this.Ma.C&&this.G&&(a.style.cursor=this.P,this.G=!1);this.C=!1}}N(a){a.addEventListener("pointerdown",this.O,!0);a.addEventListener("click",this.F,!0);a.addEventListener("auxclick",this.F,!0);a.addEventListener("contextmenu",
this.F,!0)}};Wt.K=function(a){return new (a||Wt)(r(jt),r(Vt),r(Qk,8))};Wt.pa=Md({ma:Wt,ba:Wt.K,ea:"root"});function Xt(a){a.Di||a.C();return a.Di}
var Yt=class{constructor(a,b){this.Ma=b;this.ia=a}va(){let a;null==(a=this.Di)||a.remove()}C(){if(this.Ma.isBrowser||Tt()){var a=this.ia.querySelectorAll('.cdk-overlay-container[platform="server"], .cdk-overlay-container[platform="test"]');for(let b=0;b<a.length;b++)a[b].remove()}a=this.ia.createElement("div");a.classList.add("cdk-overlay-container");Tt()?a.setAttribute("platform","test"):this.Ma.isBrowser||a.setAttribute("platform","server");this.ia.body.appendChild(a);this.Di=a}};
Yt.K=function(a){return new (a||Yt)(r(jt),r(Vt))};Yt.pa=Md({ma:Yt,ba:Yt.K,ea:"root"});const Zt=Nt();
var bu=class{constructor(a,b){this.G=a;this.D={top:"",left:""};this.C=!1;this.ia=b}attach(){}enable(){if(this.J()){const a=this.ia.documentElement;this.F=$t(this.G);this.D.left=a.style.left||"";this.D.top=a.style.top||"";a.style.left=Rs(-this.F.left);a.style.top=Rs(-this.F.top);a.classList.add("cdk-global-scrollblock");this.C=!0}}disable(){if(this.C){const a=this.ia.documentElement,b=a.style,c=this.ia.body.style,d=b.C||"",e=c.C||"";this.C=!1;b.left=this.D.left;b.top=this.D.top;a.classList.remove("cdk-global-scrollblock");Zt&&
(b.C=c.C="auto");window.scroll(this.F.left,this.F.top);Zt&&(b.C=d,c.C=e)}}J(){if(this.ia.documentElement.classList.contains("cdk-global-scrollblock")||this.C)return!1;const a=this.ia.body,b=au(this.G);return a.scrollHeight>b.height||a.scrollWidth>b.width}};var du=class{constructor(a,b,c,d){this.G=a;this.qa=b;this.F=c;this.ya=d;this.C=null;this.D=()=>{this.disable();this.la.ic()&&this.qa.run(()=>this.la.detach())}}attach(a){this.la=a}enable(){if(!this.C){var a=C(cu(this.G,0),tk(b=>!b||!this.la.N.contains(b.fc.R)));this.ya&&this.ya.threshold&&1<this.ya.threshold?(this.J=$t(this.F).top,this.C=a.subscribe(()=>{const b=$t(this.F).top;Math.abs(b-this.J)>this.ya.threshold?this.D():this.la.Wc()})):this.C=a.subscribe(this.D)}}disable(){this.C&&(this.C.unsubscribe(),
this.C=null)}detach(){this.disable();this.la=null}};var eu=class{enable(){}disable(){}attach(){}};function fu(a,b){return b.some(c=>{const d=a.top>c.bottom,e=a.right<c.left,f=a.left>c.right;return a.bottom<c.top||d||e||f})}function gu(a,b){return b.some(c=>{const d=a.bottom>c.bottom,e=a.left<c.left,f=a.right>c.right;return a.top<c.top||d||e||f})};var hu=class{constructor(a,b,c,d){this.D=a;this.F=b;this.qa=c;this.ya=d;this.C=null}attach(a){this.la=a}enable(){this.C||(this.C=cu(this.D,this.ya?this.ya.Cx:0).subscribe(()=>{this.la.Wc();if(this.ya&&this.ya.autoClose){const a=this.la.N.getBoundingClientRect(),{width:b,height:c}=au(this.F);fu(a,[{width:b,height:c,bottom:c,right:b,top:0,left:0}])&&(this.disable(),this.qa.run(()=>this.la.detach()))}}))}disable(){this.C&&(this.C.unsubscribe(),this.C=null)}detach(){this.disable();this.la=null}};new df("VIRTUAL_SCROLL_STRATEGY");function cu(a,b=20){return a.Ma.isBrowser?new Xi(c=>{a.D||a.N();const d=0<b?C(a.F,Ln(b)).subscribe(c):a.F.subscribe(c);a.G++;return()=>{d.unsubscribe();a.G--;a.G||a.J()}}):jj()}function iu(a,b){const c=a.C.get(b);c&&(c.unsubscribe(),a.C.delete(b))}function ju(a,b){const c=[];a.C.forEach((d,e)=>{a.O(e,b)&&c.push(e)});return c}
var ku=class{constructor(a,b,c){this.qa=a;this.Ma=b;this.F=new cj;this.D=null;this.G=0;this.C=new Map;this.ia=c}register(a){this.C.has(a)||this.C.set(a,a.G.subscribe(()=>this.F.next(a)))}va(){this.J();this.C.forEach((a,b)=>iu(this,b));this.F.complete()}O(a,b){b=Ss(b);a=a.fc.R;do if(b==a)return!0;while(b=b.parentElement);return!1}N(){this.D=Mk(this.qa,()=>lk((this.ia.defaultView||window).document,"scroll").subscribe(()=>this.F.next()))}J(){this.D&&(this.D.unsubscribe(),this.D=null)}};
ku.K=function(a){return new (a||ku)(r(Qk),r(Vt),r(jt,8))};ku.pa=Md({ma:ku,ba:ku.K,ea:"root"});var mu=new df("cdk-dir-doc",{ea:"root",ba:lu});function lu(){return lf(jt)};const nu=/^(ar|ckb|dv|he|iw|fa|nqo|ps|sd|ug|ur|yi|.*[-_](Adlm|Arab|Hebr|Nkoo|Rohg|Thaa))(?!.*[-_](Latn|Cyrl)($|-|_))($|-|_)/i;var ou=class{constructor(a){this.value="ltr";this.wb=new Fk;if(a){var b=a.documentElement?a.documentElement.dir:null;b=(a.body?a.body.dir:null)||b||"ltr";b=(null==b?void 0:b.toLowerCase())||"";let c;this.value="auto"===b&&"undefined"!==typeof navigator&&(null==(c=navigator)?0:c.language)?nu.test(navigator.language)?"rtl":"ltr":"rtl"===b?"rtl":"ltr"}}va(){this.wb.complete()}};
ou.K=function(a){return new (a||ou)(r(mu,8))};ou.pa=Md({ma:ou,ba:ou.K,ea:"root"});var pu=class{};pu.K=function(a){return new (a||pu)};pu.Ya=Of({type:pu});pu.Xa=Nd({});var qu=class{constructor(a,b,c,d){this.fc=a;this.C=b;this.D=c;this.dir=d;this.Ga=new cj;this.G=new Xi(e=>Mk(this.D,()=>C(lk(this.fc.R,"scroll"),Lo(this.Ga)).subscribe(e)))}Fb(){this.C.register(this)}va(){iu(this.C,this);this.Ga.next();this.Ga.complete()}scrollTo(a){const b=this.fc.R,c=this.dir&&"rtl"==this.dir.value;null==a.left&&(a.left=c?a.end:a.start);null==a.right&&(a.right=c?a.start:a.end);null!=a.bottom&&(a.top=b.scrollHeight-b.clientHeight-a.bottom);c&&0!=Ot()?(null!=a.left&&(a.right=b.scrollWidth-
b.clientWidth-a.left),2==Ot()?a.left=a.right:1==Ot()&&(a.left=a.right?-a.right:a.right)):null!=a.right&&(a.left=b.scrollWidth-b.clientWidth-a.right);this.F(a)}F(a){const b=this.fc.R;Nt()?b.scrollTo(a):(null!=a.top&&(b.scrollTop=a.top),null!=a.left&&(b.scrollLeft=a.left))}};qu.K=function(a){return new (a||qu)(z(si),z(ku),z(Qk),z(ou,8))};qu.Fa=Qf({type:qu,ha:[["","cdk-scrollable",""],["","cdkScrollable",""]],ka:!0});var ru=class{};function su(a){return a&&"function"===typeof a.connect&&!(a instanceof uj)};new df("_ViewRepeater");function tu(a,...b){b.forEach(c=>a.T(c));b=a.N();a.Gi();return b}function uu(a,b){return a.C.has(a.J(b))}
var vu=class{get selected(){this.G||(this.G=Array.from(this.C.values()));return this.G}constructor(a=!1){this.O=a;this.ca=!0;this.Pk=void 0;this.C=new Set;this.D=[];this.F=[];this.Ah=new cj}select(...a){a.forEach(b=>this.aa(b));a=this.N();this.Gi();return a}toggle(a){return uu(this,a)?tu(this,a):this.select(a)}clear(a=!0){this.P();const b=this.N();a&&this.Gi();return b}sort(a){this.O&&this.selected&&this.G.sort(a)}Gi(){this.G=null;if(this.F.length||this.D.length)this.Ah.next({source:this,added:this.F,
removed:this.D}),this.D=[],this.F=[]}aa(a){a=this.J(a);uu(this,a)||(this.O||this.P(),uu(this,a)||this.C.add(a),this.F.push(a))}T(a){a=this.J(a);uu(this,a)&&(this.C.delete(a),this.D.push(a))}P(){0===this.C.size||this.C.forEach(a=>this.T(a))}fa(){}N(){return!(!this.D.length&&!this.F.length)}J(a,b){if(this.Pk){let c;b=null!=(c=b)?c:this.C;for(let d of b)if(this.Pk(a,d))return d}return a}};function $t(a){if(!a.Ma.isBrowser)return{top:0,left:0};const b=a.ia;a=a.ia.defaultView||window;const c=b.documentElement,d=c.getBoundingClientRect();return{top:-d.top||b.body.scrollTop||a.scrollY||c.scrollTop||0,left:-d.left||b.body.scrollLeft||a.scrollX||c.scrollLeft||0}}function au(a){a.C||a.G();const b={width:a.C.width,height:a.C.height};a.Ma.isBrowser||(a.C=null);return b}
var wu=class{constructor(a,b,c){this.Ma=a;this.D=new cj;this.F=d=>{this.D.next(d)};this.ia=c;Mk(b,()=>{if(a.isBrowser){const d=this.ia.defaultView||window;d.addEventListener("resize",this.F);d.addEventListener("orientationchange",this.F)}this.wb().subscribe(()=>this.C=null)})}va(){if(this.Ma.isBrowser){const a=this.ia.defaultView||window;a.removeEventListener("resize",this.F);a.removeEventListener("orientationchange",this.F)}this.D.complete()}wb(a=20){return 0<a?C(this.D,Ln(a)):this.D}G(){const a=
this.ia.defaultView||window;this.C=this.Ma.isBrowser?{width:a.innerWidth,height:a.innerHeight}:{width:0,height:0}}};wu.K=function(a){return new (a||wu)(r(Vt),r(Qk),r(jt,8))};wu.pa=Md({ma:wu,ba:wu.K,ea:"root"});new df("VIRTUAL_SCROLLABLE");var xu=class{};xu.K=function(a){return new (a||xu)};xu.Ya=Of({type:xu});xu.Xa=Nd({});var yu=class{};yu.K=function(a){return new (a||yu)};yu.Ya=Of({type:yu});yu.Xa=Nd({imports:[pu,xu,pu,xu]});var zu=class{constructor(a,b,c,d){this.F=a;this.D=b;this.qa=c;this.G=()=>new eu;this.close=e=>new du(this.F,this.qa,this.D,e);this.block=()=>new bu(this.D,this.ia);this.C=e=>new hu(this.F,this.D,this.qa,e);this.ia=d}};zu.K=function(a){return new (a||zu)(r(ku),r(wu),r(Qk),r(jt))};zu.pa=Md({ma:zu,ba:zu.K,ea:"root"});var Au=class{constructor(a){this.vc=new eu;this.hb="";this.Tb=!1;this.dc="cdk-overlay-dark-backdrop";this.Hh=!1;if(a){const b=Object.keys(a);for(const c of b)void 0!==a[c]&&(this[c]=a[c])}}};function Bu(a,...b){return b.length?b.some(c=>a[c]):a.altKey||a.shiftKey||a.ctrlKey||a.metaKey};function Cu(a){const b=a.G;b&&(a.fa?a.T(b):(b.classList.remove("cdk-overlay-backdrop-showing"),Mk(a.qa,()=>{b.addEventListener("transitionend",a.Sa)}),b.style.pointerEvents="none",a.ua=Mk(a.qa,()=>setTimeout(()=>{a.T(b)},500))))}function Du(a){return(a=a.ya.direction)?"string"===typeof a?a:a.value:"ltr"}
var Eu=class{constructor(a,b,c,d,e,f,g,k,m,p=!1,t){this.Ve=a;this.C=b;this.D=c;this.ya=d;this.qa=e;this.ja=f;this.ia=g;this.gb=k;this.na=m;this.fa=p;this.Ua=t;this.G=null;this.sa=new cj;this.P=new cj;this.J=new cj;this.wa=Ni.EMPTY;this.Pa=w=>this.sa.next(w);this.Sa=w=>{this.T(w.target)};this.aa=new cj;this.ca=new cj;this.Ja=new cj;d.vc&&(this.Kc=d.vc,this.Kc.attach(this));this.F=d.uc;this.lb=Ks(()=>Xk(()=>{this.Ja.next()},{xa:this.Ua}))}get N(){return this.D}get df(){return this.C}attach(a){!this.C.parentElement&&
this.Ea&&this.Ea.appendChild(this.C);a=this.Ve.attach(a);this.F&&this.F.attach(this);this.Ob();this.bb();this.Nb();this.Kc&&this.Kc.enable();al(()=>{this.ic()&&this.Wc()},{xa:this.Ua});this.Va(!0);this.ya.Tb&&this.sb();this.ya.hb&&this.O(this.D,this.ya.hb,!0);this.P.next();this.ja.add(this);this.ya.Hh&&(this.wa=this.gb.subscribe(()=>this.dispose()));this.na.add(this);"function"===typeof(null==a?void 0:a.ob)&&a.ob(()=>{this.ic()&&Mk(this.qa,()=>Promise.resolve().then(()=>this.detach()))});return a}detach(){if(this.ic()){Cu(this);
this.Va(!1);this.F&&this.F.detach&&this.F.detach();this.Kc&&this.Kc.disable();var a=this.Ve.detach();this.J.next();this.ja.remove(this);this.Kb();this.wa.unsubscribe();this.na.remove(this);return a}}dispose(){const a=this.ic();this.F&&this.F.dispose();this.Lb();this.T(this.G);this.wa.unsubscribe();this.ja.remove(this);this.Ve.dispose();this.P.complete();this.sa.complete();this.aa.complete();this.ca.complete();this.na.remove(this);let b;null==(b=this.C)||b.remove();this.Ea=this.D=this.C=null;a&&this.J.next();
this.J.complete();this.lb.destroy();this.Ja.complete()}ic(){return this.Ve.ic()}dd(){return this.sa}ef(){return this.aa}Ll(){return this.ca}Wc(){this.F&&this.F.apply()}ne(a){this.ya=Object.assign({},this.ya,a);this.bb()}og(a){this.D&&this.O(this.D,a,!0)}Jj(a){this.D&&this.O(this.D,a,!1)}Nb(){this.C.setAttribute("dir",Du(this))}bb(){if(this.D){var a=this.D.style;a.width=Rs(this.ya.width);a.height=Rs(this.ya.height);a.minWidth=Rs(this.ya.minWidth);a.minHeight=Rs(this.ya.minHeight);a.maxWidth=Rs(this.ya.maxWidth);
a.maxHeight=Rs(this.ya.maxHeight)}}Va(a){this.D.style.pointerEvents=a?"":"none"}sb(){this.G=this.ia.createElement("div");this.G.classList.add("cdk-overlay-backdrop");this.fa&&this.G.classList.add("cdk-overlay-backdrop-noop-animation");this.ya.dc&&this.O(this.G,this.ya.dc,!0);this.C.parentElement.insertBefore(this.G,this.C);this.G.addEventListener("click",this.Pa);this.fa||"undefined"===typeof requestAnimationFrame?this.G.classList.add("cdk-overlay-backdrop-showing"):Mk(this.qa,()=>{requestAnimationFrame(()=>
{this.G&&this.G.classList.add("cdk-overlay-backdrop-showing")})})}Ob(){this.C.nextSibling&&this.C.parentNode.appendChild(this.C)}O(a,b,c){b=Ps(b||[]).filter(d=>!!d);b.length&&(c?a.classList.add(...b):a.classList.remove(...b))}Kb(){Mk(this.qa,()=>{const a=C(this.Ja,Lo(rk(this.P,this.J))).subscribe(()=>{this.D&&this.C&&0!==this.D.children.length||(this.D&&this.ya.hb&&this.O(this.D,this.ya.hb,!1),this.C&&this.C.parentElement&&(this.Ea=this.C.parentElement,this.C.remove()),a.unsubscribe())})})}Lb(){const a=
this.Kc;a&&(a.disable(),a.detach&&a.detach())}T(a){a&&(a.removeEventListener("click",this.Pa),a.removeEventListener("transitionend",this.Sa),a.remove(),this.G===a&&(this.G=null));this.ua&&(clearTimeout(this.ua),this.ua=void 0)}};var Fu=class{constructor(a,b){this.C=a;this.D=b}};const Gu=/([A-Za-z%]+)$/;function Hu(a,b){a.sb=b;return a}function Iu(a){if(!a.Te&&a.Ma.isBrowser){var b=a.G;b?(a.Sa=a.Ja(),a.F=a.D.getBoundingClientRect(),a.ua=a.ac(),a.Ei=Xt(a.Kb).getBoundingClientRect(),a.aa(b,a.Dc(a.Sa,a.Ei,b))):a.apply()}}function Ju(a,b){a.Xc=b;return a}function Ku(a,b){a.bb=b;-1===b.indexOf(a.G)&&(a.G=null);return a}function Lu(a,b){a.C=b;return a}function Mu(a,b=!0){a.ja=b;return a}function Nu(a,b=!0){a.fa=b;return a}function Ev(a,b=!0){a.Nb=b;return a}
function Fv(a,b=!0){a.Va=b;return a}function Gv(a,b){a.lb=b;return a}function Hv(a,b){a.md=b;return a}function Iv(a){return"rtl"===Du(a.la)}
var Mv=class{get ie(){return this.bb}constructor(a,b,c,d,e){this.Ea=b;this.ia=c;this.Ma=d;this.Kb=e;this.P={width:0,height:0};this.O=!1;this.Nb=!0;this.fa=!1;this.ja=!0;this.Va=!1;this.C=0;this.Xc=[];this.bb=[];this.na=new cj;this.wa=Ni.EMPTY;this.lb=0;this.T=[];this.gb=this.na;Hu(this,a)}attach(a){a.df.classList.add("cdk-overlay-connected-position-bounding-box");this.la=a;this.J=a.df;this.D=a.N;this.Te=!1;this.N=!0;this.G=null;this.wa.unsubscribe();this.wa=this.Ea.wb().subscribe(()=>{this.N=!0;this.apply()})}apply(){if(!this.Te&&
this.Ma.isBrowser)if(!this.N&&this.Va&&this.G)Iu(this);else{this.Ob();this.Jc();this.Sd();this.ua=this.ac();this.Sa=this.Ja();this.F=this.D.getBoundingClientRect();this.Ei=Xt(this.Kb).getBoundingClientRect();var a=this.Sa,b=this.F,c=this.ua,d=this.Ei,e=[];for(let g of this.bb){let k=this.Dc(a,d,g),m=this.Pa(k,b,g),p=this.Qd(m,b,c,g);if(p.uC){this.O=!1;this.aa(g,k);return}if(this.td(p,m,c))e.push({position:g,origin:k,pD:b,Mu:this.Lb(k,g)});else if(!f||f.oD.Ey<p.Ey)var f={oD:p,fG:m,kD:k,position:g,
pD:b}}if(e.length){a=null;b=-1;for(const g of e)e=g.Mu.width*g.Mu.height*(g.position.weight||1),e>b&&(b=e,a=g);this.O=!1;this.aa(a.position,a.origin)}else this.Nb&&(this.O=!0),this.aa(f.position,f.kD)}}detach(){this.Ob();this.sa=this.G=null;this.wa.unsubscribe()}dispose(){this.Te||(this.J&&Jv(this.J.style,{top:"",left:"",right:"",bottom:"",height:"",width:"",alignItems:"",justifyContent:""}),this.D&&this.Jc(),this.la&&this.la.df.classList.remove("cdk-overlay-connected-position-bounding-box"),this.detach(),
this.na.complete(),this.la=this.J=null,this.Te=!0)}Dc(a,b,c){if("center"==c.cb)var d=a.left+a.width/2;else{d=Iv(this)?a.right:a.left;const e=Iv(this)?a.left:a.right;d="start"==c.cb?d:e}0>b.left&&(d-=b.left);a="center"==c.eb?a.top+a.height/2:"top"==c.eb?a.top:a.bottom;0>b.top&&(a-=b.top);return{x:d,y:a}}Pa(a,b,c){return{x:a.x+("center"==c.Na?-b.width/2:"start"===c.Na?Iv(this)?-b.width:0:Iv(this)?0:-b.width),y:a.y+("center"==c.Ta?-b.height/2:"top"==c.Ta?0:-b.height)}}Qd(a,b,c,d){b=Kv(b);let {x:e,y:f}=
a;a=this.ca(d,"x");d=this.ca(d,"y");a&&(e+=a);d&&(f+=d);d=-f;a=f+b.height-c.height;c=this.Yc(b.width,-e,e+b.width-c.width);d=this.Yc(b.height,d,a);a=c*d;return{Ey:a,uC:b.width*b.height===a,QB:d===b.height,PB:c==b.width}}td(a,b,c){if(this.ja){const d=c.bottom-b.y;c=c.right-b.x;b=Lv(this.la.ya.minHeight);const e=Lv(this.la.ya.minWidth);c=a.PB||null!=e&&e<=c;return(a.QB||null!=b&&b<=d)&&c}return!1}Ic(a,b,c){if(this.sa&&this.Va)return{x:a.x+this.sa.x,y:a.y+this.sa.y};b=Kv(b);const d=this.ua;var e=Math.max(a.x+
b.width-d.width,0);const f=Math.max(a.y+b.height-d.height,0),g=Math.max(d.top-c.top-a.y,0),k=Math.max(d.left-c.left-a.x,0);e=b.width<=d.width?k||-e:a.x<this.C?d.left-c.left-a.x:0;c=b.height<=d.height?g||-f:a.y<this.C?d.top-c.top-a.y:0;this.sa={x:e,y:c};return{x:a.x+e,y:a.y+c}}aa(a,b){this.Oe(a);this.Wd(b,a);this.Vd(b,a);a.hb&&this.sd(a.hb);if(this.na.mc.length){b=this.Rd();var c;if(c=a===this.G&&this.Hc)c=this.Hc,c=c===b?!0:c.sw===b.sw&&c.tw===b.tw&&c.gr===b.gr&&c.uw===b.uw;c||this.na.next(new Fu(a,
b));this.Hc=b}this.G=a;this.N=!1}Oe(a){if(this.md){var b=this.J.querySelectorAll(this.md),c=a.Ta;a="center"===a.Na?"center":Iv(this)?"start"===a.Na?"right":"left":"start"===a.Na?"left":"right";for(let d=0;d<b.length;d++)b[d].style.transformOrigin=`${a} ${c}`}}Lb(a,b){var c=this.ua;const d=Iv(this);let e;let f;if("top"===b.Ta){var g=a.y;e=c.height-g+this.C}else if("bottom"===b.Ta)f=c.height-a.y+2*this.C,e=c.height-f+this.C;else{g=Math.min(c.bottom-a.y+c.top,a.y);var k=this.P.height;e=2*g;g=a.y-g;e>
k&&!this.N&&!this.fa&&(g=a.y-k/2)}k="start"===b.Na&&!d||"end"===b.Na&&d;let m;if("end"===b.Na&&!d||"start"===b.Na&&d)m=c.width-a.x+2*this.C,b=a.x-this.C;else if(k){var p=a.x;b=c.right-a.x}else p=Math.min(c.right-a.x+c.left,a.x),c=this.P.width,b=2*p,p=a.x-p,b>c&&!this.N&&!this.fa&&(p=a.x-c/2);return{top:g,left:p,bottom:f,right:m,width:b,height:e}}Vd(a,b){a=this.Lb(a,b);this.N||this.fa||(a.height=Math.min(a.height,this.P.height),a.width=Math.min(a.width,this.P.width));const c={};if(this.Gc())c.top=
c.left="0",c.bottom=c.right=c.maxHeight=c.maxWidth="",c.width=c.height="100%";else{const d=this.la.ya.maxHeight,e=this.la.ya.maxWidth;c.height=Rs(a.height);c.top=Rs(a.top);c.bottom=Rs(a.bottom);c.width=Rs(a.width);c.left=Rs(a.left);c.right=Rs(a.right);c.alignItems="center"===b.Na?"center":"end"===b.Na?"flex-end":"flex-start";c.justifyContent="center"===b.Ta?"center":"bottom"===b.Ta?"flex-end":"flex-start";d&&(c.maxHeight=Rs(d));e&&(c.maxWidth=Rs(e))}this.P=a;Jv(this.J.style,c)}Sd(){Jv(this.J.style,
{top:"0",left:"0",right:"0",bottom:"0",height:"",width:"",alignItems:"",justifyContent:""})}Jc(){Jv(this.D.style,{top:"",left:"",bottom:"",right:"",position:"",transform:""})}Wd(a,b){const c={},d=this.Gc(),e=this.ja,f=this.la.ya;if(d){var g=$t(this.Ea);Jv(c,this.Pd(b,a,g));Jv(c,this.ud(b,a,g))}else c.position="static";a="";g=this.ca(b,"x");b=this.ca(b,"y");g&&(a+=`translateX(${g}px) `);b&&(a+=`translateY(${b}px)`);c.transform=a.trim();f.maxHeight&&(d?c.maxHeight=Rs(f.maxHeight):e&&(c.maxHeight=""));
f.maxWidth&&(d?c.maxWidth=Rs(f.maxWidth):e&&(c.maxWidth=""));Jv(this.D.style,c)}Pd(a,b,c){let d={top:"",bottom:""};b=this.Pa(b,this.F,a);this.O&&(b=this.Ic(b,this.F,c));"bottom"===a.Ta?d.bottom=`${this.ia.documentElement.clientHeight-(b.y+this.F.height)}px`:d.top=Rs(b.y);return d}ud(a,b,c){let d={left:"",right:""};b=this.Pa(b,this.F,a);this.O&&(b=this.Ic(b,this.F,c));"right"===(Iv(this)?"end"===a.Na?"left":"right":"end"===a.Na?"right":"left")?d.right=`${this.ia.documentElement.clientWidth-(b.x+this.F.width)}px`:
d.left=Rs(b.x);return d}Rd(){const a=this.Ja(),b=this.D.getBoundingClientRect(),c=this.Xc.map(d=>d.fc.R.getBoundingClientRect());return{sw:gu(a,c),tw:fu(a,c),gr:gu(b,c),uw:fu(b,c)}}Yc(a,...b){return b.reduce((c,d)=>c-Math.max(d,0),a)}ac(){const a=this.ia.documentElement.clientWidth,b=this.ia.documentElement.clientHeight,c=$t(this.Ea);return{top:c.top+this.C,left:c.left+this.C,right:c.left+a-this.C,bottom:c.top+b-this.C,width:a-2*this.C,height:b-2*this.C}}Gc(){return!this.ja||this.O}ca(a,b){return"x"===
b?null==a.offsetX?this.lb:a.offsetX:null==a.offsetY?0:a.offsetY}fg(){}sd(a){this.D&&Ps(a).forEach(b=>{""!==b&&-1===this.T.indexOf(b)&&(this.T.push(b),this.D.classList.add(b))})}Ob(){this.D&&(this.T.forEach(a=>{this.D.classList.remove(a)}),this.T=[])}Ja(){const a=this.sb;if(a instanceof si)return a.R.getBoundingClientRect();if(a instanceof Element)return a.getBoundingClientRect();const b=a.width||0,c=a.height||0;return{top:a.y,bottom:a.y+c,left:a.x,right:a.x+b,height:c,width:b}}};
function Jv(a,b){for(let c in b)b.hasOwnProperty(c)&&(a[c]=b[c])}function Lv(a){if("number"!==typeof a&&null!=a){const [b,c]=a.split(Gu);return c&&"px"!==c?null:parseFloat(b)}return a||null}function Kv(a){return{top:Math.floor(a.top),right:Math.floor(a.right),bottom:Math.floor(a.bottom),left:Math.floor(a.left),width:Math.floor(a.width),height:Math.floor(a.height)}};function Nv(a){a.left("");a.Ff="center";return a}function Ov(a){a.top("");a.pm="center";return a}
var Pv=class{constructor(){this.Iz="static";this.wp=this.Vp=this.Fk=this.Ff=this.pm=this.gp=this.Np="";this.Te=!1}attach(a){const b=a.ya;this.la=a;this.Vp&&!b.width&&a.ne({width:this.Vp});this.wp&&!b.height&&a.ne({height:this.wp});a.df.classList.add("cdk-global-overlay-wrapper");this.Te=!1}top(a=""){this.gp="";this.Np=a;this.pm="flex-start";return this}left(a=""){this.Fk=a;this.Ff="left";return this}bottom(a=""){this.Np="";this.gp=a;this.pm="flex-end";return this}right(a=""){this.Fk=a;this.Ff="right";
return this}start(a=""){this.Fk=a;this.Ff="start";return this}end(a=""){this.Fk=a;this.Ff="end";return this}width(a=""){this.la?this.la.ne({width:a}):this.Vp=a;return this}height(a=""){this.la?this.la.ne({height:a}):this.wp=a;return this}apply(){if(this.la&&this.la.ic()){var a=this.la.N.style,b=this.la.df.style,c=this.la.ya,d=c.width,e=c.height,f=c.maxWidth;c=c.maxHeight;d=("100%"===d||"100vw"===d)&&(!f||"100%"===f||"100vw"===f);e=("100%"===e||"100vh"===e)&&(!c||"100%"===c||"100vh"===c);f=this.Ff;
c=this.Fk;var g="rtl"===this.la.ya.direction,k="",m="",p="";if(d)p="flex-start";else if("center"===f)p="center",g?m=c:k=c;else if(g)if("left"===f||"end"===f)p="flex-end",k=c;else{if("right"===f||"start"===f)p="flex-start",m=c}else if("left"===f||"start"===f)p="flex-start",k=c;else if("right"===f||"end"===f)p="flex-end",m=c;a.position=this.Iz;a.marginLeft=d?"0":k;a.marginTop=e?"0":this.Np;a.marginBottom=this.gp;a.marginRight=d?"0":m;b.justifyContent=p;b.alignItems=e?"flex-start":this.pm}}dispose(){if(!this.Te&&
this.la){var a=this.la.N.style,b=this.la.df,c=b.style;b.classList.remove("cdk-global-overlay-wrapper");c.justifyContent=c.alignItems=a.marginTop=a.marginBottom=a.marginLeft=a.marginRight=a.position="";this.la=null;this.Te=!0}}};function Qv(){return new Pv}function Rv(a,b){return new Mv(b,a.D,a.ia,a.Ma,a.C)}var Sv=class{constructor(a,b,c,d){this.D=a;this.ia=b;this.Ma=c;this.C=d}};Sv.K=function(a){return new (a||Sv)(r(wu),r(jt),r(Vt),r(Yt))};Sv.pa=Md({ma:Sv,ba:Sv.K,ea:"root"});var Tv=class{attach(a){this.D=a;return a.attach(this)}detach(){let a=this.D;null!=a&&(this.D=null,a.detach())}},Uv=class extends Tv{constructor(a,b,c,d){super();this.component=a;this.xb=b;this.xa=c;this.C=d;this.uo=void 0}},Vv=class extends Tv{constructor(a,b,c,d){super();this.cg=a;this.xb=b;this.context=c;this.xa=d}get origin(){return this.cg.fc}attach(a,b=this.context){this.context=b;return super.attach(a)}detach(){this.context=void 0;return super.detach()}};function Wv(a,b){a.mp=b}
var Xv=class{constructor(){this.Te=!1}ic(){return!!this.Yd}attach(a){if(a instanceof Uv)return this.Yd=a,this.ae(a);if(a instanceof Vv)return this.Yd=a,this.af(a)}detach(){this.Yd&&(this.Yd=this.Yd.D=null);this.Wt()}dispose(){this.ic()&&this.detach();this.Wt();this.Te=!0}Wt(){this.mp&&(this.mp(),this.mp=null)}};var Yv=class extends Xv{constructor(a,b,c,d,e){super();this.D=a;this.F=b;this.C=c;this.G=d;this.ia=e}ae(a){const b=gn(a.C||this.F,a.component);let c;a.xb?(c=qp(a.xb,b,a.xb.length,a.xa||a.xb.xa,a.uo||void 0),Wv(this,()=>c.destroy())):(c=b.create(a.xa||this.G||Xh),dp(this.C,c.C),Wv(this,()=>{0<this.C.ua&&this.C.fj(c.C);c.destroy()}));this.D.appendChild(this.J(c));this.Yd=a;return c}af(a){let b=a.xb,c=b.xd(a.cg,a.context,{xa:a.xa});c.rootNodes.forEach(d=>this.D.appendChild(d));cn(c);Wv(this,()=>{let d=
b.indexOf(c);-1!==d&&b.remove(d)});this.Yd=a;return c}dispose(){super.dispose();this.D.remove()}J(a){return a.C.rootNodes[0]}};var Zv=class extends Xv{constructor(a,b,c){super();this.G=a;this.C=b;this.D=!1;this.Zp=new Fk;this.ia=c}get Ld(){return this.Yd}set Ld(a){if(!this.ic()||a||this.D)this.ic()&&super.detach(),a&&super.attach(a),this.Yd=a||null}Fb(){this.D=!0}va(){super.dispose();this.F=this.Yd=null}ae(a){a.D=this;const b=null!=a.xb?a.xb:this.C,c=gn(a.C||this.G,a.component),d=qp(b,c,b.length,a.xa||b.xa,a.uo||void 0);b!==this.C&&this.J().appendChild(d.C.rootNodes[0]);Wv(this,()=>d.destroy());this.Yd=a;this.F=d;this.Zp.emit(d);
return d}af(a){a.D=this;const b=this.C.xd(a.cg,a.context,{xa:a.xa});Wv(this,()=>this.C.clear());this.Yd=a;this.F=b;this.Zp.emit(b);return b}J(){const a=this.C.element.R;return a.nodeType===a.ELEMENT_NODE?a:a.parentNode}};Zv.K=function(a){return new (a||Zv)(z(Yh),z(kp),z(jt))};Zv.Fa=Qf({type:Zv,ha:[["","cdkPortalOutlet",""]],inputs:{Ld:[0,"cdkPortalOutlet","portal"]},outputs:{Zp:"attached"},ib:["cdkPortalOutlet"],ka:!0,features:[Cp]});var $v=class{};$v.K=function(a){return new (a||$v)};$v.Ya=Of({type:$v});
$v.Xa=Nd({});let aw=0;
var bw=class{constructor(a,b,c,d,e,f,g,k,m,p,t,w){this.C=a;this.P=b;this.G=c;this.na=d;this.J=e;this.Ua=f;this.qa=g;this.ia=k;this.ja=m;this.N=p;this.O=t;this.T=w}create(a){const b=this.aa(),c=this.ca(b),d=this.fa(c);a=new Au(a);a.direction=a.direction||this.ja.value;return new Eu(d,b,c,a,this.qa,this.J,this.ia,this.N,this.O,"NoopAnimations"===this.T,this.Ua.get(kg))}position(){return this.na}ca(a){const b=this.ia.createElement("div");b.id=`cdk-overlay-${aw++}`;b.classList.add("cdk-overlay-pane");a.appendChild(b);
return b}aa(){const a=this.ia.createElement("div");Xt(this.P).appendChild(a);return a}fa(a){this.F||(this.F=this.Ua.get(fp));return new Yv(a,this.G,this.F,this.Ua,this.ia)}};bw.K=function(a){return new (a||bw)(r(zu),r(Yt),r(Yh),r(Sv),r(Ht),r(Wh),r(Qk),r(jt),r(ou),r(Et),r(Wt),r(mi,8))};bw.pa=Md({ma:bw,ba:bw.K,ea:"root"});const cw=[{cb:"start",eb:"bottom",Na:"start",Ta:"top"},{cb:"start",eb:"top",Na:"start",Ta:"bottom"},{cb:"end",eb:"top",Na:"end",Ta:"bottom"},{cb:"end",eb:"bottom",Na:"end",Ta:"top"}];var dw=new df("cdk-connected-overlay-scroll-strategy",{ea:"root",ba:()=>{const a=lf(bw);return()=>a.C.C()}}),ew=class{constructor(a){this.fc=a}};ew.K=function(a){return new (a||ew)(z(si))};ew.Fa=Qf({type:ew,ha:[["","cdk-overlay-origin",""],["","overlay-origin",""],["","cdkOverlayOrigin",""]],ib:["cdkOverlayOrigin"],ka:!0});
function fw(a){a.la?a.la.ya.Tb=a.Tb:gw(a);a.la.ic()||a.la.attach(a.ua);a.Tb?a.F=a.la.dd().subscribe(b=>{a.dd.emit(b)}):a.F.unsubscribe();a.D.unsubscribe();0<a.Ej.mc.length&&(a.D=C(a.C.gb,Mo(()=>0<a.Ej.mc.length)).subscribe(b=>{a.qa.run(()=>a.Ej.emit(b));0===a.Ej.mc.length&&a.D.unsubscribe()}))}
function gw(a){a.ie&&a.ie.length||(a.ie=cw);const b=a.la=a.N.create(a.na());a.O=b.P.subscribe(()=>a.attach.emit());a.xm=b.J.subscribe(()=>a.detach.emit());b.ef().subscribe(c=>{a.Tw.next(c);27!==c.keyCode||a.yd||Bu(c)||(c.preventDefault(),a.P())});a.la.Ll().subscribe(c=>{a.Uw.next(c)})}
var hw=class{get offsetX(){return this.ca}set offsetX(a){this.ca=a;this.C&&this.G(this.C)}get offsetY(){return this.fa}set offsetY(a){this.fa=a;this.C&&this.G(this.C)}get Hh(){return this.T}set Hh(a){this.T=a}constructor(a,b,c,d,e){this.N=a;this.J=e;this.D=this.xm=this.O=this.F=Ni.EMPTY;this.T=!1;this.qa=lf(Qk);this.Dy=0;this.push=this.Zv=this.Rv=this.Cw=this.Tb=this.yd=this.open=!1;this.dd=new Fk;this.Ej=new Fk;this.attach=new Fk;this.detach=new Fk;this.Tw=new Fk;this.Uw=new Fk;this.ua=new Vv(b,
c);this.ja=d;this.vc=this.ja()}get Vb(){return this.la}get dir(){return this.J?this.J.value:"ltr"}va(){this.O.unsubscribe();this.xm.unsubscribe();this.F.unsubscribe();this.D.unsubscribe();this.la&&this.la.dispose()}tc(a){this.C&&(this.G(this.C),this.la.ne({width:this.width,minWidth:this.minWidth,height:this.height,minHeight:this.minHeight}),a.origin&&this.open&&this.C.apply());a.open&&(this.open?fw(this):this.P())}na(){var a=this.C=this.uc||this.sa();a=new Au({direction:this.J,uc:a,vc:this.vc,Tb:this.Tb,
Hh:this.Hh});if(this.width||0===this.width)a.width=this.width;if(this.height||0===this.height)a.height=this.height;if(this.minWidth||0===this.minWidth)a.minWidth=this.minWidth;if(this.minHeight||0===this.minHeight)a.minHeight=this.minHeight;this.dc&&(a.dc=this.dc);this.hb&&(a.hb=this.hb);return a}G(a){const b=this.ie.map(c=>({cb:c.cb,eb:c.eb,Na:c.Na,Ta:c.Ta,offsetX:c.offsetX||this.offsetX,offsetY:c.offsetY||this.offsetY,hb:c.hb||void 0}));return Hv(Fv(Lu(Nu(Ev(Mu(Ku(Hu(a,this.aa()),b),this.Rv),this.push),
this.Zv),this.Dy),this.Cw),this.HE)}sa(){const a=Rv(this.N.position(),this.aa());this.G(a);return a}aa(){return this.origin instanceof ew?this.origin.fc:this.origin}P(){this.la&&this.la.detach();this.F.unsubscribe();this.D.unsubscribe()}};hw.K=function(a){return new (a||hw)(z(bw),z(br),z(kp),z(dw),z(ou,8))};
hw.Fa=Qf({type:hw,ha:[["","cdk-connected-overlay",""],["","connected-overlay",""],["","cdkConnectedOverlay",""]],inputs:{origin:[0,"cdkConnectedOverlayOrigin","origin"],ie:[0,"cdkConnectedOverlayPositions","positions"],uc:[0,"cdkConnectedOverlayPositionStrategy","positionStrategy"],offsetX:[0,"cdkConnectedOverlayOffsetX","offsetX"],offsetY:[0,"cdkConnectedOverlayOffsetY","offsetY"],width:[0,"cdkConnectedOverlayWidth","width"],height:[0,"cdkConnectedOverlayHeight","height"],minWidth:[0,"cdkConnectedOverlayMinWidth",
"minWidth"],minHeight:[0,"cdkConnectedOverlayMinHeight","minHeight"],dc:[0,"cdkConnectedOverlayBackdropClass","backdropClass"],hb:[0,"cdkConnectedOverlayPanelClass","panelClass"],Dy:[0,"cdkConnectedOverlayViewportMargin","viewportMargin"],vc:[0,"cdkConnectedOverlayScrollStrategy","scrollStrategy"],open:[0,"cdkConnectedOverlayOpen","open"],yd:[0,"cdkConnectedOverlayDisableClose","disableClose"],HE:[0,"cdkConnectedOverlayTransformOriginOn","transformOriginSelector"],Tb:[2,"cdkConnectedOverlayHasBackdrop",
"hasBackdrop",zs],Cw:[2,"cdkConnectedOverlayLockPosition","lockPosition",zs],Rv:[2,"cdkConnectedOverlayFlexibleDimensions","flexibleDimensions",zs],Zv:[2,"cdkConnectedOverlayGrowAfterOpen","growAfterOpen",zs],push:[2,"cdkConnectedOverlayPush","push",zs],Hh:[2,"cdkConnectedOverlayDisposeOnNavigation","disposeOnNavigation",zs]},outputs:{dd:"backdropClick",Ej:"positionChange",attach:"attach",detach:"detach",Tw:"overlayKeydown",Uw:"overlayOutsideClick"},ib:["cdkConnectedOverlay"],ka:!0,features:[Jp,Dg]});
var iw={oa:dw,Ac:[bw],Fc:function(a){return()=>a.C.C()}};var jw=class{};jw.K=function(a){return new (a||jw)};jw.Ya=Of({type:jw});jw.Xa=Nd({Wa:[bw,iw],imports:[pu,$v,yu,yu]});function kw(a,b){a.D.includes(b)||a.D.push(b)}var lw=class extends bw{constructor(){super(...arguments);this.D=[]}create(a){return super.create(Object.assign({},a,{hb:(null==a?0:a.hb)?Ps(a.hb).concat(this.D):this.D}))}};lw.K=(()=>{let a;return function(b){return(a||(a=Mh(lw)))(b||lw)}})();lw.pa=Md({ma:lw,ba:lw.K});var mw=class{constructor(a,b){a.sc||kw(b,"gmat-mdc-autocomplete")}};mw.K=function(a){return new (a||mw)(z(Os),z(bw))};mw.Fa=Qf({type:mw,ha:[["input","matAutocomplete",""],["textarea","matAutocomplete",""]],features:[Kr([{oa:bw,Ec:lw}])]});var nw=class{constructor(a,b){this.F=a;this.ra=b;this.io=()=>{};this.Fr=()=>{}}setProperty(a,b){this.F.setProperty(this.ra.R,a,b)}Ij(a){this.Fr=a}Lg(a){this.io=a}gi(a){this.setProperty("disabled",a)}};nw.K=function(a){return new (a||nw)(z(vi),z(si))};nw.Fa=Qf({type:nw});var ow=class extends nw{};ow.K=(()=>{let a;return function(b){return(a||(a=Mh(ow)))(b||ow)}})();ow.Fa=Qf({type:ow,features:[Cp]});var pw=new df("");const rw={oa:pw,Ab:af(()=>qw),Eb:!0};var qw=class extends ow{pi(a){this.setProperty("checked",a)}};qw.K=(()=>{let a;return function(b){return(a||(a=Mh(qw)))(b||qw)}})();qw.Fa=Qf({type:qw,ha:[["input","type","checkbox","formControlName",""],["input","type","checkbox","formControl",""],["input","type","checkbox","ngModel",""]],Ia:function(a,b){a&1&&V("change",function(c){return b.io(c.target.checked)})("blur",function(){return b.Fr()})},features:[Kr([rw]),Cp]});var tw={oa:pw,Ab:af(()=>sw),Eb:!0},uw=new df(""),sw=class extends nw{constructor(a,b,c){super(a,b);this.C=c;this.D=!1;null==this.C&&(this.C=!/android (\d+)/.test((kt?window.navigator.userAgent:"").toLowerCase()))}pi(a){this.setProperty("value",null==a?"":a)}sp(a){(!this.C||this.C&&!this.D)&&this.io(a)}Hz(){this.D=!0}Gz(a){this.D=!1;this.C&&this.io(a)}};sw.K=function(a){return new (a||sw)(z(vi),z(si),z(uw,8))};
sw.Fa=Qf({type:sw,ha:[["input","formControlName","",3,"type","checkbox"],["textarea","formControlName",""],["input","formControl","",3,"type","checkbox"],["textarea","formControl",""],["input","ngModel","",3,"type","checkbox"],["textarea","ngModel",""],["","ngDefaultControl",""]],Ia:function(a,b){a&1&&V("input",function(c){return b.sp(c.target.value)})("blur",function(){return b.Fr()})("compositionstart",function(){return b.Hz()})("compositionend",function(c){return b.Gz(c.target.value)})},features:[Kr([tw]),
Cp]});var vw=new df(""),ww=new df("");function xw(a){a=a.value;return null==a||("string"===typeof a||Array.isArray(a))&&0===a.length?{required:!0}:null}function yw(a){return null!=a}function zw(a){return sn(a)?Sj(a):a}function Aw(a){let b={};a.forEach(c=>{b=null!=c?Object.assign({},b,c):b});return 0===Object.keys(b).length?null:b}function Bw(a,b){return b.map(c=>c(a))}function Cw(a){return a.map(b=>b.validate?c=>b.validate(c):b)}
function Dw(a){if(!a)return null;const b=a.filter(yw);return 0==b.length?null:function(c){return Aw(Bw(c,b))}}function Ew(a){return null!=a?Dw(Cw(a)):null}function Fw(a){if(!a)return null;const b=a.filter(yw);return 0==b.length?null:function(c){c=Bw(c,b).map(zw);return C(jk(c),xj(Aw))}}function Gw(a){return null!=a?Fw(Cw(a)):null}function Hw(a,b){return null===a?[b]:Array.isArray(a)?[...a,b]:[a,b]};var Iw=class{constructor(){this.We=[];this.Df=[];this.F=[]}get value(){return this.control?this.control.value:null}get valid(){return this.control?this.control.valid:null}get Qn(){return this.control?this.control.Qn:null}get pending(){return this.control?this.control.pending:null}get disabled(){return this.control?this.control.disabled:null}get enabled(){return this.control?this.control.enabled:null}get errors(){return this.control?this.control.errors:null}get je(){return this.control?this.control.je:
null}get rc(){return this.control?this.control.rc:null}get Vc(){return this.control?this.control.Vc:null}get status(){return this.control?this.control.status:null}get Ks(){return this.control?this.control.Ks:null}get bm(){return this.control?this.control.bm:null}get ah(){return this.control?this.control.ah:null}get path(){return null}mu(a){this.We=a||[];this.vm=Ew(this.We)}lu(a){this.Df=a||[];this.um=Gw(this.Df)}get Zg(){return this.vm||null}get Ze(){return this.um||null}G(a){this.F.push(a)}J(){this.F.forEach(a=>
a());this.F=[]}reset(a){this.control&&this.control.reset(a)}ze(a,b){return this.control?this.control.ze(a,b):!1}getError(a,b){return this.control?this.control.getError(a,b):null}};var Jw=class extends Iw{constructor(){super(...arguments);this.C=this.name=this.Qa=null}};var Kw=class extends Iw{get path(){return null}};var Lw=class{constructor(a){this.C=a}get DC(){let a,b;return!!(null==(a=this.C)?0:null==(b=a.control)?0:b.Vc)}get EC(){let a,b;return!!(null==(a=this.C)?0:null==(b=a.control)?0:b.Ks)}get BC(){let a,b;return!!(null==(a=this.C)?0:null==(b=a.control)?0:b.je)}get Tn(){let a,b;return!!(null==(a=this.C)?0:null==(b=a.control)?0:b.rc)}get ir(){let a,b;return!!(null==(a=this.C)?0:null==(b=a.control)?0:b.valid)}get yC(){let a,b;return!!(null==(a=this.C)?0:null==(b=a.control)?0:b.Qn)}get AC(){let a,b;return!!(null==
(a=this.C)?0:null==(b=a.control)?0:b.pending)}};Object.assign({},{"[class.ng-untouched]":"isUntouched","[class.ng-touched]":"isTouched","[class.ng-pristine]":"isPristine","[class.ng-dirty]":"isDirty","[class.ng-valid]":"isValid","[class.ng-invalid]":"isInvalid","[class.ng-pending]":"isPending"},{"[class.ng-submitted]":"isSubmitted"});var Mw=class extends Lw{};Mw.K=function(a){return new (a||Mw)(z(Jw,2))};
Mw.Fa=Qf({type:Mw,ha:[["","formControlName",""],["","ngModel",""],["","formControl",""]],Za:14,Ia:function(a,b){a&2&&nq("ng-untouched",b.EC)("ng-touched",b.DC)("ng-pristine",b.BC)("ng-dirty",b.Tn)("ng-valid",b.ir)("ng-invalid",b.yC)("ng-pending",b.AC)},features:[Cp]});var Nw=new df("CallSetDisabledState",{ea:"root",ba:()=>"whenDisabledForLegacyCode"});function Ow(a,b,c="whenDisabledForLegacyCode"){Pw(a,b);b.C.pi(a.value);if(a.disabled||"always"===c){let d,e;null==(e=(d=b.C).gi)||e.call(d,a.disabled)}Qw(a,b);Rw(a,b);Sw(a,b);Tw(a,b)}function Uw(a,b){const c=()=>{};b.C&&(b.C.Lg(c),b.C.Ij(c));Vw(a,b);a&&(b.J(),a.Ak(()=>{}))}function Ww(a,b){a.forEach(c=>{c.Xr&&c.Xr(b)})}function Tw(a,b){if(b.C.gi){const c=d=>{b.C.gi(d)};a.Fm.push(c);b.G(()=>{a.QA(c)})}}
function Pw(a,b){var c=a.We;null!==b.Zg?Xw(a,Hw(c,b.Zg)):"function"===typeof c&&Xw(a,[c]);c=a.Df;null!==b.Ze?Yw(a,Hw(c,b.Ze)):"function"===typeof c&&Yw(a,[c]);c=()=>Zw(a);Ww(b.We,c);Ww(b.Df,c)}
function Vw(a,b){let c=!1;if(null!==a){if(null!==b.Zg){var d=a.We;if(Array.isArray(d)&&0<d.length){var e=d.filter(f=>f!==b.Zg);e.length!==d.length&&(c=!0,a.ep(e))}}null!==b.Ze&&(d=a.Df,Array.isArray(d)&&0<d.length&&(e=d.filter(f=>f!==b.Ze),e.length!==d.length&&(c=!0,a.cp(e))))}a=()=>{};Ww(b.We,a);Ww(b.Df,a);return c}function Qw(a,b){b.C.Lg(c=>{a.th=c;a.rh=!0;a.Jm=!0;"change"===a.tf&&$w(a,b)})}function Sw(a,b){b.C.Ij(()=>{a.Gp=!0;"blur"===a.tf&&a.rh&&$w(a,b);"submit"!==a.tf&&ax(a)})}
function $w(a,b){a.Jm&&bx(a);a.setValue(a.th,{Lv:!1});var c=a.th;b.D=c;b.update.emit(c);a.rh=!1}function Rw(a,b){const c=(d,e)=>{b.C.pi(d);e&&(b.D=d,b.update.emit(d))};a.Lg(c);b.G(()=>{a.PA(c)})}function cx(a,b){a.Mp();b.forEach(c=>{const d=c.control;if("submit"===d.tf&&d.rh){var e=d.th;c.D=e;c.update.emit(e);d.rh=!1}})}function dx(a){if(!a)return null;let b=void 0,c=void 0,d=void 0;a.forEach(e=>{e.constructor===sw?b=e:Object.getPrototypeOf(e.constructor)===ow?c=e:d=e});return d?d:c?c:b?b:null};var ex=class{},fx=class extends ex{constructor(a,b){super();this.value=a;this.source=b}},gx=class extends ex{constructor(a,b){super();this.je=a;this.source=b}},hx=class extends ex{constructor(a,b){super();this.Vc=a;this.source=b}},ix=class extends ex{constructor(a,b){super();this.status=a;this.source=b}},jx=class extends ex{constructor(a){super();this.source=a}},kx=class extends ex{constructor(a){super();this.source=a}};function lx(a){return null!=a&&!Array.isArray(a)&&"object"===typeof a}
function mx(a,b){a.Re((c,d)=>{if(void 0===b[d])throw new Zd(1002,"");})}function Xw(a,b){a.ep(b)}function Yw(a,b){a.cp(b)}function Zw(a,b={}){a.FA();a.Sp();a.enabled&&(a.Bz(),a.errors=a.DA(),a.status=a.pt(),"VALID"!==a.status&&"PENDING"!==a.status||a.CA(b.zb));let c;const d=null!=(c=b.Yf)?c:a;!1!==b.zb&&(a.Bb.next(new fx(a.value,d)),a.Bb.next(new ix(a.status,d)),a.ah.emit(a.value),a.bm.emit(a.status));a.Qa&&!b.Ub&&Zw(a.Qa,Object.assign({},b,{Yf:d}))}
function ax(a,b={}){const c=!1===a.Vc;a.Vc=!0;let d;const e=null!=(d=b.Yf)?d:a;a.Qa&&!b.Ub&&ax(a.Qa,Object.assign({},b,{Yf:e}));c&&!1!==b.zb&&a.Bb.next(new hx(!0,e))}function bx(a,b={}){const c=!0===a.je;a.je=!1;let d;const e=null!=(d=b.Yf)?d:a;a.Qa&&!b.Ub&&bx(a.Qa,Object.assign({},b,{Yf:e}));c&&!1!==b.zb&&a.Bb.next(new gx(!1,e))}
function nx(a,b={}){const c=!0===a.Vc;a.Vc=!1;a.Gp=!1;let d;const e=null!=(d=b.Yf)?d:a;a.Re(f=>{nx(f,{Ub:!0,zb:b.zb,Yf:e})});a.Qa&&!b.Ub&&a.Qa.Rm(b,e);c&&!1!==b.zb&&a.Bb.next(new hx(!1,e))}function ox(a,b={}){const c=!1===a.je;a.je=!0;a.Jm=!1;let d;const e=null!=(d=b.Yf)?d:a;a.Re(f=>{ox(f,{Ub:!0,zb:b.zb})});a.Qa&&!b.Ub&&a.Qa.Qm(b,e);c&&!1!==b.zb&&a.Bb.next(new gx(!0,e))}function px(a,b,c={}){a.errors=b;a.uu(!1!==c.zb,a)}
var qx=class{constructor(a,b){this.Gp=this.zm=this.Jm=!1;this.Mi=()=>{};this.Qa=null;this.je=!0;this.Vc=!1;this.Bb=new cj;this.hc=aj(this.Bb);this.Fm=[];this.ep(a);this.cp(b)}get Zg(){return this.vm}set Zg(a){this.We=this.vm=a}get Ze(){return this.um}set Ze(a){this.Df=this.um=a}get parent(){return this.Qa}get valid(){return"VALID"===this.status}get Qn(){return"INVALID"===this.status}get pending(){return"PENDING"==this.status}get disabled(){return"DISABLED"===this.status}get enabled(){return"DISABLED"!==
this.status}get rc(){return!this.je}get Ks(){return!this.Vc}get tf(){return this.Rp?this.Rp:this.parent?this.parent.tf:"change"}disable(a={}){const b=this.gu(a.Ub);this.status="DISABLED";this.errors=null;this.Re(e=>{e.disable(Object.assign({},a,{Ub:!0}))});this.Sp();let c;const d=null!=(c=a.Yf)?c:this;!1!==a.zb&&(this.Bb.next(new fx(this.value,d)),this.Bb.next(new ix(this.status,d)),this.ah.emit(this.value),this.bm.emit(this.status));this.tu(Object.assign({},a,{Rx:b}),this);this.Fm.forEach(e=>e(!0))}enable(a=
{}){const b=this.gu(a.Ub);this.status="VALID";this.Re(c=>{c.enable(Object.assign({},a,{Ub:!0}))});Zw(this,{Ub:!0,zb:a.zb});this.tu(Object.assign({},a,{Rx:b}),this);this.Fm.forEach(c=>c(!1))}tu(a,b){this.Qa&&!a.Ub&&(Zw(this.Qa,a),a.Rx||this.Qa.Qm({},b),this.Qa.Rm({},b))}wu(a={zb:!0}){this.Re(b=>b.wu(a));Zw(this,{Ub:!0,zb:a.zb})}FA(){this.status=this.ap()?"DISABLED":"VALID"}DA(){return this.Zg?this.Zg(this):null}CA(a){this.Ze&&(this.status="PENDING",this.zm=!0,this.ot=zw(this.Ze(this)).subscribe(b=>
{this.zm=!1;px(this,b,{zb:a})}))}Bz(){this.ot&&(this.ot.unsubscribe(),this.zm=!1)}get(a){if(null==a)return null;Array.isArray(a)||(a=a.split("."));return 0===a.length?null:a.reduce((b,c)=>b&&b.zt(c),this)}getError(a,b){return(b=b?this.get(b):this)&&b.errors?b.errors[a]:null}ze(a,b){return!!this.getError(a,b)}get root(){let a=this;for(;a.Qa;)a=a.Qa;return a}uu(a,b){this.status=this.pt();a&&(this.bm.emit(this.status),this.Bb.next(new ix(this.status,b)));this.Qa&&this.Qa.uu(a,b)}St(){this.ah=new Fk;
this.bm=new Fk}pt(){return this.ap()?"DISABLED":this.errors?"INVALID":this.zm||this.nt("PENDING")?"PENDING":this.nt("INVALID")?"INVALID":"VALID"}nt(a){return this.qm(b=>b.status===a)}mt(){return this.qm(a=>a.rc)}Az(){return this.qm(a=>a.Vc)}Qm(a,b){const c=!this.mt(),d=this.je!==c;this.je=c;this.Qa&&!a.Ub&&this.Qa.Qm(a,b);d&&this.Bb.next(new gx(this.je,b))}Rm(a={},b){this.Vc=this.Az();this.Bb.next(new hx(this.Vc,b));this.Qa&&!a.Ub&&this.Qa.Rm(a,b)}Ak(a){this.Mi=a}Mm(a){lx(a)&&null!=a.tf&&(this.Rp=
a.tf)}gu(a){const b=this.Qa&&this.Qa.rc;return!a&&!!b&&!this.Qa.mt()}zt(){return null}ep(a){a=this.We=Array.isArray(a)?a.slice():a;this.vm=Array.isArray(a)?Ew(a):a||null}cp(a){a=this.Df=Array.isArray(a)?a.slice():a;this.um=Array.isArray(a)?Gw(a):a||null}};var rx=class extends qx{constructor(a,b){super((lx(a)?a.PE:a)||null,(lx(a)?a.YA:b)||null);this.controls={};this.St();this.Mm(a);this.HA();Zw(this,{Ub:!0,zb:!!this.Ze})}contains(a){return this.controls.hasOwnProperty(a)&&this.controls[a].enabled}setValue(a,b={}){mx(this,a);Object.keys(a).forEach(c=>{const d=this.controls;if(!Object.keys(d).length)throw new Zd(1E3,"");if(!d[c])throw new Zd(1001,"");this.controls[c].setValue(a[c],{Ub:!0,zb:b.zb})});Zw(this,b)}reset(a={},b={}){this.Re((c,d)=>{c.reset(a?
a[d]:null,{Ub:!0,zb:b.zb})});this.Qm(b,this);this.Rm(b,this);Zw(this,b)}Mp(){let a=this.hu(!1,(b,c)=>c.Mp()?!0:b);a&&Zw(this,{Ub:!0});return a}Re(a){Object.keys(this.controls).forEach(b=>{const c=this.controls[b];c&&a(c,b)})}HA(){this.Re(a=>{a.Qa=this;a.Ak(this.Mi)})}Sp(){this.value=this.AA()}qm(a){for(const [b,c]of Object.entries(this.controls)){const d=c;if(this.contains(b)&&a(d))return!0}return!1}AA(){return this.hu({},(a,b,c)=>{if(b.enabled||this.disabled)a[c]=b.value;return a})}hu(a,b){let c=
a;this.Re((d,e)=>{c=b(c,d,e)});return c}ap(){for(const a of Object.keys(this.controls))if(this.controls[a].enabled)return!1;return 0<Object.keys(this.controls).length||this.disabled}zt(a){return this.controls.hasOwnProperty(a)?this.controls[a]:null}};const tx={oa:Kw,Ab:af(()=>sx)};Promise.resolve();
var sx=class extends Kw{constructor(a,b,c){super();this.Ym=c;this.Uj=!1;this.Kz=new Set;this.Hl=new Fk;this.form=new rx(Ew(a),Gw(b))}pd(){this.Mm()}get control(){return this.form}get path(){return[]}get controls(){return this.form.controls}setValue(a){this.control.setValue(a)}Er(a){this.Uj=!0;cx(this.form,this.Kz);this.Hl.emit(a);let b;return"dialog"===(null==a?void 0:null==(b=a.target)?void 0:b.method)}Cr(){this.es()}es(){this.form.reset(void 0);this.Uj=!1}Mm(){this.options&&null!=this.options.tf&&
(this.form.Rp=this.options.tf)}qF(a){a.pop();return a.length?this.form.get(a):this.form}};sx.K=function(a){return new (a||sx)(z(vw,10),z(ww,10),z(Nw,8))};sx.Fa=Qf({type:sx,ha:[["form",3,"ngNoForm","",3,"formGroup",""],["ng-form"],["","ngForm",""]],Ia:function(a,b){a&1&&V("submit",function(c){return b.Er(c)})("reset",function(){return b.Cr()})},inputs:{options:[0,"ngFormOptions","options"]},outputs:{Hl:"ngSubmit"},ib:["ngForm"],features:[Kr([tx]),Cp]});function ux(a){return"object"===typeof a&&null!==a&&2===Object.keys(a).length&&"value"in a&&"disabled"in a}
var vx=class extends qx{constructor(a=null){super((lx()?(void 0).PE:void 0)||null,(lx()?(void 0).YA:void 0)||null);this.defaultValue=null;this.C=[];this.rh=!1;this.D(a);this.Mm(void 0);this.St();Zw(this,{Ub:!0,zb:!!this.Ze});lx()&&((void 0).dG||(void 0).UF)&&(ux(a)?this.defaultValue=a.value:this.defaultValue=a)}setValue(a,b={}){this.value=this.th=a;this.C.length&&!1!==b.Lv&&this.C.forEach(c=>c(this.value,!1!==b.JF));Zw(this,b)}reset(a=this.defaultValue,b={}){this.D(a);ox(this,b);nx(this,b);this.setValue(this.value,
b);this.rh=!1}Sp(){}qm(){return!1}ap(){return this.disabled}Lg(a){this.C.push(a)}PA(a){var b=this.C;a=b.indexOf(a);-1<a&&b.splice(a,1)}QA(a){var b=this.Fm;a=b.indexOf(a);-1<a&&b.splice(a,1)}Re(){}Mp(){return"submit"===this.tf&&(this.Jm&&bx(this),this.Gp&&ax(this),this.rh)?(this.setValue(this.th,{Ub:!0,Lv:!1}),!0):!1}D(a){ux(a)?(this.value=this.th=a.value,a.disabled?this.disable({Ub:!0,zb:!1}):this.enable({Ub:!0,zb:!1})):this.value=this.th=a}};Promise.resolve();var wx=new df("");const yx={oa:Jw,Ab:af(()=>xx)};var xx=class extends Jw{set fr(a){}constructor(a,b,c,d,e){super();this.Ym=e;this.update=new Fk;this.O=!1;this.mu(a);this.lu(b);this.C=dx(c)}tc(a){if(this.N(a)){var b=a.form.jx;b&&Uw(b,this);Ow(this.form,this,this.Ym);Zw(this.form,{zb:!1})}b=this.D;a.hasOwnProperty("model")?(a=a.model,a=a.wn?!0:!Object.is(b,a.pq)):a=!1;a&&(this.form.setValue(this.model),this.D=this.model)}va(){this.form&&Uw(this.form,this)}get path(){return[]}get control(){return this.form}N(a){return a.hasOwnProperty("form")}};
xx.K=function(a){return new (a||xx)(z(vw,10),z(ww,10),z(pw,10),z(wx,8),z(Nw,8))};xx.Fa=Qf({type:xx,ha:[["","formControl",""]],inputs:{form:[0,"formControl","form"],fr:[0,"disabled","isDisabled"],model:[0,"ngModel","model"]},outputs:{update:"ngModelChange"},ib:["ngForm"],features:[Kr([yx]),Cp,Dg]});const Ax={oa:Kw,Ab:af(()=>zx)};
var zx=class extends Kw{constructor(a,b,c){super();this.Ym=c;this.Uj=!1;this.Mi=()=>this.vu();this.Fv=[];this.form=null;this.Hl=new Fk;this.mu(a);this.lu(b)}tc(a){a.hasOwnProperty("form")&&(this.SA(),this.vu(),this.RA(),this.Em=this.form)}va(){this.form&&(Vw(this.form,this),this.form.Mi===this.Mi&&this.form.Ak(()=>{}))}get control(){return this.form}get path(){return[]}Er(a){this.Uj=!0;cx(this.form,this.Fv);this.Hl.emit(a);this.form.Bb.next(new jx(this.control));let b;return"dialog"===(null==a?void 0:
null==(b=a.target)?void 0:b.method)}Cr(){this.es()}es(){this.form.reset(void 0);this.Uj=!1;this.form.Bb.next(new kx(this.form))}vu(){this.Fv.forEach(a=>{const b=a.control,c=this.form.get(a.path);b!==c&&(Uw(b||null,a),c instanceof vx&&(Ow(c,a,this.Ym),a.control=c))});this.form.wu({zb:!1})}wF(a){const b=this.form.get(a.path);Pw(b,a);Zw(b,{zb:!1})}oF(a){if(this.form){const b=this.form.get(a.path);b&&Vw(b,a)&&Zw(b,{zb:!1})}}RA(){this.form.Ak(this.Mi);this.Em&&this.Em.Ak(()=>{})}SA(){Pw(this.form,this);
this.Em&&Vw(this.Em,this)}nF(){}};zx.K=function(a){return new (a||zx)(z(vw,10),z(ww,10),z(Nw,8))};zx.Fa=Qf({type:zx,ha:[["","formGroup",""]],Ia:function(a,b){a&1&&V("submit",function(c){return b.Er(c)})("reset",function(){return b.Cr()})},inputs:{form:[0,"formGroup","form"]},outputs:{Hl:"ngSubmit"},ib:["ngForm"],features:[Kr([Ax]),Cp,Dg]});var Bx=class{};Bx.K=function(a){return new (a||Bx)};Bx.Ya=Of({type:Bx});Bx.Xa=Nd({});var Cx=class{};Cx.K=function(a){return new (a||Cx)};Cx.Ya=Of({type:Cx});Cx.Xa=Nd({imports:[Bx]});function Dx(a){return 0===a.buttons||0===a.detail}function Ex(a){a=a.touches&&a.touches[0]||a.changedTouches&&a.changedTouches[0];return!!a&&-1===a.identifier&&(null==a.radiusX||1===a.radiusX)&&(null==a.radiusY||1===a.radiusY)};var Fx=new df("cdk-input-modality-detector-options"),Gx={kC:[18,17,224,91,16]};const Hx=Kt({passive:!0,capture:!0});
var Ix=class{get O(){return this.C.value}constructor(a,b,c,d){this.Ma=a;this.D=null;this.C=new ej(null);this.F=0;this.G=e=>{let f,g;(null==(f=this.Ni)?0:null==(g=f.kC)?0:g.some(k=>k===e.keyCode))||(this.C.next("keyboard"),this.D=St(e))};this.Hm=e=>{650>Date.now()-this.F||(this.C.next(Dx(e)?"keyboard":"mouse"),this.D=St(e))};this.J=e=>{Ex(e)?this.C.next("keyboard"):(this.F=Date.now(),this.C.next("touch"),this.D=St(e))};this.Ni=Object.assign({},Gx,d);this.N=C(this.C,Eo());C(this.N,bo());a.isBrowser&&
Mk(b,()=>{c.addEventListener("keydown",this.G,Hx);c.addEventListener("mousedown",this.Hm,Hx);c.addEventListener("touchstart",this.J,Hx)})}va(){this.C.complete();this.Ma.isBrowser&&(document.removeEventListener("keydown",this.G,Hx),document.removeEventListener("mousedown",this.Hm,Hx),document.removeEventListener("touchstart",this.J,Hx))}};Ix.K=function(a){return new (a||Ix)(r(Vt),r(Qk),r(jt),r(Fx,8))};Ix.pa=Md({ma:Ix,ba:Ix.K,ea:"root"});var Jx=new df("cdk-focus-monitor-default-options");const Kx=Kt({passive:!0,capture:!0});function Lx(a,b,c=!1){b=Ss(b);if(!a.Ma.isBrowser||1!==b.nodeType)return jj();const d=Qt(b)||a.G(),e=a.C.get(b);if(e)return c&&(e.bn=!0),e.Tg;c={bn:c,Tg:new cj,rootNode:d};a.C.set(b,c);a.Va(c);return c.Tg}function Mx(a,b){b=Ss(b);const c=a.C.get(b);c&&(c.Tg.complete(),a.P(b),a.C.delete(b),a.bb(c))}
function Nx(a,b,c,d){b=Ss(b);b===a.G().activeElement?a.Ja(b).forEach(([e,f])=>a.na(e,c,f)):(a.ua(c),"function"===typeof b.focus&&b.focus(d))}
var Ox=class{constructor(a,b,c,d,e){this.qa=a;this.Ma=b;this.J=c;this.F=null;this.O=this.T=!1;this.C=new Map;this.ja=0;this.D=new Map;this.Ea=()=>{this.T=!0;this.lb=window.setTimeout(()=>this.T=!1)};this.wa=new cj;this.N=f=>{for(var g=St(f);g;g=g.parentElement)"focus"===f.type?this.Fp(f,g):this.xk(f,g)};this.ia=d;this.aa=(null==e?void 0:e.CF)||0}va(){this.C.forEach((a,b)=>Mx(this,b))}G(){return this.ia||document}Pa(a){return this.F?this.O?this.gb(a)?"touch":"program":this.F:this.T&&this.fa?this.fa:
a&&this.Sa(a)?"mouse":"program"}gb(a){return 1===this.aa||!(null==a||!a.contains(this.J.D))}P(a,b){a.classList.toggle("cdk-focused",!!b);a.classList.toggle("cdk-touch-focused","touch"===b);a.classList.toggle("cdk-keyboard-focused","keyboard"===b);a.classList.toggle("cdk-mouse-focused","mouse"===b);a.classList.toggle("cdk-program-focused","program"===b)}ua(a,b=!1){Mk(this.qa,()=>{this.F=a;this.O="touch"===a&&b;0===this.aa&&(clearTimeout(this.sa),this.sa=setTimeout(()=>this.F=null,this.O?650:1))})}Fp(a,
b){const c=this.C.get(b);a=St(a);c&&(c.bn||b===a)&&this.na(b,this.Pa(a),c)}xk(a,b){const c=this.C.get(b);!c||c.bn&&a.relatedTarget instanceof Node&&b.contains(a.relatedTarget)||(this.P(b),this.ca(c,null))}ca(a,b){a.Tg.mc.length&&this.qa.run(()=>a.Tg.next(b))}Va(a){if(this.Ma.isBrowser){var b=a.rootNode;(a=this.D.get(b)||0)||Mk(this.qa,()=>{b.addEventListener("focus",this.N,Kx);b.addEventListener("blur",this.N,Kx)});this.D.set(b,a+1);1===++this.ja&&(Mk(this.qa,()=>{(this.G().defaultView||window).addEventListener("focus",
this.Ea)}),C(this.J.N,Lo(this.wa)).subscribe(c=>{this.ua(c,!0)}))}}bb(a){a=a.rootNode;if(this.D.has(a)){const b=this.D.get(a);1<b?this.D.set(a,b-1):(a.removeEventListener("focus",this.N,Kx),a.removeEventListener("blur",this.N,Kx),this.D.delete(a))}--this.ja||((this.G().defaultView||window).removeEventListener("focus",this.Ea),this.wa.next(),clearTimeout(this.lb),clearTimeout(this.sa))}na(a,b,c){this.P(a,b);this.ca(c,b);this.fa=b}Ja(a){const b=[];this.C.forEach((c,d)=>{(d===a||c.bn&&d.contains(a))&&
b.push([d,c])});return b}Sa(a){const b=this.J.D;if("mouse"!==this.J.O||!b||b===a||"INPUT"!==a.nodeName&&"TEXTAREA"!==a.nodeName||a.disabled)return!1;if(a=a.labels)for(let c=0;c<a.length;c++)if(a[c].contains(b))return!0;return!1}};Ox.K=function(a){return new (a||Ox)(r(Qk),r(Vt),r(Ix),r(jt,8),r(Jx,8))};Ox.pa=Md({ma:Ox,ba:Ox.K,ea:"root"});function Px(a,b){if(!a.Ma.isBrowser)return!1;try{var c=(b.ownerDocument&&b.ownerDocument.defaultView||window).frameElement}catch(f){c=null}if(c&&(-1===Qx(c)||!a.isVisible(c)))return!1;c=b.nodeName.toLowerCase();let d=Qx(b);if(b.hasAttribute("contenteditable"))a=-1!==d;else{var e;if(!(e="iframe"===c||"object"===c)&&(e=a.Ma.F&&a.Ma.C)){e=b.nodeName.toLowerCase();let f="input"===e&&b.type;e=!("text"===f||"password"===f||"select"===e||"textarea"===e)}a=e?!1:"audio"===c?b.hasAttribute("controls")?-1!==
d:!1:"video"===c?-1===d?!1:null!==d?!0:a.Ma.N||b.hasAttribute("controls"):0<=b.tabIndex}return a}function Rx(a,b){if("input"==b.nodeName.toLowerCase()&&"hidden"==b.type)var c=!1;else c=b.nodeName.toLowerCase(),c="input"===c||"select"===c||"button"===c||"textarea"===c||"a"==b.nodeName.toLowerCase()&&b.hasAttribute("href")||b.hasAttribute("contenteditable")||Sx(b);return c&&!a.fr(b)&&a.isVisible(b)}
var Tx=class{constructor(a){this.Ma=a}fr(a){return a.hasAttribute("disabled")}isVisible(a){return!!(a.offsetWidth||a.offsetHeight||"function"===typeof a.getClientRects&&a.getClientRects().length)&&"visible"===getComputedStyle(a).visibility}};Tx.K=function(a){return new (a||Tx)(r(Vt))};Tx.pa=Md({ma:Tx,ba:Tx.K,ea:"root"});function Sx(a){if(!a.hasAttribute("tabindex")||void 0===a.tabIndex)return!1;a=a.getAttribute("tabindex");return!(!a||isNaN(parseInt(a,10)))}
function Qx(a){if(!Sx(a))return null;a=parseInt(a.getAttribute("tabindex")||"",10);return isNaN(a)?-1:a};function Ux(a){(a=a.ja("start"))&&a.focus(void 0);return!!a}function Vx(a){a.P||(Mk(a.qa,()=>{a.F||(a.F=a.ca(),a.F.addEventListener("focus",a.sa));a.D||(a.D=a.ca(),a.D.addEventListener("focus",a.na))}),a.C.parentNode&&(a.C.parentNode.insertBefore(a.F,a.C),a.C.parentNode.insertBefore(a.D,a.C.nextSibling),a.P=!0))}function Wx(a){const b=a.C.querySelector("[cdk-focus-initial], [cdkFocusInitial]");if(b){if(!Rx(a.J,b))return a=a.T(b),null==a||a.focus(void 0),!!a;b.focus(void 0);return!0}return Ux(a)}
function Xx(a){return new Promise(b=>{a.ua(()=>b(Wx(a)))})}function Yx(a,b){a.F&&a.D&&(a.N(b,a.F),a.N(b,a.D))}
var Zx=class{get enabled(){return this.G}set enabled(a){this.G=a;this.F&&this.D&&(this.N(a,this.F),this.N(a,this.D))}constructor(a,b,c,d,e=!1){this.C=a;this.J=b;this.qa=c;this.ia=d;this.P=!1;this.sa=()=>{const f=this.ja("end");f&&f.focus(void 0);return!!f};this.na=()=>Ux(this);this.G=!0;e||Vx(this)}destroy(){const a=this.F,b=this.D;a&&(a.removeEventListener("focus",this.sa),a.remove());b&&(b.removeEventListener("focus",this.na),b.remove());this.F=this.D=null;this.P=!1}ja(a){const b=this.C.querySelectorAll(`[cdk-focus-region-${a}], `+
`[cdkFocusRegion${a}], `+`[cdk-focus-${a}]`);return"start"==a?b.length?b[0]:this.T(this.C):b.length?b[b.length-1]:this.fa(this.C)}ic(){return this.P}T(a){if(Rx(this.J,a)&&Px(this.J,a))return a;a=a.children;for(let b=0;b<a.length;b++){const c=a[b].nodeType===this.ia.ELEMENT_NODE?this.T(a[b]):null;if(c)return c}return null}fa(a){if(Rx(this.J,a)&&Px(this.J,a))return a;a=a.children;for(let b=a.length-1;0<=b;b--){const c=a[b].nodeType===this.ia.ELEMENT_NODE?this.fa(a[b]):null;if(c)return c}return null}ca(){const a=
this.ia.createElement("div");this.N(this.G,a);a.classList.add("cdk-visually-hidden");a.classList.add("cdk-focus-trap-anchor");a.setAttribute("aria-hidden","true");return a}N(a,b){a?b.setAttribute("tabindex","0"):b.removeAttribute("tabindex")}ua(a){this.qa.isStable?a():C(this.qa.Jl,fo()).subscribe(a)}},$x=class{constructor(a,b,c){this.C=a;this.qa=b;this.ia=c}create(a,b=!1){return new Zx(a,this.C,this.qa,this.ia,b)}};$x.K=function(a){return new (a||$x)(r(Tx),r(Qk),r(jt))};$x.pa=Md({ma:$x,ba:$x.K,ea:"root"});const ay=new Set;let by;function cy(a,b){if(a.Ma.F||a.Ma.G){var c=a.D;if(!ay.has(b))try{by||(by=document.createElement("style"),c&&by.setAttribute("nonce",c),by.setAttribute("type","text/css"),document.head.appendChild(by)),by.sheet&&(by.sheet.insertRule(`@media ${b} {body{ }}`,0),ay.add(b))}catch(d){console.error(d)}}return a.C(b)}var ey=class{constructor(a,b){this.Ma=a;this.D=b;this.C=this.Ma.isBrowser&&window.matchMedia?window.matchMedia.bind(window):dy}};
ey.K=function(a){return new (a||ey)(r(Vt),r(ni,8))};ey.pa=Md({ma:ey,ba:ey.K,ea:"root"});function dy(a){return{matches:"all"===a||""===a,media:a,addListener:()=>{},removeListener:()=>{}}};var gy=class{constructor(a,b){this.F=a;this.yb=b;this.D=new Map;this.C=new cj}va(){this.C.next();this.C.complete()}observe(a){a=fy(Ps(a)).map(b=>this.G(b).observable);a=Pj(a);a=hk(C(a,fo()),C(a,Eo(),On(0)));return C(a,xj(b=>{const c={matches:!1,Nu:{}};b.forEach(({matches:d,query:e})=>{c.matches=c.matches||d;c.Nu[e]=d});return c}))}G(a){if(this.D.has(a))return this.D.get(a);const b=cy(this.F,a),c={observable:C(new Xi(d=>{const e=f=>this.yb.run(()=>d.next(f));b.addListener(e);return()=>{b.removeListener(e)}}),
Ho(b),xj(({matches:d})=>({query:a,matches:d})),Lo(this.C)),YF:b};this.D.set(a,c);return c}};gy.K=function(a){return new (a||gy)(r(ey),r(Qk))};gy.pa=Md({ma:gy,ba:gy.K,ea:"root"});function fy(a){return a.map(b=>b.split(",")).reduce((b,c)=>b.concat(c)).map(b=>b.trim())};function hy(a){if(!a.Ma.isBrowser)return 0;const b=a.ia.createElement("div");b.style.backgroundColor="rgb(1,2,3)";b.style.position="absolute";a.ia.body.appendChild(b);a=(a=a.ia.defaultView||window)&&a.getComputedStyle?a.getComputedStyle(b):null;a=(a&&a.backgroundColor||"").replace(/ /g,"");b.remove();switch(a){case "rgb(0,0,0)":case "rgb(45,50,54)":case "rgb(32,32,32)":return 2;case "rgb(255,255,255)":case "rgb(255,250,239)":return 1}return 0}
var iy=class{constructor(a,b){this.Ma=a;this.ia=b;this.F=lf(gy).observe("(forced-colors: active)").subscribe(()=>{this.D&&(this.D=!1,this.C())})}va(){this.F.unsubscribe()}C(){if(!this.D&&this.Ma.isBrowser&&this.ia.body){const a=this.ia.body.classList;a.remove("cdk-high-contrast-active","cdk-high-contrast-black-on-white","cdk-high-contrast-white-on-black");this.D=!0;const b=hy(this);1===b?a.add("cdk-high-contrast-active","cdk-high-contrast-black-on-white"):2===b&&a.add("cdk-high-contrast-active","cdk-high-contrast-white-on-black")}}};
iy.K=function(a){return new (a||iy)(r(Vt),r(jt))};iy.pa=Md({ma:iy,ba:iy.K,ea:"root"});var ky=new df("liveAnnouncerElement",{ea:"root",ba:jy});function jy(){return null}var ly=new df("LIVE_ANNOUNCER_DEFAULT_OPTIONS");var my=class{create(a){return"undefined"===typeof MutationObserver?null:new MutationObserver(a)}};my.K=function(a){return new (a||my)};my.pa=Md({ma:my,ba:my.K,ea:"root"});var ny=class{};ny.K=function(a){return new (a||ny)};ny.Ya=Of({type:ny});ny.Xa=Nd({Wa:[my]});let oy=0;function py(a,b,...c){const d=a.J;let e,f;1===c.length&&"number"===typeof c[0]?f=c[0]:[e,f]=c;a.clear();clearTimeout(a.G);e||(e=d&&d.Ol?d.Ol:"polite");null==f&&d&&(f=d.duration);a.C.setAttribute("aria-live",e);a.C.id&&a.O(a.C.id);Mk(a.qa,()=>{a.D||(a.D=new Promise(g=>a.F=g));clearTimeout(a.G);a.G=setTimeout(()=>{a.C.textContent=b;"number"===typeof f&&(a.G=setTimeout(()=>a.clear(),f));let g;null==(g=a.F)||g.call(a);a.D=a.F=void 0},100);return a.D})}
var qy=class{constructor(a,b,c,d){this.qa=b;this.J=d;this.ia=c;this.C=a||this.N()}clear(){this.C&&(this.C.textContent="")}va(){clearTimeout(this.G);let a;null==(a=this.C)||a.remove();this.C=null;let b;null==(b=this.F)||b.call(this);this.D=this.F=void 0}N(){const a=this.ia.getElementsByClassName("cdk-live-announcer-element"),b=this.ia.createElement("div");for(let c=0;c<a.length;c++)a[c].remove();b.classList.add("cdk-live-announcer-element");b.classList.add("cdk-visually-hidden");b.setAttribute("aria-atomic",
"true");b.setAttribute("aria-live","polite");b.id=`cdk-live-announcer-${oy++}`;this.ia.body.appendChild(b);return b}O(a){const b=this.ia.querySelectorAll('body > .cdk-overlay-container [aria-modal="true"]');for(let c=0;c<b.length;c++){const d=b[c],e=d.getAttribute("aria-owns");e?-1===e.indexOf(a)&&d.setAttribute("aria-owns",e+" "+a):d.setAttribute("aria-owns",a)}}};qy.K=function(a){return new (a||qy)(r(ky,8),r(Qk),r(jt),r(ly,8))};qy.pa=Md({ma:qy,ba:qy.K,ea:"root"});var ry=class{constructor(a){a.C()}};ry.K=function(a){return new (a||ry)(r(iy))};ry.Ya=Of({type:ry});ry.Xa=Nd({imports:[ny]});function sy(a,b,c){const d=ty(a,b);c=c.trim();d.some(e=>e.trim()===c)||(d.push(c),a.setAttribute(b,d.join(" ")))}function uy(a,b,c){var d=ty(a,b);c=c.trim();d=d.filter(e=>e!==c);d.length?a.setAttribute(b,d.join(" ")):a.removeAttribute(b)}function ty(a,b){let c,d;return null!=(d=null==(c=a.getAttribute(b))?void 0:c.match(/\S+/g))?d:[]};let vy=0;function wy(a,b,c,d){if(a.N(b,c)){var e=xy(c,d);"string"!==typeof c?(yy(c,a.Xb),a.C.set(e,{El:c,referenceCount:0})):a.C.has(e)||a.O(c,d);a.F(b,e)||a.J(b,e)}}function zy(a,b,c){if(c&&a.G(b)){var d=xy(c,"tooltip");a.F(b,d)&&a.ca(b,d);"string"===typeof c&&(b=a.C.get(d))&&0===b.referenceCount&&a.T(d);var e;0===(null==(e=a.D)?void 0:e.childNodes.length)&&(a.D.remove(),a.D=null)}}
var Ay=class{constructor(a,b){this.Ma=b;this.C=new Map;this.D=null;this.Xb=`${vy++}`;this.ia=a;this.Xb=lf(ji)+"-"+vy++}va(){const a=this.ia.querySelectorAll(`[${"cdk-describedby-host"}="${this.Xb}"]`);for(let c=0;c<a.length;c++)this.aa(a[c]),a[c].removeAttribute("cdk-describedby-host");let b;null==(b=this.D)||b.remove();this.D=null;this.C.clear()}O(a,b){const c=this.ia.createElement("div");yy(c,this.Xb);c.textContent=a;b&&c.setAttribute("role",b);this.P();this.D.appendChild(c);this.C.set(xy(a,b),
{El:c,referenceCount:0})}T(a){let b,c;null==(b=this.C.get(a))||null==(c=b.El)||c.remove();this.C.delete(a)}P(){if(!this.D){var a=this.ia.querySelectorAll('.cdk-describedby-message-container[platform="server"]');for(let b=0;b<a.length;b++)a[b].remove();a=this.ia.createElement("div");a.style.visibility="hidden";a.classList.add("cdk-describedby-message-container");a.classList.add("cdk-visually-hidden");this.Ma&&!this.Ma.isBrowser&&a.setAttribute("platform","server");this.ia.body.appendChild(a);this.D=
a}}aa(a){const b=ty(a,"aria-describedby").filter(c=>0!=c.indexOf("cdk-describedby-message"));a.setAttribute("aria-describedby",b.join(" "))}J(a,b){b=this.C.get(b);sy(a,"aria-describedby",b.El.id);a.setAttribute("cdk-describedby-host",this.Xb);b.referenceCount++}ca(a,b){b=this.C.get(b);b.referenceCount--;uy(a,"aria-describedby",b.El.id);a.removeAttribute("cdk-describedby-host")}F(a,b){a=ty(a,"aria-describedby");b=(b=this.C.get(b))&&b.El.id;return!!b&&-1!=a.indexOf(b)}N(a,b){if(!this.G(a))return!1;
if(b&&"object"===typeof b)return!0;b=null==b?"":`${b}`.trim();a=a.getAttribute("aria-label");return b?!a||a.trim()!==b:!1}G(a){return a.nodeType===this.ia.ELEMENT_NODE}};Ay.K=function(a){return new (a||Ay)(r(jt),r(Vt))};Ay.pa=Md({ma:Ay,ba:Ay.K,ea:"root"});function xy(a,b){return"string"===typeof a?`${b||""}/${a}`:a}function yy(a,b){a.id||(a.id=`${"cdk-describedby-message"}-${b}-${vy++}`)};var Dy=class extends Zx{get enabled(){return this.G}set enabled(a){(this.G=a)?this.O.register(this):By(this.O,this)}constructor(a,b,c,d,e,f,g){super(a,b,c,d,g.defer);this.O=e;this.aa=f;this.O.register(this)}destroy(){By(this.O,this);super.destroy()}yt(){Cy(this.aa,this);Yx(this,!0)}xt(){var a=this.aa;a.C&&(this.ia.removeEventListener("focus",a.C,!0),a.C=null);Yx(this,!1)}};function Cy(a,b){a.C&&b.ia.removeEventListener("focus",a.C,!0);a.C=c=>Ey(b,c);Mk(b.qa,()=>{b.ia.addEventListener("focus",a.C,!0)})}function Ey(a,b){b=b.target;const c=a.C;let d;!b||c.contains(b)||(null==(d=b.closest)?0:d.call(b,"div.cdk-overlay-pane"))||setTimeout(()=>{a.enabled&&!c.contains(a.ia.activeElement)&&Ux(a)})}var Fy=class{constructor(){this.C=null}};var Gy=new df("FOCUS_TRAP_INERT_STRATEGY");function By(a,b){b.xt();a=a.C;b=a.indexOf(b);-1!==b&&(a.splice(b,1),a.length&&a[a.length-1].yt())}var Hy=class{constructor(){this.C=[]}register(a){let b=this.C=this.C.filter(c=>c!==a);b.length&&b[b.length-1].xt();b.push(a);a.yt()}};Hy.K=function(a){return new (a||Hy)};Hy.pa=Md({ma:Hy,ba:Hy.K,ea:"root"});var Iy=class{constructor(a,b,c,d,e){this.C=a;this.qa=b;this.D=c;this.ia=d;this.F=e||new Fy}create(a,b={defer:!1}){let c;"boolean"===typeof b?c={defer:b}:c=b;return new Dy(a,this.C,this.qa,this.ia,this.D,this.F,c)}};Iy.K=function(a){return new (a||Iy)(r(Tx),r(Qk),r(Hy),r(jt),r(Gy,8))};Iy.pa=Md({ma:Iy,ba:Iy.K,ea:"root"});function Jy(a,b){a.na=b;return a}function Ky(a){a.Ja=!0;return a}function Ly(a){a.Ea=!0;return a}function My(a,b){a.fa=b;return a}function Ny(a){a.Pa=["shiftKey"];return a}function Oy(a,b=200){a.wa.unsubscribe();a.wa=C(a.ja,Po(c=>a.G.push(c)),On(b),tk(()=>0<a.G.length),xj(()=>a.G.join(""))).subscribe(c=>{const d=a.N();for(let e=1;e<d.length+1;e++){const f=(a.F+e)%d.length,g=d[f];if(!a.na(g)&&0===g.Vv().toUpperCase().trim().indexOf(c)){a.D(f);break}}a.G=[]});return a}
function Py(a){a.ua=!0;return a}function Qy(a){a.aa={enabled:!0,delta:10};return a}function Ry(a,b){var c=a.N();b="number"===typeof b?b:c.indexOf(b);c=c[b];a.P=null==c?null:c;a.F=b}function Sy(a){0>a.F?a.J(0,1):a.Va(1)}function Ty(a){0>a.F&&a.Ja?a.J(a.N().length-1,-1):a.Va(-1)}
function Uy(a,b){var c=b.keyCode,d=["altKey","ctrlKey","metaKey","shiftKey"].every(e=>!b[e]||-1<a.Pa.indexOf(e));switch(c){case 9:a.ca.next();return;case 40:if(a.Ea&&d){Sy(a);break}else return;case 38:if(a.Ea&&d){Ty(a);break}else return;case 39:if(a.fa&&d){"rtl"===a.fa?Ty(a):Sy(a);break}else return;case 37:if(a.fa&&d){"rtl"===a.fa?Sy(a):Ty(a);break}else return;case 36:if(a.ua&&d){a.J(0,1);break}else return;case 35:if(a.ua&&d){a.J(a.N().length-1,-1);break}else return;case 33:if(a.aa.enabled&&d){c=
a.F-a.aa.delta;a.J(0<c?c:0,1);break}else return;case 34:if(a.aa.enabled&&d){c=a.F+a.aa.delta;d=a.N().length;a.J(c<d?c:d-1,-1);break}else return;default:if(d||Bu(b,"shiftKey"))b.key&&1===b.key.length?a.ja.next(b.key.toLocaleUpperCase()):(65<=c&&90>=c||48<=c&&57>=c)&&a.ja.next(String.fromCharCode(c));return}a.G=[];b.preventDefault()}
var Vy=class{constructor(a,b){this.T=a;this.F=-1;this.P=null;this.Ja=!1;this.ja=new cj;this.wa=Ni.EMPTY;this.Ea=!0;this.Pa=[];this.ua=!1;this.aa={enabled:!1,delta:10};this.na=c=>c.disabled;this.G=[];this.ca=new cj;this.wb=new cj;a instanceof ar?this.gb=a.C.subscribe(c=>this.Sa(c.toArray())):yn(a)&&(this.bb=Js(()=>this.Sa(a()),{xa:b}))}D(a){const b=this.P;Ry(this,a);this.P!==b&&this.wb.next(this.F)}get O(){return this.F}get C(){return this.P}destroy(){this.wa.unsubscribe();let a;null==(a=this.gb)||
a.unsubscribe();let b;null==(b=this.bb)||b.destroy();this.ja.complete();this.ca.complete();this.wb.complete();this.G=[]}Va(a){this.Ja?this.sb(a):this.lb(a)}sb(a){const b=this.N();for(let c=1;c<=b.length;c++){const d=(this.F+a*c+b.length)%b.length;if(!this.na(b[d])){this.D(d);break}}}lb(a){this.J(this.F+a,a)}J(a,b){const c=this.N();if(c[a]){for(;this.na(c[a]);)if(a+=b,!c[a])return;this.D(a)}}N(){return yn(this.T)?this.T():this.T instanceof ar?this.T.toArray():this.T}Sa(a){this.P&&(a=a.indexOf(this.P),
-1<a&&a!==this.F&&(this.F=a))}};var Wy=class extends Vy{D(a){if(this.C){var b=this.C;b.D&&(b.D=!1,bn(b.Ba))}super.D(a);this.C&&(a=this.C,a.D||(a.D=!0,bn(a.Ba)))}};var Xy=class extends Vy{constructor(){super(...arguments);this.sa="program"}D(a){super.D(a);this.C&&this.C.focus(this.sa)}};function Yy(a,b){return{type:7,name:a,yB:b,options:{}}}function Zy(a,b=null){return{type:4,styles:b,dg:a}}function $y(a){return{type:6,styles:a,offset:null}}function az(a,b){return{type:0,name:a,styles:b,options:void 0}}function bz(a,b){return{type:1,MB:a,animation:b,options:null}};function cz(a){a.G||(a.G=!0,a.C.forEach(b=>b()),a.C=[])}function dz(a){a.D.forEach(b=>b());a.D=[]}function ez(a){queueMicrotask(()=>cz(a))}
var fz=class{constructor(a=0,b=0){this.C=[];this.D=[];this.J=[];this.O=[];this.P=[];this.G=this.Ga=this.N=!1;this.F=null;this.totalTime=a+b}onStart(a){this.P.push(a);this.D.push(a)}Kd(a){this.O.push(a);this.C.push(a)}ob(a){this.J.push(a)}Qf(){return this.N}init(){}play(){this.Qf()||(dz(this),ez(this));this.N=!0}pause(){}le(){}finish(){cz(this)}destroy(){this.Ga||(this.Ga=!0,this.Qf()||dz(this),this.finish(),this.J.forEach(a=>a()),this.J=[])}reset(){this.G=this.N=!1;this.D=this.P;this.C=this.O}setPosition(){}oi(a){a=
"start"==a?this.D:this.C;a.forEach(b=>b());a.length=0}};function gz(a){a.J||(a.J=!0,a.D.forEach(b=>b()),a.D=[])}function hz(a){a.Ga||(a.Ga=!0,gz(a),a.C.forEach(b=>b.destroy()),a.N.forEach(b=>b()),a.N=[])}function iz(a){a.Qf()||(a.O=!0,a.G.forEach(b=>b()),a.G=[])}
var jz=class{constructor(a){this.D=[];this.G=[];this.Ga=this.O=this.J=!1;this.N=[];this.F=null;this.totalTime=0;this.C=a;let b=0,c=0,d=0;const e=this.C.length;0==e?queueMicrotask(()=>gz(this)):this.C.forEach(f=>{f.Kd(()=>{++b==e&&gz(this)});f.ob(()=>{++c==e&&hz(this)});f.onStart(()=>{++d==e&&iz(this)})});this.totalTime=this.C.reduce((f,g)=>Math.max(f,g.totalTime),0)}init(){this.C.forEach(a=>a.init())}onStart(a){this.G.push(a)}Kd(a){this.D.push(a)}ob(a){this.N.push(a)}Qf(){return this.O}play(){this.F||
this.init();iz(this);this.C.forEach(a=>a.play())}pause(){this.C.forEach(a=>a.pause())}le(){this.C.forEach(a=>a.le())}finish(){gz(this);this.C.forEach(a=>a.finish())}destroy(){hz(this)}reset(){this.C.forEach(a=>a.reset());this.O=this.J=this.Ga=!1}setPosition(a){const b=a*this.totalTime;this.C.forEach(c=>{c.setPosition(c.totalTime?Math.min(1,b/c.totalTime):1)})}Jk(){this.C.forEach(a=>{a.Jk&&a.Jk()})}oi(a){a="start"==a?this.G:this.D;a.forEach(b=>b());a.length=0}};var kz=Yy("panelAnimation",[az("void, hidden",$y({opacity:0,transform:"scaleY(0.8)"})),bz(":enter, hidden => visible",[function(a,b=null){return{type:3,steps:a,options:b}}([Zy("0.03s linear",$y({opacity:1})),Zy("0.12s cubic-bezier(0, 0, 0.2, 1)",$y({transform:"scaleY(1)"}))])]),bz(":leave, visible => hidden",[Zy("0.075s linear",$y({opacity:0}))])]);var lz=new df("mat-sanity-checks",{ea:"root",ba:function(){return!0}}),mz=class{constructor(a,b,c){this.C=b;this.ia=c;this.D=!1;a.C();this.D||(this.D=!0)}F(a){return Tt()?!1:"boolean"===typeof this.C?this.C:!!this.C[a]}};mz.K=function(a){return new (a||mz)(r(iy),r(lz,8),r(jt))};mz.Ya=Of({type:mz});mz.Xa=Nd({imports:[pu,pu]});var nz=class{constructor(a,b,c,d,e){this.D=a;this.Jd=b;this.G=c;this.F=d;this.C=e;this.xe=!1}};new df("MAT_DATE_LOCALE",{ea:"root",ba:oz});function oz(){return lf(os)};new df("mat-date-formats");var pz=class{};pz.K=function(a){return new (a||pz)};pz.pa=Md({ma:pz,ba:pz.K,ea:"root"});var qz=class{};qz.K=function(a){return new (a||qz)};
qz.Da=Gf({type:qz,ha:[["div","mat-internal-form-field",""]],Ka:[1,"mdc-form-field","mat-internal-form-field"],Za:2,Ia:function(a,b){a&2&&nq("mdc-form-field--align-end","before"===b.Lh)},inputs:{Lh:"labelPosition"},ka:!0,features:[Mr],attrs:["mat-internal-form-field",""],Zb:["*"],za:1,Aa:0,ta:function(a){a&1&&(Xq(),Yq(0))},styles:[".mdc-form-field{display:inline-flex;align-items:center;vertical-align:middle}.mdc-form-field[hidden]{display:none}.mdc-form-field>label{margin-left:0;margin-right:auto;padding-left:4px;padding-right:0;order:0;cursor:inherit}[dir=rtl] .mdc-form-field>label,.mdc-form-field>label[dir=rtl]{margin-left:auto;margin-right:0}[dir=rtl] .mdc-form-field>label,.mdc-form-field>label[dir=rtl]{padding-left:0;padding-right:4px}.mdc-form-field--nowrap>label{text-overflow:ellipsis;overflow:hidden;white-space:nowrap}.mdc-form-field--align-end>label{margin-left:auto;margin-right:0;padding-left:0;padding-right:4px;order:-1}[dir=rtl] .mdc-form-field--align-end>label,.mdc-form-field--align-end>label[dir=rtl]{margin-left:0;margin-right:auto}[dir=rtl] .mdc-form-field--align-end>label,.mdc-form-field--align-end>label[dir=rtl]{padding-left:4px;padding-right:0}.mdc-form-field--space-between{justify-content:space-between}.mdc-form-field--space-between>label{margin:0}[dir=rtl] .mdc-form-field--space-between>label,.mdc-form-field--space-between>label[dir=rtl]{margin:0}.mdc-form-field{font-family:var(--mdc-form-field-label-text-font);line-height:var(--mdc-form-field-label-text-line-height);font-size:var(--mdc-form-field-label-text-size);font-weight:var(--mdc-form-field-label-text-weight);letter-spacing:var(--mdc-form-field-label-text-tracking);color:var(--mdc-form-field-label-text-color)}.mat-internal-form-field{-moz-osx-font-smoothing:grayscale;-webkit-font-smoothing:antialiased}\n"],pb:2,
Ra:0});var rz=new df("MAT_OPTION_PARENT_COMPONENT");const sz=["*",[["mat-option"],["ng-container"]]];let tz=0;var uz=new df("MatOptgroup"),vz=class{constructor(a){this.disabled=!1;this.Ue=`mat-optgroup-label-${tz++}`;let b;this.Am=null!=(b=null==a?void 0:a.pC)?b:!1}};vz.K=function(a){return new (a||vz)(z(rz,8))};
vz.Da=Gf({type:vz,ha:[["mat-optgroup"]],Ka:[1,"mat-mdc-optgroup"],Za:3,Ia:function(a,b){a&2&&Kp("role",b.Am?null:"group")("aria-disabled",b.Am?null:b.disabled.toString())("aria-labelledby",b.Am?null:b.Ue)},inputs:{label:"label",disabled:[2,"disabled","disabled",zs]},ib:["matOptgroup"],ka:!0,features:[Kr([{oa:uz,Ab:vz}]),Jp,Mr],Zb:["*","mat-option, ng-container"],za:5,Aa:4,Ca:[["role","presentation",1,"mat-mdc-optgroup-label",3,"id"],[1,"mdc-list-item__primary-text"]],ta:function(a,b){a&1&&(Xq(sz),
R(0,"span",0)(1,"span",1),Z(2),Yq(3),S()(),Yq(4,1));a&2&&(nq("mdc-list-item--disabled",b.disabled),H("id",b.Ue),D(2),Br("",b.label," "))},styles:[".mat-mdc-optgroup{color:var(--mat-optgroup-label-text-color);font-family:var(--mat-optgroup-label-text-font);line-height:var(--mat-optgroup-label-text-line-height);font-size:var(--mat-optgroup-label-text-size);letter-spacing:var(--mat-optgroup-label-text-tracking);font-weight:var(--mat-optgroup-label-text-weight)}.mat-mdc-optgroup-label{display:flex;position:relative;align-items:center;justify-content:flex-start;overflow:hidden;padding:0;padding-left:16px;padding-right:16px;min-height:48px}.mat-mdc-optgroup-label:focus{outline:none}[dir=rtl] .mat-mdc-optgroup-label,.mat-mdc-optgroup-label[dir=rtl]{padding-left:16px;padding-right:16px}.mat-mdc-optgroup-label.mdc-list-item--disabled{opacity:.38}.mat-mdc-optgroup-label .mdc-list-item__primary-text{font-size:inherit;font-weight:inherit;letter-spacing:inherit;line-height:inherit;font-family:inherit;text-decoration:inherit;text-transform:inherit;white-space:normal}\n"],
pb:2,Ra:0});const wz=Kt({passive:!0,capture:!0});function xz(a,b,c,d){var e=yz;const f=e.Bb.get(b);f?(a=f.get(c))?a.add(d):f.set(c,new Set([d])):(e.Bb.set(b,new Map([[c,new Set([d])]])),Mk(a,()=>{document.addEventListener(b,e.C,wz)}))}var zz=class{constructor(){this.Bb=new Map;this.C=a=>{const b=St(a);if(b){let c;null==(c=this.Bb.get(a.type))||c.forEach((d,e)=>{(e===b||e.contains(b))&&d.forEach(f=>f.handleEvent(a))})}}}};var Az=class{constructor(a,b,c,d=!1){this.C=a;this.element=b;this.config=c;this.D=d;this.state=3}};var Bz={Mv:225,un:150};const Cz=Kt({passive:!0,capture:!0}),Dz=["mousedown","touchstart"],Ez=["mouseup","mouseleave","touchend","touchcancel"];function Fz(a,b){if(2!==b.state&&3!==b.state){var c=b.element,d=Object.assign({},Bz,b.config.animation);c.style.transitionDuration=`${d.un}ms`;c.style.opacity="0";b.state=2;!b.D&&d.un||a.np(b)}}
function Gz(a,b,c,d={}){var e=a.Ei=a.Ei||a.Di.getBoundingClientRect();const f=Object.assign({},Bz,d.animation);d.Wi&&(b=e.left+e.width/2,c=e.top+e.height/2);var g=d.radius||Hz(b,c,e);b-=e.left;const k=c-e.top;c=f.Mv;const m=document.createElement("div");m.classList.add("mat-ripple-element");m.style.left=`${b-g}px`;m.style.top=`${k-g}px`;m.style.height=`${2*g}px`;m.style.width=`${2*g}px`;null!=d.color&&(m.style.backgroundColor=d.color);m.style.transitionDuration=`${c}ms`;a.Di.appendChild(m);g=window.getComputedStyle(m);
b=g.transitionDuration;e="none"===g.transitionProperty||"0s"===b||"0s, 0s"===b||0===e.width&&0===e.height;const p=new Az(a,m,d,e);m.style.transform="scale3d(1, 1, 1)";p.state=0;d.Nr||(a.Cp=p);let t=null;e||!c&&!f.un||Mk(a.qa,()=>{const w=()=>a.np(p),B=()=>a.wt(p);m.addEventListener("transitionend",w);m.addEventListener("transitioncancel",B);t={jD:w,iD:B}});a.kk.set(p,t);!e&&c||a.np(p)}function Iz(a,b){const c=Ss(b);a.Ma.isBrowser&&c&&c!==a.Pi&&(a.iu(),a.Pi=c,Dz.forEach(d=>{xz(a.qa,d,c,a)}))}
var Jz=class{constructor(a,b,c,d){this.Pm=a;this.qa=b;this.Ma=d;this.uk=!1;this.kk=new Map;this.Km=!1;d.isBrowser&&(this.Di=Ss(c))}Eq(){this.Ct().forEach(a=>{a.config.Nr||Fz(a.C,a)})}handleEvent(a){"mousedown"===a.type?this.Hm(a):"touchstart"===a.type?this.rA(a):this.qA();this.Km||(Mk(this.qa,()=>{Ez.forEach(b=>{this.Pi.addEventListener(b,this,Cz)})}),this.Km=!0)}np(a){0===a.state?this.IA(a):2===a.state&&this.wt(a)}IA(a){const b=a===this.Cp,c=a.config.Nr;a.state=1;c||b&&this.uk||Fz(a.C,a)}wt(a){let b;
const c=null!=(b=this.kk.get(a))?b:null;this.kk.delete(a);this.kk.size||(this.Ei=null);a===this.Cp&&(this.Cp=null);a.state=3;null!==c&&(a.element.removeEventListener("transitionend",c.jD),a.element.removeEventListener("transitioncancel",c.iD));a.element.remove()}Hm(a){const b=Dx(a),c=this.au&&Date.now()<this.au+800;this.Pm.xx||b||c||(this.uk=!0,Gz(this,a.clientX,a.clientY,this.Pm.wx))}rA(a){if(!this.Pm.xx&&!Ex(a)&&(this.au=Date.now(),this.uk=!0,a=a.changedTouches))for(let b=0;b<a.length;b++)Gz(this,
a[b].clientX,a[b].clientY,this.Pm.wx)}qA(){this.uk&&(this.uk=!1,this.Ct().forEach(a=>{const b=1===a.state||a.config.ly&&0===a.state;!a.config.Nr&&b&&Fz(a.C,a)}))}Ct(){return Array.from(this.kk.keys())}iu(){const a=this.Pi;a&&(Dz.forEach(b=>{var c=yz;const d=c.Bb.get(b);if(d){var e=d.get(a);e&&(e.delete(this),0===e.size&&d.delete(a),0===d.size&&(c.Bb.delete(b),document.removeEventListener(b,c.C,wz)))}}),this.Km&&(Ez.forEach(b=>a.removeEventListener(b,this,Cz)),this.Km=!1))}},yz=new zz;
function Hz(a,b,c){a=Math.max(Math.abs(a-c.left),Math.abs(a-c.right));b=Math.max(Math.abs(b-c.top),Math.abs(b-c.bottom));return Math.sqrt(a*a+b*b)};var Kz=new df("mat-ripple-global-options"),Lz=class{get disabled(){return this.Ib}set disabled(a){a&&this.Eq();this.Ib=a;this.G()}get Le(){return this.J||this.ra.R}set Le(a){this.J=a;this.G()}constructor(a,b,c,d,e){this.ra=a;this.ad=e;this.radius=0;this.C=this.Ib=!1;this.D=d||{};this.F=new Jz(this,b,a,c)}Fb(){this.C=!0;this.G()}va(){this.F.iu()}Eq(){this.F.Eq()}get wx(){return{Wi:this.Wi,radius:this.radius,color:this.color,animation:Object.assign({},this.D.animation,"NoopAnimations"===this.ad?{Mv:0,
un:0}:{},this.animation),ly:this.D.ly}}get xx(){return this.disabled||!!this.D.disabled}G(){!this.disabled&&this.C&&Iz(this.F,this.Le)}};Lz.K=function(a){return new (a||Lz)(z(si),z(Qk),z(Vt),z(Kz,8),z(mi,8))};
Lz.Fa=Qf({type:Lz,ha:[["","mat-ripple",""],["","matRipple",""]],Ka:[1,"mat-ripple"],Za:2,Ia:function(a,b){a&2&&nq("mat-ripple-unbounded",b.LE)},inputs:{color:[0,"matRippleColor","color"],LE:[0,"matRippleUnbounded","unbounded"],Wi:[0,"matRippleCentered","centered"],radius:[0,"matRippleRadius","radius"],animation:[0,"matRippleAnimation","animation"],disabled:[0,"matRippleDisabled","disabled"],Le:[0,"matRippleTrigger","trigger"]},ib:["matRipple"],ka:!0});var Mz=class{constructor(a){this.ad=a;this.state="unchecked";this.disabled=!1;this.Fd="full"}};Mz.K=function(a){return new (a||Mz)(z(mi,8))};
Mz.Da=Gf({type:Mz,ha:[["mat-pseudo-checkbox"]],Ka:[1,"mat-pseudo-checkbox"],Za:12,Ia:function(a,b){a&2&&nq("mat-pseudo-checkbox-indeterminate","indeterminate"===b.state)("mat-pseudo-checkbox-checked","checked"===b.state)("mat-pseudo-checkbox-disabled",b.disabled)("mat-pseudo-checkbox-minimal","minimal"===b.Fd)("mat-pseudo-checkbox-full","full"===b.Fd)("_mat-animation-noopable","NoopAnimations"===b.ad)},inputs:{state:"state",disabled:"disabled",Fd:"appearance"},ka:!0,features:[Mr],za:0,Aa:0,ta:function(){},
styles:['.mat-pseudo-checkbox{border-radius:2px;cursor:pointer;display:inline-block;vertical-align:middle;box-sizing:border-box;position:relative;flex-shrink:0;transition:border-color 90ms cubic-bezier(0, 0, 0.2, 0.1),background-color 90ms cubic-bezier(0, 0, 0.2, 0.1)}.mat-pseudo-checkbox::after{position:absolute;opacity:0;content:"";border-bottom:2px solid currentColor;transition:opacity 90ms cubic-bezier(0, 0, 0.2, 0.1)}.mat-pseudo-checkbox._mat-animation-noopable{transition:none !important;animation:none !important}.mat-pseudo-checkbox._mat-animation-noopable::after{transition:none}.mat-pseudo-checkbox-disabled{cursor:default}.mat-pseudo-checkbox-indeterminate::after{left:1px;opacity:1;border-radius:2px}.mat-pseudo-checkbox-checked::after{left:1px;border-left:2px solid currentColor;transform:rotate(-45deg);opacity:1;box-sizing:content-box}.mat-pseudo-checkbox-minimal.mat-pseudo-checkbox-checked::after,.mat-pseudo-checkbox-minimal.mat-pseudo-checkbox-indeterminate::after{color:var(--mat-minimal-pseudo-checkbox-selected-checkmark-color)}.mat-pseudo-checkbox-minimal.mat-pseudo-checkbox-checked.mat-pseudo-checkbox-disabled::after,.mat-pseudo-checkbox-minimal.mat-pseudo-checkbox-indeterminate.mat-pseudo-checkbox-disabled::after{color:var(--mat-minimal-pseudo-checkbox-disabled-selected-checkmark-color)}.mat-pseudo-checkbox-full{border-color:var(--mat-full-pseudo-checkbox-unselected-icon-color);border-width:2px;border-style:solid}.mat-pseudo-checkbox-full.mat-pseudo-checkbox-disabled{border-color:var(--mat-full-pseudo-checkbox-disabled-unselected-icon-color)}.mat-pseudo-checkbox-full.mat-pseudo-checkbox-checked,.mat-pseudo-checkbox-full.mat-pseudo-checkbox-indeterminate{background-color:var(--mat-full-pseudo-checkbox-selected-icon-color);border-color:rgba(0,0,0,0)}.mat-pseudo-checkbox-full.mat-pseudo-checkbox-checked::after,.mat-pseudo-checkbox-full.mat-pseudo-checkbox-indeterminate::after{color:var(--mat-full-pseudo-checkbox-selected-checkmark-color)}.mat-pseudo-checkbox-full.mat-pseudo-checkbox-checked.mat-pseudo-checkbox-disabled,.mat-pseudo-checkbox-full.mat-pseudo-checkbox-indeterminate.mat-pseudo-checkbox-disabled{background-color:var(--mat-full-pseudo-checkbox-disabled-selected-icon-color)}.mat-pseudo-checkbox-full.mat-pseudo-checkbox-checked.mat-pseudo-checkbox-disabled::after,.mat-pseudo-checkbox-full.mat-pseudo-checkbox-indeterminate.mat-pseudo-checkbox-disabled::after{color:var(--mat-full-pseudo-checkbox-disabled-selected-checkmark-color)}.mat-pseudo-checkbox{width:18px;height:18px}.mat-pseudo-checkbox-minimal.mat-pseudo-checkbox-checked::after{width:14px;height:6px;transform-origin:center;top:-4.2426406871px;left:0;bottom:0;right:0;margin:auto}.mat-pseudo-checkbox-minimal.mat-pseudo-checkbox-indeterminate::after{top:8px;width:16px}.mat-pseudo-checkbox-full.mat-pseudo-checkbox-checked::after{width:10px;height:4px;transform-origin:center;top:-2.8284271247px;left:0;bottom:0;right:0;margin:auto}.mat-pseudo-checkbox-full.mat-pseudo-checkbox-indeterminate::after{top:6px;width:12px}\n'],
pb:2,Ra:0});const Nz=["text"],Oz=[[["mat-icon"]],"*"];function Pz(a){a&1&&Mq(0,"mat-pseudo-checkbox",1);a&2&&(a=Y(),H("disabled",a.disabled)("state",a.selected?"checked":"unchecked"))}function Qz(a){a&1&&Mq(0,"mat-pseudo-checkbox",3);a&2&&(a=Y(),H("disabled",a.disabled))}function Rz(a){a&1&&(R(0,"span",4),Z(1),S());a&2&&(a=Y(),D(),Br("(",a.group.label,")"))}let Sz=0;var Tz=class{constructor(a,b=!1){this.source=a;this.C=b}};function Uz(a,b=!0){a.C&&(a.C=!1,bn(a.Ba),b&&a.J())}
var Vz=class{get multiple(){return this.Qa&&this.Qa.multiple}get selected(){return this.C}get disabled(){return this.group&&this.group.disabled||this.Ib}set disabled(a){this.Ib=a}get Nc(){return!(!this.Qa||!this.Qa.Nc)}get Ae(){return!(!this.Qa||!this.Qa.Ae)}constructor(a,b,c,d){this.O=a;this.Ba=b;this.Qa=c;this.group=d;this.Ib=this.D=this.C=!1;this.N="";this.id=`mat-option-${Sz++}`;this.jo=new Fk;this.G=new cj}get active(){return this.D}get F(){let a;return((null==(a=this.MA)?void 0:a.R.textContent)||
"").trim()}select(a=!0){this.C||(this.C=!0,bn(this.Ba),a&&this.J())}focus(a,b){const c=this.qe();"function"===typeof c.focus&&c.focus(b)}Vv(){return this.F}Se(a){13!==a.keyCode&&32!==a.keyCode||Bu(a)||(this.uh(),a.preventDefault())}uh(){this.disabled||(this.C=this.multiple?!this.C:!0,bn(this.Ba),this.J(!0))}Gt(){return this.disabled?"-1":"0"}qe(){return this.O.R}Lw(){if(this.C){const a=this.F;a!==this.N&&(this.N&&this.G.next(),this.N=a)}}va(){this.G.complete()}J(a=!1){this.jo.emit(new Tz(this,a))}};
Vz.K=function(a){return new (a||Vz)(z(si),z(Sr),z(rz,8),z(uz,8))};
Vz.Da=Gf({type:Vz,ha:[["mat-option"]],fb:function(a,b){a&1&&ur(Nz,7);if(a&2){let c;vr(c=wr())&&(b.MA=c.first)}},Ka:["role","option",1,"mat-mdc-option","mdc-list-item"],Za:11,Ia:function(a,b){a&1&&V("click",function(){return b.uh()})("keydown",function(c){return b.Se(c)});a&2&&(Qq("id",b.id),Kp("aria-selected",b.selected)("aria-disabled",b.disabled.toString()),nq("mdc-list-item--selected",b.selected)("mat-mdc-option-multiple",b.multiple)("mat-mdc-option-active",b.active)("mdc-list-item--disabled",
b.disabled))},inputs:{value:"value",id:"id",disabled:[2,"disabled","disabled",zs]},outputs:{jo:"onSelectionChange"},ib:["matOption"],ka:!0,features:[Jp,Mr],Zb:["mat-icon","*"],za:8,Aa:5,Ca:[["text",""],["aria-hidden","true",1,"mat-mdc-option-pseudo-checkbox",3,"disabled","state"],[1,"mdc-list-item__primary-text"],["state","checked","aria-hidden","true","appearance","minimal",1,"mat-mdc-option-pseudo-checkbox",3,"disabled"],[1,"cdk-visually-hidden"],["aria-hidden","true","mat-ripple","",1,"mat-mdc-option-ripple",
"mat-mdc-focus-indicator",3,"matRippleTrigger","matRippleDisabled"]],ta:function(a,b){a&1&&(Xq(Oz),Ap(0,Pz,1,2,"mat-pseudo-checkbox",1),Yq(1),R(2,"span",2,0),Yq(4,1),S(),Ap(5,Qz,1,1,"mat-pseudo-checkbox",3)(6,Rz,2,1,"span",4),Mq(7,"div",5));a&2&&(Eq(b.multiple?0:-1),D(5),Eq(b.multiple||!b.selected||b.Ae?-1:5),D(),Eq(b.group&&b.group.Am?6:-1),D(),H("matRippleTrigger",b.qe())("matRippleDisabled",b.disabled||b.Nc))},Ha:[Mz,Lz],styles:['.mat-mdc-option{display:flex;position:relative;align-items:center;justify-content:flex-start;overflow:hidden;padding:0;padding-left:16px;padding-right:16px;-webkit-user-select:none;user-select:none;-moz-osx-font-smoothing:grayscale;-webkit-font-smoothing:antialiased;cursor:pointer;-webkit-tap-highlight-color:rgba(0,0,0,0);color:var(--mat-option-label-text-color);font-family:var(--mat-option-label-text-font);line-height:var(--mat-option-label-text-line-height);font-size:var(--mat-option-label-text-size);letter-spacing:var(--mat-option-label-text-tracking);font-weight:var(--mat-option-label-text-weight);min-height:48px}.mat-mdc-option:focus{outline:none}[dir=rtl] .mat-mdc-option,.mat-mdc-option[dir=rtl]{padding-left:16px;padding-right:16px}.mat-mdc-option:hover:not(.mdc-list-item--disabled){background-color:var(--mat-option-hover-state-layer-color)}.mat-mdc-option:focus.mdc-list-item,.mat-mdc-option.mat-mdc-option-active.mdc-list-item{background-color:var(--mat-option-focus-state-layer-color)}.mat-mdc-option.mdc-list-item--selected:not(.mdc-list-item--disabled) .mdc-list-item__primary-text{color:var(--mat-option-selected-state-label-text-color)}.mat-mdc-option.mdc-list-item--selected:not(.mdc-list-item--disabled):not(.mat-mdc-option-multiple){background-color:var(--mat-option-selected-state-layer-color)}.mat-mdc-option.mdc-list-item{align-items:center;background:rgba(0,0,0,0)}.mat-mdc-option.mdc-list-item--disabled{cursor:default;pointer-events:none}.mat-mdc-option.mdc-list-item--disabled .mat-mdc-option-pseudo-checkbox,.mat-mdc-option.mdc-list-item--disabled .mdc-list-item__primary-text,.mat-mdc-option.mdc-list-item--disabled>mat-icon{opacity:.38}.mat-mdc-optgroup .mat-mdc-option:not(.mat-mdc-option-multiple){padding-left:32px}[dir=rtl] .mat-mdc-optgroup .mat-mdc-option:not(.mat-mdc-option-multiple){padding-left:16px;padding-right:32px}.mat-mdc-option .mat-icon,.mat-mdc-option .mat-pseudo-checkbox-full{margin-right:16px;flex-shrink:0}[dir=rtl] .mat-mdc-option .mat-icon,[dir=rtl] .mat-mdc-option .mat-pseudo-checkbox-full{margin-right:0;margin-left:16px}.mat-mdc-option .mat-pseudo-checkbox-minimal{margin-left:16px;flex-shrink:0}[dir=rtl] .mat-mdc-option .mat-pseudo-checkbox-minimal{margin-right:16px;margin-left:0}.mat-mdc-option .mat-mdc-option-ripple{top:0;left:0;right:0;bottom:0;position:absolute;pointer-events:none}.mat-mdc-option .mdc-list-item__primary-text{white-space:normal;font-size:inherit;font-weight:inherit;letter-spacing:inherit;line-height:inherit;font-family:inherit;text-decoration:inherit;text-transform:inherit;margin-right:auto}[dir=rtl] .mat-mdc-option .mdc-list-item__primary-text{margin-right:0;margin-left:auto}.cdk-high-contrast-active .mat-mdc-option.mdc-list-item--selected:not(.mat-mdc-option-multiple)::after{content:"";position:absolute;top:50%;right:16px;transform:translateY(-50%);width:10px;height:0;border-bottom:solid 10px;border-radius:10px}[dir=rtl] .cdk-high-contrast-active .mat-mdc-option.mdc-list-item--selected:not(.mat-mdc-option-multiple)::after{right:auto;left:16px}.mat-mdc-option-multiple{--mdc-list-list-item-selected-container-color:var(--mdc-list-list-item-container-color, transparent)}.mat-mdc-option-active .mat-mdc-focus-indicator::before{content:""}\n'],
pb:2,Ra:0});function Wz(a,b,c){if(c.length){b=b.toArray();c=c.toArray();let d=0;for(let e=0;e<a+1;e++)b[e].group&&b[e].group===c[d]&&d++;return d}return 0}function Xz(a,b,c,d){return a<c?a:a+b>c+d?Math.max(0,a-d+b):c};var Yz=class{};Yz.K=function(a){return new (a||Yz)};Yz.Ya=Of({type:Yz});Yz.Xa=Nd({imports:[mz,mz]});var Zz=class{};Zz.K=function(a){return new (a||Zz)};Zz.Ya=Of({type:Zz});Zz.Xa=Nd({imports:[mz]});var $z=class{};$z.K=function(a){return new (a||$z)};$z.Ya=Of({type:$z});$z.Xa=Nd({imports:[Yz,mz,Zz,Vz]});const aA={capture:!0},bA=["focus","mousedown","mouseenter","touchstart"];function cA(a,b){const c=a.C.get(b);c&&(c.va(),a.C.delete(b))}function dA(a,b,c){let d,e;b.setAttribute("mat-ripple-loader-uninitialized",null!=(e=null==(d=a.D)?void 0:d.namespace)?e:"");!c.className&&b.hasAttribute("mat-ripple-loader-class-name")||b.setAttribute("mat-ripple-loader-class-name",c.className||"");c.Wi&&b.setAttribute("mat-ripple-loader-centered","");c.disabled&&b.setAttribute("mat-ripple-loader-disabled","")}
function eA(a,b,c){b.removeAttribute("mat-ripple-loader-uninitialized");a.C.set(b,c)}
var fA=class{constructor(){this.ia=lf(jt,{optional:!0});this.ad=lf(mi,{optional:!0});this.D=lf(Kz,{optional:!0});this.Ma=lf(Vt);this.qa=lf(Qk);this.C=new Map;this.G=a=>{a=St(a);if(a instanceof HTMLElement){let b,c;(a=a.closest(`[${"mat-ripple-loader-uninitialized"}="${null!=(c=null==(b=this.D)?void 0:b.namespace)?c:""}"]`))&&this.F(a)}};Mk(this.qa,()=>{for(const a of bA){let b;null==(b=this.ia)||b.addEventListener(a,this.G,aA)}})}va(){const a=this.C.keys();for(const b of a)cA(this,b);for(const b of bA){let c;
null==(c=this.ia)||c.removeEventListener(b,this.G,aA)}}F(a){if(this.ia){var b=this.C.get(a);if(b)return b;var c;null==(c=a.querySelector(".mat-ripple"))||c.remove();b=this.ia.createElement("span");b.classList.add("mat-ripple",a.getAttribute("mat-ripple-loader-class-name"));a.append(b);b=new Lz(new si(b),this.qa,this.Ma,this.D?this.D:void 0,this.ad?this.ad:void 0);b.C=!0;b.Le=a;b.Wi=a.hasAttribute("mat-ripple-loader-centered");b.disabled=a.hasAttribute("mat-ripple-loader-disabled");eA(this,a,b);return b}}};
fA.K=function(a){return new (a||fA)};fA.pa=Md({ma:fA,ba:fA.K,ea:"root"});const gA=["panel"];function hA(a,b){if(a&1){const c=Ug();R(0,"div",1,0);V("@panelAnimation.done",function(d){u(c);const e=Y();return x(e.hg.next(d))});Yq(2);S()}a&2&&(a=b.id,b=Y(),qq(tq,b.Pe),nq("mat-mdc-autocomplete-visible",b.Zl)("mat-mdc-autocomplete-hidden",!b.Zl)("mat-primary","primary"===b.mh)("mat-accent","accent"===b.mh)("mat-warn","warn"===b.mh),H("id",b.id)("@panelAnimation",b.rw?"visible":"hidden"),Kp("aria-label",b.ariaLabel||null)("aria-labelledby",b.qp(a)))}let iA=0;
var jA=class{constructor(a,b){this.source=a;this.Pw=b}},lA=new df("mat-autocomplete-default-options",{ea:"root",ba:kA});function kA(){return{Sm:!1,Um:!1,Ae:!1,Kj:!1}}
var mA=class{get rw(){return this.D&&this.Zl}fa(a){this.mh=a;bn(this.Ba)}set classList(a){this.Pe=a;this.ra.R.className=""}get Ae(){return this.F}set Ae(a){this.F=a;if(this.options)for(const b of this.options)bn(b.Ba)}constructor(a,b,c,d){this.Ba=a;this.ra=b;this.P=c;this.G=Ni.EMPTY;this.hg=new Fk;this.D=this.Zl=!1;this.wq=null;this.Rw=new Fk;this.hf=new Fk;this.closed=new Fk;this.Qw=new Fk;this.id=`mat-autocomplete-${iA++}`;this.pC=(null==d?void 0:d.T)||!1;this.Sm=!!c.Sm;this.Um=!!c.Um;this.Kj=!!c.Kj;
let e;this.F=null!=(e=this.P.Ae)?e:!1}yj(){this.C=Jy(Ky(new Wy(this.options)),this.T);this.G=this.C.wb.subscribe(a=>{this.rw&&this.Qw.emit({source:this,Pw:this.options.toArray()[a]||null})});this.O()}va(){let a;null==(a=this.C)||a.destroy();this.G.unsubscribe();this.hg.complete()}N(a){this.Wf&&(this.Wf.R.scrollTop=a)}ca(){return this.Wf?this.Wf.R.scrollTop:0}O(){this.Zl=!!this.options.length;bn(this.Ba)}aa(a){this.Rw.emit(new jA(this,a))}qp(a){return this.ariaLabel?null:this.cd?(a?a+" ":"")+this.cd:
a}T(){return!1}};mA.K=function(a){return new (a||mA)(z(Sr),z(si),z(lA),z(Vt))};
mA.Da=Gf({type:mA,ha:[["mat-autocomplete"]],ec:function(a,b,c){a&1&&(tr(c,Vz,5),tr(c,uz,5));if(a&2){let d;vr(d=wr())&&(b.options=d);vr(d=wr())&&(b.Hr=d)}},fb:function(a,b){a&1&&(ur(br,7),ur(gA,5));if(a&2){let c;vr(c=wr())&&(b.ta=c.first);vr(c=wr())&&(b.Wf=c.first)}},Ka:[1,"mat-mdc-autocomplete"],inputs:{ariaLabel:[0,"aria-label","ariaLabel"],cd:[0,"aria-labelledby","ariaLabelledby"],wq:"displayWith",Sm:[2,"autoActiveFirstOption","autoActiveFirstOption",zs],Um:[2,"autoSelectActiveOption","autoSelectActiveOption",
zs],Kj:[2,"requireSelection","requireSelection",zs],Gg:"panelWidth",Nc:[2,"disableRipple","disableRipple",zs],classList:[0,"class","classList"],Ae:[2,"hideSingleSelectionIndicator","hideSingleSelectionIndicator",zs]},outputs:{Rw:"optionSelected",hf:"opened",closed:"closed",Qw:"optionActivated"},ib:["matAutocomplete"],ka:!0,features:[Kr([{oa:rz,Ab:mA}]),Jp,Mr],Zb:["*"],za:1,Aa:0,Ca:[["panel",""],["role","listbox",1,"mat-mdc-autocomplete-panel","mdc-menu-surface","mdc-menu-surface--open",3,"id"]],ta:function(a){a&
1&&(Xq(),Ap(0,hA,3,16,"ng-template"))},styles:["div.mat-mdc-autocomplete-panel{width:100%;max-height:256px;visibility:hidden;transform-origin:center top;overflow:auto;padding:8px 0;box-sizing:border-box;position:static;border-radius:var(--mat-autocomplete-container-shape);box-shadow:var(--mat-autocomplete-container-elevation-shadow);background-color:var(--mat-autocomplete-background-color)}.cdk-high-contrast-active div.mat-mdc-autocomplete-panel{outline:solid 1px}.cdk-overlay-pane:not(.mat-mdc-autocomplete-panel-above) div.mat-mdc-autocomplete-panel{border-top-left-radius:0;border-top-right-radius:0}.mat-mdc-autocomplete-panel-above div.mat-mdc-autocomplete-panel{border-bottom-left-radius:0;border-bottom-right-radius:0;transform-origin:center bottom}div.mat-mdc-autocomplete-panel.mat-mdc-autocomplete-visible{visibility:visible}div.mat-mdc-autocomplete-panel.mat-mdc-autocomplete-hidden{visibility:hidden;pointer-events:none}mat-autocomplete{display:none}\n"],
pb:2,data:{animation:[kz]},Ra:0});var nA=new df("MatError");let oA=0;var pA=class{constructor(){this.align="start";this.id=`mat-mdc-hint-${oA++}`}};pA.K=function(a){return new (a||pA)};pA.Fa=Qf({type:pA,ha:[["mat-hint"]],Ka:[1,"mat-mdc-form-field-hint","mat-mdc-form-field-bottom-align"],Za:4,Ia:function(a,b){a&2&&(Qq("id",b.id),Kp("align",null),nq("mat-mdc-form-field-hint-end","end"===b.align))},inputs:{align:"align",id:"id"},ka:!0});var qA=class{};qA.K=function(a){return new (a||qA)};qA.Fa=Qf({type:qA,ha:[["mat-label"]],ka:!0});var rA=new df("MatPrefix");var sA=new df("MatSuffix");var tA=Yy("transitionMessages",[az("enter",$y({opacity:1,transform:"translateY(0%)"})),bz("void => enter",[$y({opacity:0,transform:"translateY(-5px)"}),Zy("300ms cubic-bezier(0.55, 0, 0.55, 0.2)")])]);var uA=class{};uA.K=function(a){return new (a||uA)};uA.Fa=Qf({type:uA});class vA{constructor(a){this.G=a;this.Ga=new cj;this.D=new cj;this.C=new Map;"undefined"!==typeof ResizeObserver&&(this.F=new ResizeObserver(b=>this.D.next(b)))}observe(a){this.C.has(a)||this.C.set(a,C(new Xi(b=>{const c=this.D.subscribe(b);let d;null==(d=this.F)||d.observe(a,{box:this.G});return()=>{let e;null==(e=this.F)||e.unobserve(a);c.unsubscribe();this.C.delete(a)}}),tk(b=>b.some(c=>c.target===a)),Co(),Lo(this.Ga)));return this.C.get(a)}destroy(){this.Ga.next();this.Ga.complete();this.D.complete();
this.C.clear()}}var wA=class{constructor(){this.C=new Map;this.qa=lf(Qk)}va(){for(const [,a]of this.C)a.destroy();this.C.clear()}observe(a,b){b=(null==b?void 0:b.box)||"content-box";this.C.has(b)||this.C.set(b,new vA(b));return this.C.get(b).observe(a)}};wA.K=function(a){return new (a||wA)};wA.pa=Md({ma:wA,ba:wA.K,ea:"root"});var xA=new df("FloatingLabelParent"),yA=class{get Gq(){return this.F}set Gq(a){this.F=a;this.qr&&this.G()}get qr(){return this.D}set qr(a){(this.D=a)?this.N():this.C.unsubscribe()}constructor(a){this.ra=a;this.D=this.F=!1;this.J=lf(wA);this.qa=lf(Qk);this.Qa=lf(xA);this.C=new Ni}va(){this.C.unsubscribe()}get element(){return this.ra.R}G(){setTimeout(()=>this.Qa.Yz())}N(){this.C.unsubscribe();Mk(this.qa,()=>{this.C=this.J.observe(this.ra.R,{box:"border-box"}).subscribe(()=>this.G())})}};
yA.K=function(a){return new (a||yA)(z(si))};yA.Fa=Qf({type:yA,ha:[["label","matFormFieldFloatingLabel",""]],Ka:[1,"mdc-floating-label","mat-mdc-floating-label"],Za:2,Ia:function(a,b){a&2&&nq("mdc-floating-label--float-above",b.Gq)},inputs:{Gq:"floating",qr:"monitorResize"},ka:!0});var zA=class{constructor(a,b){this.ra=a;this.C=c=>{const d=this.ra.R.classList,e=d.contains("mdc-line-ripple--deactivating");"opacity"===c.propertyName&&e&&d.remove("mdc-line-ripple--active","mdc-line-ripple--deactivating")};Mk(b,()=>{a.R.addEventListener("transitionend",this.C)})}jn(){this.ra.R.classList.add("mdc-line-ripple--deactivating")}va(){this.ra.R.removeEventListener("transitionend",this.C)}};zA.K=function(a){return new (a||zA)(z(si),z(Qk))};
zA.Fa=Qf({type:zA,ha:[["div","matFormFieldLineRipple",""]],Ka:[1,"mdc-line-ripple"],ka:!0});const AA=["notch"];
var BA=class{constructor(a,b){this.ra=a;this.qa=b;this.open=!1}pd(){const a=this.ra.R.querySelector(".mdc-floating-label");a?(this.ra.R.classList.add("mdc-notched-outline--upgraded"),"function"===typeof requestAnimationFrame&&(a.style.transitionDuration="0s",Mk(this.qa,()=>{requestAnimationFrame(()=>a.style.transitionDuration="")}))):this.ra.R.classList.add("mdc-notched-outline--no-label")}C(a){this.jA.R.style.width=this.open&&a?`calc(${a}px * var(--mat-mdc-form-field-floating-label-scale, 0.75) + ${9}px)`:""}};
BA.K=function(a){return new (a||BA)(z(si),z(Qk))};
BA.Da=Gf({type:BA,ha:[["div","matFormFieldNotchedOutline",""]],fb:function(a,b){a&1&&ur(AA,5);if(a&2){let c;vr(c=wr())&&(b.jA=c.first)}},Ka:[1,"mdc-notched-outline"],Za:2,Ia:function(a,b){a&2&&nq("mdc-notched-outline--notched",b.open)},inputs:{open:[0,"matFormFieldNotchedOutlineOpen","open"]},ka:!0,features:[Mr],attrs:["matFormFieldNotchedOutline",""],Zb:["*"],za:5,Aa:0,Ca:[["notch",""],[1,"mdc-notched-outline__leading"],[1,"mdc-notched-outline__notch"],[1,"mdc-notched-outline__trailing"]],ta:function(a){a&
1&&(Xq(),Mq(0,"div",1),R(1,"div",2,0),Yq(3),S(),Mq(4,"div",3))},pb:2,Ra:0});const CA=["textField"],DA=["iconPrefixContainer"],EA=["textPrefixContainer"],FA=["*",[["mat-label"]],[["","matPrefix",""],["","matIconPrefix",""]],[["","matTextPrefix",""]],[["","matTextSuffix",""]],[["","matSuffix",""],["","matIconSuffix",""]],[["mat-error"],["","matError",""]],[["mat-hint",3,"align","end"]],[["mat-hint","align","end"]]];function GA(a){a&1&&Mq(0,"span",19)}
function HA(a){a&1&&(R(0,"label",18),Yq(1,1),Ap(2,GA,1,0,"span",19),S());a&2&&(a=Y(2),H("floating",a.Bk())("monitorResize",a.zf())("id",a.Ue),Kp("for",a.Hb.zB?null:a.Hb.id),D(2),Eq(!a.Kn&&a.Hb.required?2:-1))}function IA(a){a&1&&Ap(0,HA,3,5,"label",18);a&2&&(a=Y(),Eq(a.ph()?0:-1))}function JA(a){a&1&&Mq(0,"div",5)}function KA(){}function LA(a){a&1&&Ap(0,KA,0,0,"ng-template",11);a&2&&(Y(2),a=zr(1),H("ngTemplateOutlet",a))}
function MA(a){a&1&&(R(0,"div",7),Ap(1,LA,1,1,null,11),S());a&2&&(a=Y(),H("matFormFieldNotchedOutlineOpen",a.Bk()),D(),Eq(a.At()?-1:1))}function NA(a){a&1&&(R(0,"div",8,2),Yq(2,2),S())}function OA(a){a&1&&(R(0,"div",9,3),Yq(2,3),S())}function PA(){}function QA(a){a&1&&Ap(0,PA,0,0,"ng-template",11);a&2&&(Y(),a=zr(1),H("ngTemplateOutlet",a))}function RA(a){a&1&&(R(0,"div",12),Yq(1,4),S())}function SA(a){a&1&&(R(0,"div",13),Yq(1,5),S())}function TA(a){a&1&&Mq(0,"div",14)}
function UA(a){a&1&&(R(0,"div",16),Yq(1,6),S());a&2&&(a=Y(),H("@transitionMessages",a.Kp))}function VA(a){a&1&&(R(0,"mat-hint",20),Z(1),S());a&2&&(a=Y(2),H("id",a.Qt),D(),Ar(a.Mn))}function WA(a){a&1&&(R(0,"div",17),Ap(1,VA,2,2,"mat-hint",20),Yq(2,7),Mq(3,"div",21),Yq(4,8),S());a&2&&(a=Y(),H("@transitionMessages",a.Kp),D(),Eq(a.Mn?1:-1))}var XA=new df("MatFormField"),YA=new df("MAT_FORM_FIELD_DEFAULT_OPTIONS");let ZA=0;function $A(a){return a.ru||a.ra}
var bB=class{get Kn(){return this.N}set Kn(a){this.N=Qs(a)}get wg(){let a;return this.T||(null==(a=this.G)?void 0:a.wg)||"auto"}set wg(a){a!==this.T&&(this.T=a,bn(this.Ba))}get Fd(){return this.D}set Fd(a){const b=this.D;let c;this.D=a||(null==(c=this.G)?void 0:c.Fd)||"fill";"outline"===this.D&&this.D!==b&&(this.C=!0)}get dm(){let a;return this.fa||(null==(a=this.G)?void 0:a.dm)||"fixed"}set dm(a){let b;this.fa=a||(null==(b=this.G)?void 0:b.dm)||"fixed"}get Mn(){return this.O}set Mn(a){this.O=a;this.ca()}get Hb(){return this.sa||
this.Qz}set Hb(a){this.sa=a}constructor(a,b,c,d,e,f,g){this.ra=a;this.Ba=b;this.P=d;this.Ma=e;this.G=f;this.ad=g;this.N=!1;this.color="primary";this.D="fill";this.fa=null;this.O="";this.Ot=this.vp=this.Nt=this.tp=!1;this.Ue=`mat-mdc-form-field-label-${ZA++}`;this.Qt=`mat-mdc-hint-${ZA++}`;this.Kp="";this.Ga=new cj;this.F=null;this.C=!1;this.Ua=lf(Wh);f&&(f.Fd&&(this.Fd=f.Fd),this.N=!(null==f||!f.Kn),f.color&&(this.color=f.color))}pd(){this.ja();this.Kp="enter";cn(this.Ba)}yj(){this.ua();this.Ja();
this.Ea();this.wa()}Kw(){}va(){this.Ga.next();this.Ga.complete()}na(){this.ph()&&(this.wg="always")}ua(){const a=this.Hb;a.jv&&this.ra.R.classList.add(`mat-mdc-form-field-type-${a.jv}`);a.Tc.subscribe(()=>{this.ja();this.J();bn(this.Ba)});a.Jd&&a.Jd.ah&&C(a.Jd.ah,Lo(this.Ga)).subscribe(()=>bn(this.Ba))}aa(){this.tp=!!this.yk.find(a=>!a.C);this.Nt=!!this.yk.find(a=>a.C);this.vp=!!this.Lp.find(a=>!a.C);this.Ot=!!this.Lp.find(a=>a.C)}Ea(){this.aa();rk(this.yk.C,this.Lp.C).subscribe(()=>{this.aa();bn(this.Ba)})}Ja(){this.rk.C.subscribe(()=>
{this.ca();bn(this.Ba)});this.nk.C.subscribe(()=>{this.J();bn(this.Ba)});this.J()}bb(){}ja(){if(this.Hb.focused&&!this.F){this.F=!0;var a;null!=(a=this.bu)&&(a=a.ra.R.classList,a.remove("mdc-line-ripple--deactivating"),a.add("mdc-line-ripple--active"))}else this.Hb.focused||!this.F&&null!==this.F||(this.F=!1,null==(a=this.bu)||a.jn());let b;null==(b=this.ru)||b.R.classList.toggle("mdc-text-field--focused",this.Hb.focused)}wa(){this.yk.C.subscribe(()=>this.C=!0);Xk(()=>{this.C&&(this.C=!1,this.Va())},
{xa:this.Ua});C(this.P.wb,Lo(this.Ga)).subscribe(()=>this.C=!0)}nu(){return"always"===this.wg}zf(){return"outline"===this.Fd}At(){return!this.Ma.isBrowser&&this.yk.length&&!this.Bk()}ph(){return!!this.gA||!!this.hA}Bk(){return this.Hb.VD||this.nu()}vh(a){const b=this.Hb?this.Hb.Jd:null;return b&&b[a]}Ft(){return this.nk&&0<this.nk.length&&this.Hb.xe?"error":"hint"}Yz(){this.Sa()}Sa(){if(this.zf()&&this.ym&&this.Bk()){var a;if(null!=(a=this.cu)){var b=a.C;var c=this.ym.ra.R;if(null!==c.offsetParent)c=
c.scrollWidth;else{c=c.cloneNode(!0);c.style.setProperty("position","absolute");c.style.setProperty("transform","translate(-9999px, -9999px)");document.documentElement.appendChild(c);var d=c.scrollWidth;c.remove();c=d}b.call(a,c)}}else null==(a=this.cu)||a.C(0)}ca(){this.J()}gb(){}J(){if(this.Hb){let a=[];this.Hb.Ms&&"string"===typeof this.Hb.Ms&&a.push(...this.Hb.Ms.split(" "));if("hint"===this.Ft()){const b=this.rk?this.rk.find(d=>"start"===d.align):null,c=this.rk?this.rk.find(d=>"end"===d.align):
null;b?a.push(b.id):this.O&&a.push(this.Qt);c&&a.push(c.id)}else this.nk&&a.push(...this.nk.map(b=>b.id));aB(this.Hb,a)}}Va(){if(this.zf()&&this.ym){var a=this.ym.element;if(this.Rt||this.su)if(this.Pa()){var b,c=null==(b=this.Rt)?void 0:b.R,d;b=null==(d=this.su)?void 0:d.R;var e;c=null!=(e=null==c?void 0:c.getBoundingClientRect().width)?e:0;var f;e=null!=(f=null==b?void 0:b.getBoundingClientRect().width)?f:0;a.style.transform=`var(
        --mat-mdc-form-field-label-transform,
        ${"translateY(-50%)"} translateX(${`calc(${"rtl"===this.P.value?"-1":"1"} * (${`${c+e}px`} + ${"var(--mat-mdc-form-field-label-offset-x, 0px)"}))`})
    )`}else this.C=!0;else a.style.transform=""}}Pa(){const a=this.ra.R;if(a.getRootNode){const b=a.getRootNode();return b&&b!==a}return document.documentElement.contains(a)}};bB.K=function(a){return new (a||bB)(z(si),z(Sr),z(Qk),z(ou),z(Vt),z(YA,8),z(mi,8),z(jt))};
bB.Da=Gf({type:bB,ha:[["mat-form-field"]],ec:function(a,b,c){a&1&&(tr(c,qA,5),tr(c,qA,7),tr(c,uA,5),tr(c,rA,5),tr(c,sA,5),tr(c,nA,5),tr(c,pA,5));if(a&2){let d;vr(d=wr())&&(b.gA=d.first);vr(d=wr())&&(b.hA=d.first);vr(d=wr())&&(b.Qz=d.first);vr(d=wr())&&(b.yk=d);vr(d=wr())&&(b.Lp=d);vr(d=wr())&&(b.nk=d);vr(d=wr())&&(b.rk=d)}},fb:function(a,b){a&1&&(ur(CA,5),ur(DA,5),ur(EA,5),ur(yA,5),ur(BA,5),ur(zA,5));if(a&2){let c;vr(c=wr())&&(b.ru=c.first);vr(c=wr())&&(b.Rt=c.first);vr(c=wr())&&(b.su=c.first);vr(c=
wr())&&(b.ym=c.first);vr(c=wr())&&(b.cu=c.first);vr(c=wr())&&(b.bu=c.first)}},Ka:[1,"mat-mdc-form-field"],Za:42,Ia:function(a,b){a&2&&nq("mat-mdc-form-field-label-always-float",b.nu())("mat-mdc-form-field-has-icon-prefix",b.tp)("mat-mdc-form-field-has-icon-suffix",b.vp)("mat-form-field-invalid",b.Hb.xe)("mat-form-field-disabled",b.Hb.disabled)("mat-form-field-autofilled",b.Hb.zF)("mat-form-field-no-animations","NoopAnimations"===b.ad)("mat-form-field-appearance-fill","fill"==b.Fd)("mat-form-field-appearance-outline",
"outline"==b.Fd)("mat-form-field-hide-placeholder",b.ph()&&!b.Bk())("mat-focused",b.Hb.focused)("mat-primary","accent"!==b.color&&"warn"!==b.color)("mat-accent","accent"===b.color)("mat-warn","warn"===b.color)("ng-untouched",b.vh("untouched"))("ng-touched",b.vh("touched"))("ng-pristine",b.vh("pristine"))("ng-dirty",b.vh("dirty"))("ng-valid",b.vh("valid"))("ng-invalid",b.vh("invalid"))("ng-pending",b.vh("pending"))},inputs:{Kn:"hideRequiredMarker",color:"color",wg:"floatLabel",Fd:"appearance",dm:"subscriptSizing",
Mn:"hintLabel"},ib:["matFormField"],ka:!0,features:[Kr([{oa:XA,Ab:bB},{oa:xA,Ab:bB}]),Mr],Zb:"*;mat-label;[matPrefix], [matIconPrefix];[matTextPrefix];[matTextSuffix];[matSuffix], [matIconSuffix];mat-error, [matError];mat-hint:not([align='end']);mat-hint[align='end']".split(";"),za:18,Aa:21,Ca:[["labelTemplate",""],["textField",""],["iconPrefixContainer",""],["textPrefixContainer",""],[1,"mat-mdc-text-field-wrapper","mdc-text-field",3,"click"],[1,"mat-mdc-form-field-focus-overlay"],[1,"mat-mdc-form-field-flex"],
["matFormFieldNotchedOutline","",3,"matFormFieldNotchedOutlineOpen"],[1,"mat-mdc-form-field-icon-prefix"],[1,"mat-mdc-form-field-text-prefix"],[1,"mat-mdc-form-field-infix"],[3,"ngTemplateOutlet"],[1,"mat-mdc-form-field-text-suffix"],[1,"mat-mdc-form-field-icon-suffix"],["matFormFieldLineRipple",""],[1,"mat-mdc-form-field-subscript-wrapper","mat-mdc-form-field-bottom-align"],[1,"mat-mdc-form-field-error-wrapper"],[1,"mat-mdc-form-field-hint-wrapper"],["matFormFieldFloatingLabel","",3,"floating","monitorResize",
"id"],["aria-hidden","true",1,"mat-mdc-form-field-required-marker","mdc-floating-label--required"],[3,"id"],[1,"mat-mdc-form-field-hint-spacer"]],ta:function(a,b){if(a&1){const c=Ug();Xq(FA);Ap(0,IA,1,1,"ng-template",null,0,Qr);R(2,"div",4,1);V("click",function(){u(c);var d=b.Hb;d.focus();d.open();return x()});Ap(4,JA,1,0,"div",5);R(5,"div",6);Ap(6,MA,2,2,"div",7)(7,NA,3,0,"div",8)(8,OA,3,0,"div",9);R(9,"div",10);Ap(10,QA,1,1,null,11);Yq(11);S();Ap(12,RA,2,0,"div",12)(13,SA,2,0,"div",13);S();Ap(14,
TA,1,0,"div",14);S();R(15,"div",15);Ap(16,UA,2,1,"div",16)(17,WA,5,2,"div",17);S()}if(a&2){let c;D(2);nq("mdc-text-field--filled",!b.zf())("mdc-text-field--outlined",b.zf())("mdc-text-field--no-label",!b.ph())("mdc-text-field--disabled",b.Hb.disabled)("mdc-text-field--invalid",b.Hb.xe);D(2);Eq(b.zf()||b.Hb.disabled?-1:4);D(2);Eq(b.zf()?6:-1);D();Eq(b.tp?7:-1);D();Eq(b.Nt?8:-1);D(2);Eq(!b.zf()||b.At()?10:-1);D(2);Eq(b.Ot?12:-1);D();Eq(b.vp?13:-1);D();Eq(b.zf()?-1:14);D();nq("mat-mdc-form-field-subscript-dynamic-size",
"dynamic"===b.dm);D();Eq("error"===(c=b.Ft())?16:"hint"===c?17:-1)}},Ha:[yA,BA,gt,zA,pA],styles:['.mdc-text-field{border-top-left-radius:4px;border-top-left-radius:var(--mdc-shape-small, 4px);border-top-right-radius:4px;border-top-right-radius:var(--mdc-shape-small, 4px);border-bottom-right-radius:0;border-bottom-left-radius:0;display:inline-flex;align-items:baseline;padding:0 16px;position:relative;box-sizing:border-box;overflow:hidden;will-change:opacity,transform,color}.mdc-text-field .mdc-floating-label{top:50%;transform:translateY(-50%);pointer-events:none}.mdc-text-field__input{height:28px;width:100%;min-width:0;border:none;border-radius:0;background:none;appearance:none;padding:0}.mdc-text-field__input::-ms-clear{display:none}.mdc-text-field__input::-webkit-calendar-picker-indicator{display:none}.mdc-text-field__input:focus{outline:none}.mdc-text-field__input:invalid{box-shadow:none}@media all{.mdc-text-field__input::placeholder{opacity:0}}@media all{.mdc-text-field__input:-ms-input-placeholder{opacity:0}}@media all{.mdc-text-field--no-label .mdc-text-field__input::placeholder,.mdc-text-field--focused .mdc-text-field__input::placeholder{opacity:1}}@media all{.mdc-text-field--no-label .mdc-text-field__input:-ms-input-placeholder,.mdc-text-field--focused .mdc-text-field__input:-ms-input-placeholder{opacity:1}}.mdc-text-field__affix{height:28px;opacity:0;white-space:nowrap}.mdc-text-field--label-floating .mdc-text-field__affix,.mdc-text-field--no-label .mdc-text-field__affix{opacity:1}@supports(-webkit-hyphens: none){.mdc-text-field--outlined .mdc-text-field__affix{align-items:center;align-self:center;display:inline-flex;height:100%}}.mdc-text-field__affix--prefix{padding-left:0;padding-right:2px}[dir=rtl] .mdc-text-field__affix--prefix,.mdc-text-field__affix--prefix[dir=rtl]{padding-left:2px;padding-right:0}.mdc-text-field--end-aligned .mdc-text-field__affix--prefix{padding-left:0;padding-right:12px}[dir=rtl] .mdc-text-field--end-aligned .mdc-text-field__affix--prefix,.mdc-text-field--end-aligned .mdc-text-field__affix--prefix[dir=rtl]{padding-left:12px;padding-right:0}.mdc-text-field__affix--suffix{padding-left:12px;padding-right:0}[dir=rtl] .mdc-text-field__affix--suffix,.mdc-text-field__affix--suffix[dir=rtl]{padding-left:0;padding-right:12px}.mdc-text-field--end-aligned .mdc-text-field__affix--suffix{padding-left:2px;padding-right:0}[dir=rtl] .mdc-text-field--end-aligned .mdc-text-field__affix--suffix,.mdc-text-field--end-aligned .mdc-text-field__affix--suffix[dir=rtl]{padding-left:0;padding-right:2px}.mdc-text-field--filled{height:56px}.mdc-text-field--filled::before{display:inline-block;width:0;height:40px;content:"";vertical-align:0}.mdc-text-field--filled .mdc-floating-label{left:16px;right:initial}[dir=rtl] .mdc-text-field--filled .mdc-floating-label,.mdc-text-field--filled .mdc-floating-label[dir=rtl]{left:initial;right:16px}.mdc-text-field--filled .mdc-floating-label--float-above{transform:translateY(-106%) scale(0.75)}.mdc-text-field--filled.mdc-text-field--no-label .mdc-text-field__input{height:100%}.mdc-text-field--filled.mdc-text-field--no-label .mdc-floating-label{display:none}.mdc-text-field--filled.mdc-text-field--no-label::before{display:none}@supports(-webkit-hyphens: none){.mdc-text-field--filled.mdc-text-field--no-label .mdc-text-field__affix{align-items:center;align-self:center;display:inline-flex;height:100%}}.mdc-text-field--outlined{height:56px;overflow:visible}.mdc-text-field--outlined .mdc-floating-label--float-above{transform:translateY(-37.25px) scale(1)}.mdc-text-field--outlined.mdc-notched-outline--upgraded .mdc-floating-label--float-above,.mdc-text-field--outlined .mdc-notched-outline--upgraded .mdc-floating-label--float-above{transform:translateY(-34.75px) scale(0.75)}.mdc-text-field--outlined .mdc-floating-label--float-above{font-size:.75rem}.mdc-text-field--outlined.mdc-notched-outline--upgraded .mdc-floating-label--float-above,.mdc-text-field--outlined .mdc-notched-outline--upgraded .mdc-floating-label--float-above{font-size:1rem}.mdc-text-field--outlined .mdc-text-field__input{height:100%}.mdc-text-field--outlined .mdc-notched-outline .mdc-notched-outline__leading{border-top-left-radius:4px;border-top-left-radius:var(--mdc-shape-small, 4px);border-top-right-radius:0;border-bottom-right-radius:0;border-bottom-left-radius:4px;border-bottom-left-radius:var(--mdc-shape-small, 4px)}[dir=rtl] .mdc-text-field--outlined .mdc-notched-outline .mdc-notched-outline__leading,.mdc-text-field--outlined .mdc-notched-outline .mdc-notched-outline__leading[dir=rtl]{border-top-left-radius:0;border-top-right-radius:4px;border-top-right-radius:var(--mdc-shape-small, 4px);border-bottom-right-radius:4px;border-bottom-right-radius:var(--mdc-shape-small, 4px);border-bottom-left-radius:0}@supports(top: max(0%)){.mdc-text-field--outlined .mdc-notched-outline .mdc-notched-outline__leading{width:max(12px,var(--mdc-shape-small, 4px))}}@supports(top: max(0%)){.mdc-text-field--outlined .mdc-notched-outline .mdc-notched-outline__notch{max-width:calc(100% - max(12px,var(--mdc-shape-small, 4px))*2)}}.mdc-text-field--outlined .mdc-notched-outline .mdc-notched-outline__trailing{border-top-left-radius:0;border-top-right-radius:4px;border-top-right-radius:var(--mdc-shape-small, 4px);border-bottom-right-radius:4px;border-bottom-right-radius:var(--mdc-shape-small, 4px);border-bottom-left-radius:0}[dir=rtl] .mdc-text-field--outlined .mdc-notched-outline .mdc-notched-outline__trailing,.mdc-text-field--outlined .mdc-notched-outline .mdc-notched-outline__trailing[dir=rtl]{border-top-left-radius:4px;border-top-left-radius:var(--mdc-shape-small, 4px);border-top-right-radius:0;border-bottom-right-radius:0;border-bottom-left-radius:4px;border-bottom-left-radius:var(--mdc-shape-small, 4px)}@supports(top: max(0%)){.mdc-text-field--outlined{padding-left:max(16px,calc(var(--mdc-shape-small, 4px) + 4px))}}@supports(top: max(0%)){.mdc-text-field--outlined{padding-right:max(16px,var(--mdc-shape-small, 4px))}}@supports(top: max(0%)){.mdc-text-field--outlined+.mdc-text-field-helper-line{padding-left:max(16px,calc(var(--mdc-shape-small, 4px) + 4px))}}@supports(top: max(0%)){.mdc-text-field--outlined+.mdc-text-field-helper-line{padding-right:max(16px,var(--mdc-shape-small, 4px))}}.mdc-text-field--outlined.mdc-text-field--with-leading-icon{padding-left:0}@supports(top: max(0%)){.mdc-text-field--outlined.mdc-text-field--with-leading-icon{padding-right:max(16px,var(--mdc-shape-small, 4px))}}[dir=rtl] .mdc-text-field--outlined.mdc-text-field--with-leading-icon,.mdc-text-field--outlined.mdc-text-field--with-leading-icon[dir=rtl]{padding-right:0}@supports(top: max(0%)){[dir=rtl] .mdc-text-field--outlined.mdc-text-field--with-leading-icon,.mdc-text-field--outlined.mdc-text-field--with-leading-icon[dir=rtl]{padding-left:max(16px,var(--mdc-shape-small, 4px))}}.mdc-text-field--outlined.mdc-text-field--with-trailing-icon{padding-right:0}@supports(top: max(0%)){.mdc-text-field--outlined.mdc-text-field--with-trailing-icon{padding-left:max(16px,calc(var(--mdc-shape-small, 4px) + 4px))}}[dir=rtl] .mdc-text-field--outlined.mdc-text-field--with-trailing-icon,.mdc-text-field--outlined.mdc-text-field--with-trailing-icon[dir=rtl]{padding-left:0}@supports(top: max(0%)){[dir=rtl] .mdc-text-field--outlined.mdc-text-field--with-trailing-icon,.mdc-text-field--outlined.mdc-text-field--with-trailing-icon[dir=rtl]{padding-right:max(16px,calc(var(--mdc-shape-small, 4px) + 4px))}}.mdc-text-field--outlined.mdc-text-field--with-leading-icon.mdc-text-field--with-trailing-icon{padding-left:0;padding-right:0}.mdc-text-field--outlined .mdc-notched-outline--notched .mdc-notched-outline__notch{padding-top:1px}.mdc-text-field--outlined .mdc-floating-label{left:4px;right:initial}[dir=rtl] .mdc-text-field--outlined .mdc-floating-label,.mdc-text-field--outlined .mdc-floating-label[dir=rtl]{left:initial;right:4px}.mdc-text-field--outlined .mdc-text-field__input{display:flex;border:none !important;background-color:rgba(0,0,0,0)}.mdc-text-field--outlined .mdc-notched-outline{z-index:1}.mdc-text-field--textarea{flex-direction:column;align-items:center;width:auto;height:auto;padding:0}.mdc-text-field--textarea .mdc-floating-label{top:19px}.mdc-text-field--textarea .mdc-floating-label:not(.mdc-floating-label--float-above){transform:none}.mdc-text-field--textarea .mdc-text-field__input{flex-grow:1;height:auto;min-height:1.5rem;overflow-x:hidden;overflow-y:auto;box-sizing:border-box;resize:none;padding:0 16px}.mdc-text-field--textarea.mdc-text-field--filled::before{display:none}.mdc-text-field--textarea.mdc-text-field--filled .mdc-floating-label--float-above{transform:translateY(-10.25px) scale(0.75)}.mdc-text-field--textarea.mdc-text-field--filled .mdc-text-field__input{margin-top:23px;margin-bottom:9px}.mdc-text-field--textarea.mdc-text-field--filled.mdc-text-field--no-label .mdc-text-field__input{margin-top:16px;margin-bottom:16px}.mdc-text-field--textarea.mdc-text-field--outlined .mdc-notched-outline--notched .mdc-notched-outline__notch{padding-top:0}.mdc-text-field--textarea.mdc-text-field--outlined .mdc-floating-label--float-above{transform:translateY(-27.25px) scale(1)}.mdc-text-field--textarea.mdc-text-field--outlined.mdc-notched-outline--upgraded .mdc-floating-label--float-above,.mdc-text-field--textarea.mdc-text-field--outlined .mdc-notched-outline--upgraded .mdc-floating-label--float-above{transform:translateY(-24.75px) scale(0.75)}.mdc-text-field--textarea.mdc-text-field--outlined .mdc-floating-label--float-above{font-size:.75rem}.mdc-text-field--textarea.mdc-text-field--outlined.mdc-notched-outline--upgraded .mdc-floating-label--float-above,.mdc-text-field--textarea.mdc-text-field--outlined .mdc-notched-outline--upgraded .mdc-floating-label--float-above{font-size:1rem}.mdc-text-field--textarea.mdc-text-field--outlined .mdc-text-field__input{margin-top:16px;margin-bottom:16px}.mdc-text-field--textarea.mdc-text-field--outlined .mdc-floating-label{top:18px}.mdc-text-field--textarea.mdc-text-field--with-internal-counter .mdc-text-field__input{margin-bottom:2px}.mdc-text-field--textarea.mdc-text-field--with-internal-counter .mdc-text-field-character-counter{align-self:flex-end;padding:0 16px}.mdc-text-field--textarea.mdc-text-field--with-internal-counter .mdc-text-field-character-counter::after{display:inline-block;width:0;height:16px;content:"";vertical-align:-16px}.mdc-text-field--textarea.mdc-text-field--with-internal-counter .mdc-text-field-character-counter::before{display:none}.mdc-text-field__resizer{align-self:stretch;display:inline-flex;flex-direction:column;flex-grow:1;max-height:100%;max-width:100%;min-height:56px;min-width:fit-content;min-width:-moz-available;min-width:-webkit-fill-available;overflow:hidden;resize:both}.mdc-text-field--filled .mdc-text-field__resizer{transform:translateY(-1px)}.mdc-text-field--filled .mdc-text-field__resizer .mdc-text-field__input,.mdc-text-field--filled .mdc-text-field__resizer .mdc-text-field-character-counter{transform:translateY(1px)}.mdc-text-field--outlined .mdc-text-field__resizer{transform:translateX(-1px) translateY(-1px)}[dir=rtl] .mdc-text-field--outlined .mdc-text-field__resizer,.mdc-text-field--outlined .mdc-text-field__resizer[dir=rtl]{transform:translateX(1px) translateY(-1px)}.mdc-text-field--outlined .mdc-text-field__resizer .mdc-text-field__input,.mdc-text-field--outlined .mdc-text-field__resizer .mdc-text-field-character-counter{transform:translateX(1px) translateY(1px)}[dir=rtl] .mdc-text-field--outlined .mdc-text-field__resizer .mdc-text-field__input,[dir=rtl] .mdc-text-field--outlined .mdc-text-field__resizer .mdc-text-field-character-counter,.mdc-text-field--outlined .mdc-text-field__resizer .mdc-text-field__input[dir=rtl],.mdc-text-field--outlined .mdc-text-field__resizer .mdc-text-field-character-counter[dir=rtl]{transform:translateX(-1px) translateY(1px)}.mdc-text-field--with-leading-icon{padding-left:0;padding-right:16px}[dir=rtl] .mdc-text-field--with-leading-icon,.mdc-text-field--with-leading-icon[dir=rtl]{padding-left:16px;padding-right:0}.mdc-text-field--with-leading-icon.mdc-text-field--filled .mdc-floating-label{max-width:calc(100% - 48px);left:48px;right:initial}[dir=rtl] .mdc-text-field--with-leading-icon.mdc-text-field--filled .mdc-floating-label,.mdc-text-field--with-leading-icon.mdc-text-field--filled .mdc-floating-label[dir=rtl]{left:initial;right:48px}.mdc-text-field--with-leading-icon.mdc-text-field--filled .mdc-floating-label--float-above{max-width:calc(100%/0.75 - 64px/0.75)}.mdc-text-field--with-leading-icon.mdc-text-field--outlined .mdc-floating-label{left:36px;right:initial}[dir=rtl] .mdc-text-field--with-leading-icon.mdc-text-field--outlined .mdc-floating-label,.mdc-text-field--with-leading-icon.mdc-text-field--outlined .mdc-floating-label[dir=rtl]{left:initial;right:36px}.mdc-text-field--with-leading-icon.mdc-text-field--outlined :not(.mdc-notched-outline--notched) .mdc-notched-outline__notch{max-width:calc(100% - 60px)}.mdc-text-field--with-leading-icon.mdc-text-field--outlined .mdc-floating-label--float-above{transform:translateY(-37.25px) translateX(-32px) scale(1)}[dir=rtl] .mdc-text-field--with-leading-icon.mdc-text-field--outlined .mdc-floating-label--float-above,.mdc-text-field--with-leading-icon.mdc-text-field--outlined .mdc-floating-label--float-above[dir=rtl]{transform:translateY(-37.25px) translateX(32px) scale(1)}.mdc-text-field--with-leading-icon.mdc-text-field--outlined.mdc-notched-outline--upgraded .mdc-floating-label--float-above,.mdc-text-field--with-leading-icon.mdc-text-field--outlined .mdc-notched-outline--upgraded .mdc-floating-label--float-above{transform:translateY(-34.75px) translateX(-32px) scale(0.75)}[dir=rtl] .mdc-text-field--with-leading-icon.mdc-text-field--outlined.mdc-notched-outline--upgraded .mdc-floating-label--float-above,[dir=rtl] .mdc-text-field--with-leading-icon.mdc-text-field--outlined .mdc-notched-outline--upgraded .mdc-floating-label--float-above,.mdc-text-field--with-leading-icon.mdc-text-field--outlined.mdc-notched-outline--upgraded .mdc-floating-label--float-above[dir=rtl],.mdc-text-field--with-leading-icon.mdc-text-field--outlined .mdc-notched-outline--upgraded .mdc-floating-label--float-above[dir=rtl]{transform:translateY(-34.75px) translateX(32px) scale(0.75)}.mdc-text-field--with-leading-icon.mdc-text-field--outlined .mdc-floating-label--float-above{font-size:.75rem}.mdc-text-field--with-leading-icon.mdc-text-field--outlined.mdc-notched-outline--upgraded .mdc-floating-label--float-above,.mdc-text-field--with-leading-icon.mdc-text-field--outlined .mdc-notched-outline--upgraded .mdc-floating-label--float-above{font-size:1rem}.mdc-text-field--with-trailing-icon{padding-left:16px;padding-right:0}[dir=rtl] .mdc-text-field--with-trailing-icon,.mdc-text-field--with-trailing-icon[dir=rtl]{padding-left:0;padding-right:16px}.mdc-text-field--with-trailing-icon.mdc-text-field--filled .mdc-floating-label{max-width:calc(100% - 64px)}.mdc-text-field--with-trailing-icon.mdc-text-field--filled .mdc-floating-label--float-above{max-width:calc(100%/0.75 - 64px/0.75)}.mdc-text-field--with-trailing-icon.mdc-text-field--outlined :not(.mdc-notched-outline--notched) .mdc-notched-outline__notch{max-width:calc(100% - 60px)}.mdc-text-field--with-leading-icon.mdc-text-field--with-trailing-icon{padding-left:0;padding-right:0}.mdc-text-field--with-leading-icon.mdc-text-field--with-trailing-icon.mdc-text-field--filled .mdc-floating-label{max-width:calc(100% - 96px)}.mdc-text-field--with-leading-icon.mdc-text-field--with-trailing-icon.mdc-text-field--filled .mdc-floating-label--float-above{max-width:calc(100%/0.75 - 96px/0.75)}.mdc-text-field-helper-line{display:flex;justify-content:space-between;box-sizing:border-box}.mdc-text-field+.mdc-text-field-helper-line{padding-right:16px;padding-left:16px}.mdc-form-field>.mdc-text-field+label{align-self:flex-start}.mdc-text-field--focused .mdc-notched-outline__leading,.mdc-text-field--focused .mdc-notched-outline__notch,.mdc-text-field--focused .mdc-notched-outline__trailing{border-width:2px}.mdc-text-field--focused+.mdc-text-field-helper-line .mdc-text-field-helper-text:not(.mdc-text-field-helper-text--validation-msg){opacity:1}.mdc-text-field--focused.mdc-text-field--outlined .mdc-notched-outline--notched .mdc-notched-outline__notch{padding-top:2px}.mdc-text-field--focused.mdc-text-field--outlined.mdc-text-field--textarea .mdc-notched-outline--notched .mdc-notched-outline__notch{padding-top:0}.mdc-text-field--invalid+.mdc-text-field-helper-line .mdc-text-field-helper-text--validation-msg{opacity:1}.mdc-text-field--disabled{pointer-events:none}@media screen and (forced-colors: active){.mdc-text-field--disabled .mdc-text-field__input{background-color:Window}.mdc-text-field--disabled .mdc-floating-label{z-index:1}}.mdc-text-field--disabled .mdc-floating-label{cursor:default}.mdc-text-field--disabled.mdc-text-field--filled .mdc-text-field__ripple{display:none}.mdc-text-field--disabled .mdc-text-field__input{pointer-events:auto}.mdc-text-field--end-aligned .mdc-text-field__input{text-align:right}[dir=rtl] .mdc-text-field--end-aligned .mdc-text-field__input,.mdc-text-field--end-aligned .mdc-text-field__input[dir=rtl]{text-align:left}[dir=rtl] .mdc-text-field--ltr-text .mdc-text-field__input,[dir=rtl] .mdc-text-field--ltr-text .mdc-text-field__affix,.mdc-text-field--ltr-text[dir=rtl] .mdc-text-field__input,.mdc-text-field--ltr-text[dir=rtl] .mdc-text-field__affix{direction:ltr}[dir=rtl] .mdc-text-field--ltr-text .mdc-text-field__affix--prefix,.mdc-text-field--ltr-text[dir=rtl] .mdc-text-field__affix--prefix{padding-left:0;padding-right:2px}[dir=rtl] .mdc-text-field--ltr-text .mdc-text-field__affix--suffix,.mdc-text-field--ltr-text[dir=rtl] .mdc-text-field__affix--suffix{padding-left:12px;padding-right:0}[dir=rtl] .mdc-text-field--ltr-text .mdc-text-field__icon--leading,.mdc-text-field--ltr-text[dir=rtl] .mdc-text-field__icon--leading{order:1}[dir=rtl] .mdc-text-field--ltr-text .mdc-text-field__affix--suffix,.mdc-text-field--ltr-text[dir=rtl] .mdc-text-field__affix--suffix{order:2}[dir=rtl] .mdc-text-field--ltr-text .mdc-text-field__input,.mdc-text-field--ltr-text[dir=rtl] .mdc-text-field__input{order:3}[dir=rtl] .mdc-text-field--ltr-text .mdc-text-field__affix--prefix,.mdc-text-field--ltr-text[dir=rtl] .mdc-text-field__affix--prefix{order:4}[dir=rtl] .mdc-text-field--ltr-text .mdc-text-field__icon--trailing,.mdc-text-field--ltr-text[dir=rtl] .mdc-text-field__icon--trailing{order:5}[dir=rtl] .mdc-text-field--ltr-text.mdc-text-field--end-aligned .mdc-text-field__input,.mdc-text-field--ltr-text.mdc-text-field--end-aligned[dir=rtl] .mdc-text-field__input{text-align:right}[dir=rtl] .mdc-text-field--ltr-text.mdc-text-field--end-aligned .mdc-text-field__affix--prefix,.mdc-text-field--ltr-text.mdc-text-field--end-aligned[dir=rtl] .mdc-text-field__affix--prefix{padding-right:12px}[dir=rtl] .mdc-text-field--ltr-text.mdc-text-field--end-aligned .mdc-text-field__affix--suffix,.mdc-text-field--ltr-text.mdc-text-field--end-aligned[dir=rtl] .mdc-text-field__affix--suffix{padding-left:2px}.mdc-floating-label{position:absolute;left:0;-webkit-transform-origin:left top;transform-origin:left top;line-height:1.15rem;text-align:left;text-overflow:ellipsis;white-space:nowrap;cursor:text;overflow:hidden;will-change:transform}[dir=rtl] .mdc-floating-label,.mdc-floating-label[dir=rtl]{right:0;left:auto;-webkit-transform-origin:right top;transform-origin:right top;text-align:right}.mdc-floating-label--float-above{cursor:auto}.mdc-floating-label--required:not(.mdc-floating-label--hide-required-marker)::after{margin-left:1px;margin-right:0px;content:"*"}[dir=rtl] .mdc-floating-label--required:not(.mdc-floating-label--hide-required-marker)::after,.mdc-floating-label--required:not(.mdc-floating-label--hide-required-marker)[dir=rtl]::after{margin-left:0;margin-right:1px}.mdc-notched-outline{display:flex;position:absolute;top:0;right:0;left:0;box-sizing:border-box;width:100%;max-width:100%;height:100%;text-align:left;pointer-events:none}[dir=rtl] .mdc-notched-outline,.mdc-notched-outline[dir=rtl]{text-align:right}.mdc-notched-outline__leading,.mdc-notched-outline__notch,.mdc-notched-outline__trailing{box-sizing:border-box;height:100%;pointer-events:none}.mdc-notched-outline__trailing{flex-grow:1}.mdc-notched-outline__notch{flex:0 0 auto;width:auto}.mdc-notched-outline .mdc-floating-label{display:inline-block;position:relative;max-width:100%}.mdc-notched-outline .mdc-floating-label--float-above{text-overflow:clip}.mdc-notched-outline--upgraded .mdc-floating-label--float-above{max-width:133.3333333333%}.mdc-notched-outline--notched .mdc-notched-outline__notch{padding-left:0;padding-right:8px;border-top:none}[dir=rtl] .mdc-notched-outline--notched .mdc-notched-outline__notch,.mdc-notched-outline--notched .mdc-notched-outline__notch[dir=rtl]{padding-left:8px;padding-right:0}.mdc-notched-outline--no-label .mdc-notched-outline__notch{display:none}.mdc-line-ripple::before,.mdc-line-ripple::after{position:absolute;bottom:0;left:0;width:100%;border-bottom-style:solid;content:""}.mdc-line-ripple::before{z-index:1}.mdc-line-ripple::after{transform:scaleX(0);opacity:0;z-index:2}.mdc-line-ripple--active::after{transform:scaleX(1);opacity:1}.mdc-line-ripple--deactivating::after{opacity:0}.mdc-floating-label--float-above{transform:translateY(-106%) scale(0.75)}.mdc-notched-outline__leading,.mdc-notched-outline__notch,.mdc-notched-outline__trailing{border-top:1px solid;border-bottom:1px solid}.mdc-notched-outline__leading{border-left:1px solid;border-right:none;width:12px}[dir=rtl] .mdc-notched-outline__leading,.mdc-notched-outline__leading[dir=rtl]{border-left:none;border-right:1px solid}.mdc-notched-outline__trailing{border-left:none;border-right:1px solid}[dir=rtl] .mdc-notched-outline__trailing,.mdc-notched-outline__trailing[dir=rtl]{border-left:1px solid;border-right:none}.mdc-notched-outline__notch{max-width:calc(100% - 12px*2)}.mdc-line-ripple::before{border-bottom-width:1px}.mdc-line-ripple::after{border-bottom-width:2px}.mdc-text-field--filled{border-top-left-radius:var(--mdc-filled-text-field-container-shape);border-top-right-radius:var(--mdc-filled-text-field-container-shape);border-bottom-right-radius:0;border-bottom-left-radius:0}.mdc-text-field--filled:not(.mdc-text-field--disabled) .mdc-text-field__input{caret-color:var(--mdc-filled-text-field-caret-color)}.mdc-text-field--filled.mdc-text-field--invalid:not(.mdc-text-field--disabled) .mdc-text-field__input{caret-color:var(--mdc-filled-text-field-error-caret-color)}.mdc-text-field--filled:not(.mdc-text-field--disabled) .mdc-text-field__input{color:var(--mdc-filled-text-field-input-text-color)}.mdc-text-field--filled.mdc-text-field--disabled .mdc-text-field__input{color:var(--mdc-filled-text-field-disabled-input-text-color)}.mdc-text-field--filled:not(.mdc-text-field--disabled) .mdc-floating-label,.mdc-text-field--filled:not(.mdc-text-field--disabled) .mdc-floating-label--float-above{color:var(--mdc-filled-text-field-label-text-color)}.mdc-text-field--filled:not(.mdc-text-field--disabled).mdc-text-field--focused .mdc-floating-label,.mdc-text-field--filled:not(.mdc-text-field--disabled).mdc-text-field--focused .mdc-floating-label--float-above{color:var(--mdc-filled-text-field-focus-label-text-color)}.mdc-text-field--filled:not(.mdc-text-field--disabled):not(.mdc-text-field--focused):hover .mdc-floating-label,.mdc-text-field--filled:not(.mdc-text-field--disabled):not(.mdc-text-field--focused):hover .mdc-floating-label--float-above{color:var(--mdc-filled-text-field-hover-label-text-color)}.mdc-text-field--filled.mdc-text-field--disabled .mdc-floating-label,.mdc-text-field--filled.mdc-text-field--disabled .mdc-floating-label--float-above{color:var(--mdc-filled-text-field-disabled-label-text-color)}.mdc-text-field--filled.mdc-text-field--invalid:not(.mdc-text-field--disabled) .mdc-floating-label,.mdc-text-field--filled.mdc-text-field--invalid:not(.mdc-text-field--disabled) .mdc-floating-label--float-above{color:var(--mdc-filled-text-field-error-label-text-color)}.mdc-text-field--filled.mdc-text-field--invalid:not(.mdc-text-field--disabled).mdc-text-field--focused .mdc-floating-label,.mdc-text-field--filled.mdc-text-field--invalid:not(.mdc-text-field--disabled).mdc-text-field--focused .mdc-floating-label--float-above{color:var(--mdc-filled-text-field-error-focus-label-text-color)}.mdc-text-field--filled.mdc-text-field--invalid:not(.mdc-text-field--disabled):not(.mdc-text-field--focused):hover .mdc-floating-label,.mdc-text-field--filled.mdc-text-field--invalid:not(.mdc-text-field--disabled):not(.mdc-text-field--focused):hover .mdc-floating-label--float-above{color:var(--mdc-filled-text-field-error-hover-label-text-color)}.mdc-text-field--filled .mdc-floating-label{font-family:var(--mdc-filled-text-field-label-text-font);font-size:var(--mdc-filled-text-field-label-text-size);font-weight:var(--mdc-filled-text-field-label-text-weight);letter-spacing:var(--mdc-filled-text-field-label-text-tracking)}@media all{.mdc-text-field--filled:not(.mdc-text-field--disabled) .mdc-text-field__input::placeholder{color:var(--mdc-filled-text-field-input-text-placeholder-color)}}@media all{.mdc-text-field--filled:not(.mdc-text-field--disabled) .mdc-text-field__input:-ms-input-placeholder{color:var(--mdc-filled-text-field-input-text-placeholder-color)}}.mdc-text-field--filled:not(.mdc-text-field--disabled){background-color:var(--mdc-filled-text-field-container-color)}.mdc-text-field--filled.mdc-text-field--disabled{background-color:var(--mdc-filled-text-field-disabled-container-color)}.mdc-text-field--filled:not(.mdc-text-field--disabled) .mdc-line-ripple::before{border-bottom-color:var(--mdc-filled-text-field-active-indicator-color)}.mdc-text-field--filled:not(.mdc-text-field--disabled):not(.mdc-text-field--focused):hover .mdc-line-ripple::before{border-bottom-color:var(--mdc-filled-text-field-hover-active-indicator-color)}.mdc-text-field--filled:not(.mdc-text-field--disabled) .mdc-line-ripple::after{border-bottom-color:var(--mdc-filled-text-field-focus-active-indicator-color)}.mdc-text-field--filled.mdc-text-field--disabled .mdc-line-ripple::before{border-bottom-color:var(--mdc-filled-text-field-disabled-active-indicator-color)}.mdc-text-field--filled.mdc-text-field--invalid:not(.mdc-text-field--disabled) .mdc-line-ripple::before{border-bottom-color:var(--mdc-filled-text-field-error-active-indicator-color)}.mdc-text-field--filled.mdc-text-field--invalid:not(.mdc-text-field--disabled):not(.mdc-text-field--focused):hover .mdc-line-ripple::before{border-bottom-color:var(--mdc-filled-text-field-error-hover-active-indicator-color)}.mdc-text-field--filled.mdc-text-field--invalid:not(.mdc-text-field--disabled) .mdc-line-ripple::after{border-bottom-color:var(--mdc-filled-text-field-error-focus-active-indicator-color)}.mdc-text-field--filled .mdc-line-ripple::before{border-bottom-width:var(--mdc-filled-text-field-active-indicator-height)}.mdc-text-field--filled .mdc-line-ripple::after{border-bottom-width:var(--mdc-filled-text-field-focus-active-indicator-height)}.mdc-text-field--outlined:not(.mdc-text-field--disabled) .mdc-text-field__input{caret-color:var(--mdc-outlined-text-field-caret-color)}.mdc-text-field--outlined.mdc-text-field--invalid:not(.mdc-text-field--disabled) .mdc-text-field__input{caret-color:var(--mdc-outlined-text-field-error-caret-color)}.mdc-text-field--outlined:not(.mdc-text-field--disabled) .mdc-text-field__input{color:var(--mdc-outlined-text-field-input-text-color)}.mdc-text-field--outlined.mdc-text-field--disabled .mdc-text-field__input{color:var(--mdc-outlined-text-field-disabled-input-text-color)}.mdc-text-field--outlined:not(.mdc-text-field--disabled) .mdc-floating-label,.mdc-text-field--outlined:not(.mdc-text-field--disabled) .mdc-floating-label--float-above{color:var(--mdc-outlined-text-field-label-text-color)}.mdc-text-field--outlined:not(.mdc-text-field--disabled).mdc-text-field--focused .mdc-floating-label,.mdc-text-field--outlined:not(.mdc-text-field--disabled).mdc-text-field--focused .mdc-floating-label--float-above{color:var(--mdc-outlined-text-field-focus-label-text-color)}.mdc-text-field--outlined:not(.mdc-text-field--disabled):not(.mdc-text-field--focused):hover .mdc-floating-label,.mdc-text-field--outlined:not(.mdc-text-field--disabled):not(.mdc-text-field--focused):hover .mdc-floating-label--float-above{color:var(--mdc-outlined-text-field-hover-label-text-color)}.mdc-text-field--outlined.mdc-text-field--disabled .mdc-floating-label,.mdc-text-field--outlined.mdc-text-field--disabled .mdc-floating-label--float-above{color:var(--mdc-outlined-text-field-disabled-label-text-color)}.mdc-text-field--outlined.mdc-text-field--invalid:not(.mdc-text-field--disabled) .mdc-floating-label,.mdc-text-field--outlined.mdc-text-field--invalid:not(.mdc-text-field--disabled) .mdc-floating-label--float-above{color:var(--mdc-outlined-text-field-error-label-text-color)}.mdc-text-field--outlined.mdc-text-field--invalid:not(.mdc-text-field--disabled).mdc-text-field--focused .mdc-floating-label,.mdc-text-field--outlined.mdc-text-field--invalid:not(.mdc-text-field--disabled).mdc-text-field--focused .mdc-floating-label--float-above{color:var(--mdc-outlined-text-field-error-focus-label-text-color)}.mdc-text-field--outlined.mdc-text-field--invalid:not(.mdc-text-field--disabled):not(.mdc-text-field--focused):hover .mdc-floating-label,.mdc-text-field--outlined.mdc-text-field--invalid:not(.mdc-text-field--disabled):not(.mdc-text-field--focused):hover .mdc-floating-label--float-above{color:var(--mdc-outlined-text-field-error-hover-label-text-color)}.mdc-text-field--outlined .mdc-floating-label{font-family:var(--mdc-outlined-text-field-label-text-font);font-size:var(--mdc-outlined-text-field-label-text-size);font-weight:var(--mdc-outlined-text-field-label-text-weight);letter-spacing:var(--mdc-outlined-text-field-label-text-tracking)}@media all{.mdc-text-field--outlined:not(.mdc-text-field--disabled) .mdc-text-field__input::placeholder{color:var(--mdc-outlined-text-field-input-text-placeholder-color)}}@media all{.mdc-text-field--outlined:not(.mdc-text-field--disabled) .mdc-text-field__input:-ms-input-placeholder{color:var(--mdc-outlined-text-field-input-text-placeholder-color)}}.mdc-text-field--outlined.mdc-text-field--textarea.mdc-text-field--outlined .mdc-floating-label--float-above{font-size:calc(.75*var(--mdc-outlined-text-field-label-text-size))}.mdc-text-field--outlined.mdc-text-field--textarea.mdc-text-field--outlined.mdc-notched-outline--upgraded .mdc-floating-label--float-above,.mdc-text-field--outlined.mdc-text-field--textarea.mdc-text-field--outlined .mdc-notched-outline--upgraded .mdc-floating-label--float-above{font-size:var(--mdc-outlined-text-field-label-text-size)}.mdc-text-field--outlined .mdc-notched-outline .mdc-notched-outline__leading{border-top-left-radius:var(--mdc-outlined-text-field-container-shape);border-top-right-radius:0;border-bottom-right-radius:0;border-bottom-left-radius:var(--mdc-outlined-text-field-container-shape)}[dir=rtl] .mdc-text-field--outlined .mdc-notched-outline .mdc-notched-outline__leading,.mdc-text-field--outlined .mdc-notched-outline .mdc-notched-outline__leading[dir=rtl]{border-top-left-radius:0;border-top-right-radius:var(--mdc-outlined-text-field-container-shape);border-bottom-right-radius:var(--mdc-outlined-text-field-container-shape);border-bottom-left-radius:0}@supports(top: max(0%)){.mdc-text-field--outlined .mdc-notched-outline .mdc-notched-outline__leading{width:max(12px,var(--mdc-outlined-text-field-container-shape))}}@supports(top: max(0%)){.mdc-text-field--outlined .mdc-notched-outline .mdc-notched-outline__notch{max-width:calc(100% - max(12px,var(--mdc-outlined-text-field-container-shape))*2)}}.mdc-text-field--outlined .mdc-notched-outline .mdc-notched-outline__trailing{border-top-left-radius:0;border-top-right-radius:var(--mdc-outlined-text-field-container-shape);border-bottom-right-radius:var(--mdc-outlined-text-field-container-shape);border-bottom-left-radius:0}[dir=rtl] .mdc-text-field--outlined .mdc-notched-outline .mdc-notched-outline__trailing,.mdc-text-field--outlined .mdc-notched-outline .mdc-notched-outline__trailing[dir=rtl]{border-top-left-radius:var(--mdc-outlined-text-field-container-shape);border-top-right-radius:0;border-bottom-right-radius:0;border-bottom-left-radius:var(--mdc-outlined-text-field-container-shape)}@supports(top: max(0%)){.mdc-text-field--outlined{padding-left:max(16px,calc(var(--mdc-outlined-text-field-container-shape) + 4px))}}@supports(top: max(0%)){.mdc-text-field--outlined{padding-right:max(16px,var(--mdc-outlined-text-field-container-shape))}}@supports(top: max(0%)){.mdc-text-field--outlined+.mdc-text-field-helper-line{padding-left:max(16px,calc(var(--mdc-outlined-text-field-container-shape) + 4px))}}@supports(top: max(0%)){.mdc-text-field--outlined+.mdc-text-field-helper-line{padding-right:max(16px,var(--mdc-outlined-text-field-container-shape))}}.mdc-text-field--outlined.mdc-text-field--with-leading-icon{padding-left:0}@supports(top: max(0%)){.mdc-text-field--outlined.mdc-text-field--with-leading-icon{padding-right:max(16px,var(--mdc-outlined-text-field-container-shape))}}[dir=rtl] .mdc-text-field--outlined.mdc-text-field--with-leading-icon,.mdc-text-field--outlined.mdc-text-field--with-leading-icon[dir=rtl]{padding-right:0}@supports(top: max(0%)){[dir=rtl] .mdc-text-field--outlined.mdc-text-field--with-leading-icon,.mdc-text-field--outlined.mdc-text-field--with-leading-icon[dir=rtl]{padding-left:max(16px,var(--mdc-outlined-text-field-container-shape))}}.mdc-text-field--outlined.mdc-text-field--with-trailing-icon{padding-right:0}@supports(top: max(0%)){.mdc-text-field--outlined.mdc-text-field--with-trailing-icon{padding-left:max(16px,calc(var(--mdc-outlined-text-field-container-shape) + 4px))}}[dir=rtl] .mdc-text-field--outlined.mdc-text-field--with-trailing-icon,.mdc-text-field--outlined.mdc-text-field--with-trailing-icon[dir=rtl]{padding-left:0}@supports(top: max(0%)){[dir=rtl] .mdc-text-field--outlined.mdc-text-field--with-trailing-icon,.mdc-text-field--outlined.mdc-text-field--with-trailing-icon[dir=rtl]{padding-right:max(16px,calc(var(--mdc-outlined-text-field-container-shape) + 4px))}}.mdc-text-field--outlined.mdc-text-field--with-leading-icon.mdc-text-field--with-trailing-icon{padding-left:0;padding-right:0}.mdc-text-field--outlined:not(.mdc-text-field--disabled) .mdc-notched-outline__leading,.mdc-text-field--outlined:not(.mdc-text-field--disabled) .mdc-notched-outline__notch,.mdc-text-field--outlined:not(.mdc-text-field--disabled) .mdc-notched-outline__trailing{border-color:var(--mdc-outlined-text-field-outline-color)}.mdc-text-field--outlined:not(.mdc-text-field--disabled):not(.mdc-text-field--focused):hover .mdc-notched-outline .mdc-notched-outline__leading,.mdc-text-field--outlined:not(.mdc-text-field--disabled):not(.mdc-text-field--focused):hover .mdc-notched-outline .mdc-notched-outline__notch,.mdc-text-field--outlined:not(.mdc-text-field--disabled):not(.mdc-text-field--focused):hover .mdc-notched-outline .mdc-notched-outline__trailing{border-color:var(--mdc-outlined-text-field-hover-outline-color)}.mdc-text-field--outlined:not(.mdc-text-field--disabled).mdc-text-field--focused .mdc-notched-outline__leading,.mdc-text-field--outlined:not(.mdc-text-field--disabled).mdc-text-field--focused .mdc-notched-outline__notch,.mdc-text-field--outlined:not(.mdc-text-field--disabled).mdc-text-field--focused .mdc-notched-outline__trailing{border-color:var(--mdc-outlined-text-field-focus-outline-color)}.mdc-text-field--outlined.mdc-text-field--disabled .mdc-notched-outline__leading,.mdc-text-field--outlined.mdc-text-field--disabled .mdc-notched-outline__notch,.mdc-text-field--outlined.mdc-text-field--disabled .mdc-notched-outline__trailing{border-color:var(--mdc-outlined-text-field-disabled-outline-color)}.mdc-text-field--outlined.mdc-text-field--invalid:not(.mdc-text-field--disabled) .mdc-notched-outline__leading,.mdc-text-field--outlined.mdc-text-field--invalid:not(.mdc-text-field--disabled) .mdc-notched-outline__notch,.mdc-text-field--outlined.mdc-text-field--invalid:not(.mdc-text-field--disabled) .mdc-notched-outline__trailing{border-color:var(--mdc-outlined-text-field-error-outline-color)}.mdc-text-field--outlined.mdc-text-field--invalid:not(.mdc-text-field--disabled):not(.mdc-text-field--focused):hover .mdc-notched-outline .mdc-notched-outline__leading,.mdc-text-field--outlined.mdc-text-field--invalid:not(.mdc-text-field--disabled):not(.mdc-text-field--focused):hover .mdc-notched-outline .mdc-notched-outline__notch,.mdc-text-field--outlined.mdc-text-field--invalid:not(.mdc-text-field--disabled):not(.mdc-text-field--focused):hover .mdc-notched-outline .mdc-notched-outline__trailing{border-color:var(--mdc-outlined-text-field-error-hover-outline-color)}.mdc-text-field--outlined.mdc-text-field--invalid:not(.mdc-text-field--disabled).mdc-text-field--focused .mdc-notched-outline__leading,.mdc-text-field--outlined.mdc-text-field--invalid:not(.mdc-text-field--disabled).mdc-text-field--focused .mdc-notched-outline__notch,.mdc-text-field--outlined.mdc-text-field--invalid:not(.mdc-text-field--disabled).mdc-text-field--focused .mdc-notched-outline__trailing{border-color:var(--mdc-outlined-text-field-error-focus-outline-color)}.mdc-text-field--outlined:not(.mdc-text-field--disabled) .mdc-notched-outline .mdc-notched-outline__leading,.mdc-text-field--outlined:not(.mdc-text-field--disabled) .mdc-notched-outline .mdc-notched-outline__notch,.mdc-text-field--outlined:not(.mdc-text-field--disabled) .mdc-notched-outline .mdc-notched-outline__trailing{border-width:var(--mdc-outlined-text-field-outline-width)}.mdc-text-field--outlined:not(.mdc-text-field--disabled).mdc-text-field--focused .mdc-notched-outline .mdc-notched-outline__leading,.mdc-text-field--outlined:not(.mdc-text-field--disabled).mdc-text-field--focused .mdc-notched-outline .mdc-notched-outline__notch,.mdc-text-field--outlined:not(.mdc-text-field--disabled).mdc-text-field--focused .mdc-notched-outline .mdc-notched-outline__trailing{border-width:var(--mdc-outlined-text-field-focus-outline-width)}.mat-mdc-form-field-textarea-control{vertical-align:middle;resize:vertical;box-sizing:border-box;height:auto;margin:0;padding:0;border:none;overflow:auto}.mat-mdc-form-field-input-control.mat-mdc-form-field-input-control{-moz-osx-font-smoothing:grayscale;-webkit-font-smoothing:antialiased;font:inherit;letter-spacing:inherit;text-decoration:inherit;text-transform:inherit;border:none}.mat-mdc-form-field .mat-mdc-floating-label.mdc-floating-label{-moz-osx-font-smoothing:grayscale;-webkit-font-smoothing:antialiased;line-height:normal;pointer-events:all}.mat-mdc-form-field:not(.mat-form-field-disabled) .mat-mdc-floating-label.mdc-floating-label{cursor:inherit}.mdc-text-field--no-label:not(.mdc-text-field--textarea) .mat-mdc-form-field-input-control.mdc-text-field__input,.mat-mdc-text-field-wrapper .mat-mdc-form-field-input-control{height:auto}.mat-mdc-text-field-wrapper .mat-mdc-form-field-input-control.mdc-text-field__input[type=color]{height:23px}.mat-mdc-text-field-wrapper{height:auto;flex:auto}.mat-mdc-form-field-has-icon-prefix .mat-mdc-text-field-wrapper{padding-left:0;--mat-mdc-form-field-label-offset-x: -16px}.mat-mdc-form-field-has-icon-suffix .mat-mdc-text-field-wrapper{padding-right:0}[dir=rtl] .mat-mdc-text-field-wrapper{padding-left:16px;padding-right:16px}[dir=rtl] .mat-mdc-form-field-has-icon-suffix .mat-mdc-text-field-wrapper{padding-left:0}[dir=rtl] .mat-mdc-form-field-has-icon-prefix .mat-mdc-text-field-wrapper{padding-right:0}.mat-form-field-disabled .mdc-text-field__input::placeholder{color:var(--mat-form-field-disabled-input-text-placeholder-color)}.mat-form-field-disabled .mdc-text-field__input::-moz-placeholder{color:var(--mat-form-field-disabled-input-text-placeholder-color)}.mat-form-field-disabled .mdc-text-field__input::-webkit-input-placeholder{color:var(--mat-form-field-disabled-input-text-placeholder-color)}.mat-form-field-disabled .mdc-text-field__input:-ms-input-placeholder{color:var(--mat-form-field-disabled-input-text-placeholder-color)}.mat-mdc-form-field-label-always-float .mdc-text-field__input::placeholder{transition-delay:40ms;transition-duration:110ms;opacity:1}.mat-mdc-text-field-wrapper .mat-mdc-form-field-infix .mat-mdc-floating-label{left:auto;right:auto}.mat-mdc-text-field-wrapper.mdc-text-field--outlined .mdc-text-field__input{display:inline-block}.mat-mdc-form-field .mat-mdc-text-field-wrapper.mdc-text-field .mdc-notched-outline__notch{padding-top:0}.mat-mdc-text-field-wrapper::before{content:none}.mat-mdc-form-field.mat-mdc-form-field.mat-mdc-form-field.mat-mdc-form-field.mat-mdc-form-field.mat-mdc-form-field .mdc-notched-outline__notch{border-left:1px solid rgba(0,0,0,0)}[dir=rtl] .mat-mdc-form-field.mat-mdc-form-field.mat-mdc-form-field.mat-mdc-form-field.mat-mdc-form-field.mat-mdc-form-field .mdc-notched-outline__notch{border-left:none;border-right:1px solid rgba(0,0,0,0)}.mat-mdc-form-field-infix{min-height:var(--mat-form-field-container-height);padding-top:var(--mat-form-field-filled-with-label-container-padding-top);padding-bottom:var(--mat-form-field-filled-with-label-container-padding-bottom)}.mdc-text-field--outlined .mat-mdc-form-field-infix,.mdc-text-field--no-label .mat-mdc-form-field-infix{padding-top:var(--mat-form-field-container-vertical-padding);padding-bottom:var(--mat-form-field-container-vertical-padding)}.mat-mdc-text-field-wrapper .mat-mdc-form-field-flex .mat-mdc-floating-label{top:calc(var(--mat-form-field-container-height)/2)}.mdc-text-field--filled .mat-mdc-floating-label{display:var(--mat-form-field-filled-label-display, block)}.mat-mdc-text-field-wrapper.mdc-text-field--outlined .mdc-notched-outline--upgraded .mdc-floating-label--float-above{--mat-mdc-form-field-label-transform: translateY(calc(calc(6.75px + var(--mat-form-field-container-height) / 2) * -1)) scale(var(--mat-mdc-form-field-floating-label-scale, 0.75));transform:var(--mat-mdc-form-field-label-transform)}.mat-mdc-form-field-subscript-wrapper{box-sizing:border-box;width:100%;position:relative}.mat-mdc-form-field-hint-wrapper,.mat-mdc-form-field-error-wrapper{position:absolute;top:0;left:0;right:0;padding:0 16px}.mat-mdc-form-field-subscript-dynamic-size .mat-mdc-form-field-hint-wrapper,.mat-mdc-form-field-subscript-dynamic-size .mat-mdc-form-field-error-wrapper{position:static}.mat-mdc-form-field-bottom-align::before{content:"";display:inline-block;height:16px}.mat-mdc-form-field-bottom-align.mat-mdc-form-field-subscript-dynamic-size::before{content:unset}.mat-mdc-form-field-hint-end{order:1}.mat-mdc-form-field-hint-wrapper{display:flex}.mat-mdc-form-field-hint-spacer{flex:1 0 1em}.mat-mdc-form-field-error{display:block;color:var(--mat-form-field-error-text-color)}.mat-mdc-form-field-subscript-wrapper,.mat-mdc-form-field-bottom-align::before{-moz-osx-font-smoothing:grayscale;-webkit-font-smoothing:antialiased;font-family:var(--mat-form-field-subscript-text-font);line-height:var(--mat-form-field-subscript-text-line-height);font-size:var(--mat-form-field-subscript-text-size);letter-spacing:var(--mat-form-field-subscript-text-tracking);font-weight:var(--mat-form-field-subscript-text-weight)}.mat-mdc-form-field-focus-overlay{top:0;left:0;right:0;bottom:0;position:absolute;opacity:0;pointer-events:none;background-color:var(--mat-form-field-state-layer-color)}.mat-mdc-text-field-wrapper:hover .mat-mdc-form-field-focus-overlay{opacity:var(--mat-form-field-hover-state-layer-opacity)}.mat-mdc-form-field.mat-focused .mat-mdc-form-field-focus-overlay{opacity:var(--mat-form-field-focus-state-layer-opacity)}select.mat-mdc-form-field-input-control{-moz-appearance:none;-webkit-appearance:none;background-color:rgba(0,0,0,0);display:inline-flex;box-sizing:border-box}select.mat-mdc-form-field-input-control:not(:disabled){cursor:pointer}select.mat-mdc-form-field-input-control:not(.mat-mdc-native-select-inline) option{color:var(--mat-form-field-select-option-text-color)}select.mat-mdc-form-field-input-control:not(.mat-mdc-native-select-inline) option:disabled{color:var(--mat-form-field-select-disabled-option-text-color)}.mat-mdc-form-field-type-mat-native-select .mat-mdc-form-field-infix::after{content:"";width:0;height:0;border-left:5px solid rgba(0,0,0,0);border-right:5px solid rgba(0,0,0,0);border-top:5px solid;position:absolute;right:0;top:50%;margin-top:-2.5px;pointer-events:none;color:var(--mat-form-field-enabled-select-arrow-color)}[dir=rtl] .mat-mdc-form-field-type-mat-native-select .mat-mdc-form-field-infix::after{right:auto;left:0}.mat-mdc-form-field-type-mat-native-select.mat-focused .mat-mdc-form-field-infix::after{color:var(--mat-form-field-focus-select-arrow-color)}.mat-mdc-form-field-type-mat-native-select.mat-form-field-disabled .mat-mdc-form-field-infix::after{color:var(--mat-form-field-disabled-select-arrow-color)}.mat-mdc-form-field-type-mat-native-select .mat-mdc-form-field-input-control{padding-right:15px}[dir=rtl] .mat-mdc-form-field-type-mat-native-select .mat-mdc-form-field-input-control{padding-right:0;padding-left:15px}.cdk-high-contrast-active .mat-form-field-appearance-fill .mat-mdc-text-field-wrapper{outline:solid 1px}.cdk-high-contrast-active .mat-form-field-appearance-fill.mat-form-field-disabled .mat-mdc-text-field-wrapper{outline-color:GrayText}.cdk-high-contrast-active .mat-form-field-appearance-fill.mat-focused .mat-mdc-text-field-wrapper{outline:dashed 3px}.cdk-high-contrast-active .mat-mdc-form-field.mat-focused .mdc-notched-outline{border:dashed 3px}.mat-mdc-form-field-input-control[type=date],.mat-mdc-form-field-input-control[type=datetime],.mat-mdc-form-field-input-control[type=datetime-local],.mat-mdc-form-field-input-control[type=month],.mat-mdc-form-field-input-control[type=week],.mat-mdc-form-field-input-control[type=time]{line-height:1}.mat-mdc-form-field-input-control::-webkit-datetime-edit{line-height:1;padding:0;margin-bottom:-2px}.mat-mdc-form-field{--mat-mdc-form-field-floating-label-scale: 0.75;display:inline-flex;flex-direction:column;min-width:0;text-align:left;-moz-osx-font-smoothing:grayscale;-webkit-font-smoothing:antialiased;font-family:var(--mat-form-field-container-text-font);line-height:var(--mat-form-field-container-text-line-height);font-size:var(--mat-form-field-container-text-size);letter-spacing:var(--mat-form-field-container-text-tracking);font-weight:var(--mat-form-field-container-text-weight)}[dir=rtl] .mat-mdc-form-field{text-align:right}.mat-mdc-form-field .mdc-text-field--outlined .mdc-floating-label--float-above{font-size:calc(var(--mat-form-field-outlined-label-text-populated-size)*var(--mat-mdc-form-field-floating-label-scale))}.mat-mdc-form-field .mdc-text-field--outlined .mdc-notched-outline--upgraded .mdc-floating-label--float-above{font-size:var(--mat-form-field-outlined-label-text-populated-size)}.mat-mdc-form-field-flex{display:inline-flex;align-items:baseline;box-sizing:border-box;width:100%}.mat-mdc-text-field-wrapper{width:100%;z-index:0}.mat-mdc-form-field-icon-prefix,.mat-mdc-form-field-icon-suffix{align-self:center;line-height:0;pointer-events:auto;position:relative;z-index:1}.mat-mdc-form-field-icon-prefix>.mat-icon,.mat-mdc-form-field-icon-suffix>.mat-icon{padding:0 12px;box-sizing:content-box}.mat-mdc-form-field-icon-prefix{color:var(--mat-form-field-leading-icon-color)}.mat-form-field-disabled .mat-mdc-form-field-icon-prefix{color:var(--mat-form-field-disabled-leading-icon-color)}.mat-mdc-form-field-icon-suffix{color:var(--mat-form-field-trailing-icon-color)}.mat-form-field-disabled .mat-mdc-form-field-icon-suffix{color:var(--mat-form-field-disabled-trailing-icon-color)}.mat-form-field-invalid .mat-mdc-form-field-icon-suffix{color:var(--mat-form-field-error-trailing-icon-color)}.mat-form-field-invalid:not(.mat-focused):not(.mat-form-field-disabled) .mat-mdc-text-field-wrapper:hover .mat-mdc-form-field-icon-suffix{color:var(--mat-form-field-error-hover-trailing-icon-color)}.mat-form-field-invalid.mat-focused .mat-mdc-text-field-wrapper .mat-mdc-form-field-icon-suffix{color:var(--mat-form-field-error-focus-trailing-icon-color)}.mat-mdc-form-field-icon-prefix,[dir=rtl] .mat-mdc-form-field-icon-suffix{padding:0 4px 0 0}.mat-mdc-form-field-icon-suffix,[dir=rtl] .mat-mdc-form-field-icon-prefix{padding:0 0 0 4px}.mat-mdc-form-field-subscript-wrapper .mat-icon,.mat-mdc-form-field label .mat-icon{width:1em;height:1em;font-size:inherit}.mat-mdc-form-field-infix{flex:auto;min-width:0;width:180px;position:relative;box-sizing:border-box}.mat-mdc-form-field .mdc-notched-outline__notch{margin-left:-1px;-webkit-clip-path:inset(-9em -999em -9em 1px);clip-path:inset(-9em -999em -9em 1px)}[dir=rtl] .mat-mdc-form-field .mdc-notched-outline__notch{margin-left:0;margin-right:-1px;-webkit-clip-path:inset(-9em 1px -9em -999em);clip-path:inset(-9em 1px -9em -999em)}.mat-mdc-form-field:not(.mat-form-field-no-animations) .mdc-text-field__input{transition:opacity 150ms 0ms cubic-bezier(0.4, 0, 0.2, 1)}@media all{.mat-mdc-form-field:not(.mat-form-field-no-animations) .mdc-text-field__input::placeholder{transition:opacity 67ms 0ms cubic-bezier(0.4, 0, 0.2, 1)}}@media all{.mat-mdc-form-field:not(.mat-form-field-no-animations) .mdc-text-field__input:-ms-input-placeholder{transition:opacity 67ms 0ms cubic-bezier(0.4, 0, 0.2, 1)}}@media all{.mdc-text-field--no-label .mat-mdc-form-field:not(.mat-form-field-no-animations) .mdc-text-field__input::placeholder,.mdc-text-field--focused .mat-mdc-form-field:not(.mat-form-field-no-animations) .mdc-text-field__input::placeholder{transition-delay:40ms;transition-duration:110ms}}@media all{.mdc-text-field--no-label .mat-mdc-form-field:not(.mat-form-field-no-animations) .mdc-text-field__input:-ms-input-placeholder,.mdc-text-field--focused .mat-mdc-form-field:not(.mat-form-field-no-animations) .mdc-text-field__input:-ms-input-placeholder{transition-delay:40ms;transition-duration:110ms}}.mat-mdc-form-field:not(.mat-form-field-no-animations) .mdc-text-field__affix{transition:opacity 150ms 0ms cubic-bezier(0.4, 0, 0.2, 1)}.mat-mdc-form-field:not(.mat-form-field-no-animations) .mdc-text-field--filled.mdc-ripple-upgraded--background-focused .mdc-text-field__ripple::before,.mat-mdc-form-field:not(.mat-form-field-no-animations) .mdc-text-field--filled:not(.mdc-ripple-upgraded):focus .mdc-text-field__ripple::before{transition-duration:75ms}.mat-mdc-form-field:not(.mat-form-field-no-animations) .mdc-text-field--outlined .mdc-floating-label--shake{animation:mdc-floating-label-shake-float-above-text-field-outlined 250ms 1}@keyframes mdc-floating-label-shake-float-above-text-field-outlined{0%{transform:translateX(calc(0% - 0%)) translateY(calc(0% - 34.75px)) scale(0.75)}33%{animation-timing-function:cubic-bezier(0.5, 0, 0.701732, 0.495819);transform:translateX(calc(4% - 0%)) translateY(calc(0% - 34.75px)) scale(0.75)}66%{animation-timing-function:cubic-bezier(0.302435, 0.381352, 0.55, 0.956352);transform:translateX(calc(-4% - 0%)) translateY(calc(0% - 34.75px)) scale(0.75)}100%{transform:translateX(calc(0% - 0%)) translateY(calc(0% - 34.75px)) scale(0.75)}}.mat-mdc-form-field:not(.mat-form-field-no-animations) .mdc-text-field--textarea{transition:none}.mat-mdc-form-field:not(.mat-form-field-no-animations) .mdc-text-field--textarea.mdc-text-field--filled .mdc-floating-label--shake{animation:mdc-floating-label-shake-float-above-textarea-filled 250ms 1}@keyframes mdc-floating-label-shake-float-above-textarea-filled{0%{transform:translateX(calc(0% - 0%)) translateY(calc(0% - 10.25px)) scale(0.75)}33%{animation-timing-function:cubic-bezier(0.5, 0, 0.701732, 0.495819);transform:translateX(calc(4% - 0%)) translateY(calc(0% - 10.25px)) scale(0.75)}66%{animation-timing-function:cubic-bezier(0.302435, 0.381352, 0.55, 0.956352);transform:translateX(calc(-4% - 0%)) translateY(calc(0% - 10.25px)) scale(0.75)}100%{transform:translateX(calc(0% - 0%)) translateY(calc(0% - 10.25px)) scale(0.75)}}.mat-mdc-form-field:not(.mat-form-field-no-animations) .mdc-text-field--textarea.mdc-text-field--outlined .mdc-floating-label--shake{animation:mdc-floating-label-shake-float-above-textarea-outlined 250ms 1}@keyframes mdc-floating-label-shake-float-above-textarea-outlined{0%{transform:translateX(calc(0% - 0%)) translateY(calc(0% - 24.75px)) scale(0.75)}33%{animation-timing-function:cubic-bezier(0.5, 0, 0.701732, 0.495819);transform:translateX(calc(4% - 0%)) translateY(calc(0% - 24.75px)) scale(0.75)}66%{animation-timing-function:cubic-bezier(0.302435, 0.381352, 0.55, 0.956352);transform:translateX(calc(-4% - 0%)) translateY(calc(0% - 24.75px)) scale(0.75)}100%{transform:translateX(calc(0% - 0%)) translateY(calc(0% - 24.75px)) scale(0.75)}}.mat-mdc-form-field:not(.mat-form-field-no-animations) .mdc-text-field--with-leading-icon.mdc-text-field--outlined .mdc-floating-label--shake{animation:mdc-floating-label-shake-float-above-text-field-outlined-leading-icon 250ms 1}@keyframes mdc-floating-label-shake-float-above-text-field-outlined-leading-icon{0%{transform:translateX(calc(0% - 32px)) translateY(calc(0% - 34.75px)) scale(0.75)}33%{animation-timing-function:cubic-bezier(0.5, 0, 0.701732, 0.495819);transform:translateX(calc(4% - 32px)) translateY(calc(0% - 34.75px)) scale(0.75)}66%{animation-timing-function:cubic-bezier(0.302435, 0.381352, 0.55, 0.956352);transform:translateX(calc(-4% - 32px)) translateY(calc(0% - 34.75px)) scale(0.75)}100%{transform:translateX(calc(0% - 32px)) translateY(calc(0% - 34.75px)) scale(0.75)}}[dir=rtl] .mat-mdc-form-field:not(.mat-form-field-no-animations) .mdc-text-field--with-leading-icon.mdc-text-field--outlined .mdc-floating-label--shake,.mat-mdc-form-field:not(.mat-form-field-no-animations) .mdc-text-field--with-leading-icon.mdc-text-field--outlined[dir=rtl] .mdc-floating-label--shake{animation:mdc-floating-label-shake-float-above-text-field-outlined-leading-icon 250ms 1}@keyframes mdc-floating-label-shake-float-above-text-field-outlined-leading-icon-rtl{0%{transform:translateX(calc(0% - -32px)) translateY(calc(0% - 34.75px)) scale(0.75)}33%{animation-timing-function:cubic-bezier(0.5, 0, 0.701732, 0.495819);transform:translateX(calc(4% - -32px)) translateY(calc(0% - 34.75px)) scale(0.75)}66%{animation-timing-function:cubic-bezier(0.302435, 0.381352, 0.55, 0.956352);transform:translateX(calc(-4% - -32px)) translateY(calc(0% - 34.75px)) scale(0.75)}100%{transform:translateX(calc(0% - -32px)) translateY(calc(0% - 34.75px)) scale(0.75)}}.mat-mdc-form-field:not(.mat-form-field-no-animations) .mdc-floating-label{transition:transform 150ms cubic-bezier(0.4, 0, 0.2, 1),color 150ms cubic-bezier(0.4, 0, 0.2, 1)}.mdc-floating-label--shake{animation:mdc-floating-label-shake-float-above-standard 250ms 1}@keyframes mdc-floating-label-shake-float-above-standard{0%{transform:translateX(calc(0% - 0%)) translateY(calc(0% - 106%)) scale(0.75)}33%{animation-timing-function:cubic-bezier(0.5, 0, 0.701732, 0.495819);transform:translateX(calc(4% - 0%)) translateY(calc(0% - 106%)) scale(0.75)}66%{animation-timing-function:cubic-bezier(0.302435, 0.381352, 0.55, 0.956352);transform:translateX(calc(-4% - 0%)) translateY(calc(0% - 106%)) scale(0.75)}100%{transform:translateX(calc(0% - 0%)) translateY(calc(0% - 106%)) scale(0.75)}}.mat-mdc-form-field:not(.mat-form-field-no-animations) .mdc-line-ripple::after{transition:transform 180ms cubic-bezier(0.4, 0, 0.2, 1),opacity 180ms cubic-bezier(0.4, 0, 0.2, 1)}.mdc-notched-outline .mdc-floating-label{max-width:calc(100% + 1px)}.mdc-notched-outline--upgraded .mdc-floating-label--float-above{max-width:calc(133.3333333333% + 1px)}\n'],
pb:2,data:{animation:[tA]},Ra:0});var cB=class{};cB.K=function(a){return new (a||cB)};cB.Ya=Of({type:cB});cB.Xa=Nd({imports:[mz,ht,ny,mz]});var eB={oa:pw,Ab:af(()=>dB),Eb:!0},fB=new df("mat-autocomplete-scroll-strategy",{ea:"root",ba:()=>{const a=lf(bw);return()=>a.C.C()}}),gB={oa:fB,Ac:[bw],Fc:function(a){return()=>a.C.C()}};function hB(a){let b;return(null==(b=a.ia)?void 0:b.defaultView)||window}
function iB(a){a.ud();a.G&&(a.qb&&a.yb.run(()=>{a.autocomplete.closed.emit()}),a.autocomplete.J===a&&(a.autocomplete.D=!1,a.autocomplete.J=null),a.G=!1,a.J=null,a.la&&a.la.ic()&&(a.la.detach(),a.Dc.unsubscribe()),a.Va(),a.gb||cn(a.Ba),a.F&&uy(a.F,"aria-owns",a.autocomplete.id))}function jB(a){const b=a.C.R;return!b.readOnly&&!b.disabled&&!a.Ti}
function kB(a,b){let c=a.la;if(c)Hu(a.P,a.sa()),c.ne({width:a.ua()});else{let e;a.Ic=new Vv(a.autocomplete.ta,a.Jc,{id:null==(e=a.D)?void 0:e.ph()?e.Ue:null});c=a.bb.create(lB(a));a.la=c;a.Nb=a.Xc.wb().subscribe(()=>{a.qb&&c&&c.ne({width:a.ua()})})}c&&!c.ic()&&(c.attach(a.Ic),a.Vd=b,a.O=null,a.Dc=a.Rd());b=a.qb;a.autocomplete.D=a.G=!0;a.autocomplete.J=a;let d;a.autocomplete.fa(null==(d=a.D)?void 0:d.color);a.Va();mB(a);a.qb&&b!==a.qb&&a.lb()}
function nB(a,b=!1){a.D&&"auto"===a.D.wg&&(b?a.D.na():a.D.wg="always",a.Ea=!0)}function lB(a){let b,c;const d=Ev(Mu(Rv(a.bb.position(),a.sa()),!1),!1);a.Kb(d);a.P=d;return new Au({uc:d,vc:a.Kc(),width:a.ua(),direction:null!=(c=a.Hc)?c:void 0,hb:null==(b=a.Gc)?void 0:b.Wh})}function mB(a){const b=a.C.R.closest('body > .cdk-overlay-container [aria-modal="true"]');if(b){var c=a.autocomplete.id;a.F&&uy(a.F,"aria-owns",c);sy(b,"aria-owns",c);a.F=b}}
var dB=class{constructor(a,b,c,d,e,f,g,k,m,p,t){this.C=a;this.bb=b;this.Jc=c;this.yb=d;this.Ba=e;this.Hc=g;this.D=k;this.ia=m;this.Xc=p;this.Gc=t;this.Ea=this.gb=!1;this.Nb=Ni.EMPTY;this.ja=!0;this.na=new cj;this.Ob=()=>{this.ja=this.ia.activeElement!==this.C.R||this.qb};this.N=()=>{};this.Zd=()=>{};this.position="auto";this.Lu="off";this.fa=new cj;this.Ua=lf(Wh);this.Oe="mat-mdc-autocomplete-panel-above";this.G=!1;this.ac=ik(()=>{const w=this.autocomplete?this.autocomplete.options:null;return w?
C(w.C,Ho(w),Io(()=>rk(...w.map(B=>B.jo)))):C(this.fa,Io(()=>this.ac))});this.td=w=>{if(27===w.keyCode&&!Bu(w)||38===w.keyCode&&Bu(w,"altKey")){if(this.J){let B;this.Lb(null!=(B=this.Sd)?B:"");this.J=null}this.na.next();this.Sa();w.stopPropagation();w.preventDefault()}};this.F=null;this.Kc=f}pd(){this.fa.next();this.fa.complete();const a=hB(this);"undefined"!==typeof a&&Mk(this.yb,()=>a.addEventListener("blur",this.Ob))}tc(a){a.position&&this.P&&(this.Kb(this.P),this.qb&&this.la.Wc())}va(){const a=
hB(this);"undefined"!==typeof a&&a.removeEventListener("blur",this.Ob);this.Nb.unsubscribe();this.gb=!0;this.Yc();this.na.complete();this.F&&(uy(this.F,"aria-owns",this.autocomplete.id),this.F=null)}get qb(){return this.G&&this.autocomplete.Zl}Wc(){this.G&&this.la.Wc()}get Wd(){return C(rk(this.ac,C(this.autocomplete.C.ca,tk(()=>this.G)),this.na,this.sd(),this.la?C(this.la.J,tk(()=>this.G)):jj()),xj(a=>a instanceof Tz?a:null))}get xh(){return this.autocomplete&&this.autocomplete.C?this.autocomplete.C.C:
null}sd(){return C(rk(lk(this.ia,"click"),lk(this.ia,"auxclick"),lk(this.ia,"touchend")),tk(a=>{a=St(a);const b=this.D?$A(this.D).R:null,c=this.fn?this.fn.fc.R:null;return this.G&&a!==this.C.R&&this.ia.activeElement!==this.C.R&&(!b||!b.contains(a))&&(!c||!c.contains(a))&&!!this.la&&!this.la.N.contains(a)}))}pi(a){Promise.resolve(null).then(()=>this.T(a))}Lg(a){this.N=a}Ij(a){this.Zd=a}gi(a){this.C.R.disabled=a}Se(a){const b=a.keyCode,c=Bu(a);27!==b||c||a.preventDefault();this.O=this.C.R.value;if(this.xh&&
13===b&&this.qb&&!c)this.xh.uh(),this.Sa(),a.preventDefault();else if(this.autocomplete){const d=this.autocomplete.C.C,e=38===b||40===b;9===b||e&&!c&&this.qb?Uy(this.autocomplete.C,a):e&&jB(this)&&this.Ja(this.O);if(e||this.autocomplete.C.C!==d)this.Pd(this.autocomplete.C.O||0),this.autocomplete.Um&&this.xh&&(this.J||(this.Sd=this.O),this.J=this.xh,this.T(this.xh.value))}}sp(a){var b=a.target;let c=b.value;"number"===b.type&&(c=""==c?null:parseFloat(c));if(this.ca!==c){this.ca=c;this.J=null;this.autocomplete&&
this.autocomplete.Kj||this.N(c);if(!c)this.aa(null,!1);else if(this.qb&&!this.autocomplete.Kj){var d;if(b=null==(d=this.autocomplete.options)?void 0:d.find(e=>e.selected))d=this.sb(b.value),c!==d&&Uz(b,!1)}if(jB(this)&&this.ia.activeElement===a.target){let e;a=null!=(e=this.O)?e:this.C.R.value;this.O=null;this.Ja(a)}}}Xz(){this.ja?jB(this)&&(this.ca=this.C.R.value,kB(this,this.ca),nB(this,!0)):this.ja=!0}pk(){jB(this)&&!this.qb&&this.Ja()}ud(){this.Ea&&(this.D&&(this.D.wg="auto"),this.Ea=!1)}Rd(){const a=
new Xi(c=>{al(()=>{c.next()},{xa:this.Ua})}),b=C(this.autocomplete.options.C,Po(()=>Iu(this.P)),Xn());return C(rk(a,b),Io(()=>this.yb.run(()=>{const c=this.qb;this.Sa();this.Va();cn(this.Ba);this.qb&&this.la.Wc();c!==this.qb&&(this.qb?this.lb():this.autocomplete.closed.emit());return this.Wd})),fo()).subscribe(c=>this.Qd(c))}lb(){this.autocomplete.hf.emit()}Yc(){this.la&&(iB(this),this.la.dispose(),this.la=null)}sb(a){const b=this.autocomplete;return b&&b.wq?b.wq(a):a}T(a){const b=this.sb(a);null==
a&&this.aa(null,!1);this.Lb(null!=b?b:"")}Lb(a){this.D?this.D.Hb.value=a:this.C.R.value=a;this.ca=a}Qd(a){const b=this.autocomplete;(a=a?a.source:this.J)?(this.aa(a),this.T(a.value),this.N(a.value),b.aa(a),this.C.R.focus()):b.Kj&&this.C.R.value!==this.Vd&&(this.aa(null),this.T(null),b.hg?C(b.hg,fo()).subscribe(()=>this.N(null)):this.N(null));iB(this)}aa(a,b){let c,d;null==(c=this.autocomplete)||null==(d=c.options)||d.forEach(e=>{e!==a&&e.selected&&Uz(e,b)})}Ja(a=this.C.R.value){kB(this,a);nB(this);
this.F&&sy(this.F,"aria-owns",this.autocomplete.id)}Va(){this.autocomplete.O();if(this.qb){var a=this.la;this.wa||(this.wa=a.ef().subscribe(this.td));this.Pa||(this.Pa=a.Ll().subscribe())}else{null==(a=this.wa)||a.unsubscribe();let b;null==(b=this.Pa)||b.unsubscribe();this.wa=this.Pa=null}}Kb(a){var b=[{cb:"start",eb:"bottom",Na:"start",Ta:"top"},{cb:"end",eb:"bottom",Na:"end",Ta:"top"}];const c=[{cb:"start",eb:"top",Na:"start",Ta:"bottom",hb:"mat-mdc-autocomplete-panel-above"},{cb:"end",eb:"top",
Na:"end",Ta:"bottom",hb:"mat-mdc-autocomplete-panel-above"}];b="above"===this.position?c:"below"===this.position?b:[...b,...c];Ku(a,b)}sa(){return this.fn?this.fn.fc:this.D?$A(this.D):this.C}ua(){return this.autocomplete.Gg||this.md()}md(){return this.sa().R.getBoundingClientRect().width}Sa(){const a=this.autocomplete;if(a.Sm){let b=-1;for(let c=0;c<a.options.length;c++)if(!a.options.get(c).disabled){b=c;break}a.C.D(b)}else a.C.D(-1)}Pd(a){const b=this.autocomplete,c=Wz(a,b.options,b.Hr);0===a&&1===
c?b.N(0):b.Wf&&(a=b.options.toArray()[a])&&(a=a.qe(),b.N(Xz(a.offsetTop,a.offsetHeight,b.ca(),b.Wf.R.offsetHeight)))}};dB.K=function(a){return new (a||dB)(z(si),z(bw),z(kp),z(Qk),z(Sr),z(fB),z(ou,8),z(XA,9),z(jt,8),z(wu),z(lA,8))};
dB.Fa=Qf({type:dB,ha:[["input","matAutocomplete",""],["textarea","matAutocomplete",""]],Ka:[1,"mat-mdc-autocomplete-trigger"],Za:7,Ia:function(a,b){a&1&&V("focusin",function(){return b.Xz()})("blur",function(){return b.Zd()})("input",function(c){return b.sp(c)})("keydown",function(c){return b.Se(c)})("click",function(){return b.pk()});a&2&&Kp("autocomplete",b.Lu)("role",b.Ti?null:"combobox")("aria-autocomplete",b.Ti?null:"list")("aria-activedescendant",b.qb&&b.xh?b.xh.id:null)("aria-expanded",b.Ti?
null:b.qb.toString())("aria-controls",b.Ti||!b.qb?null:null==b.autocomplete?null:b.autocomplete.id)("aria-haspopup",b.Ti?null:"listbox")},inputs:{autocomplete:[0,"matAutocomplete","autocomplete"],position:[0,"matAutocompletePosition","position"],fn:[0,"matAutocompleteConnectedTo","connectedTo"],Lu:[0,"autocomplete","autocompleteAttribute"],Ti:[2,"matAutocompleteDisabled","autocompleteDisabled",zs]},ib:["matAutocompleteTrigger"],ka:!0,features:[Kr([eB]),Jp,Dg]});var oB=class{};oB.K=function(a){return new (a||oB)};oB.Ya=Of({type:oB});oB.Xa=Nd({Wa:[gB],imports:[jw,$z,mz,ht,xu,$z,mz]});var pB=class{};pB.K=function(a){return new (a||pB)};pB.Ya=Of({type:pB});pB.Xa=Nd({imports:[oB]});var qB=new df("GMAT_BUTTON_OPTIONS",{ea:"root",ba:()=>({sc:!1})});var rB=new df("MAT_BUTTON_CONFIG");
const sB=[{yh:"mat-button",Qh:["mdc-button","mat-mdc-button"]},{yh:"mat-flat-button",Qh:["mdc-button","mdc-button--unelevated","mat-mdc-unelevated-button"]},{yh:"mat-raised-button",Qh:["mdc-button","mdc-button--raised","mat-mdc-raised-button"]},{yh:"mat-stroked-button",Qh:["mdc-button","mdc-button--outlined","mat-mdc-outlined-button"]},{yh:"mat-fab",Qh:["mdc-fab","mat-mdc-fab"]},{yh:"mat-mini-fab",Qh:["mdc-fab","mdc-fab--mini","mat-mdc-mini-fab"]},{yh:"mat-icon-button",Qh:["mdc-icon-button","mat-mdc-icon-button"]}];
var tB=class{get vx(){var a;if(null==(a=this.C))a=void 0;else{var b=this.ra.R;a=a.C.get(b)||a.F(b)}return a}set vx(a){let b;null==(b=this.C)||eA(b,this.ra.R,a)}get Nc(){return this.F}set Nc(a){this.F=a;this.G()}get disabled(){return this.Ib}set disabled(a){this.Ib=a;this.G()}constructor(a,b,c,d){this.ra=a;this.Ma=b;this.qa=c;this.ad=d;this.D=lf(Ox);this.C=lf(fA);this.Ib=this.F=this.Yt=!1;const e=lf(rB,{optional:!0}),f=a.R,g=f.classList;let k;this.gj=null!=(k=null==e?void 0:e.gj)?k:!1;let m;null==
(m=this.C)||dA(m,f,{className:"mat-mdc-button-ripple"});for(const {yh:p,Qh:t}of sB)f.hasAttribute(p)&&g.add(...t)}pd(){Lx(this.D,this.ra,!0)}va(){Mx(this.D,this.ra);let a;null==(a=this.C)||cA(a,this.ra.R)}focus(a="program",b){a?Nx(this.D,this.ra.R,a,b):this.ra.R.focus(b)}Dt(){return null!=this.ariaDisabled?this.ariaDisabled:this.disabled&&this.gj?!0:null}Et(){return this.gj||!this.disabled?null:!0}G(){var a;if(null!=(a=this.C)){var b=this.ra.R,c=this.Nc||this.disabled;(a=a.C.get(b))?a.disabled=c:
c?b.setAttribute("mat-ripple-loader-disabled",""):b.removeAttribute("mat-ripple-loader-disabled")}}};tB.K=function(){throw Error("invalid");};tB.Fa=Qf({type:tB,inputs:{color:"color",Nc:[2,"disableRipple","disableRipple",zs],disabled:[2,"disabled","disabled",zs],ariaDisabled:[2,"aria-disabled","ariaDisabled",zs],gj:[2,"disabledInteractive","disabledInteractive",zs]},features:[Jp]});const uB=[[["",8,"material-icons",3,"iconPositionEnd",""],["mat-icon",3,"iconPositionEnd",""],["","matButtonIcon","",3,"iconPositionEnd",""]],"*",[["","iconPositionEnd","",8,"material-icons"],["mat-icon","iconPositionEnd",""],["","matButtonIcon","","iconPositionEnd",""]]];var vB=class extends tB{};vB.K=function(a){return new (a||vB)(z(si),z(Vt),z(Qk),z(mi,8))};
vB.Da=Gf({type:vB,ha:[["button","mat-button",""],["button","mat-raised-button",""],["button","mat-flat-button",""],["button","mat-stroked-button",""]],Za:14,Ia:function(a,b){a&2&&(Kp("disabled",b.Et())("aria-disabled",b.Dt()),qq(tq,b.color?"mat-"+b.color:""),nq("mat-mdc-button-disabled",b.disabled)("mat-mdc-button-disabled-interactive",b.gj)("_mat-animation-noopable","NoopAnimations"===b.ad)("mat-unthemed",!b.color)("mat-mdc-button-base",!0))},ib:["matButton"],ka:!0,features:[Cp,Mr],attrs:["mat-button",
""],Zb:[".material-icons:not([iconPositionEnd]), mat-icon:not([iconPositionEnd]), [matButtonIcon]:not([iconPositionEnd])","*",".material-icons[iconPositionEnd], mat-icon[iconPositionEnd], [matButtonIcon][iconPositionEnd]"],za:7,Aa:4,Ca:[[1,"mat-mdc-button-persistent-ripple"],[1,"mdc-button__label"],[1,"mat-mdc-focus-indicator"],[1,"mat-mdc-button-touch-target"]],ta:function(a,b){a&1&&(Xq(uB),Mq(0,"span",0),Yq(1),R(2,"span",1),Yq(3,1),S(),Yq(4,2),Mq(5,"span",2)(6,"span",3));a&2&&nq("mdc-button__ripple",
!b.Yt)("mdc-fab__ripple",b.Yt)},styles:['.mdc-button{position:relative;display:inline-flex;align-items:center;justify-content:center;box-sizing:border-box;min-width:64px;border:none;outline:none;line-height:inherit;user-select:none;-webkit-appearance:none;overflow:visible;vertical-align:middle;background:rgba(0,0,0,0)}.mdc-button .mdc-elevation-overlay{width:100%;height:100%;top:0;left:0}.mdc-button::-moz-focus-inner{padding:0;border:0}.mdc-button:active{outline:none}.mdc-button:hover{cursor:pointer}.mdc-button:disabled{cursor:default;pointer-events:none}.mdc-button[hidden]{display:none}.mdc-button .mdc-button__icon{margin-left:0;margin-right:8px;display:inline-block;position:relative;vertical-align:top}[dir=rtl] .mdc-button .mdc-button__icon,.mdc-button .mdc-button__icon[dir=rtl]{margin-left:8px;margin-right:0}.mdc-button .mdc-button__progress-indicator{font-size:0;position:absolute;transform:translate(-50%, -50%);top:50%;left:50%;line-height:initial}.mdc-button .mdc-button__label{position:relative}.mdc-button .mdc-button__focus-ring{pointer-events:none;border:2px solid rgba(0,0,0,0);border-radius:6px;box-sizing:content-box;position:absolute;top:50%;left:50%;transform:translate(-50%, -50%);height:calc(100% + 4px);width:calc(100% + 4px);display:none}@media screen and (forced-colors: active){.mdc-button .mdc-button__focus-ring{border-color:CanvasText}}.mdc-button .mdc-button__focus-ring::after{content:"";border:2px solid rgba(0,0,0,0);border-radius:8px;display:block;position:absolute;top:50%;left:50%;transform:translate(-50%, -50%);height:calc(100% + 4px);width:calc(100% + 4px)}@media screen and (forced-colors: active){.mdc-button .mdc-button__focus-ring::after{border-color:CanvasText}}@media screen and (forced-colors: active){.mdc-button.mdc-ripple-upgraded--background-focused .mdc-button__focus-ring,.mdc-button:not(.mdc-ripple-upgraded):focus .mdc-button__focus-ring{display:block}}.mdc-button .mdc-button__touch{position:absolute;top:50%;height:48px;left:0;right:0;transform:translateY(-50%)}.mdc-button__label+.mdc-button__icon{margin-left:8px;margin-right:0}[dir=rtl] .mdc-button__label+.mdc-button__icon,.mdc-button__label+.mdc-button__icon[dir=rtl]{margin-left:0;margin-right:8px}svg.mdc-button__icon{fill:currentColor}.mdc-button--touch{margin-top:6px;margin-bottom:6px}.mdc-button{padding:0 8px 0 8px}.mdc-button--unelevated{transition:box-shadow 280ms cubic-bezier(0.4, 0, 0.2, 1);padding:0 16px 0 16px}.mdc-button--unelevated.mdc-button--icon-trailing{padding:0 12px 0 16px}.mdc-button--unelevated.mdc-button--icon-leading{padding:0 16px 0 12px}.mdc-button--raised{transition:box-shadow 280ms cubic-bezier(0.4, 0, 0.2, 1);padding:0 16px 0 16px}.mdc-button--raised.mdc-button--icon-trailing{padding:0 12px 0 16px}.mdc-button--raised.mdc-button--icon-leading{padding:0 16px 0 12px}.mdc-button--outlined{border-style:solid;transition:border 280ms cubic-bezier(0.4, 0, 0.2, 1)}.mdc-button--outlined .mdc-button__ripple{border-style:solid;border-color:rgba(0,0,0,0)}.mat-mdc-button{font-family:var(--mdc-text-button-label-text-font);font-size:var(--mdc-text-button-label-text-size);letter-spacing:var(--mdc-text-button-label-text-tracking);font-weight:var(--mdc-text-button-label-text-weight);text-transform:var(--mdc-text-button-label-text-transform);height:var(--mdc-text-button-container-height);border-radius:var(--mdc-text-button-container-shape);padding:0 var(--mat-text-button-horizontal-padding, 8px)}.mat-mdc-button:not(:disabled){color:var(--mdc-text-button-label-text-color)}.mat-mdc-button:disabled{color:var(--mdc-text-button-disabled-label-text-color)}.mat-mdc-button .mdc-button__ripple{border-radius:var(--mdc-text-button-container-shape)}.mat-mdc-button:has(.material-icons,mat-icon,[matButtonIcon]){padding:0 var(--mat-text-button-with-icon-horizontal-padding, 8px)}.mat-mdc-button>.mat-icon{margin-right:var(--mat-text-button-icon-spacing, 8px);margin-left:var(--mat-text-button-icon-offset, 0)}[dir=rtl] .mat-mdc-button>.mat-icon{margin-right:var(--mat-text-button-icon-offset, 0);margin-left:var(--mat-text-button-icon-spacing, 8px)}.mat-mdc-button .mdc-button__label+.mat-icon{margin-right:var(--mat-text-button-icon-offset, 0);margin-left:var(--mat-text-button-icon-spacing, 8px)}[dir=rtl] .mat-mdc-button .mdc-button__label+.mat-icon{margin-right:var(--mat-text-button-icon-spacing, 8px);margin-left:var(--mat-text-button-icon-offset, 0)}.mat-mdc-button .mat-ripple-element{background-color:var(--mat-text-button-ripple-color)}.mat-mdc-button .mat-mdc-button-persistent-ripple::before{background-color:var(--mat-text-button-state-layer-color)}.mat-mdc-button.mat-mdc-button-disabled .mat-mdc-button-persistent-ripple::before{background-color:var(--mat-text-button-disabled-state-layer-color)}.mat-mdc-button:hover .mat-mdc-button-persistent-ripple::before{opacity:var(--mat-text-button-hover-state-layer-opacity)}.mat-mdc-button.cdk-program-focused .mat-mdc-button-persistent-ripple::before,.mat-mdc-button.cdk-keyboard-focused .mat-mdc-button-persistent-ripple::before,.mat-mdc-button.mat-mdc-button-disabled-interactive:focus .mat-mdc-button-persistent-ripple::before{opacity:var(--mat-text-button-focus-state-layer-opacity)}.mat-mdc-button:active .mat-mdc-button-persistent-ripple::before{opacity:var(--mat-text-button-pressed-state-layer-opacity)}.mat-mdc-button .mat-mdc-button-touch-target{position:absolute;top:50%;height:48px;left:0;right:0;transform:translateY(-50%);display:var(--mat-text-button-touch-target-display)}.mat-mdc-button[disabled],.mat-mdc-button.mat-mdc-button-disabled{cursor:default;pointer-events:none;color:var(--mdc-text-button-disabled-label-text-color)}.mat-mdc-button.mat-mdc-button-disabled-interactive{pointer-events:auto}.mat-mdc-unelevated-button{font-family:var(--mdc-filled-button-label-text-font);font-size:var(--mdc-filled-button-label-text-size);letter-spacing:var(--mdc-filled-button-label-text-tracking);font-weight:var(--mdc-filled-button-label-text-weight);text-transform:var(--mdc-filled-button-label-text-transform);height:var(--mdc-filled-button-container-height);border-radius:var(--mdc-filled-button-container-shape);padding:0 var(--mat-filled-button-horizontal-padding, 16px)}.mat-mdc-unelevated-button:not(:disabled){background-color:var(--mdc-filled-button-container-color)}.mat-mdc-unelevated-button:disabled{background-color:var(--mdc-filled-button-disabled-container-color)}.mat-mdc-unelevated-button:not(:disabled){color:var(--mdc-filled-button-label-text-color)}.mat-mdc-unelevated-button:disabled{color:var(--mdc-filled-button-disabled-label-text-color)}.mat-mdc-unelevated-button .mdc-button__ripple{border-radius:var(--mdc-filled-button-container-shape)}.mat-mdc-unelevated-button>.mat-icon{margin-right:var(--mat-filled-button-icon-spacing, 8px);margin-left:var(--mat-filled-button-icon-offset, -4px)}[dir=rtl] .mat-mdc-unelevated-button>.mat-icon{margin-right:var(--mat-filled-button-icon-offset, -4px);margin-left:var(--mat-filled-button-icon-spacing, 8px)}.mat-mdc-unelevated-button .mdc-button__label+.mat-icon{margin-right:var(--mat-filled-button-icon-offset, -4px);margin-left:var(--mat-filled-button-icon-spacing, 8px)}[dir=rtl] .mat-mdc-unelevated-button .mdc-button__label+.mat-icon{margin-right:var(--mat-filled-button-icon-spacing, 8px);margin-left:var(--mat-filled-button-icon-offset, -4px)}.mat-mdc-unelevated-button .mat-ripple-element{background-color:var(--mat-filled-button-ripple-color)}.mat-mdc-unelevated-button .mat-mdc-button-persistent-ripple::before{background-color:var(--mat-filled-button-state-layer-color)}.mat-mdc-unelevated-button.mat-mdc-button-disabled .mat-mdc-button-persistent-ripple::before{background-color:var(--mat-filled-button-disabled-state-layer-color)}.mat-mdc-unelevated-button:hover .mat-mdc-button-persistent-ripple::before{opacity:var(--mat-filled-button-hover-state-layer-opacity)}.mat-mdc-unelevated-button.cdk-program-focused .mat-mdc-button-persistent-ripple::before,.mat-mdc-unelevated-button.cdk-keyboard-focused .mat-mdc-button-persistent-ripple::before,.mat-mdc-unelevated-button.mat-mdc-button-disabled-interactive:focus .mat-mdc-button-persistent-ripple::before{opacity:var(--mat-filled-button-focus-state-layer-opacity)}.mat-mdc-unelevated-button:active .mat-mdc-button-persistent-ripple::before{opacity:var(--mat-filled-button-pressed-state-layer-opacity)}.mat-mdc-unelevated-button .mat-mdc-button-touch-target{position:absolute;top:50%;height:48px;left:0;right:0;transform:translateY(-50%);display:var(--mat-filled-button-touch-target-display)}.mat-mdc-unelevated-button[disabled],.mat-mdc-unelevated-button.mat-mdc-button-disabled{cursor:default;pointer-events:none;color:var(--mdc-filled-button-disabled-label-text-color);background-color:var(--mdc-filled-button-disabled-container-color)}.mat-mdc-unelevated-button.mat-mdc-button-disabled-interactive{pointer-events:auto}.mat-mdc-raised-button{font-family:var(--mdc-protected-button-label-text-font);font-size:var(--mdc-protected-button-label-text-size);letter-spacing:var(--mdc-protected-button-label-text-tracking);font-weight:var(--mdc-protected-button-label-text-weight);text-transform:var(--mdc-protected-button-label-text-transform);height:var(--mdc-protected-button-container-height);border-radius:var(--mdc-protected-button-container-shape);padding:0 var(--mat-protected-button-horizontal-padding, 16px);box-shadow:var(--mdc-protected-button-container-elevation-shadow)}.mat-mdc-raised-button:not(:disabled){background-color:var(--mdc-protected-button-container-color)}.mat-mdc-raised-button:disabled{background-color:var(--mdc-protected-button-disabled-container-color)}.mat-mdc-raised-button:not(:disabled){color:var(--mdc-protected-button-label-text-color)}.mat-mdc-raised-button:disabled{color:var(--mdc-protected-button-disabled-label-text-color)}.mat-mdc-raised-button .mdc-button__ripple{border-radius:var(--mdc-protected-button-container-shape)}.mat-mdc-raised-button>.mat-icon{margin-right:var(--mat-protected-button-icon-spacing, 8px);margin-left:var(--mat-protected-button-icon-offset, -4px)}[dir=rtl] .mat-mdc-raised-button>.mat-icon{margin-right:var(--mat-protected-button-icon-offset, -4px);margin-left:var(--mat-protected-button-icon-spacing, 8px)}.mat-mdc-raised-button .mdc-button__label+.mat-icon{margin-right:var(--mat-protected-button-icon-offset, -4px);margin-left:var(--mat-protected-button-icon-spacing, 8px)}[dir=rtl] .mat-mdc-raised-button .mdc-button__label+.mat-icon{margin-right:var(--mat-protected-button-icon-spacing, 8px);margin-left:var(--mat-protected-button-icon-offset, -4px)}.mat-mdc-raised-button .mat-ripple-element{background-color:var(--mat-protected-button-ripple-color)}.mat-mdc-raised-button .mat-mdc-button-persistent-ripple::before{background-color:var(--mat-protected-button-state-layer-color)}.mat-mdc-raised-button.mat-mdc-button-disabled .mat-mdc-button-persistent-ripple::before{background-color:var(--mat-protected-button-disabled-state-layer-color)}.mat-mdc-raised-button:hover .mat-mdc-button-persistent-ripple::before{opacity:var(--mat-protected-button-hover-state-layer-opacity)}.mat-mdc-raised-button.cdk-program-focused .mat-mdc-button-persistent-ripple::before,.mat-mdc-raised-button.cdk-keyboard-focused .mat-mdc-button-persistent-ripple::before,.mat-mdc-raised-button.mat-mdc-button-disabled-interactive:focus .mat-mdc-button-persistent-ripple::before{opacity:var(--mat-protected-button-focus-state-layer-opacity)}.mat-mdc-raised-button:active .mat-mdc-button-persistent-ripple::before{opacity:var(--mat-protected-button-pressed-state-layer-opacity)}.mat-mdc-raised-button .mat-mdc-button-touch-target{position:absolute;top:50%;height:48px;left:0;right:0;transform:translateY(-50%);display:var(--mat-protected-button-touch-target-display)}.mat-mdc-raised-button:hover{box-shadow:var(--mdc-protected-button-hover-container-elevation-shadow)}.mat-mdc-raised-button:focus{box-shadow:var(--mdc-protected-button-focus-container-elevation-shadow)}.mat-mdc-raised-button:active,.mat-mdc-raised-button:focus:active{box-shadow:var(--mdc-protected-button-pressed-container-elevation-shadow)}.mat-mdc-raised-button[disabled],.mat-mdc-raised-button.mat-mdc-button-disabled{cursor:default;pointer-events:none;color:var(--mdc-protected-button-disabled-label-text-color);background-color:var(--mdc-protected-button-disabled-container-color)}.mat-mdc-raised-button[disabled].mat-mdc-button-disabled,.mat-mdc-raised-button.mat-mdc-button-disabled.mat-mdc-button-disabled{box-shadow:var(--mdc-protected-button-disabled-container-elevation-shadow)}.mat-mdc-raised-button.mat-mdc-button-disabled-interactive{pointer-events:auto}.mat-mdc-outlined-button{font-family:var(--mdc-outlined-button-label-text-font);font-size:var(--mdc-outlined-button-label-text-size);letter-spacing:var(--mdc-outlined-button-label-text-tracking);font-weight:var(--mdc-outlined-button-label-text-weight);text-transform:var(--mdc-outlined-button-label-text-transform);height:var(--mdc-outlined-button-container-height);border-radius:var(--mdc-outlined-button-container-shape);padding:0 15px 0 15px;border-width:var(--mdc-outlined-button-outline-width);padding:0 var(--mat-outlined-button-horizontal-padding, 15px)}.mat-mdc-outlined-button:not(:disabled){color:var(--mdc-outlined-button-label-text-color)}.mat-mdc-outlined-button:disabled{color:var(--mdc-outlined-button-disabled-label-text-color)}.mat-mdc-outlined-button .mdc-button__ripple{border-radius:var(--mdc-outlined-button-container-shape)}.mat-mdc-outlined-button:not(:disabled){border-color:var(--mdc-outlined-button-outline-color)}.mat-mdc-outlined-button:disabled{border-color:var(--mdc-outlined-button-disabled-outline-color)}.mat-mdc-outlined-button.mdc-button--icon-trailing{padding:0 11px 0 15px}.mat-mdc-outlined-button.mdc-button--icon-leading{padding:0 15px 0 11px}.mat-mdc-outlined-button .mdc-button__ripple{top:-1px;left:-1px;bottom:-1px;right:-1px;border-width:var(--mdc-outlined-button-outline-width)}.mat-mdc-outlined-button .mdc-button__touch{left:calc(-1 * var(--mdc-outlined-button-outline-width));width:calc(100% + 2 * var(--mdc-outlined-button-outline-width))}.mat-mdc-outlined-button>.mat-icon{margin-right:var(--mat-outlined-button-icon-spacing, 8px);margin-left:var(--mat-outlined-button-icon-offset, -4px)}[dir=rtl] .mat-mdc-outlined-button>.mat-icon{margin-right:var(--mat-outlined-button-icon-offset, -4px);margin-left:var(--mat-outlined-button-icon-spacing, 8px)}.mat-mdc-outlined-button .mdc-button__label+.mat-icon{margin-right:var(--mat-outlined-button-icon-offset, -4px);margin-left:var(--mat-outlined-button-icon-spacing, 8px)}[dir=rtl] .mat-mdc-outlined-button .mdc-button__label+.mat-icon{margin-right:var(--mat-outlined-button-icon-spacing, 8px);margin-left:var(--mat-outlined-button-icon-offset, -4px)}.mat-mdc-outlined-button .mat-ripple-element{background-color:var(--mat-outlined-button-ripple-color)}.mat-mdc-outlined-button .mat-mdc-button-persistent-ripple::before{background-color:var(--mat-outlined-button-state-layer-color)}.mat-mdc-outlined-button.mat-mdc-button-disabled .mat-mdc-button-persistent-ripple::before{background-color:var(--mat-outlined-button-disabled-state-layer-color)}.mat-mdc-outlined-button:hover .mat-mdc-button-persistent-ripple::before{opacity:var(--mat-outlined-button-hover-state-layer-opacity)}.mat-mdc-outlined-button.cdk-program-focused .mat-mdc-button-persistent-ripple::before,.mat-mdc-outlined-button.cdk-keyboard-focused .mat-mdc-button-persistent-ripple::before,.mat-mdc-outlined-button.mat-mdc-button-disabled-interactive:focus .mat-mdc-button-persistent-ripple::before{opacity:var(--mat-outlined-button-focus-state-layer-opacity)}.mat-mdc-outlined-button:active .mat-mdc-button-persistent-ripple::before{opacity:var(--mat-outlined-button-pressed-state-layer-opacity)}.mat-mdc-outlined-button .mat-mdc-button-touch-target{position:absolute;top:50%;height:48px;left:0;right:0;transform:translateY(-50%);display:var(--mat-outlined-button-touch-target-display)}.mat-mdc-outlined-button[disabled],.mat-mdc-outlined-button.mat-mdc-button-disabled{cursor:default;pointer-events:none;color:var(--mdc-outlined-button-disabled-label-text-color);border-color:var(--mdc-outlined-button-disabled-outline-color)}.mat-mdc-outlined-button.mat-mdc-button-disabled-interactive{pointer-events:auto}.mat-mdc-button-base{text-decoration:none}.mat-mdc-button,.mat-mdc-unelevated-button,.mat-mdc-raised-button,.mat-mdc-outlined-button{-webkit-tap-highlight-color:rgba(0,0,0,0)}.mat-mdc-button .mat-mdc-button-ripple,.mat-mdc-button .mat-mdc-button-persistent-ripple,.mat-mdc-button .mat-mdc-button-persistent-ripple::before,.mat-mdc-unelevated-button .mat-mdc-button-ripple,.mat-mdc-unelevated-button .mat-mdc-button-persistent-ripple,.mat-mdc-unelevated-button .mat-mdc-button-persistent-ripple::before,.mat-mdc-raised-button .mat-mdc-button-ripple,.mat-mdc-raised-button .mat-mdc-button-persistent-ripple,.mat-mdc-raised-button .mat-mdc-button-persistent-ripple::before,.mat-mdc-outlined-button .mat-mdc-button-ripple,.mat-mdc-outlined-button .mat-mdc-button-persistent-ripple,.mat-mdc-outlined-button .mat-mdc-button-persistent-ripple::before{top:0;left:0;right:0;bottom:0;position:absolute;pointer-events:none;border-radius:inherit}.mat-mdc-button .mat-mdc-button-ripple,.mat-mdc-unelevated-button .mat-mdc-button-ripple,.mat-mdc-raised-button .mat-mdc-button-ripple,.mat-mdc-outlined-button .mat-mdc-button-ripple{overflow:hidden}.mat-mdc-button .mat-mdc-button-persistent-ripple::before,.mat-mdc-unelevated-button .mat-mdc-button-persistent-ripple::before,.mat-mdc-raised-button .mat-mdc-button-persistent-ripple::before,.mat-mdc-outlined-button .mat-mdc-button-persistent-ripple::before{content:"";opacity:0}.mat-mdc-button .mdc-button__label,.mat-mdc-unelevated-button .mdc-button__label,.mat-mdc-raised-button .mdc-button__label,.mat-mdc-outlined-button .mdc-button__label{z-index:1}.mat-mdc-button .mat-mdc-focus-indicator,.mat-mdc-unelevated-button .mat-mdc-focus-indicator,.mat-mdc-raised-button .mat-mdc-focus-indicator,.mat-mdc-outlined-button .mat-mdc-focus-indicator{top:0;left:0;right:0;bottom:0;position:absolute}.mat-mdc-button:focus .mat-mdc-focus-indicator::before,.mat-mdc-unelevated-button:focus .mat-mdc-focus-indicator::before,.mat-mdc-raised-button:focus .mat-mdc-focus-indicator::before,.mat-mdc-outlined-button:focus .mat-mdc-focus-indicator::before{content:""}.mat-mdc-button._mat-animation-noopable,.mat-mdc-unelevated-button._mat-animation-noopable,.mat-mdc-raised-button._mat-animation-noopable,.mat-mdc-outlined-button._mat-animation-noopable{transition:none !important;animation:none !important}.mat-mdc-button>.mat-icon,.mat-mdc-unelevated-button>.mat-icon,.mat-mdc-raised-button>.mat-icon,.mat-mdc-outlined-button>.mat-icon{display:inline-block;position:relative;vertical-align:top;font-size:1.125rem;height:1.125rem;width:1.125rem}.mat-mdc-outlined-button .mat-mdc-button-ripple,.mat-mdc-outlined-button .mdc-button__ripple{top:-1px;left:-1px;bottom:-1px;right:-1px;border-width:-1px}.mat-mdc-unelevated-button .mat-mdc-focus-indicator::before,.mat-mdc-raised-button .mat-mdc-focus-indicator::before{margin:calc(calc(var(--mat-mdc-focus-indicator-border-width, 3px) + 2px)*-1)}.mat-mdc-outlined-button .mat-mdc-focus-indicator::before{margin:calc(calc(var(--mat-mdc-focus-indicator-border-width, 3px) + 3px)*-1)}\n',
".cdk-high-contrast-active .mat-mdc-button:not(.mdc-button--outlined),.cdk-high-contrast-active .mat-mdc-unelevated-button:not(.mdc-button--outlined),.cdk-high-contrast-active .mat-mdc-raised-button:not(.mdc-button--outlined),.cdk-high-contrast-active .mat-mdc-outlined-button:not(.mdc-button--outlined),.cdk-high-contrast-active .mat-mdc-icon-button{outline:solid 1px}\n"],pb:2,Ra:0});var xB=new df("mat-mdc-fab-default-options",{ea:"root",ba:wB});function wB(){return{color:"accent"}};var yB=class extends tB{constructor(a,b,c,d){super(a,b,c,d);dA(this.C,this.ra.R,{Wi:!0})}};yB.K=function(a){return new (a||yB)(z(si),z(Vt),z(Qk),z(mi,8))};
yB.Da=Gf({type:yB,ha:[["button","mat-icon-button",""]],Za:14,Ia:function(a,b){a&2&&(Kp("disabled",b.Et())("aria-disabled",b.Dt()),qq(tq,b.color?"mat-"+b.color:""),nq("mat-mdc-button-disabled",b.disabled)("mat-mdc-button-disabled-interactive",b.gj)("_mat-animation-noopable","NoopAnimations"===b.ad)("mat-unthemed",!b.color)("mat-mdc-button-base",!0))},ib:["matButton"],ka:!0,features:[Cp,Mr],attrs:["mat-icon-button",""],Zb:["*"],za:4,Aa:0,Ca:[[1,"mat-mdc-button-persistent-ripple","mdc-icon-button__ripple"],
[1,"mat-mdc-focus-indicator"],[1,"mat-mdc-button-touch-target"]],ta:function(a){a&1&&(Xq(),Mq(0,"span",0),Yq(1),Mq(2,"span",1)(3,"span",2))},styles:['.mdc-icon-button{display:inline-block;position:relative;box-sizing:border-box;border:none;outline:none;background-color:rgba(0,0,0,0);fill:currentColor;color:inherit;text-decoration:none;cursor:pointer;user-select:none;z-index:0;overflow:visible}.mdc-icon-button .mdc-icon-button__touch{position:absolute;top:50%;height:48px;left:50%;width:48px;transform:translate(-50%, -50%)}@media screen and (forced-colors: active){.mdc-icon-button.mdc-ripple-upgraded--background-focused .mdc-icon-button__focus-ring,.mdc-icon-button:not(.mdc-ripple-upgraded):focus .mdc-icon-button__focus-ring{display:block}}.mdc-icon-button:disabled{cursor:default;pointer-events:none}.mdc-icon-button[hidden]{display:none}.mdc-icon-button--display-flex{align-items:center;display:inline-flex;justify-content:center}.mdc-icon-button__focus-ring{pointer-events:none;border:2px solid rgba(0,0,0,0);border-radius:6px;box-sizing:content-box;position:absolute;top:50%;left:50%;transform:translate(-50%, -50%);height:100%;width:100%;display:none}@media screen and (forced-colors: active){.mdc-icon-button__focus-ring{border-color:CanvasText}}.mdc-icon-button__focus-ring::after{content:"";border:2px solid rgba(0,0,0,0);border-radius:8px;display:block;position:absolute;top:50%;left:50%;transform:translate(-50%, -50%);height:calc(100% + 4px);width:calc(100% + 4px)}@media screen and (forced-colors: active){.mdc-icon-button__focus-ring::after{border-color:CanvasText}}.mdc-icon-button__icon{display:inline-block}.mdc-icon-button__icon.mdc-icon-button__icon--on{display:none}.mdc-icon-button--on .mdc-icon-button__icon{display:none}.mdc-icon-button--on .mdc-icon-button__icon.mdc-icon-button__icon--on{display:inline-block}.mdc-icon-button__link{height:100%;left:0;outline:none;position:absolute;top:0;width:100%}.mat-mdc-icon-button{color:var(--mdc-icon-button-icon-color)}.mat-mdc-icon-button .mdc-button__icon{font-size:var(--mdc-icon-button-icon-size)}.mat-mdc-icon-button svg,.mat-mdc-icon-button img{width:var(--mdc-icon-button-icon-size);height:var(--mdc-icon-button-icon-size)}.mat-mdc-icon-button:disabled{color:var(--mdc-icon-button-disabled-icon-color)}.mat-mdc-icon-button{border-radius:50%;flex-shrink:0;text-align:center;width:var(--mdc-icon-button-state-layer-size, 48px);height:var(--mdc-icon-button-state-layer-size, 48px);padding:calc(calc(var(--mdc-icon-button-state-layer-size, 48px) - var(--mdc-icon-button-icon-size, 24px)) / 2);font-size:var(--mdc-icon-button-icon-size);-webkit-tap-highlight-color:rgba(0,0,0,0)}.mat-mdc-icon-button svg{vertical-align:baseline}.mat-mdc-icon-button[disabled],.mat-mdc-icon-button.mat-mdc-button-disabled{cursor:default;pointer-events:none;color:var(--mdc-icon-button-disabled-icon-color)}.mat-mdc-icon-button.mat-mdc-button-disabled-interactive{pointer-events:auto}.mat-mdc-icon-button .mat-mdc-button-ripple,.mat-mdc-icon-button .mat-mdc-button-persistent-ripple,.mat-mdc-icon-button .mat-mdc-button-persistent-ripple::before{top:0;left:0;right:0;bottom:0;position:absolute;pointer-events:none;border-radius:inherit}.mat-mdc-icon-button .mat-mdc-button-ripple{overflow:hidden}.mat-mdc-icon-button .mat-mdc-button-persistent-ripple::before{content:"";opacity:0}.mat-mdc-icon-button .mdc-button__label{z-index:1}.mat-mdc-icon-button .mat-mdc-focus-indicator{top:0;left:0;right:0;bottom:0;position:absolute}.mat-mdc-icon-button:focus .mat-mdc-focus-indicator::before{content:""}.mat-mdc-icon-button .mat-ripple-element{background-color:var(--mat-icon-button-ripple-color)}.mat-mdc-icon-button .mat-mdc-button-persistent-ripple::before{background-color:var(--mat-icon-button-state-layer-color)}.mat-mdc-icon-button.mat-mdc-button-disabled .mat-mdc-button-persistent-ripple::before{background-color:var(--mat-icon-button-disabled-state-layer-color)}.mat-mdc-icon-button:hover .mat-mdc-button-persistent-ripple::before{opacity:var(--mat-icon-button-hover-state-layer-opacity)}.mat-mdc-icon-button.cdk-program-focused .mat-mdc-button-persistent-ripple::before,.mat-mdc-icon-button.cdk-keyboard-focused .mat-mdc-button-persistent-ripple::before,.mat-mdc-icon-button.mat-mdc-button-disabled-interactive:focus .mat-mdc-button-persistent-ripple::before{opacity:var(--mat-icon-button-focus-state-layer-opacity)}.mat-mdc-icon-button:active .mat-mdc-button-persistent-ripple::before{opacity:var(--mat-icon-button-pressed-state-layer-opacity)}.mat-mdc-icon-button .mat-mdc-button-touch-target{position:absolute;top:50%;height:48px;left:50%;width:48px;transform:translate(-50%, -50%);display:var(--mat-icon-button-touch-target-display)}.mat-mdc-icon-button._mat-animation-noopable{transition:none !important;animation:none !important}.mat-mdc-icon-button .mat-mdc-button-persistent-ripple{border-radius:50%}.mat-mdc-icon-button.mat-unthemed:not(.mdc-ripple-upgraded):focus::before,.mat-mdc-icon-button.mat-primary:not(.mdc-ripple-upgraded):focus::before,.mat-mdc-icon-button.mat-accent:not(.mdc-ripple-upgraded):focus::before,.mat-mdc-icon-button.mat-warn:not(.mdc-ripple-upgraded):focus::before{background:rgba(0,0,0,0);opacity:1}\n',
".cdk-high-contrast-active .mat-mdc-button:not(.mdc-button--outlined),.cdk-high-contrast-active .mat-mdc-unelevated-button:not(.mdc-button--outlined),.cdk-high-contrast-active .mat-mdc-raised-button:not(.mdc-button--outlined),.cdk-high-contrast-active .mat-mdc-outlined-button:not(.mdc-button--outlined),.cdk-high-contrast-active .mat-mdc-icon-button{outline:solid 1px}\n"],pb:2,Ra:0});var zB=class{};zB.K=function(a){return new (a||zB)};zB.Ya=Of({type:zB});zB.Xa=Nd({imports:[mz,Yz,mz]});var AB=class{constructor(a,b){this.fc=a;this.options=b}Fb(){this.fc.R.querySelector(".mat-icon, .material-icons")&&this.fc.R.classList.add("gmat-mdc-button-with-prefix")}};AB.K=function(a){return new (a||AB)(z(si),z(qB))};
AB.Fa=Qf({type:AB,ha:[["button","mat-button",""],["a","mat-button",""],["button","mat-raised-button",""],["a","mat-raised-button",""],["button","mat-flat-button",""],["a","mat-flat-button",""],["button","mat-stroked-button",""],["a","mat-stroked-button",""],["button","mat-icon-button",""],["a","mat-icon-button",""],[""]],Za:2,Ia:function(a,b){a&2&&nq("gmat-mdc-button",!b.options.sc)}});var BB=class{};BB.K=function(a){return new (a||BB)};BB.Ya=Of({type:BB});BB.Xa=Nd({Wa:[{oa:xB,nb:{color:"primary"}}],imports:[zB]});var CB=new df("GMAT_CHECKBOX_OPTIONS",{ea:"root",ba:()=>({sc:!1})});var DB=class{constructor(a){this.options=a}};DB.K=function(a){return new (a||DB)(z(CB))};DB.Fa=Qf({type:DB,ha:[["mat-checkbox"]],Za:2,Ia:function(a,b){a&2&&nq("gmat-mdc-checkbox",!b.options.sc)}});var FB=new df("mat-checkbox-default-options",{ea:"root",ba:EB});function EB(){return{color:"accent",hB:"check-indeterminate"}};const GB=["input"],HB=["label"];var JB={oa:pw,Ab:af(()=>IB),Eb:!0},KB=class{};let LB=0;const MB=EB();
var IB=class{focus(){this.ig.R.focus()}kp(a){const b=new KB;b.source=this;b.checked=a;return b}Sz(){let a;return null==(a=this.ig)?void 0:a.R}get Yq(){return`${this.id||this.Dk}-input`}constructor(a,b,c,d,e,f){this.ra=a;this.Ba=b;this.qa=c;this.ad=e;this.Ni=f;this.xf={wy:"mdc-checkbox--anim-unchecked-checked",xy:"mdc-checkbox--anim-unchecked-indeterminate",cB:"mdc-checkbox--anim-checked-unchecked",Uu:"mdc-checkbox--anim-checked-indeterminate",nC:"mdc-checkbox--anim-indeterminate-checked",oC:"mdc-checkbox--anim-indeterminate-unchecked"};
this.ariaLabel="";this.cd=null;this.Lh="after";this.name=null;this.wb=new Fk;this.Vq=new Fk;this.Zd=()=>{};this.Fi="";this.tt=0;this.jp=()=>{};this.xu=()=>{};this.Af=this.Ib=this.yf=!1;this.Ni=this.Ni||MB;this.color=this.Ni.color||MB.color;this.tabIndex=parseInt(d)||0;this.id=this.Dk=`mat-mdc-checkbox-${++LB}`}tc(a){a.required&&this.xu()}pd(){this.qu(this.Af)}get checked(){return this.yf}set checked(a){a!=this.checked&&(this.yf=a,bn(this.Ba))}get disabled(){return this.Ib}set disabled(a){a!==this.disabled&&
(this.Ib=a,bn(this.Ba))}get indeterminate(){return this.Af}set indeterminate(a){const b=a!=this.Af;this.Af=a;b&&(this.Af?this.Pp(3):this.Pp(this.checked?1:2),this.Vq.emit(this.Af));this.qu(this.Af)}sF(){return this.Nc||this.disabled}uF(){cn(this.Ba)}pi(a){this.checked=!!a}Lg(a){this.jp=a}Ij(a){this.Zd=a}gi(a){this.disabled=a}validate(a){return this.required&&!0!==a.value?{required:!0}:null}Xr(a){this.xu=a}Pp(a){let b=this.tt,c=this.Sz();if(b!==a&&c&&(this.Fi&&c.classList.remove(this.Fi),this.Fi=this.Rz(b,
a),this.tt=a,0<this.Fi.length)){c.classList.add(this.Fi);const d=this.Fi;Mk(this.qa,()=>{setTimeout(()=>{c.classList.remove(d)},1E3)})}}Gi(){this.jp(this.checked);this.wb.emit(this.kp(this.checked));this.ig&&(this.ig.R.checked=this.checked)}toggle(){this.checked=!this.checked;this.jp(this.checked)}Jt(){let a;const b=null==(a=this.Ni)?void 0:a.hB;this.disabled||"noop"===b?this.disabled||"noop"!==b||(this.ig.R.checked=this.checked,this.ig.R.indeterminate=this.indeterminate):(this.indeterminate&&"check"!==
b&&Promise.resolve().then(()=>{this.Af=!1;this.Vq.emit(this.Af)}),this.yf=!this.yf,this.Pp(this.yf?1:2),this.Gi())}pA(a){a.stopPropagation()}xk(){Promise.resolve().then(()=>{this.Zd();bn(this.Ba)})}Rz(a,b){if("NoopAnimations"===this.ad)return"";switch(a){case 0:if(1===b)return this.xf.wy;if(3==b)return this.yf?this.xf.Uu:this.xf.xy;break;case 2:return 1===b?this.xf.wy:this.xf.xy;case 1:return 2===b?this.xf.cB:this.xf.Uu;case 3:return 1===b?this.xf.nC:this.xf.oC}return""}qu(a){const b=this.ig;b&&(b.R.indeterminate=
a)}oA(){this.Jt()}sA(){this.Jt();this.disabled||this.ig.R.focus()}zA(a){a.target&&this.iA.R.contains(a.target)&&a.stopPropagation()}};IB.K=function(a){return new (a||IB)(z(si),z(Sr),z(Qk),yh("tabindex"),z(mi,8),z(FB,8))};
IB.Da=Gf({type:IB,ha:[["mat-checkbox"]],fb:function(a,b){a&1&&(ur(GB,5),ur(HB,5),ur(Lz,5));if(a&2){let c;vr(c=wr())&&(b.ig=c.first);vr(c=wr())&&(b.iA=c.first);vr(c=wr())&&(b.vx=c.first)}},Ka:[1,"mat-mdc-checkbox"],Za:14,Ia:function(a,b){a&2&&(Qq("id",b.id),Kp("tabindex",null)("aria-label",null)("aria-labelledby",null),qq(tq,b.color?"mat-"+b.color:"mat-accent"),nq("_mat-animation-noopable","NoopAnimations"===b.ad)("mdc-checkbox--disabled",b.disabled)("mat-mdc-checkbox-disabled",b.disabled)("mat-mdc-checkbox-checked",
b.checked))},inputs:{ariaLabel:[0,"aria-label","ariaLabel"],cd:[0,"aria-labelledby","ariaLabelledby"],Hk:[0,"aria-describedby","ariaDescribedby"],id:"id",required:[2,"required","required",zs],Lh:"labelPosition",name:"name",value:"value",Nc:[2,"disableRipple","disableRipple",zs],tabIndex:[2,"tabIndex","tabIndex",a=>null==a?void 0:As(a)],color:"color",checked:[2,"checked","checked",zs],disabled:[2,"disabled","disabled",zs],indeterminate:[2,"indeterminate","indeterminate",zs]},outputs:{wb:"change",Vq:"indeterminateChange"},
ib:["matCheckbox"],ka:!0,features:[Kr([JB,{oa:vw,Ab:IB,Eb:!0}]),Jp,Dg,Mr],Zb:["*"],za:15,Aa:19,Ca:[["checkbox",""],["input",""],["label",""],["mat-internal-form-field","",3,"click","labelPosition"],[1,"mdc-checkbox"],[1,"mat-mdc-checkbox-touch-target",3,"click"],["type","checkbox",1,"mdc-checkbox__native-control",3,"blur","click","change","checked","indeterminate","disabled","id","required","tabIndex"],[1,"mdc-checkbox__ripple"],[1,"mdc-checkbox__background"],["focusable","false","viewBox","0 0 24 24",
"aria-hidden","true",1,"mdc-checkbox__checkmark"],["fill","none","d","M1.73,12.91 8.1,19.28 22.79,4.59",1,"mdc-checkbox__checkmark-path"],[1,"mdc-checkbox__mixedmark"],["mat-ripple","",1,"mat-mdc-checkbox-ripple","mat-mdc-focus-indicator",3,"matRippleTrigger","matRippleDisabled","matRippleCentered"],[1,"mdc-label",3,"for"]],ta:function(a,b){if(a&1){const c=Ug();Xq();R(0,"div",3);V("click",function(d){u(c);return x(b.zA(d))});R(1,"div",4,0)(3,"div",5);V("click",function(){u(c);return x(b.sA())});S();
R(4,"input",6,1);V("blur",function(){u(c);return x(b.xk())})("click",function(){u(c);return x(b.oA())})("change",function(d){u(c);return x(b.pA(d))});S();Mq(6,"div",7);R(7,"div",8);fh();R(8,"svg",9);Mq(9,"path",10);S();Rg.Vk=null;Mq(10,"div",11);S();Mq(11,"div",12);S();R(12,"label",13,2);Yq(14);S()()}a&2&&(a=zr(2),H("labelPosition",b.Lh),D(4),nq("mdc-checkbox--selected",b.checked),H("checked",b.checked)("indeterminate",b.indeterminate)("disabled",b.disabled)("id",b.Yq)("required",b.required)("tabIndex",
b.disabled?-1:b.tabIndex),Kp("aria-label",b.ariaLabel||null)("aria-labelledby",b.cd)("aria-describedby",b.Hk)("aria-checked",b.indeterminate?"mixed":null)("name",b.name)("value",b.value),D(7),H("matRippleTrigger",a)("matRippleDisabled",b.Nc||b.disabled)("matRippleCentered",!0),D(),H("for",b.Yq))},Ha:[Lz,qz],styles:['@keyframes mdc-checkbox-unchecked-checked-checkmark-path{0%,50%{stroke-dashoffset:29.7833385}50%{animation-timing-function:cubic-bezier(0, 0, 0.2, 1)}100%{stroke-dashoffset:0}}@keyframes mdc-checkbox-unchecked-indeterminate-mixedmark{0%,68.2%{transform:scaleX(0)}68.2%{animation-timing-function:cubic-bezier(0, 0, 0, 1)}100%{transform:scaleX(1)}}@keyframes mdc-checkbox-checked-unchecked-checkmark-path{from{animation-timing-function:cubic-bezier(0.4, 0, 1, 1);opacity:1;stroke-dashoffset:0}to{opacity:0;stroke-dashoffset:-29.7833385}}@keyframes mdc-checkbox-checked-indeterminate-checkmark{from{animation-timing-function:cubic-bezier(0, 0, 0.2, 1);transform:rotate(0deg);opacity:1}to{transform:rotate(45deg);opacity:0}}@keyframes mdc-checkbox-indeterminate-checked-checkmark{from{animation-timing-function:cubic-bezier(0.14, 0, 0, 1);transform:rotate(45deg);opacity:0}to{transform:rotate(360deg);opacity:1}}@keyframes mdc-checkbox-checked-indeterminate-mixedmark{from{animation-timing-function:mdc-animation-deceleration-curve-timing-function;transform:rotate(-45deg);opacity:0}to{transform:rotate(0deg);opacity:1}}@keyframes mdc-checkbox-indeterminate-checked-mixedmark{from{animation-timing-function:cubic-bezier(0.14, 0, 0, 1);transform:rotate(0deg);opacity:1}to{transform:rotate(315deg);opacity:0}}@keyframes mdc-checkbox-indeterminate-unchecked-mixedmark{0%{animation-timing-function:linear;transform:scaleX(1);opacity:1}32.8%,100%{transform:scaleX(0);opacity:0}}.mdc-checkbox{display:inline-block;position:relative;flex:0 0 18px;box-sizing:content-box;width:18px;height:18px;line-height:0;white-space:nowrap;cursor:pointer;vertical-align:bottom}.mdc-checkbox[hidden]{display:none}.mdc-checkbox.mdc-ripple-upgraded--background-focused .mdc-checkbox__focus-ring,.mdc-checkbox:not(.mdc-ripple-upgraded):focus .mdc-checkbox__focus-ring{pointer-events:none;border:2px solid rgba(0,0,0,0);border-radius:6px;box-sizing:content-box;position:absolute;top:50%;left:50%;transform:translate(-50%, -50%);height:100%;width:100%}@media screen and (forced-colors: active){.mdc-checkbox.mdc-ripple-upgraded--background-focused .mdc-checkbox__focus-ring,.mdc-checkbox:not(.mdc-ripple-upgraded):focus .mdc-checkbox__focus-ring{border-color:CanvasText}}.mdc-checkbox.mdc-ripple-upgraded--background-focused .mdc-checkbox__focus-ring::after,.mdc-checkbox:not(.mdc-ripple-upgraded):focus .mdc-checkbox__focus-ring::after{content:"";border:2px solid rgba(0,0,0,0);border-radius:8px;display:block;position:absolute;top:50%;left:50%;transform:translate(-50%, -50%);height:calc(100% + 4px);width:calc(100% + 4px)}@media screen and (forced-colors: active){.mdc-checkbox.mdc-ripple-upgraded--background-focused .mdc-checkbox__focus-ring::after,.mdc-checkbox:not(.mdc-ripple-upgraded):focus .mdc-checkbox__focus-ring::after{border-color:CanvasText}}@media all and (-ms-high-contrast: none){.mdc-checkbox .mdc-checkbox__focus-ring{display:none}}@media screen and (forced-colors: active),(-ms-high-contrast: active){.mdc-checkbox__mixedmark{margin:0 1px}}.mdc-checkbox--disabled{cursor:default;pointer-events:none}.mdc-checkbox__background{display:inline-flex;position:absolute;align-items:center;justify-content:center;box-sizing:border-box;width:18px;height:18px;border:2px solid currentColor;border-radius:2px;background-color:rgba(0,0,0,0);pointer-events:none;will-change:background-color,border-color;transition:background-color 90ms 0ms cubic-bezier(0.4, 0, 0.6, 1),border-color 90ms 0ms cubic-bezier(0.4, 0, 0.6, 1)}.mdc-checkbox__checkmark{position:absolute;top:0;right:0;bottom:0;left:0;width:100%;opacity:0;transition:opacity 180ms 0ms cubic-bezier(0.4, 0, 0.6, 1)}.mdc-checkbox--upgraded .mdc-checkbox__checkmark{opacity:1}.mdc-checkbox__checkmark-path{transition:stroke-dashoffset 180ms 0ms cubic-bezier(0.4, 0, 0.6, 1);stroke:currentColor;stroke-width:3.12px;stroke-dashoffset:29.7833385;stroke-dasharray:29.7833385}.mdc-checkbox__mixedmark{width:100%;height:0;transform:scaleX(0) rotate(0deg);border-width:1px;border-style:solid;opacity:0;transition:opacity 90ms 0ms cubic-bezier(0.4, 0, 0.6, 1),transform 90ms 0ms cubic-bezier(0.4, 0, 0.6, 1)}.mdc-checkbox--anim-unchecked-checked .mdc-checkbox__background,.mdc-checkbox--anim-unchecked-indeterminate .mdc-checkbox__background,.mdc-checkbox--anim-checked-unchecked .mdc-checkbox__background,.mdc-checkbox--anim-indeterminate-unchecked .mdc-checkbox__background{animation-duration:180ms;animation-timing-function:linear}.mdc-checkbox--anim-unchecked-checked .mdc-checkbox__checkmark-path{animation:mdc-checkbox-unchecked-checked-checkmark-path 180ms linear 0s;transition:none}.mdc-checkbox--anim-unchecked-indeterminate .mdc-checkbox__mixedmark{animation:mdc-checkbox-unchecked-indeterminate-mixedmark 90ms linear 0s;transition:none}.mdc-checkbox--anim-checked-unchecked .mdc-checkbox__checkmark-path{animation:mdc-checkbox-checked-unchecked-checkmark-path 90ms linear 0s;transition:none}.mdc-checkbox--anim-checked-indeterminate .mdc-checkbox__checkmark{animation:mdc-checkbox-checked-indeterminate-checkmark 90ms linear 0s;transition:none}.mdc-checkbox--anim-checked-indeterminate .mdc-checkbox__mixedmark{animation:mdc-checkbox-checked-indeterminate-mixedmark 90ms linear 0s;transition:none}.mdc-checkbox--anim-indeterminate-checked .mdc-checkbox__checkmark{animation:mdc-checkbox-indeterminate-checked-checkmark 500ms linear 0s;transition:none}.mdc-checkbox--anim-indeterminate-checked .mdc-checkbox__mixedmark{animation:mdc-checkbox-indeterminate-checked-mixedmark 500ms linear 0s;transition:none}.mdc-checkbox--anim-indeterminate-unchecked .mdc-checkbox__mixedmark{animation:mdc-checkbox-indeterminate-unchecked-mixedmark 300ms linear 0s;transition:none}.mdc-checkbox__native-control:checked~.mdc-checkbox__background,.mdc-checkbox__native-control:indeterminate~.mdc-checkbox__background,.mdc-checkbox__native-control[data-indeterminate=true]~.mdc-checkbox__background{transition:border-color 90ms 0ms cubic-bezier(0, 0, 0.2, 1),background-color 90ms 0ms cubic-bezier(0, 0, 0.2, 1)}.mdc-checkbox__native-control:checked~.mdc-checkbox__background .mdc-checkbox__checkmark-path,.mdc-checkbox__native-control:indeterminate~.mdc-checkbox__background .mdc-checkbox__checkmark-path,.mdc-checkbox__native-control[data-indeterminate=true]~.mdc-checkbox__background .mdc-checkbox__checkmark-path{stroke-dashoffset:0}.mdc-checkbox__native-control{position:absolute;margin:0;padding:0;opacity:0;cursor:inherit}.mdc-checkbox__native-control:disabled{cursor:default;pointer-events:none}.mdc-checkbox--touch{margin:calc((var(--mdc-checkbox-state-layer-size) - var(--mdc-checkbox-state-layer-size)) / 2)}.mdc-checkbox--touch .mdc-checkbox__native-control{top:calc((var(--mdc-checkbox-state-layer-size) - var(--mdc-checkbox-state-layer-size)) / 2);right:calc((var(--mdc-checkbox-state-layer-size) - var(--mdc-checkbox-state-layer-size)) / 2);left:calc((var(--mdc-checkbox-state-layer-size) - var(--mdc-checkbox-state-layer-size)) / 2);width:var(--mdc-checkbox-state-layer-size);height:var(--mdc-checkbox-state-layer-size)}.mdc-checkbox__native-control:checked~.mdc-checkbox__background .mdc-checkbox__checkmark{transition:opacity 180ms 0ms cubic-bezier(0, 0, 0.2, 1),transform 180ms 0ms cubic-bezier(0, 0, 0.2, 1);opacity:1}.mdc-checkbox__native-control:checked~.mdc-checkbox__background .mdc-checkbox__mixedmark{transform:scaleX(1) rotate(-45deg)}.mdc-checkbox__native-control:indeterminate~.mdc-checkbox__background .mdc-checkbox__checkmark,.mdc-checkbox__native-control[data-indeterminate=true]~.mdc-checkbox__background .mdc-checkbox__checkmark{transform:rotate(45deg);opacity:0;transition:opacity 90ms 0ms cubic-bezier(0.4, 0, 0.6, 1),transform 90ms 0ms cubic-bezier(0.4, 0, 0.6, 1)}.mdc-checkbox__native-control:indeterminate~.mdc-checkbox__background .mdc-checkbox__mixedmark,.mdc-checkbox__native-control[data-indeterminate=true]~.mdc-checkbox__background .mdc-checkbox__mixedmark{transform:scaleX(1) rotate(0deg);opacity:1}.mdc-checkbox.mdc-checkbox--upgraded .mdc-checkbox__background,.mdc-checkbox.mdc-checkbox--upgraded .mdc-checkbox__checkmark,.mdc-checkbox.mdc-checkbox--upgraded .mdc-checkbox__checkmark-path,.mdc-checkbox.mdc-checkbox--upgraded .mdc-checkbox__mixedmark{transition:none}.mdc-checkbox{padding:calc((var(--mdc-checkbox-state-layer-size) - 18px) / 2);margin:calc((var(--mdc-checkbox-state-layer-size) - var(--mdc-checkbox-state-layer-size)) / 2)}.mdc-checkbox .mdc-checkbox__native-control[disabled]:not(:checked):not(:indeterminate):not([data-indeterminate=true])~.mdc-checkbox__background{border-color:var(--mdc-checkbox-disabled-unselected-icon-color);background-color:transparent}.mdc-checkbox .mdc-checkbox__native-control[disabled]:checked~.mdc-checkbox__background,.mdc-checkbox .mdc-checkbox__native-control[disabled]:indeterminate~.mdc-checkbox__background,.mdc-checkbox .mdc-checkbox__native-control[data-indeterminate=true][disabled]~.mdc-checkbox__background{border-color:transparent;background-color:var(--mdc-checkbox-disabled-selected-icon-color)}.mdc-checkbox .mdc-checkbox__native-control:enabled~.mdc-checkbox__background .mdc-checkbox__checkmark{color:var(--mdc-checkbox-selected-checkmark-color)}.mdc-checkbox .mdc-checkbox__native-control:enabled~.mdc-checkbox__background .mdc-checkbox__mixedmark{border-color:var(--mdc-checkbox-selected-checkmark-color)}.mdc-checkbox .mdc-checkbox__native-control:disabled~.mdc-checkbox__background .mdc-checkbox__checkmark{color:var(--mdc-checkbox-disabled-selected-checkmark-color)}.mdc-checkbox .mdc-checkbox__native-control:disabled~.mdc-checkbox__background .mdc-checkbox__mixedmark{border-color:var(--mdc-checkbox-disabled-selected-checkmark-color)}.mdc-checkbox .mdc-checkbox__native-control:enabled:not(:checked):not(:indeterminate):not([data-indeterminate=true])~.mdc-checkbox__background{border-color:var(--mdc-checkbox-unselected-icon-color);background-color:transparent}.mdc-checkbox .mdc-checkbox__native-control:enabled:checked~.mdc-checkbox__background,.mdc-checkbox .mdc-checkbox__native-control:enabled:indeterminate~.mdc-checkbox__background,.mdc-checkbox .mdc-checkbox__native-control[data-indeterminate=true]:enabled~.mdc-checkbox__background{border-color:var(--mdc-checkbox-selected-icon-color);background-color:var(--mdc-checkbox-selected-icon-color)}@keyframes mdc-checkbox-fade-in-background-8A000000FFF4433600000000FFF44336{0%{border-color:var(--mdc-checkbox-unselected-icon-color);background-color:transparent}50%{border-color:var(--mdc-checkbox-selected-icon-color);background-color:var(--mdc-checkbox-selected-icon-color)}}@keyframes mdc-checkbox-fade-out-background-8A000000FFF4433600000000FFF44336{0%,80%{border-color:var(--mdc-checkbox-selected-icon-color);background-color:var(--mdc-checkbox-selected-icon-color)}100%{border-color:var(--mdc-checkbox-unselected-icon-color);background-color:transparent}}.mdc-checkbox.mdc-checkbox--anim-unchecked-checked .mdc-checkbox__native-control:enabled~.mdc-checkbox__background,.mdc-checkbox.mdc-checkbox--anim-unchecked-indeterminate .mdc-checkbox__native-control:enabled~.mdc-checkbox__background{animation-name:mdc-checkbox-fade-in-background-8A000000FFF4433600000000FFF44336}.mdc-checkbox.mdc-checkbox--anim-checked-unchecked .mdc-checkbox__native-control:enabled~.mdc-checkbox__background,.mdc-checkbox.mdc-checkbox--anim-indeterminate-unchecked .mdc-checkbox__native-control:enabled~.mdc-checkbox__background{animation-name:mdc-checkbox-fade-out-background-8A000000FFF4433600000000FFF44336}.mdc-checkbox:hover .mdc-checkbox__native-control:enabled:not(:checked):not(:indeterminate):not([data-indeterminate=true])~.mdc-checkbox__background{border-color:var(--mdc-checkbox-unselected-hover-icon-color);background-color:transparent}.mdc-checkbox:hover .mdc-checkbox__native-control:enabled:checked~.mdc-checkbox__background,.mdc-checkbox:hover .mdc-checkbox__native-control:enabled:indeterminate~.mdc-checkbox__background,.mdc-checkbox:hover .mdc-checkbox__native-control[data-indeterminate=true]:enabled~.mdc-checkbox__background{border-color:var(--mdc-checkbox-selected-hover-icon-color);background-color:var(--mdc-checkbox-selected-hover-icon-color)}@keyframes mdc-checkbox-fade-in-background-FF212121FFF4433600000000FFF44336{0%{border-color:var(--mdc-checkbox-unselected-hover-icon-color);background-color:transparent}50%{border-color:var(--mdc-checkbox-selected-hover-icon-color);background-color:var(--mdc-checkbox-selected-hover-icon-color)}}@keyframes mdc-checkbox-fade-out-background-FF212121FFF4433600000000FFF44336{0%,80%{border-color:var(--mdc-checkbox-selected-hover-icon-color);background-color:var(--mdc-checkbox-selected-hover-icon-color)}100%{border-color:var(--mdc-checkbox-unselected-hover-icon-color);background-color:transparent}}.mdc-checkbox:hover.mdc-checkbox--anim-unchecked-checked .mdc-checkbox__native-control:enabled~.mdc-checkbox__background,.mdc-checkbox:hover.mdc-checkbox--anim-unchecked-indeterminate .mdc-checkbox__native-control:enabled~.mdc-checkbox__background{animation-name:mdc-checkbox-fade-in-background-FF212121FFF4433600000000FFF44336}.mdc-checkbox:hover.mdc-checkbox--anim-checked-unchecked .mdc-checkbox__native-control:enabled~.mdc-checkbox__background,.mdc-checkbox:hover.mdc-checkbox--anim-indeterminate-unchecked .mdc-checkbox__native-control:enabled~.mdc-checkbox__background{animation-name:mdc-checkbox-fade-out-background-FF212121FFF4433600000000FFF44336}.mdc-checkbox:not(:disabled):active .mdc-checkbox__native-control:enabled:not(:checked):not(:indeterminate):not([data-indeterminate=true])~.mdc-checkbox__background{border-color:var(--mdc-checkbox-unselected-pressed-icon-color);background-color:transparent}.mdc-checkbox:not(:disabled):active .mdc-checkbox__native-control:enabled:checked~.mdc-checkbox__background,.mdc-checkbox:not(:disabled):active .mdc-checkbox__native-control:enabled:indeterminate~.mdc-checkbox__background,.mdc-checkbox:not(:disabled):active .mdc-checkbox__native-control[data-indeterminate=true]:enabled~.mdc-checkbox__background{border-color:var(--mdc-checkbox-selected-pressed-icon-color);background-color:var(--mdc-checkbox-selected-pressed-icon-color)}@keyframes mdc-checkbox-fade-in-background-8A000000FFF4433600000000FFF44336{0%{border-color:var(--mdc-checkbox-unselected-pressed-icon-color);background-color:transparent}50%{border-color:var(--mdc-checkbox-selected-pressed-icon-color);background-color:var(--mdc-checkbox-selected-pressed-icon-color)}}@keyframes mdc-checkbox-fade-out-background-8A000000FFF4433600000000FFF44336{0%,80%{border-color:var(--mdc-checkbox-selected-pressed-icon-color);background-color:var(--mdc-checkbox-selected-pressed-icon-color)}100%{border-color:var(--mdc-checkbox-unselected-pressed-icon-color);background-color:transparent}}.mdc-checkbox:not(:disabled):active.mdc-checkbox--anim-unchecked-checked .mdc-checkbox__native-control:enabled~.mdc-checkbox__background,.mdc-checkbox:not(:disabled):active.mdc-checkbox--anim-unchecked-indeterminate .mdc-checkbox__native-control:enabled~.mdc-checkbox__background{animation-name:mdc-checkbox-fade-in-background-8A000000FFF4433600000000FFF44336}.mdc-checkbox:not(:disabled):active.mdc-checkbox--anim-checked-unchecked .mdc-checkbox__native-control:enabled~.mdc-checkbox__background,.mdc-checkbox:not(:disabled):active.mdc-checkbox--anim-indeterminate-unchecked .mdc-checkbox__native-control:enabled~.mdc-checkbox__background{animation-name:mdc-checkbox-fade-out-background-8A000000FFF4433600000000FFF44336}.mdc-checkbox .mdc-checkbox__background{top:calc((var(--mdc-checkbox-state-layer-size) - 18px) / 2);left:calc((var(--mdc-checkbox-state-layer-size) - 18px) / 2)}.mdc-checkbox .mdc-checkbox__native-control{top:calc((var(--mdc-checkbox-state-layer-size) - var(--mdc-checkbox-state-layer-size)) / 2);right:calc((var(--mdc-checkbox-state-layer-size) - var(--mdc-checkbox-state-layer-size)) / 2);left:calc((var(--mdc-checkbox-state-layer-size) - var(--mdc-checkbox-state-layer-size)) / 2);width:var(--mdc-checkbox-state-layer-size);height:var(--mdc-checkbox-state-layer-size)}.mdc-checkbox .mdc-checkbox__native-control:enabled:focus:focus:not(:checked):not(:indeterminate)~.mdc-checkbox__background{border-color:var(--mdc-checkbox-unselected-focus-icon-color)}.mdc-checkbox .mdc-checkbox__native-control:enabled:focus:checked~.mdc-checkbox__background,.mdc-checkbox .mdc-checkbox__native-control:enabled:focus:indeterminate~.mdc-checkbox__background{border-color:var(--mdc-checkbox-selected-focus-icon-color);background-color:var(--mdc-checkbox-selected-focus-icon-color)}.mdc-checkbox:hover .mdc-checkbox__ripple{opacity:var(--mdc-checkbox-unselected-hover-state-layer-opacity);background-color:var(--mdc-checkbox-unselected-hover-state-layer-color)}.mdc-checkbox:hover .mat-mdc-checkbox-ripple .mat-ripple-element{background-color:var(--mdc-checkbox-unselected-hover-state-layer-color)}.mdc-checkbox .mdc-checkbox__native-control:focus~.mdc-checkbox__ripple{opacity:var(--mdc-checkbox-unselected-focus-state-layer-opacity);background-color:var(--mdc-checkbox-unselected-focus-state-layer-color)}.mdc-checkbox .mdc-checkbox__native-control:focus~.mat-mdc-checkbox-ripple .mat-ripple-element{background-color:var(--mdc-checkbox-unselected-focus-state-layer-color)}.mdc-checkbox:active .mdc-checkbox__native-control~.mdc-checkbox__ripple{opacity:var(--mdc-checkbox-unselected-pressed-state-layer-opacity);background-color:var(--mdc-checkbox-unselected-pressed-state-layer-color)}.mdc-checkbox:active .mdc-checkbox__native-control~.mat-mdc-checkbox-ripple .mat-ripple-element{background-color:var(--mdc-checkbox-unselected-pressed-state-layer-color)}.mdc-checkbox:hover .mdc-checkbox__native-control:checked~.mdc-checkbox__ripple{opacity:var(--mdc-checkbox-selected-hover-state-layer-opacity);background-color:var(--mdc-checkbox-selected-hover-state-layer-color)}.mdc-checkbox:hover .mdc-checkbox__native-control:checked~.mat-mdc-checkbox-ripple .mat-ripple-element{background-color:var(--mdc-checkbox-selected-hover-state-layer-color)}.mdc-checkbox .mdc-checkbox__native-control:focus:checked~.mdc-checkbox__ripple{opacity:var(--mdc-checkbox-selected-focus-state-layer-opacity);background-color:var(--mdc-checkbox-selected-focus-state-layer-color)}.mdc-checkbox .mdc-checkbox__native-control:focus:checked~.mat-mdc-checkbox-ripple .mat-ripple-element{background-color:var(--mdc-checkbox-selected-focus-state-layer-color)}.mdc-checkbox:active .mdc-checkbox__native-control:checked~.mdc-checkbox__ripple{opacity:var(--mdc-checkbox-selected-pressed-state-layer-opacity);background-color:var(--mdc-checkbox-selected-pressed-state-layer-color)}.mdc-checkbox:active .mdc-checkbox__native-control:checked~.mat-mdc-checkbox-ripple .mat-ripple-element{background-color:var(--mdc-checkbox-selected-pressed-state-layer-color)}.mat-mdc-checkbox{display:inline-block;position:relative;-webkit-tap-highlight-color:rgba(0,0,0,0)}.mat-mdc-checkbox .mdc-checkbox__background{-webkit-print-color-adjust:exact;color-adjust:exact}.mat-mdc-checkbox._mat-animation-noopable *,.mat-mdc-checkbox._mat-animation-noopable *::before{transition:none !important;animation:none !important}.mat-mdc-checkbox label{cursor:pointer}.mat-mdc-checkbox.mat-mdc-checkbox-disabled label{cursor:default;color:var(--mat-checkbox-disabled-label-color)}.mat-mdc-checkbox label:empty{display:none}.cdk-high-contrast-active .mat-mdc-checkbox.mat-mdc-checkbox-disabled{opacity:.5}.cdk-high-contrast-active .mat-mdc-checkbox .mdc-checkbox__checkmark{--mdc-checkbox-selected-checkmark-color: CanvasText;--mdc-checkbox-disabled-selected-checkmark-color: CanvasText}.mat-mdc-checkbox .mdc-checkbox__ripple{opacity:0}.mat-mdc-checkbox-ripple,.mdc-checkbox__ripple{top:0;left:0;right:0;bottom:0;position:absolute;border-radius:50%;pointer-events:none}.mat-mdc-checkbox-ripple:not(:empty),.mdc-checkbox__ripple:not(:empty){transform:translateZ(0)}.mat-mdc-checkbox-ripple .mat-ripple-element{opacity:.1}.mat-mdc-checkbox-touch-target{position:absolute;top:50%;height:48px;left:50%;width:48px;transform:translate(-50%, -50%);display:var(--mat-checkbox-touch-target-display)}.mat-mdc-checkbox-ripple::before{border-radius:50%}.mdc-checkbox__native-control:focus~.mat-mdc-focus-indicator::before{content:""}\n'],
pb:2,Ra:0});var NB=class{};NB.K=function(a){return new (a||NB)};NB.Ya=Of({type:NB});NB.Xa=Nd({imports:[IB,mz,mz]});var OB=class{};OB.K=function(a){return new (a||OB)};OB.Ya=Of({type:OB});OB.Xa=Nd({Wa:[{oa:FB,nb:{color:"primary"}}],imports:[NB]});var PB=new df("GMAT_DIALOG_OPTIONS",{ea:"root",ba:()=>({sc:!1})});var QB=class{constructor(){this.role="dialog";this.hb="";this.Tb=!0;this.dc="";this.yd=!1;this.height=this.width="";this.ariaLabel=this.Yp=this.Xp=this.data=null;this.ariaModal=!0;this.Tm="first-tabbable";this.Zu=this.Av=this.Ng=!0}};var RB=class{constructor(){this.role="dialog";this.hb="";this.Tb=!0;this.dc="";this.yd=!1;this.height=this.width="";this.ariaLabel=this.Yp=this.Xp=this.data=null;this.ariaModal=!0;this.Tm="first-tabbable";this.av=this.Yu=this.Zu=this.Ng=!0}};function SB(){}function TB(a){var b=a.ra.R;switch(a.ya.Tm){case !1:case "dialog":a.fa()||b.focus();break;case !0:case "first-tabbable":b=()=>{let c;(null==(c=a.O)?0:Wx(c))||a.Sa()};a.qa.isStable?al(b,{xa:a.Ua}):C(a.qa.Jl,fo()).subscribe(b);break;case "first-heading":a.sa('h1, h2, h3, h4, h5, h6, [role="heading"]');break;default:a.sa(a.ya.Tm)}}
var UB=class extends Xv{constructor(a,b,c,d,e,f,g,k){super();this.ra=a;this.Ja=b;this.ya=d;this.lb=e;this.qa=f;this.la=g;this.P=k;this.Ma=lf(Vt);this.Bi=this.ja=this.O=null;this.Ai=[];this.Ba=lf(Sr);this.Ua=lf(Wh);this.ia=c;this.ya.Yp&&this.Ai.push(this.ya.Yp)}Pa(a){this.Ai.push(a);bn(this.Ba)}Lb(a){a=this.Ai.indexOf(a);-1<a&&(this.Ai.splice(a,1),bn(this.Ba))}G(){this.gb();this.bb();this.aa()}aa(){TB(this)}va(){this.Nb()}ae(a){a=this.Ve.ae(a);this.G();return a}af(a){a=this.Ve.af(a);this.G();return a}Kb(){this.fa()||
TB(this)}Va(a,b){Rx(this.lb,a)||(a.tabIndex=-1,Mk(this.qa,()=>{const c=()=>{a.removeEventListener("blur",c);a.removeEventListener("mousedown",c);a.removeAttribute("tabindex")};a.addEventListener("blur",c);a.addEventListener("mousedown",c)}));a.focus(b)}sa(a,b){(a=this.ra.R.querySelector(a))&&this.Va(a,b)}Nb(){var a=this.ya.Ng;let b=null;"string"===typeof a?b=this.ia.querySelector(a):"boolean"===typeof a?b=a?this.ja:null:a&&(b=a);if(this.ya.Ng&&b&&"function"===typeof b.focus){a=Rt();const c=this.ra.R;
if(!a||a===this.ia.body||a===c||c.contains(a))this.P?(Nx(this.P,b,this.Bi),this.Bi=null):b.focus()}this.O&&this.O.destroy()}Sa(){this.ra.R.focus&&this.ra.R.focus()}fa(){const a=this.ra.R,b=Rt();return a===b||a.contains(b)}gb(){this.Ma.isBrowser&&(this.O=this.Ja.create(this.ra.R),this.ia&&(this.ja=Rt()))}bb(){this.la.dd().subscribe(()=>{this.ya.yd&&this.Kb()})}};UB.K=function(a){return new (a||UB)(z(si),z($x),z(jt,8),z(RB),z(Tx),z(Qk),z(Eu),z(Ox))};
UB.Da=Gf({type:UB,ha:[["cdk-dialog-container"]],fb:function(a,b){a&1&&ur(Zv,7);if(a&2){let c;vr(c=wr())&&(b.Ve=c.first)}},Ka:["tabindex","-1",1,"cdk-dialog-container"],Za:6,Ia:function(a,b){a&2&&Kp("id",b.ya.id||null)("role",b.ya.role)("aria-modal",b.ya.ariaModal)("aria-labelledby",b.ya.ariaLabel?null:b.Ai[0])("aria-label",b.ya.ariaLabel)("aria-describedby",b.ya.Xp||null)},ka:!0,features:[Cp,Mr],za:1,Aa:0,Ca:[["cdkPortalOutlet",""]],ta:function(a){a&1&&Ap(0,SB,0,0,"ng-template",0)},Ha:[Zv],styles:[".cdk-dialog-container{display:block;width:100%;height:100%;min-height:inherit;max-height:inherit}\n"],
pb:2});var VB=new df("DialogScrollStrategy",{ea:"root",ba:()=>{const a=lf(bw);return()=>a.C.block()}}),WB=new df("DialogData"),XB=new df("DefaultDialogConfig");var YB=class{constructor(a,b){this.Vb=a;this.config=b;this.closed=new cj;this.yd=b.yd;this.dd=a.dd();this.ef=a.ef();this.Ll=a.Ll();this.id=b.id;this.ef.subscribe(c=>{27!==c.keyCode||this.yd||Bu(c)||(c.preventDefault(),this.close(void 0,{Hq:"keyboard"}))});this.dd.subscribe(()=>{this.yd||this.close(void 0,{Hq:"mouse"})});this.xm=a.J.subscribe(()=>{!1!==b.av&&this.close()})}close(a,b){if(this.rg){const c=this.closed;this.rg.Bi=(null==b?void 0:b.Hq)||"program";this.xm.unsubscribe();this.Vb.dispose();
c.next(a);c.complete();this.en=this.rg=null}}Wc(){this.Vb.Wc()}ne(a="",b=""){this.Vb.ne({width:a,height:b})}og(a){this.Vb.og(a)}Jj(a){this.Vb.Jj(a)}};let ZB=0;function $B(a,b){a.C.find(c=>c.id===b)}function aC(a,b){a=new Au({uc:b.uc||Ov(Nv(Qv(a.O.position()))),vc:b.vc||a.Kc(),hb:b.hb,Tb:b.Tb,direction:b.direction,minWidth:b.minWidth,minHeight:b.minHeight,maxWidth:b.maxWidth,maxHeight:b.maxHeight,width:b.width,height:b.height,Hh:b.Zu});b.dc&&(a.dc=b.dc);return a}
function bC(a,b,c,d){var e;const f=a.xa||(null==(e=a.xb)?void 0:e.xa);e=[{oa:WB,nb:a.data},{oa:YB,nb:b}];a.Wa&&("function"===typeof a.Wa?e.push(...a.Wa(b,a,c)):e.push(...a.Wa));!a.direction||f&&f.get(ou,null,{optional:!0})||e.push({oa:ou,nb:{value:a.direction,wb:jj()}});return Vh({parent:f||d,Wa:e})}function cC(a){const b=a.D;return b?cC(b):a.J}
var eC=class{get C(){return this.D?this.D.C:this.F}get pg(){return this.D?this.D.pg:this.N}constructor(a,b,c,d,e,f){this.O=a;this.Ua=b;this.T=c;this.D=d;this.aa=e;this.F=[];this.J=new cj;this.N=new cj;this.G=new Map;this.Kc=f}open(a,b){b=Object.assign({},this.T||new RB,b);b.id=b.id||`cdk-dialog-${ZB++}`;b.id&&$B(this,b.id);var c=aC(this,b);c=this.O.create(c);const d=new YB(c,b);c=this.ca(c,d,b);d.rg=c;this.fa(a,d,c,b);this.C.length||this.ja();this.C.push(d);d.closed.subscribe(()=>this.P(d,!0));this.pg.next(d);
return d}va(){dC(this.F,a=>{!1===a.config.Yu&&this.P(a,!1)});dC(this.F,a=>a.close());this.J.complete();this.N.complete();this.F=[]}ca(a,b,c){var d;const e=c.xa||(null==(d=c.xb)?void 0:d.xa);b=[{oa:RB,nb:c},{oa:YB,nb:b},{oa:Eu,nb:a}];c.ab?"function"===typeof c.ab?d=c.ab:(d=c.ab.type,b.push(...c.ab.Wa(c))):d=UB;c=new Uv(d,c.xb,Vh({parent:e||this.Ua,Wa:b}),c.C);return a.attach(c).instance}fa(a,b,c,d){if(a instanceof br){var e=bC(d,b,c);b={da:d.data,Yk:b};d.Ho&&(b=Object.assign({},b,"function"===typeof d.Ho?
d.Ho():d.Ho));c.af(new Vv(a,null,b,e))}else e=bC(d,b,c,this.Ua),a=c.ae(new Uv(a,d.xb,e,d.C)),b.fv=a,b.en=a.instance}P(a,b){a=this.C.indexOf(a);-1<a&&(this.C.splice(a,1),this.C.length||(this.G.forEach((c,d)=>{c?d.setAttribute("aria-hidden",c):d.removeAttribute("aria-hidden")}),this.G.clear(),b&&cC(this).next()))}ja(){const a=Xt(this.aa);if(a.parentElement){const b=a.parentElement.children;for(let c=b.length-1;-1<c;c--){const d=b[c];d===a||"SCRIPT"===d.nodeName||"STYLE"===d.nodeName||d.hasAttribute("aria-live")||
(this.G.set(d,d.getAttribute("aria-hidden")),d.setAttribute("aria-hidden","true"))}}}};eC.K=function(a){return new (a||eC)(r(bw),r(Wh),r(XB,8),r(eC,12),r(Yt),r(VB))};eC.pa=Md({ma:eC,ba:eC.K,ea:"root"});function dC(a,b){let c=a.length;for(;c--;)b(a[c])};var fC=class{};fC.K=function(a){return new (a||fC)};fC.Ya=Of({type:fC});fC.Xa=Nd({Wa:[eC],imports:[jw,$v,ry,$v]});function gC(){}
var iC=class extends UB{constructor(a,b,c,d,e,f,g,k,m){super(a,b,c,d,e,f,g,m);this.ad=k;this.C=new Fk;this.lk="NoopAnimations"!==this.ad;this.Yo=0;this.D=this.ra.R;let p;this.J=this.lk?null!=(p=hC(this.ya.KF))?p:150:0;let t;this.N=this.lk?null!=(t=hC(this.ya.NF))?t:75:0;this.F=null;this.T=!1;this.na=()=>{this.ca();this.sb(this.J)};this.Hi=()=>{this.ca();this.C.emit({state:"closed",totalTime:this.N})}}G(){super.G();this.ac()}ac(){this.C.emit({state:"opening",totalTime:this.J});this.lk?(this.D.style.setProperty("--mat-dialog-transition-duration",
`${this.J}ms`),this.ua(()=>this.D.classList.add("mdc-dialog--opening","mdc-dialog--open")),this.Ea(this.J,this.na)):(this.D.classList.add("mdc-dialog--open"),Promise.resolve().then(()=>this.na()))}Ob(){this.C.emit({state:"closing",totalTime:this.N});this.D.classList.remove("mdc-dialog--open");this.lk?(this.D.style.setProperty("--mat-dialog-transition-duration",`${this.N}ms`),this.ua(()=>this.D.classList.add("mdc-dialog--closing")),this.Ea(this.N,this.Hi)):Promise.resolve().then(()=>this.Hi())}wa(a){this.Yo+=
a;bn(this.Ba)}ca(){this.D.classList.remove("mdc-dialog--opening","mdc-dialog--closing")}Ea(a,b){null!==this.F&&clearTimeout(this.F);this.F=setTimeout(b,a)}ua(a){Mk(this.qa,()=>{"function"===typeof requestAnimationFrame?requestAnimationFrame(a):a()})}aa(){this.ya.Av||TB(this)}sb(a){this.T||(this.ya.Av&&TB(this),this.C.next({state:"opened",totalTime:a}))}va(){super.va();null!==this.F&&clearTimeout(this.F);this.T=!0}ae(a){a=super.ae(a);a.location.R.classList.add("mat-mdc-dialog-component-host");return a}};
iC.K=function(a){return new (a||iC)(z(si),z($x),z(jt,8),z(QB),z(Tx),z(Qk),z(Eu),z(mi,8),z(Ox))};
iC.Da=Gf({type:iC,ha:[["mat-dialog-container"]],Ka:["tabindex","-1",1,"mat-mdc-dialog-container","mdc-dialog"],Za:10,Ia:function(a,b){a&2&&(Qq("id",b.ya.id),Kp("aria-modal",b.ya.ariaModal)("role",b.ya.role)("aria-labelledby",b.ya.ariaLabel?null:b.Ai[0])("aria-label",b.ya.ariaLabel)("aria-describedby",b.ya.Xp||null),nq("_mat-animation-noopable",!b.lk)("mat-mdc-dialog-container-with-actions",0<b.Yo))},ka:!0,features:[Cp,Mr],za:3,Aa:0,Ca:[[1,"mdc-dialog__container"],[1,"mat-mdc-dialog-surface","mdc-dialog__surface"],
["cdkPortalOutlet",""]],ta:function(a){a&1&&(R(0,"div",0)(1,"div",1),Ap(2,gC,0,0,"ng-template",2),S()())},Ha:[Zv],styles:['.mdc-dialog,.mdc-dialog__scrim{position:fixed;top:0;left:0;align-items:center;justify-content:center;box-sizing:border-box;width:100%;height:100%}.mdc-dialog{display:none;z-index:var(--mdc-dialog-z-index, 7)}.mdc-dialog .mdc-dialog__content{padding:20px 24px 20px 24px}.mdc-dialog .mdc-dialog__surface{min-width:280px}@media(max-width: 592px){.mdc-dialog .mdc-dialog__surface{max-width:calc(100vw - 32px)}}@media(min-width: 592px){.mdc-dialog .mdc-dialog__surface{max-width:560px}}.mdc-dialog .mdc-dialog__surface{max-height:calc(100% - 32px)}.mdc-dialog.mdc-dialog--fullscreen .mdc-dialog__surface{max-width:none}@media(max-width: 960px){.mdc-dialog.mdc-dialog--fullscreen .mdc-dialog__surface{max-height:560px;width:560px}.mdc-dialog.mdc-dialog--fullscreen .mdc-dialog__surface .mdc-dialog__close{right:-12px}}@media(max-width: 720px)and (max-width: 672px){.mdc-dialog.mdc-dialog--fullscreen .mdc-dialog__surface{width:calc(100vw - 112px)}}@media(max-width: 720px)and (min-width: 672px){.mdc-dialog.mdc-dialog--fullscreen .mdc-dialog__surface{width:560px}}@media(max-width: 720px)and (max-height: 720px){.mdc-dialog.mdc-dialog--fullscreen .mdc-dialog__surface{max-height:calc(100vh - 160px)}}@media(max-width: 720px)and (min-height: 720px){.mdc-dialog.mdc-dialog--fullscreen .mdc-dialog__surface{max-height:560px}}@media(max-width: 720px){.mdc-dialog.mdc-dialog--fullscreen .mdc-dialog__surface .mdc-dialog__close{right:-12px}}@media(max-width: 720px)and (max-height: 400px),(max-width: 600px),(min-width: 720px)and (max-height: 400px){.mdc-dialog.mdc-dialog--fullscreen .mdc-dialog__surface{height:100%;max-height:100vh;max-width:100vw;width:100vw;border-radius:0}.mdc-dialog.mdc-dialog--fullscreen .mdc-dialog__surface .mdc-dialog__close{order:-1;left:-12px}.mdc-dialog.mdc-dialog--fullscreen .mdc-dialog__surface .mdc-dialog__header{padding:0 16px 9px;justify-content:flex-start}.mdc-dialog.mdc-dialog--fullscreen .mdc-dialog__surface .mdc-dialog__title{margin-left:calc(16px - 2 * 12px)}}@media(min-width: 960px){.mdc-dialog.mdc-dialog--fullscreen .mdc-dialog__surface{width:calc(100vw - 400px)}.mdc-dialog.mdc-dialog--fullscreen .mdc-dialog__surface .mdc-dialog__close{right:-12px}}.mdc-dialog.mdc-dialog__scrim--hidden .mdc-dialog__scrim{opacity:0}.mdc-dialog__scrim{opacity:0;z-index:-1}.mdc-dialog__container{display:flex;flex-direction:row;align-items:center;justify-content:space-around;box-sizing:border-box;height:100%;opacity:0;pointer-events:none}.mdc-dialog__surface{position:relative;display:flex;flex-direction:column;flex-grow:0;flex-shrink:0;box-sizing:border-box;max-width:100%;max-height:100%;pointer-events:auto;overflow-y:auto;outline:0;transform:scale(0.8)}.mdc-dialog__surface .mdc-elevation-overlay{width:100%;height:100%;top:0;left:0}[dir=rtl] .mdc-dialog__surface,.mdc-dialog__surface[dir=rtl]{text-align:right}@media screen and (forced-colors: active),(-ms-high-contrast: active){.mdc-dialog__surface{outline:2px solid windowText}}.mdc-dialog__surface::before{position:absolute;box-sizing:border-box;width:100%;height:100%;top:0;left:0;border:2px solid rgba(0,0,0,0);border-radius:inherit;content:"";pointer-events:none}@media screen and (forced-colors: active){.mdc-dialog__surface::before{border-color:CanvasText}}@media screen and (-ms-high-contrast: active),screen and (-ms-high-contrast: none){.mdc-dialog__surface::before{content:none}}.mdc-dialog__title{display:block;margin-top:0;position:relative;flex-shrink:0;box-sizing:border-box;margin:0 0 1px;padding:0 24px 9px}.mdc-dialog__title::before{display:inline-block;width:0;height:40px;content:"";vertical-align:0}[dir=rtl] .mdc-dialog__title,.mdc-dialog__title[dir=rtl]{text-align:right}.mdc-dialog--scrollable .mdc-dialog__title{margin-bottom:1px;padding-bottom:15px}.mdc-dialog--fullscreen .mdc-dialog__header{align-items:baseline;border-bottom:1px solid rgba(0,0,0,0);display:inline-flex;justify-content:space-between;padding:0 24px 9px;z-index:1}@media screen and (forced-colors: active){.mdc-dialog--fullscreen .mdc-dialog__header{border-bottom-color:CanvasText}}.mdc-dialog--fullscreen .mdc-dialog__header .mdc-dialog__close{right:-12px}.mdc-dialog--fullscreen .mdc-dialog__title{margin-bottom:0;padding:0;border-bottom:0}.mdc-dialog--fullscreen.mdc-dialog--scrollable .mdc-dialog__title{border-bottom:0;margin-bottom:0}.mdc-dialog--fullscreen .mdc-dialog__close{top:5px}.mdc-dialog--fullscreen.mdc-dialog--scrollable .mdc-dialog__actions{border-top:1px solid rgba(0,0,0,0)}@media screen and (forced-colors: active){.mdc-dialog--fullscreen.mdc-dialog--scrollable .mdc-dialog__actions{border-top-color:CanvasText}}.mdc-dialog--fullscreen--titleless .mdc-dialog__close{margin-top:4px}.mdc-dialog--fullscreen--titleless.mdc-dialog--scrollable .mdc-dialog__close{margin-top:0}.mdc-dialog__content{flex-grow:1;box-sizing:border-box;margin:0;overflow:auto}.mdc-dialog__content>:first-child{margin-top:0}.mdc-dialog__content>:last-child{margin-bottom:0}.mdc-dialog__title+.mdc-dialog__content,.mdc-dialog__header+.mdc-dialog__content{padding-top:0}.mdc-dialog--scrollable .mdc-dialog__title+.mdc-dialog__content{padding-top:8px;padding-bottom:8px}.mdc-dialog__content .mdc-deprecated-list:first-child:last-child{padding:6px 0 0}.mdc-dialog--scrollable .mdc-dialog__content .mdc-deprecated-list:first-child:last-child{padding:0}.mdc-dialog__actions{display:flex;position:relative;flex-shrink:0;flex-wrap:wrap;align-items:center;justify-content:flex-end;box-sizing:border-box;min-height:52px;margin:0;padding:8px;border-top:1px solid rgba(0,0,0,0)}@media screen and (forced-colors: active){.mdc-dialog__actions{border-top-color:CanvasText}}.mdc-dialog--stacked .mdc-dialog__actions{flex-direction:column;align-items:flex-end}.mdc-dialog__button{margin-left:8px;margin-right:0;max-width:100%;text-align:right}[dir=rtl] .mdc-dialog__button,.mdc-dialog__button[dir=rtl]{margin-left:0;margin-right:8px}.mdc-dialog__button:first-child{margin-left:0;margin-right:0}[dir=rtl] .mdc-dialog__button:first-child,.mdc-dialog__button:first-child[dir=rtl]{margin-left:0;margin-right:0}[dir=rtl] .mdc-dialog__button,.mdc-dialog__button[dir=rtl]{text-align:left}.mdc-dialog--stacked .mdc-dialog__button:not(:first-child){margin-top:12px}.mdc-dialog--open,.mdc-dialog--opening,.mdc-dialog--closing{display:flex}.mdc-dialog--opening .mdc-dialog__scrim{transition:opacity 150ms linear}.mdc-dialog--opening .mdc-dialog__container{transition:opacity 75ms linear,transform 150ms 0ms cubic-bezier(0, 0, 0.2, 1)}.mdc-dialog--closing .mdc-dialog__scrim,.mdc-dialog--closing .mdc-dialog__container{transition:opacity 75ms linear}.mdc-dialog--closing .mdc-dialog__container{transform:none}.mdc-dialog--closing .mdc-dialog__surface{transform:none}.mdc-dialog--chaining .mdc-dialog__scrim{transition:none;opacity:1}.mdc-dialog--open .mdc-dialog__scrim{opacity:1}.mdc-dialog--open .mdc-dialog__container{opacity:1}.mdc-dialog--open .mdc-dialog__surface{transform:none}.mdc-dialog--open.mdc-dialog__surface-scrim--shown .mdc-dialog__surface-scrim{opacity:1}.mdc-dialog--open.mdc-dialog__surface-scrim--hiding .mdc-dialog__surface-scrim{transition:opacity 75ms linear}.mdc-dialog--open.mdc-dialog__surface-scrim--showing .mdc-dialog__surface-scrim{transition:opacity 150ms linear}.mdc-dialog__surface-scrim{display:none;opacity:0;position:absolute;width:100%;height:100%;z-index:1}.mdc-dialog__surface-scrim--shown .mdc-dialog__surface-scrim,.mdc-dialog__surface-scrim--showing .mdc-dialog__surface-scrim,.mdc-dialog__surface-scrim--hiding .mdc-dialog__surface-scrim{display:block}.mdc-dialog-scroll-lock{overflow:hidden}.mdc-dialog--no-content-padding .mdc-dialog__content{padding:0}.mdc-dialog--sheet .mdc-dialog__container .mdc-dialog__close{right:12px;top:9px;position:absolute;z-index:1}.mdc-dialog__scrim--removed{pointer-events:none}.mdc-dialog__scrim--removed .mdc-dialog__scrim,.mdc-dialog__scrim--removed .mdc-dialog__surface-scrim{display:none}.mat-mdc-dialog-content{max-height:65vh}.mat-mdc-dialog-container{position:static;display:block}.mat-mdc-dialog-container,.mat-mdc-dialog-container .mdc-dialog__container,.mat-mdc-dialog-container .mdc-dialog__surface{max-height:inherit;min-height:inherit;min-width:inherit;max-width:inherit}.mat-mdc-dialog-container .mdc-dialog__surface{width:100%;height:100%}.mat-mdc-dialog-component-host{display:contents}.mat-mdc-dialog-container{--mdc-dialog-container-elevation: var(--mdc-dialog-container-elevation-shadow);outline:0}.mat-mdc-dialog-container .mdc-dialog__surface{background-color:var(--mdc-dialog-container-color, white)}.mat-mdc-dialog-container .mdc-dialog__surface{box-shadow:var(--mdc-dialog-container-elevation, 0px 11px 15px -7px rgba(0, 0, 0, 0.2), 0px 24px 38px 3px rgba(0, 0, 0, 0.14), 0px 9px 46px 8px rgba(0, 0, 0, 0.12))}.mat-mdc-dialog-container .mdc-dialog__surface{border-radius:var(--mdc-dialog-container-shape, 4px)}.mat-mdc-dialog-container .mdc-dialog__title{font-family:var(--mdc-dialog-subhead-font, Roboto, sans-serif);line-height:var(--mdc-dialog-subhead-line-height, 1.5rem);font-size:var(--mdc-dialog-subhead-size, 1rem);font-weight:var(--mdc-dialog-subhead-weight, 400);letter-spacing:var(--mdc-dialog-subhead-tracking, 0.03125em)}.mat-mdc-dialog-container .mdc-dialog__title{color:var(--mdc-dialog-subhead-color, rgba(0, 0, 0, 0.87))}.mat-mdc-dialog-container .mdc-dialog__content{font-family:var(--mdc-dialog-supporting-text-font, Roboto, sans-serif);line-height:var(--mdc-dialog-supporting-text-line-height, 1.5rem);font-size:var(--mdc-dialog-supporting-text-size, 1rem);font-weight:var(--mdc-dialog-supporting-text-weight, 400);letter-spacing:var(--mdc-dialog-supporting-text-tracking, 0.03125em)}.mat-mdc-dialog-container .mdc-dialog__content{color:var(--mdc-dialog-supporting-text-color, rgba(0, 0, 0, 0.6))}.mat-mdc-dialog-container .mdc-dialog__container{transition:opacity linear var(--mat-dialog-transition-duration, 0ms)}.mat-mdc-dialog-container .mdc-dialog__surface{transition:transform var(--mat-dialog-transition-duration, 0ms) 0ms cubic-bezier(0, 0, 0.2, 1)}.mat-mdc-dialog-container._mat-animation-noopable .mdc-dialog__container,.mat-mdc-dialog-container._mat-animation-noopable .mdc-dialog__surface{transition:none}.cdk-overlay-pane.mat-mdc-dialog-panel{max-width:var(--mat-dialog-container-max-width, 80vw);min-width:var(--mat-dialog-container-min-width, 0)}@media(max-width: 599px){.cdk-overlay-pane.mat-mdc-dialog-panel{max-width:var(--mat-dialog-container-small-max-width, 80vw)}}.mat-mdc-dialog-title{padding:var(--mat-dialog-headline-padding, 0 24px 9px)}.mat-mdc-dialog-content{display:block}.mat-mdc-dialog-container .mat-mdc-dialog-content{padding:var(--mat-dialog-content-padding, 20px 24px)}.mat-mdc-dialog-container-with-actions .mat-mdc-dialog-content{padding:var(--mat-dialog-with-actions-content-padding, 20px 24px)}.mat-mdc-dialog-container .mat-mdc-dialog-title+.mat-mdc-dialog-content{padding-top:0}.mat-mdc-dialog-actions{padding:var(--mat-dialog-actions-padding, 8px);justify-content:var(--mat-dialog-actions-alignment, start)}.mat-mdc-dialog-actions.mat-mdc-dialog-actions-align-start,.mat-mdc-dialog-actions[align=start]{justify-content:start}.mat-mdc-dialog-actions.mat-mdc-dialog-actions-align-center,.mat-mdc-dialog-actions[align=center]{justify-content:center}.mat-mdc-dialog-actions.mat-mdc-dialog-actions-align-end,.mat-mdc-dialog-actions[align=end]{justify-content:flex-end}.mat-mdc-dialog-actions .mat-button-base+.mat-button-base,.mat-mdc-dialog-actions .mat-mdc-button-base+.mat-mdc-button-base{margin-left:8px}[dir=rtl] .mat-mdc-dialog-actions .mat-button-base+.mat-button-base,[dir=rtl] .mat-mdc-dialog-actions .mat-mdc-button-base+.mat-mdc-button-base{margin-left:0;margin-right:8px}\n'],
pb:2});function hC(a){return null==a?null:"number"===typeof a?a:a.endsWith("ms")?Ts(a.substring(0,a.length-2)):a.endsWith("s")?1E3*Ts(a.substring(0,a.length-1)):"0"===a?0:null};var jC=class{constructor(a,b,c){this.Xe=a;this.nh=c;this.lh=new cj;this.rm=new cj;this.pc=0;this.yd=b.yd;this.id=a.id;a.og("mat-mdc-dialog-panel");C(c.C,tk(d=>"opened"===d.state),fo()).subscribe(()=>{this.lh.next();this.lh.complete()});C(c.C,tk(d=>"closed"===d.state),fo()).subscribe(()=>{clearTimeout(this.Ez);this.Hi()});a.Vb.J.subscribe(()=>{this.rm.next(this.ju);this.rm.complete();this.Hi()});rk(this.dd(),C(this.ef(),tk(d=>27===d.keyCode&&!this.yd&&!Bu(d)))).subscribe(d=>{this.yd||(d.preventDefault(),
this.Bi="keydown"===d.type?"keyboard":"mouse",this.close(void 0))})}close(a){this.ju=a;C(this.nh.C,tk(b=>"closing"===b.state),fo()).subscribe(b=>{this.rm.next(a);this.rm.complete();Cu(this.Xe.Vb);this.Ez=setTimeout(()=>this.Hi(),b.totalTime+100)});this.pc=1;this.nh.Ob()}pg(){return this.lh}dd(){return this.Xe.dd}ef(){return this.Xe.ef}Wc(a){let b=this.Xe.config.uc;a&&(a.left||a.right)?a.left?b.left(a.left):b.right(a.right):Nv(b);a&&(a.top||a.bottom)?a.top?b.top(a.top):b.bottom(a.bottom):Ov(b);this.Xe.Wc()}ne(a=
"",b=""){this.Xe.ne(a,b)}og(a){this.Xe.og(a)}Jj(a){this.Xe.Jj(a)}getState(){return this.pc}Hi(){this.pc=2;this.Xe.close(this.ju,{Hq:this.Bi});this.en=null}};var kC=new df("MatMdcDialogData"),lC=new df("mat-mdc-dialog-default-options"),mC=new df("mat-mdc-dialog-scroll-strategy",{ea:"root",ba:()=>{const a=lf(bw);return()=>a.C.block()}});let nC=0;function oC(a){const b=a.D;return b?oC(b):a.F}
var pC=class{get C(){return this.D?this.D.C:this.J}get pg(){return this.D?this.D.pg:this.G}constructor(a,b,c,d,e,f){this.aa=a;this.P=d;this.Kc=e;this.D=f;this.J=[];this.F=new cj;this.G=new cj;this.ja=QB;this.T=b.get(eC);this.O=jC;this.N=iC;this.fa=kC}open(a,b){let c;b=Object.assign({},this.P||new QB,b);b.id=b.id||`mat-mdc-dialog-${nC++}`;b.vc=b.vc||this.Kc();a=this.T.open(a,Object.assign({},b,{uc:Ov(Nv(Qv(this.aa.position()))),yd:!0,Yu:!1,av:!1,ab:{type:this.N,Wa:()=>[{oa:this.ja,nb:b},{oa:RB,nb:b}]},
Ho:()=>({Yk:c}),Wa:(d,e,f)=>{c=new this.O(d,b,f);let g;c.Wc(null==(g=b)?void 0:g.position);return[{oa:this.N,nb:f},{oa:this.fa,nb:e.data},{oa:this.O,nb:c}]}}));c.fv=a.fv;c.en=a.en;this.C.push(c);this.pg.next(c);c.Xe.closed.subscribe(()=>{const d=this.C.indexOf(c);-1<d&&(this.C.splice(d,1),this.C.length||oC(this).next())});return c}va(){this.ca(this.J);this.F.complete();this.G.complete()}ca(a){let b=a.length;for(;b--;)a[b].close()}};
pC.K=function(a){return new (a||pC)(r(bw),r(Wh),r(Et,8),r(lC,8),r(mC),r(pC,12),r(Yt),r(mi,8))};pC.pa=Md({ma:pC,ba:pC.K,ea:"root"});let qC=0;var sC=class{constructor(a,b,c){this.Yk=a;this.ra=b;this.C=c;this.type="button"}Fb(){this.Yk||(this.Yk=rC(this.ra,this.C.C))}tc(a){if(a=a._matDialogClose||a._matDialogCloseResult)this.Cv=a.pq}nA(a){var b=this.Yk,c=this.Cv;b.Bi=0===a.screenX&&0===a.screenY?"keyboard":"mouse";b.close(c)}};sC.K=function(a){return new (a||sC)(z(jC,8),z(si),z(pC))};
sC.Fa=Qf({type:sC,ha:[["","mat-dialog-close",""],["","matDialogClose",""]],Za:2,Ia:function(a,b){a&1&&V("click",function(c){return b.nA(c)});a&2&&Kp("aria-label",b.ariaLabel||null)("type",b.type)},inputs:{ariaLabel:[0,"aria-label","ariaLabel"],type:"type",Cv:[0,"mat-dialog-close","dialogResult"],tF:[0,"matDialogClose","_matDialogClose"]},ib:["matDialogClose"],ka:!0,features:[Dg]});
var tC=class{constructor(a,b,c){this.C=a;this.ra=b;this.G=c}Fb(){this.C||(this.C=rC(this.ra,this.G.C));this.C&&Promise.resolve().then(()=>{this.D()})}va(){let a;(null==(a=this.C)?0:a.nh)&&Promise.resolve().then(()=>{this.F()})}};tC.K=function(a){return new (a||tC)(z(jC,8),z(si),z(pC))};tC.Fa=Qf({type:tC,ka:!0});
var uC=class extends tC{constructor(){super(...arguments);this.id=`mat-mdc-dialog-title-${qC++}`}D(){let a,b;null==(a=this.C.nh)||null==(b=a.Pa)||b.call(a,this.id)}F(){let a,b,c;null==(a=this.C)||null==(b=a.nh)||null==(c=b.Lb)||c.call(b,this.id)}};uC.K=(()=>{let a;return function(b){return(a||(a=Mh(uC)))(b||uC)}})();
uC.Fa=Qf({type:uC,ha:[["","mat-dialog-title",""],["","matDialogTitle",""]],Ka:[1,"mat-mdc-dialog-title","mdc-dialog__title"],Za:1,Ia:function(a,b){a&2&&Qq("id",b.id)},inputs:{id:"id"},ib:["matDialogTitle"],ka:!0,features:[Cp]});var vC=class{};vC.K=function(a){return new (a||vC)};
vC.Fa=Qf({type:vC,ha:[["","mat-dialog-content",""],["mat-dialog-content"],["","matDialogContent",""]],Ka:[1,"mat-mdc-dialog-content","mdc-dialog__content"],ka:!0,features:[function(a){const b=c=>{const d=(Array.isArray(a)?a:a()).map(e=>"function"===typeof e?{rq:bf(e),inputs:Ef,outputs:Ef}:{rq:bf(e.rq),inputs:Ip(e.inputs),outputs:Ip(e.outputs)});null===c.ol?(c.nj=Hp,c.ol=d):c.ol.unshift(...d)};b.Mw=!0;return b}([qu])]});
var wC=class extends tC{D(){let a,b;null==(a=this.C.nh)||null==(b=a.wa)||b.call(a,1)}F(){let a,b;null==(a=this.C.nh)||null==(b=a.wa)||b.call(a,-1)}};wC.K=(()=>{let a;return function(b){return(a||(a=Mh(wC)))(b||wC)}})();
wC.Fa=Qf({type:wC,ha:[["","mat-dialog-actions",""],["mat-dialog-actions"],["","matDialogActions",""]],Ka:[1,"mat-mdc-dialog-actions","mdc-dialog__actions"],Za:6,Ia:function(a,b){a&2&&nq("mat-mdc-dialog-actions-align-start","start"===b.align)("mat-mdc-dialog-actions-align-center","center"===b.align)("mat-mdc-dialog-actions-align-end","end"===b.align)},inputs:{align:"align"},ka:!0,features:[Cp]});
function rC(a,b){let c=a.R.parentElement;for(;c&&!c.classList.contains("mat-mdc-dialog-container");)c=c.parentElement;return c?b.find(d=>d.id===c.id):null};var xC=class{};xC.K=function(a){return new (a||xC)};xC.Ya=Of({type:xC});xC.Xa=Nd({Wa:[pC],imports:[fC,jw,$v,mz,mz]});var yC=class extends pC{constructor(a,b,c,d,e,f,g,k){k.sc||kw(a,"gmat-mdc-dialog");super(a,b,c,d,e,f,g)}};yC.K=function(a){return new (a||yC)(r(bw),r(Wh),r(Et,8),r(lC,8),r(mC),r(pC,12),r(Yt),r(PB))};yC.pa=Md({ma:yC,ba:yC.K});var zC=class{};zC.K=function(a){return new (a||zC)};zC.Ya=Of({type:zC});zC.Xa=Nd({Wa:[{oa:pC,Ec:yC},{oa:bw,Ec:lw}],imports:[xC]});new df("GMAT_FORM_FIELD_OPTIONS",{ea:"root",ba:()=>({sc:!1})});var AC=class{};AC.K=function(a){return new (a||AC)};AC.Ya=Of({type:AC});AC.Xa=Nd({imports:[cB]});var BC=new df("GMAT_MENU_OPTIONS",{ea:"root",ba:()=>({sc:!1})});var CC=class{constructor(a,b){this.options=a;a.sc||kw(b,"gmat-mdc-menu")}};CC.K=function(a){return new (a||CC)(z(BC),z(bw))};CC.Fa=Qf({type:CC,ha:[["","mat-menu-trigger-for",""],["","matMenuTriggerFor",""]],features:[Kr([{oa:bw,Ec:lw}])]});var DC=Yy("transformMenu",[az("void",$y({opacity:0,transform:"scale(0.8)"})),bz("void => enter",Zy("120ms cubic-bezier(0, 0, 0.2, 1)",$y({opacity:1,transform:"scale(1)"}))),bz("* => void",Zy("100ms 25ms linear",$y({opacity:0})))]),EC=Yy("fadeInItems",[az("showing",$y({opacity:1})),bz("void => *",[$y({opacity:0}),Zy("400ms 100ms cubic-bezier(0.55, 0, 0.55, 0.2)")])]);var FC=new df("MatMenuContent");var GC=new df("MAT_MENU_PANEL");const HC=[[["mat-icon"],["","matMenuItemIcon",""]],"*"];function IC(a){a&1&&(fh(),R(0,"svg",2),Mq(1,"polygon",3),S())}
var JC=class{constructor(a,b,c,d,e){this.ra=a;this.ia=b;this.C=c;this.D=d;this.Ba=e;this.role="menuitem";this.Nc=this.disabled=!1;this.F=new cj;this.Qe=new cj;this.Qp=this.Pt=!1;let f;null==d||null==(f=d.UA)||f.call(d,this)}focus(a,b){this.C&&a?Nx(this.C,this.qe(),a,b):this.qe().focus(b);this.Qe.next(this)}pd(){this.C&&Lx(this.C,this.ra,!1)}va(){this.C&&Mx(this.C,this.ra);this.D&&this.D.removeItem&&this.D.removeItem(this);this.F.complete();this.Qe.complete()}Gt(){return this.disabled?"-1":"0"}qe(){return this.ra.R}Cz(a){this.disabled&&
(a.preventDefault(),a.stopPropagation())}Zz(){this.F.next(this)}Vv(){const a=this.ra.R.cloneNode(!0),b=a.querySelectorAll("mat-icon, .material-icons");for(let d=0;d<b.length;d++)b[d].remove();let c;return(null==(c=a.textContent)?void 0:c.trim())||""}G(a){this.Pt=a;let b;null==(b=this.Ba)||bn(b)}J(a){this.Qp=a;let b;null==(b=this.Ba)||bn(b)}cA(){return this.ia&&this.ia.activeElement===this.qe()}};JC.K=function(a){return new (a||JC)(z(si),z(jt),z(Ox),z(GC,8),z(Sr))};
JC.Da=Gf({type:JC,ha:[["","mat-menu-item",""]],Ka:[1,"mat-mdc-menu-item","mat-mdc-focus-indicator"],Za:8,Ia:function(a,b){a&1&&V("click",function(c){return b.Cz(c)})("mouseenter",function(){return b.Zz()});a&2&&(Kp("role",b.role)("tabindex",b.Gt())("aria-disabled",b.disabled)("disabled",b.disabled||null),nq("mat-mdc-menu-item-highlighted",b.Pt)("mat-mdc-menu-item-submenu-trigger",b.Qp))},inputs:{role:"role",disabled:[2,"disabled","disabled",zs],Nc:[2,"disableRipple","disableRipple",zs]},ib:["matMenuItem"],
ka:!0,features:[Jp,Mr],attrs:["mat-menu-item",""],Zb:["mat-icon, [matMenuItemIcon]","*"],za:5,Aa:3,Ca:[[1,"mat-mdc-menu-item-text"],["matRipple","",1,"mat-mdc-menu-ripple",3,"matRippleDisabled","matRippleTrigger"],["viewBox","0 0 5 10","focusable","false","aria-hidden","true",1,"mat-mdc-menu-submenu-icon"],["points","0,0 5,5 0,10"]],ta:function(a,b){a&1&&(Xq(HC),Yq(0),R(1,"span",0),Yq(2,1),S(),Mq(3,"div",1),Ap(4,IC,2,0,":svg:svg",2));a&2&&(D(3),H("matRippleDisabled",b.Nc||b.disabled)("matRippleTrigger",
b.qe()),D(),Eq(b.Qp?4:-1))},Ha:[Lz],pb:2,Ra:0});function KC(a){if(a&1){const b=Ug();R(0,"div",0);V("keydown",function(c){u(b);const d=Y();return x(d.Se(c))})("click",function(){u(b);const c=Y();return x(c.closed.emit("click"))})("@transformMenu.start",function(c){u(b);const d=Y();return x(d.lA(c))})("@transformMenu.done",function(c){u(b);const d=Y();return x(d.kA(c))});R(1,"div",1);Yq(2);S()()}a&2&&(a=Y(),qq(tq,a.Pe),H("id",a.Ww)("@transformMenu",a.Im),Kp("aria-label",a.ariaLabel||null)("aria-labelledby",a.cd||null)("aria-describedby",a.Hk||null))}
let LC=0;var NC=new df("mat-menu-default-options",{ea:"root",ba:MC});function MC(){return{mo:!1,ri:"after",Yj:"below",dc:"cdk-overlay-transparent-backdrop"}}function OC(a){return C(a.C.C,Ho(a.C),Io(b=>rk(...b.map(c=>c.F))))}function PC(a,b="program"){let c;null==(c=a.G)||c.destroy();a.G=al(()=>{let d=null;a.C.length&&(d=a.C.first.qe().closest('[role="menu"]'));if(!d||!d.contains(document.activeElement)){const e=a.D;e.sa=b;e.J(0,1);!e.C&&d&&d.focus()}},{xa:a.Ua})}
var QC=class{get ri(){return this.Ff}set ri(a){this.Ff=a;this.Qj()}get Yj(){return this.N}set Yj(a){this.N=a;this.Qj()}set hb(a){const b=this.O,c=Object.assign({},this.Pe);b&&b.length&&b.split(" ").forEach(d=>{c[d]=!1});(this.O=a)&&a.length&&(a.split(" ").forEach(d=>{c[d]=!0}),this.ra.R.className="");this.Pe=c}get classList(){return this.hb}set classList(a){this.hb=a}constructor(a,b,c,d){this.ra=a;this.Ba=d;this.fa="mat-elevation-z";this.ca=8;this.C=new ar;this.Pe={};this.Im="void";this.hg=new cj;
this.close=this.closed=new Fk;this.Ww=`mat-menu-panel-${LC++}`;this.Ua=lf(Wh);this.Wh=c.Wh||"";this.Ff=c.ri;this.N=c.Yj;this.dc=c.dc;this.mo=c.mo;this.Tb=c.Tb}Fb(){this.Qj()}yj(){this.aa();this.D=Py(Oy(Ky(new Xy(this.C))));this.D.ca.subscribe(()=>this.closed.emit("tab"));C(this.C.C,Ho(this.C),Io(a=>rk(...a.map(b=>b.Qe)))).subscribe(a=>Ry(this.D,a));this.C.C.subscribe(a=>{const b=this.D;var c;"enter"===this.Im&&(null==(c=b.C)?0:c.cA())&&(a=a.toArray(),c=Math.max(0,Math.min(a.length-1,b.O||0)),a[c]&&
!a[c].disabled?b.D(c):Sy(b))})}va(){let a;null==(a=this.D)||a.destroy();this.C.destroy();this.closed.complete();let b;null==(b=this.G)||b.destroy()}UA(){}removeItem(){}Se(a){const b=a.keyCode,c=this.D;switch(b){case 27:Bu(a)||(a.preventDefault(),this.closed.emit("keydown"));break;case 37:this.no&&"ltr"===this.direction&&this.closed.emit("keydown");break;case 39:this.no&&"rtl"===this.direction&&this.closed.emit("keydown");break;default:if(38===b||40===b)c.sa="keyboard";Uy(c,a);return}a.stopPropagation()}Hx(a){a=
`${"mat-elevation-z"}${Math.min(8+a,24)}`;var b=Object.keys(this.Pe).find(c=>c.startsWith("mat-elevation-z"));b&&b!==this.F||(b=Object.assign({},this.Pe),this.F&&(b[this.F]=!1),b[a]=!0,this.F=a,this.Pe=b)}Qj(a=this.ri,b=this.Yj){this.Pe=Object.assign({},this.Pe,{["mat-menu-before"]:"before"===a,["mat-menu-after"]:"after"===a,["mat-menu-above"]:"above"===b,["mat-menu-below"]:"below"===b});let c;null==(c=this.Ba)||bn(c)}T(){this.Im="enter"}P(){this.Im="void"}kA(a){this.hg.next(a);this.J=!1}lA(a){this.J=
!0;"enter"===a.me&&0===this.D.O&&(a.element.scrollTop=0)}aa(){C(this.kt.C,Ho(this.kt)).subscribe(a=>{this.C.reset(a.filter(b=>b.D===this));$q(this.C)})}};QC.K=function(a){return new (a||QC)(z(si),z(Qk),z(NC),z(Sr))};
QC.Da=Gf({type:QC,ha:[["mat-menu"]],ec:function(a,b,c){a&1&&(tr(c,FC,5),tr(c,JC,5),tr(c,JC,4));if(a&2){let d;vr(d=wr())&&(b.tj=d.first);vr(d=wr())&&(b.kt=d);vr(d=wr())&&(b.items=d)}},fb:function(a,b){a&1&&ur(br,5);if(a&2){let c;vr(c=wr())&&(b.cg=c.first)}},Za:3,Ia:function(a){a&2&&Kp("aria-label",null)("aria-labelledby",null)("aria-describedby",null)},inputs:{dc:"backdropClass",ariaLabel:[0,"aria-label","ariaLabel"],cd:[0,"aria-labelledby","ariaLabelledby"],Hk:[0,"aria-describedby","ariaDescribedby"],
ri:"xPosition",Yj:"yPosition",mo:[2,"overlapTrigger","overlapTrigger",zs],Tb:[2,"hasBackdrop","hasBackdrop",a=>null==a?null:zs(a)],hb:[0,"class","panelClass"],classList:"classList"},outputs:{closed:"closed",close:"close"},ib:["matMenu"],ka:!0,features:[Kr([{oa:GC,Ab:QC}]),Jp,Mr],Zb:["*"],za:1,Aa:0,Ca:[["tabindex","-1","role","menu",1,"mat-mdc-menu-panel","mat-mdc-elevation-specific",3,"keydown","click","id"],[1,"mat-mdc-menu-content"]],ta:function(a){a&1&&(Xq(),Ap(0,KC,3,7,"ng-template"))},styles:['mat-menu{display:none}.mat-mdc-menu-content{margin:0;padding:8px 0;list-style-type:none}.mat-mdc-menu-content:focus{outline:none}.mat-mdc-menu-content,.mat-mdc-menu-content .mat-mdc-menu-item .mat-mdc-menu-item-text{-moz-osx-font-smoothing:grayscale;-webkit-font-smoothing:antialiased;flex:1;white-space:normal;font-family:var(--mat-menu-item-label-text-font);line-height:var(--mat-menu-item-label-text-line-height);font-size:var(--mat-menu-item-label-text-size);letter-spacing:var(--mat-menu-item-label-text-tracking);font-weight:var(--mat-menu-item-label-text-weight)}.mat-mdc-menu-panel{min-width:112px;max-width:280px;overflow:auto;-webkit-overflow-scrolling:touch;box-sizing:border-box;outline:0;border-radius:var(--mat-menu-container-shape);background-color:var(--mat-menu-container-color);will-change:transform,opacity}.mat-mdc-menu-panel.ng-animating{pointer-events:none}.cdk-high-contrast-active .mat-mdc-menu-panel{outline:solid 1px}.mat-divider{color:var(--mat-menu-divider-color);margin-bottom:var(--mat-menu-divider-bottom-spacing);margin-top:var(--mat-menu-divider-top-spacing)}.mat-mdc-menu-item{display:flex;position:relative;align-items:center;justify-content:flex-start;overflow:hidden;padding:0;padding-left:var(--mat-menu-item-leading-spacing);padding-right:var(--mat-menu-item-trailing-spacing);-webkit-user-select:none;user-select:none;cursor:pointer;outline:none;border:none;-webkit-tap-highlight-color:rgba(0,0,0,0);cursor:pointer;width:100%;text-align:left;box-sizing:border-box;color:inherit;font-size:inherit;background:none;text-decoration:none;margin:0;align-items:center;min-height:48px}.mat-mdc-menu-item:focus{outline:none}[dir=rtl] .mat-mdc-menu-item,.mat-mdc-menu-item[dir=rtl]{padding-left:var(--mat-menu-item-trailing-spacing);padding-right:var(--mat-menu-item-leading-spacing)}.mat-mdc-menu-item:has(.material-icons,mat-icon,[matButtonIcon]){padding-left:var(--mat-menu-item-with-icon-leading-spacing);padding-right:var(--mat-menu-item-with-icon-trailing-spacing)}[dir=rtl] .mat-mdc-menu-item:has(.material-icons,mat-icon,[matButtonIcon]),.mat-mdc-menu-item:has(.material-icons,mat-icon,[matButtonIcon])[dir=rtl]{padding-left:var(--mat-menu-item-with-icon-trailing-spacing);padding-right:var(--mat-menu-item-with-icon-leading-spacing)}.mat-mdc-menu-item::-moz-focus-inner{border:0}.mat-mdc-menu-item,.mat-mdc-menu-item:visited,.mat-mdc-menu-item:link{color:var(--mat-menu-item-label-text-color)}.mat-mdc-menu-item .mat-icon-no-color,.mat-mdc-menu-item .mat-mdc-menu-submenu-icon{color:var(--mat-menu-item-icon-color)}.mat-mdc-menu-item[disabled]{cursor:default;opacity:.38}.mat-mdc-menu-item[disabled]::after{display:block;position:absolute;content:"";top:0;left:0;bottom:0;right:0}.mat-mdc-menu-item .mat-icon{flex-shrink:0;margin-right:var(--mat-menu-item-spacing);height:var(--mat-menu-item-icon-size);width:var(--mat-menu-item-icon-size)}[dir=rtl] .mat-mdc-menu-item{text-align:right}[dir=rtl] .mat-mdc-menu-item .mat-icon{margin-right:0;margin-left:var(--mat-menu-item-spacing)}.mat-mdc-menu-item:not([disabled]):hover{background-color:var(--mat-menu-item-hover-state-layer-color)}.mat-mdc-menu-item:not([disabled]).cdk-program-focused,.mat-mdc-menu-item:not([disabled]).cdk-keyboard-focused,.mat-mdc-menu-item:not([disabled]).mat-mdc-menu-item-highlighted{background-color:var(--mat-menu-item-focus-state-layer-color)}.cdk-high-contrast-active .mat-mdc-menu-item{margin-top:1px}.mat-mdc-menu-submenu-icon{width:var(--mat-menu-item-icon-size);height:10px;fill:currentColor;padding-left:var(--mat-menu-item-spacing)}[dir=rtl] .mat-mdc-menu-submenu-icon{padding-right:var(--mat-menu-item-spacing);padding-left:0}[dir=rtl] .mat-mdc-menu-submenu-icon polygon{transform:scaleX(-1)}.cdk-high-contrast-active .mat-mdc-menu-submenu-icon{fill:CanvasText}.mat-mdc-menu-item .mat-mdc-menu-ripple{top:0;left:0;right:0;bottom:0;position:absolute;pointer-events:none}\n'],
pb:2,data:{animation:[DC,EC]},Ra:0});var RC=new df("mat-menu-scroll-strategy",{ea:"root",ba:()=>{const a=lf(bw);return()=>a.C.C()}}),SC={oa:RC,Ac:[bw],Fc:function(a){return()=>a.C.C()}};const TC=Kt({passive:!0});function UC(a){return!!(a.G&&a.C&&a.Hd)}function VC(a){let b;null==(b=a.Hd)||b.close.emit()}
function WC(a){const b=a.Hd;if(!a.F&&b){var c=XC(a,b),d=c.ya,e=d.uc;a.gb(b,e);d.Tb=null==b.Tb?!UC(a):b.Tb;c.attach(a.Ja(b));b.tj&&b.tj.attach(a.NC);a.O=a.Va().subscribe(()=>VC(a));a.Sa(b);b instanceof QC&&(b.T(),C(b.C.C,Lo(b.close)).subscribe(()=>{Iu(Fv(e,!1));Fv(e,!0)}))}}
function XC(a,b){if(!a.la){var c=new Au({uc:Hv(Nu(Fv(Rv(a.ja.position(),a.J))),".mat-menu-panel, .mat-mdc-menu-panel"),dc:b.dc||"cdk-overlay-transparent-backdrop",hb:b.Wh,vc:a.Kc(),direction:a.P});a.lb(b,c.uc);a.la=a.ja.create(c);a.la.ef().subscribe()}return a.la}
var YC=class{get vt(){return this.Hd}set vt(a){this.Hd=a}get Hd(){return this.ua}set Hd(a){if(a!==this.ua){this.ua=a;this.aa.unsubscribe();a&&(this.aa=a.close.subscribe(c=>{this.Ea(c);"click"!==c&&"tab"!==c||!this.C||this.C.closed.emit(c)}));var b;null==(b=this.G)||b.J(UC(this))}}constructor(a,b,c,d,e,f,g,k,m){this.ja=a;this.J=b;this.wa=c;this.G=f;this.P=g;this.fa=k;this.qa=m;this.la=null;this.F=!1;this.aa=this.sa=this.O=Ni.EMPTY;this.Ba=lf(Sr);this.na=p=>{Ex(p)||(this.D="touch")};this.D=void 0;this.Ng=
!0;this.hD=this.pr=new Fk;this.gD=this.mr=new Fk;this.Kc=d;this.C=e instanceof QC?e:void 0;b.R.addEventListener("touchstart",this.na,TC)}yj(){this.Pa()}va(){this.la&&(this.la.dispose(),this.la=null);this.J.R.removeEventListener("touchstart",this.na,TC);this.aa.unsubscribe();this.O.unsubscribe();this.sa.unsubscribe()}get nr(){return this.F}get dir(){return this.P&&"rtl"===this.P.value?"rtl":"ltr"}focus(a,b){this.fa&&a?Nx(this.fa,this.J,a,b):this.J.R.focus(b)}Wc(){let a;null==(a=this.la)||a.Wc()}Ea(a){if(this.la&&
this.nr){var b=this.Hd;this.O.unsubscribe();this.la.detach();!this.Ng||"keydown"!==a&&this.D&&UC(this)||this.focus(this.D);this.D=void 0;if(b instanceof QC)b.P(),b.tj?C(b.hg,tk(c=>"void"===c.me),fo(),Lo(b.tj.C)).subscribe({next:()=>b.tj.detach(),complete:()=>this.N(!1)}):this.N(!1);else{this.N(!1);let c;null==b||null==(c=b.tj)||c.detach()}}}Sa(a){a.no=UC(this)?this.C:void 0;a.direction=this.dir;this.bb(a);PC(a,this.D||"program");this.N(!0)}bb(a){if(a.Hx){let b=0,c=a.no;for(;c;)b++,c=c.no;a.Hx(b)}}N(a){a!==
this.F&&((this.F=a)?this.pr.emit():this.mr.emit(),UC(this)&&this.G.G(a),bn(this.Ba))}lb(a,b){a.Qj&&b.gb.subscribe(c=>{const d="start"===c.C.Na?"after":"before",e="top"===c.C.Ta?"below":"above";this.qa?this.qa.run(()=>a.Qj(d,e)):a.Qj(d,e)})}gb(a,b){let [c,d]="before"===a.ri?["end","start"]:["start","end"],e=c,f=d,[g,k]="above"===a.Yj?["bottom","top"]:["top","bottom"],[m,p]=[g,k],t=m,w=p,[B,G]=[e,f],J=B,N=G,aa=0;UC(this)?(N=e="before"===a.ri?"start":"end",f=J="end"===e?"start":"end",this.C&&(null==
this.ca&&(this.ca=(a=this.C.items.first)?a.qe().offsetTop:0),aa="bottom"===g?this.ca:-this.ca)):a.mo||(t="top"===g?"bottom":"top",w="top"===k?"bottom":"top");Ku(b,[{cb:e,eb:t,Na:J,Ta:g,offsetY:aa},{cb:f,eb:t,Na:N,Ta:g,offsetY:aa},{cb:e,eb:w,Na:J,Ta:k,offsetY:-aa},{cb:f,eb:w,Na:N,Ta:k,offsetY:-aa}])}Va(){const a=this.la.dd(),b=this.la.J,c=this.C?this.C.closed:jj(),d=this.C?C(OC(this.C),tk(e=>e!==this.G),tk(()=>this.F)):jj();return rk(a,c,d,b)}bA(a){Dx(a)||(this.D=0===a.button?"mouse":void 0,UC(this)&&
a.preventDefault())}Se(a){a=a.keyCode;if(13===a||32===a)this.D="keyboard";UC(this)&&(39===a&&"ltr"===this.dir||37===a&&"rtl"===this.dir)&&(this.D="keyboard",WC(this))}pk(a){UC(this)?(a.stopPropagation(),WC(this)):this.F?VC(this):WC(this)}Pa(){UC(this)&&this.C&&(this.sa=C(OC(this.C),tk(a=>a===this.G&&!a.disabled),Xn(Dk)).subscribe(()=>{this.D="mouse";this.Hd instanceof QC&&this.Hd.J?C(this.Hd.hg,fo(),Xn(Dk),Lo(OC(this.C))).subscribe(()=>WC(this)):WC(this)}))}Ja(a){this.T&&this.T.cg===a.cg||(this.T=
new Vv(a.cg,this.wa));return this.T}};YC.K=function(a){return new (a||YC)(z(bw),z(si),z(kp),z(RC),z(GC,8),z(JC,10),z(ou,8),z(Ox),z(Qk))};
YC.Fa=Qf({type:YC,ha:[["","mat-menu-trigger-for",""],["","matMenuTriggerFor",""]],Ka:[1,"mat-mdc-menu-trigger"],Za:3,Ia:function(a,b){a&1&&V("click",function(c){return b.pk(c)})("mousedown",function(c){return b.bA(c)})("keydown",function(c){return b.Se(c)});a&2&&Kp("aria-haspopup",b.Hd?"menu":null)("aria-expanded",b.nr)("aria-controls",b.nr?b.Hd.Ww:null)},inputs:{vt:[0,"mat-menu-trigger-for","_deprecatedMatMenuTriggerFor"],Hd:[0,"matMenuTriggerFor","menu"],NC:[0,"matMenuTriggerData","menuData"],Ng:[0,
"matMenuTriggerRestoreFocus","restoreFocus"]},outputs:{pr:"menuOpened",hD:"onMenuOpen",mr:"menuClosed",gD:"onMenuClose"},ib:["matMenuTrigger"],ka:!0});var ZC=class{};ZC.K=function(a){return new (a||ZC)};ZC.Ya=Of({type:ZC});ZC.Xa=Nd({Wa:[SC],imports:[ht,Yz,mz,jw,xu,mz]});var $C=class{};$C.K=function(a){return new (a||$C)};$C.Ya=Of({type:$C});$C.Xa=Nd({Wa:[{oa:bw,Ec:lw}],imports:[ZC]});var aD={AED:[2,"dh","\u062f.\u0625."],ALL:[0,"Lek","Lek"],AUD:[2,"$","AU$"],BDT:[2,"\u09f3","Tk"],BGN:[2,"lev","lev"],BRL:[2,"R$","R$"],CAD:[2,"$","C$"],CDF:[2,"FrCD","CDF"],CHF:[2,"CHF","CHF"],CLP:[0,"$","CL$"],CNY:[2,"\u00a5","RMB\u00a5"],COP:[32,"$","COL$"],CRC:[0,"\u20a1","CR\u20a1"],CZK:[50,"K\u010d","K\u010d"],DKK:[50,"kr.","kr."],DOP:[2,"RD$","RD$"],EGP:[2,"\u00a3","LE"],ETB:[2,"Birr","Birr"],EUR:[2,"\u20ac","\u20ac"],GBP:[2,"\u00a3","GB\u00a3"],HKD:[2,"$","HK$"],HRK:[2,"kn","kn"],HUF:[34,
"Ft","Ft"],IDR:[0,"Rp","Rp"],ILS:[34,"\u20aa","IL\u20aa"],INR:[2,"\u20b9","Rs"],IRR:[0,"Rial","IRR"],ISK:[0,"kr","kr"],JMD:[2,"$","JA$"],JPY:[0,"\u00a5","JP\u00a5"],KRW:[0,"\u20a9","KR\u20a9"],LKR:[2,"Rs","SLRs"],LTL:[2,"Lt","Lt"],MNT:[0,"\u20ae","MN\u20ae"],MVR:[2,"Rf","MVR"],MXN:[2,"$","Mex$"],MYR:[2,"RM","RM"],NOK:[50,"kr","NOkr"],PAB:[2,"B/.","B/."],PEN:[2,"S/.","S/."],PHP:[2,"\u20b1","PHP"],PKR:[0,"Rs","PKRs."],PLN:[50,"z\u0142","z\u0142"],RON:[2,"RON","RON"],RSD:[0,"din","RSD"],RUB:[50,"\u20bd",
"RUB"],SAR:[2,"SAR","SAR"],SEK:[50,"kr","kr"],SGD:[2,"$","S$"],THB:[2,"\u0e3f","THB"],TRY:[2,"\u20ba","TRY"],TWD:[2,"$","NT$"],TZS:[0,"TSh","TSh"],UAH:[2,"\u0433\u0440\u043d.","UAH"],USD:[2,"$","US$"],UYU:[2,"$","$U"],VND:[48,"\u20ab","VN\u20ab"],YER:[0,"Rial","Rial"],ZAR:[2,"R","ZAR"]};var bD={Vs:".",Oo:",",bt:"%",Xo:"0",pz:"+",Zs:"-",Xs:"E",ct:"\u2030",Qo:"\u221e",kz:"NaN",ZE:"#,##0.###",iF:"#E0",oz:"#,##0%",YE:"\u00a4#,##0.00",Ws:"USD"},cD=bD;cD=bD;function dD(){this.aa=40;this.D=1;this.F=3;this.fa=this.G=0;this.sa=!1;this.ca=this.P="";this.J=cD.Zs;this.T="";this.C=1;this.O=!1;this.N=[];this.ja=this.na=!1;var a=cD.oz;const b=[0];this.P=eD(this,a,b);var c=b[0];let d=-1,e=0;var f=0,g=0;let k=-1;const m=a.length;for(let p=!0;b[0]<m&&p;b[0]++)switch(a.charAt(b[0])){case "#":0<f?g++:e++;0<=k&&0>d&&k++;break;case "0":if(0<g)throw Error('Unexpected "0" in pattern "'+a+'"');f++;0<=k&&0>d&&k++;break;case ",":0<k&&this.N.push(k);k=0;break;case ".":if(0<=
d)throw Error('Multiple decimal separators in pattern "'+a+'"');d=e+f+g;break;case "E":if(this.ja)throw Error('Multiple exponential symbols in pattern "'+a+'"');this.ja=!0;this.fa=0;b[0]+1<m&&"+"==a.charAt(b[0]+1)&&(b[0]++,this.sa=!0);for(;b[0]+1<m&&"0"==a.charAt(b[0]+1);)b[0]++,this.fa++;if(1>e+f||1>this.fa)throw Error('Malformed exponential pattern "'+a+'"');p=!1;break;default:b[0]--,p=!1}0==f&&0<e&&0<=d&&(f=d,0==f&&f++,g=e-f,e=f-1,f=1);if(0>d&&0<g||0<=d&&(d<e||d>e+f)||0==k)throw Error('Malformed pattern "'+
a+'"');g=e+f+g;this.F=0<=d?g-d:0;0<=d&&(this.G=e+f-d,0>this.G&&(this.G=0));this.D=(0<=d?d:g)-e;this.ja&&(this.aa=e+this.D,0==this.F&&0==this.D&&(this.D=1));this.N.push(Math.max(0,k));this.na=0==d||d==g;c=b[0]-c;this.ca=eD(this,a,b);b[0]<a.length&&";"==a.charAt(b[0])?(b[0]++,1!=this.C&&(this.O=!0),this.J=eD(this,a,b),b[0]+=c,this.T=eD(this,a,b)):(this.J+=this.P,this.T+=this.ca)}
dD.prototype.parse=function(a,b){b=b||[0];a=a.replace(/ |\u202f/g,"\u00a0");let c=a.indexOf(this.P,b[0])==b[0],d=a.indexOf(this.J,b[0])==b[0];c&&d&&(this.P.length>this.J.length?d=!1:this.P.length<this.J.length&&(c=!1));c?b[0]+=this.P.length:d&&(b[0]+=this.J.length);if(a.indexOf(cD.Qo,b[0])==b[0]){b[0]+=cD.Qo.length;var e=Infinity}else{e=a;let f=!1,g=!1,k=!1,m=-1,p=1;const t=cD.Vs;let w=cD.Oo;const B=cD.Xs;w=w.replace(/\u202f/g,"\u00a0");let G="";for(;b[0]<e.length;b[0]++){const J=e.charAt(b[0]),N=
fD(J);if(0<=N&&9>=N)G+=N,k=!0;else if(J==t.charAt(0)){if(f||g)break;G+=".";f=!0}else if(J==w.charAt(0)&&("\u00a0"!=w.charAt(0)||b[0]+1<e.length&&0<=fD(e.charAt(b[0]+1)))){if(f||g)break}else if(J==B.charAt(0)){if(g)break;G+="E";g=!0;m=b[0]}else if("+"==J||"-"==J){if(k&&m!=b[0]-1)break;G+=J}else if(1==this.C&&J==cD.bt.charAt(0)){if(1!=p)break;p=100;if(k){b[0]++;break}}else if(1==this.C&&J==cD.ct.charAt(0)){if(1!=p)break;p=1E3;if(k){b[0]++;break}}else break}1!=this.C&&(p=this.C);e=parseFloat(G)/p}if(c){if(a.indexOf(this.ca,
b[0])!=b[0])return NaN;b[0]+=this.ca.length}else if(d){if(a.indexOf(this.T,b[0])!=b[0])return NaN;b[0]+=this.T.length}return d?-e:e};
dD.prototype.format=function(a){if(this.G>this.F)throw Error("Min value must be less than max value");if(isNaN(a))return cD.kz;const b=[];var c=gD;a=hD(a,-c.EB);const d=0>a||0==a&&0>1/a;d?c.Iw?b.push(c.Iw):(b.push(c.prefix),b.push(this.J)):(b.push(c.prefix),b.push(this.P));if(isFinite(a))if(a*=d?-1:1,a*=this.C,this.ja){var e=a;if(0==e)iD(this,e,this.D,b),jD(this,0,b);else{var f=Math.floor(Math.log(e)/Math.log(10)+2E-15);e=hD(e,-f);var g=this.D;1<this.aa&&this.aa>this.D?(g=f%this.aa,0>g&&(g=this.aa+
g),e=hD(e,g),f-=g,g=1):1>this.D?(f++,e=hD(e,-1)):(f-=this.D-1,e=hD(e,this.D-1));iD(this,e,g,b);jD(this,f,b)}}else iD(this,a,this.D,b);else b.push(cD.Qo);d?c.Jw?b.push(c.Jw):(isFinite(a)&&b.push(c.iy),b.push(this.T)):(isFinite(a)&&b.push(c.iy),b.push(this.ca));return b.join("")};
function iD(a,b,c,d){if(a.G>a.F)throw Error("Min value must be less than max value");d||(d=[]);var e=hD(b,a.F);e=Math.round(e);isFinite(e)?(b=Math.floor(hD(e,-a.F)),e=Math.floor(e-hD(b,a.F))):e=0;var f=b,g=e,k=0<a.G||0<g||!1;e=a.G;k&&(e=a.G);var m="";for(b=f;1E20<b;)m="0"+m,b=Math.round(hD(b,-1));m=b+m;var p=cD.Vs;b=cD.Xo.codePointAt(0);var t=m.length,w=0;if(0<f||0<c){for(f=t;f<c;f++)d.push(String.fromCodePoint(b));if(2<=a.N.length)for(c=1;c<a.N.length;c++)w+=a.N[c];c=t-w;if(0<c){f=a.N;w=t=0;var B=
cD.Oo,G=m.length;for(let N=0;N<G;N++)if(d.push(String.fromCodePoint(b+Number(m.charAt(N)))),1<G-N){var J=f[w];if(N<c){let aa=c-N;(1===J||0<J&&1===aa%J)&&d.push(B)}else w<f.length&&(N===c?w+=1:J===N-c-t+1&&(d.push(B),t+=J,w+=1))}}else{c=m;m=a.N;f=cD.Oo;J=c.length;B=[];for(t=m.length-1;0<=t&&0<J;t--){w=m[t];for(G=0;G<w&&0<=J-G-1;G++)B.push(String.fromCodePoint(b+Number(c.charAt(J-G-1))));J-=w;0<J&&B.push(f)}d.push.apply(d,B.reverse())}}else k||d.push(String.fromCodePoint(b));(a.na||k)&&d.push(p);k=
String(g);g=k.split("e+");if(2==g.length){if(k=parseFloat(g[0])){p=k;if(isFinite(p)){for(c=0;1<=(p/=10);)c++;p=c}else p=0<p?p:0;p=-p-1;k=-1>p?k&&isFinite(k)?hD(Math.round(hD(k,-1)),1):k:k&&isFinite(k)?hD(Math.round(hD(k,p)),-p):k}k=String(k);k=k.replace(".","");k+=wd("0",parseInt(g[1],10)-k.length+1)}a.F+1>k.length&&(k="1"+wd("0",a.F-k.length)+k);for(a=k.length;"0"==k.charAt(a-1)&&a>e+1;)a--;for(e=1;e<a;e++)d.push(String.fromCodePoint(b+Number(k.charAt(e))))}
function jD(a,b,c){c.push(cD.Xs);0>b?(b=-b,c.push(cD.Zs)):a.sa&&c.push(cD.pz);b=""+b;var d=cD.Xo;for(let e=b.length;e<a.fa;e++)c.push(d);a=d.codePointAt(0)-kD;for(d=0;d<b.length;d++)c.push(String.fromCodePoint(a+b.codePointAt(d)))}function fD(a){a=a.codePointAt(0);if(48<=a&&58>a)return a-48;const b=cD.Xo.codePointAt(0);return b<=a&&a<b+10?a-b:-1}var kD="0".codePointAt(0);
function eD(a,b,c){let d="",e=!1;const f=b.length;for(;c[0]<f;c[0]++){var g=b.charAt(c[0]);if("'"==g)c[0]+1<f&&"'"==b.charAt(c[0]+1)?(c[0]++,d+="'"):e=!e;else if(e)d+=g;else switch(g){case "#":case "0":case ",":case ".":case ";":return d;case "\u00a4":c[0]+1<f&&"\u00a4"==b.charAt(c[0]+1)?(c[0]++,d+=cD.Ws):(g=cD.Ws,d+=g in aD?aD[g][1]:g);break;case "%":if(!a.O&&1!=a.C)throw Error("Too many percent/permill");if(a.O&&100!=a.C)throw Error("Inconsistent use of percent/permill characters");a.C=100;a.O=
!1;d+=cD.bt;break;case "\u2030":if(!a.O&&1!=a.C)throw Error("Too many percent/permill");if(a.O&&1E3!=a.C)throw Error("Inconsistent use of percent/permill characters");a.C=1E3;a.O=!1;d+=cD.ct;break;default:d+=g}}return d}var gD={EB:0,Iw:"",Jw:"",prefix:"",iy:""};function hD(a,b){if(!a||!isFinite(a)||0==b)return a;a=String(a).split("e");return parseFloat(a[0]+"e"+(parseInt(a[1]||0,10)+b))};var lD=new dD;function mD(a){const b={attributeFilter:["aria-label","aria-valuenow"]};return new Xi(c=>{(new MutationObserver(d=>{const e=d[0].target;var f;d=null!=(f=e.getAttribute("aria-label"))?f:"";let g;f=null!=(g=e.getAttribute("aria-valuenow"))?g:"indeterminate";c.next([d,f])})).observe(a,b)})};var nD=new df("GMAT_PROGRESS_SPINNER_OPTIONS",{ea:"root",ba:()=>({sc:!1})});const oD=["determinateSpinner"];function pD(a){a&1&&(fh(),R(0,"svg",11),Mq(1,"circle",12),S());a&2&&(a=Y(),Kp("viewBox",a.zu()),D(),lq("stroke-dasharray",a.Nm(),"px")("stroke-dashoffset",a.Nm()/2,"px")("stroke-width",a.qt(),"%"),Kp("r",a.tm()))}var rD=new df("mat-progress-spinner-default-options",{ea:"root",ba:qD});function qD(){return{diameter:100}}
var sD=class{get color(){return this.mh||this.C}set color(a){this.mh=a}constructor(a,b,c){this.ra=a;this.C="primary";this.Lc=0;this.D=100;this.Dp="NoopAnimations"===b&&!!c&&!c.rF;this.mode="mat-spinner"===a.R.nodeName.toLowerCase()?"indeterminate":"determinate";c&&(c.color&&(this.color=this.C=c.color),c.diameter&&(this.diameter=c.diameter),c.strokeWidth&&(this.strokeWidth=c.strokeWidth))}get value(){return"determinate"===this.mode?this.Lc:0}set value(a){this.Lc=Math.max(0,Math.min(100,a||0))}get diameter(){return this.D}set diameter(a){this.D=
a||0}get strokeWidth(){let a;return null!=(a=this.F)?a:this.diameter/10}set strokeWidth(a){this.F=a||0}tm(){return(this.diameter-10)/2}zu(){const a=2*this.tm()+this.strokeWidth;return`0 0 ${a} ${a}`}Nm(){return 2*Math.PI*this.tm()}JA(){return"determinate"===this.mode?this.Nm()*(100-this.Lc)/100:null}qt(){return this.strokeWidth/this.diameter*100}};sD.K=function(a){return new (a||sD)(z(si),z(mi,8),z(rD))};
sD.Da=Gf({type:sD,ha:[["mat-progress-spinner"],["mat-spinner"]],fb:function(a,b){a&1&&ur(oD,5);if(a&2){let c;vr(c=wr())&&(b.pF=c.first)}},Ka:["role","progressbar","tabindex","-1",1,"mat-mdc-progress-spinner","mdc-circular-progress"],Za:18,Ia:function(a,b){a&2&&(Kp("aria-valuemin",0)("aria-valuemax",100)("aria-valuenow","determinate"===b.mode?b.value:null)("mode",b.mode),qq(tq,"mat-"+b.color),lq("width",b.diameter,"px")("height",b.diameter,"px")("--mdc-circular-progress-size",b.diameter+"px")("--mdc-circular-progress-active-indicator-width",
b.diameter+"px"),nq("_mat-animation-noopable",b.Dp)("mdc-circular-progress--indeterminate","indeterminate"===b.mode))},inputs:{color:"color",mode:"mode",value:[2,"value","value",As],diameter:[2,"diameter","diameter",As],strokeWidth:[2,"strokeWidth","strokeWidth",As]},ib:["matProgressSpinner"],ka:!0,features:[Jp,Mr],za:14,Aa:11,Ca:[["circle",""],["determinateSpinner",""],["aria-hidden","true",1,"mdc-circular-progress__determinate-container"],["xmlns","http://www.w3.org/2000/svg","focusable","false",
1,"mdc-circular-progress__determinate-circle-graphic"],["cx","50%","cy","50%",1,"mdc-circular-progress__determinate-circle"],["aria-hidden","true",1,"mdc-circular-progress__indeterminate-container"],[1,"mdc-circular-progress__spinner-layer"],[1,"mdc-circular-progress__circle-clipper","mdc-circular-progress__circle-left"],[3,"ngTemplateOutlet"],[1,"mdc-circular-progress__gap-patch"],[1,"mdc-circular-progress__circle-clipper","mdc-circular-progress__circle-right"],["xmlns","http://www.w3.org/2000/svg",
"focusable","false",1,"mdc-circular-progress__indeterminate-circle-graphic"],["cx","50%","cy","50%"]],ta:function(a,b){a&1&&(Ap(0,pD,2,8,"ng-template",null,0,Qr),R(2,"div",2,1),fh(),R(4,"svg",3),Mq(5,"circle",4),S()(),Rg.Vk=null,R(6,"div",5)(7,"div",6)(8,"div",7),Pq(9,8),S(),R(10,"div",9),Pq(11,8),S(),R(12,"div",10),Pq(13,8),S()()());a&2&&(a=zr(1),D(4),Kp("viewBox",b.zu()),D(),lq("stroke-dasharray",b.Nm(),"px")("stroke-dashoffset",b.JA(),"px")("stroke-width",b.qt(),"%"),Kp("r",b.tm()),D(4),H("ngTemplateOutlet",
a),D(2),H("ngTemplateOutlet",a),D(2),H("ngTemplateOutlet",a))},Ha:[gt],styles:["@keyframes mdc-circular-progress-container-rotate{to{transform:rotate(360deg)}}@keyframes mdc-circular-progress-spinner-layer-rotate{12.5%{transform:rotate(135deg)}25%{transform:rotate(270deg)}37.5%{transform:rotate(405deg)}50%{transform:rotate(540deg)}62.5%{transform:rotate(675deg)}75%{transform:rotate(810deg)}87.5%{transform:rotate(945deg)}100%{transform:rotate(1080deg)}}@keyframes mdc-circular-progress-color-1-fade-in-out{from{opacity:.99}25%{opacity:.99}26%{opacity:0}89%{opacity:0}90%{opacity:.99}to{opacity:.99}}@keyframes mdc-circular-progress-color-2-fade-in-out{from{opacity:0}15%{opacity:0}25%{opacity:.99}50%{opacity:.99}51%{opacity:0}to{opacity:0}}@keyframes mdc-circular-progress-color-3-fade-in-out{from{opacity:0}40%{opacity:0}50%{opacity:.99}75%{opacity:.99}76%{opacity:0}to{opacity:0}}@keyframes mdc-circular-progress-color-4-fade-in-out{from{opacity:0}65%{opacity:0}75%{opacity:.99}90%{opacity:.99}to{opacity:0}}@keyframes mdc-circular-progress-left-spin{from{transform:rotate(265deg)}50%{transform:rotate(130deg)}to{transform:rotate(265deg)}}@keyframes mdc-circular-progress-right-spin{from{transform:rotate(-265deg)}50%{transform:rotate(-130deg)}to{transform:rotate(-265deg)}}.mdc-circular-progress{display:inline-flex;position:relative;direction:ltr;line-height:0;transition:opacity 250ms 0ms cubic-bezier(0.4, 0, 0.6, 1)}.mdc-circular-progress__determinate-container,.mdc-circular-progress__indeterminate-circle-graphic,.mdc-circular-progress__indeterminate-container,.mdc-circular-progress__spinner-layer{position:absolute;width:100%;height:100%}.mdc-circular-progress__determinate-container{transform:rotate(-90deg)}.mdc-circular-progress__indeterminate-container{font-size:0;letter-spacing:0;white-space:nowrap;opacity:0}.mdc-circular-progress__determinate-circle-graphic,.mdc-circular-progress__indeterminate-circle-graphic{fill:rgba(0,0,0,0)}.mdc-circular-progress__determinate-circle{transition:stroke-dashoffset 500ms 0ms cubic-bezier(0, 0, 0.2, 1)}.mdc-circular-progress__gap-patch{position:absolute;top:0;left:47.5%;box-sizing:border-box;width:5%;height:100%;overflow:hidden}.mdc-circular-progress__gap-patch .mdc-circular-progress__indeterminate-circle-graphic{left:-900%;width:2000%;transform:rotate(180deg)}.mdc-circular-progress__circle-clipper{display:inline-flex;position:relative;width:50%;height:100%;overflow:hidden}.mdc-circular-progress__circle-clipper .mdc-circular-progress__indeterminate-circle-graphic{width:200%}.mdc-circular-progress__circle-right .mdc-circular-progress__indeterminate-circle-graphic{left:-100%}.mdc-circular-progress--indeterminate .mdc-circular-progress__determinate-container{opacity:0}.mdc-circular-progress--indeterminate .mdc-circular-progress__indeterminate-container{opacity:1}.mdc-circular-progress--indeterminate .mdc-circular-progress__indeterminate-container{animation:mdc-circular-progress-container-rotate 1568.2352941176ms linear infinite}.mdc-circular-progress--indeterminate .mdc-circular-progress__spinner-layer{animation:mdc-circular-progress-spinner-layer-rotate 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both}.mdc-circular-progress--indeterminate .mdc-circular-progress__color-1{animation:mdc-circular-progress-spinner-layer-rotate 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both,mdc-circular-progress-color-1-fade-in-out 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both}.mdc-circular-progress--indeterminate .mdc-circular-progress__color-2{animation:mdc-circular-progress-spinner-layer-rotate 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both,mdc-circular-progress-color-2-fade-in-out 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both}.mdc-circular-progress--indeterminate .mdc-circular-progress__color-3{animation:mdc-circular-progress-spinner-layer-rotate 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both,mdc-circular-progress-color-3-fade-in-out 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both}.mdc-circular-progress--indeterminate .mdc-circular-progress__color-4{animation:mdc-circular-progress-spinner-layer-rotate 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both,mdc-circular-progress-color-4-fade-in-out 5332ms cubic-bezier(0.4, 0, 0.2, 1) infinite both}.mdc-circular-progress--indeterminate .mdc-circular-progress__circle-left .mdc-circular-progress__indeterminate-circle-graphic{animation:mdc-circular-progress-left-spin 1333ms cubic-bezier(0.4, 0, 0.2, 1) infinite both}.mdc-circular-progress--indeterminate .mdc-circular-progress__circle-right .mdc-circular-progress__indeterminate-circle-graphic{animation:mdc-circular-progress-right-spin 1333ms cubic-bezier(0.4, 0, 0.2, 1) infinite both}.mdc-circular-progress--closed{opacity:0}.mat-mdc-progress-spinner .mdc-circular-progress__determinate-circle,.mat-mdc-progress-spinner .mdc-circular-progress__indeterminate-circle-graphic{stroke:var(--mdc-circular-progress-active-indicator-color)}@media screen and (forced-colors: active),(-ms-high-contrast: active){.mat-mdc-progress-spinner .mdc-circular-progress__determinate-circle,.mat-mdc-progress-spinner .mdc-circular-progress__indeterminate-circle-graphic{stroke:CanvasText}}.mat-mdc-progress-spinner circle{stroke-width:var(--mdc-circular-progress-active-indicator-width)}@media screen and (forced-colors: active),(-ms-high-contrast: active){.mat-mdc-progress-spinner .mdc-circular-progress--four-color .mdc-circular-progress__color-1 .mdc-circular-progress__indeterminate-circle-graphic{stroke:CanvasText}}@media screen and (forced-colors: active),(-ms-high-contrast: active){.mat-mdc-progress-spinner .mdc-circular-progress--four-color .mdc-circular-progress__color-2 .mdc-circular-progress__indeterminate-circle-graphic{stroke:CanvasText}}@media screen and (forced-colors: active),(-ms-high-contrast: active){.mat-mdc-progress-spinner .mdc-circular-progress--four-color .mdc-circular-progress__color-3 .mdc-circular-progress__indeterminate-circle-graphic{stroke:CanvasText}}@media screen and (forced-colors: active),(-ms-high-contrast: active){.mat-mdc-progress-spinner .mdc-circular-progress--four-color .mdc-circular-progress__color-4 .mdc-circular-progress__indeterminate-circle-graphic{stroke:CanvasText}}.mat-mdc-progress-spinner .mdc-circular-progress{width:var(--mdc-circular-progress-size) !important;height:var(--mdc-circular-progress-size) !important}.mat-mdc-progress-spinner{display:block;overflow:hidden;line-height:0}.mat-mdc-progress-spinner._mat-animation-noopable,.mat-mdc-progress-spinner._mat-animation-noopable .mdc-circular-progress__determinate-circle{transition:none}.mat-mdc-progress-spinner._mat-animation-noopable .mdc-circular-progress__indeterminate-circle-graphic,.mat-mdc-progress-spinner._mat-animation-noopable .mdc-circular-progress__spinner-layer,.mat-mdc-progress-spinner._mat-animation-noopable .mdc-circular-progress__indeterminate-container{animation:none}.mat-mdc-progress-spinner._mat-animation-noopable .mdc-circular-progress__indeterminate-container circle{stroke-dasharray:0 !important}.cdk-high-contrast-active .mat-mdc-progress-spinner .mdc-circular-progress__indeterminate-circle-graphic,.cdk-high-contrast-active .mat-mdc-progress-spinner .mdc-circular-progress__determinate-circle{stroke:currentColor;stroke:CanvasText}\n"],
pb:2,Ra:0});var tD=class{};tD.K=function(a){return new (a||tD)};tD.Ya=Of({type:tD});tD.Xa=Nd({imports:[ht,mz]});function uD(a,b,c){b="indeterminate"===c?b:b+" "+lD.format(Number(c)/100);a.D.textContent=b}
var vD=class{constructor(a,b){this.options=a;this.F=b;this.C=new cj;this.D=document.createElement("label");this.D.classList.add("cdk-visually-hidden");this.D.classList.add("gmat-mdc-progress-spinner-accessible-label");C(mD(b.ra.R),Lo(this.C)).subscribe(c=>{uD(this,c[0],c[1])})}Fb(){this.F.ra.R.insertAdjacentElement("afterend",this.D);const a=this.F.ra.R.getAttribute("aria-label");uD(this,null!=a?a:"","indeterminate"===this.F.mode?"indeterminate":this.F.value.toString())}va(){this.D.remove();this.C.next();
this.C.complete()}};vD.K=function(a){return new (a||vD)(z(nD),z(sD,10))};vD.Fa=Qf({type:vD,ha:[["mat-progress-spinner"],["mat-spinner"]],Ka:["aria-hidden","true"],Za:3,Ia:function(a,b){a&2&&(Kp("tabindex",null),nq("gmat-mdc-progress-spinner",!b.options.sc))}});var wD=class{};wD.K=function(a){return new (a||wD)};wD.Ya=Of({type:wD});wD.Xa=Nd({imports:[tD]});var xD=new df("GMAT_SELECT_OPTIONS",{ea:"root",ba:()=>({sc:!1})});var yD=Yy("transformPanel",[az("void",$y({opacity:0,transform:"scale(1, 0.8)"})),bz("void => showing",Zy("120ms cubic-bezier(0, 0, 0.2, 1)",$y({opacity:1,transform:"scale(1, 1)"}))),bz("* => void",Zy("100ms linear",$y({opacity:0})))]);const zD=["trigger"],AD=["panel"],BD=[[["mat-select-trigger"]],"*"];function CD(a){a&1&&(R(0,"span",4),Z(1),S());a&2&&(a=Y(),D(),Ar(a.placeholder))}function DD(a){a&1&&Yq(0)}function ED(a){a&1&&(R(0,"span",11),Z(1),S());a&2&&(a=Y(2),D(),Ar(a.JE))}function FD(a){a&1&&(R(0,"span",5),Ap(1,DD,1,0)(2,ED,2,1,"span",11),S());a&2&&(a=Y(),D(),Eq(a.tB?1:2))}
function GD(a){if(a&1){const b=Ug();R(0,"div",12,1);V("@transformPanel.done",function(c){u(b);const d=Y();return x(d.fu.next(c.me))})("keydown",function(c){u(b);const d=Y();return x(d.Se(c))});Yq(2,1);S()}a&2&&(a=Y(),vq(a.Vz()),H("ngClass",a.hb)("@transformPanel","showing"),Kp("id",a.id+"-panel")("aria-multiselectable",a.multiple)("aria-label",a.ariaLabel||null)("aria-labelledby",a.qp()))}let HD=0;
var ID=new df("mat-select-scroll-strategy",{ea:"root",ba:()=>{const a=lf(bw);return()=>a.C.C()}}),JD=new df("MAT_SELECT_CONFIG"),KD={oa:ID,Ac:[bw],Fc:function(a){return()=>a.C.C()}},LD=new df("MatSelectTrigger"),MD=class{constructor(a,b){this.source=a;this.value=b}};function aB(a,b){b.length?a.ra.R.setAttribute("aria-describedby",b.join(" ")):a.ra.R.removeAttribute("aria-describedby")}function ND(a){return a.ja?"rtl"===a.ja.value:!1}
var OD=class{Pa(a){var b=this.options.toArray()[a];if(b){const c=this.Wf.R,d=Wz(a,this.options,this.Hr);b=b.qe();c.scrollTop=0===a&&1===d?0:Xz(b.offsetTop,b.offsetHeight,c.scrollTop,c.offsetHeight)}}md(){this.Pa(this.C.O||0)}ac(a){return new MD(this,a)}get focused(){return this.Qe||this.F}get Ae(){return this.na}set Ae(a){this.na=a;if(this.options)for(const b of this.options)bn(b.Ba)}get placeholder(){return this.Kb}set placeholder(a){this.Kb=a;this.Tc.next()}get required(){var a;let b;null==(a=this.sd)&&
(null==(a=this.Jd)?a=void 0:null==(a=a.control)?a=void 0:(a=a.We,a=Array.isArray(a)?a.includes(xw):a===xw));return null!=(b=a)?b:!1}set required(a){this.sd=a;this.Tc.next()}get multiple(){return this.G}set multiple(a){this.G=a}get Pk(){return this.aa}set Pk(a){this.aa=a;this.D&&this.wa()}get value(){return this.Lc}set value(a){this.sa(a)&&this.fa(a)}get Nv(){return this.T.lr}set Nv(a){this.T.lr=a}get id(){return this.Xb}set id(a){this.Xb=a||this.ud;this.Tc.next()}get xe(){return this.T.xe}set xe(a){this.T.xe=
a}constructor(a,b,c,d,e,f,g,k,m,p,t,w,B,G){this.Ob=a;this.Ba=b;this.ra=e;this.ja=f;this.P=m;this.Jd=p;this.sb=B;this.N=G;this.yA=[{cb:"start",eb:"bottom",Na:"start",Ta:"top"},{cb:"end",eb:"bottom",Na:"end",Ta:"top"},{cb:"start",eb:"top",Na:"start",Ta:"bottom",hb:"mat-mdc-select-panel-above"},{cb:"end",eb:"top",Na:"end",Ta:"bottom",hb:"mat-mdc-select-panel-above"}];this.F=!1;this.aa=(O,T)=>O===T;this.ud=`mat-select-${HD++}`;this.gb=null;this.J=new cj;this.Tc=new cj;this.zB=!0;this.fa=()=>{};this.Zd=
()=>{};this.yu=`mat-select-value-${HD++}`;this.fu=new cj;let J;this.wA=(null==(J=this.N)?void 0:J.Wh)||"";this.Qe=!1;this.jv="mat-select";this.Nc=this.disabled=!1;this.tabIndex=0;let N,aa;this.na=null!=(aa=null==(N=this.N)?void 0:N.Ae)?aa:!1;this.G=!1;let ca,ia;this.Iv=null!=(ia=null==(ca=this.N)?void 0:ca.Iv)?ia:!1;this.ariaLabel="";this.Gg=this.N&&"undefined"!==typeof this.N.Gg?this.N.Gg:"auto";this.ca=new cj;this.lb=ik(()=>{const O=this.options;return O?C(O.C,Ho(O),Io(()=>rk(...O.map(T=>T.jo)))):
C(this.ca,Io(()=>this.lb))});this.lo=new Fk;this.tA=C(this.lo,tk(O=>O),xj(()=>{}));this.Fz=C(this.lo,tk(O=>!O),xj(()=>{}));this.Gx=new Fk;this.Ay=new Fk;this.O=null;this.Nb=O=>this.qb?!1:O.disabled;this.Jd&&(this.Jd.C=this);null!=(null==G?void 0:G.jm)&&(this.jm=G.jm);this.T=new nz(d,p,k,g,this.Tc);this.Lb=w;this.Kc=this.Lb();this.tabIndex=parseInt(t)||0;this.id=this.id}Fb(){this.D=new vu(this.multiple);this.Tc.next();C(this.fu,bo(),Lo(this.J)).subscribe(()=>this.Yc(this.qb));C(this.Ob.wb(),Lo(this.J)).subscribe(()=>
{this.qb&&(this.eu=this.ua(this.Hp),cn(this.Ba))})}yj(){this.ca.next();this.ca.complete();this.Jc();C(this.D.Ah,Lo(this.J)).subscribe(a=>{a.added.forEach(b=>b.select());a.removed.forEach(b=>Uz(b))});C(this.options.C,Ho(null),Lo(this.J)).subscribe(()=>{this.td();this.wa()})}zj(){var a=this.Dc(),b=this.Jd;if(a!==this.gb){var c=this.ra.R;(this.gb=a)?c.setAttribute("aria-labelledby",a):c.removeAttribute("aria-labelledby")}if(b){this.Ea!==b.control&&(void 0!==this.Ea&&null!==b.disabled&&b.disabled!==this.disabled&&
(this.disabled=b.disabled),this.Ea=b.control);a=this.T;b=a.xe;c=a.G||a.F;const d=a.Jd?a.Jd.control:null;let e;c=null!=(e=null==(a.lr||a.D)?void 0:!!(d&&d.Qn&&(d.Vc||c&&c.Uj)))?e:!1;c!==b&&(a.xe=c,a.C.next())}}tc(a){(a.disabled||a.userAriaDescribedBy)&&this.Tc.next();a.typeaheadDebounceInterval&&this.C&&Oy(this.C,this.jm)}va(){let a;null==(a=this.C)||a.destroy();this.J.next();this.J.complete();this.Tc.complete();this.O&&(uy(this.O,"aria-owns",`${this.id}-panel`),this.O=null)}toggle(){this.qb?this.close():
this.open()}open(){var a;if(!this.F&&!this.disabled&&0<(null==(a=this.options)?void 0:a.length)){this.P&&(this.Hp=$A(this.P));this.eu=this.ua(this.Hp);const b=this.ra.R.closest('body > .cdk-overlay-container [aria-modal="true"]');b&&(a=`${this.id}-panel`,this.O&&uy(this.O,"aria-owns",a),sy(b,"aria-owns",a),this.O=b);this.F=!0;My(this.C,null);this.Ic();bn(this.Ba);this.Tc.next()}}close(){this.F&&(this.F=!1,My(this.C,ND(this)?"rtl":"ltr"),bn(this.Ba),this.Zd(),this.Tc.next())}pi(a){this.sa(a)}Lg(a){this.fa=
a}Ij(a){this.Zd=a}gi(a){this.disabled=a;bn(this.Ba);this.Tc.next()}get qb(){return this.F}get selected(){let a,b;return this.multiple?(null==(a=this.D)?void 0:a.selected)||[]:null==(b=this.D)?void 0:b.selected[0]}get JE(){if(this.empty)return"";if(this.G){const a=this.D.selected.map(b=>b.F);ND(this)&&a.reverse();return a.join(", ")}return this.D.selected[0].F}Se(a){this.disabled||(this.qb?this.Hc(a):this.Gc(a))}Gc(a){var b=a.keyCode;const c=40===b||38===b||37===b||39===b,d=this.C;!(0<d.G.length||
13!==b&&32!==b||Bu(a))||(this.multiple||a.altKey)&&c?(a.preventDefault(),this.open()):this.multiple||(b=this.selected,Uy(d,a),(a=this.selected)&&b!==a&&py(this.sb,a.F,1E4))}Hc(a){const b=this.C;var c=a.keyCode;const d=40===c||38===c,e=0<b.G.length;if(d&&a.altKey)a.preventDefault(),this.close();else if(e||13!==c&&32!==c||!b.C||Bu(a))if(!e&&this.G&&65===c&&a.ctrlKey){a.preventDefault();const f=this.options.some(g=>!g.disabled&&!g.selected);this.options.forEach(g=>{g.disabled||(f?g.select():Uz(g))})}else c=
b.O,Uy(b,a),this.G&&d&&a.shiftKey&&b.C&&b.O!==c&&b.C.uh();else a.preventDefault(),b.C.uh()}Fp(){this.disabled||(this.Qe=!0,this.Tc.next())}xk(){this.Qe=!1;let a;null!=(a=this.C)&&(a.G=[]);this.disabled||this.qb||(this.Zd(),bn(this.Ba),this.Tc.next())}mA(){C(this.vA.Ej,fo()).subscribe(()=>{cn(this.Ba);this.md()})}Vz(){return this.P?`mat-${this.P.color}`:""}get empty(){return!this.D||0===this.D.C.size}wa(){Promise.resolve().then(()=>{this.Jd&&(this.Lc=this.Jd.value);this.Va(this.Lc);this.Tc.next()})}Va(a){this.options.forEach(b=>
{b.D&&(b.D=!1,bn(b.Ba))});this.D.clear();this.multiple&&a?(a.forEach(b=>this.Sa(b)),this.bb()):(a=this.Sa(a))?Ry(this.C,a):this.qb||Ry(this.C,-1);bn(this.Ba)}Sa(a){const b=this.options.find(c=>{if(uu(this.D,c))return!1;try{return null!=c.value&&this.aa(c.value,a)}catch(d){return!1}});b&&this.D.select(b);return b}sa(a){return a!==this.Lc||this.G&&Array.isArray(a)?(this.options&&this.Va(a),this.Lc=a,!0):!1}ua(a){return"auto"===this.Gg?(a instanceof ew?a.fc:a||this.ra).R.getBoundingClientRect().width:
null===this.Gg?"":this.Gg}Jc(){this.C=Jy(Ny(Qy(Py(My(Ly(Oy(new Wy(this.options),this.jm)),ND(this)?"rtl":"ltr")))),this.Nb);this.C.ca.subscribe(()=>{this.qb&&(!this.multiple&&this.C.C&&this.C.C.uh(),this.focus(),this.close())});this.C.wb.subscribe(()=>{this.F&&this.Wf?this.Pa(this.C.O||0):this.F||this.multiple||!this.C.C||this.C.C.uh()})}td(){const a=rk(this.options.C,this.J);C(this.lb,Lo(a)).subscribe(b=>{this.Xc(b.source,b.C);b.C&&!this.multiple&&this.F&&(this.close(),this.focus())});C(rk(...this.options.map(b=>
b.G)),Lo(a)).subscribe(()=>{cn(this.Ba);this.Tc.next()})}Xc(a,b){const c=uu(this.D,a);null!=a.value||this.G?(c!==a.selected&&(a.selected?this.D.select(a):tu(this.D,a)),b&&this.C.D(a),this.multiple&&(this.bb(),b&&this.focus())):(Uz(a),this.D.clear(),null!=this.value&&this.Ja(a.value));c!==uu(this.D,a)&&this.Ja();this.Tc.next()}bb(){if(this.multiple){const a=this.options.toArray();this.D.sort((b,c)=>this.Xx?this.Xx(b,c,a):a.indexOf(b)-a.indexOf(c));this.Tc.next()}}Ja(a){let b;this.multiple?b=this.selected.map(c=>
c.value):b=this.selected?this.selected.value:a;this.Lc=b;this.Ay.emit(b);this.fa(b);this.Gx.emit(this.ac(b));bn(this.Ba)}Ic(){if(this.C)if(this.empty){let a=-1;for(let b=0;b<this.options.length;b++)if(!this.options.get(b).disabled){a=b;break}this.C.D(a)}else this.C.D(this.D.selected[0])}focus(a){this.ra.R.focus(a)}qp(){if(this.ariaLabel)return null;let a;const b=null==(a=this.P)?void 0:a.ph()?a.Ue:null;return this.cd?(b?b+" ":"")+this.cd:b}Tz(){return this.qb&&this.C&&this.C.C?this.C.C.id:null}Dc(){if(this.ariaLabel)return null;
var a;const b=null==(a=this.P)?void 0:a.ph()?a.Ue:null;a=(b?b+" ":"")+this.yu;this.cd&&(a+=" "+this.cd);return a}Yc(a){this.lo.emit(a)}get VD(){return this.qb||!this.empty||this.focused&&!!this.placeholder}};OD.K=function(a){return new (a||OD)(z(wu),z(Sr),z(Qk),z(pz),z(si),z(ou,8),z(sx,8),z(zx,8),z(XA,8),z(Jw,10),yh("tabindex"),z(ID),z(qy),z(JD,8))};
OD.Da=Gf({type:OD,ha:[["mat-select"]],ec:function(a,b,c){a&1&&(tr(c,LD,5),tr(c,Vz,5),tr(c,uz,5));if(a&2){let d;vr(d=wr())&&(b.tB=d.first);vr(d=wr())&&(b.options=d);vr(d=wr())&&(b.Hr=d)}},fb:function(a,b){a&1&&(ur(zD,5),ur(AD,5),ur(hw,5));if(a&2){let c;vr(c=wr())&&(b.Le=c.first);vr(c=wr())&&(b.Wf=c.first);vr(c=wr())&&(b.vA=c.first)}},Ka:["role","combobox","aria-autocomplete","none","aria-haspopup","listbox",1,"mat-mdc-select"],Za:19,Ia:function(a,b){a&1&&V("keydown",function(c){return b.Se(c)})("focus",
function(){return b.Fp()})("blur",function(){return b.xk()});a&2&&(Kp("id",b.id)("tabindex",b.disabled?-1:b.tabIndex)("aria-controls",b.qb?b.id+"-panel":null)("aria-expanded",b.qb)("aria-label",b.ariaLabel||null)("aria-required",b.required.toString())("aria-disabled",b.disabled.toString())("aria-invalid",b.xe)("aria-activedescendant",b.Tz()),nq("mat-mdc-select-disabled",b.disabled)("mat-mdc-select-invalid",b.xe)("mat-mdc-select-required",b.required)("mat-mdc-select-empty",b.empty)("mat-mdc-select-multiple",
b.multiple))},inputs:{Ms:[0,"aria-describedby","userAriaDescribedBy"],hb:"panelClass",disabled:[2,"disabled","disabled",zs],Nc:[2,"disableRipple","disableRipple",zs],tabIndex:[2,"tabIndex","tabIndex",a=>null==a?0:As(a)],Ae:[2,"hideSingleSelectionIndicator","hideSingleSelectionIndicator",zs],placeholder:"placeholder",required:[2,"required","required",zs],multiple:[2,"multiple","multiple",zs],Iv:[2,"disableOptionCentering","disableOptionCentering",zs],Pk:"compareWith",value:"value",ariaLabel:[0,"aria-label",
"ariaLabel"],cd:[0,"aria-labelledby","ariaLabelledby"],Nv:"errorStateMatcher",jm:[2,"typeaheadDebounceInterval","typeaheadDebounceInterval",As],Xx:"sortComparator",id:"id",Gg:"panelWidth"},outputs:{lo:"openedChange",tA:"opened",Fz:"closed",Gx:"selectionChange",Ay:"valueChange"},ib:["matSelect"],ka:!0,features:[Kr([{oa:uA,Ab:OD},{oa:rz,Ab:OD}]),Jp,Dg,Mr],Zb:["mat-select-trigger","*"],za:11,Aa:8,Ca:[["fallbackOverlayOrigin","cdkOverlayOrigin","trigger",""],["panel",""],["cdk-overlay-origin","",1,"mat-mdc-select-trigger",
3,"click"],[1,"mat-mdc-select-value"],[1,"mat-mdc-select-placeholder","mat-mdc-select-min-line"],[1,"mat-mdc-select-value-text"],[1,"mat-mdc-select-arrow-wrapper"],[1,"mat-mdc-select-arrow"],"viewBox;0 0 24 24;width;24px;height;24px;focusable;false;aria-hidden;true".split(";"),["d","M7 10l5 5 5-5z"],["cdk-connected-overlay","","cdkConnectedOverlayLockPosition","","cdkConnectedOverlayHasBackdrop","","cdkConnectedOverlayBackdropClass","cdk-overlay-transparent-backdrop",3,"backdropClick","attach","detach",
"cdkConnectedOverlayPanelClass","cdkConnectedOverlayScrollStrategy","cdkConnectedOverlayOrigin","cdkConnectedOverlayOpen","cdkConnectedOverlayPositions","cdkConnectedOverlayWidth"],[1,"mat-mdc-select-min-line"],["role","listbox","tabindex","-1",3,"keydown","ngClass"]],ta:function(a,b){if(a&1){const c=Ug();Xq(BD);R(0,"div",2,0);V("click",function(){u(c);return x(b.open())});R(3,"div",3);Ap(4,CD,2,1,"span",4)(5,FD,3,1,"span",5);S();R(6,"div",6)(7,"div",7);fh();R(8,"svg",8);Mq(9,"path",9);S()()()();
Ap(10,GD,3,9,"ng-template",10);V("backdropClick",function(){u(c);return x(b.close())})("attach",function(){u(c);return x(b.mA())})("detach",function(){u(c);return x(b.close())})}a&2&&(a=zr(1),D(3),Kp("id",b.yu),D(),Eq(b.empty?4:5),D(6),H("cdkConnectedOverlayPanelClass",b.wA)("cdkConnectedOverlayScrollStrategy",b.Kc)("cdkConnectedOverlayOrigin",b.Hp||a)("cdkConnectedOverlayOpen",b.qb)("cdkConnectedOverlayPositions",b.yA)("cdkConnectedOverlayWidth",b.eu))},Ha:[ew,hw,Ws],styles:['.mat-mdc-select{display:inline-block;width:100%;outline:none;-moz-osx-font-smoothing:grayscale;-webkit-font-smoothing:antialiased;color:var(--mat-select-enabled-trigger-text-color);font-family:var(--mat-select-trigger-text-font);line-height:var(--mat-select-trigger-text-line-height);font-size:var(--mat-select-trigger-text-size);font-weight:var(--mat-select-trigger-text-weight);letter-spacing:var(--mat-select-trigger-text-tracking)}div.mat-mdc-select-panel{box-shadow:var(--mat-select-container-elevation-shadow)}.mat-mdc-select-disabled{color:var(--mat-select-disabled-trigger-text-color)}.mat-mdc-select-trigger{display:inline-flex;align-items:center;cursor:pointer;position:relative;box-sizing:border-box;width:100%}.mat-mdc-select-disabled .mat-mdc-select-trigger{-webkit-user-select:none;user-select:none;cursor:default}.mat-mdc-select-value{width:100%;overflow:hidden;text-overflow:ellipsis;white-space:nowrap}.mat-mdc-select-value-text{white-space:nowrap;overflow:hidden;text-overflow:ellipsis}.mat-mdc-select-arrow-wrapper{height:24px;flex-shrink:0;display:inline-flex;align-items:center}.mat-form-field-appearance-fill .mdc-text-field--no-label .mat-mdc-select-arrow-wrapper{transform:none}.mat-mdc-form-field .mat-mdc-select.mat-mdc-select-invalid .mat-mdc-select-arrow,.mat-form-field-invalid:not(.mat-form-field-disabled) .mat-mdc-form-field-infix::after{color:var(--mat-select-invalid-arrow-color)}.mat-mdc-select-arrow{width:10px;height:5px;position:relative;color:var(--mat-select-enabled-arrow-color)}.mat-mdc-form-field.mat-focused .mat-mdc-select-arrow{color:var(--mat-select-focused-arrow-color)}.mat-mdc-form-field .mat-mdc-select.mat-mdc-select-disabled .mat-mdc-select-arrow{color:var(--mat-select-disabled-arrow-color)}.mat-mdc-select-arrow svg{fill:currentColor;position:absolute;top:50%;left:50%;transform:translate(-50%, -50%)}.cdk-high-contrast-active .mat-mdc-select-arrow svg{fill:CanvasText}.mat-mdc-select-disabled .cdk-high-contrast-active .mat-mdc-select-arrow svg{fill:GrayText}div.mat-mdc-select-panel{width:100%;max-height:275px;outline:0;overflow:auto;padding:8px 0;border-radius:4px;box-sizing:border-box;position:static;background-color:var(--mat-select-panel-background-color)}.cdk-high-contrast-active div.mat-mdc-select-panel{outline:solid 1px}.cdk-overlay-pane:not(.mat-mdc-select-panel-above) div.mat-mdc-select-panel{border-top-left-radius:0;border-top-right-radius:0;transform-origin:top center}.mat-mdc-select-panel-above div.mat-mdc-select-panel{border-bottom-left-radius:0;border-bottom-right-radius:0;transform-origin:bottom center}div.mat-mdc-select-panel .mat-mdc-option{--mdc-list-list-item-container-color: var(--mat-select-panel-background-color)}.mat-mdc-select-placeholder{transition:color 400ms 133.3333333333ms cubic-bezier(0.25, 0.8, 0.25, 1);color:var(--mat-select-placeholder-text-color)}._mat-animation-noopable .mat-mdc-select-placeholder{transition:none}.mat-form-field-hide-placeholder .mat-mdc-select-placeholder{color:rgba(0,0,0,0);-webkit-text-fill-color:rgba(0,0,0,0);transition:none;display:block}.mat-mdc-form-field-type-mat-select:not(.mat-form-field-disabled) .mat-mdc-text-field-wrapper{cursor:pointer}.mat-mdc-form-field-type-mat-select.mat-form-field-appearance-fill .mat-mdc-floating-label{max-width:calc(100% - 18px)}.mat-mdc-form-field-type-mat-select.mat-form-field-appearance-fill .mdc-floating-label--float-above{max-width:calc(100%/0.75 - 24px)}.mat-mdc-form-field-type-mat-select.mat-form-field-appearance-outline .mdc-notched-outline__notch{max-width:calc(100% - 60px)}.mat-mdc-form-field-type-mat-select.mat-form-field-appearance-outline .mdc-text-field--label-floating .mdc-notched-outline__notch{max-width:calc(100% - 24px)}.mat-mdc-select-min-line:empty::before{content:" ";white-space:pre;width:1px;display:inline-block;visibility:hidden}.mat-form-field-appearance-fill .mat-mdc-select-arrow-wrapper{transform:var(--mat-select-arrow-transform)}\n'],
pb:2,data:{animation:[yD]},Ra:0});var PD=class{};PD.K=function(a){return new (a||PD)};PD.Ya=Of({type:PD});PD.Xa=Nd({Wa:[KD],imports:[ht,jw,$z,mz,xu,cB,$z,mz]});var QD={oa:JD,Ac:[[new Rh,new Sh,new Qh(JD)],[new Rh,new Qh(xD)]],Fc:(a,b)=>!b||b.sc?a||{}:a?Object.assign({},a,{Wh:Ps(a.Wh).concat("gmat-mdc-select")}):{Wh:"gmat-mdc-select"}},RD=class{};RD.K=function(a){return new (a||RD)};RD.Ya=Of({type:RD});RD.Xa=Nd({Wa:[QD],imports:[PD,AC]});var SD=new df("GMAT_SLIDE_TOGGLE_OPTIONS",{ea:"root",ba:()=>({sc:!1})});var TD=class{constructor(a){this.options=a}};TD.K=function(a){return new (a||TD)(z(SD))};TD.Fa=Qf({type:TD,ha:[["mat-slide-toggle"]],Za:2,Ia:function(a,b){a&2&&nq("gmat-mdc-slide-toggle",!b.options.sc)}});var UD=new df("mat-slide-toggle-default-options",{ea:"root",ba:()=>({DB:!1,Jn:!1})});const VD=["switch"];function WD(a){a&1&&(R(0,"div",10),fh(),R(1,"svg",12),Mq(2,"path",13),S(),R(3,"svg",14),Mq(4,"path",15),S()())}var YD={oa:pw,Ab:af(()=>XD),Eb:!0},ZD=class{constructor(a,b){this.source=a;this.checked=b}};let $D=0;
var XD=class{kp(a){return new ZD(this,a)}get Ou(){return`${this.id||this.Dk}-button`}focus(){this.KA.R.focus()}get checked(){return this.yf}set checked(a){this.yf=a;bn(this.Ba)}get Yq(){return`${this.id||this.Dk}-input`}constructor(a,b,c,d,e,f){this.ra=a;this.D=b;this.Ba=c;this.defaults=e;this.C=()=>{};this.Zd=()=>{};this.F=()=>{};this.yf=!1;this.name=null;this.Lh="after";this.cd=this.ariaLabel=null;this.Nc=this.disabled=!1;this.tabIndex=0;this.wb=new Fk;this.ny=new Fk;this.tabIndex=parseInt(d)||
0;this.color=e.color||"accent";this.Dp="NoopAnimations"===f;this.id=this.Dk=`mat-mdc-slide-toggle-${++$D}`;let g;this.Jn=null!=(g=e.Jn)?g:!1;this.Ue=this.Dk+"-label"}yj(){Lx(this.D,this.ra,!0).subscribe(a=>{"keyboard"===a||"program"===a?(this.Qe=!0,bn(this.Ba)):a||Promise.resolve().then(()=>{this.Qe=!1;this.Zd();bn(this.Ba)})})}tc(a){a.required&&this.F()}va(){Mx(this.D,this.ra)}pi(a){this.checked=!!a}Lg(a){this.C=a}Ij(a){this.Zd=a}validate(a){return this.required&&!0!==a.value?{required:!0}:null}Xr(a){this.F=
a}gi(a){this.disabled=a;bn(this.Ba)}toggle(){this.checked=!this.checked;this.C(this.checked)}Gi(){this.C(this.checked);this.wb.emit(this.kp(this.checked))}pk(){this.ny.emit();this.defaults.DB||(this.checked=!this.checked,this.C(this.checked),this.wb.emit(new ZD(this,this.checked)))}Uz(){return this.cd?this.cd:this.ariaLabel?null:this.Ue}};XD.K=function(a){return new (a||XD)(z(si),z(Ox),z(Sr),yh("tabindex"),z(UD),z(mi,8))};
XD.Da=Gf({type:XD,ha:[["mat-slide-toggle"]],fb:function(a,b){a&1&&ur(VD,5);if(a&2){let c;vr(c=wr())&&(b.KA=c.first)}},Ka:[1,"mat-mdc-slide-toggle"],Za:13,Ia:function(a,b){a&2&&(Qq("id",b.id),Kp("tabindex",null)("aria-label",null)("name",null)("aria-labelledby",null),qq(tq,b.color?"mat-"+b.color:""),nq("mat-mdc-slide-toggle-focused",b.Qe)("mat-mdc-slide-toggle-checked",b.checked)("_mat-animation-noopable",b.Dp))},inputs:{name:"name",id:"id",Lh:"labelPosition",ariaLabel:[0,"aria-label","ariaLabel"],
cd:[0,"aria-labelledby","ariaLabelledby"],Hk:[0,"aria-describedby","ariaDescribedby"],required:[2,"required","required",zs],color:"color",disabled:[2,"disabled","disabled",zs],Nc:[2,"disableRipple","disableRipple",zs],tabIndex:[2,"tabIndex","tabIndex",a=>null==a?0:As(a)],checked:[2,"checked","checked",zs],Jn:[2,"hideIcon","hideIcon",zs]},outputs:{wb:"change",ny:"toggleChange"},ib:["matSlideToggle"],ka:!0,features:[Kr([YD,{oa:vw,Ab:XD,Eb:!0}]),Jp,Dg,Mr],Zb:["*"],za:13,Aa:24,Ca:[["switch",""],["mat-internal-form-field",
"",3,"labelPosition"],["role","switch","type","button",1,"mdc-switch",3,"click","tabIndex","disabled"],[1,"mdc-switch__track"],[1,"mdc-switch__handle-track"],[1,"mdc-switch__handle"],[1,"mdc-switch__shadow"],[1,"mdc-elevation-overlay"],[1,"mdc-switch__ripple"],["mat-ripple","",1,"mat-mdc-slide-toggle-ripple","mat-mdc-focus-indicator",3,"matRippleTrigger","matRippleDisabled","matRippleCentered"],[1,"mdc-switch__icons"],[1,"mdc-label",3,"click","for"],["viewBox","0 0 24 24","aria-hidden","true",1,"mdc-switch__icon",
"mdc-switch__icon--on"],["d","M19.69,5.23L8.96,15.96l-4.23-4.23L2.96,13.5l6,6L21.46,7L19.69,5.23z"],["viewBox","0 0 24 24","aria-hidden","true",1,"mdc-switch__icon","mdc-switch__icon--off"],["d","M20 13H4v-2h16v2z"]],ta:function(a,b){if(a&1){const c=Ug();Xq();R(0,"div",1)(1,"button",2,0);V("click",function(){u(c);return x(b.pk())});Mq(3,"div",3);R(4,"div",4)(5,"div",5)(6,"div",6);Mq(7,"div",7);S();R(8,"div",8);Mq(9,"div",9);S();Ap(10,WD,5,0,"div",10);S()()();R(11,"label",11);V("click",function(d){u(c);
return x(d.stopPropagation())});Yq(12);S()()}a&2&&(a=zr(2),H("labelPosition",b.Lh),D(),nq("mdc-switch--selected",b.checked)("mdc-switch--unselected",!b.checked)("mdc-switch--checked",b.checked)("mdc-switch--disabled",b.disabled),H("tabIndex",b.disabled?-1:b.tabIndex)("disabled",b.disabled),Kp("id",b.Ou)("name",b.name)("aria-label",b.ariaLabel)("aria-labelledby",b.Uz())("aria-describedby",b.Hk)("aria-required",b.required||null)("aria-checked",b.checked),D(8),H("matRippleTrigger",a)("matRippleDisabled",
b.Nc||b.disabled)("matRippleCentered",!0),D(),Eq(b.Jn?-1:10),D(),H("for",b.Ou),Kp("id",b.Ue))},Ha:[Lz,qz],styles:['.mdc-switch{align-items:center;background:none;border:none;cursor:pointer;display:inline-flex;flex-shrink:0;margin:0;outline:none;overflow:visible;padding:0;position:relative}.mdc-switch[hidden]{display:none}.mdc-switch:disabled{cursor:default;pointer-events:none}.mdc-switch__track{overflow:hidden;position:relative;width:100%}.mdc-switch__track::before,.mdc-switch__track::after{border:1px solid rgba(0,0,0,0);border-radius:inherit;box-sizing:border-box;content:"";height:100%;left:0;position:absolute;width:100%}@media screen and (forced-colors: active){.mdc-switch__track::before,.mdc-switch__track::after{border-color:currentColor}}.mdc-switch__track::before{transition:transform 75ms 0ms cubic-bezier(0, 0, 0.2, 1);transform:translateX(0)}.mdc-switch__track::after{transition:transform 75ms 0ms cubic-bezier(0.4, 0, 0.6, 1);transform:translateX(-100%)}[dir=rtl] .mdc-switch__track::after,.mdc-switch__track[dir=rtl]::after{transform:translateX(100%)}.mdc-switch--selected .mdc-switch__track::before{transition:transform 75ms 0ms cubic-bezier(0.4, 0, 0.6, 1);transform:translateX(100%)}[dir=rtl] .mdc-switch--selected .mdc-switch__track::before,.mdc-switch--selected .mdc-switch__track[dir=rtl]::before{transform:translateX(-100%)}.mdc-switch--selected .mdc-switch__track::after{transition:transform 75ms 0ms cubic-bezier(0, 0, 0.2, 1);transform:translateX(0)}.mdc-switch__handle-track{height:100%;pointer-events:none;position:absolute;top:0;transition:transform 75ms 0ms cubic-bezier(0.4, 0, 0.2, 1);left:0;right:auto;transform:translateX(0)}[dir=rtl] .mdc-switch__handle-track,.mdc-switch__handle-track[dir=rtl]{left:auto;right:0}.mdc-switch--selected .mdc-switch__handle-track{transform:translateX(100%)}[dir=rtl] .mdc-switch--selected .mdc-switch__handle-track,.mdc-switch--selected .mdc-switch__handle-track[dir=rtl]{transform:translateX(-100%)}.mdc-switch__handle{display:flex;pointer-events:auto;position:absolute;top:50%;transform:translateY(-50%);left:0;right:auto}[dir=rtl] .mdc-switch__handle,.mdc-switch__handle[dir=rtl]{left:auto;right:0}.mdc-switch__handle::before,.mdc-switch__handle::after{border:1px solid rgba(0,0,0,0);border-radius:inherit;box-sizing:border-box;content:"";width:100%;height:100%;left:0;position:absolute;top:0;transition:background-color 75ms 0ms cubic-bezier(0.4, 0, 0.2, 1),border-color 75ms 0ms cubic-bezier(0.4, 0, 0.2, 1);z-index:-1}@media screen and (forced-colors: active){.mdc-switch__handle::before,.mdc-switch__handle::after{border-color:currentColor}}.mdc-switch__shadow{border-radius:inherit;bottom:0;left:0;position:absolute;right:0;top:0}.mdc-elevation-overlay{bottom:0;left:0;right:0;top:0}.mdc-switch__ripple{left:50%;position:absolute;top:50%;transform:translate(-50%, -50%);z-index:-1}.mdc-switch:disabled .mdc-switch__ripple{display:none}.mdc-switch__icons{height:100%;position:relative;width:100%;z-index:1}.mdc-switch__icon{bottom:0;left:0;margin:auto;position:absolute;right:0;top:0;opacity:0;transition:opacity 30ms 0ms cubic-bezier(0.4, 0, 1, 1)}.mdc-switch--selected .mdc-switch__icon--on,.mdc-switch--unselected .mdc-switch__icon--off{opacity:1;transition:opacity 45ms 30ms cubic-bezier(0, 0, 0.2, 1)}.mat-mdc-slide-toggle .mdc-switch--disabled+label{color:var(--mdc-switch-disabled-label-text-color)}.mdc-switch{width:var(--mdc-switch-track-width)}.mdc-switch.mdc-switch--selected:enabled .mdc-switch__handle::after{background:var(--mdc-switch-selected-handle-color)}.mdc-switch.mdc-switch--selected:enabled:hover:not(:focus):not(:active) .mdc-switch__handle::after{background:var(--mdc-switch-selected-hover-handle-color)}.mdc-switch.mdc-switch--selected:enabled:focus:not(:active) .mdc-switch__handle::after{background:var(--mdc-switch-selected-focus-handle-color)}.mdc-switch.mdc-switch--selected:enabled:active .mdc-switch__handle::after{background:var(--mdc-switch-selected-pressed-handle-color)}.mdc-switch.mdc-switch--selected:disabled .mdc-switch__handle::after{background:var(--mdc-switch-disabled-selected-handle-color)}.mdc-switch.mdc-switch--unselected:enabled .mdc-switch__handle::after{background:var(--mdc-switch-unselected-handle-color)}.mdc-switch.mdc-switch--unselected:enabled:hover:not(:focus):not(:active) .mdc-switch__handle::after{background:var(--mdc-switch-unselected-hover-handle-color)}.mdc-switch.mdc-switch--unselected:enabled:focus:not(:active) .mdc-switch__handle::after{background:var(--mdc-switch-unselected-focus-handle-color)}.mdc-switch.mdc-switch--unselected:enabled:active .mdc-switch__handle::after{background:var(--mdc-switch-unselected-pressed-handle-color)}.mdc-switch.mdc-switch--unselected:disabled .mdc-switch__handle::after{background:var(--mdc-switch-disabled-unselected-handle-color)}.mdc-switch .mdc-switch__handle::before{background:var(--mdc-switch-handle-surface-color)}.mdc-switch:enabled .mdc-switch__shadow{box-shadow:var(--mdc-switch-handle-elevation)}.mdc-switch:disabled .mdc-switch__shadow{box-shadow:var(--mdc-switch-disabled-handle-elevation)}.mdc-switch .mdc-switch__focus-ring-wrapper,.mdc-switch .mdc-switch__handle{height:var(--mdc-switch-handle-height)}.mdc-switch .mdc-switch__handle{border-radius:var(--mdc-switch-handle-shape)}.mdc-switch .mdc-switch__handle{width:var(--mdc-switch-handle-width)}.mdc-switch .mdc-switch__handle-track{width:calc(100% - var(--mdc-switch-handle-width))}.mdc-switch.mdc-switch--selected:enabled .mdc-switch__icon{fill:var(--mdc-switch-selected-icon-color)}.mdc-switch.mdc-switch--selected:disabled .mdc-switch__icon{fill:var(--mdc-switch-disabled-selected-icon-color)}.mdc-switch.mdc-switch--unselected:enabled .mdc-switch__icon{fill:var(--mdc-switch-unselected-icon-color)}.mdc-switch.mdc-switch--unselected:disabled .mdc-switch__icon{fill:var(--mdc-switch-disabled-unselected-icon-color)}.mdc-switch.mdc-switch--selected:disabled .mdc-switch__icons{opacity:var(--mdc-switch-disabled-selected-icon-opacity)}.mdc-switch.mdc-switch--unselected:disabled .mdc-switch__icons{opacity:var(--mdc-switch-disabled-unselected-icon-opacity)}.mdc-switch.mdc-switch--selected .mdc-switch__icon{width:var(--mdc-switch-selected-icon-size);height:var(--mdc-switch-selected-icon-size)}.mdc-switch.mdc-switch--unselected .mdc-switch__icon{width:var(--mdc-switch-unselected-icon-size);height:var(--mdc-switch-unselected-icon-size)}.mdc-switch.mdc-switch--selected:enabled:hover:not(:focus) .mdc-switch__ripple::before,.mdc-switch.mdc-switch--selected:enabled:hover:not(:focus) .mdc-switch__ripple::after{background-color:var(--mdc-switch-selected-hover-state-layer-color)}.mdc-switch.mdc-switch--selected:enabled:focus .mdc-switch__ripple::before,.mdc-switch.mdc-switch--selected:enabled:focus .mdc-switch__ripple::after{background-color:var(--mdc-switch-selected-focus-state-layer-color)}.mdc-switch.mdc-switch--selected:enabled:active .mdc-switch__ripple::before,.mdc-switch.mdc-switch--selected:enabled:active .mdc-switch__ripple::after{background-color:var(--mdc-switch-selected-pressed-state-layer-color)}.mdc-switch.mdc-switch--unselected:enabled:hover:not(:focus) .mdc-switch__ripple::before,.mdc-switch.mdc-switch--unselected:enabled:hover:not(:focus) .mdc-switch__ripple::after{background-color:var(--mdc-switch-unselected-hover-state-layer-color)}.mdc-switch.mdc-switch--unselected:enabled:focus .mdc-switch__ripple::before,.mdc-switch.mdc-switch--unselected:enabled:focus .mdc-switch__ripple::after{background-color:var(--mdc-switch-unselected-focus-state-layer-color)}.mdc-switch.mdc-switch--unselected:enabled:active .mdc-switch__ripple::before,.mdc-switch.mdc-switch--unselected:enabled:active .mdc-switch__ripple::after{background-color:var(--mdc-switch-unselected-pressed-state-layer-color)}.mdc-switch.mdc-switch--selected:enabled:hover:not(:focus):hover .mdc-switch__ripple::before,.mdc-switch.mdc-switch--selected:enabled:hover:not(:focus).mdc-ripple-surface--hover .mdc-switch__ripple::before{opacity:var(--mdc-switch-selected-hover-state-layer-opacity)}.mdc-switch.mdc-switch--selected:enabled:focus.mdc-ripple-upgraded--background-focused .mdc-switch__ripple::before,.mdc-switch.mdc-switch--selected:enabled:focus:not(.mdc-ripple-upgraded):focus .mdc-switch__ripple::before{transition-duration:75ms;opacity:var(--mdc-switch-selected-focus-state-layer-opacity)}.mdc-switch.mdc-switch--selected:enabled:active:not(.mdc-ripple-upgraded) .mdc-switch__ripple::after{transition:opacity 150ms linear}.mdc-switch.mdc-switch--selected:enabled:active:not(.mdc-ripple-upgraded):active .mdc-switch__ripple::after{transition-duration:75ms;opacity:var(--mdc-switch-selected-pressed-state-layer-opacity)}.mdc-switch.mdc-switch--selected:enabled:active.mdc-ripple-upgraded{--mdc-ripple-fg-opacity:var(--mdc-switch-selected-pressed-state-layer-opacity)}.mdc-switch.mdc-switch--unselected:enabled:hover:not(:focus):hover .mdc-switch__ripple::before,.mdc-switch.mdc-switch--unselected:enabled:hover:not(:focus).mdc-ripple-surface--hover .mdc-switch__ripple::before{opacity:var(--mdc-switch-unselected-hover-state-layer-opacity)}.mdc-switch.mdc-switch--unselected:enabled:focus.mdc-ripple-upgraded--background-focused .mdc-switch__ripple::before,.mdc-switch.mdc-switch--unselected:enabled:focus:not(.mdc-ripple-upgraded):focus .mdc-switch__ripple::before{transition-duration:75ms;opacity:var(--mdc-switch-unselected-focus-state-layer-opacity)}.mdc-switch.mdc-switch--unselected:enabled:active:not(.mdc-ripple-upgraded) .mdc-switch__ripple::after{transition:opacity 150ms linear}.mdc-switch.mdc-switch--unselected:enabled:active:not(.mdc-ripple-upgraded):active .mdc-switch__ripple::after{transition-duration:75ms;opacity:var(--mdc-switch-unselected-pressed-state-layer-opacity)}.mdc-switch.mdc-switch--unselected:enabled:active.mdc-ripple-upgraded{--mdc-ripple-fg-opacity:var(--mdc-switch-unselected-pressed-state-layer-opacity)}.mdc-switch .mdc-switch__ripple{height:var(--mdc-switch-state-layer-size);width:var(--mdc-switch-state-layer-size)}.mdc-switch .mdc-switch__track{height:var(--mdc-switch-track-height)}.mdc-switch:disabled .mdc-switch__track{opacity:var(--mdc-switch-disabled-track-opacity)}.mdc-switch:enabled .mdc-switch__track::after{background:var(--mdc-switch-selected-track-color)}.mdc-switch:enabled:hover:not(:focus):not(:active) .mdc-switch__track::after{background:var(--mdc-switch-selected-hover-track-color)}.mdc-switch:enabled:focus:not(:active) .mdc-switch__track::after{background:var(--mdc-switch-selected-focus-track-color)}.mdc-switch:enabled:active .mdc-switch__track::after{background:var(--mdc-switch-selected-pressed-track-color)}.mdc-switch:disabled .mdc-switch__track::after{background:var(--mdc-switch-disabled-selected-track-color)}.mdc-switch:enabled .mdc-switch__track::before{background:var(--mdc-switch-unselected-track-color)}.mdc-switch:enabled:hover:not(:focus):not(:active) .mdc-switch__track::before{background:var(--mdc-switch-unselected-hover-track-color)}.mdc-switch:enabled:focus:not(:active) .mdc-switch__track::before{background:var(--mdc-switch-unselected-focus-track-color)}.mdc-switch:enabled:active .mdc-switch__track::before{background:var(--mdc-switch-unselected-pressed-track-color)}.mdc-switch:disabled .mdc-switch__track::before{background:var(--mdc-switch-disabled-unselected-track-color)}.mdc-switch .mdc-switch__track{border-radius:var(--mdc-switch-track-shape)}.mdc-switch:enabled .mdc-switch__shadow{box-shadow:var(--mdc-switch-handle-elevation-shadow)}.mdc-switch:disabled .mdc-switch__shadow{box-shadow:var(--mdc-switch-disabled-handle-elevation-shadow)}.mat-mdc-slide-toggle{display:inline-block;-webkit-tap-highlight-color:rgba(0,0,0,0);outline:0}.mat-mdc-slide-toggle .mat-mdc-slide-toggle-ripple,.mat-mdc-slide-toggle .mdc-switch__ripple::after{top:0;left:0;right:0;bottom:0;position:absolute;border-radius:50%;pointer-events:none}.mat-mdc-slide-toggle .mat-mdc-slide-toggle-ripple:not(:empty),.mat-mdc-slide-toggle .mdc-switch__ripple::after:not(:empty){transform:translateZ(0)}.mat-mdc-slide-toggle .mdc-switch__ripple::after{content:"";opacity:0}.mat-mdc-slide-toggle .mdc-switch:hover .mdc-switch__ripple::after{opacity:.04;transition:opacity 75ms 0ms cubic-bezier(0, 0, 0.2, 1)}.mat-mdc-slide-toggle.mat-mdc-slide-toggle-focused .mdc-switch .mdc-switch__ripple::after{opacity:.12}.mat-mdc-slide-toggle.mat-mdc-slide-toggle-focused .mat-mdc-focus-indicator::before{content:""}.mat-mdc-slide-toggle .mat-ripple-element{opacity:.12}.mat-mdc-slide-toggle .mat-mdc-focus-indicator::before{border-radius:50%}.mat-mdc-slide-toggle._mat-animation-noopable .mdc-switch__handle-track,.mat-mdc-slide-toggle._mat-animation-noopable .mdc-elevation-overlay,.mat-mdc-slide-toggle._mat-animation-noopable .mdc-switch__icon,.mat-mdc-slide-toggle._mat-animation-noopable .mdc-switch__handle::before,.mat-mdc-slide-toggle._mat-animation-noopable .mdc-switch__handle::after,.mat-mdc-slide-toggle._mat-animation-noopable .mdc-switch__track::before,.mat-mdc-slide-toggle._mat-animation-noopable .mdc-switch__track::after{transition:none}.mat-mdc-slide-toggle .mdc-switch:enabled+.mdc-label{cursor:pointer}.mdc-switch__handle{transition:width 75ms cubic-bezier(0.4, 0, 0.2, 1),height 75ms cubic-bezier(0.4, 0, 0.2, 1),margin 75ms cubic-bezier(0.4, 0, 0.2, 1)}.mdc-switch--selected .mdc-switch__track::before{opacity:var(--mat-switch-hidden-track-opacity);transition:var(--mat-switch-hidden-track-transition)}.mdc-switch--selected .mdc-switch__track::after{opacity:var(--mat-switch-visible-track-opacity);transition:var(--mat-switch-visible-track-transition)}.mdc-switch--unselected .mdc-switch__track::before{opacity:var(--mat-switch-visible-track-opacity);transition:var(--mat-switch-visible-track-transition)}.mdc-switch--unselected .mdc-switch__track::after{opacity:var(--mat-switch-hidden-track-opacity);transition:var(--mat-switch-hidden-track-transition)}.mat-mdc-slide-toggle .mdc-switch--unselected .mdc-switch__handle{width:var(--mat-switch-unselected-handle-size);height:var(--mat-switch-unselected-handle-size)}.mat-mdc-slide-toggle .mdc-switch--selected .mdc-switch__handle{width:var(--mat-switch-selected-handle-size);height:var(--mat-switch-selected-handle-size)}.mat-mdc-slide-toggle .mdc-switch__handle:has(.mdc-switch__icons){width:var(--mat-switch-with-icon-handle-size);height:var(--mat-switch-with-icon-handle-size)}.mat-mdc-slide-toggle:active .mdc-switch:not(.mdc-switch--disabled) .mdc-switch__handle{width:var(--mat-switch-pressed-handle-size);height:var(--mat-switch-pressed-handle-size)}.mat-mdc-slide-toggle .mdc-switch--selected .mdc-switch__handle{margin:var(--mat-switch-selected-handle-horizontal-margin)}.mat-mdc-slide-toggle .mdc-switch--selected .mdc-switch__handle:has(.mdc-switch__icons){margin:var(--mat-switch-selected-with-icon-handle-horizontal-margin)}.mat-mdc-slide-toggle .mdc-switch--unselected .mdc-switch__handle{margin:var(--mat-switch-unselected-handle-horizontal-margin)}.mat-mdc-slide-toggle .mdc-switch--unselected .mdc-switch__handle:has(.mdc-switch__icons){margin:var(--mat-switch-unselected-with-icon-handle-horizontal-margin)}.mat-mdc-slide-toggle:active .mdc-switch--selected:not(.mdc-switch--disabled) .mdc-switch__handle{margin:var(--mat-switch-selected-pressed-handle-horizontal-margin)}.mat-mdc-slide-toggle:active .mdc-switch--unselected:not(.mdc-switch--disabled) .mdc-switch__handle{margin:var(--mat-switch-unselected-pressed-handle-horizontal-margin)}.mdc-switch__track::after,.mdc-switch__track::before{border-width:var(--mat-switch-track-outline-width);border-color:var(--mat-switch-track-outline-color)}.mdc-switch--selected .mdc-switch__track::after,.mdc-switch--selected .mdc-switch__track::before{border-width:var(--mat-switch-selected-track-outline-width);border-color:var(--mat-switch-selected-track-outline-color)}.mdc-switch--disabled .mdc-switch__track::after,.mdc-switch--disabled .mdc-switch__track::before{border-width:var(--mat-switch-disabled-unselected-track-outline-width);border-color:var(--mat-switch-disabled-unselected-track-outline-color)}.mdc-switch--disabled.mdc-switch--selected .mdc-switch__handle::after{opacity:var(--mat-switch-disabled-selected-handle-opacity)}.mdc-switch--disabled.mdc-switch--unselected .mdc-switch__handle::after{opacity:var(--mat-switch-disabled-unselected-handle-opacity)}\n'],
pb:2,Ra:0});var aE=class{};aE.K=function(a){return new (a||aE)};aE.Ya=Of({type:aE});aE.Xa=Nd({imports:[XD,mz,mz]});var bE=class{};bE.K=function(a){return new (a||bE)};bE.Ya=Of({type:bE});bE.Xa=Nd({Wa:[{oa:UD,nb:{color:"primary"}}],imports:[aE]});var cE=new df("GMAT_SNACK_BAR_OPTIONS",{ea:"root",ba:()=>({sc:!1})});var dE=new df("MatSnackBarData"),eE=class{constructor(){this.Ol="assertive";this.Si="";this.duration=0;this.data=null;this.Nn="center";this.RE="bottom"}};var fE=class{};fE.K=function(a){return new (a||fE)};fE.Fa=Qf({type:fE,ha:[["","matSnackBarLabel",""]],Ka:[1,"mat-mdc-snack-bar-label","mdc-snackbar__label"],ka:!0});var gE=class{};gE.K=function(a){return new (a||gE)};gE.Fa=Qf({type:gE,ha:[["","matSnackBarActions",""]],Ka:[1,"mat-mdc-snack-bar-actions","mdc-snackbar__actions"],ka:!0});var hE=class{};hE.K=function(a){return new (a||hE)};hE.Fa=Qf({type:hE,ha:[["","matSnackBarAction",""]],Ka:[1,"mat-mdc-snack-bar-action","mdc-snackbar__action"],ka:!0});const iE=Math.pow(2,31)-1;function jE(a){a.C.closed||a.rg.Dq();clearTimeout(a.G)}var kE=class{constructor(a,b){this.la=b;this.C=new cj;this.lh=new cj;this.D=new cj;this.F=!1;this.rg=a;a.Gm.subscribe(()=>this.N())}J(a){this.G=setTimeout(()=>jE(this),Math.min(a,iE))}O(){this.lh.closed||(this.lh.next(),this.lh.complete())}N(){this.la.dispose();this.D.closed||this.D.complete();this.C.next({GF:this.F});this.C.complete();this.F=!1}pg(){return this.rg.du}};function lE(a){if(a&1){const b=Ug();R(0,"div",1)(1,"button",2);V("click",function(){u(b);const c=Y();return x(c.action())});Z(2);S()()}a&2&&(a=Y(),D(2),Br(" ",a.data.action," "))}var mE=class{constructor(a,b){this.Tx=a;this.data=b}action(){var a=this.Tx;a.D.closed||(a.F=!0,a.D.next(),a.D.complete(),jE(a));clearTimeout(a.G)}get YB(){return!!this.data.action}};mE.K=function(a){return new (a||mE)(z(kE),z(dE))};
mE.Da=Gf({type:mE,ha:[["simple-snack-bar"]],Ka:[1,"mat-mdc-simple-snack-bar"],ib:["matSnackBar"],ka:!0,features:[Mr],za:3,Aa:2,Ca:[["matSnackBarLabel",""],["matSnackBarActions",""],["mat-button","","matSnackBarAction","",3,"click"]],ta:function(a,b){a&1&&(R(0,"div",0),Z(1),S(),Ap(2,lE,3,1,"div",1));a&2&&(D(),Br(" ",b.data.message,"\n"),D(),Eq(b.YB?2:-1))},Ha:[vB,fE,gE,hE],styles:[".mat-mdc-simple-snack-bar{display:flex}\n"],pb:2,Ra:0});var nE=Yy("state",[az("void, hidden",$y({transform:"scale(0.8)",opacity:0})),az("visible",$y({transform:"scale(1)",opacity:1})),bz("* => visible",Zy("150ms cubic-bezier(0, 0, 0.2, 1)")),bz("* => void, * => hidden",Zy("75ms cubic-bezier(0.4, 0.0, 1, 1)",$y({opacity:0})))]);const oE=["label"];function pE(){}let qE=0;function rE(a,b){const c=b.zg;b=b.me;("void"===b&&"void"!==c||"hidden"===b)&&a.rt();if("visible"===b){const d=a.du;a.qa.run(()=>{d.next();d.complete()})}}
var sE=class extends Xv{constructor(a,b,c,d,e){super();this.qa=a;this.ra=b;this.Ba=c;this.Ma=d;this.Sx=e;this.ia=lf(jt);this.Op=new Set;this.zz=150;this.Ga=!1;this.Ep=new cj;this.Gm=new cj;this.du=new cj;this.bp="void";this.Bp=`mat-snack-bar-container-live-${qE++}`;this.Ii="assertive"!==e.Ol||e.Si?"off"===e.Ol?"off":"polite":"assertive";this.Ma.N&&("polite"===this.Ii&&(this.ku="status"),"assertive"===this.Ii&&(this.ku="alert"))}ae(a){a=this.Ve.ae(a);this.jt();return a}af(a){a=this.Ve.af(a);this.jt();
return a}sn(){this.Ga||(this.bp="visible",bn(this.Ba),cn(this.Ba),this.EA())}Dq(){this.qa.run(()=>{this.bp="hidden";bn(this.Ba);this.ra.R.setAttribute("mat-exit","");clearTimeout(this.lt)});return this.Gm}va(){this.Ga=!0;this.Dz();this.rt()}rt(){queueMicrotask(()=>{this.Gm.next();this.Gm.complete()})}jt(){const a=this.ra.R;var b=this.Sx.hb;b&&(Array.isArray(b)?b.forEach(c=>a.classList.add(c)):a.classList.add(b));this.Nz();b=this.fA.R;b.classList.toggle("mdc-snackbar__label",!b.querySelector(".mdc-snackbar__label"))}Nz(){const a=
this.Bp,b=this.ia.querySelectorAll('body > .cdk-overlay-container [aria-modal="true"]');for(let c=0;c<b.length;c++){const d=b[c],e=d.getAttribute("aria-owns");this.Op.add(d);e?-1===e.indexOf(a)&&d.setAttribute("aria-owns",e+" "+a):d.setAttribute("aria-owns",a)}}Dz(){this.Op.forEach(a=>{var b=a.getAttribute("aria-owns");b&&(b=b.replace(this.Bp,"").trim(),0<b.length?a.setAttribute("aria-owns",b):a.removeAttribute("aria-owns"))});this.Op.clear()}mF(){}EA(){this.lt||Mk(this.qa,()=>{this.lt=setTimeout(()=>
{const a=this.ra.R.querySelector("[aria-hidden]"),b=this.ra.R.querySelector("[aria-live]");if(a&&b){let c=null;this.Ma.isBrowser&&document.activeElement instanceof HTMLElement&&a.contains(document.activeElement)&&(c=document.activeElement);a.removeAttribute("aria-hidden");b.appendChild(a);let d;null==(d=c)||d.focus();this.Ep.next();this.Ep.complete()}},this.zz)})}};sE.K=function(a){return new (a||sE)(z(Qk),z(si),z(Sr),z(Vt),z(eE))};
sE.Da=Gf({type:sE,ha:[["mat-snack-bar-container"]],fb:function(a,b){a&1&&(ur(Zv,7),ur(oE,7));if(a&2){let c;vr(c=wr())&&(b.Ve=c.first);vr(c=wr())&&(b.fA=c.first)}},Ka:[1,"mdc-snackbar","mat-mdc-snack-bar-container","mdc-snackbar--open"],Za:1,Ia:function(a,b){a&1&&Tq("@state.done",function(c){return rE(b,c)});a&2&&Rq("@state",b.bp)},ka:!0,features:[Cp,Mr],za:6,Aa:3,Ca:[["label",""],[1,"mdc-snackbar__surface"],[1,"mat-mdc-snack-bar-label"],["aria-hidden","true"],["cdkPortalOutlet",""]],ta:function(a,
b){a&1&&(R(0,"div",1)(1,"div",2,0)(3,"div",3),Ap(4,pE,0,0,"ng-template",4),S(),Mq(5,"div"),S()());a&2&&(D(5),Kp("aria-live",b.Ii)("role",b.ku)("id",b.Bp))},Ha:[Zv],styles:['.mdc-snackbar{display:none;position:fixed;right:0;bottom:0;left:0;align-items:center;justify-content:center;box-sizing:border-box;pointer-events:none;-webkit-tap-highlight-color:rgba(0,0,0,0)}.mdc-snackbar--opening,.mdc-snackbar--open,.mdc-snackbar--closing{display:flex}.mdc-snackbar--open .mdc-snackbar__label,.mdc-snackbar--open .mdc-snackbar__actions{visibility:visible}.mdc-snackbar__surface{padding-left:0;padding-right:8px;display:flex;align-items:center;justify-content:flex-start;box-sizing:border-box;transform:scale(0.8);opacity:0}.mdc-snackbar__surface::before{position:absolute;box-sizing:border-box;width:100%;height:100%;top:0;left:0;border:1px solid rgba(0,0,0,0);border-radius:inherit;content:"";pointer-events:none}@media screen and (forced-colors: active){.mdc-snackbar__surface::before{border-color:CanvasText}}[dir=rtl] .mdc-snackbar__surface,.mdc-snackbar__surface[dir=rtl]{padding-left:8px;padding-right:0}.mdc-snackbar--open .mdc-snackbar__surface{transform:scale(1);opacity:1;pointer-events:auto}.mdc-snackbar--closing .mdc-snackbar__surface{transform:scale(1)}.mdc-snackbar__label{padding-left:16px;padding-right:8px;width:100%;flex-grow:1;box-sizing:border-box;margin:0;visibility:hidden;padding-top:14px;padding-bottom:14px}[dir=rtl] .mdc-snackbar__label,.mdc-snackbar__label[dir=rtl]{padding-left:8px;padding-right:16px}.mdc-snackbar__label::before{display:inline;content:attr(data-mdc-snackbar-label-text)}.mdc-snackbar__actions{display:flex;flex-shrink:0;align-items:center;box-sizing:border-box;visibility:hidden}.mdc-snackbar__action+.mdc-snackbar__dismiss{margin-left:8px;margin-right:0}[dir=rtl] .mdc-snackbar__action+.mdc-snackbar__dismiss,.mdc-snackbar__action+.mdc-snackbar__dismiss[dir=rtl]{margin-left:0;margin-right:8px}.mat-mdc-snack-bar-container{margin:8px;position:static}.mat-mdc-snack-bar-container .mdc-snackbar__surface{min-width:344px}@media(max-width: 480px),(max-width: 344px){.mat-mdc-snack-bar-container .mdc-snackbar__surface{min-width:100%}}@media(max-width: 480px),(max-width: 344px){.mat-mdc-snack-bar-container{width:100vw}}.mat-mdc-snack-bar-container .mdc-snackbar__surface{max-width:672px}.mat-mdc-snack-bar-container .mdc-snackbar__surface{box-shadow:0px 3px 5px -1px rgba(0, 0, 0, 0.2), 0px 6px 10px 0px rgba(0, 0, 0, 0.14), 0px 1px 18px 0px rgba(0, 0, 0, 0.12)}.mat-mdc-snack-bar-container .mdc-snackbar__surface{background-color:var(--mdc-snackbar-container-color)}.mat-mdc-snack-bar-container .mdc-snackbar__surface{border-radius:var(--mdc-snackbar-container-shape)}.mat-mdc-snack-bar-container .mdc-snackbar__label{color:var(--mdc-snackbar-supporting-text-color)}.mat-mdc-snack-bar-container .mdc-snackbar__label{font-size:var(--mdc-snackbar-supporting-text-size);font-family:var(--mdc-snackbar-supporting-text-font);font-weight:var(--mdc-snackbar-supporting-text-weight);line-height:var(--mdc-snackbar-supporting-text-line-height)}.mat-mdc-snack-bar-container .mat-mdc-button.mat-mdc-snack-bar-action:not(:disabled){color:var(--mat-snack-bar-button-color);--mat-text-button-state-layer-color:currentColor;--mat-text-button-ripple-color:currentColor}.mat-mdc-snack-bar-container .mat-mdc-button.mat-mdc-snack-bar-action:not(:disabled) .mat-ripple-element{opacity:.1}.mat-mdc-snack-bar-container .mdc-snackbar__label::before{display:none}.mat-mdc-snack-bar-handset,.mat-mdc-snack-bar-container,.mat-mdc-snack-bar-label{flex:1 1 auto}.mat-mdc-snack-bar-handset .mdc-snackbar__surface{width:100%}\n'],
pb:2,data:{animation:[nE]}});/*

 Copyright Google LLC All Rights Reserved.

 Use of this source code is governed by an MIT-style license that can be
 found in the LICENSE file at https://angular.io/license
 @type {{XSmall: string, Small: string, Medium: string, Large: string, XLarge: string, Handset: string, Tablet: string, Web: string, HandsetPortrait: string, TabletPortrait: string, WebPortrait: string, HandsetLandscape: string, TabletLandscape: string, WebLandscape: string}}
*/
var tE=new df("mat-snack-bar-default-options",{ea:"root",ba:function(){return new eE}});function uE(a,b){const c=new Au;c.direction=b.direction;let d=Qv(a.G.position());var e="rtl"===b.direction;e="left"===b.Nn||"start"===b.Nn&&!e||"end"===b.Nn&&e;const f=!e&&"center"!==b.Nn;e?d.left("0"):f?d.right("0"):Nv(d);"top"===b.RE?d.top("0"):d.bottom("0");c.uc=d;return a.G.create(c)}
var vE=class{get C(){const a=this.F;return a?a.C:this.D}set C(a){this.F?this.F.C=a:this.D=a}constructor(a,b,c,d,e,f){this.G=a;this.Ii=b;this.Ua=c;this.T=d;this.F=e;this.J=f;this.D=null;this.aa=mE;this.ca=sE}open(a,b="",c){const d=Object.assign({},this.J,c);d.data={message:a,action:b};d.Si===a&&(d.Si=void 0);return this.O(this.aa,d)}va(){this.D&&jE(this.D)}P(a,b){const c=Vh({parent:b&&b.xb&&b.xb.xa||this.Ua,Wa:[{oa:eE,nb:b}]});a=a.attach(new Uv(this.ca,b.xb,c));a.instance.Sx=b;return a.instance}O(a,
b){const c=Object.assign({},new eE,this.J,b),d=uE(this,c);b=this.P(d,c);const e=new kE(b,d);if(a instanceof br)e.instance=b.af(new Vv(a,null,{da:c.data,Tx:e}));else{const f=Vh({parent:c&&c.xb&&c.xb.xa||this.Ua,Wa:[{oa:kE,nb:e},{oa:dE,nb:c.data}]});a=b.ae(new Uv(a,void 0,f));e.instance=a.instance}C(this.T.observe("(max-width: 599.98px) and (orientation: portrait)"),Lo(d.J)).subscribe(f=>{d.N.classList.toggle("mat-mdc-snack-bar-handset",f.matches)});c.Si&&b.Ep.subscribe(()=>{py(this.Ii,c.Si,c.Ol)});
this.N(e,c);return this.C=e}N(a,b){a.C.subscribe(()=>{this.C==a&&(this.C=null);b.Si&&this.Ii.clear()});this.C?(this.C.C.subscribe(()=>{a.rg.sn()}),jE(this.C)):a.rg.sn();b.duration&&0<b.duration&&a.pg().subscribe(()=>a.J(b.duration))}};vE.K=function(a){return new (a||vE)(r(bw),r(qy),r(Wh),r(gy),r(vE,12),r(tE))};vE.pa=Md({ma:vE,ba:vE.K,ea:"root"});var wE=class{};wE.K=function(a){return new (a||wE)};wE.Ya=Of({type:wE});wE.Xa=Nd({Wa:[vE],imports:[jw,$v,zB,mz,mE,mz]});var xE=class extends vE{constructor(a,b,c,d,e,f,g){g.sc||kw(a,"gmat-mdc-snack-bar");super(a,b,c,d,e,f)}};xE.K=function(a){return new (a||xE)(r(bw),r(qy),r(Wh),r(gy),r(vE,12),r(tE),r(cE))};xE.pa=Md({ma:xE,ba:xE.K});var yE=class{};yE.K=function(a){return new (a||yE)};yE.Ya=Of({type:yE});yE.Xa=Nd({Wa:[{oa:vE,Ec:xE},{oa:bw,Ec:lw}],imports:[wE]});var zE=new df("GMAT_TOOLTIP_OPTIONS",{ea:"root",ba:()=>({sc:!1})});var AE=class{constructor(a,b){a.sc||kw(b,"gmat-mdc-tooltip")}};AE.K=function(a){return new (a||AE)(z(zE),z(bw))};AE.Fa=Qf({type:AE,ha:[["","matTooltip",""]],features:[Kr([{oa:bw,Ec:lw}])]});const BE=["tooltip"];var CE=new df("mat-tooltip-scroll-strategy",{ea:"root",ba:()=>{const a=lf(bw);return()=>a.C.C({Cx:20})}}),DE={oa:CE,Ac:[bw],Fc:function(a){return()=>a.C.C({Cx:20})}},EE=new df("mat-tooltip-default-options",{ea:"root",ba:function(){return{Yl:0,nl:0,xE:1500}}});const FE=Kt({passive:!0});function GE(a){a.la&&a.la.ic()&&a.la.detach();a.C=null}function $F(a,b=a.nl){const c=a.C;c&&(c.isVisible()?aG(c,b):(c.G(),GE(a)))}
function bG(a,b=a.Yl,c){if(a.disabled||!a.message||a.G()){let d;null==(d=a.C)||d.G()}else c=cG(a,c),GE(a),a.ua=a.ua||new Uv(a.Hc,a.Dc),c=a.C=c.attach(a.ua).instance,c.Pi=a.ra.R,c.aa=a.J,C(c.P,Lo(a.Ga)).subscribe(()=>GE(a)),a.sb(a.ca),a.Nb(),dG(c,b)}
function cG(a,b){if(a.la){var c=a.la.ya.uc;if((!a.Zh||!b)&&c.sb instanceof si)return a.la;GE(a)}c=ju(a.ac,a.ra);b=Ju(Lu(Mu(Hv(Rv(a.sa.position(),a.Zh?b||a.ra:a.ra),".mat-mdc-tooltip"),!1),a.wa),c);C(b.gb,Lo(a.Ga)).subscribe(e=>{a.Ic(e.C);a.C&&e.D.gr&&a.C.isVisible()&&a.qa.run(()=>$F(a,0))});a.la=a.sa.create({direction:a.D,uc:b,hb:"mat-mdc-tooltip-panel",vc:a.Kc()});a.ja(a.la);C(a.la.J,Lo(a.Ga)).subscribe(()=>GE(a));C(a.la.Ll(),Lo(a.Ga)).subscribe(()=>{let e;return null==(e=a.C)?void 0:e.fa()});C(a.la.ef(),
Lo(a.Ga)).subscribe(e=>{a.G()&&27===e.keyCode&&!Bu(e)&&(e.preventDefault(),e.stopPropagation(),a.qa.run(()=>$F(a,0)))});let d;(null==(d=a.O)?0:d.FF)&&a.la.og("mat-mdc-tooltip-panel-non-interactive");return a.la}
var fG=class{get position(){return this.P}set position(a){if(a!==this.P&&(this.P=a,this.la)){this.ja(this.la);let b;null==(b=this.C)||dG(b,0);this.la.Wc()}}get Zh(){return this.lb}set Zh(a){this.lb=Qs(a);GE(this);this.la=null}get disabled(){return this.Ib}set disabled(a){(this.Ib=Qs(a))?$F(this,0):this.aa()}get Yl(){return this.Lb}set Yl(a){this.Lb=Ts(a)}get nl(){return this.J}set nl(a){this.J=Ts(a);this.C&&(this.C.aa=this.J)}get message(){return this.N}set message(a){zy(this.T,this.ra.R,this.N);
this.N=null!=a?String(a).trim():"";!this.N&&this.G()?$F(this,0):(this.aa(),this.Nb(),Mk(this.qa,()=>{Promise.resolve().then(()=>{wy(this.T,this.ra.R,this.message,"tooltip")})}))}get Io(){return this.ca}set Io(a){this.ca=a;this.C&&this.sb(this.ca)}constructor(a,b,c,d,e,f,g,k,m,p,t,w){this.sa=a;this.ra=b;this.ac=c;this.Dc=d;this.qa=e;this.Ma=f;this.T=g;this.na=k;this.D=p;this.O=t;this.P="below";this.gb=this.Ob=this.Ib=this.lb=!1;this.Hc=eG;this.wa=8;this.Xc="mat-mdc";this.ni="auto";this.N="";this.F=
[];this.Ga=new cj;this.Ua=lf(Wh);this.Kc=m;this.ia=w;t&&(this.Lb=t.Yl,this.J=t.nl,t.position&&(this.position=t.position),t.Zh&&(this.Zh=t.Zh),t.ni&&(this.ni=t.ni));C(p.wb,Lo(this.Ga)).subscribe(()=>{this.la&&this.ja(this.la)});this.wa=8}pd(){this.Ob=!0;this.aa();C(Lx(this.na,this.ra),Lo(this.Ga)).subscribe(a=>{a?"keyboard"===a&&this.qa.run(()=>bG(this)):this.qa.run(()=>$F(this,0))})}va(){const a=this.ra.R;clearTimeout(this.fa);this.la&&(this.la.dispose(),this.C=null);this.F.forEach(([b,c])=>{a.removeEventListener(b,
c,FE)});this.F.length=0;this.Ga.next();this.Ga.complete();zy(this.T,a,this.message);Mx(this.na,a)}toggle(a){this.G()?$F(this):bG(this,void 0,a)}G(){return!!this.C&&this.C.isVisible()}ja(a){a=a.ya.uc;const b=this.Gc();var c=!this.D||"ltr"==this.D.value;const d=this.position;let e;if("above"==d)e={Na:"center",Ta:"bottom"};else if("below"==d)e={Na:"center",Ta:"top"};else if("before"==d||"left"==d&&c||"right"==d&&!c)e={Na:"end",Ta:"center"};else if("after"==d||"right"==d&&c||"left"==d&&!c)e={Na:"start",
Ta:"center"};const {x:f,y:g}=this.Va(e.Na,e.Ta);c={Na:f,Ta:g};Ku(a,[this.Ja(Object.assign({},b.KC,e)),this.Ja(Object.assign({},b.fallback,c))])}Ja(a){const b=!this.D||"ltr"==this.D.value;"top"===a.eb?a.offsetY=-8:"bottom"===a.eb?a.offsetY=8:"start"===a.cb?a.offsetX=b?-8:8:"end"===a.cb&&(a.offsetX=b?8:-8);return a}Gc(){const a=!this.D||"ltr"==this.D.value,b=this.position;let c;if("above"==b||"below"==b)c={cb:"center",eb:"above"==b?"top":"bottom"};else if("before"==b||"left"==b&&a||"right"==b&&!a)c=
{cb:"start",eb:"center"};else if("after"==b||"right"==b&&a||"left"==b&&!a)c={cb:"end",eb:"center"};const {x:d,y:e}=this.Va(c.cb,c.eb);return{KC:c,fallback:{cb:d,eb:e}}}Nb(){this.C&&(this.C.message=this.message,this.C.O(),al(()=>{this.C&&this.la.Wc()},{xa:this.Ua}))}sb(a){this.C&&(this.C.Io=a,this.C.O())}Va(a,b){"above"===this.position||"below"===this.position?"top"===b?b="bottom":"bottom"===b&&(b="top"):"end"===a?a="start":"start"===a&&(a="end");return{x:a,y:b}}Ic(a){var b=a.Ta,c=a.cb;a=a.eb;b="center"===
b?this.D&&"rtl"===this.D.value?"end"===c?"left":"right":"start"===c?"left":"right":"bottom"===b&&"top"===a?"above":"below";if(b!==this.Pa){if(c=this.la)c.Jj("mat-mdc-tooltip-panel-"+this.Pa),c.og("mat-mdc-tooltip-panel-"+b);this.Pa=b}}aa(){!this.Ib&&this.message&&this.Ob&&!this.F.length&&(this.bb()?this.F.push(["mouseenter",a=>{this.Kb();let b=void 0;void 0!==a.x&&void 0!==a.y&&(b=a);bG(this,void 0,b)}]):"off"!==this.ni&&(this.Sa(),this.F.push(["touchstart",a=>{let b;const c=(a=null==(b=a.targetTouches)?
void 0:b[0])?{x:a.clientX,y:a.clientY}:void 0;this.Kb();clearTimeout(this.fa);let d;this.fa=setTimeout(()=>bG(this,void 0,c),null!=(d=this.O.AG)?d:500)}])),this.Ea(this.F))}Kb(){if(!this.gb){this.gb=!0;var a=[];if(this.bb())a.push(["mouseleave",b=>{b=b.relatedTarget;let c;b&&(null==(c=this.la)?0:c.N.contains(b))||$F(this)}],["wheel",b=>this.Jc(b)]);else if("off"!==this.ni){this.Sa();const b=()=>{clearTimeout(this.fa);$F(this,this.O.xE)};a.push(["touchend",b],["touchcancel",b])}this.Ea(a);this.F.push(...a)}}Ea(a){a.forEach(([b,
c])=>{this.ra.R.addEventListener(b,c,FE)})}bb(){return!this.Ma.C&&!this.Ma.P}Jc(a){if(this.G()){a=this.ia.elementFromPoint(a.clientX,a.clientY);const b=this.ra.R;a===b||b.contains(a)||$F(this)}}Sa(){const a=this.ni;if("off"!==a){const b=this.ra.R,c=b.style;if("on"===a||"INPUT"!==b.nodeName&&"TEXTAREA"!==b.nodeName)c.userSelect=c.msUserSelect=c.webkitUserSelect=c.MozUserSelect="none";"on"!==a&&b.draggable||(c.webkitUserDrag="none");c.touchAction="none";c.webkitTapHighlightColor="transparent"}}};
fG.K=function(a){return new (a||fG)(z(bw),z(si),z(ku),z(kp),z(Qk),z(Vt),z(Ay),z(Ox),z(CE),z(ou),z(EE,8),z(jt))};
fG.Fa=Qf({type:fG,ha:[["","matTooltip",""]],Ka:[1,"mat-mdc-tooltip-trigger"],Za:2,Ia:function(a,b){a&2&&nq("mat-mdc-tooltip-disabled",b.disabled)},inputs:{position:[0,"matTooltipPosition","position"],Zh:[0,"matTooltipPositionAtOrigin","positionAtOrigin"],disabled:[0,"matTooltipDisabled","disabled"],Yl:[0,"matTooltipShowDelay","showDelay"],nl:[0,"matTooltipHideDelay","hideDelay"],ni:[0,"matTooltipTouchGestures","touchGestures"],message:[0,"matTooltip","message"],Io:[0,"matTooltipClass","tooltipClass"]},
ib:["matTooltip"],ka:!0});function dG(a,b){null!=a.C&&clearTimeout(a.C);a.D=setTimeout(()=>{a.ca(!0);a.D=void 0},b)}function aG(a,b){null!=a.D&&clearTimeout(a.D);a.C=setTimeout(()=>{a.ca(!1);a.C=void 0},b)}
var eG=class{constructor(a,b,c){this.Ba=a;this.ra=b;this.N=this.T=this.Zt=!1;this.P=new cj;this.ua="mat-mdc-tooltip-show";this.sa="mat-mdc-tooltip-hide";this.F="NoopAnimations"===c}isVisible(){return this.N}va(){this.G();this.P.complete();this.Pi=null}fa(){this.T&&aG(this,0)}O(){bn(this.Ba)}aA({relatedTarget:a}){a&&this.Pi.contains(a)||(this.isVisible()?aG(this,this.aa):this.J(!1))}na(){this.Zt=this.ja();this.O()}ja(){const a=this.ra.R.getBoundingClientRect();return 24<a.height&&200<=a.width}Wz({animationName:a}){"mat-mdc-tooltip-show"!==
a&&"mat-mdc-tooltip-hide"!==a||this.J("mat-mdc-tooltip-show"===a)}G(){null!=this.D&&clearTimeout(this.D);null!=this.C&&clearTimeout(this.C);this.D=this.C=void 0}J(a){a?this.T=!0:this.isVisible()||this.P.next()}ca(a){const b=this.OA.R;b.classList.remove(a?"mat-mdc-tooltip-hide":"mat-mdc-tooltip-show");b.classList.add(a?"mat-mdc-tooltip-show":"mat-mdc-tooltip-hide");this.N!==a&&(this.N=a,bn(this.Ba));if(a&&!this.F&&"function"===typeof getComputedStyle){const c=getComputedStyle(b);if("0s"===c.getPropertyValue("animation-duration")||
"none"===c.getPropertyValue("animation-name"))this.F=!0}a&&this.na();this.F&&(b.classList.add("_mat-animation-noopable"),this.J(a))}};eG.K=function(a){return new (a||eG)(z(Sr),z(si),z(mi,8))};
eG.Da=Gf({type:eG,ha:[["mat-tooltip-component"]],fb:function(a,b){a&1&&ur(BE,7);if(a&2){let c;vr(c=wr())&&(b.OA=c.first)}},Ka:["aria-hidden","true"],Za:2,Ia:function(a,b){a&1&&V("mouseleave",function(c){return b.aA(c)});a&2&&lq("zoom",b.isVisible()?1:null)},ka:!0,features:[Mr],za:4,Aa:4,Ca:[["tooltip",""],[1,"mdc-tooltip","mdc-tooltip--shown","mat-mdc-tooltip",3,"animationend","ngClass"],[1,"mdc-tooltip__surface","mdc-tooltip__surface-animation"]],ta:function(a,b){if(a&1){const c=Ug();R(0,"div",1,
0);V("animationend",function(d){u(c);return x(b.Wz(d))});R(2,"div",2);Z(3);S()()}a&2&&(nq("mdc-tooltip--multiline",b.Zt),H("ngClass",b.Io),D(3),Ar(b.message))},Ha:[Ws],styles:['.mdc-tooltip__surface{word-break:break-all;word-break:var(--mdc-tooltip-word-break, normal);overflow-wrap:anywhere}.mdc-tooltip--showing-transition .mdc-tooltip__surface-animation{transition:opacity 150ms 0ms cubic-bezier(0, 0, 0.2, 1),transform 150ms 0ms cubic-bezier(0, 0, 0.2, 1)}.mdc-tooltip--hide-transition .mdc-tooltip__surface-animation{transition:opacity 75ms 0ms cubic-bezier(0.4, 0, 1, 1)}.mdc-tooltip{position:fixed;display:none;z-index:9}.mdc-tooltip-wrapper--rich{position:relative}.mdc-tooltip--shown,.mdc-tooltip--showing,.mdc-tooltip--hide{display:inline-flex}.mdc-tooltip--shown.mdc-tooltip--rich,.mdc-tooltip--showing.mdc-tooltip--rich,.mdc-tooltip--hide.mdc-tooltip--rich{display:inline-block;left:-320px;position:absolute}.mdc-tooltip__surface{line-height:16px;padding:4px 8px;min-width:40px;max-width:200px;min-height:24px;max-height:40vh;box-sizing:border-box;overflow:hidden;text-align:center}.mdc-tooltip__surface::before{position:absolute;box-sizing:border-box;width:100%;height:100%;top:0;left:0;border:1px solid rgba(0,0,0,0);border-radius:inherit;content:"";pointer-events:none}@media screen and (forced-colors: active){.mdc-tooltip__surface::before{border-color:CanvasText}}.mdc-tooltip--rich .mdc-tooltip__surface{align-items:flex-start;display:flex;flex-direction:column;min-height:24px;min-width:40px;max-width:320px;position:relative}.mdc-tooltip--multiline .mdc-tooltip__surface{text-align:left}[dir=rtl] .mdc-tooltip--multiline .mdc-tooltip__surface,.mdc-tooltip--multiline .mdc-tooltip__surface[dir=rtl]{text-align:right}.mdc-tooltip__surface .mdc-tooltip__title{margin:0 8px}.mdc-tooltip__surface .mdc-tooltip__content{max-width:calc(200px - 2*8px);margin:8px;text-align:left}[dir=rtl] .mdc-tooltip__surface .mdc-tooltip__content,.mdc-tooltip__surface .mdc-tooltip__content[dir=rtl]{text-align:right}.mdc-tooltip--rich .mdc-tooltip__surface .mdc-tooltip__content{max-width:calc(320px - 2*8px);align-self:stretch}.mdc-tooltip__surface .mdc-tooltip__content-link{text-decoration:none}.mdc-tooltip--rich-actions,.mdc-tooltip__content,.mdc-tooltip__title{z-index:1}.mdc-tooltip__surface-animation{opacity:0;transform:scale(0.8);will-change:transform,opacity}.mdc-tooltip--shown .mdc-tooltip__surface-animation{transform:scale(1);opacity:1}.mdc-tooltip--hide .mdc-tooltip__surface-animation{transform:scale(1)}.mdc-tooltip__caret-surface-top,.mdc-tooltip__caret-surface-bottom{position:absolute;height:24px;width:24px;transform:rotate(35deg) skewY(20deg) scaleX(0.9396926208)}.mdc-tooltip__caret-surface-top .mdc-elevation-overlay,.mdc-tooltip__caret-surface-bottom .mdc-elevation-overlay{width:100%;height:100%;top:0;left:0}.mdc-tooltip__caret-surface-bottom{box-shadow:0px 3px 1px -2px rgba(0, 0, 0, 0.2), 0px 2px 2px 0px rgba(0, 0, 0, 0.14), 0px 1px 5px 0px rgba(0, 0, 0, 0.12);outline:1px solid rgba(0,0,0,0);z-index:-1}@media screen and (forced-colors: active){.mdc-tooltip__caret-surface-bottom{outline-color:CanvasText}}.mat-mdc-tooltip .mdc-tooltip__surface{background-color:var(--mdc-plain-tooltip-container-color)}.mat-mdc-tooltip .mdc-tooltip__surface{border-radius:var(--mdc-plain-tooltip-container-shape)}.mat-mdc-tooltip .mdc-tooltip__caret-surface-top,.mat-mdc-tooltip .mdc-tooltip__caret-surface-bottom{border-radius:var(--mdc-plain-tooltip-container-shape)}.mat-mdc-tooltip .mdc-tooltip__surface{color:var(--mdc-plain-tooltip-supporting-text-color)}.mat-mdc-tooltip .mdc-tooltip__surface{font-family:var(--mdc-plain-tooltip-supporting-text-font);line-height:var(--mdc-plain-tooltip-supporting-text-line-height);font-size:var(--mdc-plain-tooltip-supporting-text-size);font-weight:var(--mdc-plain-tooltip-supporting-text-weight);letter-spacing:var(--mdc-plain-tooltip-supporting-text-tracking)}.mat-mdc-tooltip{position:relative;transform:scale(0)}.mat-mdc-tooltip::before{content:"";top:0;right:0;bottom:0;left:0;z-index:-1;position:absolute}.mat-mdc-tooltip-panel-below .mat-mdc-tooltip::before{top:-8px}.mat-mdc-tooltip-panel-above .mat-mdc-tooltip::before{bottom:-8px}.mat-mdc-tooltip-panel-right .mat-mdc-tooltip::before{left:-8px}.mat-mdc-tooltip-panel-left .mat-mdc-tooltip::before{right:-8px}.mat-mdc-tooltip._mat-animation-noopable{animation:none;transform:scale(1)}.mat-mdc-tooltip-panel.mat-mdc-tooltip-panel-non-interactive{pointer-events:none}@keyframes mat-mdc-tooltip-show{0%{opacity:0;transform:scale(0.8)}100%{opacity:1;transform:scale(1)}}@keyframes mat-mdc-tooltip-hide{0%{opacity:1;transform:scale(1)}100%{opacity:0;transform:scale(0.8)}}.mat-mdc-tooltip-show{animation:mat-mdc-tooltip-show 150ms cubic-bezier(0, 0, 0.2, 1) forwards}.mat-mdc-tooltip-hide{animation:mat-mdc-tooltip-hide 75ms cubic-bezier(0.4, 0, 1, 1) forwards}\n'],
pb:2,Ra:0});var gG=class{};gG.K=function(a){return new (a||gG)};gG.Ya=Of({type:gG});gG.Xa=Nd({Wa:[DE],imports:[ry,ht,jw,mz,mz,xu]});var hG=class{};hG.K=function(a){return new (a||hG)};hG.Ya=Of({type:hG});hG.Xa=Nd({imports:[gG]});const iG=$y({opacity:0}),jG=$y({opacity:1});var kG=Yy("dialogContainer",[az("void, hidden",iG),az("visible",jG),bz("void => *, * => visible",Zy("200ms cubic-bezier(0.0,0.0,0.2,1)",jG)),bz("* => void, * => hidden",Zy("200ms cubic-bezier(0.4,0.0,0.2,1)",iG))]);function lG(a){a&1&&Mq(0,"a",1)}var mG=new df("XAP_IDCO");function nG(a,b){a.Hu=b?"visible":"hidden";bn(a.C)}var oG=class extends Xv{constructor(){super(...arguments);this.Hu="hidden";this.options=lf(mG);this.C=lf(Sr)}pd(){}ae(a){return this.ex.ae(a)}af(a){return this.ex.af(a)}};oG.K=(()=>{let a;return function(b){return(a||(a=Mh(oG)))(b||oG)}})();
oG.Da=Gf({type:oG,ha:[["xap-inline-dialog-container"]],fb:function(a,b){a&1&&ur(Zv,7);if(a&2){let c;vr(c=wr())&&(b.ex=c.first)}},Ka:["role","dialog",1,"xap-inline-dialog-container","gmat-body-2"],Za:3,Ia:function(a,b){a&2&&(Rq("@dialogContainer",b.Hu),Kp("aria-label",b.options.kn)("tabindex",b.options.ln))},features:[Cp],za:2,Aa:0,Ca:[["cdkPortalOutlet",""],["cdkFocusInitial","","tabindex","0"]],ta:function(a){a&1&&(R(0,"div"),Ap(1,lG,1,0,"ng-template",0),S())},Ha:[Zv],styles:[".xap-inline-dialog-container{box-shadow:0 1px 2px 0 rgba(60,64,67,.3),0 1px 3px 1px rgba(60,64,67,.15);border-radius:8px;overflow:auto}"],
pb:2,data:{animation:[kG]},Ra:0});var pG=[{cb:"start",eb:"bottom",Na:"start",Ta:"top",offsetY:8},{cb:"start",eb:"top",Na:"start",Ta:"bottom",offsetY:-8},{cb:"end",eb:"top",Na:"end",Ta:"bottom",offsetY:-8},{cb:"end",eb:"bottom",Na:"end",Ta:"top",offsetY:8}],qG={minWidth:220,maxWidth:420,minHeight:64,maxHeight:420};
function rG(a){let b;null==(b=a.Vb)||!b.ic()||a.Tj&&!a.Tj.C(a.ua)||(nG(a.G.instance,!1),setTimeout(()=>{a.F.run(()=>{a.Vb&&a.Vb.detach();a.J&&(a.J.destroy(),a.J=void 0);a.Ng();a.G&&(a.G.destroy(),a.G=void 0);a.closed.emit()})},200))}function sG(a,b){Mk(a.F,()=>{C(lk(b,"keydown"),Lo(a.C)).subscribe(c=>{switch(c.keyCode){case 13:tG(a);c.preventDefault();break;case 32:c.preventDefault()}});C(lk(b,"keyup"),Lo(a.C)).subscribe(({keyCode:c})=>{switch(c){case 32:tG(a)}})})}
function uG(a,b){Mk(a.F,()=>{C(lk(b,"focus"),tk(()=>0<a.zh.mc.length),Lo(a.C)).subscribe(()=>{a.F.run(()=>{a.zh.emit()})})})}function tG(a){if(!a.disabled){var b;if(null==(b=a.Vb)||!b.ic()){var c=a.Ld;null!=c&&(a.Tj&&(a.ua=a.Tj.register()),a.F.run(()=>{a.G=vG(a);a.G.instance.attach(c);const d=a.G.location.R;a.P(d);a.aa(d);wG(a,d);nG(a.G.instance,!0);a.hf.mc.length&&setTimeout(()=>{a.hf.emit()},200)}))}}}
function vG(a){var b=Vh({parent:a.xa,Wa:[{oa:mG,nb:{kn:a.kn,ln:a.ln}}]});b=new Uv(oG,null,b);null==a.Vb&&(a.Vb=a.T.create(a.ja(pG)));a.Vw&&a.Vb.ne(a.Vw);if(a.Kr&&0<a.Kr.length){var c=a.Vb,d=xG(a,a.Kr);d!==c.F&&(c.F&&c.F.dispose(),c.F=d,c.ic()&&(d.attach(c),c.Wc()))}return a.Vb.attach(b)}function wG(a,b){const c=a.document.activeElement,d=a.fc.R;if(a.Sv||c&&c===d)a.document&&(a.fa=a.document.activeElement),a.J=a.Ea.create(b),Vx(a.J),a.uq||yG(a)}function yG(a){a.J&&Xx(a.J).then(()=>{})}
function xG(a,b){let c,d;return Hu(Ku(Rv(a.T.position(),null!=(c=a.Ku)?c:a.fc),b),null!=(d=a.Ku)?d:a.fc)}function zG(a){const b=a.T.C;switch(a.vc){case "noop":return b.G();case "block":return b.block();case "reposition":return b.C();default:return b.close()}}
var AG=class{set Gh(a){a&&(this.Ld=a instanceof br?new Vv(a,this.xb):new Uv(a,this.xb))}set disabled(a){(this.ca=a)&&rG(this)}get disabled(){return this.ca}set uq(a){this.na=Qs(a)}get uq(){return this.na}set hb(a){if(this.N!==a){if(this.N){let b;null==(b=this.Vb)||b.Jj(this.N)}if(a){let b;null==(b=this.Vb)||b.og(a)}this.N=a}}constructor(a,b,c,d,e,f,g,k){this.F=a;this.T=b;this.fc=c;this.xb=d;this.document=e;this.Ea=f;this.wa=g;this.xa=k;this.ca=!1;this.kn=null;this.ln=-1;this.Jr=qG;this.ty="Press space for more information.";
this.Sv=this.na=!1;this.vc="close";this.ua=0;this.zh=new Fk;this.hf=new Fk;this.closed=new Fk;this.C=new mj;this.D=new ej(!1);this.sa=C(this.D,tk(m=>m&&!this.disabled));this.fa=void 0;sG(this,c.R);this.aa(c.R);uG(this,c.R)}Fb(){var a=this.fc.R;a.hasAttribute("aria-haspopup")||a.setAttribute("aria-haspopup","dialog");a.hasAttribute("aria-describedby")||wy(this.wa,a,this.ty)}va(){rG(this);this.D.complete();this.C.next();this.C.complete();this.Vb&&this.Vb.dispose()}Ng(){const a=this.fa;if(a&&"function"===
typeof a.focus){var b,c=null==(b=this.G)?void 0:b.location.R;(b=this.document.activeElement)&&null!=c&&c.contains(b)&&(a.focus(),this.fa=void 0)}}};AG.K=function(a){return new (a||AG)(z(Qk),z(bw),z(si),z(kp),z(Document),z(Iy),z(Ay),z(Wh))};
AG.Fa=Qf({type:AG,Za:1,Ia:function(a,b){a&2&&Kp("data-disabled",b.disabled)},inputs:{Ku:"attachedTo",kn:"dialogLabel",ln:"dialogTabIndex",Vw:"overlaySize",Kr:"overlayPositions",Jr:"overlayDimensions",ty:"triggerDescription",uq:"disableAutoFocus",Sv:"forceTrapFocus",hb:"panelClass",vc:"scrollStrategy",Tj:"stackManager"},outputs:{zh:"beforeOpened",hf:"opened",closed:"closed"}});function BG(a,b){const c=C(a.D,Hn(()=>C(a.O,On(b))));C(c,Lo(a.C)).subscribe(d=>{d?tG(a):rG(a)});C(a.sa,tk(d=>d&&0<a.zh.mc.length),To(()=>C(c,tk(d=>!d))),Lo(a.C)).subscribe(()=>{a.F.run(()=>{a.zh.emit()})})}function CG(a){a.disabled||(a.D.next(!0),a.O.next())}function DG(a){a.D.next(!1);a.O.next()}
var EG=class extends AG{constructor(a,b,c,d,e,f,g,k){super(a,b,c,d,e,f,g,k);this.Tq=500;this.O=new cj;this.P(c.R)}Fb(){super.Fb();BG(this,this.Tq)}P(a){this.Tj&&C(this.Tj.D(),On(this.Tq),Lo(this.C)).subscribe(()=>{a.matches(":hover")||this.xb.element.R.matches(":hover")?CG(this):DG(this)});Mk(this.F,()=>{C(lk(a,"mouseenter"),Lo(this.C)).subscribe(()=>{CG(this)});C(lk(a,"click"),Lo(this.C)).subscribe(b=>{b.target.closest("[xapInlineDialogClose]")?DG(this):(CG(this),tG(this))});C(lk(a,"mouseleave"),
Lo(this.C)).subscribe(()=>{DG(this)});C(lk(a,"mousemove"),Lo(this.C)).subscribe(()=>{this.O.next()})})}aa(a){Mk(this.F,()=>{C(lk(a,"keydown"),Lo(this.C)).subscribe(b=>{switch(b.keyCode){case 27:let c;(null==(c=this.Vb)?0:c.ic())&&b.stopPropagation();rG(this)}})})}ja(a){return new Au(Object.assign({},this.Jr,{uc:xG(this,a),vc:zG(this),hb:this.N}))}};EG.K=function(a){return new (a||EG)(z(Qk),z(bw),z(si),z(kp),z(jt),z(Iy),z(Ay),z(Wh))};
EG.Fa=Qf({type:EG,ha:[["","xapInlineDialog",""]],Ka:[1,"xap-inline-dialog"],inputs:{Gh:[0,"xapInlineDialog","dialog"],disabled:[0,"xapInlineDialogDisabled","disabled"],Tq:"hoverDelayMs"},ib:["xapInlineDialog"],features:[Cp]});function FG(a){C(a.D,Lo(a.C)).subscribe(b=>{b?tG(a):rG(a)});C(a.sa,tk(b=>b&&0<a.zh.mc.length),To(()=>C(a.D,tk(b=>!b))))}
var GG=class extends AG{constructor(a,b,c,d,e,f,g,k){super(a,b,c,d,e,f,g,k);this.P(c.R)}Fb(){super.Fb();FG(this)}P(a){Mk(this.F,()=>{C(lk(a,"mouseenter"),Lo(this.C)).subscribe(()=>{this.F.run(()=>{this.zh.emit()})});C(lk(a,"click"),Lo(this.C)).subscribe(c=>{c.target.closest("[xapInlineDialogClose]")?this.D.next(!1):this.disabled||this.D.next(!0)});let b;null==(b=this.Vb)||C(b.dd(),Lo(this.C)).subscribe(()=>{this.D.next(!1)})})}aa(a){Mk(this.F,()=>{C(lk(a,"keyup"),Lo(this.C)).subscribe(b=>{const c=
b.target;switch(b.keyCode){case 27:this.D.next(!1);break;case 32:c.closest("[xapInlineDialogClose]")&&this.D.next(!1)}});C(lk(a,"keydown"),Lo(this.C)).subscribe(b=>{const c=b.target;switch(b.keyCode){case 13:c.closest("[xapInlineDialogClose]")&&this.D.next(!1)}})})}ja(a){return new Au(Object.assign({},this.Jr,{uc:xG(this,a),vc:zG(this),Tb:!0,dc:"cdk-overlay-transparent-backdrop",hb:this.N}))}};GG.K=function(a){return new (a||GG)(z(Qk),z(bw),z(si),z(kp),z(jt),z(Iy),z(Ay),z(Wh))};
GG.Fa=Qf({type:GG,ha:[["","xapInlineDialogClick",""]],Ka:[1,"xap-inline-dialog-click"],inputs:{Gh:[0,"xapInlineDialogClick","dialog"],disabled:[0,"xapInlineDialogDisabled","disabled"]},ib:["xapInlineDialogClick"],features:[Cp]});var HG=class{};HG.K=function(a){return new (a||HG)};HG.Ya=Of({type:HG});HG.Xa=Nd({imports:[ry,ht,jw,$v]});var IG=class extends lt{constructor(){super(...arguments)}};function pt(a,b){return"window"===b?window:"document"===b?a:"body"===b?a.body:null}function JG(a,b,c){a.addEventListener(b,c);return()=>{a.removeEventListener(b,c)}}var KG=class extends IG{dispatchEvent(a,b){a.dispatchEvent(b)}remove(a){a.parentNode&&a.parentNode.removeChild(a)}createElement(a,b){b=b||document;return b.createElement(a)}};let yt=null;function En(a){ef.getAngularTestability=(b,c=!0)=>{b=a.fl(b,c);if(null==b)throw new Zd(5103,!1);return b};ef.getAllAngularTestabilities=()=>Array.from(a.C.values());ef.getAllAngularRootElements=()=>Array.from(a.C.keys());ef.frameworkStabilizers||(ef.frameworkStabilizers=[]);ef.frameworkStabilizers.push(b=>{function c(){e--;0==e&&b()}const d=ef.getAllAngularTestabilities();let e=d.length;d.forEach(f=>{f.whenStable(c)})})}
var LG=class{fl(a,b,c){if(null==b)return null;const d=a.C.get(b)||null;if(null!=d)a=d;else{var e;c?e=b instanceof DocumentFragment?this.fl(a,b.host,!0):this.fl(a,b.parentElement,!0):e=null;a=e}return a}};var MG=class{D(){return new XMLHttpRequest}};MG.K=function(a){return new (a||MG)};MG.pa=Md({ma:MG,ba:MG.K});var NG=new df(""),OG=class{constructor(a,b){this.yb=b;this.C=new Map;a.forEach(c=>{c.manager=this});this.F=a.slice().reverse()}addEventListener(a,b,c){return this.D(b).addEventListener(a,b,c)}D(a){let b=this.C.get(a);if(b)return b;b=this.F.find(c=>c.supports(a));if(!b)throw new Zd(5101,!1);this.C.set(a,b);return b}};OG.K=function(a){return new (a||OG)(r(NG),r(Qk))};OG.pa=Md({ma:OG,ba:OG.K});var PG=class{};function QG(a){let b;const c=null==(b=a.F.head)?void 0:b.querySelectorAll(`style[${"ng-app-id"}="${a.G}"]`);if(null==c?0:c.length){const d=new Map;c.forEach(e=>{null!=e.textContent&&d.set(e.textContent,e)});return d}return null}function RG(a){const b=a.D;b.clear();b.add(a.F.head)}function SG(a,b,c){a=a.C;if(a.has(b))return b=a.get(b),b.usage+=c,b.usage;a.set(b,{usage:c,elements:[]});return c}function TG(a,b){for(const c of b)if(1===SG(a,c,1)){b=a;for(const d of b.D)UG(b,d,c)}}
function VG(a,b){a=a.C;let c,d;null==(c=a.get(b))||null==(d=c.elements)||d.forEach(e=>e.remove());a.delete(b)}function UG(a,b,c){{var d=a.J;const f=null==d?void 0:d.get(c);(null==f?void 0:f.parentNode)===b?(d.delete(c),f.removeAttribute("ng-app-id"),b=f):(d=a.F.createElement("style"),a.nonce&&d.setAttribute("nonce",a.nonce),d.textContent=c,a.N&&d.setAttribute("ng-app-id",a.G),b.appendChild(d),b=d)}a=a.C;let e;(d=null==(e=a.get(c))?void 0:e.elements)?d.push(b):a.set(c,{elements:[b],usage:1})}
var WG=class{constructor(a,b,c,d={}){this.F=a;this.G=b;this.nonce=c;this.C=new Map;this.D=new Set;this.J=QG(this);this.N="server"===d;RG(this)}va(){const a=this.J;a&&(a.forEach(b=>b.remove()),a.clear());for(const b of this.C.keys())VG(this,b);RG(this)}};WG.K=function(a){return new (a||WG)(r(jt),r(ji),r(ni,8),r(li))};WG.pa=Md({ma:WG,ba:WG.K});var XG={svg:"http://www.w3.org/2000/svg",xhtml:"http://www.w3.org/1999/xhtml",xlink:"http://www.w3.org/1999/xlink",xml:"http://www.w3.org/XML/1998/namespace",xmlns:"http://www.w3.org/2000/xmlns/",math:"http://www.w3.org/1998/Math/MathML"};const YG=/%COMP%/g;var ZG=new df("",{ea:"root",ba:()=>!0});function $G(a,b){return b.map(c=>c.replace(YG,a))}
var eH=class{constructor(a,b,c,d,e,f,g,k=null){this.J=a;this.P=b;this.F=c;this.O=d;this.G=e;this.N=g;this.nonce=k;this.D=new Map;this.C="server"===f;this.defaultRenderer=new aH(a,e,g,this.C)}Uk(a,b){if(!a||!b)return this.defaultRenderer;this.C&&3===b.pb&&(b=Object.assign({},b,{pb:0}));a:{const d=this.D;var c=d.get(b.id);if(!c){c=this.G;const e=this.N,f=this.J,g=this.P,k=this.O,m=this.C;switch(b.pb){case 0:c=new bH(f,g,b,this.F,k,c,e,m);break;case 3:b=new cH(f,g,a,b,c,e,this.nonce,m);break a;default:c=
new dH(f,g,b,k,c,e,m)}d.set(b.id,c)}b=c}b instanceof bH?(TG(b.D,b.styles),b.setAttribute(a,b.P,"")):b instanceof dH&&TG(b.D,b.styles);return b}va(){this.D.clear()}};eH.K=function(a){return new (a||eH)(r(OG),r(WG),r(ji),r(ZG),r(jt),r(li),r(Qk),r(ni))};eH.pa=Md({ma:eH,ba:eH.K});function fH(a,b){return c=>{if("__ngUnwrap__"===c)return b;!1===(a.J?a.G.runGuarded(()=>b(c)):b(c))&&c.preventDefault()}}
class aH{constructor(a,b,c,d){this.F=a;this.C=b;this.G=c;this.J=d;this.data=Object.create(null);this.Xk=null}destroy(){}createElement(a,b){return b?this.C.createElementNS(XG[b]||b,a):this.C.createElement(a)}createComment(a){return this.C.createComment(a)}lq(a){return this.C.createTextNode(a)}appendChild(a,b){("TEMPLATE"===a.tagName&&void 0!==a.content?a.content:a).appendChild(b)}insertBefore(a,b,c){a&&("TEMPLATE"===a.tagName&&void 0!==a.content?a.content:a).insertBefore(b,c)}removeChild(a,b){a&&a.removeChild(b)}ns(a,
b){a="string"===typeof a?this.C.querySelector(a):a;if(!a)throw new Zd(-5104,!1);b||(a.textContent="");return a}parentNode(a){return a.parentNode}nextSibling(a){return a.nextSibling}setAttribute(a,b,c,d){d?(b=d+":"+b,(d=XG[d])?a.setAttributeNS(d,b,c):a.setAttribute(b,c)):a.setAttribute(b,c)}removeAttribute(a,b,c){if(c){const d=XG[c];d?a.removeAttributeNS(d,b):a.removeAttribute(`${c}:${b}`)}else a.removeAttribute(b)}Ri(a,b){a.classList.add(b)}wo(a,b){a.classList.remove(b)}setStyle(a,b,c,d){d&3?a.style.setProperty(b,
c,d&1?"important":""):a.style[b]=c}Zr(a,b,c){c&2?a.style.removeProperty(b):a.style[b]=""}setProperty(a,b,c){null!=a&&(a[b]=c)}setValue(a,b){a.nodeValue=b}Gd(a,b,c){if("string"===typeof a&&(a=pt(this.C,a),!a))throw Error(`Unsupported event target ${a} for event ${b}`);return this.F.addEventListener(a,b,fH(this,c))}}function gH(a,b){return b===a.N?a.shadowRoot:b}
class cH extends aH{constructor(a,b,c,d,e,f,g,k){super(a,e,f,k);this.D=b;this.N=c;this.shadowRoot=c.attachShadow({mode:"open"});a=this.D;b=this.shadowRoot;a.D.add(b);for(const m of a.C.keys())UG(a,b,m);d=$G(d.id,d.styles);for(const m of d)d=document.createElement("style"),g&&d.setAttribute("nonce",g),d.textContent=m,this.shadowRoot.appendChild(d)}appendChild(a,b){return super.appendChild(gH(this,a),b)}insertBefore(a,b,c){return super.insertBefore(gH(this,a),b,c)}removeChild(a,b){return super.removeChild(gH(this,
a),b)}parentNode(a){return gH(this,super.parentNode(gH(this,a)))}destroy(){this.D.D.delete(this.shadowRoot)}}class dH extends aH{constructor(a,b,c,d,e,f,g,k){super(a,e,f,g);this.D=b;this.N=d;this.styles=k?$G(k,c.styles):c.styles}destroy(){if(this.N){var a=this.D,b=this.styles;for(const c of b)0>=SG(a,c,-1)&&VG(a,c)}}}
class bH extends dH{constructor(a,b,c,d,e,f,g,k){d=d+"-"+c.id;super(a,b,c,e,f,g,k,d);this.O="_ngcontent-%COMP%".replace(YG,d);this.P="_nghost-%COMP%".replace(YG,d)}createElement(a,b){a=super.createElement(a,b);super.setAttribute(a,this.O,"");return a}};var hH=class extends PG{supports(){return!0}addEventListener(a,b,c){a.addEventListener(b,c,!1);return()=>this.removeEventListener(a,b,c)}removeEventListener(a,b,c){return a.removeEventListener(b,c)}};hH.K=function(a){return new (a||hH)(r(jt))};hH.pa=Md({ma:hH,ba:hH.K});const iH=["alt","control","meta","shift"],jH={"\b":"Backspace","\t":"Tab","\u007f":"Delete","\u001b":"Escape",Del:"Delete",Esc:"Escape",Left:"ArrowLeft",Right:"ArrowRight",Up:"ArrowUp",Down:"ArrowDown",Menu:"ContextMenu",Scroll:"ScrollLock",Win:"OS"},kH={alt:a=>a.altKey,control:a=>a.ctrlKey,meta:a=>a.metaKey,shift:a=>a.shiftKey};
function lH(a){const b=a.toLowerCase().split(".");a=b.shift();if(0===b.length||"keydown"!==a&&"keyup"!==a)return null;const c=mH(b.pop());let d="",e=b.indexOf("code");-1<e&&(b.splice(e,1),d="code.");iH.forEach(f=>{const g=b.indexOf(f);-1<g&&(b.splice(g,1),d+=f+".")});d+=c;if(0!=b.length||0===c.length)return null;a={domEventName:a};a.fullKey=d;return a}function nH(a,b,c){return d=>{oH(d,a)&&c.runGuarded(()=>b(d))}}function mH(a){return"esc"===a?"escape":a}
function oH(a,b){let c=jH[a.key]||a.key,d="";-1<b.indexOf("code.")&&(c=a.code,d="code.");if(null==c||!c)return!1;c=c.toLowerCase();" "===c?c="space":"."===c&&(c="dot");iH.forEach(e=>{e!==c&&(0,kH[e])(a)&&(d+=e+".")});d+=c;return d===b}var pH=class extends PG{supports(a){return null!=lH(a)}addEventListener(a,b,c){const d=lH(b),e=nH(d.fullKey,c,this.manager.yb);return Mk(this.manager.yb,()=>JG(a,d.domEventName,e))}};pH.K=function(a){return new (a||pH)(r(jt))};pH.pa=Md({ma:pH,ba:pH.K});var qH=[{oa:li,nb:"browser"},{oa:ki,nb:function(){var a=new KG;null!=kt||(kt=a)},Eb:!0},{oa:jt,Fc:function(){return di=document},Ac:[]}];us(Ls,"browser",qH);const rH=new df(""),sH=[{oa:An,Ec:LG,Ac:[]},{oa:zn,Ec:Fn,Ac:[Qk,Gn,An]},{oa:Fn,Ec:Fn,Ac:[Qk,Gn,An]}],tH=[{oa:fg,nb:"root"},{oa:wi,Fc:function(){return new wi},Ac:[]},{oa:NG,Ec:hH,Eb:!0,Ac:[jt,Qk,li]},{oa:NG,Ec:pH,Eb:!0,Ac:[jt]},eH,WG,OG,{oa:ui,Ab:eH},{oa:Ft,Ec:MG,Ac:[]},[]];var uH=class{};uH.K=function(a){return new (a||uH)(r(rH,12))};uH.Ya=Of({type:uH});
uH.Xa=Nd({Wa:[...tH,...sH],imports:[ht,Rr]});var vH=class{constructor(a){this.C=a}};vH.K=function(a){return new (a||vH)(r(jt))};vH.pa=Md({ma:vH,ba:vH.K,ea:"root"});new df("HammerGestureConfig");new df("HammerLoader");var wH=class{};function xH(a,b,c){a.Uf.has(c)||a.Uf.set(c,b)}function yH(a,b,c){c=(Array.isArray(c)?c:[c]).map(e=>e.toString());const d=b.toLowerCase();a.headers.set(d,c);xH(a,b,d)}function zH(a,b){b.init();Array.from(b.headers.keys()).forEach(c=>{a.headers.set(c,b.headers.get(c));a.Uf.set(c,b.Uf.get(c))})}
function AH(a,b){const c=b.name.toLowerCase();switch(b.op){case "a":case "s":var d=b.value;"string"===typeof d&&(d=[d]);if(0===d.length)break;xH(a,b.name,c);b=("a"===b.op?a.headers.get(c):void 0)||[];b.push(...d);a.headers.set(c,b);break;case "d":const e=b.value;if(e){if(d=a.headers.get(c))d=d.filter(f=>-1===e.indexOf(f)),0===d.length?(a.headers.delete(c),a.Uf.delete(c)):a.headers.set(c,d)}else a.headers.delete(c),a.Uf.delete(c)}}
var BH=class{constructor(a){this.Uf=new Map;this.yl=null;a?"string"===typeof a?this.ff=()=>{this.headers=new Map;a.split("\n").forEach(b=>{const c=b.indexOf(":");if(0<c){const d=b.slice(0,c),e=d.toLowerCase();b=b.slice(c+1).trim();xH(this,d,e);this.headers.has(e)?this.headers.get(e).push(b):this.headers.set(e,[b])}})}:"undefined"!==typeof Headers&&a instanceof Headers?(this.headers=new Map,a.forEach((b,c)=>{yH(this,c,b)})):this.ff=()=>{this.headers=new Map;Object.entries(a).forEach(([b,c])=>{yH(this,
b,c)})}:this.headers=new Map}has(a){this.init();return this.headers.has(a.toLowerCase())}get(a){this.init();return(a=this.headers.get(a.toLowerCase()))&&0<a.length?a[0]:null}keys(){this.init();return Array.from(this.Uf.values())}getAll(a){this.init();return this.headers.get(a.toLowerCase())||null}append(a,b){return this.clone({name:a,value:b,op:"a"})}set(a,b){return this.clone({name:a,value:b,op:"s"})}delete(a,b){return this.clone({name:a,value:b,op:"d"})}init(){this.ff&&(this.ff instanceof BH?zH(this,
this.ff):this.ff(),this.ff=null,this.yl&&(this.yl.forEach(a=>AH(this,a)),this.yl=null))}clone(a){const b=new BH;b.ff=this.ff&&this.ff instanceof BH?this.ff:this;b.yl=(this.yl||[]).concat([a]);return b}forEach(a){this.init();Array.from(this.Uf.keys()).forEach(b=>a(this.Uf.get(b),this.headers.get(b)))}};var CH=class{};function DH(a){const b=new Map;0<a.length&&a.replace(/^\?/,"").split("&").forEach(c=>{const d=c.indexOf("="),[e,f]=-1==d?[decodeURIComponent(c),""]:[decodeURIComponent(c.slice(0,d)),decodeURIComponent(c.slice(d+1))];c=b.get(e)||[];c.push(f);b.set(e,c)});return b}const EH=/%(\d[a-f0-9])/gi,FH={40:"@","3A":":",24:"$","2C":",","3B":";","3D":"=","3F":"?","2F":"/"};function GH(a){return encodeURIComponent(a).replace(EH,(b,c)=>{let d;return null!=(d=FH[c])?d:b})}
function HH(a){return`${a}`}
var IH=class{constructor(a={}){this.C=this.D=null;this.Bq=a.Bq||new CH;if(a.TB){if(a.Bn)throw Error("Cannot specify both fromString and fromObject.");this.map=DH(a.TB)}else a.Bn?(this.map=new Map,Object.keys(a.Bn).forEach(b=>{var c=a.Bn[b];c=Array.isArray(c)?c.map(HH):[`${c}`];this.map.set(b,c)})):this.map=null}has(a){this.init();return this.map.has(a)}get(a){this.init();return(a=this.map.get(a))?a[0]:null}getAll(a){this.init();return this.map.get(a)||null}keys(){this.init();return Array.from(this.map.keys())}append(a,
b){return this.clone({Hg:a,value:b,op:"a"})}set(a,b){return this.clone({Hg:a,value:b,op:"s"})}delete(a,b){return this.clone({Hg:a,value:b,op:"d"})}toString(){this.init();return this.keys().map(a=>{const b=GH(a);return this.map.get(a).map(c=>b+"="+GH(c)).join("&")}).filter(a=>""!==a).join("&")}clone(a){const b=new IH({Bq:this.Bq});b.C=this.C||this;b.D=(this.D||[]).concat(a);return b}init(){null===this.map&&(this.map=new Map);null!==this.C&&(this.C.init(),this.C.keys().forEach(a=>this.map.set(a,this.C.map.get(a))),
this.D.forEach(a=>{switch(a.op){case "a":case "s":var b=("a"===a.op?this.map.get(a.Hg):void 0)||[];b.push(`${a.value}`);this.map.set(a.Hg,b);break;case "d":if(void 0!==a.value){b=this.map.get(a.Hg)||[];const c=b.indexOf(`${a.value}`);-1!==c&&b.splice(c,1);0<b.length?this.map.set(a.Hg,b):this.map.delete(a.Hg)}else this.map.delete(a.Hg)}}),this.C=this.D=null)}};var JH=class{constructor(){this.map=new Map}set(a,b){this.map.set(a,b);return this}get(a){this.map.has(a)||this.map.set(a,a.defaultValue());return this.map.get(a)}delete(a){this.map.delete(a);return this}has(a){return this.map.has(a)}keys(){return this.map.keys()}};var KH=class{constructor(a,b,c,d){this.url=b;this.body=null;this.withCredentials=this.ei=!1;this.responseType="json";this.method=a.toUpperCase();a:switch(this.method){case "DELETE":case "GET":case "HEAD":case "OPTIONS":case "JSONP":a=!1;break a;default:a=!0}if(a||d)this.body=void 0!==c?c:null,c=d;c&&(this.ei=!!c.ei,this.withCredentials=!!c.withCredentials,c.responseType&&(this.responseType=c.responseType),c.headers&&(this.headers=c.headers),c.context&&(this.context=c.context),c.params&&(this.params=
c.params),this.Vj=c.Vj);null!=this.headers||(this.headers=new BH);null!=this.context||(this.context=new JH);this.params||(this.params=new IH)}clone(a={}){const b=a.method||this.method,c=a.url||this.url,d=a.responseType||this.responseType;var e;const f=null!=(e=a.Vj)?e:this.Vj;e=void 0!==a.body?a.body:this.body;var g;const k=null!=(g=a.withCredentials)?g:this.withCredentials;var m;g=null!=(m=a.ei)?m:this.ei;m=a.headers||this.headers;let p=a.params||this.params,t;const w=null!=(t=a.context)?t:this.context;
void 0!==a.Ix&&(m=Object.keys(a.Ix).reduce((B,G)=>B.set(G,a.Ix[G]),m));a.Jx&&(p=Object.keys(a.Jx).reduce((B,G)=>B.set(G,a.Jx[G]),p));return new KH(b,c,e,{params:p,headers:m,context:w,ei:g,responseType:d,withCredentials:k,Vj:f})}};function LH(a,b,c,d={}){if(b instanceof KH)c=b;else{var e=void 0;e=d.headers instanceof BH?d.headers:new BH(d.headers);let f=void 0;d.params&&(f=d.params instanceof IH?d.params:new IH({Bn:d.params}));c=new KH(b,c,void 0!==d.body?d.body:null,{headers:e,context:d.context,params:f,ei:d.ei,responseType:d.responseType||"json",withCredentials:d.withCredentials,Vj:d.Vj})}e=C(jj(c),Tj(f=>a.handler.handle(f),1));if(b instanceof KH||"events"===d.observe)return e;b=C(e,tk(()=>!1));switch(d.observe||"body"){case "body":switch(c.responseType){case "arraybuffer":return C(b,
xj(f=>{if(null!==f.body&&!(f.body instanceof ArrayBuffer))throw Error("Response is not an ArrayBuffer.");return f.body}));case "blob":return C(b,xj(f=>{if(null!==f.body&&!(f.body instanceof Blob))throw Error("Response is not a Blob.");return f.body}));case "text":return C(b,xj(f=>{if(null!==f.body&&"string"!==typeof f.body)throw Error("Response is not a string.");return f.body}));default:return C(b,xj(f=>f.body))}case "response":return b;default:throw Error(`Unreachable: unhandled observe type ${d.observe}}`);
}}var MH=class{constructor(a){this.handler=a}delete(a,b={}){return LH(this,"DELETE",a,b)}get(a,b={}){return LH(this,"GET",a,b)}head(a,b={}){return LH(this,"HEAD",a,b)}options(a,b={}){return LH(this,"OPTIONS",a,b)}};MH.K=function(a){return new (a||MH)(r(wH))};MH.pa=Md({ma:MH,ba:MH.K});new df("");new df("");new df("");new df("",{ea:"root",ba:()=>!0});new df("");new df("",{ea:"root",ba:()=>"XSRF-TOKEN"});new df("",{ea:"root",ba:()=>"X-XSRF-TOKEN"});new df("");new df("");new df("");var NH=class{};NH.K=function(a){return new (a||NH)};NH.pa=Md({ma:NH,ba:function(a){return a?new (a||NH):r(OH)},ea:"root"});
var OH=class extends NH{constructor(a){super();this.D=a}C(a,b){if(null==b)return null;switch(a){case 0:return b;case 1:if(Cl(b,"HTML"))var c=Bl(b);else{a:{a=this.D;var d=String(b);b=null;try{var e;if(!(e=sm)){var f=new Hl(a);try{var g=!!(new window.DOMParser).parseFromString(Fl(""),"text/html")}catch(t){g=!1}e=g?new Gl(f):f}sm=e;var k=d?String(d):"";b=sm.Dn(k);e=5;g=k;do{if(0===e)throw Error("Failed to sanitize html because the input is unstable");e--;k=g;g=b.innerHTML;b=sm.Dn(k)}while(k!==g);var m=
new Yl;let p=(tm(b)||b).firstChild;k=!0;for(e=[];p;){if(p.nodeType===Node.ELEMENT_NODE)b:{g=m;f=p;const t=$l(f).toLowerCase();if(!Sl.hasOwnProperty(t)){k=!Xl.hasOwnProperty(t);break b}g.C.push("<");g.C.push(t);const w=f.attributes;for(f=0;f<w.length;f++){const B=w.item(f),G=B.name,J=G.toLowerCase();if(!Wl.hasOwnProperty(J))continue;let N=B.value;Tl[J]&&(N=Jl(N));g.C.push(" ",G,'="',Zl(N),'"')}g.C.push(">");k=!0}else p.nodeType===Node.TEXT_NODE&&m.chars(p.nodeValue);if(k&&p.firstChild){e.push(p);const t=
p.firstChild;if(t&&(p.compareDocumentPosition(t)&Node.DOCUMENT_POSITION_CONTAINED_BY)!==Node.DOCUMENT_POSITION_CONTAINED_BY)throw am(t);p=t}else for(;p;){if(p.nodeType===Node.ELEMENT_NODE){g=m;const w=$l(p).toLowerCase();Sl.hasOwnProperty(w)&&!Ml.hasOwnProperty(w)&&(g.C.push("</"),g.C.push(w),g.C.push(">"))}const t=p.nextSibling;if(t&&p!==t.previousSibling)throw am(t);if(g=t){p=g;break}p=e.pop()}}c=Fl(m.C.join(""));break a}finally{if(b)for(m=tm(b)||b;m.firstChild;)m.removeChild(m.firstChild)}c=void 0}c=
c.toString()}return c;case 2:return Cl(b,"Style")?Bl(b):b;case 3:if(Cl(b,"Script"))return Bl(b);throw new Zd(5200,!1);case 4:return Cl(b,"URL")?Bl(b):Jl(String(b));case 5:if(Cl(b,"ResourceURL"))return Bl(b);throw new Zd(5201,!1);default:throw new Zd(5202,!1);}}G(a){return new zl(a)}F(a){return new Al(a)}};OH.K=function(a){return new (a||OH)(r(jt))};OH.pa=Md({ma:OH,ba:OH.K,ea:"root"});function PH(a,...b){if(0===b.length)return id(a[0]);let c=a[0];for(let d=0;d<b.length;d++)c+=encodeURIComponent(b[d])+a[d+1];return id(c)};function QH(a,b){switch(b){case 1:return!1;case 4:return a instanceof jd||a instanceof bd;case 5:return a instanceof bd;case 3:return!1;default:return!1}}
var RH=class extends NH{constructor(a){super();this.D=new OH(a)}C(a,b){if(null==b)return null;if(b instanceof bd||b instanceof jd){if(QH(b,a)){var c=this.D,d=c.C;a:{if(null!=b){if(b instanceof jd){b=this.D.G(kd(b));break a}if(b instanceof bd){b=this.D.F(String(b));break a}}b=null}return d.call(c,a,b)}throw Error(`Required a safe value for ${um[a]}, got ${b} `+"(see http://g.co/ng/security#xss)");}return this.D.C(a,b)}G(){throw Error("bypassSecurityTrust* functions are prohibited in the google3 version of Angular. Use LegacyDomSanitizer if you need to disable auto sanitization and get a security from ise-hardening-reviews.");
}F(){throw Error("bypassSecurityTrust* functions are prohibited in the google3 version of Angular. Use LegacyDomSanitizer if you need to disable auto sanitization and get a security from ise-hardening-reviews.");}};RH.K=function(a){return new (a||RH)(r(jt))};RH.pa=Md({ma:RH,ba:RH.K});const SH=[{oa:en,Ab:NH},{oa:NH,Ec:RH}];var TH=class{};TH.K=function(a){return new (a||TH)};TH.Ya=Of({type:TH});TH.Xa=Nd({Wa:SH,imports:[uH]});let UH;function VH(){if(void 0===UH&&(UH=null,"undefined"!==typeof window)){const a=window;void 0!==a.trustedTypes&&(UH=a.trustedTypes.createPolicy("angular#components",{createHTML:b=>b}))}return UH}function WH(a){let b;return(null==(b=VH())?void 0:b.createHTML(a))||a};function XH(a){return Error(`Unable to find icon with the name "${a}"`)}class YH{constructor(a,b){this.url=a;this.Ug=null;this.options=b}}function ZH(a,b){return a.wa.get(b)||b}function $H(a,b,c=""){const d=c+":"+b;let e=a.F.get(d);return e?a.aa(e):(e=a.Ea(c,b))?(a.F.set(d,e),a.aa(e)):(c=a.C.get(c))?a.Ja(b,c):kj(XH(d))}
var aI=class{constructor(a,b,c,d){this.ca=a;this.fa=b;this.ja=d;this.F=new Map;this.C=new Map;this.na=new Map;this.G=new Map;this.wa=new Map;this.J=[];this.sa=["material-icons","mat-ligature-font"];this.ia=c}va(){this.J=[];this.F.clear();this.C.clear();this.na.clear()}aa(a){return a.Ug?jj(this.N(a).cloneNode(!0)):C(this.Pa(a),xj(b=>b.cloneNode(!0)))}Ja(a,b){var c=this.P(a,b);if(c)return jj(c);c=b.filter(d=>!d.Ug).map(d=>C(this.Sa(d),Mn(e=>{e=`Loading icon set URL: ${this.fa.C(5,d.url)} failed: ${e.message}`;
this.ja.handleError(Error(e));return jj(null)})));return C(jk(c),xj(()=>{const d=this.P(a,b);if(!d)throw XH(a);return d}))}P(a,b){for(let d=b.length-1;0<=d;d--){var c=b[d];if(c.Ug&&-1<c.Ug.toString().indexOf(a)){const e=this.N(c);if(c=this.ua(e,a,c.options))return c}}return null}Pa(a){return C(this.T(a),Po(b=>a.Ug=b),xj(()=>this.N(a)))}Sa(a){return a.Ug?jj(null):C(this.T(a),Po(b=>a.Ug=b))}ua(a,b,c){a=a.querySelector(`[id="${b}"]`);if(!a)return null;a=a.cloneNode(!0);a.removeAttribute("id");if("svg"===
a.nodeName.toLowerCase())return this.D(a,c);if("symbol"===a.nodeName.toLowerCase())return this.D(this.Va(a),c);b=this.O(WH("<svg></svg>"));b.appendChild(a);return this.D(b,c)}O(a){const b=this.ia.createElement("DIV");b.innerHTML=a;a=b.querySelector("svg");if(!a)throw Error("<svg> tag not found");return a}Va(a){const b=this.O(WH("<svg></svg>"));var c=a.attributes;for(let d=0;d<c.length;d++){const {name:e,value:f}=c[d];"id"!==e&&b.setAttribute(e,f)}for(c=0;c<a.childNodes.length;c++)a.childNodes[c].nodeType===
this.ia.ELEMENT_NODE&&b.appendChild(a.childNodes[c].cloneNode(!0));return b}D(a,b){a.setAttribute("fit","");a.setAttribute("height","100%");a.setAttribute("width","100%");a.setAttribute("preserveAspectRatio","xMidYMid meet");a.setAttribute("focusable","false");b&&b.viewBox&&a.setAttribute("viewBox",b.viewBox);return a}T(a){var b=a.url;let c,d;a=null!=(d=null==(c=a.options)?void 0:c.withCredentials)?d:!1;if(!this.ca)throw Error("Could not find HttpClient provider for use with Angular Material icons. Please include the HttpClientModule from @angular/common/http in your app imports.");
if(null==b)throw Error(`Cannot fetch icon from URL "${b}".`);const e=this.fa.C(5,b);if(!e)throw Error("The URL provided to MatIconRegistry was not trusted as a resource URL "+`via Angular's DomSanitizer. Attempted URL was "${b}".`);if(b=this.G.get(e))return b;b=C(this.ca.get(e,{responseType:"text",withCredentials:a}),xj(f=>WH(f)),mo(()=>this.G.delete(e)),Bo());this.G.set(e,b);return b}bb(a,b,c){this.F.set(a+":"+b,c);return this}gb(a,b){const c=this.C.get(a);c?c.push(b):this.C.set(a,[b]);return this}N(a){if(!a.jy){const b=
this.O(a.Ug);this.D(b,a.options);a.jy=b}return a.jy}Ea(a,b){for(let c=0;c<this.J.length;c++){const d=this.J[c](b,a);if(d)return d.url&&d.options?new YH(d.url,d.options):new YH(d)}}};aI.K=function(a){return new (a||aI)(r(MH,8),r(NH),r(jt,8),r(wi))};aI.pa=Md({ma:aI,ba:aI.K,ea:"root"});new Rh;new Sh;new Rh;new Rh;var bI=new df("MAT_ICON_DEFAULT_OPTIONS"),dI=new df("mat-icon-location",{ea:"root",ba:cI});function cI(){const a=lf(jt),b=a?a.location:null;return{Wv:()=>b?b.pathname+b.search:""}}const eI="clip-path color-profile src cursor fill filter marker marker-start marker-mid marker-end mask stroke".split(" "),fI=eI.map(a=>`[${a}]`).join(", "),gI=/^url\(['"]?#(.*?)['"]?\)$/;
var hI=class{get color(){return this.mh||this.sa}set color(a){this.mh=a}get Cs(){return this.J}set Cs(a){a!==this.J&&(a?this.Pa(a):this.J&&this.T(),this.J=a)}get yg(){return this.ca}set yg(a){a=this.P(a);a!==this.ca&&(this.ca=a,this.N())}get xg(){return this.aa}set xg(a){a=this.P(a);a!==this.aa&&(this.aa=a,this.N())}constructor(a,b,c,d,e,f){this.ra=a;this.F=b;this.O=d;this.ua=e;this.inline=!1;this.ja=[];this.D=Ni.EMPTY;f&&(f.color&&(this.color=this.sa=f.color),f.yg&&(this.yg=f.yg));c||a.R.setAttribute("aria-hidden",
"true")}Ja(a){if(!a)return["",""];const b=a.split(":");switch(b.length){case 1:return["",b[0]];case 2:return b;default:throw Error(`Invalid icon name: "${a}"`);}}Fb(){this.N()}Lw(){var a=this.C;a&&a.size&&(a=this.O.Wv(),a!==this.na&&(this.na=a,this.fa(a)))}va(){this.D.unsubscribe();this.C&&this.C.clear()}Tp(){return!this.Cs}Ea(a){this.T();const b=this.O.Wv();this.na=b;this.wa(a);this.fa(b);this.ra.R.appendChild(a)}T(){const a=this.ra.R;let b=a.childNodes.length;for(this.C&&this.C.clear();b--;){const c=
a.childNodes[b];1===c.nodeType&&"svg"!==c.nodeName.toLowerCase()||c.remove()}}N(){if(this.Tp()){var a=this.ra.R,b=(this.yg?ZH(this.F,this.yg).split(/ +/):this.F.sa).filter(c=>0<c.length);this.ja.forEach(c=>a.classList.remove(c));b.forEach(c=>a.classList.add(c));this.ja=b;this.xg===this.G||b.includes("mat-ligature-font")||(this.G&&a.classList.remove(this.G),this.xg&&a.classList.add(this.xg),this.G=this.xg)}}P(a){return"string"===typeof a?a.trim().split(" ")[0]:a}fa(a){const b=this.C;b&&b.forEach((c,
d)=>{c.forEach(e=>{d.setAttribute(e.name,`url('${a}#${e.value}')`)})})}wa(a){const b=a.querySelectorAll(fI),c=this.C=this.C||new Map;for(let d=0;d<b.length;d++)eI.forEach(e=>{const f=b[d];var g=f.getAttribute(e);if(g=g?g.match(gI):null){let k=c.get(f);k||(k=[],c.set(f,k));k.push({name:e,value:g[1]})}})}Pa(a){this.ou=this.pu=null;this.D.unsubscribe();if(a){const [b,c]=this.Ja(a);b&&(this.pu=b);c&&(this.ou=c);this.D=C($H(this.F,c,b),fo()).subscribe(d=>this.Ea(d),d=>{this.ua.handleError(Error(`Error retrieving icon ${b}:${c}! ${d.message}`))})}}};
hI.K=function(a){return new (a||hI)(z(si),z(aI),yh("aria-hidden"),z(dI),z(wi),z(bI,8))};
hI.Da=Gf({type:hI,ha:[["mat-icon"]],Ka:["role","img",1,"mat-icon","notranslate"],Za:10,Ia:function(a,b){a&2&&(Kp("data-mat-icon-type",b.Tp()?"font":"svg")("data-mat-icon-name",b.ou||b.xg)("data-mat-icon-namespace",b.pu||b.yg)("fontIcon",b.Tp()?b.xg:null),qq(tq,b.color?"mat-"+b.color:""),nq("mat-icon-inline",b.inline)("mat-icon-no-color","primary"!==b.color&&"accent"!==b.color&&"warn"!==b.color))},inputs:{color:"color",inline:[2,"inline","inline",zs],Cs:"svgIcon",yg:"fontSet",xg:"fontIcon"},ib:["matIcon"],
ka:!0,features:[Jp,Mr],Zb:["*"],za:1,Aa:0,ta:function(a){a&1&&(Xq(),Yq(0))},styles:["mat-icon,mat-icon.mat-primary,mat-icon.mat-accent,mat-icon.mat-warn{color:var(--mat-icon-color)}.mat-icon{-webkit-user-select:none;user-select:none;background-repeat:no-repeat;display:inline-block;fill:currentColor;height:24px;width:24px;overflow:hidden}.mat-icon.mat-icon-inline{font-size:inherit;height:inherit;line-height:inherit;width:inherit}.mat-icon.mat-ligature-font[fontIcon]::before{content:attr(fontIcon)}[dir=rtl] .mat-icon-rtl-mirror{transform:scale(-1, 1)}.mat-form-field:not(.mat-form-field-appearance-legacy) .mat-form-field-prefix .mat-icon,.mat-form-field:not(.mat-form-field-appearance-legacy) .mat-form-field-suffix .mat-icon{display:block}.mat-form-field:not(.mat-form-field-appearance-legacy) .mat-form-field-prefix .mat-icon-button .mat-icon,.mat-form-field:not(.mat-form-field-appearance-legacy) .mat-form-field-suffix .mat-icon-button .mat-icon{margin:auto}\n"],
pb:2,Ra:0});var iI=class{};iI.K=function(a){return new (a||iI)};iI.Ya=Of({type:iI});iI.Xa=Nd({imports:[mz,mz]});function jI(a){if(a&1){const b=Ug();R(0,"button",1);V("click",function(){u(b);Y();var c=new URL(window.location.href),d=new URLSearchParams(c.search);d.delete("show_open_in_new_tab");c.search=d.toString();d=window;c=rd(c.toString());c=td(c);void 0!==c&&d.open(c,"_blank",void 0);return x()});R(1,"mat-icon");Z(2,"open_in_new");S();Z(3," Open in new tab ");S()}}var kI=class{constructor(){this.Co="1"===(new URLSearchParams(window.location.search)).get("show_open_in_new_tab")}};
kI.K=function(a){return new (a||kI)};kI.Da=Gf({type:kI,ha:[["open-in-new-tab-button"]],ka:!0,features:[Mr],za:1,Aa:1,Ca:[["mat-flat-button","","color","primary",1,"btn-open-in-tab"],["mat-flat-button","","color","primary",1,"btn-open-in-tab",3,"click"]],ta:function(a,b){a&1&&Ap(0,jI,4,0,"button",0);a&2&&Eq(b.Co?0:-1)},Ha:[ht,BB,AB,vB,iI,hI]});var lI={aliceblue:"#f0f8ff",antiquewhite:"#faebd7",aqua:"#00ffff",aquamarine:"#7fffd4",azure:"#f0ffff",beige:"#f5f5dc",bisque:"#ffe4c4",black:"#000000",blanchedalmond:"#ffebcd",blue:"#0000ff",blueviolet:"#8a2be2",brown:"#a52a2a",burlywood:"#deb887",cadetblue:"#5f9ea0",chartreuse:"#7fff00",chocolate:"#d2691e",coral:"#ff7f50",cornflowerblue:"#6495ed",cornsilk:"#fff8dc",crimson:"#dc143c",cyan:"#00ffff",darkblue:"#00008b",darkcyan:"#008b8b",darkgoldenrod:"#b8860b",darkgray:"#a9a9a9",darkgreen:"#006400",
darkkhaki:"#bdb76b",darkmagenta:"#8b008b",darkolivegreen:"#556b2f",darkorange:"#ff8c00",darkorchid:"#9932cc",darkred:"#8b0000",darksalmon:"#e9967a",darkseagreen:"#8fbc8f",darkslateblue:"#483d8b",darkslategray:"#2f4f4f",darkturquoise:"#00ced1",darkviolet:"#9400d3",deeppink:"#ff1493",deepskyblue:"#00bfff",dimgray:"#696969",dodgerblue:"#1e90ff",firebrick:"#b22222",floralwhite:"#fffaf0",forestgreen:"#228b22",fuchsia:"#ff00ff",gainsboro:"#dcdcdc",ghostwhite:"#f8f8ff",gold:"#ffd700",goldenrod:"#daa520",
gray:"#808080",green:"#008000",greenyellow:"#adff2f",honeydew:"#f0fff0",hotpink:"#ff69b4","indianred ":"#cd5c5c",indigo:"#4b0082",ivory:"#fffff0",khaki:"#f0e68c",lavender:"#e6e6fa",lavenderblush:"#fff0f5",lawngreen:"#7cfc00",lemonchiffon:"#fffacd",lightblue:"#add8e6",lightcoral:"#f08080",lightcyan:"#e0ffff",lightgoldenrodyellow:"#fafad2",lightgrey:"#d3d3d3",lightgreen:"#90ee90",lightpink:"#ffb6c1",lightsalmon:"#ffa07a",lightseagreen:"#20b2aa",lightskyblue:"#87cefa",lightslategray:"#778899",lightsteelblue:"#b0c4de",
lightyellow:"#ffffe0",lime:"#00ff00",limegreen:"#32cd32",linen:"#faf0e6",magenta:"#ff00ff",maroon:"#800000",mediumaquamarine:"#66cdaa",mediumblue:"#0000cd",mediumorchid:"#ba55d3",mediumpurple:"#9370d8",mediumseagreen:"#3cb371",mediumslateblue:"#7b68ee",mediumspringgreen:"#00fa9a",mediumturquoise:"#48d1cc",mediumvioletred:"#c71585",midnightblue:"#191970",mintcream:"#f5fffa",mistyrose:"#ffe4e1",moccasin:"#ffe4b5",navajowhite:"#ffdead",navy:"#000080",oldlace:"#fdf5e6",olive:"#808000",olivedrab:"#6b8e23",
orange:"#ffa500",orangered:"#ff4500",orchid:"#da70d6",palegoldenrod:"#eee8aa",palegreen:"#98fb98",paleturquoise:"#afeeee",palevioletred:"#d87093",papayawhip:"#ffefd5",peachpuff:"#ffdab9",peru:"#cd853f",pink:"#ffc0cb",plum:"#dda0dd",powderblue:"#b0e0e6",purple:"#800080",rebeccapurple:"#663399",red:"#ff0000",rosybrown:"#bc8f8f",royalblue:"#4169e1",saddlebrown:"#8b4513",salmon:"#fa8072",sandybrown:"#f4a460",seagreen:"#2e8b57",seashell:"#fff5ee",sienna:"#a0522d",silver:"#c0c0c0",skyblue:"#87ceeb",slateblue:"#6a5acd",
slategray:"#708090",snow:"#fffafa",springgreen:"#00ff7f",steelblue:"#4682b4",tan:"#d2b48c",teal:"#008080",thistle:"#d8bfd8",tomato:"#ff6347",turquoise:"#40e0d0",violet:"#ee82ee",wheat:"#f5deb3",white:"#ffffff",whitesmoke:"#f5f5f5",yellow:"#ffff00",yellowgreen:"#9acd32"};var mI={dt:"regex",Us:"attr_value_range",lz:"node_type"};function nI(a){return 0===(null==a?void 0:a.nodeType)}function oI(a){return 1===(null==a?void 0:a.nodeType)}function pI(a){const b=[];for(const [c,d]of Object.entries(a))switch(a=d,c){case "Op node id":a.selected&&b.push("id")}return b}function qI(a){const b=[];for(const [c,d]of Object.entries(a))switch(a=d,c){case "Layer node children count":a.selected&&b.push("#children");break;case "Layer node descendants count":a.selected&&b.push("#descendants")}return b}
function rI(a,b){if(nI(a))switch(b.toLowerCase()){case "id":return a.id;case "namespace":return sI(a)}else if(oI(a))switch(b.toLowerCase()){case "namespace":return sI(a);case "#children":return String((a.nsChildrenIds||[]).length);case "#descendants":return String((a.descendantsNodeIds||[]).length)}return""}function sI(a){return a.savedNamespace||a.namespace||"<root>"}function tI(){return Math.random().toString(36).slice(-6)}
function uI(a,b,c){let d=[];d=null==a?b.rootNodes.map(e=>e.id):a.nsChildrenIds||[];for(const e of d)(a=b.nodesById[e])&&oI(a)&&a.expanded&&((a.nsChildrenIds||[]).filter(f=>oI(b.nodesById[f])).every(f=>!b.nodesById[f].expanded)&&c.push(a.id),uI(a,b,c))}
function vI(a){var b=wI,c=xI,d=THREE,e=[];if(2===a.length)e=a;else if(3===a.length&&a[0].x===a[1].x&&a[1].x===a[2].x)e=a;else{e=!0;var f=0;for(var g=0;g<a.length-1;g++){var k=a[g+1]>a[g]?1:-1;if(0!==f&&f!==k){e=!1;break}f=k}const p=d.Vector3;if(e){a=b().x(t=>t.x).y(t=>t.y).curve(c)(a).split(/M|C/).filter(t=>""!==t).map(t=>t.split(",").map(w=>Number(w)));b=new p(a[0][0],a[0][1],0);c=new d.CurvePath;for(e=1;e<a.length;e++){var m=a[e];6===m.length&&(f=b,g=new p(m[0],m[1]),k=new p(m[2],m[3]),b=m=new p(m[4],
m[5]),f=new d.CubicBezierCurve3(f,g,k,m),c.add(f))}e=c.getPoints(25)}else a=a.map(t=>new p(t.x,t.y,0)),e=(new d.CatmullRomCurve3(a,!1,"catmullrom",.1)).getPoints(25)}return e}function yI(){const a=zI();return a?"INPUT"===a.tagName||"SELECT"===a.tagName||"TEXTAREA"===a.tagName||"true"===a.contentEditable:!1}function zI(a=document){return(a=a.activeElement)?a.shadowRoot?zI(a.shadowRoot):a:null}function AI(a){a=((a||{}).shape||"").replace(/ /g,"").replace(/\u00d7/g,"x");""===a&&(a="?");return a}
function BI(a,b=""){a=a.attrs||{};const c=[],d=new RegExp(b,"i");for(const e of Object.keys(a)){const f=e,g=a[e],k=[`${f}:${g}`,`${f}=${g}`];(""===b.trim()||k.some(m=>d.test(m)))&&c.push({key:f,value:g})}return c}
function CI(a,b){const c=a.incomingEdges||[],d=[];for(let k=0;k<Math.min(10,c.length);k++){var e=c[k],f=b.nodesById[e.sourceNodeId];const m=AI((f.outputsMetadata||{})[e.sourceNodeOutputId]);var g=k;e=((a.inputsMetadata||{})[e.targetNodeInputId]||{}).__tensor_tag;d.push({key:e?`Input${g}:${e} (${f.label})`:`Input${g} (${f.label})`,value:m})}10<c.length&&(a=c.length-10,d.push({key:`(${a} more input${1===a?"":"s"} omitted)`,value:"..."}));return d}
function DI(a){const b=[],c=Object.values(a.outputsMetadata||{});for(let e=0;e<Math.min(10,c.length);e++){var d=c[e];const f=AI(d);let g=`Output${e}`;"GraphInputs"===a.label?(d=d.tensor_name,null!=d&&(g=`${g} (${d})`)):(d=d.__tensor_tag)&&(g=`Output${e}:${d}`);b.push({key:g,value:f})}10<c.length&&(a=c.length-10,b.push({key:`(${a} more output${1===a?"":"s"} omitted)`,value:"..."}));return b}
function EI(a,b,c,d){const e=[],f=Object.keys(c).filter(g=>c[g].selected).filter(g=>g.startsWith("Node data provider: ")).map(g=>g.replace("Node data provider: ",""));d=Object.values(d).filter(g=>f.includes(g.runName));for(const g of d){let k,m;d=(null==(k=g.results||{})?void 0:null==(m=k[b][a.id])?void 0:m.strValue)||"-";e.push({key:g.runName,value:d})}return e}
function FI(a,b){a=a.split("/");b=b.split("/");let c="";for(let d=Math.min(a.length,b.length);0<d;d--){const e=b.slice(0,d).join("/");if(a.slice(0,d).join("/")===e){c=e;break}}return c}function GI(){var a=HI;return sa(function*(){return new Promise(b=>{const c=document.createElement("script");vd(c,a);c.onload=()=>{c.remove();b()};document.body.appendChild(c)})})}
function II(a){return(new RegExp(/Only `SavedModel`s with \d+ MetaGraph are supported. Instead, it has \d+/)).test(a)?`${a}. Try using the "TF adapter (direct)" adapter.`:a}
function JI(a,b,c,d){const e=[],f=new Set;a.has("title")&&b.test(c.label)&&(e.push({type:"title"}),f.add("title"));if(a.has("list")){var g=KI(c);for(var k of Object.keys(g)){var m=g[k],p=`${k}=${m}`;if(b.test(`${k}:${m}`)||b.test(p))e.push({type:"list",Dw:k}),f.add("list")}}if(a.has("input")&&nI(c))for(const w of c.incomingEdges||[]){k=d.nodesById[w.sourceNodeId];b.test(k.label)&&(e.push({type:"input",gf:k.label}),f.add("input"));g=c.inputsMetadata||{};(m=(g[w.targetNodeInputId]||{}).__tensor_tag)&&
b.test(m)&&(e.push({type:"input",gf:m}),f.add("input"));k=(k.outputsMetadata||{})[w.sourceNodeOutputId]||{};for(const B of Object.keys(k)){if(B.startsWith("__"))continue;m=k[B];p=`${B}:${m}`;const G=`${B}=${m}`;if(b.test(m)||b.test(p)||b.test(G))e.push({type:"input",gf:m}),f.add("input")}g=g[w.targetNodeInputId]||{};for(const B of Object.keys(g))!B.startsWith("__")&&(k=g[B],m=`${B}:${k}`,p=`${B}=${k}`,b.test(k)||b.test(m)||b.test(p))&&(e.push({type:"input",gf:k}),f.add("input"))}if(a.has("output")&&
nI(c)){a=c.outputsMetadata||{};for(var t of c.outgoingEdges||[])c=d.nodesById[t.targetNodeId],b.test(c.label)&&(e.push({type:"output",gf:c.label}),f.add("output")),(c=(a[t.sourceNodeOutputId]||{}).__tensor_tag)&&b.test(c)&&(e.push({type:"output",gf:c}),f.add("output"));for(const w of Object.values(a))for(const B of Object.keys(w))!B.startsWith("__")&&(d=w[B],t=`${B}:${d}`,c=`${B}=${d}`,b.test(d)||b.test(t)||b.test(c))&&(e.push({type:"output",gf:d}),f.add("output"))}return{matches:e,matchTypes:f}}
function KI(a){let b={};nI(a)?(b=Object.assign({},a.attrs||{}),b.id=a.id):oI(a)&&(b={"#descendants":`${(a.descendantsNodeIds||[]).length}`,"#children":`${(a.nsChildrenIds||[]).length}`});return b}function LI(a){for(const b of a.filter(c=>"node_type"!==c.type))switch(b.type){case "regex":if(""!==b.queryRegex)return!0;break;case "attr_value_range":if(""!==b.attrName)return!0}return!1}
function MI(a){return a.map(b=>({queries:b.queries.map(c=>{switch(c.type){case "regex":let d=RegExp("","i");try{d=new RegExp(c.queryRegex,"i")}catch(e){console.warn("Failed to create regex",e)}return{type:"regex",queryRegex:d,matchTypes:new Set(c.matchTypes)};case "node_type":case "attr_value_range":return c}}),nodeType:b.nodeType,styles:b.styles}))}
function NI(a,b,c){let d=!0;for(const k of b)if("node_type"===k.type){b=!0;if(nI(a)&&"layer_nodes"===k.nodeType||oI(a)&&"op_nodes"===k.nodeType)b=!1;if(!b){d=!1;break}}else if("regex"===k.type){if(0===JI(k.matchTypes,k.queryRegex,a,c).matches.length){d=!1;break}}else if("attr_value_range"===k.type){if(""!==k.attrName){let m,p;b=k.attrName;var e=null!=(m=k.min)?m:Number.NEGATIVE_INFINITY,f=null!=(p=k.max)?p:Number.POSITIVE_INFINITY;const t=[];var g=KI(a)[b];null!=g&&(g=Number(g),!isNaN(g)&&g>=e&&g<=
f&&t.push({type:"list",Dw:b}));if(0===t.length){d=!1;break}}}else d=!1;return d};var OI="Processing nodes and edges;Processing layer namespaces;Processing layout data;Splitting large layers (if any);Laying out root layer;Finding identical layers".split(";");function PI(a,b){window.localStorage.setItem(a,b)}var QI=class{};QI.K=function(a){return new (a||QI)};QI.pa=Md({ma:QI,ba:QI.K,ea:"root"});function RI(){return{deepestExpandedGroupNodeIds:[],selectedNodeId:"",selectedGraphId:"",selectedCollectionLabel:"",widthFraction:1,selected:!1}}function SI(a,b,c){a.C.update(d=>{const e=d.paneStates[c];if(!e)return d;e.deepestExpandedGroupNodeIds=b;return Object.assign({},d)})}function TI(a,b,c){a.C.update(d=>{const e=d.paneStates[c];if(!e)return d;e.selectedNodeId=b;return Object.assign({},d)})}
function UI(a,b,c,d){a.C.update(e=>{const f=e.paneStates[d];if(!f)return e;f.selectedGraphId=b;f.selectedCollectionLabel=c;return Object.assign({},e)})}function VI(a,b,c){a.C.update(d=>{const e=d.paneStates[c];if(!e)return d;e.flattenLayers=b;return Object.assign({},d)})}function WI(a){a.C.update(b=>{1<b.paneStates.length&&(b.paneStates=[b.paneStates[0]]);b.paneStates.push(RI());for(const c of b.paneStates)c.widthFraction=.5;return Object.assign({},b)})}
function XI(a,b){a.C.update(c=>{c.paneStates.splice(b,1);1===c.paneStates.length&&(c.paneStates[0].widthFraction=1);return Object.assign({},c)})}function YI(a,b){a.C.update(c=>{2===c.paneStates.length&&(c.paneStates[0].widthFraction=b,c.paneStates[1].widthFraction=1-b);return Object.assign({},c)})}function ZI(a){a.C.update(b=>{2===b.paneStates.length&&(b.paneStates=[b.paneStates[1],b.paneStates[0]]);return Object.assign({},b)})}
function $I(a,b){a.C.update(c=>{for(let d=0;d<c.paneStates.length;d++)c.paneStates[d].selected=d===b;return Object.assign({},c)})}var aJ=class{constructor(){this.C=xr({paneStates:[RI()]})}};aJ.K=function(a){return new (a||aJ)};aJ.pa=Md({ma:aJ,ba:aJ.K});var bJ=class{constructor(){const a=PH`/static_files/worker_bin.js`;this.C=new Worker(dd(a),void 0)}};bJ.K=function(a){return new (a||bJ)};bJ.pa=Md({ma:bJ,ba:bJ.K});function cJ(a){a.T.C.addEventListener("message",b=>{b=b.data;switch(b.eventType){case 1:dJ(a,b.modelGraph,b.paneId)}})}function eJ(a){let b={};a.C||(a=window.localStorage.getItem("model_explorer_show_on_node_item_types_v2"))&&(b=JSON.parse(a));return b}function fJ(a){let b={};a.C||(a=window.localStorage.getItem("model_explorer_show_on_edge_item_types_v2"))&&(b=JSON.parse(a));return b}
function gJ(a,b){a.O.update(c=>{c=[...c];const d={};for(const e of b){const f={},g={};for(const t of e.graphs){if(null==t.id||""===t.id)t.id="unnamed_graph";const w=t.id;let B=d[w];null==B?B=d[w]=0:(t.id=`${t.id} (${B+1})`,g[w]=t.id);d[w]=B+1;f[t.id]=t;t.collectionLabel=e.label}for(const t of e.graphs)for(const w of t.nodes)if(null!=w.subgraphIds&&0<w.subgraphIds.length){w.subgraphIds=w.subgraphIds.map(B=>g[B]||B);null==t.subGraphIds&&(t.subGraphIds=[]);t.subGraphIds.push(...w.subgraphIds);for(const B of w.subgraphIds)f[B].parentGraphId=
t.id}const k=e.graphs.filter(t=>null==t.parentGraphId),m=[],p=(t,w=0)=>{let B=[];B=null==t?k:(t.subGraphIds||[]).map(G=>f[G]);B.sort((G,J)=>J.nodes.length-G.nodes.length);for(const G of B)G.level=w,m.push(G),p(G,w+1)};p();e.graphs=m}c.push(...b);return c})}
function hJ(a,b,c=!1){const d=tI();a.P[d]=b;a.La.update(f=>{f[0].widthFraction=.5;f.push({id:d,widthFraction:.5,flattenLayers:c,showOnNodeItemTypes:{[d]:eJ(a)},Wb:{[d]:fJ(a)}});return[...f]});a.G.set({id:d,Lr:2});const e=iJ(a,d);WI(a.D);jJ(a,d);UI(a.D,b.id,b.collectionLabel||"",e);VI(a.D,c,e);b={eventType:0,graph:b,showOnNodeItemTypes:kJ(a,d,d),nodeDataProviderRuns:{},config:a.config?a.config():void 0,paneId:d,groupNodeChildrenCountThreshold:lJ(a),flattenLayers:c};a.T.C.postMessage(b)}
function iJ(a,b){return a.La().findIndex(c=>c.id===b)}function mJ(a,b,c=!1,d){if(null!=d){const e=nJ(a,b);e&&(e.Do=d)}oJ(a,b);b={eventType:0,graph:a.P[b],showOnNodeItemTypes:kJ(a,b,b),nodeDataProviderRuns:{},config:a.config?a.config():void 0,paneId:b,groupNodeChildrenCountThreshold:lJ(a),flattenLayers:c};a.T.C.postMessage(b)}
function pJ(a,b,c,d=!1,e){if(1===c&&1===a.La().length)hJ(a,b);else{var f;if(((null==(f=a.La()[c].modelGraph)?void 0:f.id)||"")!==b.id){c=a.La()[c];c.Pg=void 0;c.qs=void 0;f=c.id;a.P[f]=b;UI(a.D,b.id,b.collectionLabel||"",iJ(a,f));if(null!=c.Zf&&0<c.Zf.length){const g=[...c.Zf];g[g.length-1].Ag=b.id;c.Zf=g}mJ(a,f,d,e)}}}function qJ(a,b,c=!1,d){pJ(a,b,iJ(a,a.J()),c,d)}function jJ(a,b){a.J.set(b);$I(a.D,iJ(a,b))}
function kJ(a,b,c){return(a=nJ(a,b))?JSON.parse(JSON.stringify((a.showOnNodeItemTypes||{})[c]||{})):{}}function lJ(a){let b=1E3;if(a.config){let c;b=(null==(c=a.config())?void 0:c.artificialLayerNodeCountThreshold)||1E3}null!=a.sa&&(b=Number(a.sa));return b}function nJ(a,b){return a.La().find(c=>c.id===b)}function oJ(a,b){a.La.update(c=>{for(const d of c)if(d.id===b){d.modelGraph=void 0;break}return[...c]})}function rJ(a){return nJ(a,a.J())}
function sJ(a,b){const c=rJ(a);if(c){a.La.update(e=>{c.flattenLayers=b;c.Pg=void 0;return[...e]});var d=iJ(a,c.id);VI(a.D,b,d);SI(a.D,[],d)}}function tJ(a,b){const c=nJ(a,b);if(c){var d=!0===c.flattenLayers;a.La.update(e=>{c.flattenLayers=!d;c.Pg=void 0;return[...e]});b=iJ(a,b);VI(a.D,!d,b);SI(a.D,[],b)}}function uJ(a,b){let c;return!0===(null==(c=nJ(a,b))?void 0:c.flattenLayers)}function vJ(a,b){(b=a.La()[b])&&jJ(a,b.id)}
function wJ(a,b,c){a.La.update(f=>{const g=nJ(a,b);if(!g)return f;g.Qg=c;return[...f]});TI(a.D,(null==c?void 0:c.nodeId)||"",iJ(a,b));var d,e=null==(d=nJ(a,b))?void 0:d.modelGraph;if(e&&(d=(null==c?void 0:c.nodeId)||"",(e=e.nodesById[d])&&nI(e)&&window.parent)){e=e.outputsMetadata||{};const f=[];for(const g of Object.keys(e)){const k=e[g].tensor_name;k&&f.push(k)}window.parent.postMessage({cmd:"model-explorer-node-selected",nodeId:d,outputTensorNames:f},"*")}}
function xJ(a){let b;return null==(b=nJ(a,a.J()))?void 0:b.modelGraph}function yJ(a,b){let c;return null==(c=nJ(a,b))?void 0:c.modelGraph}function zJ(a){let b;return null==(b=a.La()[0])?void 0:b.modelGraph}function AJ(a,b){a.La.update(c=>{if(2!==c.length)return c;c[0].widthFraction=b;c[1].widthFraction=1-b;return[...c]});YI(a.D,b)}function BJ(a,b,c){a.La.update(d=>{const e=nJ(a,b);if(!e)return d;e.qs=c;return[...d]})}function CJ(a,b){let c;return null==(c=nJ(a,b))?void 0:c.qs}
function DJ(a,b,c){a.La.update(d=>{const e=nJ(a,b);if(!e)return d;e.Hn=c;return[...d]})}function EJ(a,b,c){a.La.update(d=>{const e=nJ(a,b);if(!e)return d;e.eD=c;return[...d]})}function FJ(a,b){delete a.P[b];a.La.update(c=>{const d=c.findIndex(e=>e.id===b);0<=d&&(c.splice(d,1),XI(a.D,d));c[0].widthFraction=1;return[...c]});jJ(a,a.La()[0].id)}function GJ(a){a.La.update(b=>2!==b.length?b:[b[1],b[0]]);ZI(a.D)}
function HJ(a,b,c,d){a.La.update(e=>{const f=nJ(a,d);f&&(null==f.Rg&&(f.Rg={}),null==f.Rg[c]&&(f.Rg[c]=[]),f.Rg[c].push(b));return[...e]})}function IJ(a,b,c,d){a.La.update(e=>{const f=nJ(a,d);f&&f.Rg&&f.Rg[c]&&f.Rg[c].splice(b,1);return[...e]})}function JJ(a,b){for(const c of a.O())for(const d of c.graphs)if(d.id===b)return d}
function KJ(a,b,c,d,e){a.La.update(f=>{const g=nJ(a,b);if(!g)return f;const k=[...(g.Zf||[])];0===k.length?k.push({Ag:c,kb:e}):k[k.length-1]={Ag:c,kb:e};k.push({Ag:d});g.Zf=k;return[...f]})}function LJ(a,b,c){a.La.update(d=>{const e=nJ(a,b);if(!e)return d;let f=[...(e.Zf||[])];f.splice(c+1);1===f.length&&(f=[]);e.Zf=f;return[...d]})}function MJ(a,b,c){a.La.update(d=>{const e=nJ(a,b);if(!e)return d;e.Pg=c;return[...d]})}
function NJ(a,b){a.La.update(c=>{const d=nJ(a,b);if(!d)return c;d.Pg={results:{}};return[...c]})}
function OJ(a,b,c,d){a.La.update(e=>{const f=nJ(a,b);if(!f)return e;f.showOnNodeItemTypes||(f.showOnNodeItemTypes={});null==f.showOnNodeItemTypes[c]&&(f.showOnNodeItemTypes[c]={});null==f.showOnNodeItemTypes[c][d]&&(f.showOnNodeItemTypes[c][d]={selected:!1});const g=f.showOnNodeItemTypes[c][d].selected;f.showOnNodeItemTypes[c]=Object.assign({},f.showOnNodeItemTypes[c]);f.showOnNodeItemTypes[c][d].selected=!g;f.showOnNodeItemTypes=Object.assign({},f.showOnNodeItemTypes);return[...e]})}
function PJ(a,b,c,d){a.La.update(e=>{const f=nJ(a,b);if(!f)return e;f.Wb||(f.Wb={});null==f.Wb[c]&&(f.Wb[c]={});null==f.Wb[c][d]&&(f.Wb[c][d]={selected:!1});const g=f.Wb[c][d].selected;f.Wb[c]=Object.assign({},f.Wb[c]);f.Wb[c][d].selected=!g;f.Wb=Object.assign({},f.Wb);return[...e]})}
function QJ(a,b,c,d,e){a.La.update(f=>{const g=nJ(a,b);if(!g)return f;g.showOnNodeItemTypes||(g.showOnNodeItemTypes={});null==g.showOnNodeItemTypes[c]&&(g.showOnNodeItemTypes[c]={});null==g.showOnNodeItemTypes[c][d]&&(g.showOnNodeItemTypes[c][d]={selected:!1});g.showOnNodeItemTypes[c][d].filterRegex=e;g.showOnNodeItemTypes=Object.assign({},g.showOnNodeItemTypes);return[...f]})}
function RJ(a,b,c,d){a.La.update(e=>{const f=nJ(a,b);if(!f)return e;f.showOnNodeItemTypes||(f.showOnNodeItemTypes={});f.showOnNodeItemTypes=Object.assign({},f.showOnNodeItemTypes);f.showOnNodeItemTypes[c]=d;return[...e]})}function SJ(a,b,c,d){a.La.update(e=>{const f=nJ(a,b);if(!f)return e;f.Wb||(f.Wb={});f.Wb=Object.assign({},f.Wb);f.Wb[c]=d;return[...e]})}
function TJ(a,b){a.La.update(c=>{for(const d of c){d.showOnNodeItemTypes=Object.assign({},d.showOnNodeItemTypes);for(const e of Object.keys(d.showOnNodeItemTypes))for(const f of b){const g=d.showOnNodeItemTypes[e][f];g&&(g.selected=!1)}}return[...c]})}function UJ(a,b,c){return(a=nJ(a,b))?JSON.parse(JSON.stringify((a.Wb||{})[c]||{})):{}}function dJ(a,b,c){a.La.update(d=>{for(const e of d)if(e.id===c){e.modelGraph=b;break}return[...d]});a.ua.next({gG:iJ(a,c),modelGraph:b})}
var VJ=class{constructor(a,b,c){this.D=b;this.T=c;this.O=xr([]);this.F=xr();this.G=xr();this.fa=new cj;this.wa=new cj;this.ja=new cj;this.aa=new cj;this.ca=new cj;this.na=new cj;this.config=xr();this.N=xr();this.La=xr([{id:tI(),widthFraction:1}]);this.J=xr(this.La()[0].id);this.ua=new cj;this.Mg=xr([]);this.P={};cJ(this);this.G.set({id:this.La()[0].id,Lr:2});a=new URLSearchParams(document.location.search);this.C="1"===a.get("test_mode");this.sa=a.get("groupNodeChildrenCountThreshold");a=this.La()[0];
a.showOnNodeItemTypes={[a.id]:eJ(this)};a.Wb={[a.id]:fJ(this)}}};VJ.K=function(a){return new (a||VJ)(r(QI),r(aJ),r(bJ))};VJ.pa=Md({ma:VJ,ba:VJ.K});var WJ=class{constructor(){this.Aw=Math.random().toString(36).slice(-6);this.NB=`url(#${this.Aw})`}};WJ.K=function(a){return new (a||WJ)};
WJ.Da=Gf({type:WJ,ha:[["me-logo"]],ka:!0,features:[Mr],za:19,Aa:2,Ca:["width;24;height;24;viewBox;0 0 256 256;fill;none;xmlns;http://www.w3.org/2000/svg".split(";"),"width 256 height 256 fill none".split(" "),["d","M229 64.5V192.5L128 248.5L127.5 115.5L229 64.5Z"],["d","M208 76V99L128 141.5V116L208 76Z","fill","white"],["d","M208 122.5V141.5L128 182.5V161.5L208 122.5Z","fill","white"],["d","M208 164.5V183L128 225.5V206L208 164.5Z","fill","white"],["d","M149 106V214.5L127 226V117.5L149 106Z","fill",
"white"],["d","M26 61L47.5 72.5V200L26 187V61Z","fill","#FBBC04"],["d","M125.5 10.5L145.5 21V60.5L125.5 56.5V10.5Z","fill","#DCA810"],["d","M164 31L184 41.5V81L164 78V31Z","fill","#309C4D"],["d","M164 31L184 41.5L87 94L65 82L164 31Z","fill","#11792D"],["d","M208 54.5L229 64.5L128 116L107 105L208 54.5Z","fill","#2C5CAC"],["d","M125.5 10.5L145.5 21L47.5 72.5L26 61L125.5 10.5Z","fill","#C78B15"],["d","M65 82L87 94V224L65 210.5V82Z","fill","#34A853"],["d","M107 105L128 116V248.5L107 236V105Z","fill",
"#4285F4"],"x1 144.5 y1 232 x2 219 y2 69.5 gradientUnits userSpaceOnUse".split(" "),["stop-color","#4285F4"],["offset","1","stop-color","#2C5CAC"]],ta:function(a,b){a&1&&(fh(),R(0,"svg",0),Mq(1,"rect",1)(2,"path",2)(3,"path",3)(4,"path",4)(5,"path",5)(6,"path",6)(7,"path",7)(8,"path",8)(9,"path",9)(10,"path",10)(11,"path",11)(12,"path",12)(13,"path",13)(14,"path",14),R(15,"defs")(16,"linearGradient",15),Mq(17,"stop",16)(18,"stop",17),S()()());a&2&&(D(2),Kp("fill",b.NB),D(14),Kp("id",b.Aw))},styles:["[_nghost-%COMP%] {display: flex; align-items: center; justify-content: center}"]});var XJ=[Yy("transformPanel",[az("void",$y({opacity:0,transform:"scale(1, 0.8)"})),bz("void => showing",Zy("120ms cubic-bezier(0, 0, 0.2, 1)",$y({opacity:1,transform:"scale(1, 1)"}))),bz("* => void",Zy("100ms linear",$y({opacity:0})))])];function YJ(a,b){let c=[b];c.push(...ZJ(a,b));tu(a.C,...c.map(d=>a.D(d)))}function $J(a,b){let c=[b];c.push(...ZJ(a,b));a.C.select(...c.map(d=>a.D(d)))}var aK=class{constructor(){this.C=new vu(!0)}toggle(a){this.C.toggle(this.D(a))}expand(a){this.C.select(this.D(a))}collapse(a){tu(this.C,this.D(a))}ul(a){return uu(this.C,this.D(a))}D(a){return this.hm?this.hm(a):a}};function ZJ(a,b){var c=a.F.indexOf(b);const d=[];for(c+=1;c<a.F.length&&a.ye(b)<a.ye(a.F[c]);c++)d.push(a.F[c]);return d}function bK(a){a.C.select(...a.F.map(b=>a.D(b)))}var cK=class extends aK{constructor(a,b){super();this.ye=a;this.G=b;this.options=void 0}};var dK=new df("CDK_TREE_NODE_OUTLET_NODE"),eK=class{constructor(a,b){this.eg=a;this.C=b}};eK.K=function(a){return new (a||eK)(z(kp),z(dK,8))};eK.Fa=Qf({type:eK,ha:[["","cdkTreeNodeOutlet",""]],ka:!0});var fK=class{constructor(a){this.da=a}},gK=class{constructor(a){this.ta=a}};gK.K=function(a){return new (a||gK)(z(br))};gK.Fa=Qf({type:gK,ha:[["","cdkTreeNodeDef",""]],inputs:{Lo:[0,"cdkTreeNodeDefWhen","when"]},ka:!0});function hK(a,b,c=a.O,d=a.Li.eg,e){if(c=c.mn(b))Xr(c,(f,g,k)=>{if(null==f.Ie){f=b[k];g=a.T(f,k);const m=new fK(f);a.rd.ye?m.level=a.rd.ye(f):"undefined"!==typeof e&&a.F.has(e)?m.level=a.F.get(e)+1:m.level=0;a.F.set(f,m.level);(d?d:a.Li.eg).xd(g.ta,m,k);iK&&(iK.data=f)}else null==k?(d.remove(g),a.F.delete(f.item)):(f=d.get(g),d.move(f,k))}),cn(a.Ba)}
var jK=class{get Fh(){return this.C}set Fh(a){this.C!==a&&this.aa(a)}constructor(a,b){this.J=a;this.Ba=b;this.G=new cj;this.F=new Map;this.Cy=new ej({start:0,end:Number.MAX_VALUE})}Fb(){this.O=this.J.find([]).create(this.hm)}va(){this.Li.eg.clear();this.Cy.complete();this.G.next();this.G.complete();this.D&&(this.D.unsubscribe(),this.D=null)}Kw(){this.P=this.Ki.filter(a=>!a.Lo)[0];this.Fh&&this.Ki&&!this.D&&this.N()}aa(a){this.D&&(this.D.unsubscribe(),this.D=null);a||this.Li.eg.clear();this.C=a;this.Ki&&
this.N()}N(){let a;su(this.C)?a=this.C.connect(this):Ek(this.C)?a=this.C:Array.isArray(this.C)&&(a=jj(this.C));a&&(this.D=C(a,Lo(this.G)).subscribe(b=>hK(this,b)))}T(a,b){return 1===this.Ki.length?this.Ki.first:this.Ki.find(c=>c.Lo&&c.Lo(b,a))||this.P}};jK.K=function(a){return new (a||jK)(z(fs),z(Sr))};
jK.Da=Gf({type:jK,ha:[["cdk-tree"]],ec:function(a,b,c){a&1&&tr(c,gK,5);if(a&2){let d;vr(d=wr())&&(b.Ki=d)}},fb:function(a,b){a&1&&ur(eK,7);if(a&2){let c;vr(c=wr())&&(b.Li=c.first)}},Ka:["role","tree",1,"cdk-tree"],inputs:{Fh:"dataSource",rd:"treeControl",hm:"trackBy"},ib:["cdkTree"],ka:!0,features:[Mr],za:1,Aa:0,Ca:[["cdkTreeNodeOutlet",""]],ta:function(a){a&1&&Pq(0,0)},Ha:[eK],pb:2});
var lK=class{get role(){return"treeitem"}set role(a){this.ra.R.setAttribute("role",a)}get data(){return this.oh}set data(a){a!==this.oh&&(this.oh=a,this.GA(),this.lp.next())}get ul(){return this.Ef.rd.ul(this.oh)}get level(){return this.Ef.rd.ye?this.Ef.rd.ye(this.oh):this.xA}constructor(a,b){this.ra=a;this.Ef=b;this.Ga=new cj;this.lp=new cj;iK=this;this.role="treeitem"}Fb(){let a=this.ra.R.parentElement;for(;a&&!kK(a);)a=a.parentElement;this.xA=a?a.classList.contains("cdk-nested-tree-node")?As(a.getAttribute("aria-level")):
0:-1;this.ra.R.setAttribute("aria-level",`${this.level+1}`)}va(){iK===this&&(iK=null);this.lp.complete();this.Ga.next();this.Ga.complete()}focus(){this.ra.R.focus()}GA(){this.role="treeitem"}},iK=null;lK.K=function(a){return new (a||lK)(z(si),z(jK))};lK.Fa=Qf({type:lK,ha:[["cdk-tree-node"]],Ka:[1,"cdk-tree-node"],Za:1,Ia:function(a,b){a&2&&Kp("aria-expanded",b.ul)},inputs:{role:"role"},ib:["cdkTreeNode"],ka:!0});
function kK(a){a=a.classList;return!!((null==a?0:a.contains("cdk-nested-tree-node"))||(null==a?0:a.contains("cdk-tree")))};const mK=/([A-Za-z%]+)$/;
var nK=class{get level(){return this.F}set level(a){this.O(a)}get tl(){return this.D}set tl(a){this.N(a)}constructor(a,b,c,d){this.J=a;this.Ef=b;this.T=c;this.G=d;this.Ga=new cj;this.P="px";this.D=40;this.C();d&&C(d.wb,Lo(this.Ga)).subscribe(()=>this.C(!0));a.lp.subscribe(()=>this.C())}va(){this.Ga.next();this.Ga.complete()}ca(){var a=this.J.data&&this.Ef.rd.ye?this.Ef.rd.ye(this.J.data):null;a=null==this.F?a:this.F;return"number"===typeof a?`${a*this.D}${this.P}`:null}C(a=!1){const b=this.ca();if(b!==
this.aa||a){a=this.T.R;const c=this.G&&"rtl"===this.G.value?"paddingRight":"paddingLeft";a.style[c]=b||"";a.style["paddingLeft"===c?"paddingRight":"paddingLeft"]="";this.aa=b}}O(a){this.F=isNaN(a)?null:a;this.C()}N(a){let b=a,c="px";"string"===typeof a&&(a=a.split(mK),b=a[0],c=a[1]||c);this.P=c;this.D=As(b);this.C()}};nK.K=function(a){return new (a||nK)(z(lK),z(jK),z(si),z(ou,8))};
nK.Fa=Qf({type:nK,ha:[["","cdkTreeNodePadding",""]],inputs:{level:[2,"cdkTreeNodePadding","level",As],tl:[0,"cdkTreeNodePaddingIndent","indent"]},ka:!0,features:[Jp]});var oK=class{constructor(a,b){this.Ef=a;this.C=b;this.recursive=!1}NA(a){if(this.recursive){var b=this.Ef.rd,c=this.C.data;uu(b.C,b.D(c))?YJ(b,c):$J(b,c)}else this.Ef.rd.toggle(this.C.data);a.stopPropagation()}};oK.K=function(a){return new (a||oK)(z(jK),z(lK))};oK.Fa=Qf({type:oK,ha:[["","cdkTreeNodeToggle",""]],Ia:function(a,b){a&1&&V("click",function(c){return b.NA(c)})},inputs:{recursive:[2,"cdkTreeNodeToggleRecursive","recursive",zs]},ka:!0,features:[Jp]});var pK=class{};pK.K=function(a){return new (a||pK)};pK.Ya=Of({type:pK});pK.Xa=Nd({});function qK(a,b){let c=[];b.forEach(d=>a.D(d,0,c,[]));return c}function rK(a,b,c){let d=[],e=[!0];b.forEach(f=>{let g=!0;for(let k=0;k<=a.ye(f);k++)g=g&&e[k];g&&d.push(f);a.G(f)&&(e[a.ye(f)+1]=c.ul(f))});return d}
var sK=class{constructor(a,b,c,d){this.J=a;this.ye=b;this.G=c;this.F=d}D(a,b,c,d){const e=this.J(a,b);c.push(e);this.G(e)&&(a=this.F(a))&&(Array.isArray(a)?this.C(a,b,c,d):C(a,fo()).subscribe(f=>{this.C(f,b,c,d)}));return c}C(a,b,c,d){a.forEach((e,f)=>{let g=d.slice();g.push(f!=a.length-1);this.D(e,b+1,c,g)})}},tK=class extends ru{get data(){return this.oh.value}set data(a){this.oh.next(a);this.C.next(qK(this.G,this.data));this.D.F=this.C.value}constructor(a,b){super();this.D=a;this.G=b;this.C=new ej([]);
this.F=new ej([]);this.oh=new ej([])}connect(a){return C(rk(a.Cy,this.D.C.Ah,this.C),xj(()=>{this.F.next(rK(this.G,this.C.value,this.D));return this.F.value}))}};var uK=class extends lK{constructor(a,b,c){super(a,b);this.disabled=!1;this.tabIndex=Number(c)||0}Fb(){super.Fb()}va(){super.va()}};uK.K=function(a){return new (a||uK)(z(si),z(jK),yh("tabindex"))};uK.Fa=Qf({type:uK,ha:[["mat-tree-node"]],Ka:[1,"mat-tree-node"],inputs:{disabled:[2,"disabled","disabled",zs],tabIndex:[2,"tabIndex","tabIndex",a=>null==a?0:As(a)]},ib:["matTreeNode"],ka:!0,features:[Kr([{oa:lK,Ab:uK}]),Jp,Cp]});var vK=class extends gK{};
vK.K=(()=>{let a;return function(b){return(a||(a=Mh(vK)))(b||vK)}})();vK.Fa=Qf({type:vK,ha:[["","matTreeNodeDef",""]],inputs:{Lo:[0,"matTreeNodeDefWhen","when"],data:[0,"matTreeNode","data"]},ka:!0,features:[Kr([{oa:gK,Ab:vK}]),Cp]});var wK=class{constructor(a,b){this.eg=a;this.C=b}};wK.K=function(a){return new (a||wK)(z(kp),z(dK,8))};wK.Fa=Qf({type:wK,ha:[["","matTreeNodeOutlet",""]],ka:!0,features:[Kr([{oa:eK,Ab:wK}])]});var xK=class extends nK{get level(){return this.F}set level(a){this.O(a)}get tl(){return this.D}set tl(a){this.N(a)}};xK.K=(()=>{let a;return function(b){return(a||(a=Mh(xK)))(b||xK)}})();xK.Fa=Qf({type:xK,ha:[["","matTreeNodePadding",""]],inputs:{level:[2,"matTreeNodePadding","level",As],tl:[0,"matTreeNodePaddingIndent","indent"]},ka:!0,features:[Kr([{oa:nK,Ab:xK}]),Jp,Cp]});var yK=class extends oK{};yK.K=(()=>{let a;return function(b){return(a||(a=Mh(yK)))(b||yK)}})();yK.Fa=Qf({type:yK,ha:[["","matTreeNodeToggle",""]],inputs:{recursive:[0,"matTreeNodeToggleRecursive","recursive"]},ka:!0,features:[Kr([{oa:oK,Ab:yK}]),Cp]});var zK=class extends jK{constructor(){super(...arguments);this.Li=void 0}};zK.K=(()=>{let a;return function(b){return(a||(a=Mh(zK)))(b||zK)}})();
zK.Da=Gf({type:zK,ha:[["mat-tree"]],fb:function(a,b){a&1&&ur(wK,7);if(a&2){let c;vr(c=wr())&&(b.Li=c.first)}},Ka:["role","tree",1,"mat-tree"],ib:["matTree"],ka:!0,features:[Kr([{oa:jK,Ab:zK}]),Cp,Mr],za:1,Aa:0,Ca:[["matTreeNodeOutlet",""]],ta:function(a){a&1&&Pq(0,0)},Ha:[wK],styles:[".mat-tree{display:block;background-color:var(--mat-tree-container-background-color)}.mat-tree-node,.mat-nested-tree-node{color:var(--mat-tree-node-text-color);font-family:var(--mat-tree-node-text-font);font-size:var(--mat-tree-node-text-size);font-weight:var(--mat-tree-node-text-weight)}.mat-tree-node{display:flex;align-items:center;flex:1;word-wrap:break-word;min-height:var(--mat-tree-node-min-height)}.mat-nested-tree-node{border-bottom-width:0}\n"],pb:2});var AK=class{};AK.K=function(a){return new (a||AK)};AK.Ya=Of({type:AK});AK.Xa=Nd({imports:[pK,mz,mz]});const BK=["tree"],CK=a=>({node:a});function DK(a){a&1&&Z(0);a&2&&(a=Y().da,Cr(" ",a.Ui," (",a.label,") "))}function EK(a){a&1&&Z(0);a&2&&(a=Y().da,Br(" ",a.label," "))}function FK(a){a&1&&Pq(0)}function GK(a){a&1&&Ap(0,FK,1,0,"ng-container",15);if(a&2){a=Y().da;Y();const b=zr(5);H("ngTemplateOutlet",b)("ngTemplateOutletContext",Pr(CK,a))}}function HK(a){a&1&&(R(0,"div",16),Z(1," Hover for values "),S())}
function IK(a,b){a&1&&(R(0,"tr")(1,"td",19)(2,"mat-icon",20),Z(3,"circle"),S(),Z(4),S(),R(5,"td",21),Z(6),S()());if(a&2){a=b.da;b=Y(2).da;const c=Y();nq("search-match",JK(c,b.metadata[a]));D(4);Br(" ",a,"");D(2);Ar(b.metadata[a])}}function KK(a){a&1&&(R(0,"table",17),Ap(1,IK,7,4,"tr",18),S());a&2&&(a=Y().da,Y(),D(),H("ngForOf",LK(a)))}function MK(a){a&1&&(R(0,"div",22)(1,"mat-icon",23),Z(2,"my_location"),S()());a&2&&(a=Y(2),H("matTooltip",a.Bl)("matTooltipPosition",a.Hs))}
function NK(a){a&1&&(R(0,"div",25),Z(1),S());a&2&&(a=Y(3),D(),Br(" Showing at most ",a.hl()," elements "))}function OK(a){a&1&&(R(0,"div",24),Ap(1,NK,2,1,"div",25),R(2,"div",26),Z(3),S()());if(a&2){a=Y().da;const b=Y();D();Eq(0!==b.hl()?1:-1);D(2);a=a.node;Ar(a?nI(a)?(a.attrs||{}).__value||"<empty>":"":"")}}
function PK(a,b){if(a&1){const c=Ug();R(0,"mat-tree-node",6)(1,"div",7);V("click",function(d){const e=u(c).da,f=Y();return x(f.il(e,d.altKey))});R(2,"div",8)(3,"div",9)(4,"div",10)(5,"span",11);Ap(6,DK,1,2)(7,EK,1,1);S();Ap(8,GK,1,4,"ng-container");S();Ap(9,HK,2,0,"div",12);S();Ap(10,KK,2,1,"table",13);S();Ap(11,MK,3,2,"div",14);S();Ap(12,OK,4,2,"ng-template",null,2,Qr);S()}if(a&2){a=b.da;b=zr(13);const c=Y();nq("has-locator",a.Cd)("has-values",QK(a))("highlight",a.Ln)("has-metadata",null!=a.metadata&&
0<Object.keys(a.metadata).length);H("xapInlineDialog",b)("overlaySize",c.gq)("overlayPositions",c.gv)("hoverDelayMs",50)("xapInlineDialogDisabled",!QK(a))("matTreeNodePaddingIndent",c.sy);Kp("data-id",a.nodeId);D(2);nq("has-extra-data",null!=a.vg);D(3);nq("color-bold",c.cv)("search-match",JK(c,a.label,a.Ui));D();Eq(a.Ui?6:7);D(2);Eq(a.vg?8:-1);D();H("ngIf",QK(a));D();H("ngIf",null!=a.metadata&&0<Object.keys(a.metadata).length);D();H("ngIf",a.Cd&&c.Cd)}}function RK(a){a&1&&Pq(0)}
function SK(a){a&1&&Ap(0,RK,1,0,"ng-container",15);if(a&2){a=Y().da;Y();const b=zr(5);H("ngTemplateOutlet",b)("ngTemplateOutletContext",Pr(CK,a))}}function TK(a){a&1&&(R(0,"div",22)(1,"mat-icon",23),Z(2,"my_location"),S()());a&2&&(a=Y(2),H("matTooltip",a.Bl)("matTooltipPosition",a.Hs))}
function UK(a,b){if(a&1){const c=Ug();R(0,"mat-tree-node",27)(1,"div",28);V("click",function(d){const e=u(c).da,f=Y();return x(f.il(e,d.altKey))});R(2,"div",9)(3,"button",29)(4,"mat-icon",30);Z(5);S()();R(6,"div",10);Z(7);Ap(8,SK,1,4,"ng-container");S()();Ap(9,TK,3,2,"div",14);S()()}a&2&&(a=b.da,b=Y(),nq("has-locator",a.Cd),H("matTreeNodePaddingIndent",b.sy),Kp("data-id",a.nodeId),D(),nq("highlight",a.RF),D(),nq("has-extra-data",null!=a.vg),D(),Kp("aria-label","Toggle "+a.label),D(2),Br(" ",b.rd.ul(a)?
"keyboard_arrow_down":"chevron_right"," "),D(2),Br(" ",a.label," "),D(),Eq(a.vg?8:-1),D(),H("ngIf",a.Cd&&b.Cd))}function VK(a,b){a&1&&(R(0,"div",32)(1,"mat-icon"),Z(2),S()());a&2&&(a=b.da,Y(2),H("matTooltip",WK(a)),D(2),Ar(a))}function XK(a,b){a&1&&(R(0,"div",31),Jq(1,VK,3,2,"div",32,Hq),S());a&2&&(a=b.node,D(),Lq(a.vg.matchTypes))}function JK(a,b,c){return a.D?null!=a.D.find(d=>d.gf===b||d.gf===c):!1}function LK(a){return Object.keys(a.metadata||{}).filter(b=>!b.startsWith("__")).sort()}
function QK(a){a=a.node;return a?nI(a)&&(a=a.attrs||{},a.__value)?"DATA_ELIDED"!==a.__value:!1:!1}function WK(a){switch(a){case "title":return"Node label matched";case "list":return"Node attribute(s) matched";case "input":return"Input(s) matched";case "output":return"Output(s) matched";default:return""}}function YK(a,b){a.data=b;a.Fh.data=a.data;bK(a.rd);bn(a.F)}
var ZK=class{constructor(a,b){this.C=a;this.F=b;this.Eo=!1;this.rendererId="";this.Hs="left";this.Cd=!0;this.cv=!1;this.he=new Fk;this.gq={minWidth:100};this.gv=[{cb:"start",eb:"top",Na:"end",Ta:"top"}];this.Bl="Click: locate\nAlt+click: select";this.G=(c,d)=>({nodeId:c.nodeId,node:c.node,vn:!!c.children&&0<c.children.length,label:c.label,Ui:c.Ui,level:d,Cd:c.Cd,Ln:c.Ln,metadata:c.metadata,vg:c.vg});this.D=[];this.sy=8;this.rd=new cK(c=>c.level,c=>c.vn);this.J=new sK(this.G,c=>c.level,c=>c.vn,c=>
c.children);this.Fh=new tK(this.rd,this.J)}tc(){null!=this.data&&(this.Fh.data=this.data,bK(this.rd))}il(a,b){a.Cd&&(this.C.F.set({nodeId:a.nodeId||"",rendererId:this.rendererId,Bd:a.vn,select:b}),this.he.emit({}))}aC(a,b){return b.vn}hl(){let a,b;return null!=(b=null==(a=this.C.config())?void 0:a.maxConstValueCount)?b:0}};ZK.K=function(a){return new (a||ZK)(z(VJ),z(Sr))};
ZK.Da=Gf({type:ZK,ha:[["io-tree"]],fb:function(a,b){a&1&&ur(BK,5);if(a&2){let c;vr(c=wr())&&(b.qy=c.first)}},inputs:{data:"data",Eo:"solidBackground",rendererId:"rendererId",Hs:"tooltipPosition",Cd:"showLocator",cv:"colorBoldNodeLabel"},outputs:{he:"onClose"},ka:!0,features:[Dg,Mr],za:6,Aa:7,Ca:[["tree",""],["matchTypes",""],["constValuesPopup",""],[1,"io-tree",3,"dataSource","treeControl"],["class","leaf","dialogLabel","const values popup","matTreeNodePadding","",3,"has-locator","has-values","highlight",
"has-metadata","xapInlineDialog","overlaySize","overlayPositions","hoverDelayMs","xapInlineDialogDisabled","matTreeNodePaddingIndent",4,"matTreeNodeDef"],["matTreeNodePadding","",3,"has-locator","matTreeNodePaddingIndent",4,"matTreeNodeDef","matTreeNodeDefWhen"],["dialogLabel","const values popup","matTreeNodePadding","",1,"leaf",3,"xapInlineDialog","overlaySize","overlayPositions","hoverDelayMs","xapInlineDialogDisabled","matTreeNodePaddingIndent"],[1,"edge-node",3,"click"],[1,"label-and-metadata"],
[1,"label-container"],[1,"label"],[1,"node-label"],["class","hover-for-values-label",4,"ngIf"],["class","metadata-table",4,"ngIf"],["class","locator-container","matTooltipClass","multiline-tooltip-left",3,"matTooltip","matTooltipPosition",4,"ngIf"],[4,"ngTemplateOutlet","ngTemplateOutletContext"],[1,"hover-for-values-label"],[1,"metadata-table"],[3,"search-match",4,"ngFor","ngForOf"],[1,"key"],[1,"bullet"],[1,"value"],["matTooltipClass","multiline-tooltip-left",1,"locator-container",3,"matTooltip",
"matTooltipPosition"],[1,"locator-icon"],[1,"model-explorer-const-values-popup"],[1,"values-count-msg"],[1,"content"],["matTreeNodePadding","",3,"matTreeNodePaddingIndent"],[1,"expandable-tree-node",3,"click"],["mat-icon-button","","matTreeNodeToggle","",1,"expand-icon-button"],[1,"expander"],[1,"match-types-container"],[1,"extra-label-container",3,"matTooltip"]],ta:function(a,b){a&1&&(R(0,"mat-tree",3,0),Ap(2,PK,14,26,"mat-tree-node",4)(3,UK,10,13,"mat-tree-node",5),S(),Ap(4,XK,3,0,"ng-template",
null,1,Qr));a&2&&(nq("solid-background",b.Eo),H("dataSource",b.Fh)("treeControl",b.rd)("@transformPanel","showing")("@.disabled",!b.Eo),D(3),H("matTreeNodeDefWhen",b.aC))},Ha:[ht,Zs,ct,gt,BB,AB,yB,hG,AE,fG,iI,hI,AK,vK,xK,yK,zK,uK,HG,EG],styles:[".io-tree[_ngcontent-%COMP%]{background-color:transparent;padding-top:4px}.io-tree.solid-background[_ngcontent-%COMP%]{-webkit-transform-origin:top center;transform-origin:top center;background-color:#fff;border:1px solid #ccc;border-radius:4px;padding-bottom:8px;min-width:270px;box-shadow:0 4px 6px -1px rgba(0,0,0,.1),0 2px 4px -2px rgba(0,0,0,.1)}.io-tree[_ngcontent-%COMP%]   mat-tree-node.has-locator[_ngcontent-%COMP%], .io-tree[_ngcontent-%COMP%]   mat-tree-node.has-values[_ngcontent-%COMP%]{cursor:pointer}.io-tree[_ngcontent-%COMP%]   mat-tree-node.has-locator[_ngcontent-%COMP%]:hover, .io-tree[_ngcontent-%COMP%]   mat-tree-node.has-values[_ngcontent-%COMP%]:hover{background-color:#f3f3f3}.io-tree[_ngcontent-%COMP%]   mat-tree-node.highlight[_ngcontent-%COMP%]   .label[_ngcontent-%COMP%]{color:#00639b;font-weight:500}.io-tree[_ngcontent-%COMP%]   mat-tree-node[_ngcontent-%COMP%]:not(:last-child).has-metadata.leaf{margin-bottom:6px}.io-tree[_ngcontent-%COMP%]   .edge-node[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:start;-webkit-align-items:flex-start;-moz-box-align:start;-ms-flex-align:start;align-items:flex-start;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between;width:100%;-moz-box-sizing:border-box;box-sizing:border-box;padding-right:12px;padding-left:27px}.io-tree[_ngcontent-%COMP%]   .edge-node[_ngcontent-%COMP%]:hover   .locator-container[_ngcontent-%COMP%]{opacity:.8}.io-tree[_ngcontent-%COMP%]   .edge-node[_ngcontent-%COMP%]   .label-and-metadata.has-extra-data[_ngcontent-%COMP%]{width:100%}.io-tree[_ngcontent-%COMP%]   .edge-node[_ngcontent-%COMP%]   .label-and-metadata.has-extra-data[_ngcontent-%COMP%]   .label[_ngcontent-%COMP%]{width:100%;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between}.io-tree[_ngcontent-%COMP%]   .edge-node[_ngcontent-%COMP%]   .label-and-metadata[_ngcontent-%COMP%]   .label-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}.io-tree[_ngcontent-%COMP%]   .edge-node[_ngcontent-%COMP%]   .label-and-metadata[_ngcontent-%COMP%]   .label-container[_ngcontent-%COMP%]   .label[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}.io-tree[_ngcontent-%COMP%]   .edge-node[_ngcontent-%COMP%]   .label-and-metadata[_ngcontent-%COMP%]   .label-container[_ngcontent-%COMP%]   .node-label.color-bold[_ngcontent-%COMP%]{color:#00639b;font-weight:500}.io-tree[_ngcontent-%COMP%]   .edge-node[_ngcontent-%COMP%]   .label-and-metadata[_ngcontent-%COMP%]   .label-container[_ngcontent-%COMP%]   .node-label.search-match[_ngcontent-%COMP%]{background-color:#f5e25a}.io-tree[_ngcontent-%COMP%]   .edge-node[_ngcontent-%COMP%]   .label-and-metadata[_ngcontent-%COMP%]   .hover-for-values-label[_ngcontent-%COMP%]{text-decoration:underline;-webkit-text-decoration-style:dotted;-moz-text-decoration-style:dotted;text-decoration-style:dotted;color:#999;margin-left:8px}.io-tree[_ngcontent-%COMP%]   .edge-node[_ngcontent-%COMP%]   .label-and-metadata[_ngcontent-%COMP%]   .metadata-table[_ngcontent-%COMP%]{border-spacing:0;border-collapse:collapse;padding:0;font-size:11px;margin-top:2px}.io-tree[_ngcontent-%COMP%]   .edge-node[_ngcontent-%COMP%]   .label-and-metadata[_ngcontent-%COMP%]   .metadata-table[_ngcontent-%COMP%]   tr[_ngcontent-%COMP%]{vertical-align:text-top}.io-tree[_ngcontent-%COMP%]   .edge-node[_ngcontent-%COMP%]   .label-and-metadata[_ngcontent-%COMP%]   .metadata-table[_ngcontent-%COMP%]   tr.search-match[_ngcontent-%COMP%]{background-color:#f5e25a}.io-tree[_ngcontent-%COMP%]   .edge-node[_ngcontent-%COMP%]   .label-and-metadata[_ngcontent-%COMP%]   .metadata-table[_ngcontent-%COMP%]   tr[_ngcontent-%COMP%]   mat-icon.bullet[_ngcontent-%COMP%]{font-size:4px;width:5px;height:5px;color:#bbb;margin-right:2px}.io-tree[_ngcontent-%COMP%]   .edge-node[_ngcontent-%COMP%]   .label-and-metadata[_ngcontent-%COMP%]   .metadata-table[_ngcontent-%COMP%]   td.key[_ngcontent-%COMP%]{color:#999;padding-right:4px;white-space:nowrap;line-height:12px}.io-tree[_ngcontent-%COMP%]   .edge-node[_ngcontent-%COMP%]   .label-and-metadata[_ngcontent-%COMP%]   .metadata-table[_ngcontent-%COMP%]   td.value[_ngcontent-%COMP%]{line-height:12px}.io-tree[_ngcontent-%COMP%]   .expandable-tree-node[_ngcontent-%COMP%], .io-tree[_ngcontent-%COMP%]   mat-tree-node[_ngcontent-%COMP%]{font-size:13px;min-height:22px;word-break:break-word}.io-tree[_ngcontent-%COMP%]   .expandable-tree-node[_ngcontent-%COMP%]{color:#999;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between;width:100%;-moz-box-sizing:border-box;box-sizing:border-box;padding-right:12px;padding-left:4px}.io-tree[_ngcontent-%COMP%]   .expandable-tree-node.highlight[_ngcontent-%COMP%]{color:rgba(0,0,0,.87)}.io-tree[_ngcontent-%COMP%]   .expandable-tree-node[_ngcontent-%COMP%]:hover   .locator-container[_ngcontent-%COMP%]{opacity:.8}.io-tree[_ngcontent-%COMP%]   .expandable-tree-node[_ngcontent-%COMP%]   mat-icon.expander[_ngcontent-%COMP%]{color:#999}.io-tree[_ngcontent-%COMP%]   .expandable-tree-node[_ngcontent-%COMP%]   .label-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}.io-tree[_ngcontent-%COMP%]   .expandable-tree-node[_ngcontent-%COMP%]   .label-container.has-extra-data[_ngcontent-%COMP%]{width:100%}.io-tree[_ngcontent-%COMP%]   .expandable-tree-node[_ngcontent-%COMP%]   .label-container.has-extra-data[_ngcontent-%COMP%]   .label[_ngcontent-%COMP%]{-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between}.io-tree[_ngcontent-%COMP%]   .expandable-tree-node[_ngcontent-%COMP%]   .label-container[_ngcontent-%COMP%]   .label[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}.io-tree[_ngcontent-%COMP%]   .expand-icon-button[_ngcontent-%COMP%]{padding:0;width:24px;height:24px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center}.io-tree[_ngcontent-%COMP%]   .expand-icon-button[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{font-size:18px;margin-top:6px}.io-tree[_ngcontent-%COMP%]   .locator-container[_ngcontent-%COMP%]{height:18px;opacity:.5;cursor:pointer;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}.io-tree[_ngcontent-%COMP%]   .locator-container[_ngcontent-%COMP%]   mat-icon.locator-icon[_ngcontent-%COMP%]{color:#333;font-size:16px;width:16px;height:16px}.io-tree[_ngcontent-%COMP%]   .locator-container[_ngcontent-%COMP%]:hover{opacity:.8}.io-tree[_ngcontent-%COMP%]   .match-types-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;justify-self:center;gap:2px;margin-left:8px}.io-tree[_ngcontent-%COMP%]   .extra-label-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;justify-self:center;padding:4px;border-radius:99px;background-color:#e8f0fe;-moz-box-sizing:border-box;box-sizing:border-box}.io-tree[_ngcontent-%COMP%]   .extra-label-container[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{font-size:12px;width:12px;height:12px;color:#9da3a9}.io-tree[_ngcontent-%COMP%]     .mat-mdc-button-touch-target{width:24px;height:24px}.io-tree-invisible[_ngcontent-%COMP%]{display:none}.io-tree[_ngcontent-%COMP%]   li[_ngcontent-%COMP%], .io-tree[_ngcontent-%COMP%]   ul[_ngcontent-%COMP%]{margin-top:0;margin-bottom:0;list-style-type:none}.io-tree[_ngcontent-%COMP%]   .mat-nested-tree-node[_ngcontent-%COMP%]   div[role=group][_ngcontent-%COMP%]{padding-left:8px}.io-tree[_ngcontent-%COMP%]   div[role=group][_ngcontent-%COMP%] > .mat-tree-node[_ngcontent-%COMP%]{padding-left:16px}  xap-inline-dialog-container:has(.model-explorer-const-values-popup){width:100%}  .model-explorer-const-values-popup{padding:8px;font-size:11px;line-height:12px;background-color:#fff}  .model-explorer-const-values-popup .values-count-msg{margin-bottom:8px;color:#999}  .model-explorer-const-values-popup .content{white-space:pre-wrap;font-family:monospace}"],
data:{animation:XJ},Ra:0});
function $K(a,b,c,d=[]){var e=[];let f={label:"<root>",children:[],type:b,Bd:!0};const g=[];for(let m=0;m<a.length;m++){const p=a[m];var k=e[m];const t=d[m];if(nI(p)&&p.hideInLayout)g.push({node:p,index:m,metadata:k});else{const w=aL(p.savedNamespace||p.namespace,f);if(w){null==w.children&&(w.children=[]);const B=w.children.find(G=>G.label===p.label&&G.Bd);null==B?(k={label:p.label,Ui:bL(k),nodeId:p.id,node:p,type:b,Cd:!0,Ln:p.id===c,Bd:1===p.nodeType,metadata:k},null!=t&&(k.vg=t),w.children.push(k)):
(B.nodeId=p.id,B.Cd=!0,B.vg=t)}}}f=cL(f);a=[f];if(0<g.length){d="weights";for(const m of g)if(!m.node.label.toLowerCase().includes("const")){d="<hidden>";break}a.push({label:d,children:g.map(m=>({label:m.node.label,Ui:bL(m.metadata),nodeId:m.node.id,node:m.node,type:b,Cd:!1,Ln:m.node.id===c,Bd:!1,metadata:e[m.index]})),type:b,Bd:!1})}return a}function bL(a){if(a)return a.__tensor_tag}
function aL(a,b){var c=a?["<root>",...a.split("/")]:["<root>"];let d;a=[];for(const e of c)"<root>"!==e&&a.push(e),"<root>"===e?d=b:(d&&!d.children&&(d.children=[]),c=d.children.find(f=>f.label===e),null==c?(c={label:e,children:[],type:b.type,Bd:!0},d.children.push(c),d=c):d=c);return d}
function cL(a){let b=a;const c=[];for(;;){c.push(b.label);let d;if((null==(d=b)?0:d.children)&&1===b.children.length&&0<(b.children[0].children||[]).length&&!b.Cd)b=b.children[0];else break}b!==a&&(b.label=c.join(" / "));return b};var dL={COLOR:"COLOR",Uo:"NUMBER"},eL=[{type:"COLOR",label:"Bg color",id:"node_bg_color",defaultValue:"#ffffff"},{type:"COLOR",label:"Border color",id:"node_border_color",defaultValue:"#777777"},{type:"COLOR",label:"Text color",id:"node_text_color",defaultValue:"#041e49"}];
function fL(a,b){const c=MI(b),d={};if(0<b.length&&b.some(e=>LI(e.queries))){const e=a.C.La();for(let f=0;f<e.length;f++){const g=e[f].modelGraph;if(g)for(const k of g.nodes)for(let m=0;m<c.length;m++){const p=c[m];if(LI(b[m].queries)&&NI(k,p.queries,g)){null==d[m]&&(d[m]={});null==d[m][f]&&(d[m][f]=[]);d[m][f].push(k);break}}}}a.D.set(d)}function gL(a,b){a.rules.set(hL(b))}
function iL(a){a.rules.update(b=>{b=[...b];b.push({queries:[{type:"node_type",nodeType:"op_nodes"},{type:"regex",queryRegex:"",matchTypes:["title"]}],nodeType:"op_nodes",styles:{},version:"v2"});return b})}function jL(a,b){a.rules.update(c=>{const d=[...c];c=c[b];d.splice(b,1);d.splice(b-1,0,c);return d})}function kL(a,b){a.rules.update(c=>{const d=[...c];c=c[b];d.splice(b,1);d.splice(b+1,0,c);return d})}
function lL(a,b){a.rules.update(c=>{const d=JSON.parse(JSON.stringify(c[b]));return[...c.slice(0,b),d,...c.slice(b)]})}function hL(a){return a.map(b=>{null==b.version&&(b.queries.unshift({type:"node_type",nodeType:b.nodeType}),b.version="v2");return b})}function mL(a,b,c){a.rules.update(d=>{d[b].queries=c;return[...d]})}
function nL(a,b,c,d){a.rules.update(e=>{const f=e[b];null==d?null==f.styles[c.id]?f.styles[c.id]={id:c.id,value:c.defaultValue}:delete f.styles[c.id]:d?f.styles[c.id]={id:c.id,value:c.defaultValue}:delete f.styles[c.id];return[...e]})}function oL(a,b,c){a.rules.update(d=>{d[b].nodeType=c;return[...d]})}function pL(a,b,c,d){a.rules.update(e=>{e[b].styles[c.id].value=d;return[...e]})}function qL(a,b){a.rules.update(c=>{c=[...c];c.splice(b,1);return c})}
var rL=class{constructor(a){this.C=a;this.rules=xr([]);this.D=xr({});this.jl=Bs(()=>0<this.rules().filter(b=>LI(b.queries)&&0<Object.keys(b.styles).length).length);Js(()=>{const b=this.rules();this.C.C||PI("model_explorer_node_styler_rules",JSON.stringify(b));fL(this,b)},{Wp:!0});this.C.C?(a=new URLSearchParams(document.location.search),a=decodeURIComponent(a.get("test_node_styler_rules")||""),gL(this,""===a?[]:JSON.parse(a))):(a=window.localStorage.getItem("model_explorer_node_styler_rules")||"",
a=""===a?[]:JSON.parse(a),gL(this,a))}};rL.K=function(a){return new (a||rL)(r(VJ),r(QI))};rL.pa=Md({ma:rL,ba:rL.K});function sL(a){a||(zg(),a=lf(yi));const b=new Xi(c=>a.ob(c.next.bind(c)));return c=>C(c,Lo(b))};function tL(a){zg();const b=lf(Wh),c=new mj(1),d=Js(()=>{let e;try{e=a()}catch(f){Ks(()=>c.error(f));return}Ks(()=>c.next(e))},{xa:b,LC:!0});b.get(yi).ob(()=>{d.destroy();c.complete()});return aj(c)};function uL(a,b,c){const d=vL(a,b);if(0<d.length){for(const e of d)null==e.results&&(e.results={}),null==e.results[b.id]&&null!=e.nodeDataProviderData&&(e.results[b.id]=wL(b,e.nodeDataProviderData));BJ(a.C,a.C.La()[c].id,d[0].runId)}}
function wL(a,b){xL(a);const c={};b=b[a.id];if(!b)return{};const d=[];for(var e of b.gradient||[]){var f={stop:e.stop};null!=e.bgColor&&(f.bgColor=yL(e.bgColor,"#ffffff"));null!=e.textColor&&(f.textColor=yL(e.textColor,"#000000"));d.push(f)}d.sort((m,p)=>m.stop-p.stop);e=Number.POSITIVE_INFINITY;f=Number.NEGATIVE_INFINITY;if(0<d.length)for(var {value:g}of Object.values(b.results))"number"===typeof g&&(e=Math.min(e,g),f=Math.max(f,g));for(const m of Object.keys(b.results)){g=b.results[m];"number"===
typeof g.value&&null==g.bgColor&&(g.bgColor=zL(g.value,b.thresholds||[],d,e,f));"number"===typeof g.value&&null==g.textColor&&(g.textColor=AL(g.value,b.thresholds||[],d,e,f));if((null==g.textColor||""===g.textColor)&&(g.bgColor||"").startsWith("#")){var k=yL(g.bgColor,"#ffffff");null!=k&&.38>.2126*Math.pow(k.r/255,2.2)+.7152*Math.pow(k.g/255,2.2)+.0722*Math.pow(k.b/255,2.2)&&(g.textColor="#ffffff")}k="-";k=g.value;k="number"===typeof k||"boolean"===typeof k||"string"===typeof k?`${k}`:JSON.stringify(k);
let p;const t=`${null!=(p=(a.outputTensorIdToNodeId||{})[m])?p:m}`,w=c[t];if(w){const B=w.allValues;B[m]=g.value;c[t]={value:g.value,bgColor:g.bgColor,textColor:g.textColor,allValues:B,strValue:`${w.strValue}, ${k}`}}else c[t]=Object.assign({},g,{strValue:k,allValues:{[m]:g.value}})}return c}
function BL(a,b,c,d,e,f=!1,g){const k=d.collectionLabel;a.D.update(m=>{if(f){var p=[];for(const [t,w]of Object.entries(m)){const B=t,G=w;f&&G.collectionId===k&&p.push(B)}for(const t of p)delete m[t]}m[b]={runId:b,runName:c,done:null==e?!1:!0,results:null==e?void 0:{[d.id]:wL(d,e)},extensionId:"",collectionId:k,remotePath:g,nodeDataProviderData:e};p=a.C.La();for(const t of p){let w,B;(null==(w=t.modelGraph)?void 0:w.id)===d.id&&(null==(B=t.modelGraph)?void 0:B.collectionLabel)===d.collectionLabel&&
BJ(a.C,t.id,b)}return Object.assign({},m)})}function CL(a,b,c,d,e){a.D.update(f=>{const g=f[b];g.done=!0;g.nodeDataProviderData=c;null==g.results&&(g.results={});g.results[d.id]=wL(d,c);e&&(g.error=e);return Object.assign({},f)})}function DL(a){const b=Object.values(a.D()).filter(c=>null!=c.remotePath).map(c=>c.remotePath);a.C.Mg.set(b)}
function EL(a,b,c){return sa(function*(){a.di.set(!0);var d=b.split("/"),e=d[d.length-1];let f;b.startsWith("node_data://")?(d=b.replace("node_data://","").split("/"),e=d[0],f=`${"/api/v1/load_node_data"}?node_data_index=${Number(d[1])}`):f=`${"/api/v1/read_text_file"}?path=${b}`;d=tI();BL(a,d,e,c,void 0,!1,b);e=yield fetch(f);if(e.ok)if(e=JSON.parse(yield e.text()),e.error)CL(a,d,{[c.id]:{results:{}}},c,`Failed to process JSON file. ${e.error}`);else try{const g=JSON.parse(e.content);e={};null!=
g.results&&null==g.results.results?c&&(e[c.id]=g):e=g;CL(a,d,e,c);DL(a)}catch(g){CL(a,d,{[c.id]:{results:{}}},c,`Failed to process JSON file. ${g}`)}else CL(a,d,{[c.id]:{results:{}}},c,`Failed to load JSON file "${b}"`);a.di.set(!1)})}function vL(a,b){const c=[];a=a.D();for(const d of Object.values(a))d.collectionId===b.collectionLabel&&(a=d.nodeDataProviderData)&&null!=a[b.id]&&c.push(d);return c}
function FL(a,b){a.D.update(c=>{delete c[b];return Object.assign({},c)});DL(a);for(const c of a.C.La())if(c.qs===b){const d=vL(a,c.modelGraph);BJ(a.C,c.id,0<d.length?d[0].runId:void 0)}}function GL(a,b,c){const d=CJ(a.C,b);if(d)return vL(a,c).find(e=>e.runId===d)}
function xL(a){if(null==a.outputTensorIdToNodeId){a.outputTensorIdToNodeId={};for(const b of a.nodes)if(nI(b)){const c=b.outputsMetadata||{};for(const d of Object.keys(c)){const e=c[d].tensor_name;null!=e&&(a.outputTensorIdToNodeId[e]=b.id)}}}}function yL(a,b){let c=a;a.startsWith("#")||(c=lI[a]);c||(c=b);c=c.replace("#","");return{r:HL(c.substring(0,2)),g:HL(c.substring(2,4)),b:HL(c.substring(4,6))}}
function zL(a,b,c,d,e){if(0<c.length)return IL(a,c,d,e,!0,"transparent");for(const f of b)if(a<=f.value)return f.bgColor;return"transparent"}function AL(a,b,c,d,e){if(0<c.length)return IL(a,c,d,e,!1,"");for(const f of b)if(a<=f.value)return f.textColor||"";return""}
function IL(a,b,c,d,e,f){d=(a-c)/(d-c);for(let g=0;g<b.length-1;g++){const k=b[g],m=b[g+1];a=e?k.bgColor:k.textColor;c=e?m.bgColor:m.textColor;if(d>=k.stop&&d<=m.stop){if(null==a||null==c)break;b=(d-k.stop)/(m.stop-k.stop);e=Math.floor(a.g+(c.g-a.g)*b);f=Math.floor(a.b+(c.b-a.b)*b);return`#${JL(Math.floor(a.r+(c.r-a.r)*b))}${JL(e)}${JL(f)}`}}return f}function JL(a){a=a.toString(16);return 1===a.length?`0${a}`:a}function HL(a){return/^[a-fA-F0-9]+$/.test(a)?parseInt(a,16):255}
var KL=class{constructor(a){this.C=a;this.D=xr({});this.di=xr(!1);this.F=tL(Bs(()=>this.C.La()[0].modelGraph));this.G=tL(Bs(()=>{const b=this.C.La();if(!(1>=b.length))return b[1].modelGraph}));this.F.subscribe(b=>{null!=b&&uL(this,b,0)});this.G.subscribe(b=>{null!=b&&uL(this,b,1)})}};KL.K=function(a){return new (a||KL)(r(VJ))};KL.pa=Md({ma:KL,ba:KL.K});/*

Copyright 2010-2017 Mike Bostock
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

 Redistributions of source code must retain the above copyright notice, this
  list of conditions and the following disclaimer.

 Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation
  and/or other materials provided with the distribution.

 Neither the name of the author nor the names of contributors may be used to
  endorse or promote products derived from this software without specific prior
  written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
var LL,ML,NL,OL,PL,QL,wI,xI,RL,SL;
(function(){function a(l){return function(){return this.matches(l)}}function b(l,h){return l<h?-1:l>h?1:l>=h?0:NaN}function c(l){return function(h,n){return b(l(h),n)}}function d(){for(var l=0,h=arguments.length,n={},q;l<h;++l){if(!(q=arguments[l]+"")||q in n)throw Error("illegal type: "+q);n[q]=[]}return new e(n)}function e(l){this.Gb=l}function f(l,h){return l.trim().split(/^|\s+/).map(function(n){var q="",v=n.indexOf(".");0<=v&&(q=n.slice(v+1),n=n.slice(0,v));if(n&&!h.hasOwnProperty(n))throw Error("unknown type: "+
n);return{type:n,name:q}})}function g(l,h,n){for(var q=0,v=l.length;q<v;++q)if(l[q].name===h){l[q]=hU;l=l.slice(0,q).concat(l.slice(q+1));break}null!=n&&l.push({name:h,value:n});return l}function k(l){var h=l,n=h.indexOf(":");0<=n&&"xmlns"!==(h=l.slice(0,n))&&(l=l.slice(n+1));return Qd.hasOwnProperty(h)?{space:Qd[h],local:l}:l}function m(l){return function(){var h=this.ownerDocument,n=this.namespaceURI;return"http://www.w3.org/1999/xhtml"===n&&"http://www.w3.org/1999/xhtml"===h.documentElement.namespaceURI?
h.createElement(l):h.createElementNS(n,l)}}function p(l){return function(){return this.ownerDocument.createElementNS(l.space,l.local)}}function t(l){l=k(l);return(l.local?p:m)(l)}function w(){}function B(l){return null==l?w:function(){return this.querySelector(l)}}function G(l){return Array(l.length)}function J(l,h){this.ownerDocument=l.ownerDocument;this.namespaceURI=l.namespaceURI;this.Oa=null;this.Qa=l;this.__data__=h}function N(l){return function(){return l}}function aa(l,h,n,q,v,y){for(var A=
0,E,K=h.length,L=y.length;A<L;++A)(E=h[A])?(E.__data__=y[A],q[A]=E):n[A]=new J(l,y[A]);for(;A<K;++A)if(E=h[A])v[A]=E}function ca(l,h,n,q,v,y,A){var E,K,L={},P=h.length,X=y.length,ka=Array(P),oa;for(E=0;E<P;++E)if(K=h[E])ka[E]=oa="$"+A.call(K,K.__data__,E,h),oa in L?v[E]=K:L[oa]=K;for(E=0;E<X;++E)oa="$"+A.call(l,y[E],E,y),(K=L[oa])?(q[E]=K,K.__data__=y[E],L[oa]=null):n[E]=new J(l,y[E]);for(E=0;E<P;++E)(K=h[E])&&L[ka[E]]===K&&(v[E]=K)}function ia(l,h){return l<h?-1:l>h?1:l>=h?0:NaN}function O(l){return function(){this.removeAttribute(l)}}
function T(l){return function(){this.removeAttributeNS(l.space,l.local)}}function Q(l,h){return function(){this.setAttribute(l,h)}}function la(l,h){return function(){this.setAttributeNS(l.space,l.local,h)}}function F(l,h){return function(){var n=h.apply(this,arguments);null==n?this.removeAttribute(l):this.setAttribute(l,n)}}function M(l,h){return function(){var n=h.apply(this,arguments);null==n?this.removeAttributeNS(l.space,l.local):this.setAttributeNS(l.space,l.local,n)}}function W(l){return function(){this.style.removeProperty(l)}}
function Aa(l,h,n){return function(){this.style.setProperty(l,h,n)}}function pb(l,h,n){return function(){var q=h.apply(this,arguments);null==q?this.style.removeProperty(l):this.style.setProperty(l,q,n)}}function qb(l,h){return l.style.getPropertyValue(h)||(l.ownerDocument&&l.ownerDocument.defaultView||l.document&&l||l.defaultView).getComputedStyle(l,null).getPropertyValue(h)}function Ia(l){return function(){delete this[l]}}function Vb(l,h){return function(){this[l]=h}}function Ma(l,h){return function(){var n=
h.apply(this,arguments);null==n?delete this[l]:this[l]=n}}function qa(){this.textContent=""}function U(l){return function(){this.textContent=l}}function ea(l){return function(){var h=l.apply(this,arguments);this.textContent=null==h?"":h}}function na(){return null}function Ja(){var l=this.parentNode;l&&l.removeChild(this)}function cb(){return this.parentNode.insertBefore(this.cloneNode(!1),this.nextSibling)}function Pa(){return this.parentNode.insertBefore(this.cloneNode(!0),this.nextSibling)}function ub(l,
h,n){l=pa(l,h,n);return function(q){var v=q.relatedTarget;v&&(v===this||v.compareDocumentPosition(this)&8)||l.call(this,q)}}function pa(l,h,n){return function(q){var v=LL;LL=q;try{l.call(this,this.__data__,h,n)}finally{LL=v}}}function Ha(l){return l.trim().split(/^|\s+/).map(function(h){var n="",q=h.indexOf(".");0<=q&&(n=h.slice(q+1),h=h.slice(0,q));return{type:h,name:n}})}function Ua(l){return function(){var h=this.__on;if(h){for(var n=0,q=-1,v=h.length,y;n<v;++n)(y=h[n],l.type&&y.type!==l.type||
y.name!==l.name)?h[++q]=y:this.removeEventListener(y.type,y.listener,y.capture);++q?h.length=q:delete this.__on}}}function kb(l,h,n){var q=HE.hasOwnProperty(l.type)?ub:pa;return function(v,y,A){v=this.__on;var E;y=q(h,y,A);if(v){A=0;for(var K=v.length;A<K;++A)if((E=v[A]).type===l.type&&E.name===l.name){this.removeEventListener(E.type,E.listener,E.capture);this.addEventListener(E.type,E.listener=y,E.capture=n);E.value=h;return}}this.addEventListener(l.type,y,n);E={type:l.type,name:l.name,value:h,listener:y,
capture:n};v?v.push(E):this.__on=[E]}}function xb(l,h){this.xc=l;this.cc=h}function Tc(){return new xb([[document.documentElement]],IE)}function ac(l){return"string"===typeof l?new xb([[document.querySelector(l)]],[document.documentElement]):new xb([[l]],IE)}function Xb(){this.Gb="@"+(++iU).toString(36)}function Jc(){for(var l=LL,h;h=l.ys;)l=h;return l}function Hf(l,h){var n=l.ownerSVGElement||l;if(n.createSVGPoint)return n=n.createSVGPoint(),n.x=h.clientX,n.y=h.clientY,n=n.matrixTransform(l.getScreenCTM().inverse()),
[n.x,n.y];n=l.getBoundingClientRect();return[h.clientX-n.left-l.clientLeft,h.clientY-n.top-l.clientTop]}function hd(l){var h=Jc();h.changedTouches&&(h=h.changedTouches[0]);return Hf(l,h)}function Ac(l,h,n){3>arguments.length&&(n=h,h=Jc().changedTouches);for(var q=0,v=h?h.length:0,y;q<v;++q)if((y=h[q]).identifier===n)return Hf(l,y);return null}function Vd(){LL.preventDefault();LL.stopImmediatePropagation()}function If(l){var h=l.document.documentElement;l=ac(l).on("dragstart.drag",Vd,!0);if("onselectstart"in
h)l.on("selectstart.drag",Vd,!0);else h.it=h.style.MozUserSelect,h.style.MozUserSelect="none"}function sr(l,h){var n=l.document.documentElement,q=ac(l).on("dragstart.drag",null);h&&(q.on("click.drag",Vd,!0),setTimeout(function(){q.on("click.drag",null)},0));if("onselectstart"in n)q.on("selectstart.drag",null);else n.style.MozUserSelect=n.it,delete n.it}function Kc(l,h,n){l.prototype=h.prototype=n;n.constructor=l}function cd(l,h){l=Object.create(l.prototype);for(var n in h)l[n]=h[n];return l}function Bc(){}
function Dd(l){var h;l=(l+"").trim().toLowerCase();return(h=jU.exec(l))?(h=parseInt(h[1],16),new Rc(h>>8&15|h>>4&240,h>>4&15|h&240,(h&15)<<4|h&15,1)):(h=kU.exec(l))?Pk(parseInt(h[1],16)):(h=lU.exec(l))?new Rc(h[1],h[2],h[3],1):(h=mU.exec(l))?new Rc(255*h[1]/100,255*h[2]/100,255*h[3]/100,1):(h=nU.exec(l))?qd(h[1],h[2],h[3],h[4]):(h=oU.exec(l))?qd(255*h[1]/100,255*h[2]/100,255*h[3]/100,h[4]):(h=pU.exec(l))?JE(h[1],h[2]/100,h[3]/100,1):(h=qU.exec(l))?JE(h[1],h[2]/100,h[3]/100,h[4]):KE.hasOwnProperty(l)?
Pk(KE[l]):"transparent"===l?new Rc(NaN,NaN,NaN,0):null}function Pk(l){return new Rc(l>>16&255,l>>8&255,l&255,1)}function qd(l,h,n,q){0>=q&&(l=h=n=NaN);return new Rc(l,h,n,q)}function ei(l){l instanceof Bc||(l=Dd(l));if(!l)return new Rc;l=l.pf();return new Rc(l.r,l.g,l.b,l.opacity)}function dm(l,h,n,q){return 1===arguments.length?ei(l):new Rc(l,h,n,null==q?1:q)}function Rc(l,h,n,q){this.r=+l;this.g=+h;this.b=+n;this.opacity=+q}function Ou(l){l=Math.max(0,Math.min(255,Math.round(l)||0));return(16>l?
"0":"")+l.toString(16)}function JE(l,h,n,q){0>=q?l=h=n=NaN:0>=n||1<=n?l=h=NaN:0>=h&&(l=NaN);return new Ne(l,h,n,q)}function rU(l){if(l instanceof Ne)return new Ne(l.h,l.s,l.l,l.opacity);l instanceof Bc||(l=Dd(l));if(!l)return new Ne;if(l instanceof Ne)return l;l=l.pf();var h=l.r/255,n=l.g/255,q=l.b/255,v=Math.min(h,n,q),y=Math.max(h,n,q),A=NaN,E=y-v,K=(y+v)/2;E?(A=h===y?(n-q)/E+6*(n<q):n===y?(q-h)/E+2:(h-n)/E+4,E/=.5>K?y+v:2-y-v,A*=60):E=0<K&&1>K?0:A;return new Ne(A,E,K,l.opacity)}function Pu(l,h,
n,q){return 1===arguments.length?rU(l):new Ne(l,h,n,null==q?1:q)}function Ne(l,h,n,q){this.h=+l;this.s=+h;this.l=+n;this.opacity=+q}function Qu(l,h,n){return 255*(60>l?h+(n-h)*l/60:180>l?n:240>l?h+(n-h)*(240-l)/60:h)}function Ru(l){if(l instanceof Oe)return new Oe(l.l,l.a,l.b,l.opacity);if(l instanceof Af){if(isNaN(l.h))return new Oe(l.l,0,0,l.opacity);var h=l.h*LE;return new Oe(l.l,Math.cos(h)*l.c,Math.sin(h)*l.c,l.opacity)}l instanceof Rc||(l=ei(l));var n=Su(l.r),q=Su(l.g),v=Su(l.b);h=Tu(.2225045*
n+.7168786*q+.0606169*v);if(n===q&&q===v)var y=n=h;else y=Tu((.4360747*n+.3850649*q+.1430804*v)/.96422),n=Tu((.0139322*n+.0971045*q+.7141733*v)/.82521);return new Oe(116*h-16,500*(y-h),200*(h-n),l.opacity)}function Oe(l,h,n,q){this.l=+l;this.a=+h;this.b=+n;this.opacity=+q}function Tu(l){return l>sU?Math.pow(l,1/3):l/ME+NE}function Uu(l){return l>Xj?l*l*l:ME*(l-NE)}function Vu(l){return 255*(.0031308>=l?12.92*l:1.055*Math.pow(l,1/2.4)-.055)}function Su(l){return.04045>=(l/=255)?l/12.92:Math.pow((l+
.055)/1.055,2.4)}function tU(l){if(l instanceof Af)return new Af(l.h,l.c,l.l,l.opacity);l instanceof Oe||(l=Ru(l));if(0===l.a&&0===l.b)return new Af(NaN,0,l.l,l.opacity);var h=Math.atan2(l.b,l.a)*OE;return new Af(0>h?h+360:h,Math.sqrt(l.a*l.a+l.b*l.b),l.l,l.opacity)}function Wu(l,h,n,q){return 1===arguments.length?tU(l):new Af(l,h,n,null==q?1:q)}function Af(l,h,n,q){this.h=+l;this.c=+h;this.l=+n;this.opacity=+q}function Pe(l,h,n,q){if(1===arguments.length){var v=l;if(v instanceof Gh)v=new Gh(v.h,
v.s,v.l,v.opacity);else{v instanceof Rc||(v=ei(v));var y=v.g/255,A=v.b/255,E=(PE*A+v.r/255*-1.7884503806-3.5172982438*y)/(PE+-1.7884503806-3.5172982438);A-=E;var K=(1.97294*(y-E)- -.29227*A)/-.90649;A=(y=Math.sqrt(K*K+A*A)/(1.97294*E*(1-E)))?Math.atan2(K,A)*OE-120:NaN;v=new Gh(0>A?A+360:A,y,E,v.opacity)}}else v=new Gh(l,h,n,null==q?1:q);return v}function Gh(l,h,n,q){this.h=+l;this.s=+h;this.l=+n;this.opacity=+q}function QE(l,h,n,q,v){var y=l*l,A=y*l;return((1-3*l+3*y-A)*h+(4-6*y+3*A)*n+(1+3*l+3*y-
3*A)*q+A*v)/6}function Np(l){return function(){return l}}function RE(l,h){return function(n){return l+n*h}}function uU(l,h,n){return l=Math.pow(l,n),h=Math.pow(h,n)-l,n=1/n,function(q){return Math.pow(l+q*h,n)}}function Xu(l,h){var n=h-l;return n?RE(l,180<n||-180>n?n-360*Math.round(n/360):n):Np(isNaN(l)?h:l)}function vU(l){return 1===(l=+l)?od:function(h,n){return n-h?uU(h,n,l):Np(isNaN(h)?n:h)}}function od(l,h){var n=h-l;return n?RE(l,n):Np(isNaN(l)?h:l)}function SE(l){return function(h){var n=h.length,
q=Array(n),v=Array(n),y=Array(n),A;for(A=0;A<n;++A){var E=dm(h[A]);q[A]=E.r||0;v[A]=E.g||0;y[A]=E.b||0}q=l(q);v=l(v);y=l(y);E.opacity=1;return function(K){E.r=q(K);E.g=v(K);E.b=y(K);return E+""}}}function wU(l,h){var n=h?h.length:0,q=l?Math.min(n,l.length):0,v=Array(q),y=Array(n),A;for(A=0;A<q;++A)v[A]=Yu(l[A],h[A]);for(;A<n;++A)y[A]=h[A];return function(E){for(A=0;A<q;++A)y[A]=v[A](E);return y}}function xU(l,h){var n=new Date;return l=+l,h-=l,function(q){return n.setTime(l+h*q),n}}function Qe(l,
h){return l=+l,h-=l,function(n){return l+h*n}}function yU(l,h){var n={},q={},v;if(null===l||"object"!==typeof l)l={};if(null===h||"object"!==typeof h)h={};for(v in h)v in l?n[v]=Yu(l[v],h[v]):q[v]=h[v];return function(y){for(v in n)q[v]=n[v](y);return q}}function zU(l){return function(){return l}}function AU(l){return function(h){return l(h)+""}}function TE(l,h){var n=Zu.lastIndex=$u.lastIndex=0,q,v,y,A=-1,E=[],K=[];l+="";for(h+="";(q=Zu.exec(l))&&(v=$u.exec(h));)(y=v.index)>n&&(y=h.slice(n,y),E[A]?
E[A]+=y:E[++A]=y),(q=q[0])===(v=v[0])?E[A]?E[A]+=v:E[++A]=v:(E[++A]=null,K.push({i:A,x:Qe(q,v)})),n=$u.lastIndex;n<h.length&&(y=h.slice(n),E[A]?E[A]+=y:E[++A]=y);return 2>E.length?K[0]?AU(K[0].x):zU(h):(h=K.length,function(L){for(var P=0,X;P<h;++P)E[(X=K[P]).i]=X.x(L);return E.join("")})}function Yu(l,h){var n=typeof h,q;return null==h||"boolean"===n?Np(h):("number"===n?Qe:"string"===n?(q=Dd(h))?(h=q,Op):TE:h instanceof Dd?Op:h instanceof Date?xU:Array.isArray(h)?wU:"function"!==typeof h.valueOf&&
"function"!==typeof h.toString||isNaN(h)?yU:Qe)(l,h)}function UE(l,h,n,q,v,y){var A,E,K;if(A=Math.sqrt(l*l+h*h))l/=A,h/=A;if(K=l*n+h*q)n-=l*K,q-=h*K;if(E=Math.sqrt(n*n+q*q))n/=E,q/=E,K/=E;l*q<h*n&&(l=-l,h=-h,K=-K,A=-A);return{translateX:v,translateY:y,rotate:Math.atan2(h,l)*VE,skewX:Math.atan(K)*VE,js:A,ks:E}}function WE(l,h,n,q){function v(L){return L.length?L.pop()+" ":""}function y(L,P,X,ka,oa,Ba){L!==X||P!==ka?(oa=oa.push("translate(",null,h,null,n),Ba.push({i:oa-4,x:Qe(L,X)},{i:oa-2,x:Qe(P,ka)})):
(X||ka)&&oa.push("translate("+X+h+ka+n)}function A(L,P,X,ka){L!==P?(180<L-P?P+=360:180<P-L&&(L+=360),ka.push({i:X.push(v(X)+"rotate(",null,q)-2,x:Qe(L,P)})):P&&X.push(v(X)+"rotate("+P+q)}function E(L,P,X,ka){L!==P?ka.push({i:X.push(v(X)+"skewX(",null,q)-2,x:Qe(L,P)}):P&&X.push(v(X)+"skewX("+P+q)}function K(L,P,X,ka,oa,Ba){L!==X||P!==ka?(oa=oa.push(v(oa)+"scale(",null,",",null,")"),Ba.push({i:oa-4,x:Qe(L,X)},{i:oa-2,x:Qe(P,ka)})):1===X&&1===ka||oa.push(v(oa)+"scale("+X+","+ka+")")}return function(L,
P){var X=[],ka=[];L=l(L);P=l(P);y(L.translateX,L.translateY,P.translateX,P.translateY,X,ka);A(L.rotate,P.rotate,X,ka);E(L.skewX,P.skewX,X,ka);K(L.js,L.ks,P.js,P.ks,X,ka);L=P=null;return function(oa){for(var Ba=-1,Va=ka.length,Ya;++Ba<Va;)X[(Ya=ka[Ba]).i]=Ya.x(oa);return X.join("")}}}function XE(l){return((l=Math.exp(l))+1/l)/2}function BU(l,h){var n=l[0],q=l[1],v=l[2];l=h[2];var y=h[0]-n,A=h[1]-q,E=y*y+A*A;if(1E-12>E){var K=Math.log(l/v)/em;l=function(X){return[n+X*y,q+X*A,v*Math.exp(em*X*K)]}}else{var L=
Math.sqrt(E);h=(l*l-v*v+4*E)/(4*v*L);l=(l*l-v*v-4*E)/(4*l*L);var P=Math.log(Math.sqrt(h*h+1)-h);K=(Math.log(Math.sqrt(l*l+1)-l)-P)/em;l=function(X){X*=K;var ka=XE(P),oa=em*X+P;var Ba=((oa=Math.exp(2*oa))-1)/(oa+1);var Va=P;oa=((Va=Math.exp(Va))-1/Va)/2;Ba=v/(2*L)*(ka*Ba-oa);return[n+Ba*y,q+Ba*A,v*ka/XE(em*X+P)]}}l.duration=1E3*K;return l}function YE(l){return function(h,n){var q=l((h=Pu(h)).h,(n=Pu(n)).h),v=od(h.s,n.s),y=od(h.l,n.l),A=od(h.opacity,n.opacity);return function(E){h.h=q(E);h.s=v(E);h.l=
y(E);h.opacity=A(E);return h+""}}}function ZE(l){return function(h,n){var q=l((h=Wu(h)).h,(n=Wu(n)).h),v=od(h.c,n.c),y=od(h.l,n.l),A=od(h.opacity,n.opacity);return function(E){h.h=q(E);h.c=v(E);h.l=y(E);h.opacity=A(E);return h+""}}}function $E(l){return function q(n){function v(y,A){var E=l((y=Pe(y)).h,(A=Pe(A)).h),K=od(y.s,A.s),L=od(y.l,A.l),P=od(y.opacity,A.opacity);return function(X){y.h=E(X);y.s=K(X);y.l=L(Math.pow(X,n));y.opacity=P(X);return y+""}}n=+n;v.gamma=q;return v}(1)}function Pp(){return Hh||
(aF(CU),Hh=fm.now()+Qp)}function CU(){Hh=0}function Rp(){this.mk=this.Ck=this.Oa=null}function bF(l,h,n){var q=new Rp;q.le(l,h,n);return q}function cF(){Hh=(Sp=fm.now())+Qp;Yj=gm=0;try{Pp();++Yj;for(var l=Tp,h;l;)0<=(h=Hh-l.Ck)&&l.mk.call(null,h),l=l.Oa;--Yj}finally{Yj=0;l=Tp;for(var n=Infinity;l;)if(l.mk){n>l.Ck&&(n=l.Ck);var q=l;l=l.Oa}else h=l.Oa,l.Oa=null,l=q?q.Oa=h:Tp=h;hm=q;av(n);Hh=0}}function DU(){var l=fm.now(),h=l-Sp;1E3<h&&(Qp-=h,Sp=l)}function av(l){Yj||(gm&&(gm=clearTimeout(gm)),24<l-
Hh?(Infinity>l&&(gm=setTimeout(cF,l-fm.now()-Qp)),im&&(im=clearInterval(im))):(im||(Sp=fm.now(),im=setInterval(DU,1E3)),Yj=1,aF(cF)))}function dF(l,h,n){var q=new Rp;h=null==h?0:+h;q.le(function(v){q.stop();l(v+h)},h,n);return q}function bv(l,h,n,q,v,y){var A=l.gg;if(!A)l.gg={};else if(n in A)return;EU(l,n,{name:h,index:q,group:v,on:FU,ld:GU,time:y.time,delay:y.delay,duration:y.duration,Nf:y.Nf,Vg:null,state:0})}function cv(l,h){l=Re(l,h);if(0<l.state)throw Error("too late; already scheduled");return l}
function Ih(l,h){l=Re(l,h);if(2<l.state)throw Error("too late; already started");return l}function Re(l,h){l=l.gg;if(!l||!(l=l[h]))throw Error("transition not found");return l}function EU(l,h,n){function q(K){var L,P;if(1!==n.state)return y();for(ka in A){var X=A[ka];if(X.name===n.name){if(3===X.state)return dF(q);4===X.state?(X.state=6,X.Vg.stop(),X.on.call("interrupt",l,l.__data__,X.index,X.group),delete A[ka]):+ka<h&&(X.state=6,X.Vg.stop(),delete A[ka])}}dF(function(){3===n.state&&(n.state=4,n.Vg.le(v,
n.delay,n.time),v(K))});n.state=2;n.on.call("start",l,l.__data__,n.index,n.group);if(2===n.state){n.state=3;E=Array(P=n.ld.length);var ka=0;for(L=-1;ka<P;++ka)if(X=n.ld[ka].value.call(l,l.__data__,n.index,n.group))E[++L]=X;E.length=L+1}}function v(K){K=K<n.duration?n.Nf.call(null,K/n.duration):(n.Vg.le(y),n.state=5,1);for(var L=-1,P=E.length;++L<P;)E[L].call(null,K);5===n.state&&(n.on.call("end",l,l.__data__,n.index,n.group),y())}function y(){n.state=6;n.Vg.stop();delete A[h];for(var K in A)return;
delete l.gg}var A=l.gg,E;A[h]=n;n.Vg=bF(function(K){n.state=1;n.Vg.le(q,n.delay,n.time);n.delay<=K&&q(K-n.delay)},0,n.time)}function Up(l,h){var n=l.gg,q,v=!0,y;if(n){h=null==h?null:h+"";for(y in n)if((q=n[y]).name!==h)v=!1;else{var A=2<q.state&&5>q.state;q.state=6;q.Vg.stop();A&&q.on.call("interrupt",l,l.__data__,q.index,q.group);delete n[y]}v&&delete l.gg}}function HU(l,h){var n,q;return function(){var v=Ih(this,l),y=v.ld;if(y!==n){q=n=y;y=0;for(var A=q.length;y<A;++y)if(q[y].name===h){q=q.slice();
q.splice(y,1);break}}v.ld=q}}function IU(l,h,n){var q,v;if("function"!==typeof n)throw Error();return function(){var y=Ih(this,l),A=y.ld;if(A!==q){v=(q=A).slice();A={name:h,value:n};for(var E=0,K=v.length;E<K;++E)if(v[E].name===h){v[E]=A;break}E===K&&v.push(A)}y.ld=v}}function dv(l,h,n){var q=l.Xb;l.each(function(){var v=Ih(this,q);(v.value||(v.value={}))[h]=n.apply(this,arguments)});return function(v){return Re(v,q).value[h]}}function eF(l,h){var n;return("number"===typeof h?Qe:h instanceof Dd?Op:
(n=Dd(h))?(h=n,Op):TE)(l,h)}function JU(l){return function(){this.removeAttribute(l)}}function KU(l){return function(){this.removeAttributeNS(l.space,l.local)}}function LU(l,h,n){var q,v;return function(){var y=this.getAttribute(l);return y===n?null:y===q?v:v=h(q=y,n)}}function MU(l,h,n){var q,v;return function(){var y=this.getAttributeNS(l.space,l.local);return y===n?null:y===q?v:v=h(q=y,n)}}function NU(l,h,n){var q,v,y;return function(){var A=n(this);if(null==A)return void this.removeAttribute(l);
var E=this.getAttribute(l);return E===A?null:E===q&&A===v?y:y=h(q=E,v=A)}}function OU(l,h,n){var q,v,y;return function(){var A=n(this);if(null==A)return void this.removeAttributeNS(l.space,l.local);var E=this.getAttributeNS(l.space,l.local);return E===A?null:E===q&&A===v?y:y=h(q=E,v=A)}}function PU(l,h){function n(){var q=this,v=h.apply(q,arguments);return v&&function(y){q.setAttributeNS(l.space,l.local,v(y))}}n.Lc=h;return n}function QU(l,h){function n(){var q=this,v=h.apply(q,arguments);return v&&
function(y){q.setAttribute(l,v(y))}}n.Lc=h;return n}function RU(l,h){return function(){cv(this,l).delay=+h.apply(this,arguments)}}function SU(l,h){return h=+h,function(){cv(this,l).delay=h}}function TU(l,h){return function(){Ih(this,l).duration=+h.apply(this,arguments)}}function UU(l,h){return h=+h,function(){Ih(this,l).duration=h}}function VU(l,h){if("function"!==typeof h)throw Error();return function(){Ih(this,l).Nf=h}}function WU(l){return(l+"").trim().split(/^|\s+/).every(function(h){var n=h.indexOf(".");
0<=n&&(h=h.slice(0,n));return!h||"start"===h})}function XU(l,h,n){var q,v,y=WU(h)?cv:Ih;return function(){var A=y(this,l),E=A.on;if(E!==q)(v=(q=E).copy()).on(h,n);A.on=v}}function YU(l){return function(){var h=this.parentNode,n;for(n in this.gg)if(+n!==l)return;h&&h.removeChild(this)}}function ZU(l,h){var n,q,v;return function(){var y=qb(this,l),A=(this.style.removeProperty(l),qb(this,l));return y===A?null:y===n&&A===q?v:v=h(n=y,q=A)}}function $U(l){return function(){this.style.removeProperty(l)}}
function aV(l,h,n){var q,v;return function(){var y=qb(this,l);return y===n?null:y===q?v:v=h(q=y,n)}}function bV(l,h,n){var q,v,y;return function(){var A=qb(this,l),E=n(this);null==E&&(E=(this.style.removeProperty(l),qb(this,l)));return A===E?null:A===q&&E===v?y:y=h(q=A,v=E)}}function cV(l,h,n){function q(){var v=this,y=h.apply(v,arguments);return y&&function(A){v.style.setProperty(l,y(A),n)}}q.Lc=h;return q}function dV(l){return function(){this.textContent=l}}function eV(l){return function(){var h=
l(this);this.textContent=null==h?"":h}}function rg(l,h,n,q){this.xc=l;this.cc=h;this.qh=n;this.Xb=q}function ev(l){return{type:l}}function fv(){this.qc=this.yc=this.Jb=this.Mb=null;this.Gb=""}function fF(){return new fv}function Vp(){}function gF(l,h){var n=new Vp;if(l instanceof Vp)l.each(function(A,E){n.set(E,A)});else if(Array.isArray(l)){var q=-1,v=l.length,y;if(null==h)for(;++q<v;)n.set(q,l[q]);else for(;++q<v;)n.set(h(y=l[q],q,l),y)}else if(l)for(q in l)n.set(q,l[q]);return n}function Wp(){}
function hF(l){return function(h){for(var n={},q=0;q<l.length;q++)n[l[q]]=h[q]||"";return n}}function fV(l,h){var n=hF(l);return function(q,v){return h(n(q),v,l)}}function gV(l){var h=Object.create(null),n=[];l.forEach(function(q){for(var v in q)v in h||n.push(h[v]=v)});return n}function iF(l){function h(A,E){function K(){if(Ba)return gv;if(Va)return Va=!1,jF;var kc,vb=X,lc;if(34===A.charCodeAt(vb)){for(;X++<P&&34!==A.charCodeAt(X)||34===A.charCodeAt(++X););(kc=X)>=P?Ba=!0:10===(lc=A.charCodeAt(X++))?
Va=!0:13===lc&&(Va=!0,10===A.charCodeAt(X)&&++X);return A.slice(vb+1,kc-1).replace(/""/g,'"')}for(;X<P;){if(10===(lc=A.charCodeAt(kc=X++)))Va=!0;else if(13===lc)Va=!0,10===A.charCodeAt(X)&&++X;else if(lc!==y)continue;return A.slice(vb,kc)}return Ba=!0,A.slice(vb,P)}var L=[],P=A.length,X=0,ka=0,oa,Ba=0>=P,Va=!1;10===A.charCodeAt(P-1)&&--P;for(13===A.charCodeAt(P-1)&&--P;(oa=K())!==gv;){for(var Ya=[];oa!==jF&&oa!==gv;)Ya.push(oa),oa=K();E&&null==(Ya=E(Ya,ka++))||L.push(Ya)}return L}function n(A){return A.map(q).join(l)}
function q(A){return null==A?"":v.test(A)?'"'+A.replace(/"/g,'""')+'"':A}var v=new RegExp('["'+l+"\n\r]"),y=l.charCodeAt(0);return{parse:function(A,E){var K,L;A=h(A,function(P,X){if(K)return K(P,X-1);L=P;K=E?fV(P,E):hF(P)});A.AF=L||[];return A},iG:h,format:function(A,E){null==E&&(E=gV(A));return[E.map(q).join(l)].concat(A.map(function(K){return E.map(function(L){return q(K[L])}).join(l)})).join("\n")},QF:function(A){return A.map(n).join("\n")}}}function hV(l){if(!l.ok)throw Error(l.status+" "+l.statusText);
return l.text()}function kF(l){return function(h,n,q){2===arguments.length&&"function"===typeof n&&(q=n,n=void 0);return fetch(h,n).then(hV).then(function(v){return l(v,q)})}}function lF(l,h,n,q){if(isNaN(h)||isNaN(n))return l;var v,y=l.vb;q={data:q};var A=l.qc,E=l.yc,K=l.Jb,L=l.Mb,P,X,ka,oa,Ba;if(!y)return l.vb=q,l;for(;y.length;)if((ka=h>=(P=(A+K)/2))?A=P:K=P,(oa=n>=(X=(E+L)/2))?E=X:L=X,v=y,!(y=y[Ba=oa<<1|ka]))return v[Ba]=q,l;var Va=+l.mg.call(null,y.data);var Ya=+l.ng.call(null,y.data);if(h===
Va&&n===Ya)return q.next=y,v?v[Ba]=q:l.vb=q,l;do v=v?v[Ba]=Array(4):l.vb=Array(4),(ka=h>=(P=(A+K)/2))?A=P:K=P,(oa=n>=(X=(E+L)/2))?E=X:L=X;while((Ba=oa<<1|ka)===(ka=(Ya>=X)<<1|Va>=P));return v[ka]=y,v[Ba]=q,l}function fd(l,h,n,q,v){this.node=l;this.Mo=h;this.No=n;this.x1=q;this.y1=v}function iV(l){return l[0]}function jV(l){return l[1]}function hv(l,h,n,q,v,y){this.mg=l;this.ng=h;this.qc=n;this.yc=q;this.Jb=v;this.Mb=y;this.vb=void 0}function mF(l){for(var h={data:l.data},n=h;l=l.next;)n=n.next={data:l.data};
return h}function Xp(l,h){if(0>(h=(l=h?l.toExponential(h-1):l.toExponential()).indexOf("e")))return null;var n=l.slice(0,h);return[1<n.length?n[0]+n.slice(2):n,+l.slice(h+1)]}function kV(l){return l=Xp(Math.abs(l)),l?l[1]:NaN}function lV(l,h){return function(n,q){for(var v=n.length,y=[],A=0,E=l[0],K=0;0<v&&0<E;){K+E+1>q&&(E=Math.max(1,q-K));y.push(n.substring(v-=E,v+E));if((K+=E+1)>q)break;E=l[A=(A+1)%l.length]}return y.reverse().join(h)}}function mV(l){return function(h){return h.replace(/[0-9]/g,
function(n){return l[+n]})}}function iv(l){return new jv(l)}function jv(l){if(!(h=nV.exec(l)))throw Error("invalid format: "+l);var h;this.fill=h[1]||" ";this.align=h[2]||">";this.sign=h[3]||"-";this.symbol=h[4]||"";this.zero=!!h[5];this.width=h[6]&&+h[6];this.dv=!!h[7];this.precision=h[8]&&+h[8].slice(1);this.trim=!!h[9];this.type=h[10]||""}function nF(l,h){h=Xp(l,h);if(!h)return l+"";l=h[0];h=h[1];return 0>h?"0."+Array(-h).join("0")+l:l.length>h+1?l.slice(0,h+1)+"."+l.slice(h+1):l+Array(h-l.length+
2).join("0")}function oF(l){return l}function oV(l){function h(E){function K(La){var Hb=lc,za=Hc,Fa;if("c"===vb)za=Eb(La)+za,La="";else{La=+La;var I=0>La;La=Eb(Math.abs(La),Ya);if(kc){var ha=La.length;var Ga=1,Ta=-1,wb;a:for(;Ga<ha;++Ga)switch(La[Ga]){case ".":Ta=wb=Ga;break;case "0":0===Ta&&(Ta=Ga);wb=Ga;break;default:if(!+La[Ga])break a;0<Ta&&(Ta=0)}La=0<Ta?La.slice(0,Ta)+La.slice(wb+1):La}I&&0===+La&&(I=!1);Hb=(I?"("===X?X:"-":"-"===X||"("===X?"":X)+Hb;za=("s"===vb?pF[8+qF/3]:"")+za+(I&&"("===
X?")":"");if(tb)for(I=-1,ha=La.length;++I<ha;)if(Fa=La.charCodeAt(I),48>Fa||57<Fa){za=(46===Fa?v+La.slice(I+1):La.slice(I))+za;La=La.slice(0,I);break}}Va&&!oa&&(La=n(La,Infinity));Fa=Hb.length+La.length+za.length;I=Fa<Ba?Array(Ba-Fa+1).join(L):"";Va&&oa&&(La=n(I+La,I.length?Ba-za.length:Infinity),I="");switch(P){case "<":La=Hb+La+za+I;break;case "=":La=Hb+I+La+za;break;case "^":La=I.slice(0,Fa=I.length>>1)+Hb+La+za+I.slice(Fa);break;default:La=I+Hb+La+za}return y(La)}E=iv(E);var L=E.fill,P=E.align,
X=E.sign,ka=E.symbol,oa=E.zero,Ba=E.width,Va=E.dv,Ya=E.precision,kc=E.trim,vb=E.type;"n"===vb?(Va=!0,vb="g"):rF[vb]||(null==Ya&&(Ya=12),kc=!0,vb="g");if(oa||"0"===L&&"="===P)oa=!0,L="0",P="=";var lc="$"===ka?q[0]:"#"===ka&&/[boxX]/.test(vb)?"0"+vb.toLowerCase():"",Hc="$"===ka?q[1]:/[%p]/.test(vb)?A:"",Eb=rF[vb],tb=/[defgprs%]/.test(vb);Ya=null==Ya?6:/[gprs]/.test(vb)?Math.max(1,Math.min(21,Ya)):Math.max(0,Math.min(20,Ya));K.toString=function(){return E+""};return K}var n=l.Yv&&l.my?lV(l.Yv,l.my):
oF,q=l.currency,v=l.wB,y=l.fD?mV(l.fD):oF,A=l.jG||"%";return{format:h,PF:function(E,K){var L=h((E=iv(E),E.type="f",E));E=3*Math.max(-8,Math.min(8,Math.floor(kV(K)/3)));var P=Math.pow(10,-E),X=pF[8+E/3];return function(ka){return L(P*ka)+X}}}}function je(){this.reset()}function sF(l,h,n){var q=l.s=h+n,v=q-h;l.t=h-(q-v)+(n-v)}function Yp(l){return 1<l?Rd:-1>l?-Rd:Math.asin(l)}function pV(){}function tF(l){var h=l[0];l=l[1];var n=Sd(l);return[n*Sd(h),n*ad(h),ad(l)]}function uF(l,h){return[l[1]*h[2]-
l[2]*h[1],l[2]*h[0]-l[0]*h[2],l[0]*h[1]-l[1]*h[0]]}function vF(l){var h=Zp(l[0]*l[0]+l[1]*l[1]+l[2]*l[2]);l[0]/=h;l[1]/=h;l[2]/=h}function qV(){var l=[],h;return{point:function(n,q){h.push([n,q])},kc:function(){l.push(h=[])},jc:pV,nG:function(){1<l.length&&l.push(l.pop().concat(l.shift()))},result:function(){var n=l;l=[];h=null;return n}}}function $p(l,h,n,q){this.x=l;this.z=h;this.ho=n;this.e=q;this.v=!1;this.n=this.p=null}function rV(l,h,n,q,v){var y=[],A=[];l.forEach(function(X){if(!(0>=(ka=X.length-
1))){var ka,oa=X[0],Ba=X[ka],Va=oa;if(1E-6>sg(Va[0]-Ba[0])&&1E-6>sg(Va[1]-Ba[1])){v.kc();for(E=0;E<ka;++E)v.point((oa=X[E])[0],oa[1]);v.jc()}else y.push(ka=new $p(oa,X,null,!0)),A.push(ka.ho=new $p(oa,null,ka,!1)),y.push(ka=new $p(Ba,X,null,!1)),A.push(ka.ho=new $p(Ba,null,ka,!0))}});if(y.length){A.sort(h);wF(y);wF(A);var E=0;for(l=A.length;E<l;++E)A[E].e=n=!n;n=y[0];for(var K;;){for(var L=n,P=!0;L.v;)if((L=L.n)===n)return;h=L.z;v.kc();do{L.v=L.ho.v=!0;if(L.e){if(P)for(E=0,l=h.length;E<l;++E)v.point((K=
h[E])[0],K[1]);else q(L.x,L.n.x,1,v);L=L.n}else{if(P)for(h=L.p.z,E=h.length-1;0<=E;--E)v.point((K=h[E])[0],K[1]);else q(L.x,L.p.x,-1,v);L=L.p}L=L.ho;h=L.z;P=!P}while(!L.v);v.jc()}}}function wF(l){if(h=l.length){for(var h,n=0,q=l[0],v;++n<h;)q.n=v=l[n],v.p=q,q=v;q.n=v=l[0];v.p=q}}function sV(l){return 1<l.length}function tV(l,h){return(0>(l=l.x)[0]?l[1]-Rd-1E-6:Rd-l[1])-(0>(h=h.x)[0]?h[1]-Rd-1E-6:Rd-h[1])}function kv(){}function xF(l){return function(h,n){var q=Sd(h),v=Sd(n);q=l(q*v);return[q*v*ad(h),
q*ad(n)]}}function jm(l){return function(h,n){var q=Zp(h*h+n*n),v=l(q),y=ad(v);v=Sd(v);return[yF(h*y,q*v),Yp(q&&n*y/q)]}}function uV(l){var h=0,n=l.children,q=n&&n.length;if(q)for(;0<=--q;)h+=n[q].value;else h=1;l.value=h}function zF(l,h){var n=new km(l);l=+l.value&&(n.value=l.value);var q,v=[n],y,A,E,K;for(null==h&&(h=vV);q=v.pop();)if(l&&(q.value=+q.data.value),(A=h(q.data))&&(K=A.length))for(q.children=Array(K),E=K-1;0<=E;--E)v.push(y=q.children[E]=new km(A[E])),y.parent=q,y.depth=q.depth+1;return n.yq(wV)}
function vV(l){return l.children}function xV(l){l.data=l.data.data}function wV(l){var h=0;do l.height=h;while((l=l.parent)&&l.height<++h)}function km(l){this.data=l;this.depth=this.height=0;this.parent=null}function Ic(l,h,n,q){function v(y){return l(y=new Date(+y)),y}v.floor=v;v.ceil=function(y){return l(y=new Date(y-1)),h(y,1),l(y),y};v.round=function(y){var A=v(y),E=v.ceil(y);return y-A<E-y?A:E};v.offset=function(y,A){return h(y=new Date(+y),null==A?1:Math.floor(A)),y};v.filter=function(y){return Ic(function(A){if(A>=
A)for(;l(A),!y(A);)A.setTime(A-1)},function(A,E){if(A>=A)if(0>E)for(;0>=++E;)for(;h(A,-1),!y(A););else for(;0<=--E;)for(;h(A,1),!y(A););})};n&&(v.count=function(y,A){lv.setTime(+y);mv.setTime(+A);l(lv);l(mv);return Math.floor(n(lv,mv))},v.every=function(y){y=Math.floor(y);return isFinite(y)&&0<y?1<y?v.filter(q?function(A){return 0===q(A)%y}:function(A){return 0===v.count(0,A)%y}):v:null});return v}function Jh(l){return Ic(function(h){h.setDate(h.getDate()-(h.getDay()+7-l)%7);h.setHours(0,0,0,0)},
function(h,n){h.setDate(h.getDate()+7*n)},function(h,n){return(n-h-6E4*(n.getTimezoneOffset()-h.getTimezoneOffset()))/6048E5})}function Kh(l){return Ic(function(h){h.setUTCDate(h.getUTCDate()-(h.getUTCDay()+7-l)%7);h.setUTCHours(0,0,0,0)},function(h,n){h.setUTCDate(h.getUTCDate()+7*n)},function(h,n){return(n-h)/6048E5})}function yV(l){if(0<=l.y&&100>l.y){var h=new Date(-1,l.ge,l.d,l.uf,l.dh,l.hk,l.vi);h.setFullYear(l.y);return h}return new Date(l.y,l.ge,l.d,l.uf,l.dh,l.hk,l.vi)}function aq(l){if(0<=
l.y&&100>l.y){var h=new Date(Date.UTC(-1,l.ge,l.d,l.uf,l.dh,l.hk,l.vi));h.setUTCFullYear(l.y);return h}return new Date(Date.UTC(l.y,l.ge,l.d,l.uf,l.dh,l.hk,l.vi))}function lm(l){return{y:l,ge:0,d:1,uf:0,dh:0,hk:0,vi:0}}function zV(l){function h(za,Fa){return function(I){var ha=[],Ga=-1,Ta=0,wb=za.length,Qa,Rb;for(I instanceof Date||(I=new Date(+I));++Ga<wb;)if(37===za.charCodeAt(Ga)){ha.push(za.slice(Ta,Ga));null!=(Ta=AF[Qa=za.charAt(++Ga)])?Qa=za.charAt(++Ga):Ta="e"===Qa?" ":"0";if(Rb=Fa[Qa])Qa=
Rb(I,Ta);ha.push(Qa);Ta=Ga+1}ha.push(za.slice(Ta,Ga));return ha.join("")}}function n(za,Fa){return function(I){var ha=lm(1900);if(q(ha,za,I,0)!=I.length)return null;if("Q"in ha)return new Date(ha.Q);"p"in ha&&(ha.uf=ha.uf%12+12*ha.p);if("V"in ha){if(1>ha.om||53<ha.om)return null;"w"in ha||(ha.w=1);if("Z"in ha){I=aq(lm(ha.y));var Ga=I.getUTCDay();I=4<Ga||0===Ga?nv.ceil(I):nv(I);I=BF.offset(I,7*(ha.om-1));ha.y=I.getUTCFullYear();ha.ge=I.getUTCMonth();ha.d=I.getUTCDate()+(ha.w+6)%7}else I=Fa(lm(ha.y)),
Ga=I.getDay(),I=4<Ga||0===Ga?ov.ceil(I):ov(I),I=CF.offset(I,7*(ha.om-1)),ha.y=I.getFullYear(),ha.ge=I.getMonth(),ha.d=I.getDate()+(ha.w+6)%7}else if("W"in ha||"U"in ha)"w"in ha||(ha.w="u"in ha?ha.KE%7:"W"in ha?1:0),Ga="Z"in ha?aq(lm(ha.y)).getUTCDay():Fa(lm(ha.y)).getDay(),ha.ge=0,ha.d="W"in ha?(ha.w+6)%7+7*ha.W-(Ga+5)%7:ha.w+7*ha.U-(Ga+6)%7;return"Z"in ha?(ha.uf+=ha.Z/100|0,ha.dh+=ha.Z%100,aq(ha)):Fa(ha)}}function q(za,Fa,I,ha){for(var Ga=0,Ta=Fa.length,wb=I.length,Qa;Ga<Ta;){if(ha>=wb)return-1;
Qa=Fa.charCodeAt(Ga++);if(37===Qa){if(Qa=Fa.charAt(Ga++),Qa=Hb[Qa in AF?Fa.charAt(Ga++):Qa],!Qa||0>(ha=Qa(za,I,ha)))return-1}else if(Qa!=I.charCodeAt(ha++))return-1}return ha}var v=l.dateTime,y=l.uB,A=l.time,E=l.tD,K=l.vB,L=l.TD,P=l.RC,X=l.UD,ka=mm(E),oa=nm(E),Ba=mm(K),Va=nm(K),Ya=mm(L),kc=nm(L),vb=mm(P),lc=nm(P),Hc=mm(X),Eb=nm(X),tb={a:function(za){return L[za.getDay()]},A:function(za){return K[za.getDay()]},b:function(za){return X[za.getMonth()]},B:function(za){return P[za.getMonth()]},c:null,d:DF,
e:DF,f:AV,H:BV,I:CV,j:DV,L:EF,m:EV,M:FV,p:function(za){return E[+(12<=za.getHours())]},Q:FF,s:GF,S:GV,u:HV,U:IV,V:JV,w:KV,W:LV,x:null,X:null,y:MV,Y:NV,Z:OV,"%":HF},La={a:function(za){return L[za.getUTCDay()]},A:function(za){return K[za.getUTCDay()]},b:function(za){return X[za.getUTCMonth()]},B:function(za){return P[za.getUTCMonth()]},c:null,d:IF,e:IF,f:PV,H:QV,I:RV,j:SV,L:JF,m:TV,M:UV,p:function(za){return E[+(12<=za.getUTCHours())]},Q:FF,s:GF,S:VV,u:WV,U:XV,V:YV,w:ZV,W:$V,x:null,X:null,y:aW,Y:bW,
Z:cW,"%":HF},Hb={a:function(za,Fa,I){return(Fa=Ya.exec(Fa.slice(I)))?(za.w=kc[Fa[0].toLowerCase()],I+Fa[0].length):-1},A:function(za,Fa,I){return(Fa=Ba.exec(Fa.slice(I)))?(za.w=Va[Fa[0].toLowerCase()],I+Fa[0].length):-1},b:function(za,Fa,I){return(Fa=Hc.exec(Fa.slice(I)))?(za.ge=Eb[Fa[0].toLowerCase()],I+Fa[0].length):-1},B:function(za,Fa,I){return(Fa=vb.exec(Fa.slice(I)))?(za.ge=lc[Fa[0].toLowerCase()],I+Fa[0].length):-1},c:function(za,Fa,I){return q(za,v,Fa,I)},d:KF,e:KF,f:dW,H:LF,I:LF,j:eW,L:fW,
m:gW,M:hW,p:function(za,Fa,I){return(Fa=ka.exec(Fa.slice(I)))?(za.p=oa[Fa[0].toLowerCase()],I+Fa[0].length):-1},Q:iW,s:jW,S:kW,u:lW,U:mW,V:nW,w:oW,W:pW,x:function(za,Fa,I){return q(za,y,Fa,I)},X:function(za,Fa,I){return q(za,A,Fa,I)},y:qW,Y:rW,Z:sW,"%":tW};tb.x=h(y,tb);tb.X=h(A,tb);tb.c=h(v,tb);La.x=h(y,La);La.X=h(A,La);La.c=h(v,La);return{format:function(za){var Fa=h(za,tb);Fa.toString=function(){return za};return Fa},parse:function(za){var Fa=n(za,yV);Fa.toString=function(){return za};return Fa},
NE:function(za){var Fa=h(za,La);Fa.toString=function(){return za};return Fa},OE:function(za){var Fa=n(za,aq);Fa.toString=function(){return za};return Fa}}}function Sb(l,h,n){var q=0>l?"-":"";l=(q?-l:l)+"";var v=l.length;return q+(v<n?Array(n-v+1).join(h)+l:l)}function uW(l){return l.replace(vW,"\\$&")}function mm(l){return new RegExp("^(?:"+l.map(uW).join("|")+")","i")}function nm(l){for(var h={},n=-1,q=l.length;++n<q;)h[l[n].toLowerCase()]=n;return h}function oW(l,h,n){return(h=Sc.exec(h.slice(n,
n+1)))?(l.w=+h[0],n+h[0].length):-1}function lW(l,h,n){return(h=Sc.exec(h.slice(n,n+1)))?(l.KE=+h[0],n+h[0].length):-1}function mW(l,h,n){return(h=Sc.exec(h.slice(n,n+2)))?(l.U=+h[0],n+h[0].length):-1}function nW(l,h,n){return(h=Sc.exec(h.slice(n,n+2)))?(l.om=+h[0],n+h[0].length):-1}function pW(l,h,n){return(h=Sc.exec(h.slice(n,n+2)))?(l.W=+h[0],n+h[0].length):-1}function rW(l,h,n){return(h=Sc.exec(h.slice(n,n+4)))?(l.y=+h[0],n+h[0].length):-1}function qW(l,h,n){return(h=Sc.exec(h.slice(n,n+2)))?
(l.y=+h[0]+(68<+h[0]?1900:2E3),n+h[0].length):-1}function sW(l,h,n){return(h=/^(Z)|([+-]\d\d)(?::?(\d\d))?/.exec(h.slice(n,n+6)))?(l.Z=h[1]?0:-(h[2]+(h[3]||"00")),n+h[0].length):-1}function gW(l,h,n){return(h=Sc.exec(h.slice(n,n+2)))?(l.ge=h[0]-1,n+h[0].length):-1}function KF(l,h,n){return(h=Sc.exec(h.slice(n,n+2)))?(l.d=+h[0],n+h[0].length):-1}function eW(l,h,n){return(h=Sc.exec(h.slice(n,n+3)))?(l.ge=0,l.d=+h[0],n+h[0].length):-1}function LF(l,h,n){return(h=Sc.exec(h.slice(n,n+2)))?(l.uf=+h[0],
n+h[0].length):-1}function hW(l,h,n){return(h=Sc.exec(h.slice(n,n+2)))?(l.dh=+h[0],n+h[0].length):-1}function kW(l,h,n){return(h=Sc.exec(h.slice(n,n+2)))?(l.hk=+h[0],n+h[0].length):-1}function fW(l,h,n){return(h=Sc.exec(h.slice(n,n+3)))?(l.vi=+h[0],n+h[0].length):-1}function dW(l,h,n){return(h=Sc.exec(h.slice(n,n+6)))?(l.vi=Math.floor(h[0]/1E3),n+h[0].length):-1}function tW(l,h,n){return(l=wW.exec(h.slice(n,n+1)))?n+l[0].length:-1}function iW(l,h,n){return(h=Sc.exec(h.slice(n)))?(l.Q=+h[0],n+h[0].length):
-1}function jW(l,h,n){return(h=Sc.exec(h.slice(n)))?(l.Q=1E3*+h[0],n+h[0].length):-1}function DF(l,h){return Sb(l.getDate(),h,2)}function BV(l,h){return Sb(l.getHours(),h,2)}function CV(l,h){return Sb(l.getHours()%12||12,h,2)}function DV(l,h){return Sb(1+CF.count(Zj(l),l),h,3)}function EF(l,h){return Sb(l.getMilliseconds(),h,3)}function AV(l,h){return EF(l,h)+"000"}function EV(l,h){return Sb(l.getMonth()+1,h,2)}function FV(l,h){return Sb(l.getMinutes(),h,2)}function GV(l,h){return Sb(l.getSeconds(),
h,2)}function HV(l){l=l.getDay();return 0===l?7:l}function IV(l,h){return Sb(xW.count(Zj(l),l),h,2)}function JV(l,h){var n=l.getDay();l=4<=n||0===n?pv(l):pv.ceil(l);return Sb(pv.count(Zj(l),l)+(4===Zj(l).getDay()),h,2)}function KV(l){return l.getDay()}function LV(l,h){return Sb(ov.count(Zj(l),l),h,2)}function MV(l,h){return Sb(l.getFullYear()%100,h,2)}function NV(l,h){return Sb(l.getFullYear()%1E4,h,4)}function OV(l){l=l.getTimezoneOffset();return(0<l?"-":(l*=-1,"+"))+Sb(l/60|0,"0",2)+Sb(l%60,"0",
2)}function IF(l,h){return Sb(l.getUTCDate(),h,2)}function QV(l,h){return Sb(l.getUTCHours(),h,2)}function RV(l,h){return Sb(l.getUTCHours()%12||12,h,2)}function SV(l,h){return Sb(1+BF.count(ak(l),l),h,3)}function JF(l,h){return Sb(l.getUTCMilliseconds(),h,3)}function PV(l,h){return JF(l,h)+"000"}function TV(l,h){return Sb(l.getUTCMonth()+1,h,2)}function UV(l,h){return Sb(l.getUTCMinutes(),h,2)}function VV(l,h){return Sb(l.getUTCSeconds(),h,2)}function WV(l){l=l.getUTCDay();return 0===l?7:l}function XV(l,
h){return Sb(yW.count(ak(l),l),h,2)}function YV(l,h){var n=l.getUTCDay();l=4<=n||0===n?qv(l):qv.ceil(l);return Sb(qv.count(ak(l),l)+(4===ak(l).getUTCDay()),h,2)}function ZV(l){return l.getUTCDay()}function $V(l,h){return Sb(nv.count(ak(l),l),h,2)}function aW(l,h){return Sb(l.getUTCFullYear()%100,h,2)}function bW(l,h){return Sb(l.getUTCFullYear()%1E4,h,4)}function cW(){return"+0000"}function HF(){return"%"}function FF(l){return+l}function GF(l){return Math.floor(+l/1E3)}function ob(l){for(var h=l.length/
6|0,n=Array(h),q=0;q<h;)n[q]="#"+l.slice(6*q,6*++q);return n}function Wb(l){return zW(l[l.length-1])}function bq(l){var h=l.length;return function(n){return l[Math.max(0,Math.min(h-1,Math.floor(n*h)))]}}function rv(l){return function(){return l}}function MF(l){this.Db=l}function NF(l){return new MF(l)}function AW(l){return l[0]}function BW(l){return l[1]}function OF(l){this.wm=l}function PF(l,h,n){var q=l.Jb-l.qc;h-=l.Jb;var v=(l.Mb-l.yc)/(q||0>h&&-0);l=(n-l.Mb)/(h||0>q&&-0);return((0>v?-1:1)+(0>
l?-1:1))*Math.min(Math.abs(v),Math.abs(l),.5*Math.abs((v*h+l*q)/(q+h)))||0}function QF(l,h){var n=l.Jb-l.qc;return n?(3*(l.Mb-l.yc)/n-h)/2:h}function sv(l,h,n){var q=l.qc,v=l.Jb,y=l.Mb,A=(v-q)/3;l.Db.bezierCurveTo(q+A,l.yc+A*h,v-A,y-A*n,v,y)}function tv(l){this.Db=l}function RF(l){this.Db=new SF(l)}function SF(l){this.Db=l}function uv(l){return function(){return l}}function CW(l,h,n){this.target=l;this.type=h;this.transform=n}function tg(l,h,n){this.k=l;this.x=h;this.y=n}function om(){LL.preventDefault();
LL.stopImmediatePropagation()}function DW(){return!LL.button}function EW(){var l=this;if(l instanceof SVGElement){l=l.ownerSVGElement||l;var h=l.width.baseVal.value;l=l.height.baseVal.value}else h=l.clientWidth,l=l.clientHeight;return[[0,0],[h,l]]}function TF(){return this.__zoom||UF}function FW(){return-LL.deltaY*(LL.deltaMode?120:1)/500}function GW(){return"ontouchstart"in this}function HW(l,h,n){var q=l.lw(h[0][0])-n[0][0],v=l.lw(h[1][0])-n[1][0],y=l.mw(h[0][1])-n[0][1];h=l.mw(h[1][1])-n[1][1];
return l.translate(v>q?(q+v)/2:Math.min(0,q)||Math.max(0,v),h>y?(y+h)/2:Math.min(0,y)||Math.max(0,h))}(function(l){1===l.length&&(l=c(l));return{left:function(h,n,q,v){null==q&&(q=0);null==v&&(v=h.length);for(;q<v;){var y=q+v>>>1;0>l(h[y],n)?q=y+1:v=y}return q},right:function(h,n,q,v){null==q&&(q=0);null==v&&(v=h.length);for(;q<v;){var y=q+v>>>1;0<l(h[y],n)?v=y:q=y+1}return q}}})(b);var hU={value:function(){}};e.prototype=d.prototype={constructor:e,on:function(l,h){var n=this.Gb,q=f(l+"",n),v,y=-1,
A=q.length;if(2>arguments.length)for(;++y<A;){var E;if(E=v=(l=q[y]).type){a:{E=n[v];for(var K=0,L=E.length;K<L;++K)if((v=E[K]).name===l.name){v=v.value;break a}v=void 0}E=v}if(E)return v}else{if(null!=h&&"function"!==typeof h)throw Error("invalid callback: "+h);for(;++y<A;)if(v=(l=q[y]).type)n[v]=g(n[v],l.name,h);else if(null==h)for(v in n)n[v]=g(n[v],l.name,null);return this}},copy:function(){var l={},h=this.Gb,n;for(n in h)l[n]=h[n].slice();return new e(l)},call:function(l,h){if(0<(v=arguments.length-
2))for(var n=Array(v),q=0,v,y;q<v;++q)n[q]=arguments[q+2];if(!this.Gb.hasOwnProperty(l))throw Error("unknown type: "+l);y=this.Gb[l];q=0;for(v=y.length;q<v;++q)y[q].value.apply(h,n)},apply:function(l,h,n){if(!this.Gb.hasOwnProperty(l))throw Error("unknown type: "+l);l=this.Gb[l];for(var q=0,v=l.length;q<v;++q)l[q].value.apply(h,n)}};var Qd={svg:"http://www.w3.org/2000/svg",xhtml:"http://www.w3.org/1999/xhtml",xlink:"http://www.w3.org/1999/xlink",xml:"http://www.w3.org/XML/1998/namespace",xmlns:"http://www.w3.org/2000/xmlns/"};
if("undefined"!==typeof document){var pm=document.documentElement;if(!pm.matches){var IW=pm.webkitMatchesSelector||pm.msMatchesSelector||pm.mozMatchesSelector||pm.oMatchesSelector;a=function(l){return function(){return IW.call(this,l)}}}}var VF=a;J.prototype={constructor:J,appendChild:function(l){return this.Qa.insertBefore(l,this.Oa)},insertBefore:function(l,h){return this.Qa.insertBefore(l,h)},querySelector:function(l){return this.Qa.querySelector(l)},querySelectorAll:function(l){return this.Qa.querySelectorAll(l)}};
var HE={};LL=null;"undefined"!==typeof document&&("onmouseenter"in document.documentElement||(HE={mouseenter:"mouseover",mouseleave:"mouseout"}));var IE=[null];xb.prototype=Tc.prototype={constructor:xb,select:function(l){"function"!==typeof l&&(l=B(l));for(var h=this.xc,n=h.length,q=Array(n),v=0;v<n;++v)for(var y=h[v],A=y.length,E=q[v]=Array(A),K,L,P=0;P<A;++P)(K=y[P])&&(L=l.call(K,K.__data__,P,y))&&("__data__"in K&&(L.__data__=K.__data__),E[P]=L);return new xb(q,this.cc)},filter:function(l){"function"!==
typeof l&&(l=VF(l));for(var h=this.xc,n=h.length,q=Array(n),v=0;v<n;++v)for(var y=h[v],A=y.length,E=q[v]=[],K,L=0;L<A;++L)(K=y[L])&&l.call(K,K.__data__,L,y)&&E.push(K);return new xb(q,this.cc)},data:function(l,h){if(!l)return oa=Array(this.size()),L=-1,this.each(function(vb){oa[++L]=vb}),oa;var n=h?ca:aa,q=this.cc,v=this.xc;"function"!==typeof l&&(l=N(l));for(var y=v.length,A=Array(y),E=Array(y),K=Array(y),L=0;L<y;++L){var P=q[L],X=v[L],ka=X.length,oa=l.call(P,P&&P.__data__,L,q),Ba=oa.length,Va=E[L]=
Array(Ba),Ya=A[L]=Array(Ba);ka=K[L]=Array(ka);n(P,X,Va,Ya,ka,oa,h);X=P=0;for(var kc;P<Ba;++P)if(ka=Va[P]){for(P>=X&&(X=P+1);!(kc=Ya[X])&&++X<Ba;);ka.Oa=kc||null}}A=new xb(A,q);A.Lz=E;A.Mz=K;return A},sn:function(){return new xb(this.Lz||this.xc.map(G),this.cc)},Dq:function(){return new xb(this.Mz||this.xc.map(G),this.cc)},merge:function(l){var h=this.xc;l=l.xc;for(var n=h.length,q=Math.min(n,l.length),v=Array(n),y=0;y<q;++y)for(var A=h[y],E=l[y],K=A.length,L=v[y]=Array(K),P,X=0;X<K;++X)if(P=A[X]||
E[X])L[X]=P;for(;y<n;++y)v[y]=h[y];return new xb(v,this.cc)},order:function(){for(var l=this.xc,h=-1,n=l.length;++h<n;)for(var q=l[h],v=q.length-1,y=q[v],A;0<=--v;)if(A=q[v])y&&y!==A.nextSibling&&y.parentNode.insertBefore(A,y),y=A;return this},sort:function(l){function h(X,ka){return X&&ka?l(X.__data__,ka.__data__):!X-!ka}l||(l=ia);for(var n=this.xc,q=n.length,v=Array(q),y=0;y<q;++y){for(var A=n[y],E=A.length,K=v[y]=Array(E),L,P=0;P<E;++P)if(L=A[P])K[P]=L;K.sort(h)}return(new xb(v,this.cc)).order()},
call:function(){var l=arguments[0];arguments[0]=this;l.apply(null,arguments);return this},nodes:function(){var l=Array(this.size()),h=-1;this.each(function(){l[++h]=this});return l},node:function(){for(var l=this.xc,h=0,n=l.length;h<n;++h)for(var q=l[h],v=0,y=q.length;v<y;++v){var A=q[v];if(A)return A}return null},size:function(){var l=0;this.each(function(){++l});return l},empty:function(){return!this.node()},each:function(l){for(var h=this.xc,n=0,q=h.length;n<q;++n)for(var v=h[n],y=0,A=v.length,
E;y<A;++y)(E=v[y])&&l.call(E,E.__data__,y,v);return this},attr:function(l,h){var n=k(l);if(2>arguments.length){var q=this.node();return n.local?q.getAttributeNS(n.space,n.local):q.getAttribute(n)}return this.each((null==h?n.local?T:O:"function"===typeof h?n.local?M:F:n.local?la:Q)(n,h))},style:function(l,h,n){return 1<arguments.length?this.each((null==h?W:"function"===typeof h?pb:Aa)(l,h,null==n?"":n)):qb(this.node(),l)},nx:function(l,h){return 1<arguments.length?this.each((null==h?Ia:"function"===
typeof h?Ma:Vb)(l,h)):this.node()[l]},text:function(l){return arguments.length?this.each(null==l?qa:("function"===typeof l?ea:U)(l)):this.node().textContent},append:function(l){var h="function"===typeof l?l:t(l);return this.select(function(){return this.appendChild(h.apply(this,arguments))})},ar:function(l,h){var n="function"===typeof l?l:t(l),q=null==h?na:"function"===typeof h?h:B(h);return this.select(function(){return this.insertBefore(n.apply(this,arguments),q.apply(this,arguments)||null)})},
remove:function(){return this.each(Ja)},clone:function(l){return this.select(l?Pa:cb)},on:function(l,h,n){var q=Ha(l+""),v=q.length,y;if(2>arguments.length){var A=this.node().__on;if(A)for(var E=0,K=A.length,L;E<K;++E){var P=0;for(L=A[E];P<v;++P)if((y=q[P]).type===L.type&&y.name===L.name)return L.value}}else{A=h?kb:Ua;null==n&&(n=!1);for(P=0;P<v;++P)this.each(A(q[P],h,n));return this}}};var iU=0;Xb.prototype=function(){return new Xb}.prototype={constructor:Xb,get:function(l){for(var h=this.Gb;!(h in
l);)if(!(l=l.parentNode))return;return l[h]},set:function(l,h){return l[this.Gb]=h},remove:function(l){return this.Gb in l&&delete l[this.Gb]},toString:function(){return this.Gb}};var bk=1/.7,jU=/^#([0-9a-f]{3})$/,kU=/^#([0-9a-f]{6})$/,lU=RegExp("^rgb\\(\\s*([+-]?\\d+)\\s*,\\s*([+-]?\\d+)\\s*,\\s*([+-]?\\d+)\\s*\\)$"),mU=RegExp("^rgb\\(\\s*([+-]?\\d*\\.?\\d+(?:[eE][+-]?\\d+)?)%\\s*,\\s*([+-]?\\d*\\.?\\d+(?:[eE][+-]?\\d+)?)%\\s*,\\s*([+-]?\\d*\\.?\\d+(?:[eE][+-]?\\d+)?)%\\s*\\)$"),nU=RegExp("^rgba\\(\\s*([+-]?\\d+)\\s*,\\s*([+-]?\\d+)\\s*,\\s*([+-]?\\d+)\\s*,\\s*([+-]?\\d*\\.?\\d+(?:[eE][+-]?\\d+)?)\\s*\\)$"),
oU=RegExp("^rgba\\(\\s*([+-]?\\d*\\.?\\d+(?:[eE][+-]?\\d+)?)%\\s*,\\s*([+-]?\\d*\\.?\\d+(?:[eE][+-]?\\d+)?)%\\s*,\\s*([+-]?\\d*\\.?\\d+(?:[eE][+-]?\\d+)?)%\\s*,\\s*([+-]?\\d*\\.?\\d+(?:[eE][+-]?\\d+)?)\\s*\\)$"),pU=RegExp("^hsl\\(\\s*([+-]?\\d*\\.?\\d+(?:[eE][+-]?\\d+)?)\\s*,\\s*([+-]?\\d*\\.?\\d+(?:[eE][+-]?\\d+)?)%\\s*,\\s*([+-]?\\d*\\.?\\d+(?:[eE][+-]?\\d+)?)%\\s*\\)$"),qU=RegExp("^hsla\\(\\s*([+-]?\\d*\\.?\\d+(?:[eE][+-]?\\d+)?)\\s*,\\s*([+-]?\\d*\\.?\\d+(?:[eE][+-]?\\d+)?)%\\s*,\\s*([+-]?\\d*\\.?\\d+(?:[eE][+-]?\\d+)?)%\\s*,\\s*([+-]?\\d*\\.?\\d+(?:[eE][+-]?\\d+)?)\\s*\\)$"),
KE={aliceblue:15792383,antiquewhite:16444375,aqua:65535,aquamarine:8388564,azure:15794175,beige:16119260,bisque:16770244,black:0,blanchedalmond:16772045,blue:255,blueviolet:9055202,brown:10824234,burlywood:14596231,cadetblue:6266528,chartreuse:8388352,chocolate:13789470,coral:16744272,cornflowerblue:6591981,cornsilk:16775388,crimson:14423100,cyan:65535,darkblue:139,darkcyan:35723,darkgoldenrod:12092939,darkgray:11119017,darkgreen:25600,darkgrey:11119017,darkkhaki:12433259,darkmagenta:9109643,darkolivegreen:5597999,
darkorange:16747520,darkorchid:10040012,darkred:9109504,darksalmon:15308410,darkseagreen:9419919,darkslateblue:4734347,darkslategray:3100495,darkslategrey:3100495,darkturquoise:52945,darkviolet:9699539,deeppink:16716947,deepskyblue:49151,dimgray:6908265,dimgrey:6908265,dodgerblue:2003199,firebrick:11674146,floralwhite:16775920,forestgreen:2263842,fuchsia:16711935,gainsboro:14474460,ghostwhite:16316671,gold:16766720,goldenrod:14329120,gray:8421504,green:32768,greenyellow:11403055,grey:8421504,honeydew:15794160,
hotpink:16738740,indianred:13458524,indigo:4915330,ivory:16777200,khaki:15787660,lavender:15132410,lavenderblush:16773365,lawngreen:8190976,lemonchiffon:16775885,lightblue:11393254,lightcoral:15761536,lightcyan:14745599,lightgoldenrodyellow:16448210,lightgray:13882323,lightgreen:9498256,lightgrey:13882323,lightpink:16758465,lightsalmon:16752762,lightseagreen:2142890,lightskyblue:8900346,lightslategray:7833753,lightslategrey:7833753,lightsteelblue:11584734,lightyellow:16777184,lime:65280,limegreen:3329330,
linen:16445670,magenta:16711935,maroon:8388608,mediumaquamarine:6737322,mediumblue:205,mediumorchid:12211667,mediumpurple:9662683,mediumseagreen:3978097,mediumslateblue:8087790,mediumspringgreen:64154,mediumturquoise:4772300,mediumvioletred:13047173,midnightblue:1644912,mintcream:16121850,mistyrose:16770273,moccasin:16770229,navajowhite:16768685,navy:128,oldlace:16643558,olive:8421376,olivedrab:7048739,orange:16753920,orangered:16729344,orchid:14315734,palegoldenrod:15657130,palegreen:10025880,paleturquoise:11529966,
palevioletred:14381203,papayawhip:16773077,peachpuff:16767673,peru:13468991,pink:16761035,plum:14524637,powderblue:11591910,purple:8388736,rebeccapurple:6697881,red:16711680,rosybrown:12357519,royalblue:4286945,saddlebrown:9127187,salmon:16416882,sandybrown:16032864,seagreen:3050327,seashell:16774638,sienna:10506797,silver:12632256,skyblue:8900331,slateblue:6970061,slategray:7372944,slategrey:7372944,snow:16775930,springgreen:65407,steelblue:4620980,tan:13808780,teal:32896,thistle:14204888,tomato:16737095,
turquoise:4251856,violet:15631086,wheat:16113331,white:16777215,whitesmoke:16119285,yellow:16776960,yellowgreen:10145074};Kc(Bc,Dd,{xq:function(){return this.pf().xq()},ew:function(){return this.pf().ew()},toString:function(){return this.pf()+""}});Kc(Rc,dm,cd(Bc,{Xm:function(l){l=null==l?bk:Math.pow(bk,l);return new Rc(this.r*l,this.g*l,this.b*l,this.opacity)},hn:function(l){l=null==l?.7:Math.pow(.7,l);return new Rc(this.r*l,this.g*l,this.b*l,this.opacity)},pf:function(){return this},xq:function(){return 0<=
this.r&&255>=this.r&&0<=this.g&&255>=this.g&&0<=this.b&&255>=this.b&&0<=this.opacity&&1>=this.opacity},ew:function(){return"#"+Ou(this.r)+Ou(this.g)+Ou(this.b)},toString:function(){var l=this.opacity;l=isNaN(l)?1:Math.max(0,Math.min(1,l));return(1===l?"rgb(":"rgba(")+Math.max(0,Math.min(255,Math.round(this.r)||0))+", "+Math.max(0,Math.min(255,Math.round(this.g)||0))+", "+Math.max(0,Math.min(255,Math.round(this.b)||0))+(1===l?")":", "+l+")")}}));Kc(Ne,Pu,cd(Bc,{Xm:function(l){l=null==l?bk:Math.pow(bk,
l);return new Ne(this.h,this.s,this.l*l,this.opacity)},hn:function(l){l=null==l?.7:Math.pow(.7,l);return new Ne(this.h,this.s,this.l*l,this.opacity)},pf:function(){var l=this.h%360+360*(0>this.h),h=this.l,n=h+(.5>h?h:1-h)*(isNaN(l)||isNaN(this.s)?0:this.s);h=2*h-n;return new Rc(Qu(240<=l?l-240:l+120,h,n),Qu(l,h,n),Qu(120>l?l+240:l-120,h,n),this.opacity)},xq:function(){return(0<=this.s&&1>=this.s||isNaN(this.s))&&0<=this.l&&1>=this.l&&0<=this.opacity&&1>=this.opacity}}));var LE=Math.PI/180,OE=180/
Math.PI,NE=4/29,Xj=6/29,ME=3*Xj*Xj,sU=Xj*Xj*Xj;Kc(Oe,function(l,h,n,q){return 1===arguments.length?Ru(l):new Oe(l,h,n,null==q?1:q)},cd(Bc,{Xm:function(l){return new Oe(this.l+18*(null==l?1:l),this.a,this.b,this.opacity)},hn:function(l){return new Oe(this.l-18*(null==l?1:l),this.a,this.b,this.opacity)},pf:function(){var l=(this.l+16)/116,h=isNaN(this.a)?l:l+this.a/500,n=isNaN(this.b)?l:l-this.b/200;h=.96422*Uu(h);l=1*Uu(l);n=.82521*Uu(n);return new Rc(Vu(3.1338561*h-1.6168667*l-.4906146*n),Vu(-.9787684*
h+1.9161415*l+.033454*n),Vu(.0719453*h-.2289914*l+1.4052427*n),this.opacity)}}));Kc(Af,Wu,cd(Bc,{Xm:function(l){return new Af(this.h,this.c,this.l+18*(null==l?1:l),this.opacity)},hn:function(l){return new Af(this.h,this.c,this.l-18*(null==l?1:l),this.opacity)},pf:function(){return Ru(this).pf()}}));var PE=1.78277*-.29227-.1347134789;Kc(Gh,Pe,cd(Bc,{Xm:function(l){l=null==l?bk:Math.pow(bk,l);return new Gh(this.h,this.s,this.l*l,this.opacity)},hn:function(l){l=null==l?.7:Math.pow(.7,l);return new Gh(this.h,
this.s,this.l*l,this.opacity)},pf:function(){var l=isNaN(this.h)?0:(this.h+120)*LE,h=+this.l,n=isNaN(this.s)?0:this.s*h*(1-h),q=Math.cos(l);l=Math.sin(l);return new Rc(255*(h+n*(-.14861*q+1.78277*l)),255*(h+n*(-.29227*q+-.90649*l)),255*(h+1.97294*n*q),this.opacity)}}));var Op=function n(h){function q(y,A){var E=v((y=dm(y)).r,(A=dm(A)).r),K=v(y.g,A.g),L=v(y.b,A.b),P=od(y.opacity,A.opacity);return function(X){y.r=E(X);y.g=K(X);y.b=L(X);y.opacity=P(X);return y+""}}var v=vU(h);q.gamma=n;return q}(1),
zW=SE(function(h){var n=h.length-1;return function(q){var v=0>=q?q=0:1<=q?(q=1,n-1):Math.floor(q*n),y=h[v],A=h[v+1];return QE((q-v/n)*n,0<v?h[v-1]:2*y-A,y,A,v<n-1?h[v+2]:2*A-y)}});SE(function(h){var n=h.length;return function(q){var v=Math.floor((0>(q%=1)?++q:q)*n);return QE((q-v/n)*n,h[(v+n-1)%n],h[v%n],h[(v+1)%n],h[(v+2)%n])}});var Zu=/[-+]?(?:\d+\.?\d*|\.?\d+)(?:[eE][-+]?\d+)?/g,$u=new RegExp(Zu.source,"g"),VE=180/Math.PI,vv={translateX:0,translateY:0,rotate:0,skewX:0,js:1,ks:1},qm,wv,WF,cq,JW=
WE(function(h){if("none"===h)return vv;qm||(qm=document.createElement("DIV"),wv=document.documentElement,WF=document.defaultView);qm.style.transform=h;h=WF.getComputedStyle(wv.appendChild(qm),null).getPropertyValue("transform");wv.removeChild(qm);h=h.slice(7,-1).split(",");return UE(+h[0],+h[1],+h[2],+h[3],+h[4],+h[5])},"px, ","px)","deg)"),KW=WE(function(h){if(null==h)return vv;cq||(cq=document.createElementNS("http://www.w3.org/2000/svg","g"));cq.setAttribute("transform",h);if(!(h=cq.transform.baseVal.consolidate()))return vv;
h=h.matrix;return UE(h.a,h.b,h.c,h.d,h.e,h.f)},", ",")",")"),em=Math.SQRT2;YE(Xu);YE(od);ZE(Xu);ZE(od);$E(Xu);var xv=$E(od),Yj=0,gm=0,im=0,Tp,hm,Sp=0,Hh=0,Qp=0,fm="object"===typeof performance&&performance.now?performance:Date,aF="object"===typeof window&&window.requestAnimationFrame?window.requestAnimationFrame.bind(window):function(h){setTimeout(h,17)};Rp.prototype=bF.prototype={constructor:Rp,le:function(h,n,q){if("function"!==typeof h)throw new TypeError("callback is not a function");q=(null==
q?Pp():+q)+(null==n?0:+n);this.Oa||hm===this||(hm?hm.Oa=this:Tp=this,hm=this);this.mk=h;this.Ck=q;av()},stop:function(){this.mk&&(this.mk=null,this.Ck=Infinity,av())}};var FU=d("start","end","interrupt"),GU=[],LW=Tc.prototype.constructor,XF=0,ck=Tc.prototype;rg.prototype=function(h){return Tc().transition(h)}.prototype={constructor:rg,select:function(h){var n=this.qh,q=this.Xb;"function"!==typeof h&&(h=B(h));for(var v=this.xc,y=v.length,A=Array(y),E=0;E<y;++E)for(var K=v[E],L=K.length,P=A[E]=Array(L),
X,ka,oa=0;oa<L;++oa)(X=K[oa])&&(ka=h.call(X,X.__data__,oa,K))&&("__data__"in X&&(ka.__data__=X.__data__),P[oa]=ka,bv(P[oa],n,q,oa,P,Re(X,q)));return new rg(A,this.cc,n,q)},filter:function(h){"function"!==typeof h&&(h=VF(h));for(var n=this.xc,q=n.length,v=Array(q),y=0;y<q;++y)for(var A=n[y],E=A.length,K=v[y]=[],L,P=0;P<E;++P)(L=A[P])&&h.call(L,L.__data__,P,A)&&K.push(L);return new rg(v,this.cc,this.qh,this.Xb)},merge:function(h){if(h.Xb!==this.Xb)throw Error();var n=this.xc;h=h.xc;for(var q=n.length,
v=Math.min(q,h.length),y=Array(q),A=0;A<v;++A)for(var E=n[A],K=h[A],L=E.length,P=y[A]=Array(L),X,ka=0;ka<L;++ka)if(X=E[ka]||K[ka])P[ka]=X;for(;A<q;++A)y[A]=n[A];return new rg(y,this.cc,this.qh,this.Xb)},selection:function(){return new LW(this.xc,this.cc)},transition:function(){for(var h=this.qh,n=this.Xb,q=++XF,v=this.xc,y=v.length,A=0;A<y;++A)for(var E=v[A],K=E.length,L,P=0;P<K;++P)if(L=E[P]){var X=Re(L,n);bv(L,h,q,P,E,{time:X.time+X.delay+X.duration,delay:0,duration:X.duration,Nf:X.Nf})}return new rg(v,
this.cc,h,q)},call:ck.call,nodes:ck.nodes,node:ck.node,size:ck.size,empty:ck.empty,each:ck.each,on:function(h,n){var q=this.Xb;return 2>arguments.length?Re(this.node(),q).on.on(h):this.each(XU(q,h,n))},attr:function(h,n){var q=k(h),v="transform"===q?KW:eF;return this.ZA(h,"function"===typeof n?(q.local?OU:NU)(q,v,dv(this,"attr."+h,n)):null==n?(q.local?KU:JU)(q):(q.local?MU:LU)(q,v,n+""))},ZA:function(h,n){var q="attr."+h;if(2>arguments.length)return(q=this.ld(q))&&q.Lc;if(null==n)return this.ld(q,
null);if("function"!==typeof n)throw Error();var v=k(h);return this.ld(q,(v.local?PU:QU)(v,n))},style:function(h,n,q){var v="transform"===h?JW:eF;return null==n?this.ey(h,ZU(h,v)).on("end.style."+h,$U(h)):this.ey(h,"function"===typeof n?bV(h,v,dv(this,"style."+h,n)):aV(h,v,n+""),q)},ey:function(h,n,q){var v="style."+h;if(2>arguments.length)return(v=this.ld(v))&&v.Lc;if(null==n)return this.ld(v,null);if("function"!==typeof n)throw Error();return this.ld(v,cV(h,n,null==q?"":q))},text:function(h){return this.ld("text",
"function"===typeof h?eV(dv(this,"text",h)):dV(null==h?"":h+""))},remove:function(){return this.on("end.remove",YU(this.Xb))},ld:function(h,n){var q=this.Xb;if(2>arguments.length){q=Re(this.node(),q).ld;for(var v=0,y=q.length,A;v<y;++v)if((A=q[v]).name===h)return A.value;return null}return this.each((null==n?HU:IU)(q,h,n))},delay:function(h){var n=this.Xb;return arguments.length?this.each(("function"===typeof h?RU:SU)(n,h)):Re(this.node(),n).delay},duration:function(h){var n=this.Xb;return arguments.length?
this.each(("function"===typeof h?TU:UU)(n,h)):Re(this.node(),n).duration},Nf:function(h){var n=this.Xb;return arguments.length?this.each(VU(n,h)):Re(this.node(),n).Nf}};var yv={time:null,delay:0,duration:250,Nf:function(h){return(1>=(h*=2)?h*h*h:(h-=2)*h*h+2)/2}};Tc.prototype.sC=function(){return this.each(function(){Up(this,void 0)})};Tc.prototype.transition=function(h){var n;if(h instanceof rg){var q=h.Xb;h=h.qh}else q=++XF,(n=yv).time=Pp(),h=null==h?null:h+"";for(var v=this.xc,y=v.length,A=0;A<
y;++A)for(var E=v[A],K=E.length,L,P=0;P<K;++P)if(L=E[P]){var X=L,ka=h,oa=q,Ba=P,Va=E,Ya;if(!(Ya=n))a:{Ya=void 0;for(var kc=q;!(Ya=L.gg)||!(Ya=Ya[kc]);)if(!(L=L.parentNode)){Ya=(yv.time=Pp(),yv);break a}}bv(X,ka,oa,Ba,Va,Ya)}return new rg(v,this.cc,h,q)};["e","w"].map(ev);["n","s"].map(ev);"n e s w nw ne se sw".split(" ").map(ev);fv.prototype=fF.prototype={constructor:fv,moveTo:function(h,n){this.Gb+="M"+(this.qc=this.Jb=+h)+","+(this.yc=this.Mb=+n)},closePath:function(){null!==this.Jb&&(this.Jb=this.qc,
this.Mb=this.yc,this.Gb+="Z")},lineTo:function(h,n){this.Gb+="L"+(this.Jb=+h)+","+(this.Mb=+n)},bezierCurveTo:function(h,n,q,v,y,A){this.Gb+="C"+ +h+","+ +n+","+ +q+","+ +v+","+(this.Jb=+y)+","+(this.Mb=+A)},rect:function(h,n,q,v){this.Gb+="M"+(this.qc=this.Jb=+h)+","+(this.yc=this.Mb=+n)+"h"+ +q+"v"+ +v+"h"+-q+"Z"},toString:function(){return this.Gb}};Vp.prototype=gF.prototype={constructor:Vp,has:function(h){return" "+h in this},get:function(h){return this[" "+h]},set:function(h,n){this[" "+h]=n;
return this},remove:function(h){h=" "+h;return h in this&&delete this[h]},clear:function(){for(var h in this)" "===h[0]&&delete this[h]},keys:function(){var h=[],n;for(n in this)" "===n[0]&&h.push(n.slice(1));return h},values:function(){var h=[],n;for(n in this)" "===n[0]&&h.push(this[n]);return h},entries:function(){var h=[],n;for(n in this)" "===n[0]&&h.push({key:n.slice(1),value:this[n]});return h},size:function(){var h=0,n;for(n in this)" "===n[0]&&++h;return h},empty:function(){for(var h in this)if(" "===
h[0])return!1;return!0},each:function(h){for(var n in this)" "===n[0]&&h(this[n],n.slice(1),this)}};var Lh=gF.prototype;Wp.prototype=function(h,n){var q=new Wp;if(h instanceof Wp)h.each(function(A){q.add(A)});else if(h){var v=-1,y=h.length;if(null==n)for(;++v<y;)q.add(h[v]);else for(;++v<y;)q.add(n(h[v],v,h))}return q}.prototype={constructor:Wp,has:Lh.has,add:function(h){this[" "+h]=h;return this},remove:Lh.remove,clear:Lh.clear,values:Lh.keys,size:Lh.size,empty:Lh.empty,each:Lh.each};var jF={},gv=
{},MW=iF(",").parse,NW=iF("\t").parse;kF(MW);kF(NW);var gd=function(h,n,q){n=new hv(null==n?iV:n,null==q?jV:q,NaN,NaN,NaN,NaN);return null==h?n:n.addAll(h)}.prototype=hv.prototype;gd.copy=function(){var h=new hv(this.mg,this.ng,this.qc,this.yc,this.Jb,this.Mb),n=this.vb,q,v;if(!n)return h;if(!n.length)return h.vb=mF(n),h;for(q=[{source:n,target:h.vb=Array(4)}];n=q.pop();)for(var y=0;4>y;++y)if(v=n.source[y])v.length?q.push({source:v,target:n.target[y]=Array(4)}):n.target[y]=mF(v);return h};gd.add=
function(h){var n=+this.mg.call(null,h),q=+this.ng.call(null,h);return lF(this.Tk(n,q),n,q,h)};gd.addAll=function(h){var n,q,v=h.length,y,A,E=Array(v),K=Array(v),L=Infinity,P=Infinity,X=-Infinity,ka=-Infinity;for(q=0;q<v;++q)isNaN(y=+this.mg.call(null,n=h[q]))||isNaN(A=+this.ng.call(null,n))||(E[q]=y,K[q]=A,y<L&&(L=y),y>X&&(X=y),A<P&&(P=A),A>ka&&(ka=A));X<L&&(L=this.qc,X=this.Jb);ka<P&&(P=this.yc,ka=this.Mb);this.Tk(L,P).Tk(X,ka);for(q=0;q<v;++q)lF(this,E[q],K[q],h[q]);return this};gd.Tk=function(h,
n){if(isNaN(h=+h)||isNaN(n=+n))return this;var q=this.qc,v=this.yc,y=this.Jb,A=this.Mb;if(isNaN(q))y=(q=Math.floor(h))+1,A=(v=Math.floor(n))+1;else if(q>h||h>y||v>n||n>A){var E=y-q,K=this.vb,L;switch(L=(n<(v+A)/2)<<1|h<(q+y)/2){case 0:do{var P=Array(4);P[L]=K;K=P}while(E*=2,y=q+E,A=v+E,h>y||n>A);break;case 1:do P=Array(4),P[L]=K,K=P;while(E*=2,q=y-E,A=v+E,q>h||n>A);break;case 2:do P=Array(4),P[L]=K,K=P;while(E*=2,y=q+E,v=A-E,h>y||v>n);break;case 3:do P=Array(4),P[L]=K,K=P;while(E*=2,q=y-E,v=A-E,q>
h||v>n)}this.vb&&this.vb.length&&(this.vb=K)}else return this;this.qc=q;this.yc=v;this.Jb=y;this.Mb=A;return this};gd.data=function(){var h=[];this.Fy(function(n){if(!n.length){do h.push(n.data);while(n=n.next)}});return h};gd.cl=function(h){return arguments.length?this.Tk(+h[0][0],+h[0][1]).Tk(+h[1][0],+h[1][1]):isNaN(this.qc)?void 0:[[this.qc,this.yc],[this.Jb,this.Mb]]};gd.find=function(h,n,q){var v=this.qc,y=this.yc,A,E,K,L,P=this.Jb,X=this.Mb,ka=[],oa=this.vb,Ba;oa&&ka.push(new fd(oa,v,y,P,X));
null==q?q=Infinity:(v=h-q,y=n-q,P=h+q,X=n+q,q*=q);for(;Ba=ka.pop();)if(!(!(oa=Ba.node)||(A=Ba.Mo)>P||(E=Ba.No)>X||(K=Ba.x1)<v||(L=Ba.y1)<y))if(oa.length){Ba=(A+K)/2;var Va=(E+L)/2;ka.push(new fd(oa[3],Ba,Va,K,L),new fd(oa[2],A,Va,Ba,L),new fd(oa[1],Ba,E,K,Va),new fd(oa[0],A,E,Ba,Va));if(oa=(n>=Va)<<1|h>=Ba)Ba=ka[ka.length-1],ka[ka.length-1]=ka[ka.length-1-oa],ka[ka.length-1-oa]=Ba}else if(Ba=h-+this.mg.call(null,oa.data),Va=n-+this.ng.call(null,oa.data),Ba=Ba*Ba+Va*Va,Ba<q){var Ya=Math.sqrt(q=Ba);
v=h-Ya;y=n-Ya;P=h+Ya;X=n+Ya;Ya=oa.data}return Ya};gd.remove=function(h){if(isNaN(L=+this.mg.call(null,h))||isNaN(P=+this.ng.call(null,h)))return this;var n,q=this.vb,v,y=this.qc,A=this.yc,E=this.Jb,K=this.Mb,L,P,X,ka,oa,Ba,Va;if(!q)return this;if(q.length)for(;;){(oa=L>=(X=(y+E)/2))?y=X:E=X;(Ba=P>=(ka=(A+K)/2))?A=ka:K=ka;if(!(n=q,q=q[Va=Ba<<1|oa]))return this;if(!q.length)break;if(n[Va+1&3]||n[Va+2&3]||n[Va+3&3]){var Ya=n;var kc=Va}}for(;q.data!==h;)if(!(v=q,q=q.next))return this;(h=q.next)&&delete q.next;
if(v)return h?v.next=h:delete v.next,this;if(!n)return this.vb=h,this;h?n[Va]=h:delete n[Va];(q=n[0]||n[1]||n[2]||n[3])&&q===(n[3]||n[2]||n[1]||n[0])&&!q.length&&(Ya?Ya[kc]=q:this.vb=q);return this};gd.pG=function(h){for(var n=0,q=h.length;n<q;++n)this.remove(h[n]);return this};gd.root=function(){return this.vb};gd.size=function(){var h=0;this.Fy(function(n){if(!n.length){do++h;while(n=n.next)}});return h};gd.Fy=function(h){var n=[],q,v=this.vb,y,A,E,K,L;for(v&&n.push(new fd(v,this.qc,this.yc,this.Jb,
this.Mb));q=n.pop();)if(!h(v=q.node,A=q.Mo,E=q.No,K=q.x1,L=q.y1)&&v.length){q=(A+K)/2;var P=(E+L)/2;(y=v[3])&&n.push(new fd(y,q,P,K,L));(y=v[2])&&n.push(new fd(y,A,P,q,L));(y=v[1])&&n.push(new fd(y,q,E,K,P));(y=v[0])&&n.push(new fd(y,A,E,q,P))}};gd.DG=function(h){var n=[],q=[],v;for(this.vb&&n.push(new fd(this.vb,this.qc,this.yc,this.Jb,this.Mb));v=n.pop();){var y=v.node;if(y.length){var A,E=v.Mo,K=v.No,L=v.x1,P=v.y1,X=(E+L)/2,ka=(K+P)/2;(A=y[0])&&n.push(new fd(A,E,K,X,ka));(A=y[1])&&n.push(new fd(A,
X,K,L,ka));(A=y[2])&&n.push(new fd(A,E,ka,X,P));(A=y[3])&&n.push(new fd(A,X,ka,L,P))}q.push(v)}for(;v=q.pop();)h(v.node,v.Mo,v.No,v.x1,v.y1);return this};gd.x=function(h){return arguments.length?(this.mg=h,this):this.mg};gd.y=function(h){return arguments.length?(this.ng=h,this):this.ng};var nV=/^(?:(.)?([<>=^]))?([+\-( ])?([$#])?(0)?(\d+)?(,)?(\.\d+)?(~)?([a-z%])?$/i;iv.prototype=jv.prototype;jv.prototype.toString=function(){return this.fill+this.align+this.sign+this.symbol+(this.zero?"0":"")+(null==
this.width?"":Math.max(1,this.width|0))+(this.dv?",":"")+(null==this.precision?"":"."+Math.max(0,this.precision|0))+(this.trim?"~":"")+this.type};var qF,rF={"%":function(h,n){return(100*h).toFixed(n)},b:function(h){return Math.round(h).toString(2)},c:function(h){return h+""},d:function(h){return Math.round(h).toString(10)},e:function(h,n){return h.toExponential(n)},f:function(h,n){return h.toFixed(n)},g:function(h,n){return h.toPrecision(n)},o:function(h){return Math.round(h).toString(8)},p:function(h,
n){return nF(100*h,n)},r:nF,s:function(h,n){var q=Xp(h,n);if(!q)return h+"";var v=q[0];q=q[1];q=q-(qF=3*Math.max(-8,Math.min(8,Math.floor(q/3))))+1;var y=v.length;return q===y?v:q>y?v+Array(q-y+1).join("0"):0<q?v.slice(0,q)+"."+v.slice(q):"0."+Array(1-q).join("0")+Xp(h,Math.max(0,n+q-1))[0]},X:function(h){return Math.round(h).toString(16).toUpperCase()},x:function(h){return Math.round(h).toString(16)}},pF="y z a f p n \u00b5 m  k M G T P E Z Y".split(" "),OW;(function(h){return OW=oV(h)})({wB:".",
my:",",Yv:[3],currency:["$",""]});je.prototype={constructor:je,reset:function(){this.s=this.t=0},add:function(h){sF(dq,h,this.t);sF(this,dq.s,this.s);this.s?this.t+=dq.t:this.s=dq.t},valueOf:function(){return this.s}};var dq=new je,tc=Math.PI,Rd=tc/2,YF=tc/4,PW=2*tc,zv=tc/180,sg=Math.abs,Av=Math.atan,yF=Math.atan2,Sd=Math.cos,ad=Math.sin,Zp=Math.sqrt;new je;new je;new je;var Bv=new je;(function(h,n,q,v){return function(y){function A(Eb,tb){h(Eb,tb)&&y.point(Eb,tb)}function E(Eb,tb){oa.point(Eb,tb)}
function K(){Hc.point=E;oa.kc()}function L(){Hc.point=A;oa.jc()}function P(Eb,tb){lc.push([Eb,tb]);Va.point(Eb,tb)}function X(){Va.kc();lc=[]}function ka(){P(lc[0][0],lc[0][1]);Va.jc();var Eb=Va.gB(),tb=Ba.result(),La=tb.length,Hb;lc.pop();kc.push(lc);lc=null;if(La)if(Eb&1){if(La=tb[0],0<(tb=La.length-1)){Ya||(y.Dj(),Ya=!0);y.kc();for(Eb=0;Eb<tb;++Eb)y.point((Hb=La[Eb])[0],Hb[1]);y.jc()}}else 1<La&&Eb&2&&tb.push(tb.pop().concat(tb.shift())),vb.push(tb.filter(sV))}var oa=n(y),Ba=qV(),Va=n(Ba),Ya=!1,
kc,vb,lc,Hc={point:A,kc:K,jc:L,Dj:function(){Hc.point=P;Hc.kc=X;Hc.jc=ka;vb=[];kc=[]},po:function(){Hc.point=A;Hc.kc=K;Hc.jc=L;var Eb=vb,tb=Eb.length;var La=-1;for(var Hb=0,za,Fa;++La<tb;)Hb+=Eb[La].length;for(za=Array(Hb);0<=--tb;)for(Fa=Eb[tb],La=Fa.length;0<=--La;)za[--Hb]=Fa[La];vb=za;Eb=kc;tb=v[0];Hb=v[1];var I=ad(Hb);La=[ad(tb),-Sd(tb),0];Fa=za=0;Bv.reset();1===I?Hb=Rd+1E-6:-1===I&&(Hb=-Rd-1E-6);I=0;for(var ha=Eb.length;I<ha;++I)if(Ta=(Ga=Eb[I]).length){var Ga,Ta,wb=Ga[Ta-1],Qa=wb[0],Rb=wb[1]/
2+YF,pd=ad(Rb),dk=Sd(Rb);for(Rb=0;Rb<Ta;++Rb,Qa=zd,pd=ek,dk=rm,wb=fk){var fk=Ga[Rb],zd=fk[0];rm=fk[1]/2+YF;var ek=ad(rm),rm=Sd(rm),gk=zd-Qa,Cv=0<=gk?1:-1,Dv=Cv*gk,eq=Dv>tc;pd*=ek;Bv.add(yF(pd*Cv*ad(Dv),dk*rm+pd*Sd(Dv)));za+=eq?gk+Cv*PW:gk;eq^Qa>=tb^zd>=tb&&(wb=uF(tF(wb),tF(fk)),vF(wb),Qa=uF(La,wb),vF(Qa),Qa=(eq^0<=gk?-1:1)*Yp(Qa[2]),Hb>Qa||Hb===Qa&&(wb[0]||wb[1]))&&(Fa+=eq^0<=gk?1:-1)}}Eb=(-1E-6>za||1E-6>za&&-1E-6>Bv)^Fa&1;vb.length?(Ya||(y.Dj(),Ya=!0),rV(vb,tV,Eb,q,y)):Eb&&(Ya||(y.Dj(),Ya=!0),y.kc(),
q(null,null,1,y),y.jc());Ya&&(y.po(),Ya=!1);vb=kc=null},Yx:function(){y.Dj();y.kc();q(null,null,1,y);y.jc();y.po()}};return Hc}})(function(){return!0},function(h){var n=NaN,q=NaN,v=NaN,y;return{kc:function(){h.kc();y=1},point:function(A,E){var K=0<A?tc:-tc,L=sg(A-n);if(1E-6>sg(L-tc))h.point(n,q=0<(q+E)/2?Rd:-Rd),h.point(v,q),h.jc(),h.kc(),h.point(K,q),h.point(A,q),y=0;else if(v!==K&&L>=tc){1E-6>sg(n-v)&&(n-=1E-6*v);1E-6>sg(A-K)&&(A-=1E-6*K);L=n;var P=q,X=A,ka,oa,Ba=ad(L-X);q=1E-6<sg(Ba)?Av((ad(P)*
(oa=Sd(E))*ad(X)-ad(E)*(ka=Sd(P))*ad(L))/(ka*oa*Ba)):(P+E)/2;h.point(v,q);h.jc();h.kc();h.point(K,q);y=0}h.point(n=A,q=E);v=K},jc:function(){h.jc();n=q=NaN},gB:function(){return 2-y}}},function(h,n,q,v){null==h?(q*=Rd,v.point(-tc,q),v.point(0,q),v.point(tc,q),v.point(tc,0),v.point(tc,-q),v.point(0,-q),v.point(-tc,-q),v.point(-tc,0),v.point(-tc,q)):1E-6<sg(h[0]-n[0])?(h=h[0]<n[0]?tc:-tc,q=q*h/2,v.point(-h,q),v.point(0,q),v.point(h,q)):v.point(n[0],n[1])},[-tc,-Rd]);new je;new je;new je;new je;kv.prototype=
{constructor:kv,point:function(h,n){this.stream.point(h,n)},Yx:function(){this.stream.Yx()},kc:function(){this.stream.kc()},jc:function(){this.stream.jc()},Dj:function(){this.stream.Dj()},po:function(){this.stream.po()}};Sd(30*zv);(function(h){return function(n){var q=new kv,v;for(v in h)q[v]=h[v];q.stream=n;return q}})({point:function(h,n){this.stream.point(h*zv,n*zv)}});xF(function(h){return Zp(2/(1+h))}).invert=jm(function(h){return 2*Yp(h/2)});xF(function(h){return(h=1<h?0:-1>h?tc:Math.acos(h))&&
h/ad(h)}).invert=jm(function(h){return h});Zp(3);jm(Av);jm(Yp);jm(function(h){return 2*Av(h)});km.prototype=zF.prototype={constructor:km,count:function(){return this.Kv(uV)},each:function(h){var n,q=[this],v;do{var y=q.reverse();for(q=[];n=y.pop();)if(h(n),n=n.children){var A=0;for(v=n.length;A<v;++A)q.push(n[A])}}while(q.length);return this},Kv:function(h){for(var n,q=[this],v=[],y,A;n=q.pop();)if(v.push(n),n=n.children)for(y=0,A=n.length;y<A;++y)q.push(n[y]);for(;n=v.pop();)h(n);return this},yq:function(h){for(var n,
q=[this],v;n=q.pop();)if(h(n),n=n.children)for(v=n.length-1;0<=v;--v)q.push(n[v]);return this},li:function(h){return this.Kv(function(n){for(var q=+h(n.data)||0,v=n.children,y=v&&v.length;0<=--y;)q+=v[y].value;n.value=q})},sort:function(h){return this.yq(function(n){n.children&&n.children.sort(h)})},path:function(h){var n=this;var q=n;var v=h;if(q!==v){var y=q.Gu(),A=v.Gu(),E=null;q=y.pop();for(v=A.pop();q===v;)E=q,q=y.pop(),v=A.pop();q=E}for(v=[n];n!==q;)n=n.parent,v.push(n);for(n=v.length;h!==q;)v.splice(n,
0,h),h=h.parent;return v},Gu:function(){for(var h=this,n=[h];h=h.parent;)n.push(h);return n},links:function(){var h=this,n=[];h.each(function(q){q!==h&&n.push({source:q.parent,target:q})});return n},copy:function(){return zF(this).yq(xV)}};(function(h,n){this.Gb=h;this.WE=this.children=this.parent=null;this.a=this;this.s=this.c=this.ge=this.z=0;this.t=null;this.i=n}).prototype=Object.create(km.prototype);var lv=new Date,mv=new Date,ZF=Ic(function(){},function(h,n){h.setTime(+h+n)},function(h,n){return n-
h});ZF.every=function(h){h=Math.floor(h);return isFinite(h)&&0<h?1<h?Ic(function(n){n.setTime(Math.floor(n/h)*h)},function(n,q){n.setTime(+n+q*h)},function(n,q){return(q-n)/h}):ZF:null};Ic(function(h){h.setTime(1E3*Math.floor(h/1E3))},function(h,n){h.setTime(+h+1E3*n)},function(h,n){return(n-h)/1E3},function(h){return h.getUTCSeconds()});Ic(function(h){h.setTime(6E4*Math.floor(h/6E4))},function(h,n){h.setTime(+h+6E4*n)},function(h,n){return(n-h)/6E4},function(h){return h.getMinutes()});Ic(function(h){var n=
6E4*h.getTimezoneOffset()%36E5;0>n&&(n+=36E5);h.setTime(36E5*Math.floor((+h-n)/36E5)+n)},function(h,n){h.setTime(+h+36E5*n)},function(h,n){return(n-h)/36E5},function(h){return h.getHours()});var CF=Ic(function(h){h.setHours(0,0,0,0)},function(h,n){h.setDate(h.getDate()+n)},function(h,n){return(n-h-6E4*(n.getTimezoneOffset()-h.getTimezoneOffset()))/864E5},function(h){return h.getDate()-1}),xW=Jh(0),ov=Jh(1);Jh(2);Jh(3);var pv=Jh(4);Jh(5);Jh(6);Ic(function(h){h.setDate(1);h.setHours(0,0,0,0)},function(h,
n){h.setMonth(h.getMonth()+n)},function(h,n){return n.getMonth()-h.getMonth()+12*(n.getFullYear()-h.getFullYear())},function(h){return h.getMonth()});var Zj=Ic(function(h){h.setMonth(0,1);h.setHours(0,0,0,0)},function(h,n){h.setFullYear(h.getFullYear()+n)},function(h,n){return n.getFullYear()-h.getFullYear()},function(h){return h.getFullYear()});Zj.every=function(h){return isFinite(h=Math.floor(h))&&0<h?Ic(function(n){n.setFullYear(Math.floor(n.getFullYear()/h)*h);n.setMonth(0,1);n.setHours(0,0,0,
0)},function(n,q){n.setFullYear(n.getFullYear()+q*h)}):null};Ic(function(h){h.setUTCSeconds(0,0)},function(h,n){h.setTime(+h+6E4*n)},function(h,n){return(n-h)/6E4},function(h){return h.getUTCMinutes()});Ic(function(h){h.setUTCMinutes(0,0,0)},function(h,n){h.setTime(+h+36E5*n)},function(h,n){return(n-h)/36E5},function(h){return h.getUTCHours()});var BF=Ic(function(h){h.setUTCHours(0,0,0,0)},function(h,n){h.setUTCDate(h.getUTCDate()+n)},function(h,n){return(n-h)/864E5},function(h){return h.getUTCDate()-
1}),yW=Kh(0),nv=Kh(1);Kh(2);Kh(3);var qv=Kh(4);Kh(5);Kh(6);Ic(function(h){h.setUTCDate(1);h.setUTCHours(0,0,0,0)},function(h,n){h.setUTCMonth(h.getUTCMonth()+n)},function(h,n){return n.getUTCMonth()-h.getUTCMonth()+12*(n.getUTCFullYear()-h.getUTCFullYear())},function(h){return h.getUTCMonth()});var ak=Ic(function(h){h.setUTCMonth(0,1);h.setUTCHours(0,0,0,0)},function(h,n){h.setUTCFullYear(h.getUTCFullYear()+n)},function(h,n){return n.getUTCFullYear()-h.getUTCFullYear()},function(h){return h.getUTCFullYear()});
ak.every=function(h){return isFinite(h=Math.floor(h))&&0<h?Ic(function(n){n.setUTCFullYear(Math.floor(n.getUTCFullYear()/h)*h);n.setUTCMonth(0,1);n.setUTCHours(0,0,0,0)},function(n,q){n.setUTCFullYear(n.getUTCFullYear()+q*h)}):null};var AF={"-":"",_:" ",0:"0"},Sc=/^\s*\d+/,wW=/^%/,vW=/[\\^$*+?|[\]().{}]/g,fq;(function(h){fq=zV(h);ML=fq.NE;NL=fq.OE;return fq})({dateTime:"%x, %X",uB:"%-m/%-d/%Y",time:"%-I:%M:%S %p",tD:["AM","PM"],vB:"Sunday Monday Tuesday Wednesday Thursday Friday Saturday".split(" "),
TD:"Sun Mon Tue Wed Thu Fri Sat".split(" "),RC:"January February March April May June July August September October November December".split(" "),UD:"Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec".split(" ")});Date.prototype.toISOString||ML("%Y-%m-%dT%H:%M:%S.%LZ");+new Date("2000-01-01T00:00:00.000Z")||NL("%Y-%m-%dT%H:%M:%S.%LZ");ob("1f77b4ff7f0e2ca02cd627289467bd8c564be377c27f7f7fbcbd2217becf");ob("393b795254a36b6ecf9c9ede6379398ca252b5cf6bcedb9c8c6d31bd9e39e7ba52e7cb94843c39ad494ad6616be7969c7b4173a55194ce6dbdde9ed6");
ob("3182bd6baed69ecae1c6dbefe6550dfd8d3cfdae6bfdd0a231a35474c476a1d99bc7e9c0756bb19e9ac8bcbddcdadaeb636363969696bdbdbdd9d9d9");ob("1f77b4aec7e8ff7f0effbb782ca02c98df8ad62728ff98969467bdc5b0d58c564bc49c94e377c2f7b6d27f7f7fc7c7c7bcbd22dbdb8d17becf9edae5");ob("7fc97fbeaed4fdc086ffff99386cb0f0027fbf5b17666666");ob("1b9e77d95f027570b3e7298a66a61ee6ab02a6761d666666");ob("a6cee31f78b4b2df8a33a02cfb9a99e31a1cfdbf6fff7f00cab2d66a3d9affff99b15928");ob("fbb4aeb3cde3ccebc5decbe4fed9a6ffffcce5d8bdfddaecf2f2f2");
ob("b3e2cdfdcdaccbd5e8f4cae4e6f5c9fff2aef1e2cccccccc");ob("e41a1c377eb84daf4a984ea3ff7f00ffff33a65628f781bf999999");ob("66c2a5fc8d628da0cbe78ac3a6d854ffd92fe5c494b3b3b3");ob("8dd3c7ffffb3bebadafb807280b1d3fdb462b3de69fccde5d9d9d9bc80bdccebc5ffed6f");var QW=Array(3).concat("d8b365f5f5f55ab4ac","a6611adfc27d80cdc1018571","a6611adfc27df5f5f580cdc1018571","8c510ad8b365f6e8c3c7eae55ab4ac01665e","8c510ad8b365f6e8c3f5f5f5c7eae55ab4ac01665e","8c510abf812ddfc27df6e8c3c7eae580cdc135978f01665e","8c510abf812ddfc27df6e8c3f5f5f5c7eae580cdc135978f01665e",
"5430058c510abf812ddfc27df6e8c3c7eae580cdc135978f01665e003c30","5430058c510abf812ddfc27df6e8c3f5f5f5c7eae580cdc135978f01665e003c30").map(ob);Wb(QW);var RW=Array(3).concat("af8dc3f7f7f77fbf7b","7b3294c2a5cfa6dba0008837","7b3294c2a5cff7f7f7a6dba0008837","762a83af8dc3e7d4e8d9f0d37fbf7b1b7837","762a83af8dc3e7d4e8f7f7f7d9f0d37fbf7b1b7837","762a839970abc2a5cfe7d4e8d9f0d3a6dba05aae611b7837","762a839970abc2a5cfe7d4e8f7f7f7d9f0d3a6dba05aae611b7837","40004b762a839970abc2a5cfe7d4e8d9f0d3a6dba05aae611b783700441b",
"40004b762a839970abc2a5cfe7d4e8f7f7f7d9f0d3a6dba05aae611b783700441b").map(ob);Wb(RW);var SW=Array(3).concat("e9a3c9f7f7f7a1d76a","d01c8bf1b6dab8e1864dac26","d01c8bf1b6daf7f7f7b8e1864dac26","c51b7de9a3c9fde0efe6f5d0a1d76a4d9221","c51b7de9a3c9fde0eff7f7f7e6f5d0a1d76a4d9221","c51b7dde77aef1b6dafde0efe6f5d0b8e1867fbc414d9221","c51b7dde77aef1b6dafde0eff7f7f7e6f5d0b8e1867fbc414d9221","8e0152c51b7dde77aef1b6dafde0efe6f5d0b8e1867fbc414d9221276419","8e0152c51b7dde77aef1b6dafde0eff7f7f7e6f5d0b8e1867fbc414d9221276419").map(ob);
Wb(SW);var TW=Array(3).concat("998ec3f7f7f7f1a340","5e3c99b2abd2fdb863e66101","5e3c99b2abd2f7f7f7fdb863e66101","542788998ec3d8daebfee0b6f1a340b35806","542788998ec3d8daebf7f7f7fee0b6f1a340b35806","5427888073acb2abd2d8daebfee0b6fdb863e08214b35806","5427888073acb2abd2d8daebf7f7f7fee0b6fdb863e08214b35806","2d004b5427888073acb2abd2d8daebfee0b6fdb863e08214b358067f3b08","2d004b5427888073acb2abd2d8daebf7f7f7fee0b6fdb863e08214b358067f3b08").map(ob);Wb(TW);var UW=Array(3).concat("ef8a62f7f7f767a9cf","ca0020f4a58292c5de0571b0",
"ca0020f4a582f7f7f792c5de0571b0","b2182bef8a62fddbc7d1e5f067a9cf2166ac","b2182bef8a62fddbc7f7f7f7d1e5f067a9cf2166ac","b2182bd6604df4a582fddbc7d1e5f092c5de4393c32166ac","b2182bd6604df4a582fddbc7f7f7f7d1e5f092c5de4393c32166ac","67001fb2182bd6604df4a582fddbc7d1e5f092c5de4393c32166ac053061","67001fb2182bd6604df4a582fddbc7f7f7f7d1e5f092c5de4393c32166ac053061").map(ob);Wb(UW);var VW=Array(3).concat("ef8a62ffffff999999","ca0020f4a582bababa404040","ca0020f4a582ffffffbababa404040","b2182bef8a62fddbc7e0e0e09999994d4d4d",
"b2182bef8a62fddbc7ffffffe0e0e09999994d4d4d","b2182bd6604df4a582fddbc7e0e0e0bababa8787874d4d4d","b2182bd6604df4a582fddbc7ffffffe0e0e0bababa8787874d4d4d","67001fb2182bd6604df4a582fddbc7e0e0e0bababa8787874d4d4d1a1a1a","67001fb2182bd6604df4a582fddbc7ffffffe0e0e0bababa8787874d4d4d1a1a1a").map(ob);Wb(VW);var WW=Array(3).concat("fc8d59ffffbf91bfdb","d7191cfdae61abd9e92c7bb6","d7191cfdae61ffffbfabd9e92c7bb6","d73027fc8d59fee090e0f3f891bfdb4575b4","d73027fc8d59fee090ffffbfe0f3f891bfdb4575b4","d73027f46d43fdae61fee090e0f3f8abd9e974add14575b4",
"d73027f46d43fdae61fee090ffffbfe0f3f8abd9e974add14575b4","a50026d73027f46d43fdae61fee090e0f3f8abd9e974add14575b4313695","a50026d73027f46d43fdae61fee090ffffbfe0f3f8abd9e974add14575b4313695").map(ob);Wb(WW);var XW=Array(3).concat("fc8d59ffffbf91cf60","d7191cfdae61a6d96a1a9641","d7191cfdae61ffffbfa6d96a1a9641","d73027fc8d59fee08bd9ef8b91cf601a9850","d73027fc8d59fee08bffffbfd9ef8b91cf601a9850","d73027f46d43fdae61fee08bd9ef8ba6d96a66bd631a9850","d73027f46d43fdae61fee08bffffbfd9ef8ba6d96a66bd631a9850",
"a50026d73027f46d43fdae61fee08bd9ef8ba6d96a66bd631a9850006837","a50026d73027f46d43fdae61fee08bffffbfd9ef8ba6d96a66bd631a9850006837").map(ob);Wb(XW);var YW=Array(3).concat("fc8d59ffffbf99d594","d7191cfdae61abdda42b83ba","d7191cfdae61ffffbfabdda42b83ba","d53e4ffc8d59fee08be6f59899d5943288bd","d53e4ffc8d59fee08bffffbfe6f59899d5943288bd","d53e4ff46d43fdae61fee08be6f598abdda466c2a53288bd","d53e4ff46d43fdae61fee08bffffbfe6f598abdda466c2a53288bd","9e0142d53e4ff46d43fdae61fee08be6f598abdda466c2a53288bd5e4fa2",
"9e0142d53e4ff46d43fdae61fee08bffffbfe6f598abdda466c2a53288bd5e4fa2").map(ob);Wb(YW);var ZW=Array(3).concat("e5f5f999d8c92ca25f","edf8fbb2e2e266c2a4238b45","edf8fbb2e2e266c2a42ca25f006d2c","edf8fbccece699d8c966c2a42ca25f006d2c","edf8fbccece699d8c966c2a441ae76238b45005824","f7fcfde5f5f9ccece699d8c966c2a441ae76238b45005824","f7fcfde5f5f9ccece699d8c966c2a441ae76238b45006d2c00441b").map(ob);Wb(ZW);var $W=Array(3).concat("e0ecf49ebcda8856a7","edf8fbb3cde38c96c688419d","edf8fbb3cde38c96c68856a7810f7c",
"edf8fbbfd3e69ebcda8c96c68856a7810f7c","edf8fbbfd3e69ebcda8c96c68c6bb188419d6e016b","f7fcfde0ecf4bfd3e69ebcda8c96c68c6bb188419d6e016b","f7fcfde0ecf4bfd3e69ebcda8c96c68c6bb188419d810f7c4d004b").map(ob);Wb($W);var aX=Array(3).concat("e0f3dba8ddb543a2ca","f0f9e8bae4bc7bccc42b8cbe","f0f9e8bae4bc7bccc443a2ca0868ac","f0f9e8ccebc5a8ddb57bccc443a2ca0868ac","f0f9e8ccebc5a8ddb57bccc44eb3d32b8cbe08589e","f7fcf0e0f3dbccebc5a8ddb57bccc44eb3d32b8cbe08589e","f7fcf0e0f3dbccebc5a8ddb57bccc44eb3d32b8cbe0868ac084081").map(ob);
Wb(aX);var bX=Array(3).concat("fee8c8fdbb84e34a33","fef0d9fdcc8afc8d59d7301f","fef0d9fdcc8afc8d59e34a33b30000","fef0d9fdd49efdbb84fc8d59e34a33b30000","fef0d9fdd49efdbb84fc8d59ef6548d7301f990000","fff7ecfee8c8fdd49efdbb84fc8d59ef6548d7301f990000","fff7ecfee8c8fdd49efdbb84fc8d59ef6548d7301fb300007f0000").map(ob);Wb(bX);var cX=Array(3).concat("ece2f0a6bddb1c9099","f6eff7bdc9e167a9cf02818a","f6eff7bdc9e167a9cf1c9099016c59","f6eff7d0d1e6a6bddb67a9cf1c9099016c59","f6eff7d0d1e6a6bddb67a9cf3690c002818a016450",
"fff7fbece2f0d0d1e6a6bddb67a9cf3690c002818a016450","fff7fbece2f0d0d1e6a6bddb67a9cf3690c002818a016c59014636").map(ob);Wb(cX);var dX=Array(3).concat("ece7f2a6bddb2b8cbe","f1eef6bdc9e174a9cf0570b0","f1eef6bdc9e174a9cf2b8cbe045a8d","f1eef6d0d1e6a6bddb74a9cf2b8cbe045a8d","f1eef6d0d1e6a6bddb74a9cf3690c00570b0034e7b","fff7fbece7f2d0d1e6a6bddb74a9cf3690c00570b0034e7b","fff7fbece7f2d0d1e6a6bddb74a9cf3690c00570b0045a8d023858").map(ob);Wb(dX);var eX=Array(3).concat("e7e1efc994c7dd1c77","f1eef6d7b5d8df65b0ce1256",
"f1eef6d7b5d8df65b0dd1c77980043","f1eef6d4b9dac994c7df65b0dd1c77980043","f1eef6d4b9dac994c7df65b0e7298ace125691003f","f7f4f9e7e1efd4b9dac994c7df65b0e7298ace125691003f","f7f4f9e7e1efd4b9dac994c7df65b0e7298ace125698004367001f").map(ob);Wb(eX);var fX=Array(3).concat("fde0ddfa9fb5c51b8a","feebe2fbb4b9f768a1ae017e","feebe2fbb4b9f768a1c51b8a7a0177","feebe2fcc5c0fa9fb5f768a1c51b8a7a0177","feebe2fcc5c0fa9fb5f768a1dd3497ae017e7a0177","fff7f3fde0ddfcc5c0fa9fb5f768a1dd3497ae017e7a0177","fff7f3fde0ddfcc5c0fa9fb5f768a1dd3497ae017e7a017749006a").map(ob);
Wb(fX);var gX=Array(3).concat("edf8b17fcdbb2c7fb8","ffffcca1dab441b6c4225ea8","ffffcca1dab441b6c42c7fb8253494","ffffccc7e9b47fcdbb41b6c42c7fb8253494","ffffccc7e9b47fcdbb41b6c41d91c0225ea80c2c84","ffffd9edf8b1c7e9b47fcdbb41b6c41d91c0225ea80c2c84","ffffd9edf8b1c7e9b47fcdbb41b6c41d91c0225ea8253494081d58").map(ob);Wb(gX);var hX=Array(3).concat("f7fcb9addd8e31a354","ffffccc2e69978c679238443","ffffccc2e69978c67931a354006837","ffffccd9f0a3addd8e78c67931a354006837","ffffccd9f0a3addd8e78c67941ab5d238443005a32",
"ffffe5f7fcb9d9f0a3addd8e78c67941ab5d238443005a32","ffffe5f7fcb9d9f0a3addd8e78c67941ab5d238443006837004529").map(ob);Wb(hX);var iX=Array(3).concat("fff7bcfec44fd95f0e","ffffd4fed98efe9929cc4c02","ffffd4fed98efe9929d95f0e993404","ffffd4fee391fec44ffe9929d95f0e993404","ffffd4fee391fec44ffe9929ec7014cc4c028c2d04","ffffe5fff7bcfee391fec44ffe9929ec7014cc4c028c2d04","ffffe5fff7bcfee391fec44ffe9929ec7014cc4c02993404662506").map(ob);Wb(iX);var jX=Array(3).concat("ffeda0feb24cf03b20","ffffb2fecc5cfd8d3ce31a1c",
"ffffb2fecc5cfd8d3cf03b20bd0026","ffffb2fed976feb24cfd8d3cf03b20bd0026","ffffb2fed976feb24cfd8d3cfc4e2ae31a1cb10026","ffffccffeda0fed976feb24cfd8d3cfc4e2ae31a1cb10026","ffffccffeda0fed976feb24cfd8d3cfc4e2ae31a1cbd0026800026").map(ob);Wb(jX);var kX=Array(3).concat("deebf79ecae13182bd","eff3ffbdd7e76baed62171b5","eff3ffbdd7e76baed63182bd08519c","eff3ffc6dbef9ecae16baed63182bd08519c","eff3ffc6dbef9ecae16baed64292c62171b5084594","f7fbffdeebf7c6dbef9ecae16baed64292c62171b5084594","f7fbffdeebf7c6dbef9ecae16baed64292c62171b508519c08306b").map(ob);
Wb(kX);var lX=Array(3).concat("e5f5e0a1d99b31a354","edf8e9bae4b374c476238b45","edf8e9bae4b374c47631a354006d2c","edf8e9c7e9c0a1d99b74c47631a354006d2c","edf8e9c7e9c0a1d99b74c47641ab5d238b45005a32","f7fcf5e5f5e0c7e9c0a1d99b74c47641ab5d238b45005a32","f7fcf5e5f5e0c7e9c0a1d99b74c47641ab5d238b45006d2c00441b").map(ob);Wb(lX);var mX=Array(3).concat("f0f0f0bdbdbd636363","f7f7f7cccccc969696525252","f7f7f7cccccc969696636363252525","f7f7f7d9d9d9bdbdbd969696636363252525","f7f7f7d9d9d9bdbdbd969696737373525252252525",
"fffffff0f0f0d9d9d9bdbdbd969696737373525252252525","fffffff0f0f0d9d9d9bdbdbd969696737373525252252525000000").map(ob);Wb(mX);var nX=Array(3).concat("efedf5bcbddc756bb1","f2f0f7cbc9e29e9ac86a51a3","f2f0f7cbc9e29e9ac8756bb154278f","f2f0f7dadaebbcbddc9e9ac8756bb154278f","f2f0f7dadaebbcbddc9e9ac8807dba6a51a34a1486","fcfbfdefedf5dadaebbcbddc9e9ac8807dba6a51a34a1486","fcfbfdefedf5dadaebbcbddc9e9ac8807dba6a51a354278f3f007d").map(ob);Wb(nX);var oX=Array(3).concat("fee0d2fc9272de2d26","fee5d9fcae91fb6a4acb181d",
"fee5d9fcae91fb6a4ade2d26a50f15","fee5d9fcbba1fc9272fb6a4ade2d26a50f15","fee5d9fcbba1fc9272fb6a4aef3b2ccb181d99000d","fff5f0fee0d2fcbba1fc9272fb6a4aef3b2ccb181d99000d","fff5f0fee0d2fcbba1fc9272fb6a4aef3b2ccb181da50f1567000d").map(ob);Wb(oX);var pX=Array(3).concat("fee6cefdae6be6550d","feeddefdbe85fd8d3cd94701","feeddefdbe85fd8d3ce6550da63603","feeddefdd0a2fdae6bfd8d3ce6550da63603","feeddefdd0a2fdae6bfd8d3cf16913d948018c2d04","fff5ebfee6cefdd0a2fdae6bfd8d3cf16913d948018c2d04","fff5ebfee6cefdd0a2fdae6bfd8d3cf16913d94801a636037f2704").map(ob);
Wb(pX);xv(Pe(300,.5,0),Pe(-240,.5,1));xv(Pe(-100,.75,.35),Pe(80,1.5,.8));xv(Pe(260,.75,.35),Pe(80,1.5,.8));Pe();dm();bq(ob("44015444025645045745055946075a46085c460a5d460b5e470d60470e6147106347116447136548146748166848176948186a481a6c481b6d481c6e481d6f481f70482071482173482374482475482576482677482878482979472a7a472c7a472d7b472e7c472f7d46307e46327e46337f463480453581453781453882443983443a83443b84433d84433e85423f854240864241864142874144874045884046883f47883f48893e49893e4a893e4c8a3d4d8a3d4e8a3c4f8a3c508b3b518b3b528b3a538b3a548c39558c39568c38588c38598c375a8c375b8d365c8d365d8d355e8d355f8d34608d34618d33628d33638d32648e32658e31668e31678e31688e30698e306a8e2f6b8e2f6c8e2e6d8e2e6e8e2e6f8e2d708e2d718e2c718e2c728e2c738e2b748e2b758e2a768e2a778e2a788e29798e297a8e297b8e287c8e287d8e277e8e277f8e27808e26818e26828e26828e25838e25848e25858e24868e24878e23888e23898e238a8d228b8d228c8d228d8d218e8d218f8d21908d21918c20928c20928c20938c1f948c1f958b1f968b1f978b1f988b1f998a1f9a8a1e9b8a1e9c891e9d891f9e891f9f881fa0881fa1881fa1871fa28720a38620a48621a58521a68522a78522a88423a98324aa8325ab8225ac8226ad8127ad8128ae8029af7f2ab07f2cb17e2db27d2eb37c2fb47c31b57b32b67a34b67935b77937b87838b9773aba763bbb753dbc743fbc7340bd7242be7144bf7046c06f48c16e4ac16d4cc26c4ec36b50c46a52c56954c56856c66758c7655ac8645cc8635ec96260ca6063cb5f65cb5e67cc5c69cd5b6ccd5a6ece5870cf5773d05675d05477d1537ad1517cd2507fd34e81d34d84d44b86d54989d5488bd6468ed64590d74393d74195d84098d83e9bd93c9dd93ba0da39a2da37a5db36a8db34aadc32addc30b0dd2fb2dd2db5de2bb8de29bade28bddf26c0df25c2df23c5e021c8e020cae11fcde11dd0e11cd2e21bd5e21ad8e219dae319dde318dfe318e2e418e5e419e7e419eae51aece51befe51cf1e51df4e61ef6e620f8e621fbe723fde725"));
bq(ob("00000401000501010601010802010902020b02020d03030f03031204041405041606051806051a07061c08071e0907200a08220b09240c09260d0a290e0b2b100b2d110c2f120d31130d34140e36150e38160f3b180f3d19103f1a10421c10441d11471e114920114b21114e22115024125325125527125829115a2a115c2c115f2d11612f116331116533106734106936106b38106c390f6e3b0f703d0f713f0f72400f74420f75440f764510774710784910784a10794c117a4e117b4f127b51127c52137c54137d56147d57157e59157e5a167e5c167f5d177f5f187f601880621980641a80651a80671b80681c816a1c816b1d816d1d816e1e81701f81721f817320817521817621817822817922827b23827c23827e24828025828125818326818426818627818827818928818b29818c29818e2a81902a81912b81932b80942c80962c80982d80992d809b2e7f9c2e7f9e2f7fa02f7fa1307ea3307ea5317ea6317da8327daa337dab337cad347cae347bb0357bb2357bb3367ab5367ab73779b83779ba3878bc3978bd3977bf3a77c03a76c23b75c43c75c53c74c73d73c83e73ca3e72cc3f71cd4071cf4070d0416fd2426fd3436ed5446dd6456cd8456cd9466bdb476adc4869de4968df4a68e04c67e24d66e34e65e44f64e55064e75263e85362e95462ea5661eb5760ec5860ed5a5fee5b5eef5d5ef05f5ef1605df2625df2645cf3655cf4675cf4695cf56b5cf66c5cf66e5cf7705cf7725cf8745cf8765cf9785df9795df97b5dfa7d5efa7f5efa815ffb835ffb8560fb8761fc8961fc8a62fc8c63fc8e64fc9065fd9266fd9467fd9668fd9869fd9a6afd9b6bfe9d6cfe9f6dfea16efea36ffea571fea772fea973feaa74feac76feae77feb078feb27afeb47bfeb67cfeb77efeb97ffebb81febd82febf84fec185fec287fec488fec68afec88cfeca8dfecc8ffecd90fecf92fed194fed395fed597fed799fed89afdda9cfddc9efddea0fde0a1fde2a3fde3a5fde5a7fde7a9fde9aafdebacfcecaefceeb0fcf0b2fcf2b4fcf4b6fcf6b8fcf7b9fcf9bbfcfbbdfcfdbf"));
bq(ob("00000401000501010601010802010a02020c02020e03021004031204031405041706041907051b08051d09061f0a07220b07240c08260d08290e092b10092d110a30120a32140b34150b37160b39180c3c190c3e1b0c411c0c431e0c451f0c48210c4a230c4c240c4f260c51280b53290b552b0b572d0b592f0a5b310a5c320a5e340a5f3609613809623909633b09643d09653e0966400a67420a68440a68450a69470b6a490b6a4a0c6b4c0c6b4d0d6c4f0d6c510e6c520e6d540f6d550f6d57106e59106e5a116e5c126e5d126e5f136e61136e62146e64156e65156e67166e69166e6a176e6c186e6d186e6f196e71196e721a6e741a6e751b6e771c6d781c6d7a1d6d7c1d6d7d1e6d7f1e6c801f6c82206c84206b85216b87216b88226a8a226a8c23698d23698f24699025689225689326679526679727669827669a28659b29649d29649f2a63a02a63a22b62a32c61a52c60a62d60a82e5fa92e5eab2f5ead305dae305cb0315bb1325ab3325ab43359b63458b73557b93556ba3655bc3754bd3853bf3952c03a51c13a50c33b4fc43c4ec63d4dc73e4cc83f4bca404acb4149cc4248ce4347cf4446d04545d24644d34743d44842d54a41d74b3fd84c3ed94d3dda4e3cdb503bdd513ade5238df5337e05536e15635e25734e35933e45a31e55c30e65d2fe75e2ee8602de9612bea632aeb6429eb6628ec6726ed6925ee6a24ef6c23ef6e21f06f20f1711ff1731df2741cf3761bf37819f47918f57b17f57d15f67e14f68013f78212f78410f8850ff8870ef8890cf98b0bf98c0af98e09fa9008fa9207fa9407fb9606fb9706fb9906fb9b06fb9d07fc9f07fca108fca309fca50afca60cfca80dfcaa0ffcac11fcae12fcb014fcb216fcb418fbb61afbb81dfbba1ffbbc21fbbe23fac026fac228fac42afac62df9c72ff9c932f9cb35f8cd37f8cf3af7d13df7d340f6d543f6d746f5d949f5db4cf4dd4ff4df53f4e156f3e35af3e55df2e661f2e865f2ea69f1ec6df1ed71f1ef75f1f179f2f27df2f482f3f586f3f68af4f88ef5f992f6fa96f8fb9af9fc9dfafda1fcffa4"));
bq(ob("0d088710078813078916078a19068c1b068d1d068e20068f2206902406912605912805922a05932c05942e05952f059631059733059735049837049938049a3a049a3c049b3e049c3f049c41049d43039e44039e46039f48039f4903a04b03a14c02a14e02a25002a25102a35302a35502a45601a45801a45901a55b01a55c01a65e01a66001a66100a76300a76400a76600a76700a86900a86a00a86c00a86e00a86f00a87100a87201a87401a87501a87701a87801a87a02a87b02a87d03a87e03a88004a88104a78305a78405a78606a68707a68808a68a09a58b0aa58d0ba58e0ca48f0da4910ea3920fa39410a29511a19613a19814a099159f9a169f9c179e9d189d9e199da01a9ca11b9ba21d9aa31e9aa51f99a62098a72197a82296aa2395ab2494ac2694ad2793ae2892b02991b12a90b22b8fb32c8eb42e8db52f8cb6308bb7318ab83289ba3388bb3488bc3587bd3786be3885bf3984c03a83c13b82c23c81c33d80c43e7fc5407ec6417dc7427cc8437bc9447aca457acb4679cc4778cc4977cd4a76ce4b75cf4c74d04d73d14e72d24f71d35171d45270d5536fd5546ed6556dd7566cd8576bd9586ada5a6ada5b69db5c68dc5d67dd5e66de5f65de6164df6263e06363e16462e26561e26660e3685fe4695ee56a5de56b5de66c5ce76e5be76f5ae87059e97158e97257ea7457eb7556eb7655ec7754ed7953ed7a52ee7b51ef7c51ef7e50f07f4ff0804ef1814df1834cf2844bf3854bf3874af48849f48948f58b47f58c46f68d45f68f44f79044f79143f79342f89441f89540f9973ff9983ef99a3efa9b3dfa9c3cfa9e3bfb9f3afba139fba238fca338fca537fca636fca835fca934fdab33fdac33fdae32fdaf31fdb130fdb22ffdb42ffdb52efeb72dfeb82cfeba2cfebb2bfebd2afebe2afec029fdc229fdc328fdc527fdc627fdc827fdca26fdcb26fccd25fcce25fcd025fcd225fbd324fbd524fbd724fad824fada24f9dc24f9dd25f8df25f8e125f7e225f7e425f6e626f6e826f5e926f5eb27f4ed27f3ee27f3f027f2f227f1f426f1f525f0f724f0f921"));
MF.prototype={kc:function(){this.re=0},jc:function(){(this.Cf||0!==this.Cf&&1===this.re)&&this.Db.closePath();this.Cf=1-this.Cf},point:function(h,n){h=+h;n=+n;switch(this.re){case 0:this.re=1;this.Cf?this.Db.lineTo(h,n):this.Db.moveTo(h,n);break;case 1:this.re=2;default:this.Db.lineTo(h,n)}}};(function(h){function n(q){return new OF(h(q))}n.wm=h;return n})(NF);OF.prototype={kc:function(){this.wm.kc()},jc:function(){this.wm.jc()},point:function(h,n){this.wm.point(n*Math.sin(h),n*-Math.cos(h))}};tv.prototype=
{kc:function(){this.qc=this.Jb=this.yc=this.Mb=this.Om=NaN;this.re=0},jc:function(){switch(this.re){case 2:this.Db.lineTo(this.Jb,this.Mb);break;case 3:sv(this,this.Om,QF(this,this.Om))}(this.Cf||0!==this.Cf&&1===this.re)&&this.Db.closePath();this.Cf=1-this.Cf},point:function(h,n){var q=NaN;h=+h;n=+n;if(h!==this.Jb||n!==this.Mb){switch(this.re){case 0:this.re=1;this.Cf?this.Db.lineTo(h,n):this.Db.moveTo(h,n);break;case 1:this.re=2;break;case 2:this.re=3;sv(this,QF(this,q=PF(this,h,n)),q);break;default:sv(this,
this.Om,q=PF(this,h,n))}this.qc=this.Jb;this.Jb=h;this.yc=this.Mb;this.Mb=n;this.Om=q}}};(RF.prototype=Object.create(tv.prototype)).point=function(h,n){tv.prototype.point.call(this,n,h)};SF.prototype={moveTo:function(h,n){this.Db.moveTo(n,h)},closePath:function(){this.Db.closePath()},lineTo:function(h,n){this.Db.lineTo(n,h)},bezierCurveTo:function(h,n,q,v,y,A){this.Db.bezierCurveTo(n,h,v,q,A,y)}};tg.prototype={constructor:tg,scale:function(h){return 1===h?this:new tg(this.k*h,this.x,this.y)},translate:function(h,
n){return 0===h&0===n?this:new tg(this.k,this.x+this.k*h,this.y+this.k*n)},apply:function(h){return[h[0]*this.k+this.x,h[1]*this.k+this.y]},invert:function(h){return[(h[0]-this.x)/this.k,(h[1]-this.y)/this.k]},lw:function(h){return(h-this.x)/this.k},mw:function(h){return(h-this.y)/this.k},toString:function(){return"translate("+this.x+","+this.y+") scale("+this.k+")"}};var UF=new tg(1,0,0);OL=function(h){return 1-Math.pow(2,-10*h)};PL=Yu;QL=ac;wI=function(){function h(K){var L,P=K.length,X,ka=!1,oa;
null==y&&(E=A(oa=fF()));for(L=0;L<=P;++L)!(L<P&&v(X=K[L],L,K))===ka&&((ka=!ka)?E.kc():E.jc()),ka&&E.point(+n(X,L,K),+q(X,L,K));if(oa)return E=null,oa+""||null}var n=AW,q=BW,v=rv(!0),y=null,A=NF,E=null;h.x=function(K){return arguments.length?(n="function"===typeof K?K:rv(+K),h):n};h.y=function(K){return arguments.length?(q="function"===typeof K?K:rv(+K),h):q};h.curve=function(K){return arguments.length?(A=K,null!=y&&(E=A(y)),h):A};h.context=function(K){return arguments.length?(null==K?y=E=null:E=A(y=
K),h):y};return h};xI=function(h){return new RF(h)};RL=function(){function h(I){I.nx("__zoom",TF).on("wheel.zoom",E).on("mousedown.zoom",K).on("dblclick.zoom",L).filter(kc).on("touchstart.zoom",P).on("touchmove.zoom",X).on("touchend.zoom touchcancel.zoom",ka).style("touch-action","none").style("-webkit-tap-highlight-color","rgba(0,0,0,0)")}function n(I,ha){ha=Math.max(vb[0],Math.min(vb[1],ha));return ha===I.k?I:new tg(ha,I.x,I.y)}function q(I,ha,Ga){var Ta=ha[0]-Ga[0]*I.k;ha=ha[1]-Ga[1]*I.k;return Ta===
I.x&&ha===I.y?I:new tg(I.k,Ta,ha)}function v(I,ha,Ga){I.on("start.zoom",function(){y(this,arguments).start()}).on("interrupt.zoom end.zoom",function(){y(this,arguments).end()}).ld("zoom",function(){var Ta=arguments,wb=y(this,Ta),Qa=Ba.apply(this,Ta),Rb=Ga||[(+Qa[0][0]+ +Qa[1][0])/2,(+Qa[0][1]+ +Qa[1][1])/2],pd=Math.max(Qa[1][0]-Qa[0][0],Qa[1][1]-Qa[0][1]);Qa=this.__zoom;var dk="function"===typeof ha?ha.apply(this,Ta):ha,fk=Eb(Qa.invert(Rb).concat(pd/Qa.k),dk.invert(Rb).concat(pd/dk.k));return function(zd){if(1===
zd)zd=dk;else{zd=fk(zd);var ek=pd/zd[2];zd=new tg(ek,Rb[0]-zd[0]*ek,Rb[1]-zd[1]*ek)}wb.zoom(null,zd)}})}function y(I,ha){for(var Ga=0,Ta=tb.length,wb;Ga<Ta;++Ga)if((wb=tb[Ga]).mi===I)return wb;return new A(I,ha)}function A(I,ha){this.mi=I;this.Ye=ha;this.index=-1;this.active=0;this.cl=Ba.apply(I,ha)}function E(){if(oa.apply(this,arguments)){var I=y(this,arguments),ha=this.__zoom,Ga=Math.max(vb[0],Math.min(vb[1],ha.k*Math.pow(2,Ya.apply(this,arguments)))),Ta=hd(this);if(I.Ss){if(I.Id[0][0]!==Ta[0]||
I.Id[0][1]!==Ta[1])I.Id[1]=ha.invert(I.Id[0]=Ta);clearTimeout(I.Ss)}else{if(ha.k===Ga)return;I.Id=[Ta,ha.invert(Ta)];Up(this);I.start()}om();I.Ss=setTimeout(function(){I.Ss=null;I.end()},150);I.zoom("mouse",Va(q(n(ha,Ga),I.Id[0],I.Id[1]),I.cl,lc))}}function K(){if(!za&&oa.apply(this,arguments)){var I=y(this,arguments),ha=ac(LL.view).on("mousemove.zoom",function(){om();if(!I.xj){var Qa=LL.clientX-Ta,Rb=LL.clientY-wb;I.xj=Qa*Qa+Rb*Rb>Fa}I.zoom("mouse",Va(q(I.mi.__zoom,I.Id[0]=hd(I.mi),I.Id[1]),I.cl,
lc))},!0).on("mouseup.zoom",function(){ha.on("mousemove.zoom mouseup.zoom",null);sr(LL.view,I.xj);om();I.end()},!0),Ga=hd(this),Ta=LL.clientX,wb=LL.clientY;If(LL.view);LL.stopImmediatePropagation();I.Id=[Ga,this.__zoom.invert(Ga)];Up(this);I.start()}}function L(){if(oa.apply(this,arguments)){var I=this.__zoom,ha=hd(this),Ga=I.invert(ha);I=Va(q(n(I,I.k*(LL.shiftKey?.5:2)),ha,Ga),Ba.apply(this,arguments),lc);om();0<Hc?ac(this).transition().duration(Hc).call(v,I,ha):ac(this).call(h.transform,I)}}function P(){if(oa.apply(this,
arguments)){var I=y(this,arguments),ha=LL.changedTouches,Ga=ha.length,Ta;LL.stopImmediatePropagation();for(Ta=0;Ta<Ga;++Ta){var wb=ha[Ta];var Qa=Ac(this,ha,wb.identifier);Qa=[Qa,this.__zoom.invert(Qa),wb.identifier];if(I.wc)I.Uc||(I.Uc=Qa);else{I.wc=Qa;var Rb=!0}}if(Hb&&(Hb=clearTimeout(Hb),!I.Uc)){I.end();(Qa=ac(this).on("dblclick.zoom"))&&Qa.apply(this,arguments);return}Rb&&(Hb=setTimeout(function(){Hb=null},500),Up(this),I.start())}}function X(){var I=y(this,arguments),ha=LL.changedTouches,Ga=
ha.length,Ta;om();Hb&&(Hb=clearTimeout(Hb));for(Ta=0;Ta<Ga;++Ta){var wb=ha[Ta];var Qa=Ac(this,ha,wb.identifier);I.wc&&I.wc[2]===wb.identifier?I.wc[0]=Qa:I.Uc&&I.Uc[2]===wb.identifier&&(I.Uc[0]=Qa)}wb=I.mi.__zoom;if(I.Uc){Qa=I.wc[0];ha=I.wc[1];Ta=I.Uc[0];Ga=I.Uc[1];var Rb=(Rb=Ta[0]-Qa[0])*Rb+(Rb=Ta[1]-Qa[1])*Rb;var pd=(pd=Ga[0]-ha[0])*pd+(pd=Ga[1]-ha[1])*pd;wb=n(wb,Math.sqrt(Rb/pd));Qa=[(Qa[0]+Ta[0])/2,(Qa[1]+Ta[1])/2];Rb=[(ha[0]+Ga[0])/2,(ha[1]+Ga[1])/2]}else if(I.wc)Qa=I.wc[0],Rb=I.wc[1];else return;
I.zoom("touch",Va(q(wb,Qa,Rb),I.cl,lc))}function ka(){var I=y(this,arguments),ha=LL.changedTouches,Ga=ha.length,Ta;LL.stopImmediatePropagation();za&&clearTimeout(za);za=setTimeout(function(){za=null},500);for(Ta=0;Ta<Ga;++Ta){var wb=ha[Ta];I.wc&&I.wc[2]===wb.identifier?delete I.wc:I.Uc&&I.Uc[2]===wb.identifier&&delete I.Uc}I.Uc&&!I.wc&&(I.wc=I.Uc,delete I.Uc);I.wc?I.wc[1]=this.__zoom.invert(I.wc[0]):I.end()}var oa=DW,Ba=EW,Va=HW,Ya=FW,kc=GW,vb=[0,Infinity],lc=[[-Infinity,-Infinity],[Infinity,Infinity]],
Hc=250,Eb=BU,tb=[],La=d("start","zoom","end"),Hb,za,Fa=0;h.transform=function(I,ha){var Ga=I.selection?I.selection():I;Ga.nx("__zoom",TF);I!==Ga?v(I,ha):Ga.sC().each(function(){y(this,arguments).start().zoom(null,"function"===typeof ha?ha.apply(this,arguments):ha).end()})};h.IE=function(I,ha,Ga){h.transform(I,function(){return Va(this.__zoom.translate("function"===typeof ha?ha.apply(this,arguments):ha,"function"===typeof Ga?Ga.apply(this,arguments):Ga),Ba.apply(this,arguments),lc)})};A.prototype=
{start:function(){1===++this.active&&(this.index=tb.push(this)-1,this.emit("start"));return this},zoom:function(I,ha){this.Id&&"mouse"!==I&&(this.Id[1]=ha.invert(this.Id[0]));this.wc&&"touch"!==I&&(this.wc[1]=ha.invert(this.wc[0]));this.Uc&&"touch"!==I&&(this.Uc[1]=ha.invert(this.Uc[0]));this.mi.__zoom=ha;this.emit("zoom");return this},end:function(){0===--this.active&&(tb.splice(this.index,1),this.index=-1,this.emit("end"));return this},emit:function(I){var ha=new CW(h,I,this.mi.__zoom),Ga=La.apply;
I=[I,this.mi,this.Ye];var Ta=LL;ha.ys=LL;LL=ha;try{Ga.apply(La,I)}finally{LL=Ta}}};h.wheelDelta=function(I){return arguments.length?(Ya="function"===typeof I?I:uv(+I),h):Ya};h.filter=function(I){return arguments.length?(oa="function"===typeof I?I:uv(!!I),h):oa};h.cl=function(I){return arguments.length?(Ba="function"===typeof I?I:uv([[+I[0][0],+I[0][1]],[+I[1][0],+I[1][1]]]),h):Ba};h.PD=function(I){return arguments.length?(vb[0]=+I[0],vb[1]=+I[1],h):[vb[0],vb[1]]};h.duration=function(I){return arguments.length?
(Hc=+I,h):Hc};h.interpolate=function(I){return arguments.length?(Eb=I,h):Eb};h.on=function(){var I=La.on.apply(La,arguments);return I===La?h:I};return h};SL=UF;Qd.xG=Qd.svg;Qd.EG=Qd.xhtml;Qd.FG=Qd.xlink;Qd.xml=Qd.xml;Qd.GG=Qd.xmlns})();function TL(a){0<=a.Sd&&(clearTimeout(a.Sd),a.Sd=-1);var b=a.qx;b.C&&jE(b.C);cn(a.P)}function UL(a,b,c,d=!1,e=!1){VL(a);a.aa(b)?(TL(a),a.kh(c,a.ga,b,[],d,e,!0)):a.si.C.postMessage({eventType:6,modelGraphId:a.ga.id,showOnNodeItemTypes:a.F,nodeDataProviderRuns:a.lb,nodeId:b,rendererId:c,noNodeShake:d,select:e})}
function WL(a,b,c,d=!1,e,f=!1,g,k){VL(a);a.si.C.postMessage({eventType:4,modelGraphId:a.ga.id,showOnNodeItemTypes:g||a.F,nodeDataProviderRuns:a.lb,selectedNodeId:b,targetDeepestGroupNodeIdsToExpand:c,rendererId:a.rendererId,forRestoringUiState:d,rectToZoomFit:e,clearAllExpandStates:f,forRestoringSnapshotAfterTogglingFlattenLayers:k})}
function XL(a,b,c=!1,d){VL(a);b={eventType:2,modelGraphId:a.ga.id,groupNodeId:null==b?void 0:b.id,expand:null==d?!(null==b?0:b.expanded):d,showOnNodeItemTypes:a.F,nodeDataProviderRuns:a.lb,rendererId:a.rendererId,paneId:a.paneId,all:c,ts:Date.now()};a.si.C.postMessage(b)}
function YL(a,b){const c=QL(b);let d=0,e=0;a.zoom.PD([1E-4,20]).wheelDelta(()=>-LL.deltaY*(LL.deltaMode?120:1)/150).filter(()=>{"mousedown"===LL.type&&(d=a.hh,e=a.ih);if(2===LL.button||a.ml&&LL.ctrlKey&&0===LL.button&&"mousedown"===LL.type)return!1;if("dblclick"===LL.type)return LL.stopPropagation(),a.Ik(LL.altKey),!1;if("wheel"===LL.type&&!LL.ctrlKey){const f=.5/a.J;a.zoom.IE(c,-Number(LL.deltaX)*f,-Number(LL.deltaY)*f);LL.preventDefault();return!1}return!0}).on("zoom",()=>{a.dl()}).on("end",()=>
{a.By(d,e)});a.zoom.interpolate(PL);c.call(a.zoom)}function VL(a){TL(a);a.Sd=window.setTimeout(()=>{0>a.Sd||(a.qx.open("Processing. Please wait..."),cn(a.P))},1E3)}function ZL(a,b){b=a.ga.nodesById[b];null==b||b.expanded||XL(a,b,!1,!0)}function $L(a,b){wJ(a.C,a.paneId,{nodeId:b,rendererId:a.rendererId,Bd:""===b?!1:oI(a.ga.nodesById[b])})}function aM(a,b){a.ga=Object.assign({},b,{edgesByGroupNodeIds:Object.assign({},a.ga.edgesByGroupNodeIds,b.edgesByGroupNodeIds)})}
function bM(a){if(a.ga){a.Va=[];a.N=[];a.sa={};a.Lb=[];var b=!0,c=!1,d=e=>{const f=a.ga.nodesById[e||""];if(b&&f){a.Va.push({type:0,id:f.id,node:f});var g={node:f,index:a.Va.length-1};a.N.push(g);a.sa[g.node.id]=g;oI(f)&&f.sectionContainer&&(c=!0)}b=!1;if(e&&f&&f.expanded||!e)for(const k of a.ga.edgesByGroupNodeIds[e||""]||[])a.Va.push({type:1,id:k.id,edge:k}),a.Lb.push({edge:k,index:a.Va.length-1});g=[];e?f.expanded&&(g=(f.nsChildrenIds||[]).map(k=>a.ga.nodesById[k])):g=a.ga.rootNodes;for(const k of g)e=
{type:0,id:k.id,node:k},!c&&oI(k)&&k.sectionContainer&&(c=!0),a.Va.push(e),e={node:k,index:a.Va.length-1},a.N.push(e),a.sa[e.node.id]=e,oI(k)&&k.expanded&&d(k.id)};d(a.Vl);DJ(a.C,a.paneId,c)}}function cM(a){return(a.x||0)+(a.globalX||0)}function dM(a){return(a.y||0)+(a.globalY||0)}function eM(a,b,c,d){b?setTimeout(()=>{const e=c.nodesById[b];a.Yc({x:cM(e),y:dM(e),width:e.width||0,height:e.height||0},.9,d,!1,!0,!0)},0):setTimeout(()=>{a.Wd(.9,d)})}
function fM(a,b,c=!1,d){1===b.nodeType&&0!==(b.nsChildrenIds||[]).length&&XL(a,b,c,d)}function gM(a){if(!a.selectedNodeId||oI(a.ga.nodesById[a.selectedNodeId]))return!1;const b=a.ga.nodesById[a.Vl||""];return b&&oI(b)&&!(b.descendantsOpNodeIds||[]).includes(a.selectedNodeId)||!a.aa(a.selectedNodeId)?!1:!0}function hM(a,b,c,d){return a.ga.edgesByGroupNodeIds[""===b?"":`${b}___group___`].find(e=>{var f=a.ga.nodesById[e.toNodeId];e=iM(a.ga.nodesById[e.fromNodeId],c);f=iM(f,d);return e&&f})}
function jM(a,b,c){let d=b;const e=[];for(;d;){oI(d)&&!d.expanded&&e.push(d);if(d.namespace===c)break;d=a.ga.nodesById[d.nsParentId||""]}return 0<e.length?e[e.length-1]:b}
function kM(a,b,c){var d=cM(c);const e=dM(c),f=c.width||0;c=c.height||0;d=[{point:{x:d+f/2,y:e},distance:0,direction:"horizontal"},{point:{x:d+f,y:e+c/2},distance:0,direction:"vertical"},{point:{x:d+f/2,y:e+c},distance:0,direction:"horizontal"},{point:{x:d,y:e+c/2},distance:0,direction:"vertical"}];for(const g of d)g.distance=Math.pow(a-g.point.x,2)+Math.pow(b-g.point.y,2);d.sort((g,k)=>g.distance-k.distance);return d[0].direction!==d[1].direction?lM(a,b,d[0].point.x,d[0].point.y,d[0].direction)>=
lM(a,b,d[1].point.x,d[1].point.y,d[1].direction)?d[0].point:d[1].point:d[0].point}
function mM(a){const b=a.ga.nodesById[a.selectedNodeId],c=[],d=[],e={},f=[];for(const p of b.incomingEdges||[]){var g=a.ga.nodesById[p.sourceNodeId];if(!g)continue;var k=FI(g.namespace,b.namespace),m=hM(a,k,g.id,b.id);if(!m)continue;c.push(m);const t=jM(a,g,k);d.push(t);null==e[t.id]&&(e[t.id]=[]);e[t.id].push(g);g=[];const w=a.ga.nodesById[m.fromNodeId];m.fromNodeId!==t.id&&(k=kM(m.points[0].x+(w.globalX||0),m.points[0].y+(w.globalY||0),t),g.push({x:k.x-(t.globalX||0),y:k.y-(t.globalY||0)}));g.push(...m.points.map(B=>
({x:B.x-(t.globalX||0)+(w.globalX||0),y:B.y-(t.globalY||0)+(w.globalY||0)})));m.toNodeId!==a.selectedNodeId&&(m=kM(m.points[m.points.length-1].x+(w.globalX||0),m.points[m.points.length-1].y+(w.globalY||0),b),g.push({x:m.x-(t.globalX||0),y:m.y-(t.globalY||0)}));f.push({id:`overlay_${t.id}___${b.id}`,fromNodeId:t.id,toNodeId:b.id,points:g,type:"incoming"})}return{cs:c,hC:d,Ee:e,Ml:f}}
function nM(a){const b=a.ga.nodesById[a.selectedNodeId],c=[],d=[],e={},f=[];for(const p of b.outgoingEdges||[]){var g=a.ga.nodesById[p.targetNodeId];if(!g)continue;var k=FI(g.namespace,b.namespace),m=hM(a,k,b.id,g.id);if(!m)continue;c.push(m);k=jM(a,g,k);d.push(k);null==e[k.id]&&(e[k.id]=[]);e[k.id].push(g);g=[];const t=a.ga.nodesById[m.fromNodeId];if(m.fromNodeId!==a.selectedNodeId){const w=kM(m.points[0].x+(t.globalX||0),m.points[0].y+(t.globalY||0),b);g.push({x:w.x-(b.globalX||0),y:w.y-(b.globalY||
0)})}g.push(...m.points.map(w=>({x:w.x-(b.globalX||0)+(t.globalX||0),y:w.y-(b.globalY||0)+(t.globalY||0)})));m.toNodeId!==k.id&&(m=kM(m.points[m.points.length-1].x+(t.globalX||0),m.points[m.points.length-1].y+(t.globalY||0),k),g.push({x:m.x-(b.globalX||0),y:m.y-(b.globalY||0)}));f.push({id:`overlay_${b.id}___${k.id}`,fromNodeId:b.id,toNodeId:k.id,points:g,type:"outgoing"})}return{cs:c,hC:d,kf:e,Ml:f}}
function oM(a){if(a.selectedNodeId&&nI(a.ga.nodesById[a.selectedNodeId])){for(var b=new Set,c=new Set,d=[a.selectedNodeId];0<d.length;){var e=d.shift();if(!c.has(e)){c.add(e);var f=a.ga.nodesById[e];f.hideInLayout||b.add(e);for(const g of f.incomingEdges||[])d.push(g.sourceNodeId)}}c=new Set;for(d=[a.selectedNodeId];0<d.length;)if(e=d.shift(),!c.has(e)){c.add(e);f=a.ga.nodesById[e];f.hideInLayout||b.add(e);for(const g of f.outgoingEdges||[])d.push(g.targetNodeId)}for(const g of[...b])for(d=g;;){d=
a.ga.nodesById[d];if(!d.nsParentId||b.has(d.nsParentId))break;d=d.nsParentId;b.add(d)}a.ua={BG:a.selectedNodeId,Os:b}}}function pM(a){const b=a.gh()||{},c={},d=new Set;for(const {node:e}of a.N)if(oI(e)&&!e.expanded){const f={};for(const g of e.descendantsOpNodeIds||[]){let k;const m=(null==(k=b[a.ga.nodesById[g].id])?void 0:k.bgColor)||"";m&&(null==f[m]&&(f[m]=0),f[m]++,d.add(m))}c[e.id]=f}return{XB:c,pE:[...d].sort((e,f)=>e.localeCompare(f))}}
function qM(a,b){b=new Au({uc:Lu(Gv(Ku(Rv(a.ox.position(),b),[{cb:"start",eb:"bottom",Na:"start",Ta:"top"},{cb:"start",eb:"top",Na:"start",Ta:"bottom"}]),b.clientWidth),20),Tb:!0,dc:"cdk-overlay-transparent-backdrop",maxHeight:"400px",hb:"io-tree-popup-container"});const c=a.ox.create(b);a.Ld=new Uv(ZK,a.xb);c.dd().subscribe(()=>{c.dispose()});return c}
function rM(a,b,c,d){const e=qM(a,b);b=e.attach(a.Ld);c=$K(c,d);b.instance.Eo=!0;b.instance.rendererId=a.rendererId;YK(b.instance,c);b.instance.he.subscribe(()=>{e.dispose()})}function sM(a){return nI(a)?"Placeholder"===a.label?a.id:a.label:oI(a)?a.label:"-"}function iM(a,b){return nI(a)&&a.id===b||oI(a)&&(a.descendantsOpNodeIds||[]).includes(b)}function lM(a,b,c,d,e){return"horizontal"===e?Math.atan(Math.abs(d-b)/Math.abs(c-a)):Math.atan(Math.abs(c-a)/Math.abs(d-b))}
var tM=class{constructor(){this.Qb=!1;this.Kl=new Fk;this.Ag="";this.Wg=!1;this.Va=[];this.N=[];this.sa={};this.Lb=[];this.zoom=RL();this.J=1;this.ih=this.hh=0;this.Gl=this.selectedNodeId="";this.lb={};this.Ld=null;this.Sd=-1;this.Py=void 0;this.C=lf(VJ);this.si=lf(bJ);this.Zw=lf(KL);this.G=lf(yi);this.Oe=lf(aJ);this.ox=lf(bw);this.xb=lf(kp);this.P=lf(Sr);this.qx=lf(vE);this.ml="undefined"!==typeof navigator&&/Macintosh/.test(navigator.userAgent);this.F={};this.ja={};this.Oy=a=>{TL(this);a=a.data;
switch(a.eventType){case 3:this.rendererId===a.rendererId&&this.Nk(a.modelGraph,a.rendererId,a.groupNodeId,a.expanded,a.deepestExpandedGroupNodeIds);break;case 5:this.rendererId===a.rendererId&&this.al(a.rendererId,a.modelGraph,a.selectedNodeId,a.forRestoringUiState,a.rectToZoomFit,a.forRestoringSnapshotAfterTogglingFlattenLayers,a.targetDeepestGroupNodeIdsToExpand);break;case 7:this.rendererId===a.rendererId&&this.kh(a.rendererId,a.modelGraph,a.nodeId,a.deepestExpandedGroupNodeIds,!0===a.noNodeShake,
!0===a.select);break;case 10:this.paneId===a.paneId&&this.Kl.emit({id:a.rendererId,Fn:a.modelGraph.nodesById[a.groupNodeId],initialPosition:a.initialPosition,ga:a.modelGraph})}};this.Qg=Bs(()=>{const a=nJ(this.C,this.paneId);if(a)return a.Qg});this.gh=Bs(()=>{let a;return((null==(a=GL(this.Zw,this.paneId,this.ga))?void 0:a.results)||{})[this.ga.id]});this.si.C.addEventListener("message",this.Oy);Js(()=>{this.iz=this.C.G()});C(this.C.fa,sL(this.G)).subscribe(()=>{let a;this.rendererId===(null==(a=
this.iz)?void 0:a.id)&&this.Wd()});Js(()=>{const a=this.C.F();(null==a?void 0:a.rendererId)===this.rendererId&&(a&&UL(this,a.nodeId,a.rendererId,a.noNodeShake,a.select),this.C.F.set(void 0))},{Wp:!0});Js(()=>{var a=nJ(this.C,this.paneId);if(a&&a.modelGraph&&(a=a.eD)){{const b=this.ga.nodesById[a];b?(WL(this,a,b.nsParentId?[b.nsParentId]:[]),a=!0):a=!1}a&&EJ(this.C,this.paneId)}},{Wp:!0});Js(()=>{const a=vL(this.Zw,this.ga);this.lb={};for(const b of a)this.lb[b.runId]=b});Js(()=>{const a=this.gh();
a!==this.Py&&(this.uy(),this.Py=a)});Js(()=>{if(nJ(this.C,this.paneId)){var a=kJ(this.C,this.paneId,this.rendererId);JSON.stringify(a)!==JSON.stringify(this.F)&&(this.F=a,WL(this,this.selectedNodeId))}});C(this.C.ca,sL(this.G)).subscribe(a=>{a.rendererId===this.rendererId&&XL(this,void 0,!0,a.Ov)})}va(){this.si.C.removeEventListener("message",this.Oy)}Ik(){}Gk(){}dl(){this.J=LL.transform.k;this.hh=LL.transform.x;this.ih=LL.transform.y}By(){}Nk(a){aM(this,a);bM(this);this.selectedNodeId&&!this.aa(this.selectedNodeId)&&
wJ(this.C,this.paneId,{nodeId:"",rendererId:this.rendererId,Bd:!1})}al(a,b){aM(this,b);bM(this)}kh(a,b,c,d,e,f){aM(this,b);bM(this);f&&wJ(this.C,this.paneId,{nodeId:c,rendererId:a,Bd:oI(this.ga.nodesById[c])})}Yc(){}Wd(){}uy(){}aa(){return!1}Dl(){(this.Wg=!this.Wg)?oM(this):this.ua=void 0}};tM.K=function(a){return new (a||tM)};
tM.Da=Gf({type:tM,ha:[["renderer"]],inputs:{modelGraph:"modelGraph",rendererId:"rendererId",paneId:"paneId",Vl:"rootNodeId",Qb:"inPopup"},outputs:{Kl:"openInPopupClicked"},ka:!0,features:[Mr],za:0,Aa:0,ta:function(){},pb:2});PH`https://www.gstatic.com/external_hosted/threejs-r134/three.min.js`;const HI=PH`/static_files/main_deps.js`;
function uM(a){return sa(function*(){"undefined"===typeof THREE&&(yield GI());const b=yield Promise.all([vM("/static_files/GoogleSansTextRegular.png"),vM("/static_files/GoogleSansTextMedium.png"),vM("/static_files/GoogleSansTextBold.png"),vM("/static_files/icons_2024021202.png"),wM("/static_files/GoogleSansTextRegular.json"),wM("/static_files/GoogleSansTextMedium.json"),wM("/static_files/GoogleSansTextBold.json"),wM("/static_files/icons_2024021202.json")]);a.ja=b[0];a.fa=b[1];a.aa=b[2];a.ca=b[3];
a.D=b[4].an;a.J=b[5].an;a.F=b[6].an;a.G=b[7].an;a.C=b[4].zn;a.T=b[5].zn;a.O=b[6].zn;a.P=b[7].zn})}function vM(a){return sa(function*(){return new Promise(b=>{(new THREE.TextureLoader).load(a,c=>{b(c)})})})}function wM(a){return sa(function*(){const b=yield(yield fetch(a)).json(),c={};for(const d of b.chars)c[d.char]=d;return{zn:b,an:c}})}function xM(a,b){switch(b){case 0:return a.D;case 1:return a.J;case 2:return a.F;case 4:return a.G;default:return a.D}}
function yM(a,b){switch(b){case 0:return a.C;case 1:return a.T;case 2:return a.O;case 4:return a.P;default:return a.C}}var zM=class{constructor(){this.D={};this.J={};this.F={};this.G={};this.N=new Promise(a=>{const b=this;return sa(function*(){yield uM(b);a()})})}};zM.K=function(a){return new (a||zM)};zM.pa=Md({ma:zM,ba:zM.K,ea:"root"});function AM(a,b,c){if(0===b.length)a.C=void 0,a.D=void 0;else{var d=[],e=[],f=[],g=[],k=[],m=[],p=[],t=[],w={},B={};a.G={};var G=0,J=0;for(const {edge:ia,index:O}of b){b=ia.curvePoints||[];const T=c.nodesById[ia.fromNodeId],Q=c.nodesById[ia.toNodeId],la=T.globalX||0,F=T.globalY||0;for(let M=0;M<b.length-1;M++){var N=b[M],aa=b[M+1],ca=`${T.id}__${Q.id}___${M}`;N=[N.x+la,N.y+F,aa.x+la,aa.y+F];(aa=a.ca[ca])?d.push(...aa.JB):d.push(...N);e.push(...N);m.push(.001*O);k.push(a.color.r,a.color.g,a.color.b);
w[ca]={JB:N,index:G};null==a.G[ia.id]&&(a.G[ia.id]=[]);a.G[ia.id].push(w[ca]);M===b.length-2&&(ca=ia.id,(aa=a.N[ca])?f.push(...aa.HC):f.push(...N),g.push(...N),p.push(.001*O+5E-4),t.push(a.color.r,a.color.g,a.color.b),B[ca]={index:J,HC:N});G++}J++}a.ca=w;a.N=B;a.F=k;a.sa=m;a.na=p;c=(new THREE.InstancedBufferGeometry).copy(a.fa);c.instanceCount=m.length;c.setAttribute("endPoints",new THREE.InstancedBufferAttribute(new Float32Array(d),4));c.setAttribute("targetEndPoints",new THREE.InstancedBufferAttribute(new Float32Array(e),
4));c.setAttribute("color",new THREE.InstancedBufferAttribute(new Float32Array(k),3));c.setAttribute("yOffset",new THREE.InstancedBufferAttribute(new Float32Array(m),1));a.C=new THREE.Mesh(c,a.material);a.C.frustumCulled=!1;d=(new THREE.InstancedBufferGeometry).copy(a.ja);d.instanceCount=p.length;d.setAttribute("endPoints",new THREE.InstancedBufferAttribute(new Float32Array(f),4));d.setAttribute("targetEndPoints",new THREE.InstancedBufferAttribute(new Float32Array(g),4));d.setAttribute("color",new THREE.InstancedBufferAttribute(new Float32Array(t),
3));d.setAttribute("yOffset",new THREE.InstancedBufferAttribute(new Float32Array(p),1));a.D=new THREE.Mesh(d,a.ua);a.D.frustumCulled=!1}}function BM(a,b){if(a.C){var c=a.C.geometry.getAttribute("color");if(0<b.length)for(const d of b)for(const e of a.G[d]||[])c.setXYZ(e.index,.92,.92,.92),a.P.push(e);c.needsUpdate=!0}if(a.D){c=a.D.geometry.getAttribute("color");if(0<b.length)for(const d of b)b=a.N[d],c.setXYZ(b.index,.92,.92,.92),a.O.push(b);c.needsUpdate=!0}}
function CM(a,b){if(a.C){var c=a.C.geometry.getAttribute("yOffset");if(0<b.length)for(const d of b)for(const e of a.G[d]||[])c.setX(e.index,1E3),a.aa.push(e);c.needsUpdate=!0}if(a.D){c=a.D.geometry.getAttribute("yOffset");if(0<b.length)for(const d of b)b=a.N[d],c.setX(b.index,1E3),a.T.push(b);c.needsUpdate=!0}}
function DM(a){if(a.C){var b=a.C.geometry.getAttribute("color");if(0<a.P.length){for(var c of a.P){const d=c.index;b.setXYZ(d,a.F[3*d],a.F[3*d+1],a.F[3*d+2])}b.needsUpdate=!0}a.P=[]}if(a.D){b=a.D.geometry.getAttribute("color");if(0<a.O.length){for(const d of a.O)c=d.index,b.setXYZ(c,a.F[3*c],a.F[3*c+1],a.F[3*c+2]);b.needsUpdate=!0}a.O=[]}}
function EM(a){if(a.C){var b=a.C.geometry.getAttribute("yOffset");if(0<a.aa.length){for(var c of a.aa){const d=c.index;b.setX(d,a.sa[d])}b.needsUpdate=!0}a.aa=[]}if(a.D){b=a.D.geometry.getAttribute("yOffset");if(0<a.T.length){for(const d of a.T)c=d.index,b.setX(c,a.na[c]);b.needsUpdate=!0}a.T=[]}}function FM(a){a.ca={};a.G={};a.N={}}
var GM=class{constructor(a,b){this.color=a;this.wa=b;this.ca={};this.G={};this.N={};this.J={value:-1};this.F=[];this.sa=[];this.na=[];this.P=[];this.O=[];this.aa=[];this.T=[];this.fa=new THREE.PlaneGeometry(1,1);this.fa.rotateX(-Math.PI/2);this.material=new THREE.ShaderMaterial({uniforms:{edgeWidth:{value:this.wa},animationProgress:this.J},vertexShader:"\nprecision highp float;\n\n#define PI 3.1415926535897932384626433832795\n\nuniform float edgeWidth;\nuniform float animationProgress;\n\nattribute vec4 endPoints;\nattribute vec4 targetEndPoints;\nattribute vec3 color;\nattribute float yOffset;\n\nvarying vec3 vColor;\n\nfloat atan2(in float y, in float x) {\n  bool s = (abs(x) > abs(y));\n  return mix(PI/2.0 - atan(x,y), atan(y,x), s);\n}\n\nvoid main() {\n  vColor = color;\n\n  vec3 pos = position;\n\n  float curStartX = endPoints.x;\n  float curStartY = endPoints.y;\n  float curEndX = endPoints.z;\n  float curEndY = endPoints.w;\n  float targetStartX = targetEndPoints.x;\n  float targetStartY = targetEndPoints.y;\n  float targetEndX = targetEndPoints.z;\n  float targetEndY = targetEndPoints.w;\n\n  float progress = animationProgress * step(0.0, animationProgress); \n  float startX = curStartX + (targetStartX - curStartX) * progress;\n  float startY = curStartY + (targetStartY - curStartY) * progress;\n  float endX = curEndX + (targetEndX - curEndX) * progress;\n  float endY = curEndY + (targetEndY - curEndY) * progress;\n\n  float length = distance(vec2(startX, startY), vec2(endX, endY));\n  pos.x = (step(0.0, pos.x) * 2.0 - 1.0) * (edgeWidth / 2.0);\n  pos.z = (step(0.0, pos.z) * 2.0 - 1.0) * (length / 2.0);\n\n  float angle = PI / 2.0 - atan2(endY - startY, endX - startX);\n  float c = cos(angle);\n  float s = sin(angle);\n\n  float posX = pos.x;\n  float posZ = pos.z;\n  pos.x = posX * c + posZ * s;\n  pos.z = posZ * c - posX * s;\n\n  float centerX = (startX + endX) / 2.0;\n  float centerZ = (startY + endY) / 2.0;\n\n  gl_Position = projectionMatrix * modelViewMatrix *\n      vec4(pos.x + centerX, yOffset, pos.z + centerZ, 1.0);\n}\n",
fragmentShader:"\nprecision highp float;\n\nvarying vec3 vColor;\n\nvoid main() {\n  gl_FragColor = vec4(vColor, 1.0);\n}\n",transparent:!0});a=new THREE.Shape;a.moveTo(-3,-6).lineTo(0,-4).lineTo(3,-6).lineTo(0,0).lineTo(-3,-6);this.ja=new THREE.ShapeGeometry(a);this.ja.rotateX(-Math.PI/2);this.ua=new THREE.ShaderMaterial({uniforms:{animationProgress:this.J},vertexShader:"\nprecision highp float;\n\n#define PI 3.1415926535897932384626433832795\n\nuniform float animationProgress;\n\n// End points of the last segment of the edge.\nattribute vec4 endPoints;\nattribute vec4 targetEndPoints;\nattribute vec3 color;\nattribute float yOffset;\n\nvarying vec3 vColor;\n\nfloat atan2(in float y, in float x) {\n  bool s = (abs(x) > abs(y));\n  return mix(PI/2.0 - atan(x,y), atan(y,x), s);\n}\n\nvoid main() {\n  vColor = color;\n\n  vec3 pos = position;\n\n  float curStartX = endPoints.x;\n  float curStartY = endPoints.y;\n  float curEndX = endPoints.z;\n  float curEndY = endPoints.w;\n  float targetStartX = targetEndPoints.x;\n  float targetStartY = targetEndPoints.y;\n  float targetEndX = targetEndPoints.z;\n  float targetEndY = targetEndPoints.w;\n\n  float progress = animationProgress * step(0.0, animationProgress); \n  float startX = curStartX + (targetStartX - curStartX) * progress;\n  float startY = curStartY + (targetStartY - curStartY) * progress;\n  float endX = curEndX + (targetEndX - curEndX) * progress;\n  float endY = curEndY + (targetEndY - curEndY) * progress;\n\n  float angle = PI - atan2(endY - startY, endX - startX) + PI / 2.0;\n  float c = cos(angle);\n  float s = sin(angle);\n\n  float posX = pos.x;\n  float posZ = pos.z;\n  pos.x = posX * c + posZ * s;\n  pos.z = posZ * c - posX * s;\n\n  gl_Position = projectionMatrix * modelViewMatrix *\n      vec4(pos.x + endX, yOffset, pos.z + endY, 1.0);\n}\n",
fragmentShader:"\nprecision highp float;\n\nvarying vec3 vColor;\n\nvoid main() {\n  gl_FragColor = vec4(vColor, 1.0);\n}\n",transparent:!0})}clear(){this.C&&this.C.removeFromParent();this.D&&this.D.removeFromParent()}};function HM(a,b,c=!1,d=!1,e=!1,f=!1){if(0===b.length)a.F={},a.C=void 0,a.D=void 0;else{a.ua=b;var g=[],k=[],m=[],p=[],t=[],w=[],B=[],G=[],J=[],N=[],aa=0<Object.keys(a.F).length,ca=Object.assign({},a.F);a.F={};var ia=0;for(let O=0;O<b.length;O++){const T=b[O],Q=T.bound;let la;const F=null==(la=ca[T.id])?void 0:la.bound;if(aa&&!d||e){let M,W,Aa,pb,qb,Ia,Vb,Ma;g.push(null!=(qb=null==(M=F)?void 0:M.x)?qb:Q.x,null!=(Ia=null==(W=F)?void 0:W.y)?Ia:Q.y,null!=(Vb=null==(Aa=F)?void 0:Aa.width)?Vb:f?Q.width:
0,null!=(Ma=null==(pb=F)?void 0:pb.height)?Ma:f?Q.height:0)}else g.push(Q.x,Q.y,Q.width,Q.height);m.push(Q.x,Q.y,Q.width,Q.height);k.push(T.Me);p.push(T.Fe?1:0);t.push(T.borderWidth);w.push(T.bgColor.r,T.bgColor.g,T.bgColor.b);B.push(T.borderColor.r,T.borderColor.g,T.borderColor.b);G.push(0);J.push(T.opacity);N.push(T.bB?1:0);a.F[T.id]=T;ia++}a.J=B;a.G=w;a.fa=t;a.ja=J;d=(new THREE.InstancedBufferGeometry).copy(a.sa);d.instanceCount=b.length;d.setAttribute("bound",new THREE.InstancedBufferAttribute(new Float32Array(g),
4));d.setAttribute("targetBound",new THREE.InstancedBufferAttribute(new Float32Array(m),4));d.setAttribute("yOffset",new THREE.InstancedBufferAttribute(new Float32Array(k),1));d.setAttribute("isRounded",new THREE.InstancedBufferAttribute(new Float32Array(p),1));d.setAttribute("borderWidth",new THREE.InstancedBufferAttribute(new Float32Array(t),1));d.setAttribute("bgColor",new THREE.InstancedBufferAttribute(new Float32Array(w),3));d.setAttribute("borderColor",new THREE.InstancedBufferAttribute(new Float32Array(B),
3));d.setAttribute("angle",new THREE.InstancedBufferAttribute(new Float32Array(G),1));d.setAttribute("opacity",new THREE.InstancedBufferAttribute(new Float32Array(J),1));d.setAttribute("changeColorWhenFar",new THREE.InstancedBufferAttribute(new Float32Array(N),1));a.C=new THREE.Mesh(d,a.material);a.C.frustumCulled=!1;if(c){a.D=new THREE.InstancedMesh(new THREE.BoxGeometry(1,1,1),a.wa,b.length);for(t=0;t<b.length;t++)w=b[t].bound,c=a,g=a.D,k=t,m=w.width,p=w.height,c.O.position.set(w.x,-10,w.y),c.O.scale.set(m,
1,p),c.O.updateMatrix(),g.setMatrixAt(k,c.O.matrix),g.instanceMatrix.needsUpdate=!0;a.D.frustumCulled=!1}}}function IM(a,b,c,d=!0){if(a.D){var e=b.intersectObject(a.D);b="";0<e.length&&(e=e[e.length-1].instanceId,null!=e&&(b=a.ua[e].id));a.N!==b&&(a.N=b,d&&(document.body.style.cursor=""===a.N?"default":"pointer"),c(a.N,a.F[a.N]))}}
function JM(a,b,c){if(a.C){b=b.filter(e=>""!==e);var d=a.C.geometry.getAttribute("borderColor");if(0<b.length)for(const e of b)if(b=a.F[e])d.setXYZ(b.index,c.r,c.g,c.b),b.borderColor.r=c.r,b.borderColor.g=c.g,b.borderColor.b=c.b,a.T.push(b);d.needsUpdate=!0}}
function KM(a){if(a.C){var b=a.C.geometry.getAttribute("borderColor");if(0<=a.T.length){for(const c of a.T){const d=c.index;b.setXYZ(d,a.J[3*d],a.J[3*d+1],a.J[3*d+2]);c.borderColor.r=a.J[3*d];c.borderColor.g=a.J[3*d+1];c.borderColor.b=a.J[3*d+2]}b.needsUpdate=!0}a.T=[]}}
function LM(a,b,c,d=!1){if(a.C){b=b.filter(f=>""!==f);var e=a.C.geometry.getAttribute("bgColor");if(0<b.length)for(const f of b){b=a.F[f];if(!b)continue;const g=b.index;if(d){const k=a.G[3*g+1],m=a.G[3*g+2];if(1!==a.G[3*g]||1!==k||1!==m)continue}e.setXYZ(g,c.r,c.g,c.b);b.bgColor.r=c.r;b.bgColor.g=c.g;b.bgColor.b=c.b;a.P.push(b)}e.needsUpdate=!0}}
function MM(a){if(a.C){var b=a.C.geometry.getAttribute("bgColor");if(0<=a.P.length){for(const c of a.P){const d=c.index;b.setXYZ(d,a.G[3*d],a.G[3*d+1],a.G[3*d+2]);c.bgColor.r=a.G[3*d];c.bgColor.g=a.G[3*d+1];c.bgColor.b=a.G[3*d+2]}b.needsUpdate=!0}a.P=[]}}function NM(a,b,c){if(a.C){b=b.filter(e=>""!==e);var d=a.C.geometry.getAttribute("borderWidth");if(0<b.length)for(const e of b)if(b=a.F[e])d.setX(b.index,c),b.borderWidth=c,a.aa.push(b);d.needsUpdate=!0}}
function OM(a){if(a.C){var b=a.C.geometry.getAttribute("borderWidth");if(0<=a.aa.length){for(const c of a.aa){const d=c.index;b.setX(d,a.fa[d]);c.borderWidth=a.fa[d]}b.needsUpdate=!0}a.aa=[]}}function PM(a,b,c){if(a.C){b=b.filter(e=>""!==e);var d=a.C.geometry.getAttribute("opacity");if(0<b.length)for(const e of b)if(b=a.F[e])d.setX(b.index,c),b.opacity=c,a.ca.push(b);d.needsUpdate=!0}}
function QM(a){if(a.C){var b=a.C.geometry.getAttribute("opacity");if(0<=a.ca.length){for(const c of a.ca){const d=c.index;b.setX(d,a.ja[d]);c.opacity=a.ja[d]}b.needsUpdate=!0}a.ca=[]}}function RM(a,b,c){if(a.C){var d=a.C.geometry.getAttribute("angle");if(a=a.F[b])d.setX(a.index,c/180*Math.PI),d.needsUpdate=!0}}function SM(a,b){a.C&&(a.na.value=b)}function TM(a,b,c){a.C&&(a.material.uniforms.bgColorWhenFar.value=[b.r,b.g,b.b,c])}
var UM=class{constructor(a){this.radius=a;this.N="";this.ua=[];this.F={};this.na={value:-1};this.O=new THREE.Object3D;this.J=[];this.G=[];this.fa=[];this.ja=[];this.T=[];this.P=[];this.aa=[];this.ca=[];this.sa=new THREE.PlaneGeometry(1,1);this.sa.rotateX(-Math.PI/2);this.material=new THREE.ShaderMaterial({extensions:{derivatives:!0},uniforms:{borderRadius:{value:this.radius},animationProgress:this.na,bgColorWhenFar:{value:[0,0,0,0]}},vertexShader:'\nprecision highp float;\n\nuniform float borderRadius;\n// Set this to <0 to disable animation.\nuniform float animationProgress;\nuniform vec4 bgColorWhenFar;\n\nattribute vec4 bound;\nattribute vec4 targetBound;\nattribute float yOffset;\nattribute float isRounded;\nattribute float borderWidth;\nattribute vec3 bgColor;\nattribute vec3 borderColor;\nattribute float angle;\nattribute float opacity;\nattribute float changeColorWhenFar;\n\nvarying vec2 vUv;\nvarying vec2 vSize;\nvarying float vIsRounded;\nvarying float vBorderWidth;\nvarying vec3 vBgColor;\nvarying vec3 vBorderColor;\nvarying float vOpacity;\nvarying float vChangeColorWhenFar;\n\nvoid main() {\n  vUv = uv;\n  vIsRounded = isRounded;\n  vBorderWidth = borderWidth;\n  vBgColor = bgColor;\n  vBorderColor = borderColor;\n  vOpacity = opacity;\n  vChangeColorWhenFar = changeColorWhenFar;\n\n  vec3 pos = position;\n  float curX = bound.x;\n  float curY = bound.y;\n  float curW = bound.z;\n  float curH = bound.w;\n\n  float progress = animationProgress * step(0.0, animationProgress); \n  float x = curX + (targetBound.x - curX) * progress;\n  float y = curY + (targetBound.y - curY) * progress;\n  float w = curW + (targetBound.z - curW) * progress;\n  float h = curH + (targetBound.w - curH) * progress;\n  vSize = vec2(w, h);\n\n  // For each vertex, move it by delta calculated below so that the final\n  // rectangle\'s width and height match the width and height stored in "bound".\n  //\n  // pos.x < 0: the first () below returns -1.\n  // pos.x > 0: the first () below returns 1.\n  pos.x = (step(0.0, pos.x) * 2.0 - 1.0) * (w / 2.0);\n  pos.z = (step(0.0, pos.z) * 2.0 - 1.0) * (h / 2.0);\n\n  // Rotate.\n  //\n  float c = cos(angle);\n  float s = sin(angle);\n  float posX = pos.x * c + pos.z * s;\n  float posZ = pos.z * c - pos.x * s;\n\n  gl_Position = projectionMatrix * modelViewMatrix *\n      vec4(posX + x, yOffset, posZ + y, 1.0);\n}\n',
fragmentShader:"\nprecision highp float;\n\nuniform float borderRadius;\nuniform vec4 bgColorWhenFar;\n\nvarying vec2 vUv;\nvarying vec2 vSize;\nvarying float vIsRounded;\nvarying float vBorderWidth;\nvarying vec3 vBgColor;\nvarying vec3 vBorderColor;\nvarying float vOpacity;\nvarying float vChangeColorWhenFar;\n\n// See:\n// https://www.shadertoy.com/view/4tc3DX#\n\n// Clamp [0..1] range\n#define saturate(a) clamp(a, 0.0, 1.0)\n\n// This function will make a signed distance field that says how far you are\n// from the edge of the line at any point U,V.\n// Pass it UVs, line end points, line thickness (x is along the line and y is\n// perpendicular), How rounded the end points should be (0.0 is rectangular,\n// setting rounded to thick.y will be circular).\nfloat LineDistField(vec2 uv, vec2 pA, vec2 pB, vec2 thick, float rounded, float dashOn) {\n  // Don't let it get more round than circular.\n  rounded = min(thick.y, rounded);\n  // midpoint\n  vec2 mid = (pB + pA) * 0.5;\n  // vector from point A to B\n  vec2 delta = pB - pA;\n  // Distance between endpoints\n  float lenD = length(delta);\n  // unit vector pointing in the line's direction\n  vec2 unit = delta / lenD;\n  // Check for when line endpoints are the same\n  if (lenD < 0.0001) unit = vec2(1.0, 0.0);\t// if pA and pB are same\n  // Perpendicular vector to unit - also length 1.0\n  vec2 perp = unit.yx * vec2(-1.0, 1.0);\n  // position along line from midpoint\n  float dpx = dot(unit, uv - mid);\n  // distance away from line at a right angle\n  float dpy = dot(perp, uv - mid);\n  // Make a distance function that is 0 at the transition from black to white\n  float disty = abs(dpy) - thick.y + rounded;\n  float distx = abs(dpx) - lenD * 0.5 - thick.x + rounded;\n\n  // Too tired to remember what this does. Something like rounded endpoints for distance function.\n  float dist = length(vec2(max(0.0, distx), max(0.0,disty))) - rounded;\n  dist = min(dist, max(distx, disty));\n\n  return dist;\n}\n\n// This makes a line in UV units. A 1.0 thick line will span a whole 0..1 in\n// UV space.\nfloat FillLine(vec2 uv, vec2 pA, vec2 pB, vec2 thick, float rounded) {\n  float df = LineDistField(uv, pA, pB, vec2(thick), rounded, 0.0);\n  return saturate(df / abs(dFdy(uv).y));\n}\n\nvoid main() {\n  // Normalize uv.\n  vec2 uv = vUv;\n  uv -= 0.5;\n  float aspect = vSize.x / vSize.y;\n  uv.x *= aspect;\n\n  vec4 finalColor = vec4(1.0);\n\n  float radius = mix(1.0 / vSize.y, borderRadius / vSize.y, step(0.5, vIsRounded));\n  float borderWidth = vBorderWidth / vSize.y;\n\n  // Border.\n  float c = FillLine(uv,\n    vec2(-0.5 * aspect, 0.0), vec2(0.5 * aspect, 0.0), vec2(0.0, 0.5), radius);\n  finalColor = mix(vec4(vBorderColor.rgb, 1.0), vec4(1.0, 1.0, 1.0, 0.0), c);\n\n  // Body.\n  float c2 = FillLine(uv,\n    vec2(-0.5 * aspect + borderWidth, 0.0),\n    vec2(0.5 * aspect - borderWidth, 0.0), vec2(0.0, 0.5 - borderWidth),\n    radius - 1.0 / vSize.y);\n  finalColor = mix(vec4(vBgColor.rgb, 1.0), finalColor, c2);\n  finalColor = saturate(finalColor);\n  finalColor.rgb = mix(finalColor.rgb, bgColorWhenFar.rgb, bgColorWhenFar.a * vChangeColorWhenFar);\n\n  float alpha = finalColor.w * vOpacity;\n  if (alpha < 0.00001) {\n    discard;\n  }\n  gl_FragColor = vec4(finalColor.rgb, alpha);\n}\n",
transparent:!0});this.wa=new THREE.MeshBasicMaterial({color:"",transparent:!0})}};function VM(a,b,c,d,e,f,g,k){let m=`${b}__${c}__${g}`,p=a.ca[m];var t=void 0;if(null==p){p=WM(a,b,c,f,g,k);if(null!=e){t=d/a.G;for(d=0;d<p.Gj.length;d++)if(g=p.Gj[d],(g.x+g.width)*t>e){b=b.substring(0,d-1);b+="...";break}t=b;p=WM(a,b,c,f);m=`${b}__${c}__${void 0}`}a.ca[m]=p}return{sizes:a.ca[m],ME:t}}
function XM(a,b,c=!1,d=!1,e=!1){let f=0;a.O={};a.N={};const g=[],k=[],m=[],p=[],t=[],w=[],B=[],G=[],J=[],N=0<Object.keys(a.J).length,aa=Object.assign({},a.J);a.J={};for(const F of b){b=xM(a.F,F.weight);const M=yM(a.F,F.weight).common.scaleW,W=VM(a,F.label,F.weight,F.height,F.maxWidth,F.im,F.angle,F.zq).sizes,Aa=F.height/a.G;let pb=F.x,qb=F.z;var ca=(F.im?{Gj:[{x:0,y:0,width:20,height:F.height}]}:VM(a,"a",F.weight,F.height).sizes).Gj[0].height*Aa;switch(F.vAlign){case "top":qb-=W.ao*Aa;break;case "bottom":qb-=
W.Zn*Aa;break;case "center":qb-=(W.ao+W.Zn)/2*Aa+ca/2}switch(F.cf){case "left":pb-=W.Sh*Aa;break;case "right":pb-=W.Ph*Aa;break;case "center":pb-=(W.Sh+W.Ph)/2*Aa}ca=F.y;const Ia=w.length;for(let Vb=0;Vb<W.Gj.length;Vb++){var ia=W.Gj[Vb],O=F.im?F.label:F.label[Vb],T=b[O]||b["?"];O=`${F.id}_${O}_${Vb}`;var Q=ia.width*Aa,la=ia.height*Aa;const Ma=pb+ia.x*Aa;ia=qb+ia.y*Aa;ia={x:F.zq?Ma:Ma+Q/2,y:F.zq?ia:ia+la/2,width:Q,height:la};Q=aa[O];if(N&&!c||d){let xb,Tc,ac,Xb,Jc,Hf,hd,Ac;m.push(null!=(Jc=null==
(xb=Q)?void 0:xb.x)?Jc:ia.x,null!=(Hf=null==(Tc=Q)?void 0:Tc.y)?Hf:ia.y,null!=(hd=null==(ac=Q)?void 0:ac.width)?hd:e?ia.width:0,null!=(Ac=null==(Xb=Q)?void 0:Xb.height)?Ac:e?ia.height:0)}else m.push(ia.x,ia.y,ia.width,ia.height);p.push(ia.x,ia.y,ia.width,ia.height);t.push(ca);B.push(T.x/M,1-T.y/M-T.height/M,T.width/M,T.height/M);let qa,U;T=null!=(U=null==(qa=F.color)?void 0:qa.r)?U:0;let ea,na;Q=null!=(na=null==(ea=F.color)?void 0:ea.g)?na:0;let Ja,cb;la=null!=(cb=null==(Ja=F.color)?void 0:Ja.b)?
cb:0;k.push(T,Q,la);g.push(F.weight);w.push(1);G.push(F.angle||0);let Pa,ub,pa,Ha,Ua,kb;J.push(null!=(Ha=null==(Pa=F.borderColor)?void 0:Pa.r)?Ha:-1,null!=(Ua=null==(ub=F.borderColor)?void 0:ub.g)?Ua:-1,null!=(kb=null==(pa=F.borderColor)?void 0:pa.b)?kb:-1);a.J[O]=ia;f++}F.nodeId&&(a.O[F.nodeId]||(a.O[F.nodeId]=[]),a.O[F.nodeId].push({Rh:Ia,Oh:w.length-1}),a.N[F.nodeId]||(a.N[F.nodeId]=[]),a.N[F.nodeId].push({Rh:Ia,Oh:w.length-1}))}a.P=k;c=(new THREE.InstancedBufferGeometry).copy(a.fa);c.instanceCount=
t.length;c.setAttribute("bound",new THREE.InstancedBufferAttribute(new Float32Array(m),4));c.setAttribute("targetBound",new THREE.InstancedBufferAttribute(new Float32Array(p),4));c.setAttribute("uvBound",new THREE.InstancedBufferAttribute(new Float32Array(B),4));c.setAttribute("yOffset",new THREE.InstancedBufferAttribute(new Float32Array(t),1));c.setAttribute("weight",new THREE.InstancedBufferAttribute(new Float32Array(g),1));c.setAttribute("color",new THREE.InstancedBufferAttribute(new Float32Array(k),
3));c.setAttribute("opacity",new THREE.InstancedBufferAttribute(new Float32Array(w),1));c.setAttribute("angle",new THREE.InstancedBufferAttribute(new Float32Array(G),1));c.setAttribute("borderColor",new THREE.InstancedBufferAttribute(new Float32Array(J),3));a.C=new THREE.Mesh(c,a.material);a.C.frustumCulled=!1}
function YM(a,b){if(a.C){var c=a.C.geometry.getAttribute("opacity");if(0<b.length)for(const d of b)if(a.O[d])for(const {Rh:e,Oh:f}of a.O[d]){for(b=e;b<=f;b++)c.setX(b,.3);a.aa.push({Rh:e,Oh:f})}c.needsUpdate=!0}}function ZM(a,b,c){if(a.C){var d=a.C.geometry.getAttribute("color");if(0<b.length)for(const e of b)if(a.N[e])for(const {Rh:f,Oh:g}of a.N[e]){for(b=f;b<=g;b++)d.setXYZ(b,c.r,c.g,c.b);a.T.push({Rh:f,Oh:g})}d.needsUpdate=!0}}
function $M(a){if(a.C){var b=a.C.geometry.getAttribute("opacity");if(0<=a.aa.length){for(const {Rh:c,Oh:d}of a.aa)for(let e=c;e<=d;e++)b.setX(e,1);b.needsUpdate=!0}a.aa=[]}}function aN(a){if(a.C){var b=a.C.geometry.getAttribute("color");if(0<=a.T.length){for(const {Rh:c,Oh:d}of a.T)for(let e=c;e<=d;e++)b.setXYZ(e,a.P[3*e],a.P[3*e+1],a.P[3*e+2]);b.needsUpdate=!0}a.T=[]}}
function WM(a,b,c,d,e,f){c=xM(a.F,c);const g=[];let k=0,m=Number.MAX_VALUE,p=Number.NEGATIVE_INFINITY;for(const t of d?[b]:b)(b=c[t])||(b=c["?"]),d=b.yoffset,f?(k+=Math.sin(e||0)*(b.yoffset+b.height/2),d=Math.cos(e||0)*(b.yoffset+b.height/2)):k+=b.xoffset,d={x:k,y:d,width:b.width,height:b.height},g.push(d),m=Math.min(m,d.x),p=Math.max(p,d.x+d.width),k+=.98*b.xadvance;return{Gj:g,Sh:m,ao:0,Ph:p,Zn:a.G}}
var bN=class{constructor(a){this.F=a;this.ca={};this.J={};this.O={};this.N={};this.aa=[];this.T=[];this.P=[];this.D={value:-1};this.fa=new THREE.PlaneGeometry(1,1);this.fa.rotateX(-Math.PI/2);this.G=this.F.C.info.size;this.distanceRange=this.F.C.distanceField.distanceRange;this.material=new THREE.ShaderMaterial({uniforms:{textureRegular:{value:this.F.ja},textureMedium:{value:this.F.fa},textureBold:{value:this.F.aa},textureIcons:{value:this.F.ca},distanceRange:{value:this.distanceRange},animationProgress:this.D},
extensions:{derivatives:!0},vertexShader:'\nprecision highp float;\n\n// Set this to <0 to disable animation.\nuniform float animationProgress;\n\n// 0: regular. 1: medium. 2: bold.\nattribute float weight;\nattribute vec3 color;\nattribute vec4 bound;\nattribute vec4 targetBound;\nattribute vec4 uvBound;\nattribute float yOffset;\nattribute float opacity;\nattribute float angle;\nattribute vec3 borderColor;\n\nvarying vec2 vUv;\nvarying vec3 vColor;\nvarying float vWeight;\nvarying float vOpacity;\nvarying vec3 vBorderColor;\n\nvoid main() {\n  vUv = vec2(0.0, 0.0);\n  vec3 pos = position;\n  if (pos.x < 0.0 && pos.z < 0.0) {\n    vUv = vec2(uvBound.x, uvBound.y + uvBound.w);\n  } else if (pos.x > 0.0 && pos.z < 0.0) {\n    vUv = vec2(uvBound.x + uvBound.z, uvBound.y + uvBound.w);\n  } else if (pos.x > 0.0 && pos.z > 0.0) {\n    vUv = vec2(uvBound.x + uvBound.z, uvBound.y);\n  } else {\n    vUv = vec2(uvBound.x, uvBound.y);\n  }\n\n  vColor = color;\n  vWeight = weight;\n  vOpacity = opacity;\n  vBorderColor = borderColor;\n\n  float curX = bound.x;\n  float curY = bound.y;\n  float curW = bound.z;\n  float curH = bound.w;\n\n  float x = curX;\n  float y = curY;\n  float w = curW;\n  float h = curH;\n\n  if (animationProgress >= 0.0) {\n    x = curX + (targetBound.x - curX) * animationProgress;\n    y = curY + (targetBound.y - curY) * animationProgress;\n    w = curW + (targetBound.z - curW) * animationProgress;\n    h = curH + (targetBound.w - curH) * animationProgress;\n  }\n\n  // For each vertex, move it by delta calculated below so that the final\n  // rectangle\'s width and height match the width and height stored in "bound".\n  //\n  if (pos.x < 0.0) {\n    pos.x = - w / 2.0;\n  } else if (pos.x > 0.0) {\n    pos.x = w / 2.0;\n  }\n\n  if (pos.z < 0.0) {\n    pos.z = - h / 2.0;\n  } else if (pos.z > 0.0) {\n    pos.z = h / 2.0;\n  }\n\n  // Rotate.\n  //\n  float c = cos(angle);\n  float s = sin(angle);\n  float posX = pos.x * c + pos.z * s;\n  float posZ = pos.z * c - pos.x * s;\n\n  gl_Position = projectionMatrix * modelViewMatrix *\n      vec4(posX + x, yOffset, posZ + y, 1.0);\n}\n',
fragmentShader:"\nprecision highp float;\n\nuniform sampler2D textureRegular;\nuniform sampler2D textureMedium;\nuniform sampler2D textureBold;\nuniform sampler2D textureIcons;\nuniform float distanceRange;\nvarying vec2 vUv;\nvarying float vWeight;\nvarying vec3 vColor;\nvarying float vOpacity;\nvarying vec3 vBorderColor;\n\nvec3 getSampleColor() {\n  if (vWeight < 0.5) {\n    return texture2D(textureRegular, vUv).rgb;\n  }\n  if (vWeight < 1.5) {\n    return texture2D(textureMedium, vUv).rgb;\n  }\n  if (vWeight < 2.5) {\n    return texture2D(textureBold, vUv).rgb;\n  }\n  return texture2D(textureIcons, vUv).rgb;\n}\n\nfloat median(float r, float g, float b) {\n  return max(min(r, g), min(max(r, g), b));\n}\n\nfloat screenPxRange() {\n  vec2 unitRange = vec2(distanceRange) / vec2(textureSize(textureRegular, 0));\n  vec2 screenTexSize = vec2(1.0) / fwidth(vUv);\n  return max(0.5 * dot(unitRange, screenTexSize), 1.0);\n}\n\nvoid main() {\n  vec3 sampleColor = getSampleColor();\n\n  float dist = median(sampleColor.r, sampleColor.g, sampleColor.b);\n  vec3 color = vColor;\n  float threshold = step(0.0, vBorderColor.r) * 0.35;\n  float pxDist = screenPxRange() * (dist - 0.5 + threshold) + 0.5 - threshold;\n\n  // Has border.\n  if (vBorderColor.r >= 0.0) {\n    float blur = fwidth(dist) / 2.0;\n    color = mix(vBorderColor, vColor, smoothstep(0.5 - blur, 0.5 + blur, dist));\n  }\n\n  float opacity = clamp(pxDist, 0.0, 1.0);\n  if (opacity > 0.001) {\n    opacity *= vOpacity;\n  }\n  gl_FragColor = vec4(color, opacity);\n}\n",
transparent:!0,alphaToCoverage:!0})}};const cN=["container"],dN=["canvas"],eN=["snapshotCanvas"],fN=["pngDownloaderCanvas"],gN=["hoverToolbar"],hN=["ioPicker"],iN=["subgraphMenuTrigger"],jN=["groupNodeIconMatTooltip"],kN=["ioPickerMatTooltip"],lN=["moreActionsMenuTrigger"];function mN(a){if(a&1){const b=Ug();R(0,"div",30);V("click",function(){u(b);const c=Y();var d=c.rl;if(d=null!=d?d:c.Be)$L(c,d),fM(c,c.ga.nodesById[d],!0,!1);return x()});R(1,"mat-icon");Z(2,"unfold_less_double");S();Z(3," Collapse layer & all sub-layers ");S()}}
function nN(a){if(a&1){const b=Ug();R(0,"div",31);V("click",function(){u(b);const c=Y();var d=c.rl;d=c.ga.nodesById[null!=d?d:c.Be];var e=cM(d);const f=d.width||0;if(d.expanded){const g=VM(c.Dd,sM(d),2,11).sizes;e=oN(c,e+f/2+11/c.Dd.G*(g.Ph-g.Sh)/2+22,0).x}else e=oN(c,e+f+1,0).x;d={eventType:9,modelGraphId:c.ga.id,paneId:c.paneId,rendererId:tI(),groupNodeId:d.id,initialPosition:{x:e,y:oN(c,0,dM(d)).y}};c.si.C.postMessage(d);return x()});R(1,"mat-icon");Z(2,"open_in_new");S();Z(3," Open layer in popup ");
S()}}function pN(a,b){if(a&1){const c=Ug();R(0,"div",32);V("click",function(){const d=u(c).da,e=Y();qN(e,d);return x()});R(1,"mat-icon");Z(2,"subdirectory_arrow_right");S();Z(3);S()}a&2&&(a=b.da,D(3),Br(" ",a," "))}function rN(a){a&1&&Mq(0,"div",33);a&2&&(a=Y(),lq("top",a.GD,"px")("left",a.FD,"px")("width",a.HD,"px")("height",a.ED,"px"))}function sN(a){a&1&&(R(0,"div",29),Z(1),S());a&2&&(a=Y(),D(),Br("FPS: ",a.Tv,""))}
function tN(a){if(a.ga)if(uN(a),gM(a)){var b=mM(a);if(0<b.Ml.length){var c=b.Ml.map(f=>({edge:Object.assign({},f,{curvePoints:vI(f.points)}),index:95E3}));AM(a.wa,c,a.ga);a.wa.C&&a.scene.add(a.wa.C);a.wa.D&&a.scene.add(a.wa.D);var d;if(null==(d=a.ja["Tensor shape"])?0:d.selected)c=vN(a,c,a.Yy),XM(a.Ob,c,!1,!0,!0),a.Ob.C&&a.scene.add(a.Ob.C)}a.Ee=b.Ee;a.gl=b.cs;b=nM(a);if(0<b.Ml.length){c=b.Ml.map(f=>({edge:Object.assign({},f,{curvePoints:vI(f.points)}),index:95E3}));AM(a.Ea,c,a.ga);a.Ea.C&&a.scene.add(a.Ea.C);
a.Ea.D&&a.scene.add(a.Ea.D);var e;if(null==(e=a.ja["Tensor shape"])?0:e.selected)e=vN(a,c,a.Zy),XM(a.Dc,e,!1,!0,!0),a.Dc.C&&a.scene.add(a.Dc.C)}a.kf=b.kf;a.wl=b.cs;e=[];b=[];for(const f of Object.keys(Object.assign({},a.Ee,a.kf)))if(c=a.ga.nodesById[f],oI(c)){const g=(d=null!=a.Ee[f])?a.Ee[f].length:a.kf[f].length;e.push({id:`${f}${"||||"}${d?"input":"output"}`,index:e.length,bound:{x:cM(c)+20,y:dM(c)-3.5,width:40,height:14},Me:95,Fe:!0,borderColor:{r:1,g:1,b:1},bgColor:d?a.md:a.sd,borderWidth:0,
opacity:1});b.push({id:`${f}${"||||"}${d?"input":"output"}`,nodeId:f,label:`${g} ${d?"input":"output"}${1!==g?"s":""}`,height:8,cf:"center",vAlign:"center",weight:1,color:{r:1,g:1,b:1},x:cM(c)+20,y:96,z:dM(c)-3.5+1})}XM(a.ac,b,!1,!0,!0);a.ac.C&&a.scene.add(a.ac.C);HM(a.O,e,!0,!1,!1,!0);a.O.C&&a.scene.add(a.O.C);a.O.D&&a.scene.add(a.O.D);wN(a,f=>{var g=a.wa;g.C&&(g.J.value=f);g=a.Ea;g.C&&(g.J.value=f);a.Ob.D.value=f;a.Dc.D.value=f;SM(a.O,f);a.ac.D.value=f})}else FM(a.wa),FM(a.Ea),a.Ob.J={},a.Dc.J=
{},a.O.F={},a.ac.J={}}
function xN(a){if(a.ga){yN(a);var b=a.ga.nodesById[a.selectedNodeId],c=[],d=[];if(oI(b)&&null!=(null==b?void 0:b.identicalGroupIndex)){const f=b.identicalGroupIndex;var e=a.N.filter(({node:g})=>oI(g)&&g.identicalGroupIndex===f).map(({node:g})=>a.ga.nodesById[g.id]);for(const g of e){if(g.id===b.id)continue;e=cM(g)+34;const k=dM(g)-7+3.5;c.push({id:g.id,index:c.length,bound:{x:e,y:k,width:68,height:14},Me:95,Fe:!0,borderColor:a.hz,bgColor:a.gz,borderWidth:1,opacity:1});d.push({id:g.id,label:"Identical layer",
height:8,cf:"center",vAlign:"center",weight:1,color:{r:0,g:0,b:0},x:e,y:96,z:k})}}HM(a.Qd,c,!1,!1,!0,!0);a.Qd.C&&a.scene.add(a.Qd.C);XM(a.Rd,d,!1,!0,!0);a.Rd.C&&a.scene.add(a.Rd.C);wN(a,f=>{SM(a.Qd,f);a.Rd.D.value=f})}}
function zN(a){var b=!1;a.selectedNodeId!==a.Gl&&(a.Gl=a.selectedNodeId,b=!0);var c=!1;a.ua!==a.Qy&&(a.Qy=a.ua,c=!0);KM(a.D);MM(a.D);OM(a.D);QM(a.D);QM(a.na);$M(a.Dd);aN(a.Dd);$M(a.Sa);$M(a.Nb);$M(a.fa);(b||c)&&DM(a.edges);b&&EM(a.edges);var d=a.ga.nodesById[a.selectedNodeId];if(null!=d&&oI(d)){const p=d.identicalGroupIndex;if(null!=p){var e=a.N.filter(({node:t})=>oI(t)&&t.identicalGroupIndex===p).map(({node:t})=>t.id);LM(a.D,e,a.fz)}}e=a.ga.nodesById[a.Be];let f=oI(e)?a.cz:a.dz,g;nI(e)&&(null==(g=
e.style)?0:g.hoveredBorderColor)&&(f=new THREE.Color(e.style.hoveredBorderColor));JM(a.D,[a.Be],f);a.selectedNodeId&&null!=d&&(JM(a.D,[a.selectedNodeId],a.sz),NM(a.D,[a.selectedNodeId],2),LM(a.D,[a.selectedNodeId],a.rz,nI(d)));PM(a.na,[a.pl],.07);d=Object.keys(a.Ee);if(0<d.length){JM(a.D,d,new THREE.Color(a.md.r,a.md.g,a.md.b));for(var k of d)NM(a.D,[k],1.5)}k=Object.keys(a.kf);if(0<k.length){JM(a.D,k,new THREE.Color(a.sd.r,a.sd.g,a.sd.b));for(const p of k)NM(a.D,[p],1.5)}(b||c)&&CM(a.edges,[...a.gl,
...a.wl].map(p=>p.id));b=a.gh()||{};for(var m of Object.keys(b))a.aa(m)&&nI(a.ga.nodesById[m])&&((c=b[m].bgColor)&&"transparent"!==c&&LM(a.D,[m],new THREE.Color(c)),(c=b[m].textColor)&&ZM(a.Dd,[m],new THREE.Color(c)));null!=a.ua&&(m=Object.keys(a.ga.nodesById).filter(p=>!a.ua.Os.has(p)&&a.aa(p)),PM(a.D,m,.2),YM(a.Dd,m),YM(a.Nb,m),YM(a.fa,m),YM(a.Sa,m),m=a.Lb.filter(({edge:p})=>!a.ua.Os.has(p.fromNodeId)||!a.ua.Os.has(p.toNodeId)).map(({edge:p})=>p.id),BM(a.edges,m))}
function AN(a){sa(function*(){BN(a);const b=yield CN(a);HJ(a.C,b,a.ga.id,a.paneId)})}
function DN(a,b){sa(function*(){var c=a.ab.R.offsetWidth/EN();let d=a.ab.R.offsetHeight/EN();var e=a.camera;if(b){var f=a.ud+20;e=a.sb-20;const g=a.Pd+20,k=a.Kb-20;c=f-e;d=g-k;const m=5E3/EN();c>m&&(d=d*m/c,c=m);d>m&&(c=c*m/d,d=m);f=new THREE.OrthographicCamera(e,f,-k,-g,.001,1E3);f.position.y=200;f.lookAt(new THREE.Vector3(0,0,0));f.updateMatrixWorld();f.updateProjectionMatrix();e=f}f=a.xD.R;a.bb||(a.bb=new THREE.WebGLRenderer({canvas:f,powerPreference:"high-performance",precision:"highp",antialias:!0,
alpha:!0,preserveDrawingBuffer:!0}),a.bb.setPixelRatio(EN()));a.bb.setSize(c,d,!1);TM(a.D,a.td,0);a.bb.render(a.scene,e);TM(a.D,a.td,a.zl/3);c=document.createElement("a");c.download="model_explorer_graph.png";ud(c,rd(f.toDataURL()));c.click()})}
function FN(a){if(a.gb){GN(a);var b=new Set;for(var c of Object.keys(a.gb.results)){var d=a.ga.nodesById[c];if(a.aa(c))b.add(c);else{for(;d&&(d=a.ga.nodesById[d.nsParentId||""],d&&!a.aa(d.id)););b.add(d.id)}}c=[];for(var e of b){b=a.ga.nodesById[e];d=(b.width||0)+4;var f=(b.height||0)+4;c.push({id:e,index:c.length,bound:{x:cM(b)-2+d/2,y:dM(b)-2+f/2,width:d,height:f},Me:.001*a.sa[e].index+-3E-4,Fe:!0,borderColor:{r:1,g:1,b:1},bgColor:a.Bx,borderWidth:0,opacity:1})}HM(a.Pa,c);a.Pa.C&&a.scene.add(a.Pa.C);
e=[];b=11/a.Dd.G;for(const g of Object.keys(a.gb.results))if(a.aa(g)){d=a.gb.results[g];for(const k of d)if("title"===k.type){d=a.ga.nodesById[g];f=a.sa[g].index;const m=VM(a.Dd,d.label,oI(d)?2:1,11).sizes;e.push({id:g,index:c.length,bound:{x:cM(d)+(d.width||0)/2,y:dM(d)+14-2*b,width:(m.Ph-m.Sh)*b+4,height:(m.Zn-m.ao)*b+4},Me:.001*f+3E-4,Fe:!0,borderColor:{r:1,g:1,b:1},bgColor:a.Bx,borderWidth:0,opacity:1})}}HM(a.Jc,e);a.Jc.C&&a.scene.add(a.Jc.C);wN(a,g=>{SM(a.Pa,g);SM(a.Jc,g)})}}
function HN(a){var b=[a.Pa.C,a.Jc.C];for(var c=a.scene.children.length-1;0<=c;c--){var d=a.scene.children[c];b.includes(d)||(d.geometry&&d.geometry.dispose(),a.scene.remove(d))}a.Gl="";a.Qy=void 0;0<a.Lb.length&&(AM(a.edges,a.Lb,a.ga),a.edges.C&&a.scene.add(a.edges.C),a.edges.D&&a.scene.add(a.edges.D));b=[];for(const {node:T,index:Q}of a.N){c=a.td;for(var e of a.jk)if(NI(T,e.queries,a.ga)){var f=void 0;d=(null==(f=e.styles.node_text_color)?void 0:f.value)||"";""!==d&&(c=new THREE.Color(d));break}b.push({id:`${T.id}_label`,
nodeId:T.id,label:sM(T),height:11,cf:"center",vAlign:"center",weight:nI(T)?1:2,x:cM(T)+(T.width||0)/2,y:.001*Q+4E-4,z:dM(T)+14,color:c})}XM(a.Dd,b);a.Dd.C&&a.scene.add(a.Dd.C);var g;if(null==(g=a.ja["Tensor shape"])?0:g.selected)g=vN(a,a.Lb,a.td),XM(a.Sa,g),a.Sa.C&&a.scene.add(a.Sa.C);IN(a);a.sb=Number.POSITIVE_INFINITY;a.Kb=Number.POSITIVE_INFINITY;a.ud=Number.NEGATIVE_INFINITY;a.Pd=Number.NEGATIVE_INFINITY;g=a.N.length;e=[];f=[];b=[];c=[];d=[];const k=11/a.Dd.G;for(let T=0;T<g;T++){const Q=a.N[T].node,
la=a.N[T].index,F=cM(Q),M=dM(Q),W=Q.width||0,Aa=Q.height||0;var m=oI(Q),p=m?JN(a,Q):{r:1,g:1,b:1},t=KN(m?a.az:a.qz);nI(Q)&&Q.style&&(Q.style.backgroundColor&&(p=new THREE.Color(Q.style.backgroundColor)),Q.style.borderColor&&(t=new THREE.Color(Q.style.borderColor)));m=a.bz;for(var w of a.jk)if(NI(Q,w.queries,a.ga)){var B=void 0,G=(null==(B=w.styles.node_bg_color)?void 0:B.value)||"";""!==G&&(p=new THREE.Color(G));var J=void 0;G=(null==(J=w.styles.node_border_color)?void 0:J.value)||"";""!==G&&(t=new THREE.Color(G));
var N=void 0;G=(null==(N=w.styles.node_text_color)?void 0:N.value)||"";""!==G&&(m=new THREE.Color(G));break}G=!0;oI(Q)&&Q.sectionContainer&&(G=!1);e.push({id:Q.id,index:T,bound:{x:F+W/2,y:M+Aa/2,width:W,height:Aa},Me:.001*la,Fe:G,borderColor:t,bgColor:p,borderWidth:1.2,opacity:1,bB:(nI(Q)||!Q.expanded)&&1===p.r&&1===p.g&&1===p.b});nI(Q)&&Q.subgraphIds&&(c.push({id:`${Q.id}`,index:c.length,bound:{x:cM(Q)+(Q.width||0)+9,y:dM(Q)+7,width:14,height:14},Me:.001*a.sa[Q.id].index,Fe:!0,borderColor:a.ky,bgColor:a.uz,
borderWidth:1,opacity:1}),d.push({id:Q.id,nodeId:Q.id,label:"0xe5da",height:28,cf:"center",vAlign:"center",weight:4,color:a.ky,x:cM(Q)+(Q.width||0)+10,y:.001*a.sa[Q.id].index+4E-4,z:dM(Q)+23,im:!0}));oI(Q)&&(G=VM(a.Dd,sM(Q),2,11).sizes,t=(G.Ph-G.Sh)*k,p=F+W/2-t/2,t=F+W/2+t/2,G=M+14+(G.Zn-G.ao)*k+7.5,p=Q.expanded?p-13:(F+p+1)/2+1,t=Q.expanded?t+12:(F+W+t-1)/2-1,f.push({id:Q.id,nodeId:Q.id,label:Q.expanded?"0xe5d6":"0xe5d7",height:32,cf:"center",vAlign:"center",weight:4,color:m,x:p,y:.001*la+4E-4,z:G,
im:!0}),f.push({id:Q.id,nodeId:Q.id,label:"0xe5d4",height:32,cf:"center",vAlign:"center",weight:4,color:m,x:t,y:.001*la+4E-4,z:G,im:!0}),m=M+14-1,b.push({id:`${Q.id}_${"left"}`,nodeId:Q.id,index:b.length,bound:{x:p,y:m,width:16,height:16},Me:.001*la+3E-4,Fe:!0,borderColor:{r:1,g:1,b:1},bgColor:{r:0,g:0,b:0},borderWidth:0,opacity:0}),b.push({id:`${Q.id}_${"right"}`,nodeId:Q.id,index:b.length,bound:{x:t,y:m,width:16,height:16},Me:.001*la+3E-4,Fe:!0,borderColor:{r:1,g:1,b:1},bgColor:{r:0,g:0,b:0},borderWidth:0,
opacity:0}));a.sb=Math.min(a.sb,F);a.ud=Math.max(a.ud,F+W);a.Kb=Math.min(a.Kb,M);a.Pd=Math.max(a.Pd,M+Aa)}HM(a.D,e,!0);a.D.C&&a.scene.add(a.D.C);a.D.D&&a.scene.add(a.D.D);XM(a.Nb,f);a.Nb.C&&a.scene.add(a.Nb.C);HM(a.na,b,!0);a.na.C&&a.scene.add(a.na.C);a.na.D&&a.scene.add(a.na.D);HM(a.Vd,c,!0);a.Vd.C&&a.scene.add(a.Vd.C);XM(a.zi,d);a.zi.C&&a.scene.add(a.zi.C);w=a.gh()||{};if(0!==Object.keys(w).length){var {XB:aa,pE:ca}=pM(a);w=[];for(const {node:T,index:Q}of a.N)if(aa[T.id]){B=T;J=B.width||0;N=aa[T.id];
g=0;for(var ia of Object.values(N))g+=ia;f=e=0;for(var O of ca)null!=N[O]&&"transparent"!==O&&(b=N[O]/g*J,w.push({id:`${T.id}_${f}`,index:w.length,bound:{x:cM(B)+e+b/2,y:dM(B)+(B.height||0)-5+2.5,width:b,height:5},Me:.001*Q+5E-4,Fe:!1,borderColor:{r:1,g:1,b:1},bgColor:new THREE.Color(O),borderWidth:0,opacity:1}),e+=b,f++)}HM(a.wi,w);a.wi.C&&a.scene.add(a.wi.C)}ia=[];for(const T of a.ga.artificialGroupNodeIds||[])a.aa(T)&&(O=a.ga.nodesById[T],w=(O.width||0)+2,B=(O.height||0)+2,ia.push({id:T,index:ia.length,
bound:{x:cM(O)-1+w/2,y:dM(O)-1+B/2,width:w,height:B},Me:.001*a.sa[T].index+-5E-4,Fe:!1,borderColor:{r:1,g:1,b:1},bgColor:a.VE,borderWidth:0,opacity:1}));HM(a.eh,ia,!1,!1,!0);a.eh.C&&a.scene.add(a.eh.C);FN(a);LN(a);wN(a)}
function MN(a){const b=a.canvas.R;a.scene=new THREE.Scene;a.scene.background=new THREE.Color(16777215);a.camera=new THREE.OrthographicCamera(0,b.clientWidth/b.clientHeight*1E3,0,-1E3,.001,1E3);a.camera.position.y=200;a.camera.lookAt(new THREE.Vector3(0,0,0));a.camera.updateMatrixWorld();a.camera.updateProjectionMatrix();a.ca=new THREE.WebGLRenderer({canvas:b,powerPreference:"high-performance",precision:"highp",antialias:!0,alpha:!0});const c=EN();a.ca.setPixelRatio(c);a.ca.setSize(b.clientWidth,b.clientHeight);
a.Xc=new THREE.WebGLRenderer({canvas:a.Ux.R,powerPreference:"high-performance",precision:"highp",antialias:!0,alpha:!0,preserveDrawingBuffer:!0});a.Xc.setPixelRatio(c);a.render();(new ResizeObserver(()=>{requestAnimationFrame(()=>{NN(a)})})).observe(a.ab.R);a.Ic=new THREE.Raycaster;a.Ic.params.Points.threshold=5.5}
function ON(a,b){if(!(0<a.nf)){var c=a.canvas.R;a.sl.x=b.offsetX/c.offsetWidth*2-1;a.sl.y=2*-(b.offsetY/c.offsetHeight)+1;a.Ic.setFromCamera(a.sl,a.camera);IM(a.D,a.Ic,d=>{a.Be=d;zN(a);a.render()});IM(a.na,a.Ic,(d,e)=>{a.pl=d;a.fo=(null==e?void 0:e.nodeId)||"";zN(a);PN(a,e);a.render()},!1);IM(a.O,a.Ic,(d,e)=>{a.ql=d;a.Sn=-1E3;a.Rn=-1E3;a.cr="";d=a.ql.endsWith("input");if(""!==a.ql){const {x:f,y:g}=oN(a,e.bound.x-e.bound.width/2,e.bound.y-e.bound.height/2),{x:k,y:m}=oN(a,e.bound.x+e.bound.width/2,
e.bound.y+e.bound.height/2);a.Sn=g;a.Rn=f;a.dr=k-f;a.br=m-g;a.cr=`Click to reveal ${d?"input":"output"} node(s)`}cn(a.P)});IM(a.Vd,a.Ic,(d,e)=>{a.On=d;a.Go=-1E3;a.Fo=-1E3;if(""!==a.On){const {x:f,y:g}=oN(a,e.bound.x-e.bound.width/2,e.bound.y-e.bound.height/2),{x:k,y:m}=oN(a,e.bound.x+e.bound.width/2,e.bound.y+e.bound.height/2);a.Go=g;a.Fo=f;a.gy=k-f;a.As=m-g}cn(a.P)})}}function QN(a){const b=()=>{a.render(!0);requestAnimationFrame(b)};b()}
function RN(a,b,c){const d=new THREE.Vector3;d.set(b/a.canvas.R.offsetWidth*2-1,2*-(c/a.canvas.R.offsetHeight)+1,-1);a=d.unproject(a.camera);return{x:a.x,y:a.z}}
function SN(a,b){if(a.ml&&(b.metaKey||b.ctrlKey)||!a.ml&&b.ctrlKey){b.preventDefault();b.stopPropagation();a.nf=b.offsetX;a.Ja=b.offsetY;a.Gc=a.nf;a.Hc=a.Ja;b=lk(document,"mousemove");const c=lk(window,"mouseup");let d=0,e=0;C(Pj([b]),Lo(c)).subscribe({next:([f])=>{d=f.offsetX-a.nf;e=f.offsetY-a.Ja;a.Gc=a.nf+d;a.Hc=a.Ja+e;cn(a.P)},complete:()=>{if(5<=Math.abs(d)||5<=Math.abs(e)){const f=RN(a,a.nf,a.Ja),g=RN(a,a.Gc,a.Hc),k=Math.min(f.x,g.x),m=Math.min(f.y,g.y);a.Yc({x:k,y:m,width:Math.max(f.x,g.x)-
k,height:Math.max(f.y,g.y)-m},.9,200,!1,!1)}a.nf=-1;a.Ja=-1;a.Gc=-1;a.Hc=-1;cn(a.P)}})}}
function PN(a,b){a.Pb.top=-1E3;a.Pb.left=-1E3;a.Pb.width=0;a.Pb.height=0;a.Pb.Od=void 0;if(""!==a.pl&&null!=b){const {x:c,y:d}=oN(a,b.bound.x-b.bound.width/2,b.bound.y-b.bound.height/2),{x:e,y:f}=oN(a,b.bound.x+b.bound.width/2,b.bound.y+b.bound.height/2);a.Pb.top=d;a.Pb.left=c;a.Pb.width=e-c;a.Pb.height=f-d;b.id.includes("_left")?a.Pb.Od=a.ga.nodesById[b.nodeId].expanded?"Collapse layer":"Expand layer":b.id.includes("_right")&&(a.Pb.Od="More actions")}cn(a.P)}
function oN(a,b,c){var d=a.ab.R;const e=d.clientWidth;d=d.clientHeight;b=new THREE.Vector3(b,0,c);b.project(a.camera);return{x:b.x*e/2+e/2,y:-(b.y*d/2)+d/2}}function qN(a,b){sa(function*(){KJ(a.C,a.paneId,a.ga.id,b,yield CN(a));const c=JJ(a.C,b);c&&(wJ(a.C,a.paneId),sJ(a.C,!1),a.C.N.set(void 0),a.C.F.set(void 0),qJ(a.C,c))})}function TN(a,b){a=a.ga.nodesById[null!=b?b:a.Be];return null!=a&&oI(a)?a.expanded:!1}
function UN(a){var b=a.ab.R;const c=b.clientWidth;b=b.clientHeight;const d=c/b,e=a.hh-c/2,f=a.ih-b/2;a.camera.left=-500/a.J*d-e/c*1E3/a.J*d;a.camera.right=500/a.J*d-e/c*1E3/a.J*d;a.camera.top=500/a.J+500*f/a.J/b*2;a.camera.bottom=-500/a.J+500*f/a.J/b*2;a.camera.updateProjectionMatrix()}function LN(a){const b=Math.max(0,Math.min(1,(7.5-(a.ab?30*a.ab.R.clientHeight/500/2:0)*a.J)/.5));1E-5>Math.abs(b-a.zl)||(a.zl=b,TM(a.D,a.td,b/3))}
function VN(a,b){if(!a.C.C){var c=Date.now(),d=()=>{var e=Date.now()-c;e=Math.min(1,e/1100);e=-(Math.cos(Math.PI*e)-1)/2;const f=8*Math.sin(e*Math.PI*9);RM(a.D,b,f);RM(a.Pa,b,f);a.render();1<=e?(RM(a.D,b,0),RM(a.Pa,b,0),a.render()):requestAnimationFrame(d)};d()}}function WN(a,b,c=1){return a.ab?1E3*b/c/a.ab.R.clientHeight:0}
function XN(a,b,c,d,e=300){if(a.ab){var f=a.ab.R;b=f.clientWidth/f.clientHeight*-1E3/2/d+b;c=-c+500/d;d=SL.scale(d);var g=d.translate;if(a.ab){var k=a.ab.R,m=k.clientWidth;b=b/(m/k.clientHeight*500)/-2*m}else b=0;b=g.call(d,b,a.ab?c*a.ab.R.clientHeight/500/2:0);f=QL(f);0===e?f.call(a.zoom.transform,b):f.transition().duration(a.C.C?0:e).Nf(OL).call(a.zoom.transform,b)}}function EN(){return 1===window.devicePixelRatio?1.5:window.devicePixelRatio}
function NN(a){var b=a.ab.R;const c=a.ca.domElement,d=b.clientWidth;b=b.clientHeight;if(0!==d&&0!==b&&(c.width!==d||c.height!==b)){if(null==a.xi&&null==a.yi){const {x:e,y:f}=RN(a,d/2,b/2);a.xi=e;a.yi=f}a.ca.setSize(d,b,!1);a.canvas.R.style.width="100%";a.canvas.R.style.height="100%";UN(a);a.render();a.Yc({x:a.xi,y:a.yi,width:1E-7,height:1E-7},.9,0,!0);0<=a.xl&&window.clearTimeout(a.xl);a.xl=window.setTimeout(()=>{a.xi=null;a.yi=null},500)}}
function IN(a){if(0!==Object.keys(a.F).filter(k=>a.F[k].selected).length){var b=[],c=9/a.fa.G,d=[];for(const {node:k,index:m}of a.N){const p=[];var e=38,f=0;let t=0;const w=[];if(nI(k)){var g=pI(a.F);for(const N of g)w.push({key:N,value:rI(k,N)});let B;if(null==(B=a.F["Op node attributes"])?0:B.selected){let N;w.push(...BI(k,(null==(N=a.F["Op node attributes"])?void 0:N.filterRegex)||""))}let G;(null==(G=a.F["Op node inputs"])?0:G.selected)&&w.push(...CI(k,a.ga));let J;(null==(J=a.F["Op node outputs"])?
0:J.selected)&&w.push(...DI(k));w.push(...EI(k,a.ga.id,a.F,a.lb))}else if(oI(k)){g=qI(a.F);for(const B of g)w.push({key:B,value:rI(k,B)})}for(const {key:B,value:G}of w){const {sj:J,FC:N,km:aa,QE:ca}=YN(a,k,m,B,G,e,c);b.push(J,aa);f=Math.max(N,f);t=Math.max(ca,t);p.push({sj:J,km:aa});e+=12}e=((k.width||0)-(f+t+4))/2;for(const B of p)B.sj.x=cM(k)+f,B.km.x=cM(k)+f+4,B.sj.x+=e,B.km.x+=e;0<p.length&&nI(k)&&(f=(k.width||0)-16,e=12*p.length,d.push({id:k.id,index:d.length,bound:{x:cM(k)+8+f/2,y:p[0].sj.z+
e/2-6,width:f,height:e},Me:.001*m+2E-4,Fe:!0,borderColor:{r:1,g:1,b:1},bgColor:{r:1,g:1,b:1},borderWidth:1,opacity:1}))}0<b.length&&(XM(a.fa,b),a.fa.C&&a.scene.add(a.fa.C),HM(a.fh,d),a.fh.C&&a.scene.add(a.fh.C))}}
function wN(a,b=c=>{SM(a.D,c);a.Nb.D.value=c;SM(a.na,c);SM(a.Vd,c);a.zi.D.value=c;a.Dd.D.value=c;a.Sa.D.value=c;a.fa.D.value=c;SM(a.fh,c);SM(a.wi,c);SM(a.eh,c);var d=a.edges;d.C&&(d.J.value=c)}){const c=Date.now(),d=()=>{var e=Date.now()-c;e=a.C.C?1:Math.min(1,e/200);e=Math.sin(e*Math.PI/2);b(e);a.render();1<=e?(b(e),a.render()):requestAnimationFrame(d)};d()}function JN(a,b){b=(b.namespace||"").split("/").filter(c=>""!==c).length;return KN(a.Ar[Math.min(a.Ar.length-1,b)])}
function KN(a){return{r:a.r,g:a.g,b:a.b}}
function vN(a,b,c){var d;const e=(null==(d=a.C.config())?void 0:d.edgeLabelFontSize)||7.5;var f;d=(null==(f=a.C.config())?void 0:f.disallowVerticalEdgeLabels)||!1;const g=[],k=xM(a.T,1);for(const {edge:ia}of b){var m=a.ga.nodesById[ia.fromNodeId];b=a.ga.nodesById[ia.toNodeId];if(!nI(m)||!nI(b))continue;let O="?";b=m.outputsMetadata||{};for(const Q of Object.keys(b))if(null!=(m.outgoingEdges||[]).find(la=>la.sourceNodeOutputId===Q&&la.targetNodeId===ia.toNodeId)){O=b[Q].shape||"?";O=O.split("").map(la=>
{"x"===la&&(la="x");"\u2217"===la&&(la="*");""===la&&(la="");return null==k[la]?"?":la}).join("");break}b=ia.curvePoints||[];f=m.globalX||0;m=m.globalY||0;const T=new THREE.CurvePath;for(var p=0;p<b.length-1;p++){var t=b[p],w=b[p+1];t=new THREE.LineCurve(new THREE.Vector2(t.x+f,t.y+m),new THREE.Vector2(w.x+f,w.y+m));T.add(t)}b=T.getLength();f=e/2/b;p=()=>{const Q=T.getPointAt(.5);g.push({id:`${ia.id}_${O}`,nodeId:ia.toNodeId,label:O,height:e,cf:"center",vAlign:"center",weight:1,x:Q.x,y:95,z:Q.y,color:c,
borderColor:{r:1,g:1,b:1}})};if(1<f*(O.length+3))p();else{m=[];t=Math.max(0,Math.min(.25,1-O.length*f-5/b));w=Math.max(.05,1-5/b-t-f*O.length);var B=!0;f=e/a.Sa.G;for(var G=0;G<w;G+=.05){var J=t+G;B=!0;var N=void 0;m=[];for(let Q=0;Q<O.length;Q++){var aa=O[Q];const la=T.getPointAt(Math.min(J,1)),F=T.getTangentAt(Math.min(J,1));var ca=(2*Math.PI-Math.atan(F.y/F.x))%(2*Math.PI);0>ca&&(ca+=2*Math.PI);if(d&&ca>=Math.PI/4&&ca<=1.75*Math.PI){B=!1;break}m.push({ro:la,position:Math.min(J,1),angle:ca,tan:F,
char:aa});if(null!=N&&(N=Math.abs(ca-N),.15<Math.min(N,Math.abs(N-Math.PI))&&(B=!1,G+.05<w)))break;N=ca;aa=k[aa];ca=0;Q!==O.length-1&&(ca=k[O[Q+1]].xadvance);J+=(aa.xadvance/2+ca/2)*f/b}if(B)break}if(B){if(p=m[0].ro.x>=m[m.length-1].ro.x){t=[];m=m[0].position;for(w=O.length-1;0<=w;w--)B=O[w],G=T.getPointAt(Math.min(1,m)),N=T.getTangentAt(Math.min(1,m)),J=(2*Math.PI-Math.atan(N.y/N.x))%(2*Math.PI),0>J&&(J+=2*Math.PI),t.push({ro:G,angle:J,tan:N,position:m,char:B}),B=k[B],G=0,1<=w&&(G=k[O[w-1]].xadvance),
m+=(B.xadvance/2+G/2)*f/b;m=t}for(b=0;b<m.length;b++)f=m[b],t=f.char,w=f.ro,B=f.angle,1E-7>Math.abs(f.tan.x)&&(B=(p?1:-1)*Math.PI/2),g.push({id:`${ia.id}_${t}_${b}`,nodeId:ia.toNodeId,label:t,height:e,cf:"",vAlign:"",weight:1,x:w.x+1.5*Math.sin(B)*-e,y:95,z:w.y+1.5*Math.cos(B)*-e,color:c,angle:B,zq:!0,borderColor:{r:1,g:1,b:1}})}else p()}}return g}
function YN(a,b,c,d,e,f,g){const k={id:`${b.id}_attrs_table_${d}_key`,nodeId:b.id,label:`${d}:`,height:9,cf:"right",vAlign:"center",weight:1,x:cM(b),y:.001*c+4E-4,z:dM(b)+f,color:a.Uy};var m=VM(a.fa,k.label,k.weight,k.height).sizes;m=(m.Ph-m.Sh)*g;b={id:`${b.id}_attrs_table_${d}_value`,nodeId:b.id,label:e,height:9,cf:"left",vAlign:"center",weight:0,x:cM(b),y:.001*c+4E-4,z:dM(b)+f,color:a.Wy,maxWidth:200};const {sizes:p,ME:t}=VM(a.fa,b.label,b.weight,b.height,b.maxWidth);null!=t&&(b.label=t);return{sj:k,
FC:m,km:b,QE:(p.Ph-p.Sh)*g}}function uN(a){a.wa.clear();a.Ea.clear();a.Ee={};a.kf={};a.gl=[];a.wl=[];for(const b of[a.O.C,a.O.D,a.ac.C,a.Ob.C,a.Dc.C])b&&(b.geometry&&b.geometry.dispose(),a.scene.remove(b));a.O.D=void 0}function yN(a){for(const b of[a.Qd.C,a.Rd.C])b&&(b.geometry&&b.geometry.dispose(),a.scene.remove(b))}function BN(a){a.Fq=!0;cn(a.P);setTimeout(()=>{a.Fq=!1;cn(a.P)},300)}
function CN(a){return sa(function*(){var b=a.ab.R,c=RN(a,0,0);b=RN(a,b.clientWidth,b.clientHeight);c={x:c.x,y:c.y,width:b.x-c.x,height:b.y-c.y};b=[];uI(void 0,a.ga,b);var d=a.canvas.R,e=a.Ux.R;d=320/d.width*d.height;a.Xc.setSize(320,d,!1);a.Xc.render(a.scene,a.camera);var f=window.devicePixelRatio;d=new OffscreenCanvas(320*f,d*f);f=d.getContext("2d");f.imageSmoothingQuality="high";f.drawImage(e,0,0,e.width,e.height,0,0,d.width,d.height);e=yield createImageBitmap(d);d=uJ(a.C,a.paneId);return{id:tI(),
rect:c,Uq:e,selectedNodeId:a.selectedNodeId,deepestExpandedGroupNodeIds:b,showOnNodeItemTypes:Object.assign({},a.F),Wb:Object.assign({},a.ja),flattenLayers:d}})}function GN(a){for(const b of[a.Pa.C,a.Jc.C])b&&(b.geometry&&b.geometry.dispose(),a.scene.remove(b))}
var ZN=class extends tM{constructor(){super();this.zc=!1;this.sz=new THREE.Color("#1A73E8");this.rz=new THREE.Color("#C2E7FF");this.dz=new THREE.Color("#000");this.cz=new THREE.Color("#666");this.fz=new THREE.Color("#e2edff");this.gz=new THREE.Color("#e3e3e3");this.hz=new THREE.Color("#ccc");this.td=new THREE.Color("#041E49");this.qz=new THREE.Color("#777");this.az=new THREE.Color("#aaa");new THREE.Color("#DADCE0");this.bz=new THREE.Color("#444746");this.Xy=new THREE.Color("#aaa");this.md=new THREE.Color("#009e73");
this.Yy=new THREE.Color("#125341");this.sd=new THREE.Color("#d55e00");this.Zy=new THREE.Color("#994d11");this.Uy=new THREE.Color("#808080");this.Wy=new THREE.Color("#0d0d0d");this.VE=new THREE.Color("#800080");this.Bx=new THREE.Color("#f5d55a");this.ky=new THREE.Color("#135cbb");this.uz=new THREE.Color("#d5e7ff");this.Ar=(()=>{const a=[];for(let b=0;6>b;b++)a.push(new THREE.Color(`hsl(212, 40%, ${Math.round(96+-2.4*b)}%)`));return a})();new THREE.Color("#C2E7FF");new THREE.Color("#7facc9");this.Pb=
{top:-1E3,left:-1E3,width:0,height:0};this.rl="";this.Rn=this.Sn=-1E3;this.br=this.dr=0;this.cr="";this.Fo=this.Go=-1E3;this.As=this.gy=0;this.vv=[];this.Hc=this.Gc=this.Ja=this.nf=-1;this.Fq=!1;this.jh=-1;this.fg=0;this.Tv="";this.mz=lf(Qk);this.T=lf(zM);this.nz=lf(rL);this.D=new UM(6);this.Nb=new bN(this.T);this.na=new UM(99);this.wi=new UM(0);this.eh=new UM(6);this.Pa=new UM(8);this.Jc=new UM(4);this.Qd=new UM(99);this.Rd=new bN(this.T);this.O=new UM(99);this.ac=new bN(this.T);this.Vd=new UM(3);
this.zi=new bN(this.T);this.edges=new GM(this.Xy,1);this.wa=new GM(this.md,1.5);this.Ea=new GM(this.sd,1.5);this.Ob=new bN(this.T);this.Dc=new bN(this.T);this.Dd=new bN(this.T);this.Sa=new bN(this.T);this.fa=new bN(this.T);this.fh=new UM(4);this.sl=new THREE.Vector2;this.Pd=this.Kb=this.ud=this.sb=0;this.Ee={};this.kf={};this.gl=[];this.wl=[];this.On=this.ql=this.fo=this.pl=this.Be="";this.yi=this.xi=null;this.zl=this.xl=-1;this.gb=void 0;this.ik=[];this.jk=[];Js(()=>{const a=this.Qg();(null==a?void 0:
a.rendererId)===this.rendererId&&(this.selectedNodeId=(null==a?void 0:a.nodeId)||"",this.Wg&&(this.selectedNodeId&&nI(this.ga.nodesById[this.selectedNodeId])?oM(this):this.ua=void 0),tN(this),xN(this),zN(this),this.render())});C(this.C.ja,sL(this.G)).subscribe(a=>{a.rendererId===this.rendererId&&AN(this)});C(this.C.aa,sL(this.G)).subscribe(a=>{if(a.rendererId===this.rendererId){a=a.kb;a.showOnNodeItemTypes&&(this.F=Object.assign({},a.showOnNodeItemTypes),RJ(this.C,this.paneId,this.rendererId,this.F));
a.Wb&&(this.ja=Object.assign({},a.Wb),SJ(this.C,this.paneId,this.rendererId,this.ja));const b=uJ(this.C,this.paneId),c=!0===a.flattenLayers;b!==c?(mJ(this.C,this.paneId,c,a),sJ(this.C,c)):WL(this,a.selectedNodeId||"",a.deepestExpandedGroupNodeIds||[],!1,a.rect,!0,a.showOnNodeItemTypes)}});C(this.C.na,sL(this.G)).subscribe(a=>{a.rendererId===this.rendererId&&DN(this,a.UB)});Js(()=>{const a=nJ(this.C,this.paneId);a&&a.modelGraph&&this.gb!==a.Pg&&(this.gb=a.Pg,FN(this),this.render())});Js(()=>{var a=
this.nz.rules().filter(b=>LI(b.queries)&&0<Object.keys(b.styles).length);a=JSON.stringify(a);JSON.stringify(this.ik)!==a&&(this.ik=JSON.parse(a),this.jk=MI(this.ik),HN(this),tN(this),xN(this),zN(this),this.render())});Js(()=>{if(nJ(this.C,this.paneId)){var a=UJ(this.C,this.paneId,this.rendererId);JSON.stringify(a)!==JSON.stringify(this.ja)&&(this.ja=a,HN(this),tN(this),xN(this),zN(this),this.render())}})}Fb(){this.Ag=this.modelGraph.id;this.ga=this.modelGraph;this.Qb||(this.F=eJ(this.C),this.ja=fJ(this.C));
YL(this,this.ab.R);MN(this);Mk(this.mz,()=>{this.canvas.R.addEventListener("mousemove",d=>{ON(this,d)})});var a=this.C.N(),b=d=>{bM(this);HN(this);this.Wd(.9,0);const e=nJ(this.C,this.paneId);if(null!=(null==e?void 0:e.Do)){const f=e.Do;this.F=e.Do.showOnNodeItemTypes||{};RJ(this.C,this.paneId,this.rendererId,this.F);WL(this,f.selectedNodeId||"",f.deepestExpandedGroupNodeIds||[],!1,f.rect,!0,f.showOnNodeItemTypes,!0);e.Do=void 0}else null!=d&&""!==d&&setTimeout(()=>{this.C.F.set({nodeId:d,rendererId:this.rendererId,
Bd:!1,noNodeShake:!0})});ZL(this,this.Vl||"")};if(!a||0===a.paneStates.length||this.Qb){var c;let d;a=this.Qb?void 0:null==(c=nJ(this.C,this.paneId))?void 0:null==(d=c.Qg)?void 0:d.nodeId;b(a)}else c=iJ(this.C,this.paneId),(a=a.paneStates[c])?(WL(this,a.selectedNodeId,a.deepestExpandedGroupNodeIds,!0),SI(this.Oe,a.deepestExpandedGroupNodeIds,c)):b();b=window;null==b.me_test&&(b.me_test={ID:{}});a=this.Qb?-1:iJ(this.C,this.paneId);b.me_test.ID[a]=this;this.zc&&QN(this)}va(){super.va();this.ca&&(this.ca.dispose(),
this.ca.forceContextLoss());this.Xc&&(this.Xc.dispose(),this.Xc.forceContextLoss());this.bb&&(this.bb.dispose(),this.bb.forceContextLoss());document.body.style.cursor="default"}Dl(){super.Dl();tN(this);zN(this);this.render()}get eE(){return!this.Qb}get GD(){return Math.min(this.Ja,this.Hc)}get FD(){return Math.min(this.nf,this.Gc)}get HD(){return Math.abs(this.Gc-this.nf)}get ED(){return Math.abs(this.Hc-this.Ja)}Gk(){super.Gk();this.Be?this.ga.nodesById[this.Be]&&$L(this,this.Be):$L(this,"")}Ik(a){super.Ik(a);
""!==this.selectedNodeId&&fM(this,this.ga.nodesById[this.selectedNodeId],a)}dl(){super.dl();requestAnimationFrame(()=>{this.camera&&(UN(this),LN(this),this.render(),PN(this))})}By(a,b){LL.ys&&"mouseup"===LL.ys.type&&(a=Math.abs(this.hh-a),b=Math.abs(this.ih-b),0<=a&&3>=a&&0<=b&&3>=b&&this.Gk())}Nk(a,b,c,d,e){super.Nk(a,b,c,d,e);HN(this);null!=c?eM(this,c,a,400):this.Wd();tN(this);xN(this);zN(this);this.render();this.Qb||SI(this.Oe,e,iJ(this.C,this.paneId))}al(a,b,c,d,e,f,g){super.al(a,b,c,d,e,f,g);
HN(this);tN(this);xN(this);zN(this);if(e){const k=()=>{this.Yc(e,1,200,!1,!1)};f?setTimeout(()=>{k()}):k()}else"______"!==c&&eM(this,c,b,d?0:400);this.selectedNodeId!==c&&$L(this,c||"");this.Qb||SI(this.Oe,g||[],iJ(this.C,this.paneId))}uy(){HN(this);zN(this);this.render()}kh(a,b,c,d,e,f,g=!1){super.kh(a,b,c,d,e,f);eM(this,c,b,400);g||(HN(this),tN(this),xN(this),zN(this),this.render(),this.Qb||SI(this.Oe,d,iJ(this.C,this.paneId)));e||setTimeout(()=>{VN(this,c)},250)}Yc(a,b=.9,c=300,d=!1,e=!0,f=!1){if(this.ab){var g=
this.ab.R,k=g.clientWidth*b;b*=g.clientHeight;g=a.width/a.height;if(d)k=this.J;else{if(g>k/b){if(this.ab){g=this.ab.R;var m=g.clientWidth;k=k/m*-2*(m/g.clientHeight)*500}else k=0;k/=a.width}else k=WN(this,b)/a.height;k=Math.abs(k)}g=a.x+a.width/2;m=a.y+a.height/2;!d&&e&&(k=Math.min(WN(this,45)/30,k),f&&(d=WN(this,20)/30,k<d&&(m=a.y+WN(this,b/2-60,d),k=Math.max(d,k))));XN(this,g,m,k,c)}}Wd(a=.9,b=200){this.Yc({x:this.sb,y:this.Kb,width:this.ud-this.sb,height:this.Pd-this.Kb},a,b)}aa(a){return null!=
this.sa[a]}render(a=!1){this.ca&&this.scene&&this.camera&&(this.ca.render(this.scene,this.camera),this.zc&&a&&(0>this.jh&&(this.jh=performance.now()),this.fg+=1,a=performance.now()-this.jh,1E3<a&&(this.Tv=(this.fg/a*1E3).toFixed(1),this.jh=-1,this.fg=0,bn(this.P))))}};ZN.K=function(a){return new (a||ZN)};
ZN.Da=Gf({type:ZN,ha:[["webgl-renderer"]],fb:function(a,b){a&1&&(ur(cN,7),ur(dN,7),ur(eN,7),ur(fN,7),ur(gN,7),ur(hN,7),ur(iN,7),ur(jN,5),ur(kN,5),ur(lN,7));if(a&2){let c;vr(c=wr())&&(b.ab=c.first);vr(c=wr())&&(b.canvas=c.first);vr(c=wr())&&(b.Ux=c.first);vr(c=wr())&&(b.xD=c.first);vr(c=wr())&&(b.SF=c.first);vr(c=wr())&&(b.nw=c.first);vr(c=wr())&&(b.uE=c.first);vr(c=wr())&&(b.Xv=c.first);vr(c=wr())&&(b.ow=c.first);vr(c=wr())&&(b.SC=c.first)}},inputs:{zc:"benchmark"},ka:!0,features:[Cp,Mr],za:36,Aa:60,
Ca:[["container",""],["canvas",""],["snapshotCanvas",""],["pngDownloaderCanvas",""],["groupNodeIconMatTooltip","matTooltip"],["moreActionsMenuTrigger","matMenuTrigger"],["moreActionsMenu","matMenu"],["ioPicker",""],["ioPickerMatTooltip","matTooltip"],["subgraphIndicator",""],["subgraphMenuTrigger","matMenuTrigger"],["menu","matMenu"],[1,"container",3,"mouseleave"],[1,"main",3,"mousedown","contextmenu"],[1,"snapshot"],[1,"download-png"],[1,"group-node-icon",3,"mouseenter","mouseleave","mousedown"],
["matTooltipPosition","above",1,"group-node-icon-tooltip",3,"matTooltip","matTooltipShowDelay","matTooltipDisabled"],[1,"more-actions-menu-trigger",3,"matMenuTriggerFor"],[1,"model-explorer-menu-item-with-icon","expand-all",3,"click"],["class","model-explorer-menu-item-with-icon collapse-all",3,"click",4,"ngIf"],["class","model-explorer-menu-item-with-icon open-in-popup",3,"click",4,"ngIf"],[1,"io-picker",3,"mousedown","mouseenter","mouseleave"],["matTooltipPosition","above",1,"io-picker-tooltip",
3,"matTooltip"],["matTooltip","Jump to subgraph","matTooltipPosition","above",1,"subgraph-indicator",3,"mousedown","click"],[1,"subgraph-indicator-menu-trigger",3,"matMenuTriggerFor"],[1,"model-explorer-menu-item-with-icon"],["class","range-container",3,"top","left","width","height",4,"ngIf"],[1,"flash"],[1,"fps"],[1,"model-explorer-menu-item-with-icon","collapse-all",3,"click"],[1,"model-explorer-menu-item-with-icon","open-in-popup",3,"click"],[1,"model-explorer-menu-item-with-icon",3,"click"],[1,
"range-container"]],ta:function(a,b){if(a&1){const c=Ug();R(0,"div",12,0);V("mouseleave",function(d){u(c);d=d.relatedTarget;null!=d&&d.classList.contains("cdk-overlay-backdrop")||(b.Be="",zN(b),PN(b),b.render());return x()});R(2,"canvas",13,1);V("mousedown",function(d){u(c);return x(SN(b,d))})("contextmenu",function(d){u(c);return x(d.preventDefault())});S();Mq(4,"canvas",14,2)(6,"canvas",15,3);R(8,"div",16);V("mouseenter",function(){u(c);bG(b.Xv);return x()})("mouseleave",function(){u(c);$F(b.Xv);
return x()})("mousedown",function(d){u(c);d.stopPropagation();b.pl.includes("_left")?($L(b,b.fo),fM(b,b.ga.nodesById[b.fo])):b.pl.includes("_right")&&(b.rl=b.fo,WC(b.SC));return x()});S();Mq(9,"div",17,4)(11,"div",18,5);R(13,"mat-menu",null,6)(15,"div",19);V("click",function(){u(c);var d=b.rl;if(d=null!=d?d:b.Be)$L(b,d),fM(b,b.ga.nodesById[d],!0,!0);return x()});R(16,"mat-icon");Z(17,"unfold_more_double");S();Z(18," Expand layer & all sub-layers ");S();Ap(19,mN,4,0,"div",20)(20,nN,4,0,"div",21);S();
R(21,"div",22,7);V("mousedown",function(d){u(c);d.stopPropagation();d=b.ql.split("||||")[0];b.ql.endsWith("input")?1===b.Ee[d].length?UL(b,b.Ee[d][0].id,b.rendererId):rM(b,b.nw.R,b.Ee[d],"incoming"):1===b.kf[d].length?UL(b,b.kf[d][0].id,b.rendererId):rM(b,b.nw.R,b.kf[d],"outgoing");return x()})("mouseenter",function(){u(c);bG(b.ow);return x()})("mouseleave",function(){u(c);$F(b.ow);return x()});S();Mq(23,"div",23,8);R(25,"div",24,9);V("mousedown",function(d){u(c);return x(d.stopPropagation())})("click",
function(){u(c);if(b.On){var d=b.ga.nodesById[b.On];nI(d)&&(d=d.subgraphIds,1===d.length?qN(b,d[0]):1<d.length&&(b.vv=d,WC(b.uE)))}return x()});S();Mq(27,"div",25,10);R(29,"mat-menu",null,11);Jq(31,pN,4,1,"div",26,Hq);S();Ap(33,rN,1,8,"div",27);Mq(34,"div",28);Ap(35,sN,2,1,"div",29);S()}if(a&2){a=zr(14);const c=zr(30);D(8);lq("top",b.Pb.top,"px")("left",b.Pb.left,"px")("width",b.Pb.width,"px")("height",b.Pb.height,"px");D();lq("top",b.Pb.top,"px")("left",b.Pb.left,"px")("width",b.Pb.width,"px")("height",
b.Pb.height,"px");H("matTooltip",b.Pb.Od||"")("matTooltipShowDelay",200)("matTooltipDisabled",null==b.Pb.Od);D(2);lq("top",b.Pb.top+b.Pb.height,"px")("left",b.Pb.left,"px");H("matMenuTriggerFor",a);D(8);H("ngIf",TN(b,b.rl));D();H("ngIf",b.eE);D();lq("top",b.Sn,"px")("left",b.Rn,"px")("width",b.dr,"px")("height",b.br,"px");D(2);lq("top",b.Sn,"px")("left",b.Rn,"px")("width",b.dr,"px")("height",b.br,"px");H("matTooltip",b.cr);D(2);lq("top",b.Go,"px")("left",b.Fo,"px")("width",b.gy,"px")("height",b.As,
"px");D(2);lq("top",b.Go+b.As,"px")("left",b.Fo,"px");H("matMenuTriggerFor",c);D(4);Lq(b.vv);D(2);H("ngIf",0<=b.nf);D();nq("flashing",b.Fq);D();Eq(b.zc?35:-1)}},Ha:[ht,ct,$C,CC,QC,YC,iI,hI,gG,fG],styles:[".container[_ngcontent-%COMP%]{width:100%;height:100%;overflow:hidden;position:relative}.container[_ngcontent-%COMP%]   canvas.main[_ngcontent-%COMP%]{width:100%!important;height:100%!important;-moz-box-sizing:border-box;box-sizing:border-box}.container[_ngcontent-%COMP%]   canvas.download-png[_ngcontent-%COMP%], .container[_ngcontent-%COMP%]   canvas.snapshot[_ngcontent-%COMP%]{position:absolute;left:0;bottom:0;width:400px;height:300px;border:1px solid red;-moz-box-sizing:border-box;box-sizing:border-box;background-color:#fff;display:none}.container[_ngcontent-%COMP%]   .hover-toolbar[_ngcontent-%COMP%]{position:absolute;width:50px;height:40px;-moz-box-sizing:border-box;box-sizing:border-box;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-transition:opacity .15s;transition:opacity .15s;opacity:1;pointer-events:none}.container[_ngcontent-%COMP%]   .hover-toolbar.hide[_ngcontent-%COMP%]{-webkit-transition:none;transition:none;opacity:0}.container[_ngcontent-%COMP%]   .hover-toolbar[_ngcontent-%COMP%]   .content[_ngcontent-%COMP%]{width:100%;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center}.container[_ngcontent-%COMP%]   .hover-toolbar[_ngcontent-%COMP%]   .content[_ngcontent-%COMP%]   .main-content-wrapper[_ngcontent-%COMP%]{height:100%;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;pointer-events:all}.container[_ngcontent-%COMP%]   .hover-toolbar[_ngcontent-%COMP%]   .content[_ngcontent-%COMP%]   .main-content[_ngcontent-%COMP%]{font-size:12px;border:1px solid #ccc;background-color:hsla(0,0%,98%,.9);border-radius:99px;padding:0 8px;-moz-box-sizing:border-box;box-sizing:border-box;box-shadow:0 1px 3px 0 rgba(0,0,0,.1),0 1px 2px -1px rgba(0,0,0,.1);display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}.container[_ngcontent-%COMP%]   .hover-toolbar[_ngcontent-%COMP%]   .content[_ngcontent-%COMP%]   .main-content[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;cursor:pointer;opacity:.8;width:24px;padding:4px 0}.container[_ngcontent-%COMP%]   .hover-toolbar[_ngcontent-%COMP%]   .content[_ngcontent-%COMP%]   .main-content[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]:hover{opacity:1;background-color:#eee}.container[_ngcontent-%COMP%]   .hover-toolbar[_ngcontent-%COMP%]   .content[_ngcontent-%COMP%]   .main-content[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{font-size:16px;font-weight:700;width:16px;height:16px;line-height:16px;color:#ea8600}.container[_ngcontent-%COMP%]   .group-node-icon[_ngcontent-%COMP%], .container[_ngcontent-%COMP%]   .io-picker[_ngcontent-%COMP%], .container[_ngcontent-%COMP%]   .subgraph-indicator[_ngcontent-%COMP%]{position:absolute;-moz-box-sizing:border-box;box-sizing:border-box;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;background-color:transparent}.container[_ngcontent-%COMP%]   .group-node-icon[_ngcontent-%COMP%], .container[_ngcontent-%COMP%]   .io-picker[_ngcontent-%COMP%]{z-index:100}.container[_ngcontent-%COMP%]   .group-node-icon-tooltip[_ngcontent-%COMP%], .container[_ngcontent-%COMP%]   .io-picker-tooltip[_ngcontent-%COMP%]{position:absolute;z-index:50}.container[_ngcontent-%COMP%]   .more-actions-menu-trigger[_ngcontent-%COMP%], .container[_ngcontent-%COMP%]   .subgraph-indicator-menu-trigger[_ngcontent-%COMP%]{position:absolute;visibility:hidden}.container[_ngcontent-%COMP%]   .range-container[_ngcontent-%COMP%]{position:absolute;z-index:5000;-moz-box-sizing:border-box;box-sizing:border-box;border:1px solid #1976d2;background-color:rgba(97,175,252,.2);pointer-events:none}.container[_ngcontent-%COMP%]   .flash[_ngcontent-%COMP%]{position:absolute;top:0;left:0;width:100%;height:100%;z-index:5000;-moz-box-sizing:border-box;box-sizing:border-box;border:5px solid #1976d2;pointer-events:none;opacity:0}@-webkit-keyframes _ngcontent-%COMP%_flashing{5%{opacity:1}to{opacity:0}}@keyframes _ngcontent-%COMP%_flashing{5%{opacity:1}to{opacity:0}}.container[_ngcontent-%COMP%]   .flashing[_ngcontent-%COMP%]{-webkit-animation:_ngcontent-%COMP%_flashing .3s ease-out;animation:_ngcontent-%COMP%_flashing .3s ease-out}.container[_ngcontent-%COMP%]   .fps[_ngcontent-%COMP%]{position:absolute;top:12px;left:12px;z-index:100;font-size:32px;border:1px solid #777;padding:8px;border-radius:4px;background-color:#fff}  .multiline-tooltip{white-space:pre-line}  .multiline-tooltip-left{white-space:pre-line}  .multiline-tooltip-left>div{text-align:left}  .model-explorer-menu-item-with-icon{font-family:Google Sans Text,Arial,Helvetica,sans-serif;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;font-size:12px;padding:5px 8px;line-height:15px;letter-spacing:normal}  .model-explorer-menu-item-with-icon mat-icon{font-size:16px;width:16px;height:16px;margin-right:4px;color:#999}  .model-explorer-menu-item-with-icon:hover{background-color:rgba(0,0,0,.04)}"],
Ra:0});function $N(a){if(a&1){const b=Ug();R(0,"button",10);V("click",function(){u(b);const c=Y(2);return x(aO(c))});Z(1," Start ");S()}}function bO(a){a&1&&(R(0,"div",9),Z(1,"Generating graph. Please wait..."),S())}
function cO(a){a&1&&(R(0,"div",2)(1,"div",4)(2,"div",5),Z(3,"Node count"),S(),Mq(4,"input",6),S(),R(5,"div",4)(6,"div",5),Z(7,"Edge count"),S(),Mq(8,"input",6),S(),R(9,"div",4)(10,"div",5),Z(11,"Colorize"),S(),Mq(12,"input",7),S(),Ap(13,$N,2,0,"button",8)(14,bO,2,0,"div",9),S());a&2&&(a=Y(),D(4),H("formControl",a.ov),D(4),H("formControl",a.mv),D(4),H("formControl",a.lv),D(),Eq(a.Uv?14:13))}function dO(a){a&1&&Mq(0,"webgl-renderer",3);a&2&&(a=Y(),H("modelGraph",a.modelGraph)("benchmark",!0))}
function aO(a){a.Uv=!0;cn(a.C);setTimeout(()=>{const b=a.ov.value,c=a.mv.value;var d=Math.max(500,Math.floor(300*Math.sqrt(b)));const e=[];for(var f=0;f<b;f++){var g=`n${f}`;g={nodeType:0,id:g,label:g,namespace:"",level:0,x:Math.random()*d,y:Math.random()*d,globalX:0,globalY:0,width:150+60*(Math.random()-.5),height:50+30*(Math.random()-.5)};a.lv.value&&(g.style={backgroundColor:`rgb(${Math.floor(255*Math.random())}, ${Math.floor(255*Math.random())}, ${Math.floor(255*Math.random())})`});e.push(g)}d=
{};for(var k of e)d[k.id]=k;k={"":[]};for(f=0;f<c;f++){g=`n${Math.floor(Math.random()*b)}`;const w=`n${Math.floor(Math.random()*b)}`;var m=d[g],p=d[w];const B=`${g}_${w}`;var t=m.x+m.width/2;m=m.y+m.height/2;const G=p.x+p.width/2;p=p.y+p.height/2;t=[{x:t,y:m},{x:(t+G)/2+100*(Math.random()-.5),y:(m+p)/2+100*(Math.random()-.5)},{x:G,y:p}];p=vI(t);k[""].push({id:B,fromNodeId:g,toNodeId:w,points:t,curvePoints:p})}a.modelGraph={id:"benchmark_graph",collectionLabel:"benchmark_collection",nodes:e,nodesById:d,
rootNodes:e,edgesByGroupNodeIds:k,maxDescendantOpNodeCount:0,minDescendantOpNodeCount:0,layoutGraphEdges:{}};cn(a.C)},100)}var eO=class{constructor(a){this.C=a;this.ov=new vx(1E4);this.mv=new vx(5E3);this.lv=new vx(!1);this.Uv=!1}};eO.K=function(a){return new (a||eO)(z(Sr))};
eO.Da=Gf({type:eO,ha:[["benchmark-runner"]],ka:!0,features:[Mr],za:6,Aa:1,Ca:[[1,"container"],[1,"title"],[1,"form"],["rendererId","benchmark","paneId","benchmark",3,"modelGraph","benchmark"],[1,"row"],[1,"label"],["type","text",3,"formControl"],["type","checkbox",3,"formControl"],["mat-flat-button","","color","primary"],[1,"gen-msg"],["mat-flat-button","","color","primary",3,"click"]],ta:function(a,b){a&1&&(R(0,"div",0)(1,"div",1),Mq(2,"me-logo"),Z(3," Model Explorer Benchmark Test "),S(),Ap(4,cO,
15,4,"div",2)(5,dO,1,2,"webgl-renderer",3),S());a&2&&(D(4),Eq(b.modelGraph?5:4))},Ha:[ht,BB,AB,vB,WJ,Cx,sw,qw,Mw,xx,ZN],styles:[".container[_ngcontent-%COMP%]{width:100%;height:100%;overflow:hidden;-moz-box-sizing:border-box;box-sizing:border-box;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;position:relative}.container[_ngcontent-%COMP%]   .form[_ngcontent-%COMP%]{margin:12px}.container[_ngcontent-%COMP%]   .title[_ngcontent-%COMP%]{font-weight:400;cursor:pointer;color:#444746;font-size:20px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;padding:8px;padding-left:12px;border-bottom:1px solid #ddd}.container[_ngcontent-%COMP%]   .title[_ngcontent-%COMP%]   me-logo[_ngcontent-%COMP%]{margin-right:6px}.container[_ngcontent-%COMP%]   .row[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;margin-top:12px}.container[_ngcontent-%COMP%]   .row[_ngcontent-%COMP%]   .label[_ngcontent-%COMP%]{font-size:12px;color:#777}.container[_ngcontent-%COMP%]   .row[_ngcontent-%COMP%]   input[type=text][_ngcontent-%COMP%]{width:200px;font-size:13px}.container[_ngcontent-%COMP%]   .row[_ngcontent-%COMP%]   input[type=checkbox][_ngcontent-%COMP%]{width:-webkit-fit-content;width:-moz-fit-content;width:fit-content}.container[_ngcontent-%COMP%]   .row[_ngcontent-%COMP%]:first-child{margin-top:0}.container[_ngcontent-%COMP%]   button[_ngcontent-%COMP%]{margin-top:12px}.container[_ngcontent-%COMP%]   .gen-msg[_ngcontent-%COMP%]{font-size:12px;margin-top:12px}.container[_ngcontent-%COMP%]   webgl-renderer[_ngcontent-%COMP%]{position:absolute;top:48px;bottom:0;width:100%}"],
Ra:0});var fO=class{constructor(){this.extensions={}}};fO.K=function(a){return new (a||fO)};fO.pa=Md({ma:fO,ba:fO.K});function gO(a){a&1&&(R(0,"tr")(1,"td"),Mq(2,"div",9),S(),R(3,"td")(4,"div",2),Z(5,"Artificial layer to reduce layout load"),S()()())}function hO(a){a&1&&(R(0,"tr")(1,"td"),Mq(2,"div",10),S(),R(3,"td")(4,"div",2),Z(5),S()()());a&2&&(a=Y(),D(2),nq("group",a.vl),D(3),Br(" Selected ",a.vl?"layer":"op"," "))}function iO(a){a&1&&(R(0,"tr")(1,"td"),Mq(2,"div",11),S(),R(3,"td")(4,"div",2),Z(5,"Identical layer (if any)"),S()()())}
function jO(a){a&1&&(Nq(0),R(1,"tr")(2,"td"),Mq(3,"div",12),S(),R(4,"td")(5,"div",2),Z(6,"Inputs (if any)"),S()()(),R(7,"tr")(8,"td"),Mq(9,"div",13),S(),R(10,"td")(11,"div",2),Z(12,"Outputs (if any)"),S()()(),Oq())}var kO=class{constructor(a,b){this.C=a;this.D=b;this.Hn=this.vl=this.am=!1;Js(()=>{var c=nJ(this.C,this.paneId);c&&(this.Hn=!0===c.Hn,(c=c.Qg)?(this.am=""!==c.nodeId,this.vl=c.Bd):this.am=!1,bn(this.D))})}};kO.K=function(a){return new (a||kO)(z(VJ),z(Sr))};
kO.Da=Gf({type:kO,ha:[["legends-panel"]],inputs:{paneId:"paneId"},ka:!0,features:[Mr],za:35,Aa:4,Ca:[[1,"container"],[1,"key","op-node-key"],[1,"label"],[1,"key","group-node-key"],[4,"ngIf"],["colspan","2"],[1,"divider"],[1,"key","shortcut"],[1,"label","shortcut"],[1,"key","artificial-layer-key"],[1,"key","selected-node-key"],[1,"key","identical-group-key"],[1,"key","input-key"],[1,"key","output-key"]],ta:function(a,b){a&1&&(R(0,"div",0)(1,"table")(2,"tr")(3,"td"),Mq(4,"div",1),S(),R(5,"td")(6,"div",
2),Z(7,"Op"),S()()(),R(8,"tr")(9,"td"),Mq(10,"div",3),S(),R(11,"td")(12,"div",2),Z(13,"Layer"),S()()(),Ap(14,gO,6,0,"tr",4)(15,hO,6,3,"tr",4)(16,iO,6,0,"tr",4)(17,jO,13,0,"ng-container",4),R(18,"tr")(19,"td",5),Mq(20,"div",6),S()(),R(21,"tr")(22,"td")(23,"div",7),Z(24,"Zoom"),S()(),R(25,"td")(26,"div",8),Z(27,"Ctrl+Scroll"),S()()(),R(28,"tr")(29,"td")(30,"div",7),Z(31,"Pan"),S()(),R(32,"td")(33,"div",8),Z(34,"Drag or scroll"),S()()()()());a&2&&(D(14),H("ngIf",b.Hn),D(),H("ngIf",b.am),D(),H("ngIf",
b.am&&b.vl),D(),H("ngIf",b.am&&!b.vl))},Ha:[ht,ct,iI],styles:[".container[_ngcontent-%COMP%]{height:-webkit-fit-content;height:-moz-fit-content;height:fit-content;background-color:hsla(0,0%,100%,.9);padding:8px}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]{border-spacing:0;border-collapse:collapse;padding:0;font-size:11px}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .key[_ngcontent-%COMP%]{margin-right:4px;-moz-box-sizing:border-box;box-sizing:border-box}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .op-node-key[_ngcontent-%COMP%]{width:30px;height:14px;border-radius:5px;border:1px solid #666;background-color:#fff}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .group-node-key[_ngcontent-%COMP%]{width:30px;height:14px;border:1px solid #777;border-radius:5px;background-color:#eaeaf2}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .artificial-layer-key[_ngcontent-%COMP%]{border:1px solid purple;width:30px;height:14px;background-color:#f1f1f1}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .selected-node-key[_ngcontent-%COMP%]{width:30px;height:14px;border:2px solid #1a73e8;border-radius:5px;background-color:#c2e7ff}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .identical-group-key[_ngcontent-%COMP%]{width:30px;height:14px;border:1px solid #777;background-color:#e2edff}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .input-key[_ngcontent-%COMP%]{width:30px;height:14px;border-radius:5px;border:2px solid #009e73}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .output-key[_ngcontent-%COMP%]{width:30px;height:14px;border-radius:5px;border:2px solid #d55e00}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .shortcut[_ngcontent-%COMP%]{color:#777;font-size:10px;line-height:10px}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .divider[_ngcontent-%COMP%]{height:1px;border-top:1px solid #ccc;margin:8px 0 4px}"],
Ra:0});function lO(a,b){const c=b.clientX,d=b.clientY,e=b.offsetX,f=Number(a.C.style.top.replace("px","")),g=Number(a.C.style.left.replace("px","")),k=Number(a.C.style.width.replace("px","")),m=Number(a.C.style.height.replace("px","")),p=lk(document,"mousemove"),t=lk(window,"mouseup"),w=b.target.dataset.position,B=a.C.parentElement.offsetHeight,G=a.C.parentElement.offsetWidth;C(Pj([p]),Lo(t)).subscribe({next:([J])=>{J.preventDefault();var N=J.clientX-c;J=J.clientY-d;if(0!==N||0!==J)null==w?(a.C.style.top=
`${Math.min(B-28,Math.max(f+J,0))}px`,a.C.style.left=`${Math.min(G-e,Math.max(-e,g+N))}px`):(w.includes("right")&&(a.C.style.width=`${Math.max(48,k+N)}px`),w.includes("bottom")&&(a.C.style.height=`${Math.max(48,m+J)}px`),w.includes("left")&&(N=Math.max(48,k-N),a.C.style.width=`${N}px`,a.C.style.left=`${g+(k-N)}px`),w.includes("top")&&(N=Math.max(48,m-J),a.C.style.height=`${N}px`,a.C.style.top=`${f+(m-N)}px`))},complete:()=>{}})}var mO=class{constructor(a){this.C=a}};var nO=class{constructor(a){this.C=a;this.wb=new Fk;this.Ed=0}reset(){this.Ed=0;bn(this.C)}get BB(){return 0===this.Ed}get AB(){return this.Ed===Math.ceil(this.Un/this.He)-1}get sB(){const a=Math.min(this.Un,this.Ed*this.He+1),b=Math.min((this.Ed+1)*this.He,this.Un);return a===b?`${a}`:`${a} - ${b}`}};nO.K=function(a){return new (a||nO)(z(Sr))};
nO.Da=Gf({type:nO,ha:[["paginator"]],inputs:{He:"pageSize",Un:"itemsCount"},outputs:{wb:"change"},ka:!0,features:[Mr],za:9,Aa:5,Ca:[[1,"container"],[1,"icon-container","prev-page",3,"click"],[1,"range-text"],[1,"icon-container","next-page",3,"click"]],ta:function(a,b){a&1&&(R(0,"div",0)(1,"div",1),V("click",function(){b.Ed--;b.Ed=Math.max(0,b.Ed);b.wb.emit(b.Ed)}),R(2,"mat-icon"),Z(3,"chevron_left"),S()(),R(4,"div",2),Z(5),S(),R(6,"div",3),V("click",function(){b.Ed++;b.Ed=Math.min(Math.ceil(b.Un/
b.He)-1,b.Ed);b.wb.emit(b.Ed)}),R(7,"mat-icon"),Z(8,"chevron_right"),S()()());a&2&&(D(),nq("disabled",b.BB),D(4),Br(" ",b.sB," "),D(),nq("disabled",b.AB))},Ha:[ht,iI,hI,HG],styles:[".container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none}.container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;width:24px;height:24px;border-radius:99px;cursor:pointer}.container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]:hover{background-color:#ddd}.container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]:hover   mat-icon[_ngcontent-%COMP%]{opacity:1}.container[_ngcontent-%COMP%]   .icon-container.disabled[_ngcontent-%COMP%]{pointer-events:none;opacity:.3}.container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{opacity:.7;color:#333;font-size:20px;width:20px;height:20px}.container[_ngcontent-%COMP%]   .range-text[_ngcontent-%COMP%]{color:#333;font-size:12px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center}"],
Ra:0});const oO=["searchInput"],pO=["content"];function qO(a){if(a&1){const b=Ug();R(0,"div",11);V("click",function(){u(b);const c=Y();c.Ch.setValue("");rO(c,"");return x()});R(1,"mat-icon",12);Z(2,"close");S()()}}function sO(a){if(a&1){const b=Ug();R(0,"paginator",25);V("change",function(c){u(b);const d=Y(2);d.bh=c;tO(d);return x()});S()}a&2&&(a=Y(2),H("pageSize",a.He)("itemsCount",a.gs))}
function uO(a,b){if(a&1){const c=Ug();R(0,"div",26);V("click",function(){const d=u(c).da,e=Y(2);return x(vO(e,d))});R(1,"div",27)(2,"mat-icon");Z(3);S()();R(4,"div",28);Z(5);S()()}a&2&&(a=b.da,nq("selected",a.selected),D(3),Ar(a.Cl),D(2),Br(" ",a.label," "))}function wO(a){a&1&&Mq(0,"io-tree",22);a&2&&(a=Y(2),H("data",a.sv)("rendererId",a.rendererId)("tooltipPosition","right")("showLocator",!1))}function xO(a){a&1&&(R(0,"div",23),Z(1," No matching results "),S())}
function yO(a){a&1&&(R(0,"div",13)(1,"div",14)(2,"div",15)(3,"div",16),Z(4),S(),Ap(5,sO,1,2,"paginator",17),S(),R(6,"div",18)(7,"div",19),Jq(8,uO,6,4,"div",20,Hq),S()()(),R(10,"div",21,2),Ap(12,wO,1,4,"io-tree",22)(13,xO,2,0,"div",23),S(),R(14,"div",24)(15,"mat-icon"),Z(16,"tips_and_updates"),S(),Z(17," Click to locate node. Alt+Click to locate and select node. "),S()());a&2&&(a=Y(),lq("max-height",a.RD,"px"),D(4),Br(" ",a.SD," "),D(),Eq(a.gE?5:-1),D(3),Lq(a.ms),D(4),Eq(0<a.gs?12:13))}
function zO(a){a&1&&(R(0,"div",29),Z(1," No matching results "),S())}
function AO(a){a&1&&(R(0,"div",30),Z(1," Use regex to match nodes by label, attributes, input/output node label, and input/output metadata. "),Mq(2,"br")(3,"br"),Z(4," For attributes and I/O metadata, regex will try to match "),R(5,"span",31),Z(6),S(),Z(7," or "),R(8,"span",31),Z(9),S(),Z(10,". "),Mq(11,"br")(12,"br"),Z(13," Examples:"),Mq(14,"br"),R(15,"ul")(16,"li")(17,"span",32),Z(18,"axis:"),S(),Z(19," will match attributes whose key is "),R(20,"span",31),Z(21,"axis"),S(),Z(22,". "),S(),R(23,
"li")(24,"span",32),Z(25,"padding=[SAME|VALID]"),S(),Z(26," will match attributes whose key is "),R(27,"span",31),Z(28,"padding"),S(),Z(29," and value is either "),R(30,"span",31),Z(31,"SAME"),S(),Z(32," or "),R(33,"span",31),Z(34,"VALID"),S(),Z(35,". "),S()()());a&2&&(D(6),Dr("","{","key","}",":","{","value","}",""),D(3),Dr("","{","key","}","=","{","value","}",""))}function vO(a,b){b.selected=!b.selected;bn(a.J);rO(a,a.Ch.value||"");setTimeout(()=>{a.content.R.scrollTop=0})}
function rO(a,b){if(b){var c=[],d=[],e={results:{}},f=new Set(a.ms.filter(g=>g.selected).map(g=>g.Cl));try{const g=new RegExp(b,"i");for(const k of a.ga.nodes){if(nI(k)&&k.hideInLayout)continue;const {matches:m,matchTypes:p}=JI(f,g,k,a.ga);0<m.length&&(c.push(k),d.push({matches:m,matchTypes:p}),e.results[k.id]=m)}MJ(a.D,a.rendererId,e)}catch(g){console.warn("Failed to search",g)}a.bh=0;a.C=[...c];a.N=d;tO(a);a.F=!1;bn(a.J)}else a.bh=0,a.sv=void 0,a.C=[],bn(a.J),a.F=!1,NJ(a.D,a.rendererId)}
function tO(a){a.sv=$K(a.C.slice(a.bh*a.He,(a.bh+1)*a.He),"incoming",void 0,a.N.slice(a.bh*a.He,(a.bh+1)*a.He))}
var BO=class{constructor(a,b,c){this.D=a;this.J=b;this.G=c;this.Ch=new vx("");this.ms=[{Cl:"title",label:"Label",selected:!0},{Cl:"list",label:"Attrs",selected:!0},{Cl:"input",label:"Inputs",selected:!0},{Cl:"output",label:"Outputs",selected:!0}];this.He=this.D.C?12:50;this.C=[];this.N=[];this.F=!1;this.bh=0;C(this.Ch.ah,Po(()=>{this.F=!0;bn(this.J)}),On(300),sL(this.G)).subscribe(d=>{rO(this,((null==d?void 0:d.toLowerCase())||"").trim())});C(this.D.wa,sL(this.G)).subscribe(()=>{let d;this.rendererId===
(null==(d=this.D.G())?void 0:d.id)&&this.QD.R.focus()})}get SD(){let a;const b=(null==(a=this.C)?void 0:a.length)||0;return`${b} result${1===b?"":"s"}`}get WD(){return""!==(this.Ch.value||"").trim()}get hE(){return!this.F&&""!==(this.Ch.value||"").trim()&&!this.Lx}get Lx(){return!this.F&&""!==(this.Ch.value||"").trim()&&0===this.C.length&&this.ms.every(a=>a.selected)}get gs(){return this.C.length}get RD(){return document.body.offsetHeight-300}get gE(){return this.gs>this.He}};
BO.K=function(a){return new (a||BO)(z(VJ),z(Sr),z(yi))};
BO.Da=Gf({type:BO,ha:[["search-bar"]],fb:function(a,b){a&1&&(ur(oO,5),ur(pO,5));if(a&2){let c;vr(c=wr())&&(b.QD=c.first);vr(c=wr())&&(b.content=c.first)}},inputs:{ga:"curModelGraph",rendererId:"rendererId"},ka:!0,features:[Mr],za:14,Aa:6,Ca:[["searchInput",""],["helpPopup",""],["content",""],[1,"container"],[1,"input-container"],[1,"search"],["placeholder","Search nodes/layers by regex",3,"formControl"],["class","btn-clear-container",3,"click",4,"ngIf"],["dialogLabel","help popup",1,"icon-container",
3,"xapInlineDialog","hoverDelayMs"],["class","search-results-container",3,"max-height",4,"ngIf"],["class","no-matches",4,"ngIf"],[1,"btn-clear-container",3,"click"],[1,"clear"],[1,"search-results-container"],[1,"title-container"],[1,"title"],[1,"title-label"],[3,"pageSize","itemsCount"],[1,"search-result-type-selector-container"],[1,"options-container"],[1,"option-container",3,"selected"],[1,"content"],[3,"data","rendererId","tooltipPosition","showLocator"],[1,"empty-results"],[1,"footer"],[3,"change",
"pageSize","itemsCount"],[1,"option-container",3,"click"],[1,"icon-container"],[1,"option-label"],[1,"no-matches"],[1,"model-explorer-search-help-popup"],[1,"code"],[1,"code","regex"]],ta:function(a,b){a&1&&(R(0,"div",3)(1,"div",4)(2,"mat-icon",5),Z(3,"search"),S(),Mq(4,"input",6,0),Ap(6,qO,3,0,"div",7),R(7,"div",8)(8,"mat-icon"),Z(9,"help_outline"),S()()(),Ap(10,yO,18,5,"div",9)(11,zO,2,0,"div",10),S(),Ap(12,AO,36,8,"ng-template",null,1,Qr));a&2&&(a=zr(13),D(4),H("formControl",b.Ch),D(2),H("ngIf",
b.WD),D(),H("xapInlineDialog",a)("hoverDelayMs",100),D(3),H("ngIf",b.hE),D(),H("ngIf",b.Lx))},Ha:[ht,ct,ZK,OB,RD,iI,hI,nO,Cx,sw,Mw,xx,HG,EG],styles:[".container[_ngcontent-%COMP%]{position:relative;overflow:visible;background-color:#fff;border-radius:99px;height:30px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;padding:0 8px}.container[_ngcontent-%COMP%]   .input-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;width:250px}.container[_ngcontent-%COMP%]   .input-container[_ngcontent-%COMP%]   mat-icon.clear[_ngcontent-%COMP%], .container[_ngcontent-%COMP%]   .input-container[_ngcontent-%COMP%]   mat-icon.search[_ngcontent-%COMP%]{font-size:20px;width:20px;height:20px}.container[_ngcontent-%COMP%]   .input-container[_ngcontent-%COMP%]   mat-icon.clear[_ngcontent-%COMP%]{opacity:.6;cursor:pointer}.container[_ngcontent-%COMP%]   .input-container[_ngcontent-%COMP%]   mat-icon.clear[_ngcontent-%COMP%]:hover{opacity:.9}.container[_ngcontent-%COMP%]   .input-container[_ngcontent-%COMP%]   input[_ngcontent-%COMP%]{height:24px;border:none;outline:none;-moz-box-sizing:border-box;box-sizing:border-box;padding:0 4px;font-size:12px;-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1}.container[_ngcontent-%COMP%]   .input-container[_ngcontent-%COMP%]   .btn-clear-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center}.container[_ngcontent-%COMP%]   .input-container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;cursor:pointer;color:#999;opacity:.8}.container[_ngcontent-%COMP%]   .input-container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]:hover{opacity:1}.container[_ngcontent-%COMP%]   .input-container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{font-size:20px;width:20px;height:20px}.container[_ngcontent-%COMP%]   .input-container[_ngcontent-%COMP%]:has(input:focus)   mat-icon.search[_ngcontent-%COMP%]{color:#1a73e8}.container[_ngcontent-%COMP%]   .search-results-container[_ngcontent-%COMP%]{overflow:hidden;position:absolute;top:calc(100% + 12px);left:-4px;border:1px solid #ccc;border-radius:4px;width:380px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;background-color:#fff;box-shadow:0 4px 6px -1px rgba(0,0,0,.1),0 2px 4px -2px rgba(0,0,0,.1)}.container[_ngcontent-%COMP%]   .search-results-container[_ngcontent-%COMP%]   .title-container[_ngcontent-%COMP%]{background-color:#f6f6f6;border-bottom:1px solid #ddd;-moz-box-sizing:border-box;box-sizing:border-box;padding:0 10px 10px}.container[_ngcontent-%COMP%]   .search-results-container[_ngcontent-%COMP%]   .title[_ngcontent-%COMP%]{height:28px;min-height:28px;font-size:12px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between}.container[_ngcontent-%COMP%]   .search-results-container[_ngcontent-%COMP%]   .title[_ngcontent-%COMP%]   .title-label[_ngcontent-%COMP%]{white-space:nowrap}.container[_ngcontent-%COMP%]   .search-results-container[_ngcontent-%COMP%]   .content[_ngcontent-%COMP%]{-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1;min-height:0;overflow-y:auto;padding-bottom:6px}.container[_ngcontent-%COMP%]   .search-results-container[_ngcontent-%COMP%]   .content[_ngcontent-%COMP%]   .empty-results[_ngcontent-%COMP%]{padding:12px 8px 6px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;font-size:12px;color:#ccc}.container[_ngcontent-%COMP%]   .search-results-container[_ngcontent-%COMP%]   .search-result-type-selector-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;font-size:12px;margin-top:2px}.container[_ngcontent-%COMP%]   .search-results-container[_ngcontent-%COMP%]   .search-result-type-selector-container[_ngcontent-%COMP%]   .options-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;gap:6px}.container[_ngcontent-%COMP%]   .search-results-container[_ngcontent-%COMP%]   .search-result-type-selector-container[_ngcontent-%COMP%]   .option-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;cursor:pointer;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;border:1px solid #ccc;padding:4px 14px;border-radius:6px;background-color:#fff;color:#474747}.container[_ngcontent-%COMP%]   .search-results-container[_ngcontent-%COMP%]   .search-result-type-selector-container[_ngcontent-%COMP%]   .option-container.selected[_ngcontent-%COMP%]{border-color:#a3c9ff;background-color:#a3c9ff;color:#001d35}.container[_ngcontent-%COMP%]   .search-results-container[_ngcontent-%COMP%]   .search-result-type-selector-container[_ngcontent-%COMP%]   .option-container.selected[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{color:#001d35}.container[_ngcontent-%COMP%]   .search-results-container[_ngcontent-%COMP%]   .search-result-type-selector-container[_ngcontent-%COMP%]   .option-container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]{width:18px;height:18px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;-moz-box-sizing:border-box;box-sizing:border-box;margin-right:4px}.container[_ngcontent-%COMP%]   .search-results-container[_ngcontent-%COMP%]   .search-result-type-selector-container[_ngcontent-%COMP%]   .option-container[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{font-size:16px;width:16px;height:16px;color:#474747}.container[_ngcontent-%COMP%]   .search-results-container[_ngcontent-%COMP%]   .search-result-type-selector-container[_ngcontent-%COMP%]   .option-container[_ngcontent-%COMP%]   .option-label[_ngcontent-%COMP%]{font-size:12px;font-family:Google Sans Text,Arial,Helvetica,sans-serif;letter-spacing:normal}.container[_ngcontent-%COMP%]   .footer[_ngcontent-%COMP%]{height:32px;min-height:32px;background-color:#f6f6f6;border-top:1px solid #ddd;-moz-box-sizing:border-box;box-sizing:border-box;padding:0 10px;font-size:12px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;color:#777}.container[_ngcontent-%COMP%]   .footer[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{font-size:16px;height:16px;width:16px;color:#777;margin-right:4px}.container[_ngcontent-%COMP%]   .no-matches[_ngcontent-%COMP%]{position:absolute;top:calc(100% + 12px);left:-4px;color:#999;padding:8px;border:1px solid #ccc;border-radius:4px;background-color:#fff;font-size:12px;box-shadow:0 4px 6px -1px rgba(0,0,0,.1),0 2px 4px -2px rgba(0,0,0,.1)}  .model-explorer-search-help-popup{padding:12px;font-size:12px;background-color:#fff}  .model-explorer-search-help-popup ul{-webkit-margin-after:0;margin-block-end:0}  .model-explorer-search-help-popup .code{display:inline-block;background-color:#fffdd0;font-family:monospace}  .model-explorer-search-help-popup .code.regex{background-color:#e6d0ff}"],
Ra:0});const CO=["dialog"],DO=["trigger"];function EO(a){a&1&&Z(0," Maximum state count reached ")}function FO(a){a&1&&Z(0," Save the current graph state ")}function GO(a){a&1&&(R(0,"div",7),Ap(1,EO,1,0)(2,FO,1,0),R(3,"div",8),Z(4," The saved state can be restored by clicking the corresponding state number. You can save up to 9 states. "),S()());a&2&&(a=Y(),D(),Eq(a.tq?1:2))}
function HO(a){if(a&1){const c=Ug();R(0,"div",12)(1,"div",13)(2,"div");Z(3);S();R(4,"div",14);V("click",function(){u(c);const d=Y().index,e=Y(2);e.ga&&IJ(e.ak,d,e.ga.id,e.paneId);return x()});R(5,"mat-icon");Z(6,"delete");S();Z(7," Delete ");S()();Mq(8,"canvas");S()}if(a&2){var b=Y();a=b.da;b=b.index;Y(2);D(3);Br("Shortcut key: ",b+1,"");D(5);lq("width",IO(a,!0),"px")("height",JO(a,!0),"px")}}
function KO(a,b){if(a&1){const c=Ug();R(0,"div",11,1);V("opened",function(){const d=u(c).da;Y(2);const e=document.querySelector(".model-explorer-snapshot-popup canvas"),f=IO(d),g=JO(d);e.width=f;e.height=g;const k=e.getContext("2d");k.imageSmoothingQuality="high";k.drawImage(d.Uq,0,0,f,g);e.classList.add("show");return x()})("click",function(){const d=u(c).index,e=Y(2);return x(LO(e,d))});Z(3);Ap(4,HO,9,5,"ng-template",null,2,Qr);S()}if(a&2){a=b.index;b=zr(5);const c=Y(2);H("xapInlineDialog",b)("overlaySize",
c.oE)("overlayPositions",c.nE)("hoverDelayMs",150);D(3);Br(" ",a+1," ")}}function MO(a){a&1&&(R(0,"div",9),Ap(1,KO,6,5,"div",10),S());a&2&&(a=Y(),D(),H("ngForOf",a.bj)("ngForTrackBy",a.FE))}const NO=new Set("123456789".split(""));function IO(a,b=!1){return a.Uq.width/(b?window.devicePixelRatio:1)}function JO(a,b=!1){return a.Uq.height/(b?window.devicePixelRatio:1)}function LO(a,b){const c=a.bj[b];let d;null==(d=a.Vx.get(b))||rG(d);a.ak.aa.next({kb:c,rendererId:a.rendererId})}
var OO=class{constructor(a,b){this.ak=a;this.C=b;this.Vx=new ar;this.Wx=new ar;this.hd={minWidth:0,minHeight:0,maxWidth:340};this.oE={maxWidth:1E3,maxHeight:1E3};this.nE=[{cb:"start",eb:"bottom",Na:"start",Ta:"top",offsetY:12}];this.bj=[];Js(()=>{const c=nJ(this.ak,this.paneId);this.ga=null==c?void 0:c.modelGraph;null!=(null==c?void 0:c.modelGraph)&&(this.bj=((null==c?void 0:c.Rg)||{})[c.modelGraph.id]||[],bn(this.C))})}Oq(a){let b;if(NO.has(a.key)&&(null==(b=this.ak.G())?void 0:b.id)===this.rendererId&&
!yI()&&(a=Number(a.key)-1,a<=this.bj.length-1)){LO(this,a);let c;const d=null==(c=this.Wx.get(a))?void 0:c.R;d&&(d.classList.add("clicked"),setTimeout(()=>{d.classList.remove("clicked")},50))}}FE(a,b){return b.id}get tq(){return 9<=this.bj.length}get dC(){return 0<this.bj.length}};OO.K=function(a){return new (a||OO)(z(VJ),z(Sr))};
OO.Da=Gf({type:OO,ha:[["snapshot-manager"]],fb:function(a,b){a&1&&(ur(CO,5),ur(DO,5));if(a&2){let c;vr(c=wr())&&(b.Vx=c);vr(c=wr())&&(b.Wx=c)}},Ia:function(a,b){a&1&&V("keypress",function(c){return b.Oq(c)},!1,Ci)},inputs:{paneId:"paneId",rendererId:"rendererId"},ka:!0,features:[Mr],za:7,Aa:6,Ca:[["help",""],["dialog","xapInlineDialog","trigger",""],["snapshotPopup",""],[1,"container"],["dialogLabel","help",1,"mat-icon-container","take-snapshot",3,"click","xapInlineDialog","overlaySize","hoverDelayMs"],
[1,"toolbar-icon"],["class","snapshots-container",4,"ngIf"],[1,"model-explorer-help-popup"],[1,"description"],[1,"snapshots-container"],["class","snapshot","dialogLabel","snapshot popup",3,"xapInlineDialog","overlaySize","overlayPositions","hoverDelayMs","opened","click",4,"ngFor","ngForOf","ngForTrackBy"],["dialogLabel","snapshot popup",1,"snapshot",3,"opened","click","xapInlineDialog","overlaySize","overlayPositions","hoverDelayMs"],[1,"model-explorer-snapshot-popup"],[1,"info-row"],[1,"icon-container",
"delete",3,"click"]],ta:function(a,b){if(a&1){const c=Ug();R(0,"div",3)(1,"div",4);V("click",function(){u(c);b.tq||b.ak.ja.next({rendererId:b.rendererId});return x()});R(2,"mat-icon",5);Z(3,"bookmark_add");S()();Ap(4,GO,5,1,"ng-template",null,0,Qr)(6,MO,2,2,"div",6);S()}a&2&&(a=zr(5),D(),nq("disabled",b.tq),H("xapInlineDialog",a)("overlaySize",b.hd)("hoverDelayMs",10),D(5),H("ngIf",b.dC))},Ha:[ht,Zs,ct,hG,iI,hI,HG,EG],styles:[".container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;margin-right:3px}.container[_ngcontent-%COMP%]   .mat-icon-container[_ngcontent-%COMP%]{width:24px;height:24px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;cursor:pointer}.container[_ngcontent-%COMP%]   .mat-icon-container.disabled[_ngcontent-%COMP%]{cursor:default;opacity:.5}.container[_ngcontent-%COMP%]   .mat-icon-container.disabled[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]:hover{opacity:.6}.container[_ngcontent-%COMP%]   mat-icon.toolbar-icon[_ngcontent-%COMP%]{font-size:20px;width:20px;height:20px;opacity:.6}.container[_ngcontent-%COMP%]   mat-icon.toolbar-icon[_ngcontent-%COMP%]:hover{opacity:.9}.container[_ngcontent-%COMP%]   .snapshots-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;font-size:11px;margin-left:6px;margin-right:6px;gap:2px}.container[_ngcontent-%COMP%]   .snapshots-container[_ngcontent-%COMP%]   .snapshot[_ngcontent-%COMP%]{width:16px;height:16px;border-radius:8px;background-color:#7cacf8;color:#fff;-moz-box-sizing:border-box;box-sizing:border-box;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;font-weight:500;cursor:pointer}.container[_ngcontent-%COMP%]   .snapshots-container[_ngcontent-%COMP%]   .snapshot.clicked[_ngcontent-%COMP%], .container[_ngcontent-%COMP%]   .snapshots-container[_ngcontent-%COMP%]   .snapshot[_ngcontent-%COMP%]:hover{background-color:#0b57d0;color:#fff}  .model-explorer-snapshot-popup{font-size:11px;line-height:12px;background-color:#fff}  .model-explorer-snapshot-popup .info-row{color:#333;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between;background-color:#f6f6f6;padding:4px 8px;border-bottom:1px solid #ddd}  .model-explorer-snapshot-popup .info-row .icon-container{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;cursor:pointer;opacity:.8}  .model-explorer-snapshot-popup .info-row .icon-container:hover{opacity:1}  .model-explorer-snapshot-popup .info-row .icon-container mat-icon{font-size:18px;width:18px;height:19px}  .model-explorer-snapshot-popup canvas{opacity:0;-webkit-transition:opacity .2s ease-out;transition:opacity .2s ease-out}  .model-explorer-snapshot-popup canvas.show{opacity:1}"],
Ra:0});function PO(a){a&1&&(R(0,"mat-icon",2),Z(1,"chevron_right"),S())}
function QO(a,b){if(a&1){const c=Ug();R(0,"div",1);V("click",function(){var d=u(c).Cb;const e=Y();if(d!==e.Mf.length-1){LJ(e.ti,e.paneId,d);var f=e.Mf[d];if(d=f.kb){var g;let k;f.Ag===(null==(g=nJ(e.ti,e.paneId))?void 0:null==(k=g.modelGraph)?void 0:k.id)?e.ti.aa.next({rendererId:e.paneId,kb:d}):(g=JJ(e.ti,f.Ag))&&qJ(e.ti,g,d.flattenLayers,d)}}return x()});Z(1);S();Ap(2,PO,2,0,"mat-icon",2)}if(a&2){a=b.da;b=b.Cb;const c=Y();nq("current",b===c.Mf.length-1);D();Br(" ",a.Ag," ");D();Eq(b!==c.Mf.length-
1?2:-1)}}var RO=class{constructor(a,b){this.ti=a;this.C=b;this.Mf=[];Js(()=>{let c;const d=null==(c=nJ(this.ti,this.paneId))?void 0:c.Zf;d!==this.D&&(this.Mf=(this.D=d)||[],bn(this.C))})}};RO.K=function(a){return new (a||RO)(z(VJ),z(Sr))};
RO.Da=Gf({type:RO,ha:[["subgraph-breadcrumbs"]],inputs:{paneId:"paneId"},ka:!0,features:[Mr],za:3,Aa:0,Ca:[[1,"container"],[1,"item",3,"click"],[1,"separator"]],ta:function(a,b){a&1&&(R(0,"div",0),Jq(1,QO,3,4,null,null,Hq),S());a&2&&(D(),Lq(b.Mf))},Ha:[ht,iI,hI,gG],styles:[".container[_ngcontent-%COMP%]{border:1px solid #3876c7;background-color:#d5e7ff;border-radius:4px;-moz-box-sizing:border-box;box-sizing:border-box;padding:4px 8px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;font-size:13px}.container[_ngcontent-%COMP%]   .item[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;color:#777}.container[_ngcontent-%COMP%]   .item[_ngcontent-%COMP%]:hover:not(.current){cursor:pointer;text-decoration:underline}.container[_ngcontent-%COMP%]   .item.current[_ngcontent-%COMP%]{color:#000}.container[_ngcontent-%COMP%]   .separator[_ngcontent-%COMP%]{color:#999;font-size:16px;width:16px;height:16px;margin:0 3px}"],
Ra:0});const SO=(a,b)=>b.type;function TO(a){a&1&&(R(0,"div",7),Z(1," Select data to view on nodes / edges "),S())}
function UO(a){if(a&1){const b=Ug();R(0,"div",14)(1,"input",15,3);V("keydown.enter",function(){u(b);const c=zr(2);return x(c.blur())})("input",function(){u(b);const c=zr(2),d=Y(3);return x(d.gn=c.value)})("change",function(){u(b);const c=Y().da,d=Y(2);QJ(d.Ne,d.paneId,d.rendererId,c.type,d.gn);VO(d);return x()});S();R(3,"div",16)(4,"mat-icon");Z(5,"help_outline");S()();R(6,"div",17);V("click",function(){u(b);const c=zr(2);return x(c.blur())});R(7,"mat-icon");Z(8,"done");S()()()}if(a&2){a=Y().da;const b=
Y(2),c=zr(9);D();H("disabled",!a.selected)("value",b.gn);D(2);H("xapInlineDialog",c)("hoverDelayMs",100)}}function WO(a,b){if(a&1){const c=Ug();R(0,"div",12);V("click",function(){const d=u(c).da,e=Y(2);OJ(e.Ne,e.paneId,e.rendererId,d.type);VO(e);return x()});Mq(1,"input",13);Z(2);S();Ap(3,UO,9,4,"div",14)}a&2&&(a=b.da,Y(2),D(),H("checked",a.selected),D(),Br(" ",a.type," "),D(),Eq("Op node attributes"===a.type?3:-1))}
function XO(a,b){if(a&1){const c=Ug();R(0,"div",12);V("click",function(){var d=u(c).da;const e=Y(2);PJ(e.Ne,e.paneId,e.rendererId,d.type);e.Qb||e.Ne.C||(d=UJ(e.Ne,e.paneId,e.rendererId),PI("model_explorer_show_on_edge_item_types_v2",JSON.stringify(d)));return x()});Mq(1,"input",13);Z(2);S()}a&2&&(a=b.da,D(),H("checked",a.selected),D(),Br(" ",a.type," "))}
function YO(a){a&1&&(R(0,"div",8)(1,"div",9)(2,"div"),Z(3,"View on nodes"),S(),R(4,"div",10)(5,"mat-icon"),Z(6,"close"),S()()(),Jq(7,WO,4,3,null,null,SO),R(9,"div",9)(10,"div"),Z(11,"View on edges"),S()(),Jq(12,XO,3,2,"div",11,SO),S());a&2&&(a=Y(),D(7),Lq(a.Ox),D(5),Lq(a.Nx))}
function ZO(a){a&1&&(R(0,"div",18),Z(1," Use regex to match "),R(2,"span",19),Z(3),S(),Z(4," or "),R(5,"span",19),Z(6),S(),Z(7,". "),Mq(8,"br")(9,"br"),Z(10," Examples:"),Mq(11,"br"),R(12,"ul")(13,"li")(14,"span",20),Z(15,"axis:"),S(),Z(16," will match attributes whose key is "),R(17,"span",19),Z(18,"axis"),S(),Z(19,". "),S(),R(20,"li")(21,"span",20),Z(22,"padding=[SAME|VALID]"),S(),Z(23," will match attributes whose key is "),R(24,"span",19),Z(25,"padding"),S(),Z(26," and value is either "),R(27,
"span",19),Z(28,"SAME"),S(),Z(29," or "),R(30,"span",19),Z(31,"VALID"),S(),Z(32,". "),S()()());a&2&&(D(3),Dr("","{","key","}",":","{","value","}",""),D(3),Dr("","{","key","}","=","{","value","}",""))}const $O="Op node id;Op node attributes;Op node inputs;Op node outputs;Layer node children count;Layer node descendants count".split(";"),aP=["Tensor shape"];
function VO(a){if(!a.Qb&&!a.Ne.C){a=kJ(a.Ne,a.paneId,a.rendererId);const b=Object.keys(a).filter(c=>c.startsWith("Node data provider: "));for(const c of b)delete a[c];PI("model_explorer_show_on_node_item_types_v2",JSON.stringify(a))}}
var bP=class{constructor(a,b,c,d){this.Ne=a;this.D=b;this.F=d;this.Qb=!1;this.G=Bs(()=>{const e=yJ(this.Ne,this.paneId);return(e?Object.values(vL(this.F,e)):[]).map(f=>f.runName)});this.C=[];this.hd={minWidth:0,minHeight:0};this.TE={minWidth:280,minHeight:0};this.Ox=[];this.Nx=[];this.gn="";this.hf=!1;Js(()=>{var e;const f=(null==(e=nJ(this.Ne,this.paneId))?void 0:e.showOnNodeItemTypes)||{};e=this.G();if(f!==this.N||JSON.stringify(e)!==JSON.stringify(this.C)){this.C=e;this.N=f;e=[];for(var g of $O){let k;
const m={type:g,selected:null==(k=(f[this.rendererId]||{})[g])?void 0:k.selected};e.push(m);if("Op node attributes"===g){let p;m.filterRegex=(null==(p=(f[this.rendererId]||{})[g])?void 0:p.filterRegex)||"";this.gn=m.filterRegex}}for(const k of this.C){g=`${"Node data provider: "}${k}`;let m;e.push({type:g,selected:null==(m=(f[this.rendererId]||{})[g])?void 0:m.selected})}this.Ox=e;bn(this.D)}});Js(()=>{var e;const f=(null==(e=nJ(this.Ne,this.paneId))?void 0:e.Wb)||{};if(f!==this.J){this.J=f;e=[];
for(const g of aP){let k;const m={type:g,selected:null==(k=(f[this.rendererId]||{})[g])?void 0:k.selected};e.push(m)}this.Nx=e;bn(this.D)}})}};bP.K=function(a){return new (a||bP)(z(VJ),z(Sr),z(QI),z(KL))};
bP.Da=Gf({type:bP,ha:[["view-on-node"]],fb:function(a,b){a&1&&ur(GG,5);if(a&2){let c;vr(c=wr())&&(b.yD=c.first)}},inputs:{rendererId:"rendererId",paneId:"paneId",Qb:"inPopup"},ka:!0,features:[Mr],za:10,Aa:5,Ca:[["help",""],["viewPopup",""],["helpPopup",""],["input",""],["dialogLabel","help",1,"container",3,"xapInlineDialog","overlaySize","hoverDelayMs"],["dialogLabel","view popup",1,"mat-icon-container","view",3,"opened","closed","click","xapInlineDialogClick","overlaySize"],[1,"toolbar-icon"],[1,
"model-explorer-help-popup"],[1,"model-explorer-view-popup"],[1,"label"],["xapInlineDialogClose","",1,"icon-container","close"],[1,"show-on-node-item"],[1,"show-on-node-item",3,"click"],["type","checkbox",3,"checked"],[1,"show-on-node-filter"],["placeholder","Filter by regex",1,"input-attrs-filter",3,"keydown.enter","input","change","disabled","value"],["dialogLabel","help popup",1,"icon-container",3,"xapInlineDialog","hoverDelayMs"],[1,"icon-container","ok",3,"click"],[1,"model-explorer-view-on-node-help-popup"],
[1,"code"],[1,"code","regex"]],ta:function(a,b){if(a&1){const c=Ug();R(0,"div",4)(1,"div",5);V("opened",function(){u(c);return x(b.hf=!0)})("closed",function(){u(c);return x(b.hf=!1)})("click",function(){u(c);b.hf&&rG(b.yD);return x()});R(2,"mat-icon",6);Z(3,"visibility");S()()();Ap(4,TO,2,0,"ng-template",null,0,Qr)(6,YO,14,0,"ng-template",null,1,Qr)(8,ZO,33,8,"ng-template",null,2,Qr)}if(a&2){a=zr(5);const c=zr(7);H("xapInlineDialog",a)("overlaySize",b.hd)("hoverDelayMs",10);D();H("xapInlineDialogClick",
c)("overlaySize",b.TE)}},Ha:[ht,hG,iI,hI,HG,EG,GG],styles:[".container[_ngcontent-%COMP%]   .mat-icon-container[_ngcontent-%COMP%]{width:20px;height:20px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;cursor:pointer;opacity:.6}.container[_ngcontent-%COMP%]   .mat-icon-container[_ngcontent-%COMP%]:hover{opacity:.9}.container[_ngcontent-%COMP%]   .mat-icon-container[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{font-size:20px;width:20px;height:20px}  .model-explorer-view-popup{padding:12px;padding-top:10px;font-size:12px;background-color:#fff;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column}  .model-explorer-view-popup .label{font-weight:500;font-size:11px;text-transform:uppercase;letter-spacing:.0727em;margin-bottom:6px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between}  .model-explorer-view-popup .label:not(:first-child){margin-top:12px}  .model-explorer-view-popup .label .icon-container{cursor:pointer;opacity:.7;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center}  .model-explorer-view-popup .label .icon-container:hover{opacity:1}  .model-explorer-view-popup .label mat-icon{font-size:20px;width:20px;height:20px;color:#999}  .model-explorer-view-popup .show-on-node-item{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;gap:4px;cursor:pointer;color:#555;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none}  .model-explorer-view-popup .show-on-node-item input{cursor:pointer}  .model-explorer-view-popup .show-on-node-filter{margin:1px 0 4px 25px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;position:relative}  .model-explorer-view-popup .show-on-node-filter:focus-within .icon-container.ok{visibility:visible}  .model-explorer-view-popup .show-on-node-filter input{-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1;-moz-box-sizing:border-box;box-sizing:border-box;border:1px solid #aaa;border-radius:3px;padding:3px 26px 3px 4px;min-width:200px}  .model-explorer-view-popup .show-on-node-filter input::-webkit-input-placeholder{color:#aaa}  .model-explorer-view-popup .show-on-node-filter input::-moz-placeholder{color:#aaa}  .model-explorer-view-popup .show-on-node-filter input:-ms-input-placeholder{color:#aaa}  .model-explorer-view-popup .show-on-node-filter input::-ms-input-placeholder{color:#aaa}  .model-explorer-view-popup .show-on-node-filter input::placeholder{color:#aaa}  .model-explorer-view-popup .show-on-node-filter .icon-container{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;opacity:.8;cursor:pointer;margin-left:4px}  .model-explorer-view-popup .show-on-node-filter .icon-container:hover{opacity:1}  .model-explorer-view-popup .show-on-node-filter .icon-container.ok{position:absolute;right:26px;visibility:hidden}  .model-explorer-view-popup .show-on-node-filter .icon-container mat-icon{font-size:16px;width:16px;height:16px;color:#999}  .model-explorer-view-on-node-help-popup{padding:12px;font-size:12px;background-color:#fff}  .model-explorer-view-on-node-help-popup ul{-webkit-margin-after:0;margin-block-end:0}  .model-explorer-view-on-node-help-popup .code{display:inline-block;background-color:#fffdd0;font-family:monospace}  .model-explorer-view-on-node-help-popup .code.regex{background-color:#e6d0ff}  .cdk-overlay-container:has(.model-explorer-view-popup) .cdk-overlay-backdrop{pointer-events:none}"],
Ra:0});const cP=["webglRenderer"];function dP(a){a&1&&(Nq(0),Mq(1,"search-bar",21)(2,"div",15),Oq());a&2&&(a=Y(2),D(),H("curModelGraph",a.modelGraph)("rendererId",a.rendererId))}function eP(a){a&1&&(R(0,"div",22),Z(1," Zoom to fit "),R(2,"div",23)(3,"mat-icon"),Z(4,"keyboard"),S(),Z(5," Shortcut: SPACE "),S()())}function fP(a){a&1&&(R(0,"div",22),Z(1," Expand all layers in the graph "),S())}function gP(a){a&1&&(R(0,"div",22),Z(1," Collapse all layers in the graph "),S())}
function hP(a){if(a&1){const b=Ug();R(0,"div",24);V("click",function(){u(b);const c=Y(2);c.Zc.ca.next({Ov:!0,rendererId:c.rendererId});return x()});R(1,"mat-icon",25);Z(2,"unfold_more_double");S()();Ap(3,fP,2,0,"ng-template",null,3,Qr);R(5,"div",26);V("click",function(){u(b);const c=Y(2);c.Zc.ca.next({Ov:!1,rendererId:c.rendererId});return x()});R(6,"mat-icon",27);Z(7,"unfold_less_double");S()();Ap(8,gP,2,0,"ng-template",null,4,Qr);Mq(10,"div",15)}if(a&2){a=zr(4);const b=zr(9),c=Y(2);nq("disable",
c.Hv);H("xapInlineDialog",a)("overlaySize",c.hd)("hoverDelayMs",10);D(5);nq("disable",c.Hv);H("xapInlineDialog",b)("overlaySize",c.hd)("hoverDelayMs",10)}}function iP(a){a&1&&(R(0,"div",22),Z(1," Flatten all layers in the graph "),R(2,"div",30),Z(3," Nodes will be laid out without any layers data "),S()())}
function jP(a){if(a&1){const b=Ug();R(0,"div",28);V("click",function(){u(b);const c=Y(2);let d,e;const f=null==(d=nJ(c.Zc,c.paneId))?void 0:null==(e=d.Qg)?void 0:e.nodeId;null!=f&&oI(c.modelGraph.nodesById[f])&&wJ(c.Zc,c.paneId);tJ(c.Zc,c.paneId);mJ(c.Zc,c.paneId,uJ(c.Zc,c.paneId));c.Zc.N.set(void 0);return x()});R(1,"mat-icon",29);Z(2,"layers_clear");S()();Mq(3,"div",15);Ap(4,iP,4,0,"ng-template",null,5,Qr)}if(a&2){a=zr(5);const b=Y(2);nq("enabled",b.RB());H("xapInlineDialog",a)("overlaySize",b.hd)("hoverDelayMs",
10)}}function kP(a){a&1&&(R(0,"mat-icon",19),Z(1," multiple_stop "),S())}function lP(a){a&1&&(R(0,"mat-icon",20),Z(1," multiple_stop "),S())}function mP(a){a&1&&(R(0,"span"),Z(1,"Disable inputs/outputs tracing"),S())}function nP(a){a&1&&(R(0,"span"),Z(1,"Enable inputs/outputs tracing"),S())}function oP(a){a&1&&(R(0,"div",22),Ap(1,mP,2,0,"span")(2,nP,2,0,"span"),R(3,"div",30),Z(4," Highlight selected op node's ancestors and descendants and dim the rest "),S()());a&2&&(a=Y(2),D(),Eq(a.Wg?1:2))}
function pP(a){a&1&&(R(0,"div",22),Z(1," Download graph as PNG "),R(2,"div",30),Z(3," The maximum PNG size is limited to 5000 x 5000 "),S()())}
function qP(a){if(a&1){const b=Ug();Mq(0,"div",15);R(1,"div",31);V("menuOpened",function(){u(b);const c=Y(2);return x(c.vq=!0)})("menuClosed",function(){u(b);const c=Y(2);return x(c.vq=!1)});R(2,"mat-icon",32);Z(3," image ");S()();Ap(4,pP,4,0,"ng-template",null,6,Qr);R(6,"mat-menu",33,7)(8,"div",34);V("click",function(){u(b);const c=Y(2);return x(rP(c,!1))});Z(9," Graph in current viewport ");S();R(10,"div",34);V("click",function(){u(b);const c=Y(2);return x(rP(c,!0))});Z(11," Full graph ");S()()}if(a&
2){a=zr(5);const b=zr(7),c=Y(2);D();H("xapInlineDialog",a)("overlaySize",c.hd)("hoverDelayMs",10)("xapInlineDialogDisabled",c.vq)("matMenuTriggerFor",b)}}function sP(a){a&1&&(Nq(0),Mq(1,"div",15)(2,"snapshot-manager",35),Oq());a&2&&(a=Y(2),D(2),H("rendererId",a.rendererId)("paneId",a.paneId))}
function tP(a){if(a&1){const b=Ug();R(0,"div",12);Ap(1,dP,3,2,"ng-container",13);Mq(2,"view-on-node",14)(3,"div",15);R(4,"div",16);V("click",function(){u(b);Y().Zc.fa.next({});return x()});R(5,"mat-icon",17);Z(6,"fit_screen");S()();Mq(7,"div",15);Ap(8,eP,6,0,"ng-template",null,1,Qr)(10,hP,11,10)(11,jP,6,5);R(12,"div",18);V("click",function(){u(b);let c;null==(c=Y().Rs)||c.Dl();return x()});Ap(13,kP,2,0,"mat-icon",19)(14,lP,2,0,"mat-icon",20);S();Ap(15,oP,5,1,"ng-template",null,2,Qr)(17,qP,12,5)(18,
sP,3,2,"ng-container",13);S()}if(a&2){a=zr(9);const b=zr(16),c=Y();nq("in-popup",c.Qb)("pushed-down",c.Qx);D();H("ngIf",c.iE);D();H("rendererId",c.rendererId)("paneId",c.paneId)("inPopup",c.Qb);D(2);H("xapInlineDialog",a)("overlaySize",c.hd)("hoverDelayMs",10);D(6);Eq(c.YD?10:-1);D();Eq(c.ZD?11:-1);D();nq("enabled",c.Wg);H("xapInlineDialog",b)("overlaySize",c.hd)("hoverDelayMs",10);D();Eq(c.CC||c.Ad?13:14);D(4);Eq(c.XD?17:-1);D();H("ngIf",c.jE)}}
function uP(a){a&1&&Mq(0,"subgraph-breadcrumbs",36);a&2&&(a=Y(),H("paneId",a.paneId))}function rP(a,b){a.Zc.na.next({rendererId:a.rendererId,UB:b})}
var vP=class{constructor(a,b){this.Zc=a;this.D=b;this.Qb=!1;this.Kl=new Fk;this.hd={minWidth:0,minHeight:0,maxWidth:340};this.RB=Bs(()=>uJ(this.Zc,this.paneId));this.C=0;this.vq=!1;this.Mf=[];Js(()=>{let c;this.Mf=(null==(c=nJ(this.Zc,this.paneId))?void 0:c.Zf)||[];bn(this.D)})}Fb(){let a,b;this.C=null!=(b=null==(a=this.Zc.config())?void 0:a.defaultRenderer)?b:0}Qq(a){this.Kl.emit(a)}get iE(){return!this.Qb}get YD(){return!this.Qb}get ZD(){return!this.Qb}get XD(){return!this.Qb&&0===this.C}get jE(){return!this.Qb&&
0===this.C}get Qx(){return!this.Qb&&1<this.Mf.length}get Hv(){return uJ(this.Zc,this.paneId)}get Wg(){let a;return!0===(null==(a=this.Rs)?void 0:a.Wg)}get lE(){let a;return!(null==(a=this.Zc.config())?0:a.hideToolBar)}get CC(){return this.Zc.C}get Ad(){return!0}};vP.K=function(a){return new (a||vP)(z(VJ),z(Sr))};
vP.Da=Gf({type:vP,ha:[["renderer-wrapper"]],fb:function(a,b){a&1&&ur(cP,5);if(a&2){let c;vr(c=wr())&&(b.Rs=c.first)}},inputs:{modelGraph:"modelGraph",rendererId:"rendererId",paneId:"paneId",Vl:"rootNodeId",Qb:"inPopup"},outputs:{Kl:"openInPopupClicked"},ka:!0,features:[Mr],za:5,Aa:7,Ca:[["webglRenderer",""],["helpZoomToFit",""],["helpTraceIO",""],["helpExpandAll",""],["helpCollapseAll",""],["helpFlattenLayers",""],["helpDownloadPng",""],["menu","matMenu"],[1,"container"],[3,"openInPopupClicked","modelGraph",
"rendererId","paneId","rootNodeId","inPopup"],[1,"toolbar",3,"in-popup","pushed-down"],[3,"paneId",4,"ngIf"],[1,"toolbar"],[4,"ngIf"],[3,"rendererId","paneId","inPopup"],[1,"vertical-divider"],["dialogLabel","help",1,"mat-icon-container","zoom-fit",3,"click","xapInlineDialog","overlaySize","hoverDelayMs"],[1,"toolbar-icon"],["dialogLabel","help",1,"mat-icon-container","trace-io",3,"click","xapInlineDialog","overlaySize","hoverDelayMs"],[1,"toolbar-icon","trace"],["fontSet","google-material-icons",
1,"toolbar-icon","trace","material-icons"],[3,"curModelGraph","rendererId"],[1,"model-explorer-help-popup"],[1,"shortcut"],["dialogLabel","help",1,"mat-icon-container","expand-all",3,"click","xapInlineDialog","overlaySize","hoverDelayMs"],[1,"toolbar-icon","expand-all"],["dialogLabel","help",1,"mat-icon-container","collapse-all",3,"click","xapInlineDialog","overlaySize","hoverDelayMs"],[1,"toolbar-icon","collapse-all"],["dialogLabel","help",1,"mat-icon-container","flatten-layers",3,"click","xapInlineDialog",
"overlaySize","hoverDelayMs"],[1,"toolbar-icon","flatten-layers"],[1,"description"],["dialogLabel","help",1,"mat-icon-container","download-png",3,"menuOpened","menuClosed","xapInlineDialog","overlaySize","hoverDelayMs","xapInlineDialogDisabled","matMenuTriggerFor"],[1,"toolbar-icon","download-png"],[1,"model-explorer-download-png-menu"],[1,"menu-item",3,"click"],[3,"rendererId","paneId"],[3,"paneId"]],ta:function(a,b){if(a&1){const c=Ug();R(0,"div",8)(1,"webgl-renderer",9,0);V("openInPopupClicked",
function(d){u(c);return x(b.Qq(d))});S();Ap(3,tP,19,21,"div",10)(4,uP,1,1,"subgraph-breadcrumbs",11);S()}a&2&&(D(),H("modelGraph",b.modelGraph)("rendererId",b.rendererId)("paneId",b.paneId)("rootNodeId",b.Vl)("inPopup",b.Qb),D(2),Eq(b.lE?3:-1),D(),H("ngIf",b.Qx))},Ha:[ht,ct,BB,$C,CC,QC,YC,hG,iI,hI,BO,OO,RO,bP,ZN,HG,EG],styles:[".container[_ngcontent-%COMP%]{width:100%;height:100%;overflow:hidden;position:relative}.container[_ngcontent-%COMP%]   svg-rednerer[_ngcontent-%COMP%], .container[_ngcontent-%COMP%]   webgl-rednerer[_ngcontent-%COMP%]{width:100%;height:100%}.container[_ngcontent-%COMP%]   .toolbar[_ngcontent-%COMP%]{position:absolute;top:8px;left:12px;padding:0 8px 0 6px;-moz-box-sizing:border-box;box-sizing:border-box;height:40px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;background-color:#edf2fa;border-radius:99px;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;outline:2px solid #fff}.container[_ngcontent-%COMP%]   .toolbar.in-popup[_ngcontent-%COMP%]{padding-left:10px}.container[_ngcontent-%COMP%]   .toolbar[_ngcontent-%COMP%]:has(search-bar  input:focus){border-color:#1a73e8}.container[_ngcontent-%COMP%]   .toolbar[_ngcontent-%COMP%]   search-bar[_ngcontent-%COMP%]{margin-right:4px}.container[_ngcontent-%COMP%]   .toolbar.pushed-down[_ngcontent-%COMP%]{top:44px}.container[_ngcontent-%COMP%]   .toolbar[_ngcontent-%COMP%]   .mat-icon-container[_ngcontent-%COMP%]{width:24px;height:24px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center}.container[_ngcontent-%COMP%]   .toolbar[_ngcontent-%COMP%]   .mat-icon-container.enabled[_ngcontent-%COMP%]{background-color:#ffdeb1;outline:1px solid #ea8600;border-radius:4px}.container[_ngcontent-%COMP%]   .toolbar[_ngcontent-%COMP%]   .mat-icon-container.disable[_ngcontent-%COMP%]{opacity:.2;pointer-events:none}.container[_ngcontent-%COMP%]   .toolbar[_ngcontent-%COMP%]   mat-icon.toolbar-icon[_ngcontent-%COMP%]{font-size:20px;width:20px;height:20px;cursor:pointer;opacity:.6}.container[_ngcontent-%COMP%]   .toolbar[_ngcontent-%COMP%]   mat-icon.toolbar-icon[_ngcontent-%COMP%]:hover{opacity:.9}.container[_ngcontent-%COMP%]   .toolbar[_ngcontent-%COMP%]   mat-icon.toolbar-icon.collapse-all[_ngcontent-%COMP%], .container[_ngcontent-%COMP%]   .toolbar[_ngcontent-%COMP%]   mat-icon.toolbar-icon.expand-all[_ngcontent-%COMP%]{font-size:18px;width:18px;height:18px}.container[_ngcontent-%COMP%]   .toolbar[_ngcontent-%COMP%]   mat-icon.toolbar-icon.flatten-layers[_ngcontent-%COMP%]{font-size:20px;width:20px;height:20px;border:1px solid transparent}.container[_ngcontent-%COMP%]   .toolbar[_ngcontent-%COMP%]   mat-icon.toolbar-icon.trace[_ngcontent-%COMP%]{-webkit-transform:rotate(90deg);transform:rotate(90deg);border:1px solid transparent}.container[_ngcontent-%COMP%]   .toolbar[_ngcontent-%COMP%]   .vertical-divider[_ngcontent-%COMP%]{width:1px;border-left:1px solid #d0d0d0;margin:2px 5px;height:20px}.container[_ngcontent-%COMP%]   subgraph-breadcrumbs[_ngcontent-%COMP%]{position:absolute;top:8px;left:12px}  xap-inline-dialog-container:has(.model-explorer-view-popup){border-top-left-radius:0;border-top-right-radius:0}  .model-explorer-download-png-menu .menu-item{letter-spacing:normal!important;font-family:Google Sans Text,Arial,Helvetica,sans-serif!important;font-size:12px;padding:0 8px;cursor:pointer;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}  .model-explorer-download-png-menu .menu-item:hover{background-color:#eee}"],
Ra:0});const wP=["rendererWrapper"];function xP(a){a.vj=!a.vj;a.vj?(a.F=a.root.R.style.height,a.root.R.style.height="26px"):a.root.R.style.height=a.F}function yP(a,b){b.stopPropagation();2!==b.button&&lO(new mO(a.root.R),b)}
var zP=class{constructor(a,b){this.C=a;this.root=b;this.Xu=new Fk;this.selected=Bs(()=>{let c,d;return(null==(c=this.C.G())?void 0:c.id)===(null==(d=this.Tl)?void 0:d.rendererId)&&this.C.J()===this.paneId});this.vj=!1;this.F="";this.D=()=>{if(this.Tl){this.C.G.set({id:this.Tl.rendererId,Lr:1});var c;null==(c=this.Tl.Rs)?c=void 0:c=c.selectedNodeId?{nodeId:c.selectedNodeId,rendererId:c.rendererId,Bd:oI(c.ga.nodesById[c.selectedNodeId])}:void 0;wJ(this.C,this.paneId,c);jJ(this.C,this.paneId)}}}Fb(){this.root.R.addEventListener("mousedown",
this.D,!0);const a=this.root.R;a.style.left=`${this.initialPosition.x}px`;a.style.top=`${this.initialPosition.y}px`;a.style.width="400px";a.style.height="400px"}va(){this.root.R.removeEventListener("mousedown",this.D,!0)}get title(){return this.Fn.label}};zP.K=function(a){return new (a||zP)(z(VJ),z(si))};
zP.Da=Gf({type:zP,ha:[["popup-panel"]],fb:function(a,b){a&1&&ur(wP,5);if(a&2){let c;vr(c=wr())&&(b.Tl=c.first)}},inputs:{id:"id",paneId:"paneId",Fn:"groupNode",initialPosition:"initialPosition",ga:"curModelGraph"},outputs:{Xu:"closeClicked"},ka:!0,features:[Mr],za:22,Aa:13,Ca:[["rendererWrapper",""],[1,"container",3,"mousedown","mouseup"],[1,"title",3,"dblclick"],[1,"label"],[1,"icons-container"],[1,"mat-icon-container","minimize",3,"click"],[1,"mat-icon-container","close",3,"click"],[3,"modelGraph",
"rendererId","paneId","rootNodeId","inPopup"],[1,"resize-box",3,"mousedown","mouseup"],["data-position","top"],["data-position","bottom"],["data-position","left"],["data-position","right"],["data-position","top-right"],["data-position","top-left"],["data-position","bottom-right"],["data-position","bottom-left"]],ta:function(a,b){if(a&1){const c=Ug();R(0,"div",1);V("mousedown",function(d){u(c);return x(yP(b,d))})("mouseup",function(){u(c);return x()});R(1,"div",2);V("dblclick",function(){u(c);return x(xP(b))});
R(2,"div",3);Z(3);S();R(4,"div",4)(5,"div",5);V("click",function(){u(c);return x(xP(b))});R(6,"mat-icon");Z(7);S()();R(8,"div",6);V("click",function(){u(c);return x(b.Xu.emit(b.id))});R(9,"mat-icon");Z(10,"close");S()()()();Mq(11,"renderer-wrapper",7,0);S();R(13,"div",8);V("mousedown",function(d){u(c);return x(yP(b,d))})("mouseup",function(){u(c);return x()});Mq(14,"div",9)(15,"div",10)(16,"div",11)(17,"div",12)(18,"div",13)(19,"div",14)(20,"div",15)(21,"div",16);S()}a&2&&(nq("selected",b.selected())("minimized",
b.vj),D(3),Ar(b.title),D(4),Ar(b.vj?"unfold_more":"unfold_less"),D(4),H("modelGraph",b.ga)("rendererId",b.id)("paneId",b.paneId)("rootNodeId",b.Fn.id)("inPopup",!0),D(2),nq("disabled",b.vj))},Ha:[iI,hI,vP],styles:["[_nghost-%COMP%]{position:absolute;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none}.container[_ngcontent-%COMP%]{width:100%;height:100%;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;font-size:13px;outline:2px solid transparent;position:relative;background-color:#fff;border-radius:4px;overflow:hidden;border:2px solid #aaa;cursor:pointer}.container[_ngcontent-%COMP%]   .title[_ngcontent-%COMP%]{height:27px;border-bottom:1px solid #ccc;-moz-box-sizing:border-box;box-sizing:border-box;background-color:#e7f3ff;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between;-webkit-flex-shrink:0;-ms-flex-negative:0;flex-shrink:0;overflow:hidden;border-top-left-radius:2px;border-top-right-radius:2px;white-space:nowrap}.container[_ngcontent-%COMP%]   .title[_ngcontent-%COMP%]   .label[_ngcontent-%COMP%]{-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1;overflow:hidden;text-overflow:ellipsis;-moz-box-sizing:border-box;box-sizing:border-box;padding-left:8px;padding-right:4px}.container[_ngcontent-%COMP%]   .title[_ngcontent-%COMP%]   .icons-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}.container[_ngcontent-%COMP%]   .title[_ngcontent-%COMP%]   .mat-icon-container[_ngcontent-%COMP%]{width:24px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;opacity:.6;cursor:pointer}.container[_ngcontent-%COMP%]   .title[_ngcontent-%COMP%]   .mat-icon-container[_ngcontent-%COMP%]:hover{opacity:1}.container[_ngcontent-%COMP%]   .title[_ngcontent-%COMP%]   .mat-icon-container[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{font-size:16px;font-weight:700;width:16px;height:16px}.container[_ngcontent-%COMP%]   renderer-wrapper[_ngcontent-%COMP%]{-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1;min-height:0}.container.selected[_ngcontent-%COMP%]{box-shadow:0 4px 6px -1px rgba(0,0,0,.1),0 2px 4px -2px rgba(0,0,0,.1)}.container.selected[_ngcontent-%COMP%]   .title[_ngcontent-%COMP%]{background-color:#ea8600;color:#fff}.container.minimized[_ngcontent-%COMP%]   svg-renderer[_ngcontent-%COMP%]{display:none}.resize-box[_ngcontent-%COMP%]{position:absolute;inset:-4px;pointer-events:none;z-index:100}.resize-box.disabled[_ngcontent-%COMP%]   [data-position][_ngcontent-%COMP%]{pointer-events:none}.resize-box[_ngcontent-%COMP%]   div[_ngcontent-%COMP%]{position:absolute;pointer-events:all;background-color:transparent}.resize-box[_ngcontent-%COMP%]   [data-position=top][_ngcontent-%COMP%]{top:0;width:100%;height:8px;cursor:ns-resize}.resize-box[_ngcontent-%COMP%]   [data-position=bottom][_ngcontent-%COMP%]{bottom:0;width:100%;height:8px;cursor:ns-resize}.resize-box[_ngcontent-%COMP%]   [data-position=left][_ngcontent-%COMP%]{left:0;height:100%;width:8px;cursor:ew-resize}.resize-box[_ngcontent-%COMP%]   [data-position=right][_ngcontent-%COMP%]{right:0;height:100%;width:8px;cursor:ew-resize}.resize-box[_ngcontent-%COMP%]   [data-position=top-right][_ngcontent-%COMP%]{top:0;right:0;height:8px;width:8px;cursor:nesw-resize}.resize-box[_ngcontent-%COMP%]   [data-position=top-left][_ngcontent-%COMP%]{top:0;left:0;height:8px;width:8px;cursor:nwse-resize}.resize-box[_ngcontent-%COMP%]   [data-position=bottom-left][_ngcontent-%COMP%]{bottom:0;left:0;height:8px;width:8px;cursor:nesw-resize}.resize-box[_ngcontent-%COMP%]   [data-position=bottom-right][_ngcontent-%COMP%]{bottom:0;right:0;height:8px;width:8px;cursor:nwse-resize}"],
Ra:0});const AP=["popupPanel"];function BP(a){if(a&1){const b=Ug();R(0,"renderer-wrapper",5);V("openInPopupClicked",function(c){u(b);const d=Y();return x(d.Qq(c))});S()}a&2&&(a=Y(),H("modelGraph",a.modelGraph)("rendererId",a.rendererId)("paneId",a.paneId))}
function CP(a,b){if(a&1){const c=Ug();R(0,"popup-panel",6,0);V("closeClicked",function(d){u(c);const e=Y();return x(DP(e,d))});S()}a&2&&(a=b.da,b=Y(),H("id",a.id)("paneId",b.paneId)("groupNode",a.Fn)("initialPosition",a.initialPosition)("curModelGraph",a.ga))}function EP(a){a&1&&Mq(0,"legends-panel",4);a&2&&(a=Y(),H("paneId",a.paneId))}function DP(a,b){const c=a.qo.findIndex(d=>d.id===b);0<=c&&a.qo.splice(c,1)}
var FP=class{constructor(a,b,c){this.root=a;this.C=b;this.F=c;this.Or=new ar;this.qo=[];this.vs=!0;this.G=d=>{null==d.target.closest("popup-panel")&&this.C.G.set({id:this.rendererId,Lr:2})};Js(()=>{const d=this.C.G();var e=this.root.R.querySelector("popup-panel.top");e&&e.classList.remove("top");(e=this.Or.find(f=>{let g;return(null==(g=f.Tl)?void 0:g.rendererId)===(null==d?void 0:d.id)}))&&e.root.R.classList.add("top")})}Fb(){this.root.R.addEventListener("mousedown",this.G,!0)}tc(a){a.modelGraph&&
this.modelGraph&&(this.vs=!1,cn(this.F),this.vs=!0,cn(this.F))}pd(){const a=this.root.R;this.D=new ResizeObserver(()=>{this.handleResize()});this.D.observe(a)}va(){this.root.R.removeEventListener("mousedown",this.G,!0);this.D&&this.D.unobserve(this.root.R)}CE(a,b){return b.id}Qq(a){this.qo.push(a)}get dE(){let a;return!(null==(a=this.C.config())?0:a.hideLegends)}handleResize(){const a=this.root.R;for(const b of this.Or){const c=b.root.R;if(!c)continue;const d=Number(c.style.width.replace("px","")),
e=Number(c.style.height.replace("px","")),f=Number(c.style.top.replace("px",""))+e;Number(c.style.left.replace("px",""))+d>a.offsetWidth&&(c.style.left=`${a.offsetWidth-d}px`);f>a.offsetHeight&&(c.style.top=`${a.offsetHeight-e}px`)}}};FP.K=function(a){return new (a||FP)(z(si),z(VJ),z(Sr))};
FP.Da=Gf({type:FP,ha:[["graph-panel"]],fb:function(a,b){a&1&&ur(AP,5);if(a&2){let c;vr(c=wr())&&(b.Or=c)}},inputs:{modelGraph:"modelGraph",rendererId:"rendererId",paneId:"paneId"},ka:!0,features:[Dg,Mr],za:4,Aa:4,Ca:[["popupPanel",""],[1,"container"],[3,"modelGraph","rendererId","paneId","openInPopupClicked",4,"ngIf"],[3,"id","paneId","groupNode","initialPosition","curModelGraph","closeClicked",4,"ngFor","ngForOf","ngForTrackBy"],[3,"paneId"],[3,"openInPopupClicked","modelGraph","rendererId","paneId"],
[3,"closeClicked","id","paneId","groupNode","initialPosition","curModelGraph"]],ta:function(a,b){a&1&&(R(0,"div",1),Ap(1,BP,1,3,"renderer-wrapper",2)(2,CP,2,5,"popup-panel",3)(3,EP,1,1,"legends-panel",4),S());a&2&&(D(),H("ngIf",b.vs),D(),H("ngForOf",b.qo)("ngForTrackBy",b.CE),D(),Eq(b.dE?3:-1))},Ha:[ht,Zs,ct,kO,zP,vP],styles:[".container[_ngcontent-%COMP%]{width:100%;height:100%;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;position:relative;overflow:hidden}.container[_ngcontent-%COMP%]   renderer-wrapper[_ngcontent-%COMP%]{width:100%;height:100%}.container[_ngcontent-%COMP%]   popup-panel.top[_ngcontent-%COMP%]{z-index:100}.container[_ngcontent-%COMP%]   legends-panel[_ngcontent-%COMP%]{position:absolute;left:8px;bottom:8px;border-radius:6px;border:1px solid #ddd;overflow:hidden}"],
Ra:0});const GP=["container"],HP=["oneLineText"];function IP(a){a&1&&(R(0,"div",7),Z(1," Showing at most "),R(2,"span",8),Z(3),S()());a&2&&(a=Y(2),D(2),H("matTooltipShowDelay",300),D(),Br(" ",a.hl()," elements "))}function JP(a){a&1&&(Ap(0,IP,4,2,"div",7),R(1,"div",4),Z(2),S());a&2&&(a=Y(),Eq(0!==a.hl()?0:-1),D(2),Ar(a.text))}function KP(a,b){a&1&&(R(0,"div",10)(1,"div",11),Z(2),S(),R(3,"div",12),Z(4),S()());a&2&&(a=b.da,b=b.Cb,D(2),Ar(b),D(2),Ar(a))}
function LP(a){a&1&&(R(0,"div",7),Z(1," Namespace hierarchy "),S(),R(2,"div",9),Jq(3,KP,5,2,"div",10,Gq),S());a&2&&(a=Y(),D(3),Lq(a.UC))}function MP(a){a&1&&(R(0,"div",4),Z(1),S());a&2&&(a=Y(),D(),Ar(a.SB))}function NP(a){a&1&&Z(0);a&2&&(a=Y(),Br(" ",a.text," "))}function OP(a){a.Gr?(a.C=a.Gr.R.scrollWidth>a.Gr.R.offsetWidth,!a.expanded||"namespace"!==a.type&&"values"!==a.type||(a.C=!0)):a.C=!1}function PP(a,b,c=!1){a.bw&&(b.stopPropagation(),c&&a.expanded||(a.expanded=!a.expanded))}
var QP=class{constructor(a,b){this.G=a;this.D=b;this.type=this.text="";this.bgColor="transparent";this.textColor="inherit";this.C=this.expanded=!1}get iC(){return this.expanded}pd(){setTimeout(()=>{OP(this);bn(this.D)});this.ab&&(this.F=new ResizeObserver(()=>{OP(this);bn(this.D)}),this.F.observe(this.ab.R))}tc(){setTimeout(()=>{OP(this);bn(this.D)})}va(){this.F&&this.F.disconnect()}hl(){let a,b;return null!=(b=null==(a=this.G.config())?void 0:a.maxConstValueCount)?b:0}get bw(){OP(this);return this.C}get jC(){return this.expanded?
"unfold_less":"unfold_more"}get ZB(){return"transparent"!==this.bgColor}get UC(){const a=this.text.split("/");"<root>"!==this.text&&a.unshift("<root>");return a}get SB(){return this.text.replace("[","").replace("]","").split(",").map(a=>a.trim()).join("\n")}};QP.K=function(a){return new (a||QP)(z(VJ),z(Sr))};
QP.Da=Gf({type:QP,ha:[["expandable-info-text"]],fb:function(a,b){a&1&&(ur(GP,5),ur(HP,5));if(a&2){let c;vr(c=wr())&&(b.ab=c.first);vr(c=wr())&&(b.Gr=c.first)}},Za:2,Ia:function(a,b){a&2&&nq("expanded",b.iC)},inputs:{text:"text",type:"type",bgColor:"bgColor",textColor:"textColor"},ka:!0,features:[Dg,Mr],za:13,Aa:13,Ca:[["container",""],["oneLineText",""],[1,"container",3,"click"],[1,"expanded-text"],[1,"monospace-content"],[1,"one-line-text"],[1,"icon-container","toggle",3,"click"],[1,"info-msg"],
["matTooltip","configurable in Settings in homepage","matTooltipPosition","above",1,"underline",3,"matTooltipShowDelay"],[1,"namespace-content"],[1,"namespace-row"],[1,"index"],[1,"namespace"]],ta:function(a,b){if(a&1){const c=Ug();R(0,"div",2,0);V("click",function(d){u(c);return x(PP(b,d,!0))});R(2,"div",3);Ap(3,JP,3,2,"div",4)(4,LP,5,0)(5,MP,2,1,"div",4)(6,NP,1,1);S();R(7,"div",5,1);Z(9);S();R(10,"div",6);V("click",function(d){u(c);return x(PP(b,d))});R(11,"mat-icon");Z(12);S()()()}a&2&&(lq("background-color",
b.bgColor)("color",b.textColor),nq("has-bg-color",b.ZB)("has-overflow",b.bw)("expanded",b.expanded),D(3),Eq("values"===b.type?3:"namespace"===b.type?4:"quantization"===b.type?5:6),D(6),Br(" ",b.text," "),D(3),Ar(b.jC))},Ha:[ht,hG,AE,fG,iI,hI],styles:["[_nghost-%COMP%]{overflow:hidden}.container[_ngcontent-%COMP%]{overflow:hidden;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:start;-webkit-align-items:flex-start;-moz-box-align:start;-ms-flex-align:start;align-items:flex-start;position:relative}.container.has-overflow[_ngcontent-%COMP%]{cursor:pointer}.container.has-overflow[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex}.container.has-overflow[_ngcontent-%COMP%]:hover   .icon-container[_ngcontent-%COMP%]{opacity:1}.container.has-bg-color[_ngcontent-%COMP%]{border-radius:4px;padding-left:4px}.container.expanded[_ngcontent-%COMP%]   .one-line-text[_ngcontent-%COMP%]{position:absolute;inset:0;visibility:hidden}.container.expanded[_ngcontent-%COMP%]   .expanded-text[_ngcontent-%COMP%]{display:block;-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1;cursor:text}.container.expanded[_ngcontent-%COMP%]   .expanded-text[_ngcontent-%COMP%]   .info-msg[_ngcontent-%COMP%]{margin-bottom:2px;color:#999;font-size:12px}.container.expanded[_ngcontent-%COMP%]   .expanded-text[_ngcontent-%COMP%]   .info-msg[_ngcontent-%COMP%]   .underline[_ngcontent-%COMP%]{-webkit-text-decoration:underline dashed;-moz-text-decoration:underline dashed;text-decoration:underline dashed}.container.expanded[_ngcontent-%COMP%]   .expanded-text[_ngcontent-%COMP%]   .monospace-content[_ngcontent-%COMP%]{font-size:11px;white-space:pre-wrap;font-family:monospace;max-height:500px;overflow:auto}.container.expanded[_ngcontent-%COMP%]   .expanded-text[_ngcontent-%COMP%]   .namespace-content[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;gap:2px}.container.expanded[_ngcontent-%COMP%]   .expanded-text[_ngcontent-%COMP%]   .namespace-row[_ngcontent-%COMP%]{font-size:12px;line-height:14px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:start;-webkit-align-items:flex-start;-moz-box-align:start;-ms-flex-align:start;align-items:flex-start}.container.expanded[_ngcontent-%COMP%]   .expanded-text[_ngcontent-%COMP%]   .namespace-row[_ngcontent-%COMP%]   .index[_ngcontent-%COMP%]{width:16px;color:#999;-webkit-flex-shrink:0;-ms-flex-negative:0;flex-shrink:0;line-height:14px}.container.expanded[_ngcontent-%COMP%]   .expanded-text[_ngcontent-%COMP%]   .namespace-row[_ngcontent-%COMP%]   .namespace[_ngcontent-%COMP%]{line-height:14px}.container.expanded[_ngcontent-%COMP%]   .expanded-text[_ngcontent-%COMP%]   .namespace-row[_ngcontent-%COMP%]   .namespace.hidden[_ngcontent-%COMP%]{color:#999}.container.expanded[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]{background-color:#eee;border:1px solid #999}.container.expanded[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{color:#999}.container[_ngcontent-%COMP%]   .one-line-text[_ngcontent-%COMP%]{-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1;overflow:hidden;text-overflow:ellipsis;white-space:nowrap}.container[_ngcontent-%COMP%]   .expanded-text[_ngcontent-%COMP%]{display:none}.container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]{width:12px;height:12px;-webkit-flex-shrink:0;-ms-flex-negative:0;flex-shrink:0;display:none;background-color:#d7f1ff;border-radius:2px;margin-left:2px;cursor:pointer;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;border:1px solid #4e90b5;opacity:.75}.container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{color:#00639b;font-size:11px;width:11px;height:11px;font-weight:700}"],
Ra:0});const RP=["container"];function SP(a){a&1&&(R(0,"div",3),Z(1),S());a&2&&(a=Y(),D(),Br(" ",a.label," "))}var TP=class{constructor(a){this.C=a;this.label="";this.Px=!1;this.Pr={minWidth:0,minHeight:0};this.zD=[{cb:"end",eb:"top",Na:"end",Ta:"top",offsetY:-1}]}pd(){setTimeout(()=>{const a=this.ab.R;this.Px=a.scrollWidth>a.offsetWidth;bn(this.C)})}};TP.K=function(a){return new (a||TP)(z(Sr))};
TP.Da=Gf({type:TP,ha:[["hoverable-label"]],fb:function(a,b){a&1&&ur(RP,5);if(a&2){let c;vr(c=wr())&&(b.ab=c.first)}},inputs:{label:"label"},ka:!0,features:[Mr],za:5,Aa:6,Ca:[["container",""],["popup",""],["dialogLabel","popup",1,"container",3,"xapInlineDialog","overlaySize","overlayPositions","hoverDelayMs","xapInlineDialogDisabled"],[1,"model-explorer-hoverable-label-popup"]],ta:function(a,b){a&1&&(R(0,"div",2,0),Z(2),S(),Ap(3,SP,2,1,"ng-template",null,1,Qr));a&2&&(a=zr(4),H("xapInlineDialog",a)("overlaySize",
b.Pr)("overlayPositions",b.zD)("hoverDelayMs",10)("xapInlineDialogDisabled",!b.Px),D(2),Br(" ",b.label,"\n"))},Ha:[HG,EG],styles:["[_nghost-%COMP%] {\n  overflow: hidden;\n}\n\n.container[_ngcontent-%COMP%] {\n  overflow: hidden;\n  text-overflow: ellipsis;\n}\n\n  xap-inline-dialog-container:has(.model-explorer-hoverable-label-popup) {\n  width: 100%;\n  box-shadow: none;\n  border: 1px solid #ccc;\n  border-radius: 4px;\n}\n\n  .model-explorer-hoverable-label-popup {\n  padding: 2px;\n  font-size: 12px;\n  line-height: 12px;\n  background-color: white;\n  color: #999;\n  font-family: 'Google Sans Text', Arial, Helvetica, sans-serif;\n}"],
Ra:0});var UP=class{constructor(){this.ve=-2;this.ug="asc";this.ue=-2;this.sg="asc";this.Tf=this.If=this.Sg=!1;this.C=new Set}};UP.K=function(a){return new (a||UP)};UP.pa=Md({ma:UP,ba:UP.K});const VP=["paginator"],WP=["childrenStatsPaginator"];function XP(a){a&1&&(R(0,"div",26),Z(1),S());a&2&&(a=Y().index,D(),Ar(a+1))}function YP(a){a&1&&Mq(0,"mat-spinner",27)}function ZP(a){a&1&&(R(0,"div",28)(1,"mat-icon"),Z(2,"error"),S()());a&2&&(a=Y().da,H("matTooltip",a.error)("matTooltipDisabled",!a.error))}
function $P(a){if(a&1){const b=Ug();R(0,"div",29);V("click",function(c){u(b);const d=Y().da,e=Y();c.stopPropagation();d.runId===e.Lf||BJ(e.ui,e.paneId,d.runId);return x()});R(1,"mat-icon");Z(2);S()()}if(a&2){a=Y().da;const b=Y();nq("selected",a.runId===b.Lf)("hide",!a.done);H("matTooltip",aQ(b,a));D(2);Ar(a.runId===b.Lf?"visibility":"visibility_off")}}
function bQ(a){if(a&1){const b=Ug();R(0,"div",30);V("click",function(){u(b);const c=Y().da,d=Y();d.ga&&(FL(d.bk,c.runId),TJ(d.ui,[`${"Node data provider: "}${c.runName}`]));return x()});R(1,"mat-icon");Z(2,"delete");S()()}a&2&&H("matTooltip","Delete")}
function cQ(a,b){a&1&&(R(0,"div",17)(1,"div",18),Ap(2,XP,2,1,"div",19)(3,YP,1,0,"mat-spinner",20),S(),R(4,"div",21),Z(5),S(),R(6,"div",22),Ap(7,ZP,3,2,"div",23)(8,$P,3,6,"div",24)(9,bQ,3,1,"div",25),S()());a&2&&(a=b.da,b=Y(),nq("selected",a.runId===b.Lf),D(2),H("ngIf",a.done),D(),H("ngIf",!a.done),D(2),Ar(a.runName),D(2),H("ngIf",a.error),D(),H("ngIf",!a.error),D(),H("ngIf",a.done))}function dQ(a,b){a&1&&(R(0,"th",35)(1,"div",36)(2,"div",26),Z(3),S()()());a&2&&(a=b.index,D(3),Ar(a+1))}
function eQ(a,b){a&1&&(R(0,"td",35),Z(1),S());a&2&&(a=b.da,Y(3),D(),Br(" ",a===Number.POSITIVE_INFINITY||a===Number.NEGATIVE_INFINITY||isNaN(a)?"-":`${a}`," "))}function fQ(a,b){a&1&&(R(0,"tr")(1,"td"),Z(2),S(),Ap(3,eQ,2,1,"td",33),S());a&2&&(a=b.da,D(2),Ar(a.stat),D(),H("ngForOf",a.values))}
function gQ(a){a&1&&(R(0,"table",31)(1,"thead")(2,"tr")(3,"th",32),Z(4," Stat "),S(),Ap(5,dQ,4,1,"th",33),S()(),R(6,"tbody"),Ap(7,fQ,4,2,"tr",34),S()());a&2&&(a=Y(),D(5),H("ngForOf",a.Oj),D(2),H("ngForOf",a.cj)("ngForTrackBy",a.GE))}function hQ(a){if(a&1){const b=Ug();R(0,"paginator",37,3);V("change",function(c){u(b);const d=Y();return x(iQ(d,c))});S()}a&2&&(a=Y(),H("pageSize",a.bg)("itemsCount",a.Vu))}
function jQ(a){a&1&&(R(0,"mat-icon",43),Z(1),S());a&2&&(a=Y(2),D(),Br(" ","asc"===a.sg?"arrow_upward":"arrow_downward"," "))}function kQ(a){a&1&&(R(0,"mat-icon",43),Z(1),S());a&2&&(a=Y(2),D(),Br(" ","asc"===a.sg?"arrow_upward":"arrow_downward"," "))}function lQ(a){a&1&&(R(0,"mat-icon",43),Z(1),S());a&2&&(a=Y(3),D(),Br(" ","asc"===a.sg?"arrow_upward":"arrow_downward"," "))}
function mQ(a,b){if(a&1){const c=Ug();R(0,"th",44);V("click",function(){const d=u(c).da,e=Y(2);return x(nQ(e,d.bv))});R(1,"div",36)(2,"div",26);Z(3);S();R(4,"div",45);Z(5);S();Ap(6,lQ,2,1,"mat-icon",40);S()()}a&2&&(a=b.da,b=Y(2),D(3),Ar(a.OD+1),D(2),Ar(a.label),D(),H("ngIf",a.bv===b.ue))}function oQ(a,b){a&1&&(R(0,"td",35),Z(1),S());a&2&&(a=b.da,D(),Br(" ",a," "))}
function pQ(a,b){if(a&1){const c=Ug();R(0,"tr")(1,"td");Z(2);S();R(3,"td",46);V("click",function(){const d=u(c).da,e=Y(2);return x(qQ(e,d.id))});Z(4);S();Ap(5,oQ,2,1,"td",33);S()}a&2&&(a=b.da,D(2),Ar(a.index),D(2),Br(" ",a.label," "),D(),H("ngForOf",a.jB))}
function rQ(a){if(a&1){const b=Ug();R(0,"table",38)(1,"thead")(2,"tr")(3,"th",39);V("click",function(){u(b);const c=Y();return x(nQ(c,-2))});R(4,"div",36);Z(5," # ");Ap(6,jQ,2,1,"mat-icon",40);S()();R(7,"th",41);V("click",function(){u(b);const c=Y();return x(nQ(c,-1))});R(8,"div",36);Z(9," Node ");Ap(10,kQ,2,1,"mat-icon",40);S()();Ap(11,mQ,7,3,"th",42);S()();R(12,"tbody");Ap(13,pQ,6,3,"tr",34);S()()}a&2&&(a=Y(),D(6),H("ngIf",-2===a.ue),D(4),H("ngIf",-1===a.ue),D(),H("ngForOf",a.cn),D(2),H("ngForOf",
a.qv)("ngForTrackBy",a.oy))}function sQ(a){if(a&1){const b=Ug();R(0,"paginator",37,4);V("change",function(c){u(b);const d=Y();return x(tQ(d,c))});S()}a&2&&(a=Y(),H("pageSize",a.bg)("itemsCount",a.yx))}function uQ(a){a&1&&(R(0,"mat-icon",43),Z(1),S());a&2&&(a=Y(2),D(),Br(" ","asc"===a.ug?"arrow_upward":"arrow_downward"," "))}function vQ(a){a&1&&(R(0,"mat-icon",43),Z(1),S());a&2&&(a=Y(2),D(),Br(" ","asc"===a.ug?"arrow_upward":"arrow_downward"," "))}
function wQ(a){if(a&1){const b=Ug();R(0,"div",49);V("click",function(c){u(b);const d=Y().da,e=Y(2);c.stopPropagation();d.runId===e.Lf||BJ(e.ui,e.paneId,d.runId);return x()});R(1,"mat-icon");Z(2);S()()}if(a&2){a=Y().da;const b=Y(2);nq("selected",a.runId===b.Lf)("hide",!a.done);H("matTooltip",aQ(b,a));D(2);Ar(a.runId===b.Lf?"visibility":"visibility_off")}}function xQ(a){a&1&&(R(0,"mat-icon",43),Z(1),S());a&2&&(a=Y(3),D(),Br(" ","asc"===a.ug?"arrow_upward":"arrow_downward"," "))}
function yQ(a,b){if(a&1){const c=Ug();R(0,"th",44);V("click",function(){const d=u(c).index,e=Y(2);return x(zQ(e,d))});R(1,"div",36)(2,"div",26);Z(3);S();Ap(4,wQ,3,6,"div",48)(5,xQ,2,1,"mat-icon",40);S()()}if(a&2){a=b.da;b=b.index;const c=Y(2);D(3);Ar(b+1);D();H("ngIf",!a.error);D();H("ngIf",b===c.ve)}}function AQ(a,b){a&1&&(R(0,"td",35),Z(1),S());a&2&&(a=b.da,lq("background-color",a.bgColor)("color",a.textColor),D(),Br(" ",a.strValue," "))}
function BQ(a,b){if(a&1){const c=Ug();R(0,"tr")(1,"td");Z(2);S();R(3,"td",46);V("click",function(){const d=u(c).da,e=Y(2);return x(qQ(e,d.id))});Z(4);S();Ap(5,AQ,2,5,"td",50);S()}a&2&&(a=b.da,D(2),Ar(a.index),D(),nq("input",a.xC)("output",a.zC),D(),Br(" ",a.label," "),D(),H("ngForOf",a.cols))}
function CQ(a){if(a&1){const b=Ug();R(0,"table",47)(1,"thead")(2,"tr")(3,"th",39);V("click",function(){u(b);const c=Y();return x(zQ(c,-2))});R(4,"div",36);Z(5," # ");Ap(6,uQ,2,1,"mat-icon",40);S()();R(7,"th",41);V("click",function(){u(b);const c=Y();return x(zQ(c,-1))});R(8,"div",36);Z(9," Node ");Ap(10,vQ,2,1,"mat-icon",40);S()();Ap(11,yQ,6,3,"th",42);S()();R(12,"tbody");Ap(13,BQ,6,7,"tr",34);S()()}a&2&&(a=Y(),D(6),H("ngIf",-2===a.ve),D(4),H("ngIf",-1===a.ve),D(),H("ngForOf",a.Oj),D(2),H("ngForOf",
a.rv)("ngForTrackBy",a.oy))}const DQ=["Sum %"];function aQ(a,b){return b.runId===a.Lf?"Visualizing in graph":"Click to visualize in graph"}function iQ(a,b){a.qv=a.G.slice(b*a.bg,(b+1)*a.bg);bn(a.D)}function nQ(a,b){a.C.sg=a.C.ue===b?EQ(a.C.sg):0>b?"asc":"desc";a.C.ue=b;FQ(a);let c;null==(c=a.eq)||c.reset();iQ(a,0)}function qQ(a,b){a.ui.F.set({nodeId:b,rendererId:a.paneId,Bd:!1})}function tQ(a,b){a.rv=null==a.F?[]:a.F.slice(b*a.bg,(b+1)*a.bg);bn(a.D)}
function zQ(a,b){a.C.ug=a.C.ve===b?EQ(a.ug):0>b?"asc":"desc";a.C.ve=b;GQ(a);let c;null==(c=a.Mr)||c.reset();tQ(a,0)}function HQ(a){return a.map(b=>{const c=[];c.push(b.runId);c.push(String(b.done));c.push(String(Object.keys(b.results||{}).length));return c.join("__")}).join(",")}
function IQ(a){if(a.ga){var b=JQ(a),c=a.N[b];if(null!=c)a.J=c;else{const d=null==a.Og?void 0:a.ga.nodesById[a.Og];let e=new Set;null!=d&&(e=new Set(d.descendantsOpNodeIds||[]));a.J=a.ga.nodes.filter(f=>nI(f)&&!f.hideInLayout&&"GraphInputs"!==f.id&&"GraphOutputs"!==f.id&&(null==d||e.has(f.id)));a.N[b]=a.J}}}
function KQ(a){if(a.ga&&0!==a.J.length){var b=vL(a.bk,a.ga);a.cj=[{stat:"Min",values:[]},{stat:"Max",values:[]},{stat:"Sum",values:[]},{stat:"Avg",values:[]}];var c=[];for(var d=0;d<b.length;d++)c.push({min:Number.POSITIVE_INFINITY,max:Number.NEGATIVE_INFINITY,li:0,count:0});a.F=[];for(d=0;d<a.J.length;d++){var e=a.J[d],f=e.id,g=[];for(var k=0;k<b.length;k++){var m=((b[k].results||{})[a.ga.id]||{})[f],p=void 0,t=null==(p=m)?void 0:p.value,w=void 0,B=(null==(w=m)?void 0:w.strValue)||"-",G=void 0;const aa=
(null==(G=m)?void 0:G.bgColor)||"";var J=void 0;m=(null==(J=m)?void 0:J.textColor)||"black";g.push({value:t,strValue:B,bgColor:aa,textColor:m});null!=t&&"number"===typeof t&&(B=c[k],B.min=Math.min(t,B.min),B.max=Math.max(t,B.max),B.count++,B.li+=t)}k=e.incomingEdges||[];k=0===k.length||k.some(aa=>"GraphInputs"===aa.sourceNodeId);e=e.outgoingEdges||[];e=0===e.length||e.some(aa=>"GraphOutputs"===aa.targetNodeId);a.F.push({id:f,index:d,xC:k,zC:e,label:a.ga.nodesById[f].label||"?",cols:g})}a.aa=[...a.F];
GQ(a);tQ(a,0);a.cj[0].values=c.map(aa=>aa.min);a.cj[1].values=c.map(aa=>aa.max);a.cj[2].values=c.map(aa=>aa.li);a.cj[3].values=c.map(aa=>aa.li/aa.count);a.cn=[];p=0;for(w=0;w<b.length;w++)for(var N of DQ)a.cn.push({bv:p,OD:w,label:N}),p++;a.G=[];N=a.Og?a.ga.nodesById[a.Og].nsChildrenIds||[]:a.ga.rootNodes.map(aa=>aa.id);for(p=0;p<N.length;p++){w=N[p];G=a.ga.nodesById[w];J=[];d=[];for(f=0;f<b.length;f++){e=b[f].results||{};k=0;g=!1;if(nI(G)){let aa;e=null==(aa=(e[a.ga.id]||{})[w])?void 0:aa.value;
null!=e&&"number"===typeof e&&(k=e/c[f].li*100,g=!0)}else if(oI(G)){k=0;t=G.descendantsOpNodeIds||[];for(const aa of t){let ca;t=null==(ca=(e[a.ga.id]||{})[aa])?void 0:ca.value;null!=t&&"number"===typeof t&&(k+=t,g=!0)}k=k/c[f].li*100}J.push(k);d.push(g?k.toFixed(1):"-")}a.G.push({id:w,label:G.label,index:p,kB:J,jB:d})}a.P=[...a.G];FQ(a);iQ(a,0);bn(a.D)}}function EQ(a){switch(a){case "desc":return"asc";case "asc":return"desc";default:return a}}
function GQ(a){a.F=[...(a.aa||[])];a.F.sort((b,c)=>LQ(MQ(b,a.C.ve),MQ(c,a.C.ve),a.C.ug))}function FQ(a){a.G=[...(a.P||[])];a.G.sort((b,c)=>LQ(NQ(b,a.C.ue),NQ(c,a.C.ue),a.C.sg))}function OQ(a,b){b.style.maxHeight=a.C.Sg?`${b.scrollHeight}px`:`${b.offsetHeight}px`;bn(a.D);setTimeout(()=>{a.C.Sg=!a.C.Sg;bn(a.D);a.C.Sg||setTimeout(()=>{b.style.maxHeight="fit-content"},150)})}
function PQ(a,b){b.style.maxHeight=a.C.If?`${b.scrollHeight}px`:`${b.offsetHeight}px`;bn(a.D);setTimeout(()=>{a.C.If=!a.C.If;bn(a.D);a.C.If||setTimeout(()=>{b.style.maxHeight="fit-content"},150)})}function QQ(a,b){b.style.maxHeight=a.C.Tf?`${b.scrollHeight}px`:`${b.offsetHeight}px`;bn(a.D);setTimeout(()=>{a.C.Tf=!a.C.Tf;bn(a.D);a.C.Tf||setTimeout(()=>{b.style.maxHeight="fit-content"},150)})}
function JQ(a){let b,c;return`${null==(b=a.ga)?void 0:b.collectionLabel}___${null==(c=a.ga)?void 0:c.id}___${a.Og}`}function LQ(a,b,c){if(null==a&&null==b)return 0;if(null==a&&null!=b)return"asc"===c?-1:1;if(null!=a&&null==b)return"asc"===c?1:-1;if("number"===typeof a&&"number"===typeof b)return"asc"===c?a-b:b-a;a=JSON.stringify(a);b=JSON.stringify(b);return"asc"===c?a.localeCompare(b):b.localeCompare(a)}
function MQ(a,b){switch(b){case -2:return a.index;case -1:return a.label;default:return a.cols[b].value}}function NQ(a,b){switch(b){case -2:return a.index;case -1:return a.label;default:return a.kB[b]}}
var RQ=class{constructor(a,b,c,d){this.ui=a;this.C=b;this.bk=c;this.D=d;this.rv=[];this.cj=[];this.G=[];this.qv=[];this.P=[];this.Oj=[];this.Lf="";this.J=[];this.cn=[];this.bg=50;this.O="";this.N={};a=new URLSearchParams(document.location.search);a.has("nodeDataProviderDataSummaryTablePageSize")&&(this.bg=Number(a.get("nodeDataProviderDataSummaryTablePageSize")));Js(()=>{let e;const f=null==(e=nJ(this.ui,this.paneId))?void 0:e.modelGraph;if(f){var g;this.Lf=(null==(g=GL(this.bk,this.paneId,f))?void 0:
g.runId)||"";bn(this.D)}});Js(()=>{var e,f=(this.ga=null==(e=nJ(this.ui,this.paneId))?void 0:e.modelGraph)?vL(this.bk,this.ga):[];let g=e=!1;this.T!==this.ga&&(this.T=this.ga,e=!0);f=HQ(f);this.O!==f&&(this.O=f,g=!0);if(this.ga&&(e||g)){this.Oj=[];e=vL(this.bk,this.ga);for(const p of e)this.Oj.push({runId:p.runId,runName:p.runName,done:p.done,error:p.error});bn(this.D);this.C.ve=Math.min(this.C.ve,e.length-1);let k;null==(k=this.Mr)||k.reset();IQ(this);KQ(this);this.C.ue=Math.min(this.C.ue,this.cn.length-
1);let m;null==(m=this.eq)||m.reset()}})}tc(a){if(a.rootGroupNodeId){let b;null==(b=this.Mr)||b.reset();let c;null==(c=this.eq)||c.reset();IQ(this);KQ(this)}}DE(a,b){return b.runId}oy(a,b){return b.id}GE(a,b){return b.stat}get ws(){return this.Oj.some(a=>a.done)}get yx(){return null==this.F?0:this.F.length}get Vu(){return this.G.length}get rE(){return this.Sg?"arrow_right":"arrow_drop_down"}get qE(){return null==this.Og?"Aggregated stats":"Aggregated stats in selected layer"}get Sg(){return this.C.Sg}get eB(){return this.If?
"arrow_right":"arrow_drop_down"}get dB(){return null==this.Og?"Root-level nodes stats":"Child nodes stats in selected layer"}get If(){return this.C.If}get dD(){return this.Tf?"arrow_right":"arrow_drop_down"}get cD(){return null==this.Og?"Node data":"Node data in selected layer"}get Tf(){return this.C.Tf}get ug(){return this.C.ug}get ve(){return this.C.ve}get sg(){return this.C.sg}get ue(){return this.C.ue}};RQ.K=function(a){return new (a||RQ)(z(VJ),z(UP),z(KL),z(Sr))};
RQ.Da=Gf({type:RQ,ha:[["node-data-provider-summary-panel"]],fb:function(a,b){a&1&&(ur(VP,5),ur(WP,5));if(a&2){let c;vr(c=wr())&&(b.Mr=c.first);vr(c=wr())&&(b.eq=c.first)}},inputs:{paneId:"paneId",Og:"rootGroupNodeId"},ka:!0,features:[Dg,Mr],za:29,Aa:19,Ca:[["statsTableContainer",""],["childrenStatsTableContainer",""],["nodeDataTableContainer",""],["childrenStatsPaginator",""],["paginator",""],[1,"container"],[1,"index-container"],["class","index-row",3,"selected",4,"ngFor","ngForOf","ngForTrackBy"],
[1,"table-container"],[1,"table-title-container"],[1,"title-label-container",3,"click"],["border","0","cellspacing","0","cellpadding","0",4,"ngIf"],[1,"table-container","children-stats"],[3,"pageSize","itemsCount"],["class","children-stats-results","border","0","cellspacing","0","cellpadding","0",4,"ngIf"],[1,"table-container","results"],["class","results","border","0","cellspacing","0","cellpadding","0",4,"ngIf"],[1,"index-row"],[1,"index-number-container"],["class","index-number",4,"ngIf"],["color",
"primary","diameter","16",4,"ngIf"],[1,"run-name"],[1,"action-icons-container"],["class","error",3,"matTooltip","matTooltipDisabled",4,"ngIf"],["class","icon-container visibility",3,"selected","hide","matTooltip","click",4,"ngIf"],["class","icon-container action",3,"matTooltip","click",4,"ngIf"],[1,"index-number"],["color","primary","diameter","16"],[1,"error",3,"matTooltip","matTooltipDisabled"],[1,"icon-container","visibility",3,"click","matTooltip"],[1,"icon-container","action",3,"click","matTooltip"],
"border 0 cellspacing 0 cellpadding 0".split(" "),[1,"stats"],["class","value-col",4,"ngFor","ngForOf"],[4,"ngFor","ngForOf","ngForTrackBy"],[1,"value-col"],[1,"header-content"],[3,"change","pageSize","itemsCount"],["border","0","cellspacing","0","cellpadding","0",1,"children-stats-results"],[1,"index",3,"click"],["class","sort",4,"ngIf"],[1,"header","node-label",3,"click"],["class","value-col",3,"click",4,"ngFor","ngForOf"],[1,"sort"],[1,"value-col",3,"click"],[1,"stat-label"],[1,"node-label",3,
"click"],["border","0","cellspacing","0","cellpadding","0",1,"results"],["class","icon-container visibility","matTooltipPosition","above",3,"selected","hide","matTooltip","click",4,"ngIf"],["matTooltipPosition","above",1,"icon-container","visibility",3,"click","matTooltip"],["class","value-col",3,"background-color","color",4,"ngFor","ngForOf"]],ta:function(a,b){if(a&1){const c=Ug();R(0,"div",5)(1,"div",6);Ap(2,cQ,10,8,"div",7);S();R(3,"div",8,0)(5,"div",9)(6,"div",10);V("click",function(){u(c);const d=
zr(4);return x(OQ(b,d))});R(7,"mat-icon");Z(8);S();Z(9);S()();Ap(10,gQ,8,3,"table",11);S();R(11,"div",12,1)(13,"div",9)(14,"div",10);V("click",function(){u(c);const d=zr(12);return x(PQ(b,d))});R(15,"mat-icon");Z(16);S();Z(17);S();Ap(18,hQ,2,2,"paginator",13);S();Ap(19,rQ,14,5,"table",14);S();R(20,"div",15,2)(22,"div",9)(23,"div",10);V("click",function(){u(c);const d=zr(21);return x(QQ(b,d))});R(24,"mat-icon");Z(25);S();Z(26);S();Ap(27,sQ,2,2,"paginator",13);S();Ap(28,CQ,14,5,"table",16);S()()}a&
2&&(D(2),H("ngForOf",b.Oj)("ngForTrackBy",b.DE),D(),nq("collapsed",b.Sg),D(5),Ar(b.rE),D(),Br(" ",b.qE," "),D(),H("ngIf",b.ws),D(),nq("collapsed",b.If),D(5),Ar(b.eB),D(),Br(" ",b.dB," "),D(),Eq(b.Vu>b.bg&&!b.If?18:-1),D(),H("ngIf",b.ws),D(),nq("collapsed",b.Tf),D(5),Ar(b.dD),D(),Br(" ",b.cD," "),D(),Eq(b.yx>b.bg&&!b.Tf?27:-1),D(),H("ngIf",b.ws))},Ha:[ht,Zs,ct,wD,vD,sD,iI,hI,gG,fG,nO,Cx],styles:[".container[_ngcontent-%COMP%]{padding-top:12px;-moz-box-sizing:border-box;box-sizing:border-box;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;height:100%}.container[_ngcontent-%COMP%]   .index-number[_ngcontent-%COMP%]{font-size:10px;width:16px;height:16px;border-radius:8px;-moz-box-sizing:border-box;box-sizing:border-box;border:1px solid #999;background-color:#eee;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center}.container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;width:16px;height:16px;opacity:.3;cursor:pointer}.container[_ngcontent-%COMP%]   .icon-container.selected[_ngcontent-%COMP%]{opacity:1;cursor:default}.container[_ngcontent-%COMP%]   .icon-container.selected[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{color:#e1a011}.container[_ngcontent-%COMP%]   .icon-container.hide[_ngcontent-%COMP%]{visibility:hidden}.container[_ngcontent-%COMP%]   .icon-container.action[_ngcontent-%COMP%]{opacity:.7;margin-left:4px}.container[_ngcontent-%COMP%]   .icon-container.action[_ngcontent-%COMP%]:hover{opacity:1}.container[_ngcontent-%COMP%]   .icon-container.visibility[_ngcontent-%COMP%]{margin-left:4px}.container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{font-size:16px;width:16px;height:16px;line-height:16px;color:#000}.container[_ngcontent-%COMP%]   .index-container[_ngcontent-%COMP%]   .index-row[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;overflow:hidden;padding:2px 8px}.container[_ngcontent-%COMP%]   .index-container[_ngcontent-%COMP%]   .index-row.selected[_ngcontent-%COMP%]{background-color:#fff2d5}.container[_ngcontent-%COMP%]   .index-container[_ngcontent-%COMP%]   .index-row[_ngcontent-%COMP%]   .index-number-container[_ngcontent-%COMP%]{width:16px;height:16px;-webkit-flex-shrink:0;-ms-flex-negative:0;flex-shrink:0;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;margin-right:6px}.container[_ngcontent-%COMP%]   .index-container[_ngcontent-%COMP%]   .index-row[_ngcontent-%COMP%]   .run-name[_ngcontent-%COMP%]{-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1;white-space:nowrap;overflow:hidden;text-overflow:ellipsis}.container[_ngcontent-%COMP%]   .index-container[_ngcontent-%COMP%]   .index-row[_ngcontent-%COMP%]   .action-icons-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-flex-shrink:0;-ms-flex-negative:0;flex-shrink:0}.container[_ngcontent-%COMP%]   select[_ngcontent-%COMP%]{width:100%}.container[_ngcontent-%COMP%]   .running[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;margin-top:16px}.container[_ngcontent-%COMP%]   .running[_ngcontent-%COMP%]   mat-spinner[_ngcontent-%COMP%]{margin-right:4px}.container[_ngcontent-%COMP%]   .running[_ngcontent-%COMP%]   .label[_ngcontent-%COMP%]{color:#777}.container[_ngcontent-%COMP%]   .error[_ngcontent-%COMP%]{width:16px;height:16px}.container[_ngcontent-%COMP%]   .error[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{font-size:16px;height:16px;width:16px;color:#a00}.container[_ngcontent-%COMP%]   .table-container[_ngcontent-%COMP%]{width:100%;-moz-box-sizing:border-box;box-sizing:border-box;padding:8px;margin-top:2px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;-webkit-transition:max-height .15s ease-out;transition:max-height .15s ease-out;overflow:hidden}.container[_ngcontent-%COMP%]   .table-container.collapsed[_ngcontent-%COMP%]{max-height:30px!important}.container[_ngcontent-%COMP%]   .table-container[_ngcontent-%COMP%]   .table-title-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between;line-height:22px}.container[_ngcontent-%COMP%]   .table-container[_ngcontent-%COMP%]   .table-title-container[_ngcontent-%COMP%]   .title-label-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;cursor:pointer;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;line-height:15px}.container[_ngcontent-%COMP%]   .table-container[_ngcontent-%COMP%]   .table-title-container[_ngcontent-%COMP%]   .title-label-container[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{margin-left:-4px}.container[_ngcontent-%COMP%]   .table-container[_ngcontent-%COMP%]   paginator[_ngcontent-%COMP%]{-webkit-align-self:flex-end;-ms-flex-item-align:end;align-self:flex-end}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]{border:1px solid #ccc;border-radius:4px;width:100%;-moz-box-sizing:border-box;box-sizing:border-box;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;background-color:#fdfdfd;overflow:hidden}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   thead[_ngcontent-%COMP%]{height:24px;background-color:#f2f2f2}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   thead[_ngcontent-%COMP%]   .header[_ngcontent-%COMP%]{font-weight:500}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   tbody[_ngcontent-%COMP%]{font-size:11px;font-family:Arial,Helvetica,sans-serif}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   tr[_ngcontent-%COMP%]:hover{outline:1px solid #999}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   td[_ngcontent-%COMP%], .container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   th[_ngcontent-%COMP%]{padding:1px 4px;text-align:left}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   td.node-label[_ngcontent-%COMP%], .container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   th.node-label[_ngcontent-%COMP%]{border-left:1px solid #ddd}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   td.value-col[_ngcontent-%COMP%], .container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   th.value-col[_ngcontent-%COMP%]{border-left:1px solid #ddd;max-width:80px;text-overflow:ellipsis;overflow:hidden}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .header-content[_ngcontent-%COMP%]{height:100%;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .header-content[_ngcontent-%COMP%]   mat-icon.sort[_ngcontent-%COMP%]{font-size:12px;width:12px;height:12px;color:#999;margin-left:4px}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .header-content[_ngcontent-%COMP%]   .stat-label[_ngcontent-%COMP%]{margin-left:6px;white-space:nowrap}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   th[_ngcontent-%COMP%]{cursor:pointer}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   td.node-label[_ngcontent-%COMP%]{cursor:pointer}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   td.node-label[_ngcontent-%COMP%]:hover{-webkit-text-decoration:underline dotted #0085f2;-moz-text-decoration:underline dotted #0085f2;text-decoration:underline dotted #0085f2}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   td.input[_ngcontent-%COMP%]{color:#009e73;font-weight:500}.container[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   td.output[_ngcontent-%COMP%]{color:#d55e00;font-weight:500}"],
Ra:0});const SQ=["inputValueContent"],TQ=(a,b)=>b.id||b.label,UQ=(a,b)=>b.Vh.id,VQ=(a,b)=>b.key,WQ=(a,b)=>b.mD,XQ=(a,b)=>b.id,YQ=a=>({value:a});function ZQ(a,b){a&1&&(R(0,"tr")(1,"td",21),Mq(2,"hoverable-label",22),S(),R(3,"td",23),Mq(4,"expandable-info-text",24),S()());a&2&&(a=b.da,b=Y(2),nq("search-match",$Q(b,a.label)),D(2),H("label",a.label),D(2),H("text",a.value)("type",a.label)("bgColor",a.bgColor||"transparent")("textColor",a.textColor||"black"))}
function aR(a,b){if(a&1){const c=Ug();R(0,"div",15,1)(2,"div",16)(3,"button",17);V("click",function(){const d=u(c).da,e=zr(1),f=Y();return x(bR(f,d.label,e))});R(4,"mat-icon");Z(5);S()();Z(6);S();R(7,"div",18)(8,"table",19);Jq(9,ZQ,5,7,"tr",20,TQ);S()()()}a&2&&(a=b.da,b=Y(),nq("collapsed",cR(b,a.label)),D(5),Ar(dR(b,a.label)),D(),Br(" ",a.label," "),D(3),Lq(a.items))}
function eR(a){if(a&1){const b=Ug();R(0,"div",15,2)(2,"div",16)(3,"button",17);V("click",function(){u(b);const c=zr(1),d=Y();return x(bR(d,d.Xd.To,c))});R(4,"mat-icon");Z(5);S()();Z(6," Node data providers ");S();Mq(7,"node-data-provider-summary-panel",25);S()}a&2&&(a=Y(),nq("collapsed",cR(a,a.Xd.To)),D(5),Ar(dR(a,a.Xd.To)),D(2),H("paneId",a.paneId)("rootGroupNodeId",a.tv))}
function fR(a){if(a&1){const b=Ug();R(0,"paginator",32);V("change",function(c){u(b);const d=Y(2);return x(gR(d,c))});S()}a&2&&(a=Y(2),H("pageSize",a.Cc)("itemsCount",a.mq))}function hR(a){a&1&&(R(0,"div",35),Z(1),S());a&2&&(a=Y().da,Y(2),D(),Ar(iR(a)))}
function jR(a){if(a&1){const b=Ug();R(0,"div",38);V("click",function(c){u(b);const d=Y().da,e=Y(2);return x(e.il(d.Vh.id,c))});Z(1);R(2,"div",39)(3,"mat-icon",40);Z(4,"my_location");S()()()}if(a&2){a=Y().da;const b=Y(2);Kp("data-id",a.Vh.id);D();Br(" ",iR(a)," ");D();H("matTooltip",b.Bl)}}
function kR(a){a&1&&(R(0,"tr",null,4)(2,"td",21)(3,"div",41)(4,"mat-icon",42),Z(5,"circle"),S(),Mq(6,"hoverable-label",22),S()(),R(7,"td",23),Mq(8,"expandable-info-text",43),S()());if(a&2){a=Y().da;const b=Y(3);nq("search-match",lR(b,a.value));D(6);H("label",a.key);D(2);H("text",a.value)("type",a.key)}}function mR(a,b){a&1&&Ap(0,kR,9,5,"tr",20);a&2&&(a=b.da,Y(3),Eq(a.key.startsWith("__")?-1:0))}
function nR(a,b){a&1&&(R(0,"div",31)(1,"div",33)(2,"div",34),Z(3),S(),Ap(4,hR,2,1,"div",35)(5,jR,5,3,"div",36),S(),R(6,"table",37),Jq(7,mR,1,1,null,null,VQ),S()());a&2&&(a=b.da,b=Y(2),D(),nq("search-match",lR(b,oR(a))||lR(b,a.Vh.label)),D(2),Ar(a.index),D(),Eq(a.Vh.hideInLayout?4:5),D(3),Lq(a.uj))}
function pR(a){if(a&1){const b=Ug();R(0,"div",26,3)(2,"div",27)(3,"div",28)(4,"button",17);V("click",function(){u(b);const c=zr(1),d=Y();return x(bR(d,d.Xd.Ro,c))});R(5,"mat-icon");Z(6);S()();Z(7);S();Ap(8,fR,1,2,"paginator",29);S();R(9,"div",30);Jq(10,nR,9,4,"div",31,UQ);S()()}a&2&&(a=Y(),nq("collapsed",cR(a,a.Xd.Ro)),D(6),Ar(dR(a,a.Xd.Ro)),D(),Br(" inputs (",a.mq,") "),D(),Eq(a.cE?8:-1),D(2),Lq(a.oj))}
function qR(a){if(a&1){const b=Ug();R(0,"paginator",32);V("change",function(c){u(b);const d=Y(2);d.Ir=d.Vf.slice(c*d.Cc,(c+1)*d.Cc);bn(d.Td);return x()});S()}a&2&&(a=Y(2),H("pageSize",a.Cc)("itemsCount",a.Vf.length))}function rR(a){a&1&&Pq(0)}function sR(a){a&1&&(R(0,"div",48),Ap(1,rR,1,0,"ng-container",49),S());if(a&2){a=Y().da;const b=Y(6),c=zr(11);D();H("ngTemplateOutlet",c)("ngTemplateOutletContext",Pr(YQ,b.VB(a)))}}
function tR(a,b){if(a&1){const c=Ug();R(0,"div",47);V("click",function(d){const e=u(c).da,f=Y(6);return x(f.il(e.id,d))});Z(1);R(2,"div",39)(3,"mat-icon",40);Z(4,"my_location");S()()();Ap(5,sR,2,4,"ng-template",null,6,Qr)}if(a&2){a=b.da;b=zr(6);const c=Y(6);nq("search-match",uR(c,a.label));H("xapInlineDialog",b)("overlaySize",c.gq)("overlayPositions",c.lD)("hoverDelayMs",50);Kp("data-id",a.id);D();Br(" ",a.label," ");D();H("matTooltip",c.Bl)}}
function vR(a){a&1&&Jq(0,tR,7,9,null,null,XQ);a&2&&(a=Y().da,Lq(a.mB))}function wR(a){a&1&&Mq(0,"expandable-info-text",46);a&2&&(a=Y().da,H("text",a.value))}function xR(a,b){a&1&&(R(0,"tr")(1,"td",21)(2,"div",41)(3,"mat-icon",42),Z(4,"circle"),S(),Mq(5,"hoverable-label",22),S()(),R(6,"td",23),Ap(7,vR,2,0)(8,wR,1,1,"expandable-info-text",46),S()());a&2&&(a=b.da,b=Y(4),nq("search-match",uR(b,a.value)),D(5),H("label",a.key),D(2),Eq(a.key===b.nD?7:8))}
function yR(a){a&1&&(R(0,"table",37),Jq(1,xR,9,4,"tr",20,VQ),S());a&2&&(a=Y().da,D(),Lq(a.uj))}function zR(a,b){a&1&&(R(0,"div",31)(1,"div",33)(2,"div",34),Z(3),S(),R(4,"div",35),Z(5),S()(),Ap(6,yR,3,0,"table",37),S());a&2&&(a=b.da,b=Y(2),D(),nq("search-match",uR(b,a.Fs)),D(2),Ar(a.index),D(2),Ar(""===a.Fs?"output":a.Fs),D(),Eq(0<a.uj.length?6:-1))}
function AR(a){if(a&1){const b=Ug();R(0,"div",44,5)(2,"div",45)(3,"div",28)(4,"button",17);V("click",function(){u(b);const c=zr(1),d=Y();return x(bR(d,d.Xd.Vo,c))});R(5,"mat-icon");Z(6);S()();Z(7);S();Ap(8,qR,1,2,"paginator",29);S();R(9,"div",30);Jq(10,zR,7,5,"div",31,WQ);S()()}a&2&&(a=Y(),nq("collapsed",cR(a,a.Xd.Vo)),D(6),Ar(dR(a,a.Xd.Vo)),D(),Br(" outputs (",a.pv,") "),D(),Eq(a.fE?8:-1),D(2),Lq(a.Ir))}
function BR(a){if(a&1){const b=Ug();R(0,"paginator",32);V("change",function(c){u(b);const d=Y(2);let e;d.Pn=$K(d.Kh.slice(c*d.Cc,(c+1)*d.Cc),"incoming",(null==(e=d.Mc)?void 0:e.id)||"");return x()});S()}a&2&&(a=Y(2),H("pageSize",a.Cc)("itemsCount",a.Kh.length))}
function CR(a){if(a&1){const b=Ug();R(0,"div",15,7)(2,"div",50)(3,"div",28)(4,"button",17);V("click",function(){u(b);const c=zr(1),d=Y();return x(bR(d,d.Xd.Po,c))});R(5,"mat-icon");Z(6);S()();Z(7);S();Ap(8,BR,1,2,"paginator",29);S();Mq(9,"io-tree",51);S()}a&2&&(a=Y(),nq("collapsed",cR(a,a.Xd.Po)),D(6),Ar(dR(a,a.Xd.Po)),D(),Br(" Identical layers (",a.Kh.length,") "),D(),Eq(a.aE?8:-1),D(),H("data",a.Pn)("rendererId",a.nq))}
function DR(a,b){a&1&&(R(0,"div",54),Z(1),S());a&2&&(a=b.da,D(),Br(" ",a," "))}function ER(a,b){a&1&&(R(0,"div",52),Z(1," Namespace hierarchy "),S(),R(2,"div",53),Jq(3,DR,2,1,"div",54,Gq),S());a&2&&(a=b.value,D(3),Lq(a.split("/")))}const FR={bF:"Graph info",eF:"Node info",dF:"Layer info",XE:"Attributes",To:"Node data providers",Po:"Identical groups",Ro:"inputs",Vo:"outputs"};function $Q(a,b){return null!=a.aa.find(c=>c.Dw===b)}
function bR(a,b,c){c&&(cR(a,b)?c.style.maxHeight=`${c.scrollHeight}px`:(c.style.maxHeight=`${c.offsetHeight}px`,c.style.overflow="hidden"),bn(a.Td),setTimeout(()=>{a.J.C.has(b)?a.J.C.delete(b):a.J.C.add(b);bn(a.Td);setTimeout(()=>{cR(a,b)?c.style.overflow="hidden":(c.style.overflow="visible",c.style.maxHeight="fit-content")},150)}))}function cR(a,b){return a.J.C.has(b)}function dR(a,b){return cR(a,b)?"expand_more":"expand_less"}
function gR(a,b){a.oj=GR(b*a.Cc,a.C.slice(b*a.Cc,(b+1)*a.Cc),a.F.slice(b*a.Cc,(b+1)*a.Cc));bn(a.Td);setTimeout(()=>{HR(a)})}function iR(a){const b=a.uj.find(c=>"__tensor_tag"===c.key);return b?`${b.value} (${a.Vh.label})`:a.Vh.label}function lR(a,b){return a.O?null!=a.O.find(c=>c.gf===b):!1}function oR(a){let b,c;return null!=(c=null==(b=a.uj.find(d=>"__tensor_tag"===d.key))?void 0:b.value)?c:""}function uR(a,b){return a.P?null!=a.P.find(c=>c.gf===b):!1}
function IR(a,b){a.ga&&b?a.Mc=a.ga.nodesById[b]:a.Mc=void 0;JR(a);bn(a.Td);setTimeout(()=>{HR(a)})}function KR(a){if(a.Mc&&a.G){var b=a.G.results[a.Mc.id]||[],c=[],d=[],e=[];for(const f of b)switch(f.type){case "input":c.push(f);break;case "output":d.push(f);break;case "list":e.push(f)}a.aa=e;a.P=d;a.O=c;bn(a.Td)}}
function JR(a){a.fi=[];a.oj=[];a.C=[];a.F=[];a.Vf=[];a.Kh=[];a.Pn=void 0;if(a.fa){if(a.ga){var b={label:"Graph info",zo:"graph",items:[]};a.fi.push(b);var c=0,d=0;for(var e of a.ga.nodes)nI(e)&&!e.hideInLayout?c++:oI(e)&&d++;b.items.push({qf:b,label:"op node count",value:String(c)},{qf:b,label:"layer count",value:String(d)})}}else if(a.Mc)if(nI(a.Mc)){if(a.ga&&a.Mc){e=a.Mc;var f={label:"Node info",zo:"op",items:[]};a.fi.push(f);f.items.push({qf:f,label:"op name",value:`${e.label}`});var g="id";f.items.push({qf:f,
label:g,value:e.id,Vi:!0,Rj:a.ca.has(g)});g="namespace";f.items.push({qf:f,label:g,value:sI(e),Vi:!0,Rj:a.ca.has(g)});if(0<Object.keys(e.attrs||{}).length){f={label:"Attributes",zo:"op",items:[]};g=e.attrs||{};for(var k of Object.keys(g))k.startsWith("__")||f.items.push({qf:f,label:k,value:g[k],Vi:!0,Rj:a.na.has(k)});0<f.items.length&&a.fi.push(f)}f=vL(a.N,a.ga);if(0<f.length){k={label:"Node data providers",zo:"op",items:[]};a.fi.push(k);for(b of f){var m=((b.results||{})[a.ga.id]||{})[e.id];f=(null==
(c=m)?void 0:c.strValue)||"-";g=(null==(d=m)?void 0:d.bgColor)||"transparent";let p;m=(null==(p=m)?void 0:p.textColor)||"black";k.items.push({id:b.runId,qf:k,label:b.runName,value:f,Vi:b.done,Rj:null!=a.ja[b.runId],bgColor:g,textColor:m,kd:!b.done})}}}LR(a)}else oI(a.Mc)&&MR(a)}function HR(a){for(let b=0;b<a.Zq.length;b++){let c;const d=null==(c=a.Zq.get(b))?void 0:c.R;d&&d.scrollHeight>d.offsetHeight&&d.classList.add("expandable")}}
function GR(a,b,c){const d=[];for(let e=0;e<b.length;e++){const f=b[e],g=[];Object.entries(c[e]).forEach(([m,p])=>{g.push({key:m,value:p})});g.sort((m,p)=>m.key.localeCompare(p.key));g.push({key:"namespace",value:sI(b[e])});const k=f.attrs||{};k.__value&&g.push({key:"values",value:k.__value});d.push({index:e+a,Vh:f,uj:g})}return d}
function NR(a,b){const c=Date.now(),d=a.width,e=()=>{var f=Date.now()-c;f=a.D.C?1:Math.min(1,f/150);f=Math.sin(f*Math.PI/2);const g=d+(b-d)*f;a.width=g;a.minWidth=g;bn(a.Td);1<=f?(a.width=b,a.minWidth=b,bn(a.Td)):requestAnimationFrame(e)};e()}
function LR(a){if(a.ga&&a.Mc){var b=a.Mc,c=b.incomingEdges||[];a.F=[];a.C=[];a.oj=[];for(var d of c){var e=void 0,f=null==(e=a.ga)?void 0:e.nodesById[d.sourceNodeId];a.C.push(f);var g=(b.inputsMetadata||{})[d.targetNodeInputId]||{};f=Object.assign({},(f.outputsMetadata||{})[d.sourceNodeOutputId]||{});for(var k of Object.keys(f))null==g[k]&&"__tensor_tag"!==k&&(g[k]=f[k]);a.F.push(g)}a.mq=a.C.length;0<c.length&&(a.oj=GR(0,a.C.slice(0,a.Cc),a.F.slice(0,a.Cc)));a.Vf=[];c=b.outputsMetadata||{};b=b.outgoingEdges||
[];d=0;for(const m of Object.keys(c)){e=[];g="";for(const p of Object.keys(c[m]))k=c[m][p],"__tensor_tag"===p&&(g=k),p.startsWith("__")||e.push({key:p,value:k});e.sort((p,t)=>p.key.localeCompare(t.key));k=b.filter(p=>p.sourceNodeOutputId===m).map(p=>a.ga.nodesById[p.targetNodeId]);e.push({key:"connects to",value:"",mB:k});a.Vf.push({index:d,Fs:g,mD:m,uj:e});d++}a.pv=a.Vf.length;a.Ir=a.Vf.slice(0,a.Cc)}}
function MR(a){if(a.ga&&a.Mc){var b=a.Mc,c={label:"Layer info",zo:"group",items:[]};a.fi.push(c);c.items.push({qf:c,label:"name",value:b.label});var d="namespace";c.items.push({qf:c,label:d,value:sI(b),Vi:!0,Rj:a.T.has(d)});d="#children";c.items.push({qf:c,label:d,value:String((b.nsChildrenIds||[]).length),Vi:!0,Rj:a.T.has(d)});d="#descendants";c.items.push({qf:c,label:d,value:String((b.descendantsNodeIds||[]).length),Vi:!0,Rj:a.T.has(d)});null!=b.identicalGroupIndex&&(a.Kh=a.ga.nodes.filter(e=>oI(e)&&
e.identicalGroupIndex===b.identicalGroupIndex),a.Pn=$K(a.Kh.slice(0,a.Cc),"incoming",b.id))}}
var OR=class{constructor(a,b,c,d){this.D=a;this.N=b;this.Td=c;this.J=d;this.Zq=new ar;this.ca=new Set;this.na=new Set;this.T=new Set;this.ja={};this.G=void 0;this.minWidth=this.width=370;this.fi=[];this.oj=[];this.Vf=[];this.Ir=[];this.Kh=[];this.nq="";this.pv=this.mq=0;this.ck=this.fs=!1;this.Cc=this.D.C?5:25;this.Xd=FR;this.nD="connects to";this.Bl="Click: locate\nAlt+click: select";this.VB=sI;this.gq={minWidth:100,minHeight:0,maxWidth:600};this.gv=[{cb:"start",eb:"top",Na:"end",Ta:"top",offsetX:-32}];
this.lD=[{cb:"start",eb:"top",Na:"end",Ta:"top",offsetX:-4}];this.aa=[];this.O=[];this.P=[];this.C=[];this.F=[];this.Ax=0;Js(()=>{var e=nJ(this.D,this.paneId);if(e&&e.modelGraph){this.ga=e.modelGraph;var f;this.nq=(null==(f=e.Qg)?void 0:f.rendererId)||"";var g;e=(null==(g=e.Qg)?void 0:g.nodeId)||"";this.tv!==e&&(IR(this,e),setTimeout(()=>{KR(this)}))}});Js(()=>{const e=nJ(this.D,this.paneId);e&&e.modelGraph&&this.G!==e.Pg&&(this.G=e.Pg,KR(this))});Js(()=>{this.N.D();JR(this);bn(this.Td);setTimeout(()=>
{HR(this)})})}Pq(a){a.preventDefault();document.body.style.cursor="ew-resize";const b=lk(document,"mousemove"),c=lk(window,"mouseup"),d=this.width,e=a.clientX;this.fs=!0;bn(this.Td);C(Pj([b]),Lo(c)).subscribe({next:([f])=>{this.minWidth=this.width=Math.max(64,d-(f.clientX-e));bn(this.Td)},complete:()=>{document.body.style.cursor="default";this.fs=!1;bn(this.Td)}})}il(a,b){this.D.F.set({nodeId:a,rendererId:this.nq,Bd:!1,select:b.altKey})}EE(a,b){return b.label}get fa(){return null!=this.ga&&null==
this.Mc}get Mx(){return this.ga?(null==this.Mc||oI(this.Mc))&&0<vL(this.N,this.ga).length:!1}get tv(){return this.Mc?this.Mc.id:void 0}get cE(){return this.C.length>this.Cc&&!cR(this,"inputs")}get fE(){return this.Vf.length>this.Cc&&!cR(this,"outputs")}get aE(){return this.Kh.length>this.Cc&&!cR(this,"Identical groups")}get gC(){return this.ck?"Show info panel":"Hide info panel"}get fC(){return this.ck?"chevron_left":"chevron_right"}};OR.K=function(a){return new (a||OR)(z(VJ),z(KL),z(Sr),z(UP))};
OR.Da=Gf({type:OR,ha:[["info-panel"]],fb:function(a,b){a&1&&ur(SQ,5);if(a&2){let c;vr(c=wr())&&(b.Zq=c)}},Za:4,Ia:function(a,b){a&2&&lq("width",b.width,"px")("min-width",b.minWidth,"px")},inputs:{paneId:"paneId"},ka:!0,features:[Kr([UP]),Mr],za:12,Aa:12,Ca:[["namespaceHierarchy",""],["sectionEle",""],["ndpSectionEle",""],["inputsSectionEle",""],["rowEle",""],["outputsSectionEle",""],["namespacePopup",""],["identicalGroupsSectionEle",""],[1,"container"],["class","section",3,"collapsed",4,"ngFor","ngForOf",
"ngForTrackBy"],["class","section",3,"collapsed",4,"ngIf"],["class","section inputs",3,"collapsed",4,"ngIf"],["class","section outputs",3,"collapsed",4,"ngIf"],[1,"resizer",3,"mousedown"],[1,"hide-toggle",3,"click","matTooltip"],[1,"section"],[1,"header"],["mat-icon-button","",1,"toggle",3,"click"],[1,"items-container"],[1,"metadata-table","info-attrs"],[3,"search-match"],[1,"key"],[3,"label"],[1,"value"],[3,"text","type","bgColor","textColor"],[3,"paneId","rootGroupNodeId"],[1,"section","inputs"],
[1,"header","input"],[1,"header-label-container"],[3,"pageSize","itemsCount"],[1,"flat-items-container"],[1,"flat-item"],[3,"change","pageSize","itemsCount"],[1,"name-row"],[1,"index"],[1,"name"],[1,"name","locator"],[1,"metadata-table"],[1,"name","locator",3,"click"],["matTooltipClass","multiline-tooltip-left","matTooltipPosition","right",1,"locator-icon-container",3,"matTooltip"],[1,"locator-icon"],[1,"key-container"],[1,"bullet"],[3,"text","type"],[1,"section","outputs"],[1,"header","output"],
[3,"text"],["dialogLabel","namespace popup",1,"target-node-container",3,"click","xapInlineDialog","overlaySize","overlayPositions","hoverDelayMs"],[1,"model-explorer-const-values-popup"],[4,"ngTemplateOutlet","ngTemplateOutletContext"],[1,"header","identical-groups"],[3,"data","rendererId"],[1,"values-count-msg"],[1,"namespace-content"],[1,"namespace-row"]],ta:function(a,b){if(a&1){const c=Ug();R(0,"div",8);Ap(1,aR,11,4,"div",9)(2,eR,8,5,"div",10)(3,pR,12,5,"div",11)(4,AR,12,5,"div",12)(5,CR,10,7,
"div",10);S();R(6,"div",13);V("mousedown",function(d){u(c);return x(b.Pq(d))});S();R(7,"div",14);V("click",function(){u(c);b.ck=!b.ck;let d=0;b.ck?b.Ax=b.width:d=b.Ax;NR(b,d);return x()});R(8,"mat-icon");Z(9);S()();Ap(10,ER,5,0,"ng-template",null,0,Qr)}a&2&&(nq("graph-info",b.Mx),D(),H("ngForOf",b.fi)("ngForTrackBy",b.EE),D(),H("ngIf",b.Mx),D(),H("ngIf",0<b.oj.length),D(),H("ngIf",0<b.Vf.length),D(),H("ngIf",b.Pn),D(),nq("resizing",b.fs),D(),H("matTooltip",b.gC),D(2),Ar(b.fC))},Ha:[ht,Zs,ct,gt,QP,
BB,AB,yB,wD,bE,hG,AE,fG,TP,iI,hI,nO,ZK,RQ,HG,EG],styles:["[_nghost-%COMP%]{position:relative;width:370px;min-width:370px}.container[_ngcontent-%COMP%]{width:100%;height:100%;-moz-box-sizing:border-box;box-sizing:border-box;border-left:1px solid #e1e3e1;background-color:#fff;font-size:12px;overflow-y:auto;position:relative}.container[_ngcontent-%COMP%]   .section[_ngcontent-%COMP%]{-webkit-transition:max-height .15s ease-out;transition:max-height .15s ease-out;padding-bottom:8px;-moz-box-sizing:border-box;box-sizing:border-box}.container[_ngcontent-%COMP%]   .section.collapsed[_ngcontent-%COMP%]{max-height:32px!important;overflow:hidden}.container[_ngcontent-%COMP%]   .section.inputs[_ngcontent-%COMP%]{padding-bottom:16px}.container[_ngcontent-%COMP%]   .section[_ngcontent-%COMP%]   .flat-items-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;margin-top:10px;gap:8px}.container[_ngcontent-%COMP%]   .section[_ngcontent-%COMP%]   .flat-item[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;padding:0 2px 0 12px;font-size:12px}.container[_ngcontent-%COMP%]   .section[_ngcontent-%COMP%]   .flat-item[_ngcontent-%COMP%]   .name-row[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}.container[_ngcontent-%COMP%]   .section[_ngcontent-%COMP%]   .flat-item[_ngcontent-%COMP%]   .name-row.search-match[_ngcontent-%COMP%]{background-color:#f5e25a}.container[_ngcontent-%COMP%]   .section[_ngcontent-%COMP%]   .flat-item[_ngcontent-%COMP%]   .name-row[_ngcontent-%COMP%]   .index[_ngcontent-%COMP%]{color:#999;min-width:14px;margin-right:2px}.container[_ngcontent-%COMP%]   .section[_ngcontent-%COMP%]   .flat-item[_ngcontent-%COMP%]   .name-row[_ngcontent-%COMP%]   .name[_ngcontent-%COMP%]{color:#00639b;font-weight:500}.container[_ngcontent-%COMP%]   .section[_ngcontent-%COMP%]   .flat-item[_ngcontent-%COMP%]   .name-row[_ngcontent-%COMP%]   .name.locator[_ngcontent-%COMP%]{-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;cursor:pointer}.container[_ngcontent-%COMP%]   .section[_ngcontent-%COMP%]   .flat-item[_ngcontent-%COMP%]   .name-row[_ngcontent-%COMP%]   .name.locator[_ngcontent-%COMP%]:hover{background-color:#f6f6f6}.container[_ngcontent-%COMP%]   .section[_ngcontent-%COMP%]   .flat-item[_ngcontent-%COMP%]   .name-row[_ngcontent-%COMP%]   .name.locator[_ngcontent-%COMP%]:hover   .locator-icon-container[_ngcontent-%COMP%]{opacity:.8}.container[_ngcontent-%COMP%]   .section[_ngcontent-%COMP%]   .flat-item[_ngcontent-%COMP%]   .metadata-table[_ngcontent-%COMP%]{margin-top:3px;margin-left:18px;margin-right:0}.container[_ngcontent-%COMP%]   .section[_ngcontent-%COMP%]:not(:first-child){border-top:1px solid #dadce0}.container[_ngcontent-%COMP%]   .header[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;font-size:11px;font-weight:700;padding:4px 12px 0;text-transform:uppercase;letter-spacing:.0327em;position:-webkit-sticky;position:sticky;top:0;z-index:100;-webkit-flex-shrink:0;-ms-flex-negative:0;flex-shrink:0;-moz-box-sizing:border-box;box-sizing:border-box;background-color:#fff;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;color:rgba(0,0,0,.87)}.container[_ngcontent-%COMP%]   .header.identical-groups[_ngcontent-%COMP%], .container[_ngcontent-%COMP%]   .header.input[_ngcontent-%COMP%], .container[_ngcontent-%COMP%]   .header.output[_ngcontent-%COMP%]{-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between}.container[_ngcontent-%COMP%]   .header[_ngcontent-%COMP%]   .header-label-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}.container[_ngcontent-%COMP%]   .header[_ngcontent-%COMP%]   button.toggle[_ngcontent-%COMP%]{padding:0;width:24px;height:24px;margin-left:-8px;margin-right:2px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center}.container[_ngcontent-%COMP%]   .header[_ngcontent-%COMP%]   button.toggle[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{color:inherit;font-size:20px;width:20px;height:20px}.container[_ngcontent-%COMP%]   .header[_ngcontent-%COMP%]   button.toggle[_ngcontent-%COMP%]     .mat-mdc-button-touch-target{display:none}.container[_ngcontent-%COMP%]   .output-item-container[_ngcontent-%COMP%]{font-family:Roboto,sans-serif}.container[_ngcontent-%COMP%]   .output-item-container[_ngcontent-%COMP%]:not(.last){margin-top:8px}.container[_ngcontent-%COMP%]   .output-item-label[_ngcontent-%COMP%]{font-size:13px;padding:4px 12px;color:#00639b;font-weight:500}.container[_ngcontent-%COMP%]   .metadata-table[_ngcontent-%COMP%]{border-spacing:0;border-collapse:collapse;font-size:12px;margin:0 12px;word-break:break-all;max-width:calc(100% - 28px)}.container[_ngcontent-%COMP%]   .metadata-table.info-attrs[_ngcontent-%COMP%]{margin-left:30px}.container[_ngcontent-%COMP%]   .metadata-table[_ngcontent-%COMP%]   tr[_ngcontent-%COMP%]{vertical-align:text-top}.container[_ngcontent-%COMP%]   .metadata-table[_ngcontent-%COMP%]   tr.search-match[_ngcontent-%COMP%]{background-color:#f5e25a}.container[_ngcontent-%COMP%]   .metadata-table[_ngcontent-%COMP%]   tr[_ngcontent-%COMP%]:hover.has-hover-values{cursor:pointer;background-color:#f6f6f6}.container[_ngcontent-%COMP%]   .metadata-table[_ngcontent-%COMP%]   tr[_ngcontent-%COMP%]   mat-icon.bullet[_ngcontent-%COMP%]{-webkit-flex-shrink:0;-ms-flex-negative:0;flex-shrink:0;font-size:4px;width:5px;height:5px;color:#bbb;margin-right:4px;margin-top:6px}.container[_ngcontent-%COMP%]   .metadata-table[_ngcontent-%COMP%]   td.key[_ngcontent-%COMP%]{color:#999;padding-right:6px;white-space:nowrap;min-width:100px;width:100px;max-width:100px;overflow:hidden;text-overflow:ellipsis;position:relative}.container[_ngcontent-%COMP%]   .metadata-table[_ngcontent-%COMP%]   td.key[_ngcontent-%COMP%]   .key-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:start;-webkit-align-items:flex-start;-moz-box-align:start;-ms-flex-align:start;align-items:flex-start;overflow:hidden;position:absolute;inset:0}.container[_ngcontent-%COMP%]   .metadata-table[_ngcontent-%COMP%]   td.value[_ngcontent-%COMP%]{max-width:10px;overflow:hidden;text-overflow:ellipsis;white-space:nowrap}.container[_ngcontent-%COMP%]   .metadata-table[_ngcontent-%COMP%]   td.value[_ngcontent-%COMP%]:has(expandable-info-text.expanded){white-space:normal}.container[_ngcontent-%COMP%]   .metadata-table[_ngcontent-%COMP%]   .target-node-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;cursor:pointer}.container[_ngcontent-%COMP%]   .metadata-table[_ngcontent-%COMP%]   .target-node-container[_ngcontent-%COMP%]:hover   .locator-icon-container[_ngcontent-%COMP%]{opacity:.8}.container[_ngcontent-%COMP%]   .metadata-table[_ngcontent-%COMP%]   .target-node-container.search-match[_ngcontent-%COMP%]{background-color:#f5e25a}.container[_ngcontent-%COMP%]   .metadata-table[_ngcontent-%COMP%]   .values-hover-trigger[_ngcontent-%COMP%]{-moz-box-sizing:border-box;box-sizing:border-box;border-radius:99px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;background-color:#d7f1ff;color:#00639b;width:-webkit-fit-content;width:-moz-fit-content;width:fit-content;height:16px;padding:0 6px;font-size:10px}.container[_ngcontent-%COMP%]   .items-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;gap:4px;margin-top:10px;margin-bottom:6px}.container[_ngcontent-%COMP%]   .item-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;padding:0 12px}.container[_ngcontent-%COMP%]   .item-container.search-match[_ngcontent-%COMP%]{background-color:#f5e25a}.container[_ngcontent-%COMP%]   .item-container[_ngcontent-%COMP%]   .label-row[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between}.container[_ngcontent-%COMP%]   .item-container[_ngcontent-%COMP%]   .label[_ngcontent-%COMP%]{font-weight:500;color:#aaa;font-size:12px;margin-bottom:1px}.container[_ngcontent-%COMP%]   .item-container[_ngcontent-%COMP%]   .show-on-node-toggle[_ngcontent-%COMP%]{height:16px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;font-size:11px;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;-moz-box-sizing:border-box;box-sizing:border-box;cursor:pointer;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;-webkit-transform:scale(.7);transform:scale(.7);margin-right:-6px;margin-top:-1px}.container[_ngcontent-%COMP%]   .item-container[_ngcontent-%COMP%]   .value[_ngcontent-%COMP%]{font-size:13px;word-break:break-word}.container[_ngcontent-%COMP%]   .item-container[_ngcontent-%COMP%]   .value.big-text[_ngcontent-%COMP%]{font-size:14px;font-weight:500}.container[_ngcontent-%COMP%]   .item-container[_ngcontent-%COMP%]   .value.has-bg-color[_ngcontent-%COMP%]{padding:2px 4px;border-radius:4px;margin-top:2px}.container[_ngcontent-%COMP%]   .locator-icon-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;opacity:.5;margin-left:4px}.container[_ngcontent-%COMP%]   .locator-icon-container.left[_ngcontent-%COMP%]{margin-right:4px;margin-left:0;display:inline-block}.container[_ngcontent-%COMP%]   .locator-icon-container.left[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{margin-bottom:-2px}.container[_ngcontent-%COMP%]   .locator-icon-container[_ngcontent-%COMP%]:hover{opacity:.8}.container[_ngcontent-%COMP%]   .locator-icon-container[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{color:#333;font-size:12px;width:12px;height:12px}.resizer[_ngcontent-%COMP%]{position:absolute;top:0;bottom:0;left:0;width:6px;cursor:ew-resize;background-color:transparent;z-index:200}.resizer.resizing[_ngcontent-%COMP%], .resizer[_ngcontent-%COMP%]:hover{background-color:rgba(0,0,0,.04)}.hide-toggle[_ngcontent-%COMP%]{position:absolute;right:100%;bottom:2px;width:16px;height:20px;border:1px solid #ccc;border-right:none;-moz-box-sizing:border-box;box-sizing:border-box;border-radius:99px;border-top-right-radius:0;border-bottom-right-radius:0;cursor:pointer;background-color:#fff}.hide-toggle[_ngcontent-%COMP%]:hover{background-color:#eee}.hide-toggle[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{width:18px;height:18px;font-size:18px;color:#999}  xap-inline-dialog-container:has(.model-explorer-const-values-popup){width:100%}  .model-explorer-const-values-popup{padding:8px;line-height:12px;background-color:#fff}  .model-explorer-const-values-popup .values-count-msg{margin-bottom:8px;color:#999;font-size:12px}  .model-explorer-const-values-popup .content{font-size:11px;white-space:pre-wrap;font-family:monospace}  .model-explorer-const-values-popup .namespace-row{font-size:12px;padding:3px 0}"],
Ra:0});const PR=["panesContainer"],QR=(a,b)=>b.label;function RR(a){a&1&&Mq(0,"mat-spinner",21)}function SR(a){a&1&&(R(0,"mat-icon"),Z(1,"check_circle"),S())}function TR(a,b){a&1&&(R(0,"div",20),Ap(1,RR,1,0,"mat-spinner",21)(2,SR,2,0,"mat-icon"),R(3,"div",22),Z(4),S()());a&2&&(a=b.da,D(),Eq(a.Tr?1:2),D(3),Br(" ",a.label," "))}function UR(a){a&1&&(R(0,"div",19),Jq(1,TR,5,2,"div",20,QR),S());if(a&2){a=Y(2).da;const b=Y();D();Lq(b.to[a.id]||[])}}
function VR(a){a&1&&(R(0,"div",11)(1,"div",12),Mq(2,"mat-spinner",13),Z(3," Processing graph... "),S(),R(4,"div",14),Z(5," Please remain on this page while the model is being processed "),S(),R(6,"div",15)(7,"div",16),Mq(8,"div",17),S(),R(9,"div",18)(10,"mat-icon"),Z(11,"info"),S()(),Ap(12,UR,3,0,"ng-template",null,1,Qr),S()());if(a&2){a=zr(13);const b=Y().da,c=Y();H("@hideProcessingPanel",void 0)("@.disabled",c.Gv);D(8);lq("width",WR(c,b.id),"%");D();H("xapInlineDialog",a)("hoverDelayMs",100)}}
function YR(a){if(a&1){const b=Ug();R(0,"div",23)(1,"div",24);Z(2);S();R(3,"div",25)(4,"div",26);V("click",function(){u(b);const c=Y(2);GJ(c.oe);return x()})("mousedown",function(c){u(b);return x(c.stopPropagation())});R(5,"mat-icon");Z(6,"swap_horiz");S()();R(7,"div",27);V("click",function(){u(b);const c=Y().da,d=Y();FJ(d.oe,c.id);return x()})("mousedown",function(c){u(b);return x(c.stopPropagation())});R(8,"mat-icon");Z(9,"close");S()()()()}a&2&&(a=Y().da,Y(),D(2),Br(" ",ZR(a)," "))}
function $R(a){a&1&&Mq(0,"graph-panel",28);if(a&2){a=Y().da;const b=Y();H("@showModelGraph",void 0)("@.disabled",b.Gv)("modelGraph",a.modelGraph)("paneId",a.id)("rendererId",a.id)}}function aS(a){a&1&&Mq(0,"info-panel",10);a&2&&(a=Y().da,H("paneId",a.id))}
function bS(a,b){a&1&&(R(0,"div",5),Ap(1,VR,14,6,"div",6)(2,YR,10,1,"div",7),R(3,"div",8),Ap(4,$R,1,5,"graph-panel",9)(5,aS,1,1,"info-panel",10),S()());if(a&2){a=b.da;b=b.index;const c=Y();lq("width",100*(0===b?c.aj:1-c.aj),"%");nq("selected",a.id===c.oe.J());Kp("data-pane-id",a.id);D();H("ngIf",!a.modelGraph);D();H("ngIf",c.dw);D(2);H("ngIf",a.modelGraph);D();Eq(c.bE?5:-1)}}
function cS(a){if(a&1){const b=Ug();R(0,"div",29);V("mousedown",function(c){u(b);const d=Y(),e=zr(1);return x(d.Pq(c,e))});Mq(1,"div",30);S()}a&2&&(a=Y(),lq("left",a.JD))}function WR(a,b){return(a=a.to[b])?a.filter(c=>!c.Tr).length/a.length*100:0}function ZR(a){return(a=a.modelGraph)?`${a.collectionLabel} | ${a.id}`:"-"}function dS(a,b){var c=b.paneId;nJ(a.oe,c)&&(c=a.to[c].find(d=>d.label===b.label),null!=c&&(c.Tr=!1,cn(a.C)))}
var eS=class{constructor(a,b,c){this.C=a;this.oe=b;this.D=c;this.to={};this.aj=1;this.La=this.oe.La;Js(()=>{const d=this.La();1<=d.length&&(this.aj=d[0].widthFraction);for(const e of d)e.modelGraph||(this.to[e.id]=OI.map(f=>({label:f,Tr:!0})));cn(this.C)});this.D.C.addEventListener("message",d=>{switch(d.data.eventType){case 8:dS(this,d.data)}})}pd(){this.rD.R.addEventListener("mousedown",a=>{(a=a.target.closest(".pane-container"))&&jJ(this.oe,a.dataset.paneId||"")},!0)}Pq(a,b){a.preventDefault();
document.body.style.cursor="ew-resize";const c=lk(document,"mousemove"),d=lk(window,"mouseup"),e=this.oe.La()[0].widthFraction,f=b.offsetWidth,g=f*e,k=a.clientX;bn(this.C);C(Pj([c]),Lo(d)).subscribe({next:([m])=>{this.aj=Math.min(f-200,Math.max(200,g+(m.clientX-k)))/f;bn(this.C)},complete:()=>{document.body.style.cursor="default";AJ(this.oe,this.aj);bn(this.C)}})}BE(a,b){return b.id}get dw(){return 1<this.oe.La().length}get JD(){return`calc(${100*this.aj}% - 5px)`}get bE(){let a;return!(null==(a=
this.oe.config())?0:a.hideInfoPanel)}get Gv(){return this.oe.C}};eS.K=function(a){return new (a||eS)(z(Sr),z(VJ),z(bJ))};
eS.Da=Gf({type:eS,ha:[["split-panes-container"]],fb:function(a,b){a&1&&ur(PR,5);if(a&2){let c;vr(c=wr())&&(b.rD=c.first)}},ka:!0,features:[Mr],za:4,Aa:3,Ca:[["panesContainer",""],["progressPopup",""],[1,"container"],["class","pane-container",3,"selected","width",4,"ngFor","ngForOf","ngForTrackBy"],["class","resizer split-pane",3,"left","mousedown",4,"ngIf"],[1,"pane-container"],["class","loading",4,"ngIf"],["class","pane-title-container",4,"ngIf"],[1,"pane-content"],[3,"modelGraph","paneId","rendererId",
4,"ngIf"],[3,"paneId"],[1,"loading"],[1,"processing-msg-container"],["color","primary","diameter","18"],[1,"stay-on-page-info"],[1,"progress-bar-wrapper"],[1,"progress-bar-container"],[1,"progress-bar-body"],["dialogLabel","progress details",1,"icon-container",3,"xapInlineDialog","hoverDelayMs"],[1,"model-explorer-processing-tasks-container"],[1,"processing-task-container"],["color","primary","diameter","16"],[1,"task-label"],[1,"pane-title-container"],[1,"pane-title"],[1,"buttons-container"],["matTooltip",
"Swap pane",1,"icon-container","swap",3,"click","mousedown"],["matTooltip","Close pane",1,"icon-container","close",3,"click","mousedown"],[3,"modelGraph","paneId","rendererId"],[1,"resizer","split-pane",3,"mousedown"],[1,"resizer-line"]],ta:function(a,b){a&1&&(R(0,"div",2,0),Ap(2,bS,6,9,"div",3)(3,cS,2,2,"div",4),S());a&2&&(D(2),H("ngForOf",b.La())("ngForTrackBy",b.BE),D(),H("ngIf",b.dw))},Ha:[ht,Zs,ct,FP,wD,vD,sD,hG,AE,fG,OR,iI,hI,HG,EG],styles:[".container[_ngcontent-%COMP%]{-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;position:relative;width:100%;height:100%}.container[_ngcontent-%COMP%]   .pane-container[_ngcontent-%COMP%]{position:relative;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column}.container[_ngcontent-%COMP%]   .pane-container[_ngcontent-%COMP%]   .pane-title-container[_ngcontent-%COMP%]{height:24px;-moz-box-sizing:border-box;box-sizing:border-box;background-color:#eee;padding:0 12px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between;font-size:12px;color:#999;cursor:pointer;-webkit-flex-shrink:0;-ms-flex-negative:0;flex-shrink:0}.container[_ngcontent-%COMP%]   .pane-container[_ngcontent-%COMP%]   .pane-title-container[_ngcontent-%COMP%]   .buttons-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}.container[_ngcontent-%COMP%]   .pane-container[_ngcontent-%COMP%]   .pane-title-container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;cursor:pointer;color:#999;opacity:.8;margin-left:6px}.container[_ngcontent-%COMP%]   .pane-container[_ngcontent-%COMP%]   .pane-title-container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]:hover{opacity:1}.container[_ngcontent-%COMP%]   .pane-container[_ngcontent-%COMP%]   .pane-title-container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{font-size:18px;height:18px;width:18px}.container[_ngcontent-%COMP%]   .pane-container[_ngcontent-%COMP%]   .pane-content[_ngcontent-%COMP%]{width:100%;-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;overflow:hidden}.container[_ngcontent-%COMP%]   .pane-container[_ngcontent-%COMP%]   .pane-content[_ngcontent-%COMP%]   graph-panel[_ngcontent-%COMP%]{-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1}.container[_ngcontent-%COMP%]   .pane-container.selected[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{color:#fff}.container[_ngcontent-%COMP%]   .pane-container.selected[_ngcontent-%COMP%]   .pane-title-container[_ngcontent-%COMP%]{background-color:#ea8600;color:#fff}.container[_ngcontent-%COMP%]   .pane-container[_ngcontent-%COMP%]   .loading[_ngcontent-%COMP%]{position:absolute;top:0;left:0;width:100%;height:100%;z-index:150;color:#999;background-color:#fff;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column}.container[_ngcontent-%COMP%]   .pane-container[_ngcontent-%COMP%]   .loading[_ngcontent-%COMP%]   .processing-msg-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}.container[_ngcontent-%COMP%]   .pane-container[_ngcontent-%COMP%]   .loading[_ngcontent-%COMP%]   .processing-msg-container[_ngcontent-%COMP%]   mat-spinner[_ngcontent-%COMP%]{margin-right:8px}.container[_ngcontent-%COMP%]   .pane-container[_ngcontent-%COMP%]   .loading[_ngcontent-%COMP%]   .progress-bar-wrapper[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;margin-top:18px;margin-right:-30px}.container[_ngcontent-%COMP%]   .pane-container[_ngcontent-%COMP%]   .loading[_ngcontent-%COMP%]   .progress-bar-wrapper[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]{cursor:pointer;opacity:.8;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center}.container[_ngcontent-%COMP%]   .pane-container[_ngcontent-%COMP%]   .loading[_ngcontent-%COMP%]   .progress-bar-wrapper[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]:hover{opacity:1}.container[_ngcontent-%COMP%]   .pane-container[_ngcontent-%COMP%]   .loading[_ngcontent-%COMP%]   .progress-bar-wrapper[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{color:#ccc;font-size:18px;width:18px;height:18px;margin-left:6px}.container[_ngcontent-%COMP%]   .pane-container[_ngcontent-%COMP%]   .loading[_ngcontent-%COMP%]   .progress-bar-container[_ngcontent-%COMP%]{height:7px;width:280px;border:1px solid #ccc;border-radius:99px;-moz-box-sizing:border-box;box-sizing:border-box;overflow:hidden}.container[_ngcontent-%COMP%]   .pane-container[_ngcontent-%COMP%]   .loading[_ngcontent-%COMP%]   .progress-bar-container[_ngcontent-%COMP%]   .progress-bar-body[_ngcontent-%COMP%]{height:100%;-webkit-transition:width .1s;transition:width .1s;background-color:#ccc}.container[_ngcontent-%COMP%]   .pane-container[_ngcontent-%COMP%]   .loading[_ngcontent-%COMP%]   .stay-on-page-info[_ngcontent-%COMP%]{font-size:12px;width:100%;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;margin-top:4px}.container[_ngcontent-%COMP%]   .resizer[_ngcontent-%COMP%]{height:100%;width:10px;top:0;left:50%;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;position:absolute;padding-left:1px;cursor:ew-resize;z-index:200}.container[_ngcontent-%COMP%]   .resizer[_ngcontent-%COMP%]:hover{background-color:rgba(0,0,0,.04)}.container[_ngcontent-%COMP%]   .resizer[_ngcontent-%COMP%]:hover   .resizer-line[_ngcontent-%COMP%]{border-color:#fcc986}.container[_ngcontent-%COMP%]   .resizer[_ngcontent-%COMP%]   .resizer-line[_ngcontent-%COMP%]{height:100%;width:1px;border-left:1px solid #999}  .model-explorer-processing-tasks-container{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;padding:12px}  .model-explorer-processing-tasks-container .processing-task-container{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;font-size:13px;margin:2px 0}  .model-explorer-processing-tasks-container .processing-task-container mat-spinner{margin-right:8px;width:16px}  .model-explorer-processing-tasks-container .processing-task-container mat-icon{color:#0a0;font-size:16px;width:16px;height:16px;margin-right:8px}"],
data:{animation:[Yy("hideProcessingPanel",[bz("* => void",Zy("150ms 100ms ease-out",$y({opacity:0,transform:"scale(0.95, 0.95)"})))]),Yy("showModelGraph",[az("void",$y({opacity:0,transform:"scale(1.03, 1.03)"})),bz("void => *",Zy("150ms 100ms ease-out",$y({opacity:1,transform:"scale(1, 1)"})))])]},Ra:0});function fS(a){a&1&&(R(0,"mat-icon",15),Z(1," turn_left "),S())}function gS(a){if(a&1){const b=Ug();R(0,"div",16);V("click",function(c){u(b);const d=Y().da,e=Y(2);return x(e.Jq(c,d))});R(1,"div",17);Mq(2,"div",18);S()()}}
function hS(a,b){if(a&1){const c=Ug();R(0,"div",7)(1,"div",8)(2,"div",9);V("click",function(){const d=u(c).da,e=Y(2);e.he.next({});e.dk="";e.qj=!1;qJ(e.vf,d.graph);e.vf.N.set(void 0);wJ(e.vf,e.vf.J());e.vf.F.set(void 0);sJ(e.vf,!1);return x()});R(3,"div",10);Ap(4,fS,2,0,"mat-icon",11);R(5,"div",12);Z(6);S()();R(7,"div",13);Z(8);S()();Ap(9,gS,3,0,"div",14);S()()}a&2&&(a=b.da,b=Y(2),Kp("data-id",a.id),D(),lq("padding-left",iS(b,a),"px"),nq("selected",a.id===b.selectedGraphId()),D(3),H("ngIf",jS(b,a)),
D(2),Br(" ",a.id," "),D(2),Br(" ",kS(a.yr)," "),D(),H("ngIf",b.us))}function lS(a,b){a&1&&(R(0,"div",4)(1,"div",5),Z(2),S(),Ap(3,hS,10,9,"div",6),S());a&2&&(a=b.da,b=Y(),D(2),Br(" ",a.label," "),D(),H("ngForOf",a.graphs)("ngForTrackBy",b.zE))}function iS(a,b){if(a.qj)return 24;let c;return 24+12*(null!=(c=b.graph.level)?c:0)}function jS(a,b){let c;return!a.qj&&0<(null!=(c=b.graph.level)?c:0)}function kS(a){return`${a} node${1===a?"":"s"}`}
var mS=class{constructor(a){this.vf=a;this.Jh=[];this.he=new Fk;this.qj=!1;this.selectedGraphId=Bs(()=>{const b=rJ(this.vf);return b&&b.modelGraph?b.modelGraph.id:""});this.dk=""}Nq(a){this.dk=a.toLowerCase()}Jq(a,b){a.stopPropagation();this.he.next({});this.dk="";this.qj=!1;hJ(this.vf,b.graph)}yE(a){return`${a}`}zE(a,b){return`${b.graph.collectionLabel}___${b.graph.id}`}get rB(){const a=[];this.qj=!1;for(const {label:b,collection:c,graphs:d}of this.Jh){const e={label:b,collection:c,graphs:[]};for(const f of d)""===
this.dk||f.id.toLowerCase().includes(this.dk)?e.graphs.push(f):this.qj=!0;0<e.graphs.length&&a.push(e)}return a}get us(){return 1===this.vf.La().length}};mS.K=function(a){return new (a||mS)(z(VJ))};
mS.Da=Gf({type:mS,ha:[["graph-selector-panel"]],inputs:{Jh:"graphCollectionItems"},outputs:{he:"onClose"},ka:!0,features:[Mr],za:4,Aa:3,Ca:[["input",""],[1,"container"],["placeholder","Filter graphs",3,"input"],["class","collection-container",4,"ngFor","ngForOf","ngForTrackBy"],[1,"collection-container"],[1,"collection-label"],["class","graph-item-container",4,"ngFor","ngForOf","ngForTrackBy"],[1,"graph-item-container"],[1,"graph-item"],[1,"graph-info-wrapper",3,"click"],[1,"graph-label-wrapper"],
["class","indent-symbol",4,"ngIf"],[1,"graph-label"],[1,"node-count-label"],["class","action-button","matTooltip","Open in split pane","matTooltipPosition","right",3,"click",4,"ngIf"],[1,"indent-symbol"],["matTooltip","Open in split pane","matTooltipPosition","right",1,"action-button",3,"click"],[1,"block-container"],[1,"right-block"]],ta:function(a,b){if(a&1){const c=Ug();R(0,"div",1)(1,"input",2,0);V("input",function(){u(c);const d=zr(2);return x(b.Nq(d.value))});S();Ap(3,lS,4,3,"div",3);S()}a&
2&&(H("@transformPanel","showing"),D(3),H("ngForOf",b.rB)("ngForTrackBy",b.yE))},Ha:[ht,Zs,ct,hG,AE,fG,cB,iI,hI,PD,Cx],styles:[".container[_ngcontent-%COMP%]{box-shadow:0 3px 5px -1px rgba(0,0,0,.2),0 6px 10px 0 rgba(0,0,0,.14),0 1px 18px 0 rgba(0,0,0,.12);-webkit-transform-origin:top center;transform-origin:top center;font-size:12px;padding:8px 0;border-radius:4px;border-top-left-radius:0;border-top-right-radius:0;max-height:100%;overflow-y:auto}.container[_ngcontent-%COMP%]   input[_ngcontent-%COMP%]{font-size:12px;margin:4px 10px;width:calc(100% - 20px);-moz-box-sizing:border-box;box-sizing:border-box;pointer-events:all;border:1px solid #ccc;height:20px;border-radius:4px;padding:0 2px}.container[_ngcontent-%COMP%]   .collection-container[_ngcontent-%COMP%]   .collection-label[_ngcontent-%COMP%]{font-weight:700;padding:4px 12px}.container[_ngcontent-%COMP%]   .graph-item-container[_ngcontent-%COMP%]   .graph-item[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between;padding:4px 12px 4px 24px;cursor:pointer}.container[_ngcontent-%COMP%]   .graph-item-container[_ngcontent-%COMP%]   .graph-item[_ngcontent-%COMP%]:hover{background-color:rgba(0,0,0,.04)}.container[_ngcontent-%COMP%]   .graph-item-container[_ngcontent-%COMP%]   .graph-item.selected[_ngcontent-%COMP%]{background-color:#c2e7ff}.container[_ngcontent-%COMP%]   .graph-item-container[_ngcontent-%COMP%]   .graph-item[_ngcontent-%COMP%]   .graph-info-wrapper[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between;-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1}.container[_ngcontent-%COMP%]   .graph-item-container[_ngcontent-%COMP%]   .graph-item[_ngcontent-%COMP%]   .graph-label-wrapper[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}.container[_ngcontent-%COMP%]   .graph-item-container[_ngcontent-%COMP%]   .graph-item[_ngcontent-%COMP%]   .indent-symbol[_ngcontent-%COMP%]{font-size:16px;width:16px;height:16px;color:#999;margin-right:4px;-webkit-transform:rotate(-180deg);transform:rotate(-180deg)}.container[_ngcontent-%COMP%]   .graph-item-container[_ngcontent-%COMP%]   .graph-item[_ngcontent-%COMP%]   .graph-label[_ngcontent-%COMP%]{overflow:hidden;white-space:nowrap;text-overflow:ellipsis}.container[_ngcontent-%COMP%]   .graph-item-container[_ngcontent-%COMP%]   .graph-item[_ngcontent-%COMP%]   .node-count-label[_ngcontent-%COMP%]{color:#999;margin-left:12px}.container[_ngcontent-%COMP%]   .graph-item-container[_ngcontent-%COMP%]   .graph-item[_ngcontent-%COMP%]   .action-button[_ngcontent-%COMP%]{-webkit-flex-shrink:0;-ms-flex-negative:0;flex-shrink:0;width:16px;height:16px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;margin-left:6px;opacity:.8;cursor:pointer}.container[_ngcontent-%COMP%]   .graph-item-container[_ngcontent-%COMP%]   .graph-item[_ngcontent-%COMP%]   .action-button[_ngcontent-%COMP%]:hover{opacity:1}.container[_ngcontent-%COMP%]   .graph-item-container[_ngcontent-%COMP%]   .graph-item[_ngcontent-%COMP%]   .action-button[_ngcontent-%COMP%]   .block-container[_ngcontent-%COMP%]{border-radius:3px;border:1px solid #999;height:100%;width:100%;-moz-box-sizing:border-box;box-sizing:border-box;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:end;-webkit-justify-content:flex-end;-moz-box-pack:end;-ms-flex-pack:end;justify-content:flex-end;padding:1px}.container[_ngcontent-%COMP%]   .graph-item-container[_ngcontent-%COMP%]   .graph-item[_ngcontent-%COMP%]   .action-button[_ngcontent-%COMP%]   .block-container[_ngcontent-%COMP%]   .right-block[_ngcontent-%COMP%]{height:100%;width:50%;background-color:#999;border-top-right-radius:2px;border-bottom-right-radius:2px}"],
data:{animation:XJ},Ra:0});const nS=["input"];function oS(a){a&1&&(R(0,"div",15),Z(1),S());a&2&&(a=Y(),D(),Br(" ",a.Fx," nodes "))}function pS(a){if(a&1){const b=Ug();R(0,"mat-optgroup",16)(1,"mat-option")(2,"input",17,1);V("click",function(c){u(b);return x(c.stopPropagation())})("keydown.space",function(c){u(b);return x(c.stopPropagation())})("input",function(){u(b);const c=zr(3),d=Y();return x(d.Nq(c.value))});S()()()}}
function qS(a){if(a&1){const b=Ug();R(0,"div",24);V("click",function(c){u(b);const d=Y().da,e=Y(2);return x(e.Jq(c,d))});R(1,"div",25);Mq(2,"div",26);S()()}}function rS(a,b){a&1&&(R(0,"mat-option",20)(1,"div",21)(2,"div",22)(3,"div",7),Z(4),S(),R(5,"div",15),Z(6),S()(),Ap(7,qS,3,0,"div",23),S()());a&2&&(a=b.da,b=Y(2),nq("selected",a.graph===b.Xf.value),H("value",a.graph),Kp("data-id",a.id),D(4),Ar(a.id),D(2),Br(" ",a.yr," nodes "),D(),H("ngIf",b.us))}
function sS(a,b){a&1&&(R(0,"mat-optgroup",18),Ap(1,rS,8,7,"mat-option",19),S());a&2&&(a=b.da,H("label",a.label),D(),H("ngForOf",a.graphs))}const tS=new OffscreenCanvas(500,300),uS={};function vS(a,b=12,c=!1){var d=uS[a];null==d&&(d=tS.getContext("2d"),d.font=`${b}px "Google Sans Text", Arial, Helvetica, sans-serif`,c&&(d.font=`bold ${d.font}`),b=d.measureText(a).width,d=uS[a]=b);return d}
function wS(a,b){let c=!1;for(const d of a.Jh()){for(const e of d.graphs)if(e.graph.id===b){a.Fx=e.yr;a.os=d.label;a.Ex=d.collection;a.Xf.setValue(e.graph);c=!0;break}if(c)break}}function xS(a,b){b=new Au({uc:Ku(Rv(a.C.position(),b),[{cb:"end",eb:"bottom",Na:"end",Ta:"top"}]),maxHeight:"calc(100% - 70px)",Tb:!0,dc:"cdk-overlay-transparent-backdrop",vc:a.C.C.C(),hb:"graph-selector-panel"});const c=a.C.create(b);a.Ld=new Uv(mS,a.xb);c.dd().subscribe(()=>{c.dispose()});return c}
function yS(a,b){const c=xS(a,b);b=c.attach(a.Ld);b.instance.Jh=a.Jh();b.instance.he.subscribe(()=>{c.dispose()})}
var zS=class{constructor(a,b,c){this.Ud=a;this.C=b;this.xb=c;this.Xf=new vx;this.Fx=0;this.os="";this.Jh=Bs(()=>{var d=this.Ud.config();if(!d)return[];var e=this.Ud.O();this.nodeLabelsToHide=new Set((d.nodeLabelsToHide||[]).map(g=>g.toLowerCase()));d=[];const f=this.So().toLowerCase();for(const g of e){e={label:g.label,collection:g,graphs:[]};for(const k of g.graphs){if(""!==f&&!k.id.toLowerCase().includes(f))continue;const m=k.nodes.filter(t=>!this.nodeLabelsToHide.has(t.label.toLowerCase())).length,
p=vS(` ${k.id}    ${m} nodes`)+30;e.graphs.push({id:k.id,graph:k,yr:m,width:p})}0<e.graphs.length&&(d.push(e),vS(g.label,12,!0))}return d});this.WB=Bs(()=>{let d=0;const e=this.Ud.O();for(const f of e)d+=f.graphs.length;return d});this.nodeLabelsToHide=new Set;this.So=xr("");this.Ld=null;this.selectedGraphId=Bs(()=>{const d=rJ(this.Ud);return d&&d.modelGraph?d.modelGraph.id:""});Js(()=>{const d=this.selectedGraphId();d&&wS(this,d)})}Nq(a){this.So.set(a)}Jq(a,b){a.stopPropagation();this.TC.close();
hJ(this.Ud,b.graph)}get us(){return 1===this.Ud.La().length}};zS.K=function(a){return new (a||zS)(z(VJ),z(bw),z(kp))};
zS.Da=Gf({type:zS,ha:[["graph-selector"]],fb:function(a,b){a&1&&(ur(OD,5),ur(nS,5));if(a&2){let c;vr(c=wr())&&(b.TC=c.first);vr(c=wr())&&(b.OB=c.first)}},ka:!0,features:[Mr],za:21,Aa:6,Ca:[["selector",""],["input",""],[1,"container"],[1,"collection-label"],[1,"graph-selector-label",3,"click"],[1,"selector-label-container"],[1,"selector-label-content"],[1,"option-label"],["class","node-count-label",4,"ngIf"],[1,"triangle"],["matTooltip","Download processed graphs json",1,"mat-icon-container",3,"click"],
[1,"select-form"],["color","primary","panelClass","graph-selector-panel",3,"selectionChange","openedChange","formControl"],["class","graph-search-optgroup",4,"ngIf"],[3,"label",4,"ngFor","ngForOf"],[1,"node-count-label"],[1,"graph-search-optgroup"],["placeholder","Filter graphs",3,"click","keydown.space","input"],[3,"label"],[3,"selected","value",4,"ngFor","ngForOf"],[3,"value"],[1,"option-wrapper"],[1,"option-container"],["class","action-button","matTooltip","Open in split pane","matTooltipPosition",
"right",3,"click",4,"ngIf"],["matTooltip","Open in split pane","matTooltipPosition","right",1,"action-button",3,"click"],[1,"block-container"],[1,"right-block"]],ta:function(a,b){if(a&1){const c=Ug();R(0,"div",2)(1,"div",3);Z(2);S();R(3,"div",4,0);V("click",function(){u(c);const d=zr(4);return x(yS(b,d))});R(5,"div",5)(6,"div",6)(7,"div",7);Z(8);S();Ap(9,oS,2,1,"div",8);S();R(10,"div",9);Z(11,"\u25bc");S()()();R(12,"div",10);V("click",function(){u(c);if(null!=b.Ex){var d=document.createElement("a");
d.download=`${b.os}.json`;var e=`data:text/json;charset=utf-8, ${encodeURIComponent(JSON.stringify(b.Ex,null,2))}`;ud(d,rd(e));d.click()}return x()});R(13,"mat-icon");Z(14,"download");S()();R(15,"mat-form-field",11)(16,"mat-label");Z(17,"Select an option");S();R(18,"mat-select",12);V("selectionChange",function(){u(c);b.Xf.value&&(wS(b,b.Xf.value.id),qJ(b.Ud,b.Xf.value),b.Ud.N.set(void 0),wJ(b.Ud,b.Ud.J()),b.Ud.F.set(void 0),sJ(b.Ud,!1));return x()})("openedChange",function(d){u(c);d||(b.OB.R.value=
"",b.So.set(""));return x()});Ap(19,pS,4,0,"mat-optgroup",13)(20,sS,2,2,"mat-optgroup",14);S()()()}a&2&&(D(2),Br(" ",b.os," "),D(6),Ar((null==b.Xf.value?null:b.Xf.value.id)||"-"),D(),H("ngIf",b.Xf.value),D(9),H("formControl",b.Xf),D(),H("ngIf",1<b.WB()),D(),H("ngForOf",b.Jh()))},Ha:[ht,Zs,ct,hG,AE,fG,cB,bB,qA,iI,hI,PD,OD,Vz,vz,Cx,Mw,xx],styles:[".container[_ngcontent-%COMP%]{font-size:12px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;line-height:14px;position:relative}.container[_ngcontent-%COMP%]   .collection-label[_ngcontent-%COMP%]{font-weight:700;margin-right:6px}.container[_ngcontent-%COMP%]   .select-form[_ngcontent-%COMP%]{height:0;max-height:0;position:absolute;top:26px;right:28px}.container[_ngcontent-%COMP%]   .select-form[_ngcontent-%COMP%]    >*{height:0}.container[_ngcontent-%COMP%]   .selector-label-content[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between;-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1}.container[_ngcontent-%COMP%]   .option-label[_ngcontent-%COMP%]{overflow:hidden;white-space:nowrap;text-overflow:ellipsis;margin-right:2px}.container[_ngcontent-%COMP%]   .node-count-label[_ngcontent-%COMP%]{color:#999;margin-left:12px}.container[_ngcontent-%COMP%]   .triangle[_ngcontent-%COMP%]{font-size:10px;margin-left:8px}.container[_ngcontent-%COMP%]   .graph-selector-label[_ngcontent-%COMP%]{font-size:12px;border:1px solid #aaa;border-radius:3px;padding:2px 4px;cursor:pointer;-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1;height:24px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-moz-box-sizing:border-box;box-sizing:border-box}.container[_ngcontent-%COMP%]   .graph-selector-label[_ngcontent-%COMP%]   .selector-label-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between;-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1}.container[_ngcontent-%COMP%]   .graph-selector-label[_ngcontent-%COMP%]:hover{border-color:#ea8600}.container[_ngcontent-%COMP%]   .mat-icon-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;margin-left:4px;cursor:pointer;opacity:.6}.container[_ngcontent-%COMP%]   .mat-icon-container[_ngcontent-%COMP%]:hover{opacity:1}  .graph-selector-panel{background-color:#fff}  .graph-selector-panel mat-option{height:24px;font-size:12px;min-height:unset;padding:1px 0;padding-left:20px!important;background-color:none!important}  .graph-selector-panel mat-option .option-wrapper{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}  .graph-selector-panel mat-option .option-container{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between;letter-spacing:normal;font-family:Google Sans Text,Arial,Helvetica,sans-serif;-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1}  .graph-selector-panel mat-option .option-container .option-label{overflow:hidden;white-space:nowrap;text-overflow:ellipsis;color:#000!important}  .graph-selector-panel mat-option .option-container .node-count-label{color:#999}  .graph-selector-panel mat-option .action-button{-webkit-flex-shrink:0;-ms-flex-negative:0;flex-shrink:0;width:16px;height:16px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;margin-left:4px;opacity:.8;cursor:pointer}  .graph-selector-panel mat-option .action-button:hover{opacity:1}  .graph-selector-panel mat-option .action-button .block-container{border-radius:3px;border:1px solid #999;height:100%;width:100%;-moz-box-sizing:border-box;box-sizing:border-box;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:end;-webkit-justify-content:flex-end;-moz-box-pack:end;-ms-flex-pack:end;justify-content:flex-end;padding:1px}  .graph-selector-panel mat-option .action-button .block-container .right-block{height:100%;width:50%;background-color:#999;border-top-right-radius:2px;border-bottom-right-radius:2px}  .graph-selector-panel mat-option span{white-space:break-spaces!important;word-break:break-all;line-height:14px!important;width:100%;-moz-box-sizing:border-box;box-sizing:border-box;padding-right:10px}  .graph-selector-panel mat-option mat-pseudo-checkbox{display:none}  .graph-selector-panel mat-option.selected{background-color:#fff1de!important}  .graph-selector-panel mat-optgroup span{min-height:unset;font-size:12px;font-weight:700;padding-left:8px}  .graph-selector-panel mat-optgroup.graph-search-optgroup{pointer-events:none}  .graph-selector-panel mat-optgroup.graph-search-optgroup span{padding:0}  .graph-selector-panel mat-optgroup.graph-search-optgroup mat-option{padding-left:16px!important;padding-right:12px!important;height:28px}  .graph-selector-panel mat-optgroup.graph-search-optgroup input{font-size:12px;width:100%;-moz-box-sizing:border-box;box-sizing:border-box;pointer-events:all;border:1px solid #ccc;height:20px;border-radius:4px;padding:0 2px}"],
Ra:0});function AS(a){a&1&&(R(0,"div",2),Z(1," Model Explorer "),R(2,"span",3),Z(3),S(),Z(4," is available. You are running "),R(5,"span",3),Z(6),S(),Z(7,". "),Mq(8,"br")(9,"br"),Z(10," Run the following command in your console to upgrade: "),R(11,"div",4),Z(12," pip install -U ai-edge-model-explorer "),S()());a&2&&(a=Y(2),D(3),Br("v",a.info().version,""),D(3),Br("v",a.info().runningVersion,""))}
function BS(a){a&1&&(R(0,"div",1)(1,"mat-icon"),Z(2,"upgrade"),S(),Z(3," New version available "),S(),Ap(4,AS,13,2,"ng-template",null,0,Qr));a&2&&(a=zr(5),H("xapInlineDialog",a)("hoverDelayMs",100))}function CS(a){sa(function*(){try{const b=yield fetch("/api/v1/check_new_version");if(b.ok){const c=yield b.json();a.info.set(c)}}catch(b){}})}var DS=class{constructor(){this.info=xr({version:"",runningVersion:""});CS(this)}};DS.K=function(a){return new (a||DS)};DS.pa=Md({ma:DS,ba:DS.K,ea:"root"});
var ES=class{constructor(a){this.C=a;this.info=this.C.info}};ES.K=function(a){return new (a||ES)(z(DS))};ES.Da=Gf({type:ES,ha:[["new-version-chip"]],ka:!0,features:[Mr],za:1,Aa:1,Ca:[["upgrade",""],["dialogLabel","upgrade",1,"container",3,"xapInlineDialog","hoverDelayMs"],[1,"model-explorer-upgrade-popup"],[1,"bold"],[1,"code"]],ta:function(a,b){a&1&&Ap(0,BS,6,2);a&2&&Eq(""!==b.info().version?0:-1)},Ha:[ht,iI,hI,HG,EG],styles:[".container[_ngcontent-%COMP%] {\n  font-size: 12px;\n  display: flex;\n  align-items: center;\n  color: #ab6c17;\n  background-color: #ffefd9;\n  line-height: 18px;\n  border-radius: 99px;\n  font-weight: 500;\n  padding: 2px 10px 2px 4px;\n  cursor: pointer;\n\n  mat-icon {\n    font-size: 16px;\n    width: 16px;\n    height: 16px;\n  }\n}\n\n  .model-explorer-upgrade-popup {\n  padding: 8px;\n  font-size: 12px;\n  background-color: white;\n  line-height: normal;\n\n  .bold {\n    font-weight: 500;\n  }\n\n  .code {\n    background-color: #f1f1f1;\n    font-family: monospace;\n    margin-top: 8px;\n    padding: 4px;\n    font-size: 11px;\n  }\n}"]});function FS(a){a&1&&Mq(0,"mat-spinner",6)}function GS(a){a&1&&(R(0,"mat-icon"),Z(1,"add_circle_outline"),S())}function HS(a){a&1&&(R(0,"div",8),Z(1," Add custom data and styles to the nodes in the current model "),S())}
function IS(a){if(a&1){const b=Ug();R(0,"div",16)(1,"div",17);Z(2,"OR");S()();R(3,"div",18)(4,"div",13);Z(5,"From CNS");S();R(6,"textarea",19,3);V("focus",function(){u(b);const c=zr(7);return x(c.select())})("input",function(){u(b);const c=Y(2);bn(c.ez);return x()})("change",function(){u(b);const c=zr(7),d=Y(2);PI("extension_dropdown_json_cns_path",c.value);d.Wq=c.value;return x()});S();R(8,"button",20);V("click",function(){u(b);const c=zr(7),d=Y(2);return x(JS(d,c))});Z(9);S()()}if(a&2){a=zr(7);
const b=Y(2);D(6);H("value",b.Wq);D(2);H("disabled",KS(b,a));D();Br(" ",b.di()?"Loading...":"Load"," ")}}
function LS(a){if(a&1){const b=Ug();R(0,"div",9)(1,"div",10)(2,"div");Z(3,"Load json data file");S();R(4,"a",11);Z(5," Guide ");S()();R(6,"div",12)(7,"div",13);Z(8,"From computer");S();R(9,"button",14);V("click",function(){u(b);const c=zr(12);return x(c.click())});Z(10," Upload ");S()();R(11,"input",15,2);V("change",function(){u(b);const c=zr(12),d=Y();return x(d.Kq(c))});S();Ap(13,IS,10,3);S()}a&2&&(a=Y(),D(13),Eq(a.ee?13:-1))}
function JS(a,b){return sa(function*(){const c=xJ(a.D);if(c){var d=b.value.trim();yield EL(a.C,d,c);var e;null==(e=a.Jv)||rG(e)}})}function KS(a,b){b=b.value.trim();return a.di()||""===b||!b.startsWith("/cns")||!b.endsWith(".json")}
var MS=class{constructor(a,b,c,d){this.D=a;this.ez=b;this.C=d;this.extensions=[];this.Bw=!0;this.Wq="";this.hd={minWidth:0,minHeight:0};this.GB={minWidth:0,minHeight:0,maxHeight:500};this.di=this.C.di;this.Wq=window.localStorage.getItem("extension_dropdown_json_cns_path")||""}Kq(a){var b=a.files;if(b){var c=xJ(this.D);if(c){for(const d of b)b=new FileReader,b.onload=e=>{const f=tI();try{let k;var g=null==(k=e.target)?void 0:k.result;const m=JSON.parse(g);e={};null!=m.results&&null==m.results.results?
c&&(e[c.id]=m):e=m;BL(this.C,f,d.name,c,e)}catch(k){BL(this.C,f,d.name,c),CL(this.C,f,{[c.id]:{results:{}}},c,`Failed to process JSON file. ${k}`)}finally{let k;null==(k=this.Jv)||rG(k)}},b.readAsText(d);a.value=""}}}get ee(){return!1}};MS.K=function(a){return new (a||MS)(z(VJ),z(Sr),z(QI),z(KL))};
MS.Da=Gf({type:MS,ha:[["node-data-provider-dropdown"]],fb:function(a,b){a&1&&ur(GG,5);if(a&2){let c;vr(c=wr())&&(b.Jv=c.first)}},ka:!0,features:[Mr],za:10,Aa:6,Ca:[["help",""],["dropdown",""],["input",""],["textarea",""],["dialogLabel","help",1,"container",3,"xapInlineDialog","overlaySize","hoverDelayMs"],["dialogLabel","dropdown",1,"icon-container",3,"xapInlineDialogClick","overlaySize"],["color","primary","diameter","18"],[1,"label"],[1,"model-explorer-help-popup"],[1,"model-explorer-node-data-provider-dropdown"],
[1,"section-label","upload"],["href","https://github.com/google-ai-edge/model-explorer/wiki/2.-User-Guide#custom-node-data","target","_blank"],[1,"upload-container"],[1,"description"],["mat-flat-button","","color","primary",1,"upload-json-file-button","upload",3,"click"],["type","file","multiple","","accept",".json",1,"upload-per-node-data-input",3,"change"],[1,"or-divider"],[1,"or-label"],[1,"load-json-from-cns-container"],[3,"focus","input","change","value"],["mat-flat-button","","color","primary",
1,"upload-json-file-button",3,"click","disabled"]],ta:function(a,b){a&1&&(R(0,"div",4)(1,"div",5),Ap(2,FS,1,0,"mat-spinner",6)(3,GS,2,0,"mat-icon"),R(4,"span",7),Z(5,"Add per-node data"),S()()(),Ap(6,HS,2,0,"ng-template",null,0,Qr)(8,LS,14,1,"ng-template",null,1,Qr));if(a&2){a=zr(7);const c=zr(9);H("xapInlineDialog",a)("overlaySize",b.hd)("hoverDelayMs",10);D();H("xapInlineDialogClick",c)("overlaySize",b.GB);D();Eq(b.di()?2:3)}},Ha:[ht,BB,AB,vB,wD,vD,sD,iI,hI,gG,HG,EG,GG],styles:[".container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;cursor:pointer;color:#666;padding-bottom:12px;margin-bottom:-12px}.container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{margin-right:4px}.container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]   mat-spinner[_ngcontent-%COMP%]{margin-right:6px}.container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]   .label[_ngcontent-%COMP%]{font-size:13px}.container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]:hover{color:#444}  .model-explorer-node-data-provider-dropdown{font-size:12px;background-color:#fff;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;padding-bottom:8px}  .model-explorer-node-data-provider-dropdown .section-label{padding:8px 12px;font-size:11px;background:#f1f1f1;font-weight:500;text-transform:uppercase}  .model-explorer-node-data-provider-dropdown .section-label .address-container{text-transform:none;font-weight:400;letter-spacing:normal;color:#777;line-height:12px;padding-bottom:5px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:start;-webkit-align-items:flex-start;-moz-box-align:start;-ms-flex-align:start;align-items:flex-start;margin-top:-4px;margin-left:-2px}  .model-explorer-node-data-provider-dropdown .section-label .address-container .address-label{overflow:hidden;line-break:anywhere;line-height:12px;-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1;padding-top:3px}  .model-explorer-node-data-provider-dropdown .section-label .address-container .settings-icon-container{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;margin-right:4px;pointer-events:all;cursor:pointer;opacity:.7}  .model-explorer-node-data-provider-dropdown .section-label .address-container .settings-icon-container:hover{opacity:1}  .model-explorer-node-data-provider-dropdown .section-label .address-container .settings-icon-container mat-icon{font-size:16px;width:16px;height:16px}  .model-explorer-node-data-provider-dropdown .section-label.upload{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between;min-width:180px}  .model-explorer-node-data-provider-dropdown .section-label.upload a{font-size:11px;letter-spacing:normal;text-transform:none;pointer-events:all}  .model-explorer-node-data-provider-dropdown .description{font-size:12px;color:#777;line-height:18px;letter-spacing:normal}  .model-explorer-node-data-provider-dropdown .upload-container{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;-webkit-box-align:start;-webkit-align-items:flex-start;-moz-box-align:start;-ms-flex-align:start;align-items:flex-start;padding:0 16px;margin-top:12px}  .model-explorer-node-data-provider-dropdown .load-json-from-cns-container{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;-webkit-box-align:start;-webkit-align-items:flex-start;-moz-box-align:start;-ms-flex-align:start;align-items:flex-start;padding:0 16px;margin-top:10px;width:200px;-moz-box-sizing:border-box;box-sizing:border-box}  .model-explorer-node-data-provider-dropdown .load-json-from-cns-container textarea{width:100%;height:48px;-moz-box-sizing:border-box;box-sizing:border-box;margin:2px 0;resize:none;border-radius:3px;font-family:Google Sans Text,Google Sans,Arial,Helvetica,sans-serif;font-size:11px;padding:2px}  .model-explorer-node-data-provider-dropdown .upload-json-file-button{margin:4px 0;width:90px;height:30px;font-size:12px!important;letter-spacing:normal!important}  .model-explorer-node-data-provider-dropdown .upload-json-file-button.upload{margin-top:2px}  .model-explorer-node-data-provider-dropdown .upload-json-file-button   .mat-mdc-button-touch-target{display:none}  .model-explorer-node-data-provider-dropdown .or-divider{height:1px;border-top:1px solid #eee;position:relative;margin-top:12px}  .model-explorer-node-data-provider-dropdown .or-divider .or-label{font-size:10px;top:-9px;color:#aaa;position:absolute;padding:2px;background-color:#fff;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;width:16px;left:calc(50% - 8px)}  .model-explorer-node-data-provider-dropdown .upload-per-node-data-input{display:none}  .model-explorer-node-data-provider-dropdown .loading-extension,   .model-explorer-node-data-provider-dropdown .no-extension{padding:12px 0 4px 16px;color:#999;font-size:13px;letter-spacing:normal}"],
Ra:0});const NS=(a,b)=>b.type;
function OS(a,b){if(a&1){const c=Ug();R(0,"div",18);V("click",function(){var d=u(c).da,e=Y(2).Cb,f=Y();d=d.type;var g=f.Rc[e];if(1!==g.matchTypes.length||g.matchTypes[0]!==d)e=f.Rc[e],g=e.matchTypes.indexOf(d),0<=g?e.matchTypes.splice(g,1):e.matchTypes.push(d),f.ci.emit([...f.Rc]);return x()});R(1,"mat-icon");Z(2);S()()}a&2&&(a=b.da,b=Y(2).da,Y(),nq("selected",b.matchTypes.includes(a.type))("disabled",1===b.matchTypes.length&&b.matchTypes[0]===a.type),H("matTooltip",a.Od),D(2),Ar(a.type))}
function PS(a){if(a&1){const b=Ug();R(0,"div",12)(1,"div",13);Z(2,"R");S();R(3,"div",14);Z(4,"E");S()();R(5,"input",15,1);V("change",function(){u(b);const c=zr(6),d=Y().Cb,e=Y();e.Rc[d].queryRegex=c.value.trim();e.ci.emit([...e.Rc]);return x()});S();R(7,"div",16);Jq(8,OS,3,6,"div",17,NS);S()}if(a&2){a=Y().da;const b=Y();D(5);H("value",a.queryRegex);D(3);Lq(b.Du)}}
function QS(a){if(a&1){const b=Ug();R(0,"div",19);Z(1," [\u00b7\u00b7] ");S();R(2,"div",20)(3,"input",21,2);V("change",function(){u(b);const c=zr(4),d=zr(8),e=zr(12),f=Y().Cb,g=Y();return x(RS(g,f,c.value.trim(),d.value.trim(),e.value.trim()))});S();R(5,"div",22);Z(6,":");S();R(7,"input",23,3);V("change",function(){u(b);const c=zr(4),d=zr(8),e=zr(12),f=Y().Cb,g=Y();return x(RS(g,f,c.value.trim(),d.value.trim(),e.value.trim()))});S();R(9,"div",24);Z(10,",");S();R(11,"input",25,4);V("change",function(){u(b);
const c=zr(4),d=zr(8),e=zr(12),f=Y().Cb,g=Y();return x(RS(g,f,c.value.trim(),d.value.trim(),e.value.trim()))});S()()}a&2&&(a=Y().da,Y(),D(3),H("value",a.attrName),D(4),H("value",SS(a.min)),D(4),H("value",SS(a.max)))}function TS(a,b){a&1&&(R(0,"option",28),Z(1),S());a&2&&(a=b.da,b=Y(2).da,H("value",a.type)("selected",a.type===b.nodeType),D(),Br(" ",a.label," "))}
function US(a){if(a&1){const b=Ug();R(0,"div",26)(1,"mat-icon");Z(2,"device_hub");S()();R(3,"select",27,5);V("change",function(){u(b);const c=zr(4),d=Y().Cb,e=Y();return x(e.aw(d,c.value))});Jq(5,TS,2,3,"option",28,NS);S()}a&2&&(a=Y(2),D(5),Lq(a.Eu))}function VS(a){if(a&1){const b=Ug();R(0,"div",29);V("click",function(){u(b);const c=Y().Cb,d=Y();d.Rc.splice(c,1);d.ci.emit([...d.Rc]);return x()});R(1,"mat-icon");Z(2,"delete");S()()}}
function WS(a,b){a&1&&(R(0,"div",7),Ap(1,PS,10,1)(2,QS,13,3)(3,US,7,0)(4,VS,3,0,"div",11),S());if(a&2){let c;a=b.da;b=Y();D();Eq((c=a.type)===b.gk.dt?1:c===b.gk.Us?2:c===b.gk.lz?3:-1);D(3);Eq("node_type"!==a.type?4:-1)}}function XS(a){a&1&&(R(0,"div",31)(1,"div",13),Z(2,"R"),S(),R(3,"div",14),Z(4,"E"),S()())}function YS(a){a&1&&(R(0,"div",32),Z(1," [\u00b7\u00b7] "),S())}
function ZS(a,b){if(a&1){const c=Ug();R(0,"div",30);V("click",function(){var d=u(c).da,e=Y();d=d.type;switch(d){case "regex":e.Rc.push({type:d,queryRegex:"",matchTypes:["title"]});break;case "attr_value_range":e.Rc.push({type:d,attrName:"",min:Number.NEGATIVE_INFINITY,max:Number.POSITIVE_INFINITY});break;case "node_type":e.Rc.push({type:d,nodeType:"op_nodes"})}e.ci.emit([...e.Rc]);return x()});Ap(1,XS,5,0,"div",31)(2,YS,2,0,"div",32);R(3,"div",33);Z(4);S()()}if(a&2){let c;a=b.da;b=Y();D();Eq((c=a.type)===
b.gk.dt?1:c===b.gk.Us?2:-1);D(3);Ar(a.label)}}function RS(a,b,c,d,e){let f=Number.NEGATIVE_INFINITY,g=Number.POSITIVE_INFINITY;""===d||isNaN(Number(d))||(f=Number(d));""===e||isNaN(Number(e))||(g=Number(e));b=a.Rc[b];b.attrName=c;b.min=f;b.max=g;a.ci.emit([...a.Rc])}function SS(a){return null==a||a===Number.NEGATIVE_INFINITY||a===Number.POSITIVE_INFINITY?"":`${a}`}
var $S=class{constructor(){this.queries=[];this.ci=new Fk;this.gk=mI;this.Du=[{type:"title",Od:"Match label"},{type:"list",Od:"Match attributes"},{type:"input",Od:"Match inputs (op node only)"},{type:"output",Od:"Match outputs (op node only)"}];this.Eu=[{type:"op_nodes",label:"Op nodes only"},{type:"layer_nodes",label:"Layer nodes only"},{type:"op_and_layer_nodes",label:"Op and layer nodes"}];this.Bu=[{type:"regex",label:"Regex"},{type:"attr_value_range",label:"Attribute value range"}];this.Rc=[]}Fb(){this.Rc=
JSON.parse(JSON.stringify(this.queries))}aw(a,b){this.Rc[a].nodeType=b;this.ci.emit([...this.Rc])}};$S.K=function(a){return new (a||$S)(z(Sr))};
$S.Da=Gf({type:$S,ha:[["complex-queries"]],inputs:{queries:"queries"},outputs:{ci:"queriesUpdated"},ka:!0,features:[Mr],za:11,Aa:1,Ca:[["menu","matMenu"],["queryInput",""],["inputAttrName",""],["inputMin",""],["inputMax",""],["select",""],[1,"container"],[1,"query-container"],[1,"btn-add-query",3,"matMenuTriggerFor"],[1,"model-explorer-add-query-menu"],[1,"menu-item"],["matTooltip","Delete query","matTooltipPosition","right",1,"icon-container","delete"],["matTooltip","Regex","n","","matTooltipPosition",
"left",1,"model-explorer-regex-icon"],[1,"r"],[1,"e"],["placeholder","Regex",1,"regex","common-input",3,"change","value"],[1,"targets-container"],[1,"icon-container","match-type",3,"matTooltip","selected","disabled"],[1,"icon-container","match-type",3,"click","matTooltip"],["matTooltip","Attribute value range","matTooltipPosition","left",1,"model-explorer-attr-value-range-icon"],[1,"attrs-value-range-container"],["placeholder","Attr name",1,"common-input","attr-name",3,"change","value"],[1,"colon"],
["placeholder","min",1,"common-input","min",3,"change","value"],[1,"to"],["placeholder","max",1,"common-input","max",3,"change","value"],["matTooltip","Node type","matTooltipPosition","left",1,"model-explorer-node-type-icon"],[1,"node-type-selector",3,"change"],[3,"value","selected"],["matTooltip","Delete query","matTooltipPosition","right",1,"icon-container","delete",3,"click"],[1,"menu-item",3,"click"],[1,"model-explorer-regex-icon"],[1,"model-explorer-attr-value-range-icon"],[1,"label"]],ta:function(a,
b){a&1&&(R(0,"div",6),Jq(1,WS,5,2,"div",7,Hq),R(3,"div",8)(4,"mat-icon"),Z(5,"add"),S(),Z(6," Add query "),S(),R(7,"mat-menu",9,0),Jq(9,ZS,5,2,"div",10,NS),S()());a&2&&(a=zr(8),D(),Lq(b.Rc),D(2),H("matMenuTriggerFor",a),D(6),Lq(b.Bu))},Ha:[ht,$C,CC,QC,YC,hG,AE,fG,iI,hI,HG],styles:[".query-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;gap:6px;margin-bottom:4px;position:relative;padding-right:36px}.query-container[_ngcontent-%COMP%]   input[_ngcontent-%COMP%]{height:22px;border:1px solid #ccc;border-radius:4px;-moz-box-sizing:border-box;box-sizing:border-box;padding:0 4px;outline:none}.query-container[_ngcontent-%COMP%]   input.regex[_ngcontent-%COMP%]{-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1}.query-container[_ngcontent-%COMP%]   .attrs-value-range-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;width:240px}.query-container[_ngcontent-%COMP%]   .attrs-value-range-container[_ngcontent-%COMP%]   .colon[_ngcontent-%COMP%], .query-container[_ngcontent-%COMP%]   .attrs-value-range-container[_ngcontent-%COMP%]   .to[_ngcontent-%COMP%]{font-weight:700;margin:0 2px;height:22px}.query-container[_ngcontent-%COMP%]   .attrs-value-range-container[_ngcontent-%COMP%]   input[_ngcontent-%COMP%]{min-width:0}.query-container[_ngcontent-%COMP%]   .attrs-value-range-container[_ngcontent-%COMP%]   input.attr-name[_ngcontent-%COMP%]{-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1}.query-container[_ngcontent-%COMP%]   .attrs-value-range-container[_ngcontent-%COMP%]   input.max[_ngcontent-%COMP%], .query-container[_ngcontent-%COMP%]   .attrs-value-range-container[_ngcontent-%COMP%]   input.min[_ngcontent-%COMP%]{width:50px}.query-container[_ngcontent-%COMP%]   .node-type-selector[_ngcontent-%COMP%]{border:1px solid #ccc;border-radius:4px;outline:none;height:23px;cursor:pointer;-moz-box-sizing:border-box;box-sizing:border-box;width:160px}.query-container[_ngcontent-%COMP%]   .icon-container.delete[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;position:absolute;right:0;top:4px;cursor:pointer;opacity:.6}.query-container[_ngcontent-%COMP%]   .icon-container.delete[_ngcontent-%COMP%]:hover{opacity:1}.query-container[_ngcontent-%COMP%]   .icon-container.delete[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{font-size:16px;width:16px;height:16px}.targets-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;gap:2px}.targets-container[_ngcontent-%COMP%]   .icon-container.match-type[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;background-color:#ddd;border-radius:4px;cursor:pointer;padding:4px}.targets-container[_ngcontent-%COMP%]   .icon-container.match-type.selected[_ngcontent-%COMP%]{background-color:#a3c9ff}.targets-container[_ngcontent-%COMP%]   .icon-container.match-type.disabled[_ngcontent-%COMP%]{cursor:default}.targets-container[_ngcontent-%COMP%]   .icon-container.match-type[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{font-size:14px;width:14px;height:14px;color:#333}.btn-add-query[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;cursor:pointer;width:-webkit-fit-content;width:-moz-fit-content;width:fit-content;line-height:16px;opacity:.6;white-space:nowrap;margin-top:2px;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;margin-left:19px}.btn-add-query[_ngcontent-%COMP%]:hover{opacity:1}.btn-add-query[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{font-size:16px;width:16px;height:16px;margin-right:2px}  .model-explorer-add-query-menu .menu-item{letter-spacing:normal!important;font-family:Google Sans Text,Arial,Helvetica,sans-serif!important;font-size:12px;padding:0 8px;cursor:pointer;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}  .model-explorer-add-query-menu .menu-item .label{margin-left:6px}  .model-explorer-add-query-menu .menu-item:hover{background-color:#eee}  .model-explorer-regex-icon{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;width:16px;height:16px;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;font-weight:700;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;color:#aaa}  .model-explorer-regex-icon .r{font-size:12px}  .model-explorer-regex-icon .e{font-size:8px;margin-top:2px}  .model-explorer-attr-value-range-icon{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;width:16px;height:16px;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;font-size:10px;font-weight:700;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;color:#aaa}  .model-explorer-node-type-icon{width:16px;height:16px}  .model-explorer-node-type-icon mat-icon{font-size:16px;width:16px;height:16px;color:#aaa}"],
Ra:0});function aT(a){a&1&&(R(0,"div",3),Mq(1,"io-tree",4),S());a&2&&(a=Y(),D(),H("data",a.nv)("rendererId",a.rendererId)("tooltipPosition","right"))}var bT=class{constructor(){this.nodes=[];this.rendererId="";this.xw="node";this.Pr={minWidth:320,maxWidth:640,minHeight:0};this.nv=[]}tc(a){a.nodes&&(this.nv=$K(this.nodes,"incoming"))}get label(){const a=this.nodes.length;return`${a} ${this.xw}${1===a?"":"s"}`}};bT.K=function(a){return new (a||bT)};
bT.Da=Gf({type:bT,ha:[["node-list-viewer"]],inputs:{nodes:"nodes",rendererId:"rendererId",xw:"labelSuffix"},ka:!0,features:[Dg,Mr],za:5,Aa:5,Ca:[["nodeList",""],[1,"container"],["panelClass","model-explorer-node-list-panel","dialogLabel","nodeList",1,"label-container",3,"xapInlineDialogClick","overlaySize"],[1,"model-explorer-node-list"],[3,"data","rendererId","tooltipPosition"]],ta:function(a,b){a&1&&(R(0,"div",1)(1,"div",2),Z(2),S(),Ap(3,aT,2,3,"ng-template",null,0,Qr),S());a&2&&(a=zr(4),D(),nq("disabled",
0===b.nodes.length),H("xapInlineDialogClick",a)("overlaySize",b.Pr),D(),Br(" ",b.label," "))},Ha:[ht,ZK,iI,HG,GG],styles:[".container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;position:relative}.label-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;padding:4px 6px;border-radius:4px;background-color:#a3c9ff;line-height:14px;cursor:pointer;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;opacity:.8}.label-container[_ngcontent-%COMP%]:hover{opacity:1}.label-container.disabled[_ngcontent-%COMP%]{pointer-events:none;background-color:#ddd}  .model-explorer-node-list{font-size:12px;background-color:#fff;line-height:normal}  .model-explorer-node-list-panel xap-inline-dialog-container{width:100%}"],
Ra:0});var cT=new df("CDK_DRAG_PARENT");var dT=new df("CdkDragHandle"),eT=class{get disabled(){return this.Ib}set disabled(a){this.Ib=a;this.C.next(this)}constructor(a,b){this.element=a;this.D=b;this.C=new cj;this.Ib=!1;null==b||b.T(this)}va(){let a;null==(a=this.D)||a.ca(this);this.C.complete()}};eT.K=function(a){return new (a||eT)(z(si),z(cT,12))};eT.Fa=Qf({type:eT,ha:[["","cdkDragHandle",""]],Ka:[1,"cdk-drag-handle"],inputs:{disabled:[2,"cdkDragHandleDisabled","disabled",zs]},ka:!0,features:[Kr([{oa:dT,Ab:eT}]),Jp]});var fT=new df("CDK_DRAG_CONFIG");const gT=Kt({passive:!1,capture:!0});function hT(a,b){a.D.add(b);1===a.D.size&&Mk(a.qa,()=>{a.ia.addEventListener("touchmove",a.P,gT)})}function iT(a,b){b=a.C.indexOf(b);-1<b&&(a.C.splice(b,1),0===a.C.length&&a.N())}function jT(a,b){a.D.delete(b);iT(a,b);0===a.D.size&&a.ia.removeEventListener("touchmove",a.P,gT)}
function kT(a,b,c){-1<a.C.indexOf(b)||(a.C.push(b),1===a.C.length&&(b=c.type.startsWith("touch"),a.F.set(b?"touchend":"mouseup",{handler:d=>a.J.next(d),options:!0}).set("scroll",{handler:d=>a.scroll.next(d),options:!0}).set("selectstart",{handler:a.aa,options:gT}),b||a.F.set("mousemove",{handler:d=>a.G.next(d),options:gT}),Mk(a.qa,()=>{a.F.forEach((d,e)=>{a.ia.addEventListener(e,d.handler,d.options)})})))}
function lT(a,b){const c=[a.scroll];b&&b!==a.ia&&c.push(new Xi(d=>Mk(a.qa,()=>{const e=f=>{a.C.length&&d.next(f)};b.addEventListener("scroll",e,!0);return()=>{b.removeEventListener("scroll",e,!0)}})));return rk(...c)}
var mT=class{constructor(a,b){this.qa=a;this.O=new Set;this.D=new Set;this.C=[];this.F=new Map;this.T=c=>c.de();this.G=new cj;this.J=new cj;this.scroll=new cj;this.aa=c=>{0<this.C.length&&c.preventDefault()};this.P=c=>{0<this.C.length&&(this.C.some(this.T)&&c.preventDefault(),this.G.next(c))};this.ia=b}de(a){return-1<this.C.indexOf(a)}va(){this.D.forEach(a=>jT(this,a));this.O.forEach(a=>{this.O.delete(a)});this.N();this.G.complete();this.J.complete()}N(){this.F.forEach((a,b)=>{this.ia.removeEventListener(b,
a.handler,a.options)});this.F.clear()}};mT.K=function(a){return new (a||mT)(r(Qk),r(jt))};mT.pa=Md({ma:mT,ba:mT.K,ea:"root"});function nT(a){const b=a.cloneNode(!0),c=b.querySelectorAll("[id]"),d=a.nodeName.toLowerCase();b.removeAttribute("id");for(let e=0;e<c.length;e++)c[e].removeAttribute("id");"canvas"===d?oT(a,b):("input"===d||"select"===d||"textarea"===d)&&pT(a,b);qT("canvas",a,b,oT);qT("input, textarea, select",a,b,pT);return b}function qT(a,b,c,d){b=b.querySelectorAll(a);if(b.length)for(a=c.querySelectorAll(a),c=0;c<b.length;c++)d(b[c],a[c])}let rT=0;
function pT(a,b){"file"!==b.type&&(b.value=a.value);"radio"===b.type&&b.name&&(b.name=`mat-clone-${b.name}-${rT++}`)}function oT(a,b){if(b=b.getContext("2d"))try{b.drawImage(a,0,0)}catch(c){}};function sT(a){a=a.getBoundingClientRect();return{top:a.top,right:a.right,bottom:a.bottom,left:a.left,width:a.width,height:a.height,x:a.x,y:a.y}}function tT(a,b,c){a.top+=b;a.bottom=a.top+a.height;a.left+=c;a.right=a.left+a.width};function uT(a,b){const c=St(b);b=a.ie.get(c);if(!b)return null;b=b.ls;let d;if(c===a.ia){var e=window.scrollX;d=window.scrollY}else d=c.scrollTop,e=c.scrollLeft;const f=b.top-d,g=b.left-e;a.ie.forEach((k,m)=>{k.Wu&&c!==m&&c.contains(m)&&tT(k.Wu,f,g)});b.top=d;b.left=e;return{top:f,left:g}}
var vT=class{constructor(a){this.ia=a;this.ie=new Map}clear(){this.ie.clear()}cache(a){this.clear();this.ie.set(this.ia,{ls:{top:window.scrollY,left:window.scrollX}});a.forEach(b=>{this.ie.set(b,{ls:{top:b.scrollTop,left:b.scrollLeft},Wu:sT(b)})})}};function wT(a,b){a=a.rootNodes;if(1===a.length&&a[0].nodeType===b.ELEMENT_NODE)return a[0];const c=b.createElement("div");a.forEach(d=>c.appendChild(d));return c};function xT(a,b,c){for(let d in b)if(b.hasOwnProperty(d)){const e=b[d];if(e){let f;a.setProperty(d,e,(null==(f=c)?0:f.has(d))?"important":"")}else a.removeProperty(d)}}function yT(a,b){const c=b?"":"none";xT(a.style,{"touch-action":b?"":"none","-webkit-user-drag":b?"":"none","-webkit-tap-highlight-color":b?"":"transparent","user-select":c,"-ms-user-select":c,"-webkit-user-select":c,"-moz-user-select":c})}
function zT(a,b){xT(a.style,{position:b?"":"fixed",top:b?"":"0",opacity:b?"":"0",left:b?"":"-999em"},AT)}function BT(a,b){return b&&"none"!=b?a+" "+b:a}function CT(a,b){a.style.width=`${b.width}px`;a.style.height=`${b.height}px`;a.style.transform=DT(b.left,b.top)}function DT(a,b){return`translate3d(${Math.round(a)}px, ${Math.round(b)}px, 0)`};function ET(a){return parseFloat(a)*(-1<a.toLowerCase().indexOf("ms")?1:1E3)}function FT(a){a=getComputedStyle(a);var b=GT(a,"transition-property"),c=b.find(d=>"transform"===d||"all"===d);if(!c)return 0;b=b.indexOf(c);c=GT(a,"transition-duration");a=GT(a,"transition-delay");return ET(c[b])+ET(a[b])}function GT(a,b){return a.getPropertyValue(b).split(",").map(c=>c.trim())};const HT=new Set(["position"]);
var IT=class{constructor(a,b,c,d,e,f,g,k,m){this.ia=a;this.P=b;this.N=c;this.D=d;this.O=e;this.aa=f;this.G=g;this.F=k;this.ca=m}attach(a){this.C=this.T();a.appendChild(this.C);"showPopover"in this.C&&this.C.showPopover()}destroy(){this.C.remove();let a;null==(a=this.J)||a.destroy();this.C=this.J=null}getBoundingClientRect(){return this.C.getBoundingClientRect()}Ri(a){this.C.classList.add(a)}addEventListener(a,b){this.C.addEventListener(a,b)}removeEventListener(a,b){this.C.removeEventListener(a,b)}T(){const a=
this.O,b=this.aa;var c=a?a.ta:null;let d;if(c&&a){const e=a.Yn?this.D:null;c=a.eg.xd(c,a.context);cn(c);d=wT(c,this.ia);this.J=c;a.Yn?CT(d,e):d.style.transform=DT(this.G.x,this.G.y)}else d=nT(this.P),CT(d,this.D),this.F&&(d.style.transform=this.F);xT(d.style,{"pointer-events":"none",margin:"0",position:"fixed",top:"0",left:"0","z-index":this.ca+""},HT);yT(d,!1);d.classList.add("cdk-drag-preview");d.setAttribute("popover","manual");d.setAttribute("dir",this.N);b&&(Array.isArray(b)?b.forEach(e=>d.classList.add(e)):
d.classList.add(b));return d}};const JT=Kt({passive:!0}),KT=Kt({passive:!1}),LT=Kt({passive:!1,capture:!0}),AT=new Set(["position"]);function MT(a,b){const c=Ss(b);c!==a.C&&(a.C&&a.jh(a.C),Mk(a.qa,()=>{c.addEventListener("mousedown",a.Kb,KT);c.addEventListener("touchstart",a.Kb,JT);c.addEventListener("dragstart",a.fh,KT)}),a.ca=void 0,a.C=c);"undefined"!==typeof SVGElement&&a.C instanceof SVGElement&&(a.lb=a.C.ownerSVGElement);return a}function NT(a,b){a.Hc=b}
function OT(a,b){a.F=b.map(d=>Ss(d));a.F.forEach(d=>yT(d,a.disabled));a.Lb();const c=new Set;a.T.forEach(d=>{-1<a.F.indexOf(d)&&c.add(d)});a.T=c}function PT(a,b){a.fa=b?Ss(b):null;a.Ob.unsubscribe();b&&(a.Ob=a.wi.wb(10).subscribe(()=>a.zi()));return a}function QT(a,b){a.N={x:0,y:0};a.P.x=b.x;a.P.y=b.y;a.D||a.Dc(b.x,b.y)}function RT(a){void 0===a.Xc&&(a.Xc=Qt(a.C));return a.Xc}
var UT=class{get disabled(){return this.Ib||!(!this.D||!this.D.disabled)}set disabled(a){a!==this.Ib&&(this.Ib=a,this.Lb(),this.F.forEach(b=>yT(b,a)))}constructor(a,b,c,d,e,f){this.ya=b;this.ia=c;this.qa=d;this.wi=e;this.O=f;this.P={x:0,y:0};this.N={x:0,y:0};this.Ja=!1;this.gb=new cj;this.Ob=this.sd=this.ih=this.hh=Ni.EMPTY;this.fa=null;this.gh=!0;this.F=[];this.T=new Set;this.Yc="ltr";this.hj=0;this.Ib=!1;this.Sa=new cj;this.ji=new cj;this.released=new cj;this.ended=new cj;this.kj=new cj;this.mj=
new cj;this.jj=new cj;this.xj=this.gb;this.Kb=g=>{this.Sa.next();if(this.F.length){const k=this.Sd(g);!k||this.T.has(k)||this.disabled||this.Oe(k,g)}else this.disabled||this.Oe(this.C,g)};this.dl=g=>{var k=this.Va(g);if(this.Ja){g.cancelable&&g.preventDefault();var m=this.Rd(k);this.Wd=!0;this.fg=k;this.Dl(m);if(this.D)this.kh(m,k);else{k=this.te?this.ja:this.G;var p=this.N;p.x=m.x-k.x+this.P.x;p.y=m.y-k.y+this.P.y;this.Dc(p.x,p.y)}this.gb.mc.length&&this.qa.run(()=>{this.gb.next({source:this,bx:m,
event:g,distance:this.Gc(m),delta:this.Ic})})}else Math.abs(k.x-this.G.x)+Math.abs(k.y-this.G.y)>=this.ya.pn&&((k=Date.now()>=this.jk+this.Gk(g),p=this.D,k)?p&&(p.de()||p.T())||(g.cancelable&&g.preventDefault(),this.Ja=!0,this.qa.run(()=>this.xl(g))):this.Qd(g))};this.ml=g=>{this.Qd(g)};this.fh=g=>{if(this.F.length){const k=this.Sd(g);!k||this.T.has(k)||this.disabled||g.preventDefault()}else this.disabled||g.preventDefault()};NT(MT(this,a),b.hG||null);this.wa=new vT(c);hT(f,this)}dispose(){this.jh(this.C);
if(this.de()){let b;null==(b=this.C)||b.remove()}let a;null==(a=this.Ea)||a.remove();this.Pd();this.ud();jT(this.O,this);this.Jc();this.Sa.complete();this.ji.complete();this.released.complete();this.ended.complete();this.kj.complete();this.mj.complete();this.jj.complete();this.gb.complete();this.F=[];this.T.clear();this.D=void 0;this.Ob.unsubscribe();this.wa.clear();this.fa=this.C=this.lb=this.md=this.Pa=this.Ea=this.Hc=null}de(){return this.Ja&&this.O.de(this)}reset(){this.C.style.transform=this.ca||
"";this.N={x:0,y:0};this.P={x:0,y:0}}Gl(a){this.D=a}Ar(){const a=this.fg;a&&this.D&&this.kh(this.Rd(a),a)}Jc(){this.hh.unsubscribe();this.ih.unsubscribe();this.sd.unsubscribe();let a;null==(a=RT(this))||a.removeEventListener("selectstart",ST,LT)}Pd(){let a;null==(a=this.J)||a.destroy();this.J=null}ud(){let a;null==(a=this.Nb)||a.remove();let b;null==(b=this.sb)||b.destroy();this.Nb=this.sb=null}Qd(a){if(this.O.de(this)&&(this.Jc(),iT(this.O,this),this.Lb(),this.F&&(this.C.style.webkitTapHighlightColor=
this.wl),this.Ja))if(this.released.next({source:this,event:a}),this.D)this.D.N(),this.xi().then(()=>{this.yi(a);this.td();iT(this.O,this)});else{this.P.x=this.N.x;const b=this.Va(a);this.P.y=this.N.y;this.qa.run(()=>{this.ended.next({source:this,distance:this.Gc(b),ij:b,event:a})});this.td();iT(this.O,this)}}xl(a){TT(a)&&(this.eh=Date.now());this.Lb();const b=RT(this),c=this.D;b&&Mk(this.qa,()=>{b.addEventListener("selectstart",ST,LT)});if(c){const d=this.C,e=d.parentNode,f=this.Nb=this.ik(),g=this.Ea=
this.Ea||this.ia.createComment("");e.insertBefore(g,d);this.ca=d.style.transform||"";this.J=new IT(this.ia,this.C,this.Yc,this.ja,this.Pa||null,this.Pl||null,this.G,this.ca,this.ya.zIndex||1E3);this.J.attach(this.Nk(e,b));zT(d,!1);this.ia.body.appendChild(e.replaceChild(f,d));this.ji.next({source:this,event:a});c.start();this.aa=c;this.bb=c.C(this)}else this.ji.next({source:this,event:a}),this.aa=this.bb=void 0;this.wa.cache(c?c.P():[])}Oe(a,b){this.Hc&&b.stopPropagation();var c=this.de(),d=TT(b);
const e=!d&&0!==b.button,f=this.C,g=St(b),k=!d&&this.eh&&this.eh+800>Date.now();d=d?Ex(b):Dx(b);g&&g.draggable&&"mousedown"===b.type&&b.preventDefault();c||e||k||d||(this.F.length&&(c=f.style,this.wl=c.webkitTapHighlightColor||"",c.webkitTapHighlightColor="transparent"),this.Ja=this.Wd=!1,this.Jc(),this.ja=this.C.getBoundingClientRect(),this.hh=this.O.G.subscribe(this.dl),this.ih=this.O.J.subscribe(this.ml),this.sd=lT(this.O,RT(this)).subscribe(m=>this.zl(m)),this.fa&&(this.na=sT(this.fa)),this.sa=
(c=this.Pa)&&c.ta&&!c.Yn?{x:0,y:0}:this.Ik(this.ja,a,b),a=this.G=this.fg=this.Va(b),this.Ic={x:0,y:0},this.gl={x:a.x,y:a.y},this.jk=Date.now(),kT(this.O,this,b))}yi(a){zT(this.C,!0);this.Ea.parentNode.replaceChild(this.C,this.Ea);this.Pd();this.ud();this.ja=this.na=this.ua=this.ca=void 0;this.qa.run(()=>{const b=this.D,c=b.C(this),d=this.Va(a),e=this.Gc(d),f=b.D(d.x,d.y);this.ended.next({source:this,distance:e,ij:d,event:a});this.jj.next({item:this,fd:c,Ie:this.bb,ab:b,ix:this.aa,vw:f,distance:e,
ij:d,event:a});b.O(this,c,this.bb,this.aa,f,e,d,a);this.D=this.aa})}kh({x:a,y:b},{x:c,y:d}){let e=this.aa.F(this,a,b);!e&&this.D!==this.aa&&this.aa.D(a,b)&&(e=this.aa);e&&e!==this.D&&this.qa.run(()=>{this.mj.next({item:this,ab:this.D});this.D.Dq(this);this.D=e;this.D.sn(this,a,b,e===this.aa&&e.aa?this.bb:void 0);this.kj.next({item:this,ab:e,fd:e.C(this)})});this.de()&&(this.D.J(c,d),this.D.G(this,a,b,this.Ic),this.te?this.ac(a,b):this.ac(a-this.sa.x,b-this.sa.y))}xi(){if(!this.Wd)return Promise.resolve();
const a=this.Nb.getBoundingClientRect();this.J.Ri("cdk-drag-animating");this.ac(a.left,a.top);const b=FT(this.J.C);return 0===b?Promise.resolve():Mk(this.qa,()=>new Promise(c=>{const d=f=>{if(!f||St(f)===this.J&&"transform"===f.propertyName){let g;null==(g=this.J)||g.removeEventListener("transitionend",d);c();clearTimeout(e)}},e=setTimeout(d,1.5*b);this.J.addEventListener("transitionend",d)}))}ik(){var a=this.md;const b=a?a.ta:null;b?(this.sb=a.eg.xd(b,a.context),cn(this.sb),a=wT(this.sb,this.ia)):
a=nT(this.C);a.style.pointerEvents="none";a.classList.add("cdk-drag-placeholder");return a}Ik(a,b,c){b=(b=b===this.C?null:b)?b.getBoundingClientRect():a;c=TT(c)?c.targetTouches[0]:c;const d=this.Vd();return{x:b.left-a.left+(c.pageX-b.left-d.left),y:b.top-a.top+(c.pageY-b.top-d.top)}}Va(a){var b=this.Vd(),c=TT(a)?a.touches[0]||a.changedTouches[0]||{pageX:0,pageY:0}:a;a=c.pageX-b.left;b=c.pageY-b.top;if(this.lb&&(c=this.lb.getScreenCTM())){const d=this.lb.createSVGPoint();d.x=a;d.y=b;return d.matrixTransform(c.inverse())}return{x:a,
y:b}}Rd(a){var b=this.D?this.D.Mh:null;let {x:c,y:d}=this.te?this.te(a,this,this.ja,this.sa):a;if("x"===this.Mh||"x"===b)d=this.G.y-(this.te?this.sa.y:0);else if("y"===this.Mh||"y"===b)c=this.G.x-(this.te?this.sa.x:0);if(this.na){const {x:e,y:f}=this.te?{x:0,y:0}:this.sa;a=this.na;const {width:g,height:k}=this.al();b=a.top+f;const m=a.bottom-(k-f);c=Math.max(a.left+e,Math.min(a.right-(g-e),c));d=Math.max(b,Math.min(m,d))}return{x:c,y:d}}Dl(a){const b=a.x;a=a.y;const c=this.Ic,d=this.gl,e=Math.abs(a-
d.y);Math.abs(b-d.x)>this.ya.Nl&&(c.x=b>d.x?1:-1,d.x=b);e>this.ya.Nl&&(c.y=a>d.y?1:-1,d.y=a);return c}Lb(){if(this.C&&this.F){var a=0<this.F.length||!this.de();a!==this.gh&&(this.gh=a,yT(this.C,a))}}jh(a){a.removeEventListener("mousedown",this.Kb,KT);a.removeEventListener("touchstart",this.Kb,JT);a.removeEventListener("dragstart",this.fh,KT)}Dc(a,b){a=DT(a,b);b=this.C.style;null==this.ca&&(this.ca=b.transform&&"none"!=b.transform?b.transform:"");b.transform=BT(a,this.ca)}ac(a,b){let c;const d=(null==
(c=this.Pa)?0:c.ta)?void 0:this.ca;this.J.C.style.transform=BT(DT(a,b),d)}Gc(a){const b=this.G;return b?{x:a.x-b.x,y:a.y-b.y}:{x:0,y:0}}td(){this.na=this.ua=void 0;this.wa.clear()}zi(){let {x:a,y:b}=this.P;if(!(0===a&&0===b||this.de())&&this.fa){var c=this.C.getBoundingClientRect(),d=this.fa.getBoundingClientRect();if(!(0===d.width&&0===d.height||0===c.width&&0===c.height)){var e=d.left-c.left,f=c.right-d.right,g=d.top-c.top,k=c.bottom-d.bottom;d.width>c.width?(0<e&&(a+=e),0<f&&(a-=f)):a=0;d.height>
c.height?(0<g&&(b+=g),0<k&&(b-=k)):b=0;a===this.P.x&&b===this.P.y||QT(this,{y:b,x:a})}}}Gk(a){const b=this.hj;return"number"===typeof b?b:TT(a)?b.zG:b?b.Id:0}zl(a){const b=uT(this.wa,a);b&&(a=St(a),this.na&&a!==this.fa&&a.contains(this.fa)&&tT(this.na,b.top,b.left),this.G.x+=b.left,this.G.y+=b.top,this.D||(this.N.x-=b.left,this.N.y-=b.top,this.Dc(this.N.x,this.N.y)))}Vd(){let a;return(null==(a=this.wa.ie.get(this.ia))?void 0:a.ls)||{top:window.scrollY,left:window.scrollX}}Nk(a,b){const c=this.sl||
"global";return"parent"===c?a:"global"===c?(a=this.ia,b||a.fullscreenElement||a.webkitFullscreenElement||a.mozFullScreenElement||a.msFullscreenElement||a.body):Ss(c)}al(){this.ua&&(this.ua.width||this.ua.height)||(this.ua=this.J?this.J.getBoundingClientRect():this.ja);return this.ua}Sd(a){return this.F.find(b=>a.target&&(a.target===b||b.contains(a.target)))}};function TT(a){return"t"===a.type[0]}function ST(a){a.preventDefault()};const VT={pn:5,Nl:5},WT=new Set;var Ns=class{};Ns.K=function(a){return new (a||Ns)};Ns.Da=Gf({type:Ns,ha:[["ng-component"]],Ka:["cdk-drag-resets-container",""],ka:!0,features:[Mr],za:0,Aa:0,ta:function(){},styles:["@layer cdk-resets{.cdk-drag-preview{background:none;border:none;padding:0;color:inherit}}\n"],pb:2,Ra:0});function XT(a,b,c=VT){a.J();return new UT(b,c,a.ia,a.qa,a.F,a.D)}
var YT=class{constructor(a,b,c,d){this.ia=a;this.qa=b;this.F=c;this.D=d;this.C=lf(fp);this.G=lf(kg)}J(){if(!WT.has(this.C)){WT.add(this.C);const a=Ms({Of:this.G});this.C.ob(()=>{WT.delete(this.C);0===WT.size&&a.destroy()})}}};YT.K=function(a){return new (a||YT)(r(jt),r(Qk),r(wu),r(mT))};YT.pa=Md({ma:YT,ba:YT.K,ea:"root"});var ZT=new df("CdkDropList");function $T(a,b){const c=b.Mh,d=b.hj,e=b.te,f=b.Pl,g=b.bq,k=b.HF,m=b.Mj;b=b.Qr;a.disabled=null==k?!1:k;a.hj=d||0;c&&(a.Mh=c);e&&(a.te=e);f&&(a.Pl=f);g&&(a.bq=g);m&&(a.Mj=m);b&&(a.Qr=b)}
function aU(a,b){b.ji.subscribe(c=>{a.ji.emit({source:a,event:c.event});bn(a.Ba)});b.released.subscribe(c=>{a.released.emit({source:a,event:c.event})});b.ended.subscribe(c=>{a.ended.emit({source:a,distance:c.distance,ij:c.ij,event:c.event});bn(a.Ba)});b.kj.subscribe(c=>{a.kj.emit({ab:c.ab.data,item:a,fd:c.fd})});b.mj.subscribe(c=>{a.mj.emit({ab:c.ab.data,item:a})});b.jj.subscribe(c=>{a.jj.emit({Ie:c.Ie,fd:c.fd,ix:c.ix.data,ab:c.ab.data,vw:c.vw,item:a,distance:c.distance,ij:c.ij,event:c.event})})}
var cU=class{get disabled(){return this.Ib||this.G&&this.G.disabled}set disabled(a){this.Ib=a;this.bd.disabled=this.Ib}constructor(a,b,c,d,e,f,g,k,m,p,t){this.element=a;this.G=b;this.qa=d;this.N=e;this.P=g;this.Ba=m;this.fa=p;this.J=t;this.Ga=new cj;this.C=new ej([]);this.ji=new Fk;this.released=new Fk;this.ended=new Fk;this.kj=new Fk;this.mj=new Fk;this.jj=new Fk;this.xj=new Xi(w=>{const B=C(this.bd.xj,xj(G=>({source:this,bx:G.bx,event:G.event,delta:G.delta,distance:G.distance}))).subscribe(w);return()=>
{B.unsubscribe()}});this.Ua=lf(Wh);this.bd=XT(k,a,{pn:f&&null!=f.pn?f.pn:5,Nl:f&&null!=f.Nl?f.Nl:5,zIndex:null==f?void 0:f.zIndex});this.bd.data=this;bU.push(this);f&&$T(this,f);b&&this.bd.Gl(b.C);this.na(this.bd);aU(this,this.bd)}reset(){this.bd.reset()}pd(){al(()=>{this.O();this.ja();this.An&&QT(this.bd,this.An)},{xa:this.Ua})}tc(a){const b=a.rootElementSelector;a=a.freeDragPosition;b&&!b.wn&&this.O();a&&!a.wn&&this.An&&QT(this.bd,this.An)}va(){this.G&&this.G.removeItem(this);const a=bU.indexOf(this);
-1<a&&bU.splice(a,1);Mk(this.qa,()=>{this.C.complete();this.Ga.next();this.Ga.complete();this.bd.dispose()})}T(a){const b=dj(this.C);b.push(a);this.C.next(b)}ca(a){const b=dj(this.C);a=b.indexOf(a);-1<a&&(b.splice(a,1),this.C.next(b))}Ea(a){this.D=a}ua(a){a===this.D&&(this.D=null)}wa(a){this.F=a}sa(a){a===this.F&&(this.F=null)}O(){const a=this.element.R;let b=a;if(this.Mj){let c;b=void 0!==a.closest?a.closest(this.Mj):null==(c=a.parentElement)?void 0:c.closest(this.Mj)}MT(this.bd,b||a)}aa(){const a=
this.bq;return a?"string"===typeof a?this.element.R.closest(a):Ss(a):null}na(a){a.Sa.subscribe(()=>{if(!a.de()){const c=this.P;var b=this.hj;const d=this.F?{ta:this.F.cg,context:this.F.data,eg:this.N}:null,e=this.D?{ta:this.D.cg,context:this.D.data,Yn:this.D.Yn,eg:this.N}:null;a.disabled=this.disabled;a.Mh=this.Mh;a.hj="object"===typeof b&&b?b:Ts(b);a.te=this.te;a.Pl=this.Pl;b=PT(a,this.aa());b.md=d;b.Pa=e;b.sl=this.Qr||"global";c&&(a.Yc=c.value)}});C(a.Sa,fo()).subscribe(()=>{if(this.J)a.Hc=this.J.bd;
else for(var b=this.element.R.parentElement;b;){if(b.classList.contains("cdk-drag")){let c;NT(a,(null==(c=bU.find(d=>d.element.R===b))?void 0:c.bd)||null);break}b=b.parentElement}})}ja(){C(this.C,Po(a=>{a=a.map(b=>b.element);this.fa&&this.Mj&&a.push(this.element);OT(this.bd,a)}),Io(a=>rk(...a.map(b=>C(b.C,Ho(b))))),Lo(this.Ga)).subscribe(a=>{const b=this.bd,c=a.element.R;a.disabled?!b.T.has(c)&&-1<b.F.indexOf(c)&&(b.T.add(c),yT(c,!0)):b.T.has(c)&&(b.T.delete(c),yT(c,b.disabled))})}},bU=[];
cU.K=function(a){return new (a||cU)(z(si),z(ZT,12),z(jt),z(Qk),z(kp),z(fT,8),z(ou,8),z(YT),z(Sr),z(dT,10),z(cT,12))};
cU.Fa=Qf({type:cU,ha:[["","cdkDrag",""]],Ka:[1,"cdk-drag"],Za:4,Ia:function(a,b){a&2&&nq("cdk-drag-disabled",b.disabled)("cdk-drag-dragging",b.bd.de())},inputs:{data:[0,"cdkDragData","data"],Mh:[0,"cdkDragLockAxis","lockAxis"],Mj:[0,"cdkDragRootElement","rootElementSelector"],bq:[0,"cdkDragBoundary","boundaryElement"],hj:[0,"cdkDragStartDelay","dragStartDelay"],An:[0,"cdkDragFreeDragPosition","freeDragPosition"],disabled:[2,"cdkDragDisabled","disabled",zs],te:[0,"cdkDragConstrainPosition","constrainPosition"],
Pl:[0,"cdkDragPreviewClass","previewClass"],Qr:[0,"cdkDragPreviewContainer","previewContainer"]},outputs:{ji:"cdkDragStarted",released:"cdkDragReleased",ended:"cdkDragEnded",kj:"cdkDragEntered",mj:"cdkDragExited",jj:"cdkDragDropped",xj:"cdkDragMoved"},ib:["cdkDrag"],ka:!0,features:[Kr([{oa:cT,Ab:cU}]),Jp,Dg]});new df("CdkDragPlaceholder");new df("CdkDragPreview");new df("CdkDropListGroup");var dU=class{};dU.K=function(a){return new (a||dU)};dU.Ya=Of({type:dU});dU.Xa=Nd({Wa:[YT],imports:[xu]});const eU=(a,b)=>b.type;function fU(a){a&1&&(R(0,"div",24),Z(1," Rules use your defined queries to match nodes for styling. All queries within a rule are connected with "),R(2,"span",25),Z(3,"AND"),S(),Z(4," logic. "),S())}function gU(a){a&1&&(R(0,"div",28),Mq(1,"node-list-viewer",36),S());if(a&2){a=Y().Cb;const b=Y(2);D();H("nodes",(b.nm[a]||{})[0]||[])("rendererId",b.zw)}}
function qX(a){a&1&&(R(0,"div",28)(1,"div",37),Z(2,"Left pane"),S(),Mq(3,"node-list-viewer",36),S(),R(4,"div",28)(5,"div",37),Z(6,"Right pane"),S(),Mq(7,"node-list-viewer",36),S());if(a&2){a=Y().Cb;const b=Y(2);D(3);H("nodes",(b.nm[a]||{})[0]||[])("rendererId",b.zw);D(4);H("nodes",(b.nm[a]||{})[1]||[])("rendererId",b.KD)}}
function rX(a){if(a&1){const b=Ug();R(0,"div",41)(1,"label")(2,"input",42,3);V("change",function(){u(b);const c=zr(3),d=Y(2).da,e=Y().Cb,f=Y(2);return x(sX(f,e,d,c.value))});S()();R(4,"input",43,4);V("change",function(){u(b);const c=zr(5),d=Y(2).da,e=Y().Cb,f=Y(2);return x(sX(f,e,d,c.value))});S()()}if(a&2){a=Y(2).da;const b=Y().da;Y(2);D();lq("background-color",tX(b,a));D();H("value",tX(b,a));D(2);H("value",tX(b,a))}}
function uX(a){a&1&&Ap(0,rX,6,4,"div",41);if(a&2){a=Y().da;const b=Y().da;Y(2);Eq(null!=b.styles[a.id]?0:-1)}}function vX(a){if(a&1){const b=Ug();R(0,"div",44)(1,"input",45,5);V("change",function(){u(b);const c=zr(2),d=Y(2).da,e=Y().Cb,f=Y(2);return x(sX(f,e,d,c.value))});S()()}if(a&2){a=Y(2).da;const b=Y().da;Y(2);D();H("value",tX(b,a))}}function wX(a){a&1&&Ap(0,vX,3,1,"div",44);if(a&2){a=Y().da;const b=Y().da;Y(2);Eq(null!=b.styles[a.id]?0:-1)}}
function xX(a,b){if(a&1){const c=Ug();R(0,"div",30)(1,"div",38)(2,"input",39,2);V("change",function(){const d=u(c).da,e=zr(3),f=Y().Cb,g=Y(2);nL(g.vd,f,d,e.checked);return x()});S();R(4,"div",40);V("click",function(){const d=u(c).da,e=Y().Cb,f=Y(2);nL(f.vd,e,d);return x()});Z(5);S()();Ap(6,uX,1,1)(7,wX,1,1);S()}if(a&2){let c;a=b.da;b=Y().da;const d=Y(2);D(2);H("checked",null!=b.styles[a.id]);D(3);Br(" ",a.label," ");D();Eq((c=a.type)===d.ft.COLOR?6:c===d.ft.Uo?7:-1)}}
function yX(a){if(a&1){const b=Ug();R(0,"div",46);V("click",function(){u(b);const c=Y().Cb,d=Y(2);jL(d.vd,c);return x()});R(1,"mat-icon");Z(2,"arrow_upward_alt");S()()}}function zX(a){if(a&1){const b=Ug();R(0,"div",47);V("click",function(){u(b);const c=Y().Cb,d=Y(2);kL(d.vd,c);return x()});R(1,"mat-icon");Z(2,"arrow_downward_alt");S()()}}
function AX(a,b){if(a&1){const c=Ug();R(0,"tr")(1,"td",19)(2,"complex-queries",26);V("queriesUpdated",function(d){const e=u(c).Cb,f=Y(2);mL(f.vd,e,d);return x()});S();R(3,"div",27);Ap(4,gU,2,2,"div",28)(5,qX,8,4);S()();R(6,"td",22)(7,"div",29);Jq(8,xX,8,3,"div",30,eU);S()();R(10,"td",23)(11,"div",31);Ap(12,yX,3,0,"div",32)(13,zX,3,0,"div",33);R(14,"div",34);V("click",function(){const d=u(c).Cb,e=Y(2);lL(e.vd,d);return x()});R(15,"mat-icon");Z(16,"content_copy");S()();R(17,"div",35);V("click",function(){const d=
u(c).Cb,e=Y(2);qL(e.vd,d);return x()});R(18,"mat-icon");Z(19,"delete");S()()()()()}if(a&2){a=b.da;const c=b.Cb;b=b.Ty;const d=Y(2);D(2);H("queries",a.queries);D(2);Eq(1===d.Xw?4:2===d.Xw?5:-1);D(4);Lq(d.WA);D(4);Eq(0!==c?12:-1);D();Eq(c!==b-1?13:-1)}}
function BX(a){a&1&&(R(0,"table",10)(1,"thead")(2,"th",19)(3,"div",20),Z(4," Queries to match nodes "),R(5,"div",21)(6,"mat-icon"),Z(7,"help_outline"),S()(),Ap(8,fU,5,0,"ng-template",null,1,Qr),S()(),R(10,"th",22),Z(11,"Styles to apply"),S(),Mq(12,"th",23),S(),R(13,"tbody"),Jq(14,AX,20,4,"tr",null,Hq),S()());if(a&2){a=zr(9);const b=Y();D(5);H("xapInlineDialog",a)("overlaySize",b.hd)("hoverDelayMs",10);D(9);Lq(b.rules())}}
function CX(a){a&1&&(R(0,"div",11),Z(1,' No rules configured. Click the "Add rule" button below to get started. '),S())}function sX(a,b,c,d){let e=d;d.startsWith("rgb")?e=DX(d):d.startsWith("#")||(e=lI[d]||d);pL(a.vd,b,c,e)}function tX(a,b){let c;return(null==(c=a.styles[b.id])?void 0:c.value)||""}function EX(a,b){if(b.files&&0!==b.files.length){var c=new FileReader;c.onload=d=>{let e;d=JSON.parse(null==(e=d.target)?void 0:e.result);gL(a.vd,d)};c.readAsText(b.files[0])}}
function DX(a){return(a=a.match(/^rgba?\((\d+),\s*(\d+),\s*(\d+)(?:,\s*(\d+\.{0,1}\d*))?\)$/))?(a=a.slice(1,4).map(b=>Number(b).toString(16).padStart(2,"0")).join(""),a[0]===a[1]&&a[2]===a[3]&&a[4]===a[5]?`#${a[0]}${a[2]}${a[4]}`:`#${a}`):"unknown"}
var FX=class{constructor(a,b,c){this.C=a;this.D=b;this.vd=c;this.rules=this.vd.rules;this.jl=this.vd.jl;this.cC=Bs(()=>0<this.rules().length);this.ft=dL;this.WA=eL;this.Du=[{type:"title",Od:"Match label"},{type:"list",Od:"Match attributes"},{type:"input",Od:"Match inputs (op node only)"},{type:"output",Od:"Match outputs (op node only)"}];this.Eu=[{type:"op_nodes",label:"Ops only"},{type:"layer_nodes",label:"Layers only"},{type:"op_and_layer_nodes",label:"Ops & layers"}];this.Bu=[{type:"regex",label:"Regex"},
{type:"attr_value_range",label:"Attribute value range"}];this.hd={minWidth:0,minHeight:0,maxWidth:340};this.nm={};Js(()=>{this.nm=this.vd.D();bn(this.D)})}aw(a,b){oL(this.vd,a,b)}get Xw(){return this.C.La().length}get zw(){return this.C.La()[0].id}get KD(){return this.C.La()[1].id}};FX.K=function(a){return new (a||FX)(z(VJ),z(Sr),z(rL))};
FX.Da=Gf({type:FX,ha:[["node-styler-dialog"]],ka:!0,features:[Mr],za:26,Aa:1,Ca:[["input",""],["helpQuery",""],["cb",""],["colorPicker",""],["colorInput",""],["numberInput",""],["mat-dialog-title","","cdkDrag","","cdkDragRootElement",".cdk-overlay-pane","cdkDragHandle","",1,"title-container"],[1,"title"],[1,"description"],["mat-dialog-content","",1,"dialog-content"],["border","0","cellspacing","0","cellpadding","0",1,"rules-table"],[1,"no-rules-message"],["mat-dialog-actions","",1,"btns-container"],
[1,"left-btns"],["mat-flat-button","","color","primary",1,"action-button","add-rule",3,"click"],["mat-flat-button","","color","primary",1,"action-button","load-rules",3,"click"],["mat-flat-button","","color","primary",1,"action-button","save-rules",3,"click"],["type","file","accept","application/JSON",1,"import-rules-input",3,"change"],["mat-button","","mat-dialog-close","",1,"close"],[1,"query"],[1,"query-header-container"],["dialogLabel","help",1,"icon-container",3,"xapInlineDialog","overlaySize",
"hoverDelayMs"],[1,"styles"],[1,"action-btns"],[1,"model-explorer-help-popup"],[1,"bold"],[3,"queriesUpdated","queries"],[1,"node-list-viewers-container"],[1,"node-list-viewer-container"],[1,"styles-container"],[1,"color-style-container"],[1,"action-buttons-container"],["matTooltip","Move up",1,"icon-container","move-up"],["matTooltip","Move down",1,"icon-container","move-down"],["matTooltip","Duplicate rule",1,"icon-container","duplicate-rule",3,"click"],["matTooltip","Delete rule",1,"icon-container",
"delete-rule",3,"click"],["labelSuffix","matched node",3,"nodes","rendererId"],[1,"label"],[1,"style-name-container"],["type","checkbox",3,"change","checked"],[3,"click"],[1,"color-picker-container"],["type","color",3,"change","value"],["type","text",3,"change","value"],[1,"number-editor-container"],[3,"change","value"],["matTooltip","Move up",1,"icon-container","move-up",3,"click"],["matTooltip","Move down",1,"icon-container","move-down",3,"click"]],ta:function(a,b){if(a&1){const c=Ug();R(0,"div",
6)(1,"div",7);Z(2,"Node styler");S();R(3,"div",8);Z(4," Use the buttons below to apply styling rules to this graph ");S()();R(5,"div",9);Ap(6,BX,16,3,"table",10)(7,CX,2,0,"div",11);S();R(8,"div",12)(9,"div",13)(10,"button",14);V("click",function(){u(c);iL(b.vd);return x()});R(11,"mat-icon");Z(12,"add");S();Z(13," Add rule ");S();R(14,"button",15);V("click",function(){u(c);const d=zr(23);return x(d.click())});R(15,"mat-icon");Z(16,"file_upload");S();Z(17," Import rules ");S();R(18,"button",16);V("click",
function(){u(c);const d=document.createElement("a");d.download="node_styler_rules.json";const e=`data:text/json;charset=utf-8, ${encodeURIComponent(JSON.stringify(b.rules(),null,2))}`;ud(d,rd(e));d.click();return x()});R(19,"mat-icon");Z(20,"file_download");S();Z(21," Export rules ");S();R(22,"input",17,0);V("change",function(){u(c);const d=zr(23);return x(EX(b,d))});S()();R(24,"button",18);Z(25,"Done");S()()}a&2&&(D(6),Eq(b.cC()?6:7))},Ha:[ht,$S,dU,cU,eT,BB,AB,vB,zC,sC,uC,wC,vC,$C,hG,AE,fG,iI,hI,
bT,HG,EG],styles:["[_ngcontent-%COMP%]:not(mat-icon){font-family:Google Sans Text,Arial,Helvetica,sans-serif!important;letter-spacing:normal!important}.title-container[_ngcontent-%COMP%]{position:relative}.title[_ngcontent-%COMP%]{font-size:24px;font-weight:500;margin-top:-30px}.description[_ngcontent-%COMP%]{line-height:14px;font-size:12px;color:#777;margin-top:6px;margin-bottom:4px;font-weight:400}.btns-container[_ngcontent-%COMP%]{width:100%;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between;padding:0 24px}.action-button[_ngcontent-%COMP%]{height:30px;padding:0 10px;font-size:13px}.action-button[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{margin-right:2px}.import-rules-input[_ngcontent-%COMP%]{display:none}.dialog-content[_ngcontent-%COMP%]{padding:0;-moz-box-sizing:border-box;box-sizing:border-box;border-top:1px solid #e6e6e6;border-bottom:1px solid #e6e6e6;background-color:#f9f9f9;overflow:hidden auto}.dialog-content[_ngcontent-%COMP%]   .no-rules-message[_ngcontent-%COMP%]{padding:12px 24px;font-size:12px;color:#777}.common-input[_ngcontent-%COMP%]{padding:0 4px;border:1px solid #ccc;border-radius:4px;height:22px;-moz-box-sizing:border-box;box-sizing:border-box;outline:none}.common-input[_ngcontent-%COMP%]:focus{border-color:#1a73e8;outline:1px solid #1a73e8}.rules-table[_ngcontent-%COMP%]{width:100%;font-size:12px}.rules-table[_ngcontent-%COMP%]   td[_ngcontent-%COMP%]:not(.action-btns), .rules-table[_ngcontent-%COMP%]   th[_ngcontent-%COMP%]:not(.action-btns){border-right:1px solid #e6e6e6}.rules-table[_ngcontent-%COMP%]   tr[_ngcontent-%COMP%]   td[_ngcontent-%COMP%]{border-bottom:1px solid #e6e6e6}.rules-table[_ngcontent-%COMP%]   thead[_ngcontent-%COMP%]{text-align:left;height:32px;position:-webkit-sticky;position:sticky;top:0;background-color:#f1f1f1;z-index:100}.rules-table[_ngcontent-%COMP%]   thead[_ngcontent-%COMP%]   th[_ngcontent-%COMP%]{font-weight:500;border-bottom:1px solid #e6e6e6}.rules-table[_ngcontent-%COMP%]   thead[_ngcontent-%COMP%]   th.query[_ngcontent-%COMP%]{padding-left:24px;width:410px}.rules-table[_ngcontent-%COMP%]   thead[_ngcontent-%COMP%]   th.query[_ngcontent-%COMP%]   .query-header-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}.rules-table[_ngcontent-%COMP%]   thead[_ngcontent-%COMP%]   th.query[_ngcontent-%COMP%]   .query-header-container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]{width:16px;height:16px;opacity:.6;cursor:pointer;margin-left:4px}.rules-table[_ngcontent-%COMP%]   thead[_ngcontent-%COMP%]   th.query[_ngcontent-%COMP%]   .query-header-container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]:hover{opacity:1}.rules-table[_ngcontent-%COMP%]   thead[_ngcontent-%COMP%]   th.query[_ngcontent-%COMP%]   .query-header-container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{font-size:16px;width:16px;height:16px}.rules-table[_ngcontent-%COMP%]   thead[_ngcontent-%COMP%]   th.target[_ngcontent-%COMP%]{padding-left:12px}.rules-table[_ngcontent-%COMP%]   thead[_ngcontent-%COMP%]   th.styles[_ngcontent-%COMP%]{padding-left:12px;min-width:254px}.rules-table[_ngcontent-%COMP%]   tbody[_ngcontent-%COMP%]   td[_ngcontent-%COMP%]{padding-top:8px;padding-bottom:8px}.rules-table[_ngcontent-%COMP%]   tbody[_ngcontent-%COMP%]   td.query[_ngcontent-%COMP%]{padding-left:24px;padding-right:12px;vertical-align:top}.rules-table[_ngcontent-%COMP%]   tbody[_ngcontent-%COMP%]   td.query[_ngcontent-%COMP%]   .node-list-viewers-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;gap:12px}.rules-table[_ngcontent-%COMP%]   tbody[_ngcontent-%COMP%]   td.query[_ngcontent-%COMP%]   .node-list-viewer-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;width:-webkit-fit-content;width:-moz-fit-content;width:fit-content;margin-top:12px}.rules-table[_ngcontent-%COMP%]   tbody[_ngcontent-%COMP%]   td.query[_ngcontent-%COMP%]   .node-list-viewer-container[_ngcontent-%COMP%]   .label[_ngcontent-%COMP%]{line-height:18px}.rules-table[_ngcontent-%COMP%]   tbody[_ngcontent-%COMP%]   td.target[_ngcontent-%COMP%]{vertical-align:top;padding-left:12px;padding-right:12px}.rules-table[_ngcontent-%COMP%]   tbody[_ngcontent-%COMP%]   td.styles[_ngcontent-%COMP%]{padding-left:12px;padding-right:18px;vertical-align:top}.rules-table[_ngcontent-%COMP%]   tbody[_ngcontent-%COMP%]   td.styles[_ngcontent-%COMP%]   .styles-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;gap:2px}.rules-table[_ngcontent-%COMP%]   tbody[_ngcontent-%COMP%]   td.action-btns[_ngcontent-%COMP%]{vertical-align:top;padding-right:12px;width:80px}.rules-table[_ngcontent-%COMP%]   tbody[_ngcontent-%COMP%]   td.action-btns[_ngcontent-%COMP%]   .action-buttons-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-pack:end;-webkit-justify-content:flex-end;-moz-box-pack:end;-ms-flex-pack:end;justify-content:flex-end;gap:2px}.rules-table[_ngcontent-%COMP%]   tbody[_ngcontent-%COMP%]   td.action-btns[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;opacity:.6;cursor:pointer;margin-top:2px}.rules-table[_ngcontent-%COMP%]   tbody[_ngcontent-%COMP%]   td.action-btns[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]:hover{opacity:1}.rules-table[_ngcontent-%COMP%]   tbody[_ngcontent-%COMP%]   td.action-btns[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{font-size:16px;width:16px;height:16px}.rules-table[_ngcontent-%COMP%]   tbody[_ngcontent-%COMP%]   .color-style-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}.rules-table[_ngcontent-%COMP%]   tbody[_ngcontent-%COMP%]   .color-style-container[_ngcontent-%COMP%]   .style-name-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;cursor:pointer;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;width:100px}.rules-table[_ngcontent-%COMP%]   tbody[_ngcontent-%COMP%]   .color-style-container[_ngcontent-%COMP%]   .style-name-container[_ngcontent-%COMP%]   input[_ngcontent-%COMP%]{cursor:pointer}.rules-table[_ngcontent-%COMP%]   tbody[_ngcontent-%COMP%]   .color-style-container[_ngcontent-%COMP%]   .color-picker-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;overflow:hidden;border-radius:4px;border:1px solid #ccc;margin-left:4px;width:-webkit-fit-content;width:-moz-fit-content;width:fit-content;background-color:#fff;-moz-box-sizing:border-box;box-sizing:border-box;height:24px;position:relative}.rules-table[_ngcontent-%COMP%]   tbody[_ngcontent-%COMP%]   .color-style-container[_ngcontent-%COMP%]   .color-picker-container[_ngcontent-%COMP%]:focus-within{border-color:#1a73e8;outline:1px solid #1a73e8}.rules-table[_ngcontent-%COMP%]   tbody[_ngcontent-%COMP%]   .color-style-container[_ngcontent-%COMP%]   .color-picker-container[_ngcontent-%COMP%]   label[_ngcontent-%COMP%]{display:inline-block;width:16px;height:16px;margin:2px;background-color:red;border-radius:4px;cursor:pointer;border:1px solid #ddd}.rules-table[_ngcontent-%COMP%]   tbody[_ngcontent-%COMP%]   .color-style-container[_ngcontent-%COMP%]   .color-picker-container[_ngcontent-%COMP%]   label[_ngcontent-%COMP%]   input[_ngcontent-%COMP%]{visibility:hidden}.rules-table[_ngcontent-%COMP%]   tbody[_ngcontent-%COMP%]   .color-style-container[_ngcontent-%COMP%]   .color-picker-container[_ngcontent-%COMP%]   input[type=text][_ngcontent-%COMP%]{width:100px;border:none;height:100%;background-color:transparent;outline:none}.rules-table[_ngcontent-%COMP%]   tbody[_ngcontent-%COMP%]   .color-style-container[_ngcontent-%COMP%]   .number-editor-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;overflow:hidden;margin-left:4px;width:-webkit-fit-content;width:-moz-fit-content;width:fit-content}.rules-table[_ngcontent-%COMP%]   tbody[_ngcontent-%COMP%]   .color-style-container[_ngcontent-%COMP%]   .number-editor-container[_ngcontent-%COMP%]   input[_ngcontent-%COMP%]{width:80px;height:28px;border:1px solid #ccc;border-radius:4px;background-color:transparent;-moz-box-sizing:border-box;box-sizing:border-box;padding:0 4px;outline:none}"],
Ra:0});var GX=class{constructor(a,b,c){this.Gh=a;this.C=b;this.xb=c;this.jl=this.C.jl}};GX.K=function(a){return new (a||GX)(z(pC),z(rL),z(kp))};
GX.Da=Gf({type:GX,ha:[["node-styler"]],ka:!0,features:[Mr],za:3,Aa:2,Ca:[["mat-icon-button","","aria-label","node-styler","matTooltip","Style nodes with custom rules",3,"click"]],ta:function(a,b){a&1&&(R(0,"button",0),V("click",function(){b.Gh.open(FX,{width:"800px",height:"600px",xb:b.xb,Tb:!1,Tm:!1})}),R(1,"mat-icon"),Z(2,"palette"),S()());a&2&&(D(),nq("highlight",b.jl()))},Ha:[ht,BB,AB,yB,hG,AE,fG,xC,iI,hI,HG],styles:["button[_ngcontent-%COMP%]   mat-icon.highlight[_ngcontent-%COMP%]{background:-webkit-linear-gradient(45deg,#0089ff 0 30%,#f1af00 70% 100%);background:linear-gradient(45deg,#0089ff 0 30%,#f1af00 70% 100%);background-clip:text;-webkit-background-clip:text;-webkit-text-fill-color:transparent}"],
Ra:0});function HX(a){a&1&&(R(0,"div",5)(1,"a",6)(2,"button",7)(3,"mat-icon"),Z(4,"bug_report"),S()()(),R(5,"a",8)(6,"button",9)(7,"mat-icon"),Z(8,"article"),S()()()())}function IX(a){a&1&&(R(0,"div",5)(1,"a",10)(2,"button",11)(3,"mat-icon"),Z(4,"article"),S()()(),R(5,"a",12)(6,"button",13)(7,"mat-icon"),Z(8,"bug_report"),S()()()(),Mq(9,"open-in-new-tab-button"))}var JX=class{constructor(a){this.C=a;this.fm=new Fk}get ee(){return!1}get Ad(){return!0}get CB(){return this.C.C}};
JX.K=function(a){return new (a||JX)(z(VJ))};
JX.Da=Gf({type:JX,ha:[["title-bar"]],outputs:{fm:"titleClicked"},ka:!0,features:[Mr],Zb:["*"],za:14,Aa:3,Ca:[[1,"container"],["matTooltip","Back home",1,"title",3,"click","matTooltipDisabled"],[1,"right-container"],[1,"divider"],[1,"divider","tighter-right"],[1,"icons-container"],["href","https://b.corp.google.com/issues/new?component=1338731&template=1850894","target","_blank",1,"bug-report"],"mat-icon-button;;aria-label;bug;matTooltip;File bug or feature request".split(";"),["href","http://go/model-explorer-doc",
"target","_blank"],"mat-icon-button  aria-label doc matTooltip g3doc".split(" "),["href","https://github.com/google-ai-edge/model-explorer","target","_blank"],"mat-icon-button  aria-label doc matTooltip GitHub".split(" "),["href","https://github.com/google-ai-edge/model-explorer/issues","target","_blank"],"mat-icon-button;;aria-label;bug;matTooltip;File issue or feature request".split(";")],ta:function(a,b){a&1&&(Xq(),R(0,"div",0)(1,"div",1),V("click",function(){return b.fm.emit()}),Mq(2,"me-logo"),
Z(3," Model Explorer "),Mq(4,"new-version-chip"),S(),R(5,"div",2),Mq(6,"node-data-provider-dropdown")(7,"div",3)(8,"graph-selector")(9,"div",4)(10,"node-styler"),Ap(11,HX,9,0,"div",5)(12,IX,10,0),Yq(13),S()());a&2&&(D(),H("matTooltipDisabled",b.CB),D(10),Eq(b.ee?11:-1),D(),Eq(b.Ad?12:-1))},Ha:[ht,BB,AB,yB,hG,AE,fG,zS,WJ,iI,hI,gG,ES,MS,GX,kI],styles:[".container[_ngcontent-%COMP%]{width:100%;height:100%;padding:8px;padding-left:12px;-moz-box-sizing:border-box;box-sizing:border-box;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between;border-bottom:1px solid #ddd}.container[_ngcontent-%COMP%]   .title[_ngcontent-%COMP%]{font-weight:400;cursor:pointer;color:#444746;font-size:20px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}.container[_ngcontent-%COMP%]   .title[_ngcontent-%COMP%]   me-logo[_ngcontent-%COMP%]{margin-right:6px}.container[_ngcontent-%COMP%]   .title[_ngcontent-%COMP%]   new-version-chip[_ngcontent-%COMP%]{margin-left:16px}.container[_ngcontent-%COMP%]   .right-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}.container[_ngcontent-%COMP%]   .divider[_ngcontent-%COMP%]{width:1px;height:28px;border-left:1px solid #ccc;margin:0 16px}.container[_ngcontent-%COMP%]   .divider.tighter-right[_ngcontent-%COMP%]{margin-right:4px}.container[_ngcontent-%COMP%]   .icons-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}.container[_ngcontent-%COMP%]   .icons-container[_ngcontent-%COMP%]   a[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;color:#000}.container[_ngcontent-%COMP%]   open-in-new-tab-button[_ngcontent-%COMP%]{margin-left:8px}"],
Ra:0});function KX(a){a&1&&Mq(0,"benchmark-runner")}function LX(a){if(a&1){const b=Ug();R(0,"title-bar",4);V("titleClicked",function(){u(b);const c=Y(3);return x(c.fm.emit())});Yq(1);S()}}function MX(a){a&1&&(R(0,"div",3),Z(1,"No graphs"),S())}function NX(a){a&1&&Mq(0,"split-panes-container")}function OX(a){a&1&&(Ap(0,LX,2,0,"title-bar"),R(1,"div",2),Ap(2,MX,2,0,"div",3)(3,NX,1,0,"split-panes-container"),S());a&2&&(a=Y(2),Eq(a.kE?0:-1),D(2),Eq(a.bC?2:3))}
function PX(a){if(a&1){const b=Ug();R(0,"div",1);V("dragover",function(c){u(b);const d=Y();return x(d.Lq(c))})("drop",function(c){u(b);const d=Y();return x(d.Mq(c))});Ap(1,KX,1,0,"benchmark-runner")(2,OX,4,2,"div",2);S()}a&2&&(a=Y(),D(),Eq(a.zc?1:2))}PH`https://fonts.googleapis.com/icon?family=Material+Icons`;function QX(a){sa(function*(){yield a.N.N;a.ready=!0;bn(a.F)})}
function RX(a,b,c){for(const d of a.graphCollections)for(const e of d.graphs)if(e.id===c&&(""===b||e.collectionLabel===b))return e}function SX(a,b,c){const d=zJ(a.C);d?BL(a.D,tI(),b,d,{[d.id]:c},!1):console.warn("Model graph in pane with index 0 doesn't exist")}function TX(a,b,c){const d=zJ(a.C);d?BL(a.D,tI(),b,d,c,!1):console.warn("Model graph in pane with index 0 doesn't exist")}
function UX(a,b,c){var d=a.graphCollections,e=void 0;for(const f of d)if(d=f.graphs.find(g=>g.id===c)){e=d;break}e?(pJ(a.C,e,0),e=a.C.La()[0].id,a.C.N.set(void 0),wJ(a.C,e),a.C.F.set(void 0),EJ(a.C,e,b)):console.warn(`Failed to locate graph with id "${c}"`)}function VX(a,b,c){sa(function*(){yield Promise.all(b.map(d=>EL(a.D,d,c)))})}
var WX=class{constructor(a,b,c,d,e,f,g){this.C=a;this.F=b;this.G=c;this.J=d;this.N=e;this.O=f;this.D=g;this.zc=!1;this.bD=[];this.fm=new Fk;this.vy=new Fk;this.Gw=new Fk;this.ux=new Fk;this.ready=!1;Js(()=>{const k=this.O.C();k&&this.vy.emit(k)});Js(()=>{this.ux.emit(this.C.Mg())});window.addEventListener("message",k=>{k=k.data;switch(k.cmd){case "model-explorer-load-node-data-file":var m=k.path;if(m){var p=zJ(this.C);p?VX(this,[m],p):console.warn("Model graph in pane with index 0 doesn't exist")}break;
case "model-explorer-select-node-by-output-tensor-name":if(m=k.tensorName){const t=xJ(this.C);if(t){k=void 0;for(p of t.nodes)if(nI(p)){const w=p.outputsMetadata||{};for(const B of Object.keys(w))if(m===w[B].tensor_name){k=p;break}if(k)break}k&&UX(this,k.id,t.id)}}break;case "model-explorer-select-node-by-node-id":k.nodeId&&(p=xJ(this.C))&&(m=p.nodesById[m])&&UX(this,m.id,p.id)}});C(this.C.ua,sL(this.G)).subscribe(k=>{this.Gw.next(k)});QX(this)}Fb(){this.C.config.set(this.config||{});gJ(this.C,this.graphCollections);
this.C.N.set(this.Ce);if(!this.Ce||0===this.Ce.paneStates.length){if(0<this.graphCollections.length&&0<this.graphCollections[0].graphs.length){var a=[...this.graphCollections[0].graphs].sort((d,e)=>e.nodes.length-d.nodes.length)[0];qJ(this.C,a)}}else if(1===this.Ce.paneStates.length){var b=this.Ce.paneStates[0];a=RX(this,b.selectedCollectionLabel,b.selectedGraphId);b=!0===b.flattenLayers;a?qJ(this.C,a,b):qJ(this.C,this.graphCollections[0].graphs[0],b);sJ(this.C,b)}else if(2===this.Ce.paneStates.length){a=
this.Ce.paneStates[0];b=RX(this,a.selectedCollectionLabel,a.selectedGraphId);var c=!0===a.flattenLayers;b?qJ(this.C,b,c):qJ(this.C,this.graphCollections[0].graphs[0],c);sJ(this.C,c);b=this.Ce.paneStates[1];c=!0===b.flattenLayers;const d=RX(this,b.selectedCollectionLabel,b.selectedGraphId);d?hJ(this.C,d,c):hJ(this.C,this.graphCollections[0].graphs[0],c);a.selected?vJ(this.C,0):b.selected&&vJ(this.C,1);AJ(this.C,a.widthFraction)}}Oq(a){if(" "===a.key)yI()||this.C.fa.next({});else if("f"===a.key&&(a.ctrlKey||
a.metaKey)){let b;(null==(b=this.config)?0:b.hideTitleBar)||a.preventDefault();this.C.wa.next({})}}Lq(a){a.preventDefault()}Mq(a){const b=this;return sa(function*(){if(a.dataTransfer){a.stopPropagation();a.preventDefault();var c=[],d;if(null==(d=a.dataTransfer)?0:d.items)[...a.dataTransfer.items].forEach(f=>{"file"===f.kind&&(f=f.getAsFile())&&c.push(f)});else{var e;c.push(...((null==(e=a.dataTransfer)?void 0:e.files)||[]))}d=!1;if(0!==c.length)for(const f of c){if(!f.name.endsWith(".json"))continue;
const g=new FileReader;e=yield new Promise(k=>{g.onload=m=>{let p;m=JSON.parse(null==(p=m.target)?void 0:p.result);k(m)};g.readAsText(f)});null!=e.results&&null==e.results.results?(SX(b,f.name,e),d=!0):Object.values(e).some(k=>null!=k.results)&&(TX(b,f.name,e),d=!0)}d||b.J.open("File(s) not supported","Dismiss")}})}get bC(){return 0===this.graphCollections.reduce((a,b)=>a+b.graphs.length,0)}get kE(){let a;return!(null==(a=this.config)?0:a.hideTitleBar)}};
WX.K=function(a){return new (a||WX)(z(VJ),z(Sr),z(yi),z(vE),z(zM),z(aJ),z(KL))};
WX.Da=Gf({type:WX,ha:[["model-graph-visualizer"]],Ia:function(a,b){a&1&&V("keydown",function(c){return b.Oq(c)},!1,Ci)},inputs:{graphCollections:"graphCollections",config:"config",Ce:"initialUiState",zc:"benchmark",bD:"nodeDataSources"},outputs:{fm:"titleClicked",vy:"uiStateChanged",Gw:"modelGraphProcessed",ux:"remoteNodeDataPathsChanged"},ka:!0,features:[Kr([VJ,fO,KL,rL,aJ,bJ]),Mr],Zb:["*"],za:1,Aa:1,Ca:[["class","container",3,"dragover","drop",4,"ngIf"],[1,"container",3,"dragover","drop"],[1,"main"],
[1,"no-graphs-message"],[3,"titleClicked"]],ta:function(a,b){a&1&&(Xq(),Ap(0,PX,3,1,"div",0));a&2&&H("ngIf",b.ready)},Ha:[eO,ht,ct,JX,eS],styles:[".container[_ngcontent-%COMP%]{width:100%;height:100%;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;position:relative}.container[_ngcontent-%COMP%]   benchmark-runner[_ngcontent-%COMP%]{-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1}.container[_ngcontent-%COMP%]   title-bar[_ngcontent-%COMP%]{height:48px;min-height:48px}.container[_ngcontent-%COMP%]   .main[_ngcontent-%COMP%]{-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;position:relative;overflow:hidden}.container[_ngcontent-%COMP%]   .main[_ngcontent-%COMP%]   .no-graphs-message[_ngcontent-%COMP%]{position:absolute;top:0;left:0;width:100%;height:100%;z-index:200;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;color:#999}.container[_ngcontent-%COMP%]   .main[_ngcontent-%COMP%]   split-panes-container[_ngcontent-%COMP%]{-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1}.container[_ngcontent-%COMP%]   .main[_ngcontent-%COMP%]   graph-selector[_ngcontent-%COMP%]{position:absolute;top:12px;left:12px;z-index:100}  .model-explorer-help-popup{font-size:12px;padding:8px;background-color:#fff;line-height:normal}  .model-explorer-help-popup .bold{font-weight:700}  .model-explorer-help-popup .shortcut{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;color:#777;margin-top:8px}  .model-explorer-help-popup .shortcut mat-icon{font-size:20px;width:20px;height:20px;margin-right:2px;color:#777}  .model-explorer-help-popup .description{color:#777;margin-top:8px}"],
Ra:0});const XX=new Map([[1,"float"],[2,"double"],[3,"int32"],[4,"uint8"],[5,"int16"],[6,"int8"],[7,"string"],[8,"complex64"],[9,"int64"],[10,"bool"],[11,"qint8"],[12,"qint8"],[13,"qint32"],[14,"bfloat16"],[15,"qint16"],[16,"qint16"],[17,"uint16"],[18,"complex128"],[19,"half"],[20,"resource"],[21,"variant"],[22,"uint32"],[23,"uint64"]]);
function YX(a,b){const c={};if(null!=b.weightsManifest)for(var d of b.weightsManifest)for(var e of d.weights)c[e.name]=e;const f={};d=b.modelTopology.node.map(p=>{p={id:p.name,label:p.op,namespace:p.name,incomingEdges:ZX(p),attrs:$X(p)};return f[p.id]=p});for(var g of b.modelTopology.node)if(f[g.name])for(b=g.input||[],e=0;e<b.length;e++){var k=b[e],m=k.split(":");let p=k;k="0";2===m.length&&(p=m[0],k=m[1]);if(m=f[p])null==m.outputsMetadata&&(m.outputsMetadata=[]),m.outputsMetadata.push({id:k,attrs:[]})}for(const p of d)if(g=
c[p.id])null==p.outputsMetadata&&(p.outputsMetadata=[]),p.outputsMetadata.push({id:"0",attrs:[{key:"tensor_name",value:g.name},{key:"shape",value:g.shape.join("x")},{key:"dtype",value:g.dtype}]});return{label:a,graphs:[{id:"default",nodes:d}]}}function ZX(a){return(a.input||[]).map((b,c)=>{const d=b.split(":");return{sourceNodeId:2===d.length?d[0]:b,sourceNodeOutputId:2===d.length?d[1]:"0",targetNodeInputId:`${c}`}})}
function $X(a){a=a.attr||{};const b=[];for(const c of Object.keys(a)){const d=a[c];let e="";null!=d.list?0<(d.list.s||[]).length?e=(d.list.s||[]).map(f=>aY({s:f})).join(", "):0<(d.list.b||[]).length?e=(d.list.b||[]).map(f=>aY({b:f})).join(", "):0<(d.list.f||[]).length?e=(d.list.f||[]).map(f=>aY({f})).join(", "):0<(d.list.i||[]).length?e=(d.list.i||[]).map(f=>aY({i:f})).join(", "):e="[]":e=aY(d);b.push({key:c,value:e})}return b}
function aY(a){let b="";null!=a.s?b=Array.isArray(a.s)?`${a.s.map(c=>"number"===typeof c?String.fromCharCode(c):`${c}`).join("")}`:atob(a.s):null!=a.i?b=`${a.i}`:null!=a.f?b=`${a.f}`:null!=a.b?b=a?"true":"false":null!=a.type&&(b=XX.get(a.type)||"unknown");return b};const bY=/^\/cns\/(.*)|^\/placer\/(.*)|^\/tfhub\/(.*)|^\/google_src\/(.*)|^\/google3\/(.*)|^google3\/(.*)/;function cY(a){return sa(function*(){return new Promise((b,c)=>{const d=new FileReader;d.onload=()=>{const e=JSON.parse(d.result),{graphCollections:f,error:g}=dY(a.name,e);g?c(g):f&&b(f)};d.readAsText(a)})})}
function dY(a,b){var c;null==b.modelTopology?c=null!=b.label&&null!=b.graphs?{graphCollections:[b]}:Array.isArray(b)&&null!=b[0].subgraphs?{graphCollections:eY(b,a)}:Array.isArray(b)&&(0===b.length||0<b.length&&null!=b[0].id&&null!=b[0].nodes)?{graphCollections:[{label:a,graphs:b}]}:{error:"Unsupported JSON format"}:c={graphCollections:[YX(a,b)]};return c}function eY(a,b){return a.map(c=>({label:""===c.label?b:`${b} (${c.label})`,graphs:c.subgraphs}))};var fY={fF:"Not started",gF:"Converting",lF:"Uploading",DONE:"Done",aF:"Error"};const gY=(a,b)=>b.id;function hY(a){a&1&&(R(0,"span",4),Z(1,"Default"),S())}function iY(a,b){if(a&1){const c=Ug();R(0,"div",2);V("click",function(){const d=u(c).da;Y().he.next(d);return x()});R(1,"div",3);Z(2);Ap(3,hY,2,0,"span",4);S();R(4,"div",5);Z(5);S()()}if(a&2){a=b.da;b=b.Cb;const c=Y();nq("selected",(null==c.Sc?null:c.Sc.id)===a.id);D(2);Br(" ",a.name||a.id," ");D();Eq(0===b?3:-1);D(2);Br(" ",a.description||"-"," ")}}var jY=class{constructor(){this.cq=[];this.he=new Fk}};
jY.K=function(a){return new (a||jY)};
jY.Da=Gf({type:jY,ha:[["adapter-selector-panel"]],inputs:{cq:"candidates",Sc:"selectedAdapter"},outputs:{he:"onClose"},ka:!0,features:[Mr],za:3,Aa:1,Ca:[[1,"container"],[1,"candidate-container",3,"selected"],[1,"candidate-container",3,"click"],[1,"name"],[1,"default-label"],[1,"description"]],ta:function(a,b){a&1&&(R(0,"div",0),Jq(1,iY,6,5,"div",1,gY),S());a&2&&(H("@transformPanel","showing"),D(),Lq(b.cq))},Ha:[ht,hG,cB,iI,PD,Cx],styles:[".container[_ngcontent-%COMP%]{box-shadow:0 2px 6px 2px rgba(0,0,0,.15),0 1px 2px 0 rgba(0,0,0,.3);-webkit-transform-origin:top center;transform-origin:top center;font-size:13px;padding:8px 0;border-radius:8px;background-color:#fff}.container[_ngcontent-%COMP%]   .candidate-container[_ngcontent-%COMP%]{padding:8px 16px}.container[_ngcontent-%COMP%]   .candidate-container.selected[_ngcontent-%COMP%]{background-color:#c2e7ff}.container[_ngcontent-%COMP%]   .candidate-container[_ngcontent-%COMP%]:hover:not(.selected){background-color:#f2f2f2;cursor:pointer}.container[_ngcontent-%COMP%]   .candidate-container[_ngcontent-%COMP%]   .name[_ngcontent-%COMP%]{font-size:14px;color:#1f1f1f;line-height:20px;margin-bottom:2px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}.container[_ngcontent-%COMP%]   .candidate-container[_ngcontent-%COMP%]   .name[_ngcontent-%COMP%]   .default-label[_ngcontent-%COMP%]{display:inline-block;padding:0 10px;-moz-box-sizing:border-box;box-sizing:border-box;background-color:#f3e0b4;color:#70571e;border:1px solid #dbac41;font-size:11px;line-height:14px;border-radius:99px;margin-left:8px}.container[_ngcontent-%COMP%]   .candidate-container[_ngcontent-%COMP%]   .description[_ngcontent-%COMP%]{font-size:12px;color:#444746}"],data:{animation:[Yy("transformPanel",
[az("void",$y({opacity:0,transform:"scale(1, 0.8)"})),bz("void => showing",Zy("120ms cubic-bezier(0, 0, 0.2, 1)",$y({opacity:1,transform:"scale(1, 1)"}))),bz("* => void",Zy("100ms linear",$y({opacity:0})))])]},Ra:0});function kY(a,b){a=a.split("/").filter(c=>""!==c);a=a[a.length-1].split(".");return 1<a.length?b.C[a[a.length-1]]||[]:[]};function lY(a){sa(function*(){let b;b=yield mY();a.extensions=b;a.kd.set(!1)})}function nY(a){return sa(function*(){const b={method:"POST",headers:{"Content-Type":"application/json"}};b.body=JSON.stringify(a);try{const c=yield fetch("/apipost/v1/send_command",b);return c.ok?{iB:yield c.json()}:{Sw:`Failed to convert model. ${c.status}`}}catch(c){return{Sw:c}}})}
function mY(){return sa(function*(){try{const a=yield fetch("/api/v1/get_extensions",{credentials:"include"});return a.ok?yield a.json():(console.error(`Failed to get extensions: ${a.status}`),[])}catch(a){return console.error("Failed to get extensions.",a),[]}})}var oY=class{constructor(){this.kd=xr(!0);this.extensions=[];lY(this)}};oY.K=function(a){return new (a||oY)};oY.pa=Md({ma:oY,ba:oY.K,ea:"root"});function pY(a,b){for(const c of Object.values(a.C))for(const d of c)if(d.id===b)return d}var qY=class{constructor(){this.C={}}register(a){for(const b of a.fileExts){null==this.C[b]&&(this.C[b]=[]);const c=this.C[b];null!=c.find(d=>d.id===a.id)?console.warn("Adapter extension exists",a):c.push(a)}}},rY=class extends qY{constructor(a){super();this.D=a;Js(()=>{if(!this.D.kd())for(const b of this.D.extensions)"adapter"===b.type&&this.register(b)})}};rY.K=function(a){return new (a||rY)(r(oY))};
rY.pa=Md({ma:rY,ba:rY.K,ea:"root"});var sY=Symbol("RouteTitle");function tY(a,b,c){const d=c.path.split("/");if(d.length>a.length||"full"===c.Yw&&(0<b.C||d.length<a.length))return null;b={};for(c=0;c<d.length;c++){const e=d[c],f=a[c];if(":"===e[0])b[e.substring(1)]=f;else if(e!==f.path)return null}return{Qk:a.slice(0,d.length),gx:b}};function uY(a,b){const c=a?vY(a):void 0;var d=b?vY(b):void 0;if(!c||!d||c.length!=d.length)return!1;for(let e=0;e<c.length;e++)if(d=c[e],!wY(a[d],b[d]))return!1;return!0}function vY(a){return[...Object.keys(a),...Object.getOwnPropertySymbols(a)]}function wY(a,b){if(Array.isArray(a)&&Array.isArray(b)){if(a.length!==b.length)return!1;a=[...a].sort();const c=[...b].sort();return a.every((d,e)=>c[e]===d)}return a===b}function xY(a){return Ek(a)?a:sn(a)?Sj(Promise.resolve(a)):jj(a)};const AY={exact:yY,subset:zY},DY={exact:BY,subset:CY,ignored:()=>!0};function EY(a,b,c){return AY[c.paths](a.root,b.root,c.Fw)&&DY[c.tb](a.tb,b.tb)&&!("exact"===c.Yb&&a.Yb!==b.Yb)}function BY(a,b){return uY(a,b)}function yY(a,b,c){if(!FY(a.segments,b.segments)||!GY(a.segments,b.segments,c)||a.C!==b.C)return!1;for(const d in b.children)if(!a.children[d]||!yY(a.children[d],b.children[d],c))return!1;return!0}
function CY(a,b){return Object.keys(b).length<=Object.keys(a).length&&Object.keys(b).every(c=>wY(a[c],b[c]))}function zY(a,b,c){return HY(a,b,b.segments,c)}
function HY(a,b,c,d){if(a.segments.length>c.length)return a=a.segments.slice(0,c.length),!FY(a,c)||0<b.C||!GY(a,c,d)?!1:!0;if(a.segments.length===c.length){if(!FY(a.segments,c)||!GY(a.segments,c,d))return!1;for(var e in b.children)if(!a.children[e]||!zY(a.children[e],b.children[e],d))return!1;return!0}e=c.slice(0,a.segments.length);c=c.slice(a.segments.length);return FY(a.segments,e)&&GY(a.segments,e,d)&&a.children.primary?HY(a.children.primary,b,c,d):!1}
function GY(a,b,c){return b.every((d,e)=>DY[c](a[e].parameters,d.parameters))}var KY=class{constructor(a=new IY([],{}),b={},c=null){this.root=a;this.tb=b;this.Yb=c}toString(){return JY(this)}},IY=class{constructor(a,b){this.segments=a;this.children=b;this.parent=null;Object.values(b).forEach(c=>c.parent=this)}get C(){return Object.keys(this.children).length}toString(){return LY(this)}},NY=class{constructor(a,b){this.path=a;this.parameters=b}toString(){return MY(this)}};
function OY(a,b){return FY(a,b)&&a.every((c,d)=>uY(c.parameters,b[d].parameters))}function FY(a,b){return a.length!==b.length?!1:a.every((c,d)=>c.path===b[d].path)}function PY(a,b){let c=[];Object.entries(a.children).forEach(([d,e])=>{"primary"===d&&(c=c.concat(b(e,d)))});Object.entries(a.children).forEach(([d,e])=>{"primary"!==d&&(c=c.concat(b(e,d)))});return c}var QY=class{};QY.K=function(a){return new (a||QY)};QY.pa=Md({ma:QY,ba:()=>new RY,ea:"root"});
function JY(a){const b=`/${SY(a.root,!0)}`,c=TY(a.tb);a="string"===typeof a.Yb?`#${encodeURI(a.Yb)}`:"";return`${b}${c}${a}`}
var RY=class{parse(a){a=new UY(a);VY(a,"/");var b=""===a.C||a.C.startsWith("?")||a.C.startsWith("#")?new IY([],{}):new IY([],WY(a));const c={};if(VY(a,"?")){do{var d,e,f=a.C.match(XY);if(e=f?f[0]:"")a.capture(e),f="",VY(a,"=")&&(d=(d=a.C.match(YY))?d[0]:"")&&(f=d,a.capture(f)),e=decodeURIComponent(e.replace(/\+/g,"%20")),f=decodeURIComponent(f.replace(/\+/g,"%20")),c.hasOwnProperty(e)?(d=c[e],Array.isArray(d)||(d=[d],c[e]=d),d.push(f)):c[e]=f}while(VY(a,"&"))}return new KY(b,c,VY(a,"#")?decodeURIComponent(a.C):
null)}};function LY(a){return a.segments.map(b=>MY(b)).join("/")}
function SY(a,b){if(!(0<a.C))return LY(a);if(b){b=a.children.primary?SY(a.children.primary,!1):"";const c=[];Object.entries(a.children).forEach(([d,e])=>{"primary"!==d&&c.push(`${d}:${SY(e,!1)}`)});return 0<c.length?`${b}(${c.join("//")})`:b}b=PY(a,(c,d)=>"primary"===d?[SY(a.children.primary,!1)]:[`${d}:${SY(c,!1)}`]);return 1===Object.keys(a.children).length&&null!=a.children.primary?`${LY(a)}/${b[0]}`:`${LY(a)}/(${b.join("//")})`}
function ZY(a){return encodeURIComponent(a).replace(/%40/g,"@").replace(/%3A/gi,":").replace(/%24/g,"$").replace(/%2C/gi,",")}function $Y(a){return ZY(a).replace(/%3B/gi,";")}function aZ(a){return ZY(a).replace(/\(/g,"%28").replace(/\)/g,"%29").replace(/%26/gi,"&")}function MY(a){return`${aZ(a.path)}${bZ(a.parameters)}`}function bZ(a){return Object.entries(a).map(([b,c])=>`;${aZ(b)}=${aZ(c)}`).join("")}
function TY(a){a=Object.entries(a).map(([b,c])=>Array.isArray(c)?c.map(d=>`${$Y(b)}=${$Y(d)}`).join("&"):`${$Y(b)}=${$Y(c)}`).filter(b=>b);return a.length?`?${a.join("&")}`:""}const cZ=/^[^\/()?;#]+/;function dZ(a){return(a=a.match(cZ))?a[0]:""}const eZ=/^[^\/()?;=#]+/,XY=/^[^=?&#]+/,YY=/^[^&#]+/;function VY(a,b){return a.C.startsWith(b)?(a.C=a.C.substring(b.length),!0):!1}
function WY(a){if(""===a.C)return{};VY(a,"/");const b=[];for(a.C.startsWith("(")||b.push(fZ(a));a.C.startsWith("/")&&!a.C.startsWith("//")&&!a.C.startsWith("/(");)a.capture("/"),b.push(fZ(a));let c={};a.C.startsWith("/(")&&(a.capture("/"),c=gZ(a,!0));let d={};a.C.startsWith("(")&&(d=gZ(a,!1));if(0<b.length||0<Object.keys(c).length)d.primary=new IY(b,c);return d}
function fZ(a){var b=dZ(a.C);if(""===b&&a.C.startsWith(";"))throw new Zd(4009,!1);a.capture(b);b=decodeURIComponent(b);const c={};for(;VY(a,";");){var d,e=a;if(d=(d=e.C.match(eZ))?d[0]:""){e.capture(d);var f="";if(VY(e,"=")){const g=dZ(e.C);g&&(f=g,e.capture(f))}c[decodeURIComponent(d)]=decodeURIComponent(f)}}return new NY(b,c)}
function gZ(a,b){const c={};for(a.capture("(");!VY(a,")")&&0<a.C.length;){var d=dZ(a.C),e=a.C[d.length];if("/"!==e&&")"!==e&&";"!==e)throw new Zd(4010,!1);e=void 0;-1<d.indexOf(":")?(e=d.slice(0,d.indexOf(":")),a.capture(e),a.capture(":")):b&&(e="primary");d=WY(a);c[e]=1===Object.keys(d).length?d.primary:new IY([],d);VY(a,"//")}return c}class UY{constructor(a){this.C=this.url=a}capture(a){if(!VY(this,a))throw new Zd(4011,!1);}}function hZ(a){return 0<a.segments.length?new IY([],{primary:a}):a}
function iZ(a){var b={};for(const [c,d]of Object.entries(a.children)){const e=c,f=iZ(d);if("primary"===e&&0===f.segments.length&&0<f.C)for(const [g,k]of Object.entries(f.children))b[g]=k;else if(0<f.segments.length||0<f.C)b[e]=f}a=new IY(a.segments,b);1===a.C&&a.children.primary&&(b=a.children.primary,a=new IY(a.segments.concat(b.segments),b.children));return a}function jZ(a){return a instanceof KY};function kZ(a,b=null,c=null){a=lZ(a);return mZ(a,[],b,c)}function lZ(a){function b(f){var g={};for(const k of f.children){const m=b(k);g[k.mb]=m}g=new IY(f.url,g);f===a&&(c=g);return g}let c;var d=b(a.root);d=hZ(d);let e;return null!=(e=c)?e:d}
function mZ(a,b,c,d){let e=a;for(;e.parent;)e=e.parent;if(0===b.length)return nZ(e,e,e,c,d);b=oZ(b);if(b.D&&1===b.C.length&&"/"==b.C[0])return nZ(e,e,new IY([],{}),c,d);var f=e;if(b.D)a=new pZ(f,!0,0);else if(a)if(null===a.parent)a=new pZ(a,!0,0);else{f=a;a=a.segments.length-1+(qZ(b.C[0])?0:1);let g=b.F;for(;g>a;){g-=a;f=f.parent;if(!f)throw new Zd(4005,!1);a=f.segments.length}a=new pZ(f,!1,a-g)}else a=new pZ(f,!1,NaN);b=a.C?rZ(a.rf,a.index,b.C):sZ(a.rf,a.index,b.C);return nZ(e,a.rf,b,c,d)}
function qZ(a){return"object"===typeof a&&null!=a&&!a.Fg&&!a.Dx}function tZ(a){return"object"===typeof a&&null!=a&&a.Fg}function nZ(a,b,c,d,e){let f={};d&&Object.entries(d).forEach(([g,k])=>{f[g]=Array.isArray(k)?k.map(m=>`${m}`):`${k}`});a=a===b?c:uZ(a,b,c);a=hZ(iZ(a));return new KY(a,f,e)}function uZ(a,b,c){const d={};Object.entries(a.children).forEach(([e,f])=>{d[e]=f===b?c:uZ(f,b,c)});return new IY(a.segments,d)}
class vZ{constructor(a,b,c){this.D=a;this.F=b;this.C=c;if(a&&0<c.length&&qZ(c[0]))throw new Zd(4003,!1);if((a=c.find(tZ))&&a!==(0<c.length?c[c.length-1]:null))throw new Zd(4004,!1);}}
function oZ(a){if("string"===typeof a[0]&&1===a.length&&"/"===a[0])return new vZ(!0,0,a);let b=0,c=!1;a=a.reduce((d,e,f)=>{if("object"===typeof e&&null!=e){if(e.Fg){const g={};Object.entries(e.Fg).forEach(([k,m])=>{g[k]="string"===typeof m?m.split("/"):m});return[...d,{Fg:g}]}if(e.Dx)return[...d,e.Dx]}return"string"!==typeof e?[...d,e]:0===f?(e.split("/").forEach((g,k)=>{if(0!=k||"."!==g)0==k&&""===g?c=!0:".."===g?b++:""!=g&&d.push(g)}),d):[...d,e]},[]);return new vZ(c,b,a)}
class pZ{constructor(a,b,c){this.rf=a;this.C=b;this.index=c}}
function sZ(a,b,c){null!=a||(a=new IY([],{}));if(0===a.segments.length&&0<a.C)return rZ(a,b,c);a:{var d=a;var e=0;let k=b;const m={match:!1,oo:0,ev:0};for(;k<d.segments.length;){if(e>=c.length){d=m;break a}const p=d.segments[k];var f=c[e];if(tZ(f))break;f=`${f}`;var g=e<c.length-1?c[e+1]:null;if(0<k&&void 0===f)break;if(f&&g&&"object"===typeof g&&void 0===g.Fg){if(f!=p.path||!uY(g,p.parameters)){d=m;break a}e+=2}else{g={};if(f!=p.path||!uY(g,p.parameters)){d=m;break a}e++}k++}d={match:!0,oo:k,ev:e}}e=
c.slice(d.ev);return d.match&&d.oo<a.segments.length?(b=new IY(a.segments.slice(0,d.oo),{}),b.children.primary=new IY(a.segments.slice(d.oo),a.children),rZ(b,0,e)):d.match&&0===e.length?new IY(a.segments,{}):!d.match||0<a.C?d.match?rZ(a,0,e):wZ(a,b,c):wZ(a,b,c)}
function rZ(a,b,c){if(0===c.length)return new IY(a.segments,{});const d=tZ(c[0])?c[0].Fg:{primary:c},e={};if(Object.keys(d).some(f=>"primary"!==f)&&a.children.primary&&1===a.C&&0===a.children.primary.segments.length)return c=rZ(a.children.primary,b,c),new IY(a.segments,c.children);Object.entries(d).forEach(([f,g])=>{"string"===typeof g&&(g=[g]);null!==g&&(e[f]=sZ(a.children[f],b,g))});Object.entries(a.children).forEach(([f,g])=>{void 0===d[f]&&(e[f]=g)});return new IY(a.segments,e)}
function wZ(a,b,c){const d=a.segments.slice(0,b);let e=0;for(;e<c.length;){var f=c[e];if(tZ(f))return a=xZ(f.Fg),new IY(d,a);if(0===e&&qZ(c[0])){d.push(new NY(a.segments[b].path,yZ(c[0])));e++;continue}f=tZ(f)?f.Fg.primary:`${f}`;const g=e<c.length-1?c[e+1]:null;f&&g&&qZ(g)?(d.push(new NY(f,yZ(g))),e+=2):(d.push(new NY(f,{})),e++)}return new IY(d,{})}function xZ(a){const b={};Object.entries(a).forEach(([c,d])=>{"string"===typeof d&&(d=[d]);null!==d&&(b[c]=wZ(new IY([],{}),0,d))});return b}
function yZ(a){const b={};Object.entries(a).forEach(([c,d])=>b[c]=`${d}`);return b};var zZ=class{constructor(a,b){this.id=a;this.url=b}},AZ=class extends zZ{constructor(a,b,c=null){super(a,b);this.type=0;this.Lj=c}toString(){return`NavigationStart(id: ${this.id}, url: '${this.url}')`}},BZ=class extends zZ{constructor(a,b,c){super(a,b);this.bc=c;this.type=1}toString(){return`NavigationEnd(id: ${this.id}, url: '${this.url}', urlAfterRedirects: '${this.bc}')`}},CZ=class extends zZ{constructor(a,b,c,d){super(a,b);this.reason=c;this.code=d;this.type=2}toString(){return`NavigationCancel(id: ${this.id}, url: '${this.url}')`}},
DZ=class extends zZ{constructor(a,b){super(a,b);this.reason="";this.code=0;this.type=16}},EZ=class extends zZ{constructor(a,b,c,d){super(a,b);this.error=c;this.target=d;this.type=3}toString(){return`NavigationError(id: ${this.id}, url: '${this.url}', error: ${this.error})`}},FZ=class extends zZ{constructor(a,b,c,d){super(a,b);this.bc=c;this.state=d;this.type=4}toString(){return`RoutesRecognized(id: ${this.id}, url: '${this.url}', urlAfterRedirects: '${this.bc}', state: ${this.state})`}},GZ=class extends zZ{constructor(a,
b,c,d){super(a,b);this.bc=c;this.state=d;this.type=7}toString(){return`GuardsCheckStart(id: ${this.id}, url: '${this.url}', urlAfterRedirects: '${this.bc}', state: ${this.state})`}},HZ=class extends zZ{constructor(a,b,c,d,e){super(a,b);this.bc=c;this.state=d;this.C=e;this.type=8}toString(){return`GuardsCheckEnd(id: ${this.id}, url: '${this.url}', urlAfterRedirects: '${this.bc}', state: ${this.state}, shouldActivate: ${this.C})`}},IZ=class extends zZ{constructor(a,b,c,d){super(a,b);this.bc=c;this.state=
d;this.type=5}toString(){return`ResolveStart(id: ${this.id}, url: '${this.url}', urlAfterRedirects: '${this.bc}', state: ${this.state})`}},JZ=class extends zZ{constructor(a,b,c,d){super(a,b);this.bc=c;this.state=d;this.type=6}toString(){return`ResolveEnd(id: ${this.id}, url: '${this.url}', urlAfterRedirects: '${this.bc}', state: ${this.state})`}},KZ=class{constructor(a){this.C=a;this.type=9}toString(){return`RouteConfigLoadStart(path: ${this.C.path})`}},LZ=class{constructor(a){this.C=a;this.type=
10}toString(){return`RouteConfigLoadEnd(path: ${this.C.path})`}},MZ=class{constructor(a){this.kb=a;this.type=11}toString(){return`ChildActivationStart(path: '${this.kb.jb&&this.kb.jb.path||""}')`}},NZ=class{constructor(a){this.kb=a;this.type=12}toString(){return`ChildActivationEnd(path: '${this.kb.jb&&this.kb.jb.path||""}')`}},OZ=class{constructor(a){this.kb=a;this.type=13}toString(){return`ActivationStart(path: '${this.kb.jb&&this.kb.jb.path||""}')`}},PZ=class{constructor(a){this.kb=a;this.type=
14}toString(){return`ActivationEnd(path: '${this.kb.jb&&this.kb.jb.path||""}')`}},QZ=class{},RZ=class{constructor(a,b){this.url=a;this.Fl=b}};var TZ=class{constructor(a){this.xa=a;this.C=this.mb=null;this.children=new SZ(this.xa);this.D=null}};function UZ(a,b){let c=a.getContext(b);c||(c=new TZ(a.Cj),a.C.set(b,c));return c}function VZ(a,b){if(a=a.getContext(b))a.mb=null,a.D=null}var SZ=class{constructor(a){this.Cj=a;this.C=new Map}getContext(a){return this.C.get(a)||null}};SZ.K=function(a){return new (a||SZ)(r(kg))};SZ.pa=Md({ma:SZ,ba:SZ.K,ea:"root"});function WZ(a,b){return XZ(b,a.vb).map(c=>c.value)}var ZZ=class{constructor(a){this.vb=a}get root(){return this.vb.value}parent(a){a=WZ(this,a);return 1<a.length?a[a.length-2]:null}children(a){return(a=YZ(a,this.vb))?a.children.map(b=>b.value):[]}firstChild(a){return(a=YZ(a,this.vb))&&0<a.children.length?a.children[0].value:null}};function YZ(a,b){if(a===b.value)return b;for(const c of b.children)if(b=YZ(a,c))return b;return null}
function XZ(a,b){if(a===b.value)return[b];for(const c of b.children){const d=XZ(a,c);if(d.length)return d.unshift(b),d}return[]}var $Z=class{constructor(a,b){this.value=a;this.children=b}toString(){return`TreeNode(${this.value})`}};function a_(a){const b={};a&&a.children.forEach(c=>b[c.value.mb]=c);return b};var c_=class extends ZZ{constructor(a,b){super(a);this.kb=b;b_(this,a)}toString(){return this.kb.toString()}},d_=class{constructor(a,b,c,d,e,f,g,k){this.N=a;this.G=b;this.J=c;this.F=d;this.D=e;this.mb=f;this.component=g;this.C=k;let m,p;this.title=null!=(p=null==(m=this.D)?void 0:C(m,xj(t=>t[sY])))?p:jj(void 0);this.url=a;this.params=b;this.tb=c;this.Yb=d;this.data=e}get jb(){return this.C.jb}get root(){return this.kg.root}get parent(){return this.kg.parent(this)}get firstChild(){return this.kg.firstChild(this)}get children(){return this.kg.children(this)}toString(){return this.kb?
this.kb.toString():`Future(${this.C})`}};
function e_(a,b,c="emptyOnly"){const d=a.jb;let e;if(null===b||"always"!==c&&""!==(null==d?void 0:d.path)&&(b.component||(null==(e=b.jb)?0:e.Al))){let f;a={params:Object.assign({},a.params),data:Object.assign({},a.data),resolve:Object.assign({},a.data,null!=(f=a.C)?f:{})}}else a={params:Object.assign({},b.params,a.params),data:Object.assign({},b.data,a.data),resolve:Object.assign({},a.data,b.data,null==d?void 0:d.data,a.C)};!d||"string"!==typeof d.title&&null!==d.title||(a.resolve[sY]=d.title);return a}
var f_=class{get title(){let a;return null==(a=this.data)?void 0:a[sY]}constructor(a,b,c,d,e,f,g,k,m){this.url=a;this.params=b;this.tb=c;this.Yb=d;this.data=e;this.mb=f;this.component=g;this.jb=k;this.D=m}get root(){return this.kg.root}get parent(){return this.kg.parent(this)}get firstChild(){return this.kg.firstChild(this)}get children(){return this.kg.children(this)}toString(){return`Route(url:'${this.url.map(a=>a.toString()).join("/")}', path:'${this.jb?this.jb.path:""}')`}},h_=class extends ZZ{constructor(a){super(a);
this.url="";b_(this,a)}toString(){return g_(this.vb)}};function b_(a,b){b.value.kg=a;b.children.forEach(c=>b_(a,c))}function g_(a){const b=0<a.children.length?` { ${a.children.map(g_).join(", ")} } `:"";return`${a.value}${b}`}
function i_(a){if(a.kb){const d=a.kb,e=a.C;a.kb=e;uY(d.tb,e.tb)||a.J.next(e.tb);d.Yb!==e.Yb&&a.F.next(e.Yb);uY(d.params,e.params)||a.G.next(e.params);a:{var b=d.url;var c=e.url;if(b.length!==c.length)b=!1;else{for(let f=0;f<b.length;++f)if(!uY(b[f],c[f])){b=!1;break a}b=!0}}b||a.N.next(e.url);uY(d.data,e.data)||a.D.next(e.data)}else a.kb=a.C,a.D.next(a.C.data)}function j_(a,b){const c=uY(a.params,b.params)&&OY(a.url,b.url),d=!a.parent!==!b.parent;return c&&!d&&(!a.parent||j_(a.parent,b.parent))};function k_(a,b){let c;return(null==(c=a.D.getContext(b))?void 0:c.mb)===a}function l_(a){var b=a.D,c=a.name;const d=UZ(b,c);d.mb=a;b.C.set(c,d);!a.C&&(b=a.D.getContext(a.name),null==b?0:b.C)&&(b.D?a.attach(b.D,b.C):m_(a,b.C,b.xa))}function m_(a,b,c){if(a.er)throw new Zd(4013,!1);a.F=b;const d=a.location,e=b.kb.component,f=UZ(a.D,a.name).children;a.C=qp(d,e,{index:d.length,xa:new n_(b,f,d.xa),Of:c});bn(a.J);let g;null==(g=a.G)||g.C(a);a.Au.emit(a.C.instance)}
var p_=class{constructor(){this.F=this.C=null;this.name="primary";this.Au=new Fk;this.xv=new Fk;this.Ju=new Fk;this.Bv=new Fk;this.D=lf(SZ);this.location=lf(kp);this.J=lf(Sr);this.G=lf(o_,{optional:!0})}tc(a){if(a.name){const {wn:b,jx:c}=a.name;b||(k_(this,c)&&(this.jn(),VZ(this.D,c)),l_(this))}}va(){k_(this,this.name)&&VZ(this.D,this.name);let a;null==(a=this.G)||a.D(this)}Fb(){l_(this)}get er(){return!!this.C}get component(){if(!this.C)throw new Zd(4012,!1);return this.C.instance}detach(){if(!this.C)throw new Zd(4012,
!1);this.location.detach();const a=this.C;this.F=this.C=null;this.Bv.emit(a.instance);return a}attach(a,b){this.C=a;this.F=b;this.location.ar(a.C);let c;null==(c=this.G)||c.C(this);this.Ju.emit(a.instance)}jn(){if(this.C){const a=this.component;this.C.destroy();this.F=this.C=null;this.xv.emit(a)}}};p_.K=function(a){return new (a||p_)};p_.Fa=Qf({type:p_,ha:[["router-outlet"]],inputs:{name:"name"},outputs:{Au:"activate",xv:"deactivate",Ju:"attach",Bv:"detach"},ib:["outlet"],ka:!0,features:[Dg]});
class n_{F(a){return new n_(this.C,this.D,a)}constructor(a,b,c){this.C=a;this.D=b;this.parent=c}get(a,b){return a===d_?this.C:a===SZ?this.D:this.parent.get(a,b)}}var o_=new df("");function q_(a,b,c){if(c&&b.value.jb===c.value.kb.jb){var d=c.value;d.C=b.value;b=r_(a,b,c);return new $Z(d,b)}d=s_(b.value);b=b.children.map(e=>q_(a,e));return new $Z(d,b)}function r_(a,b,c){return b.children.map(d=>{for(const e of c.children)if(d.value.jb===e.value.kb.jb)return q_(a,d,e);return q_(a,d)})}function s_(a){return new d_(new ej(a.url),new ej(a.params),new ej(a.tb),new ej(a.Yb),new ej(a.data),a.mb,a.component,a)};function t_(a){const {Ql:b,Fl:c}=jZ(a)?{Ql:a,Fl:void 0}:a;a=u_(0);a.url=b;a.Fl=c;return a}function u_(a){const b=Error("NavigationCancelingError: ");b.ngNavigationCancelingError=!0;b.aB=a;return b}function v_(a){return!!a&&a.ngNavigationCancelingError};var w_=class{};w_.K=function(a){return new (a||w_)};w_.Da=Gf({type:w_,ha:[["ng-component"]],ka:!0,features:[Mr],za:1,Aa:0,ta:function(a){a&1&&Mq(0,"router-outlet")},Ha:[p_],pb:2});function x_(a,b){a.Wa&&!a.Ua&&(a.Ua=rn(a.Wa,b,`Route: ${a.path}`));let c;return null!=(c=a.Ua)?c:b}function y_(a){const b=a.children&&a.children.map(y_);a=b?Object.assign({},a,{children:b}):Object.assign({},a);a.component||a.Al||!b&&!a.Xn||!a.mb||"primary"===a.mb||(a.component=w_);return a}function z_(a){return a.mb||"primary"}function A_(a,b){const c=a.filter(d=>z_(d)===b);c.push(...a.filter(d=>z_(d)!==b));return c}
function B_(a){if(!a)return null;var b;if(null==(b=a.jb)?0:b.Ua)return a.jb.Ua;for(a=a.parent;a;a=a.parent){b=a.jb;let c;if(null==(c=b)?0:c.wk)return b.wk;let d;if(null==(d=b)?0:d.Ua)return b.Ua}return null};var F_=(a,b,c)=>xj(d=>{var e=new C_(b,d.em,d.oq,c);const f=e.F.vb,g=e.C?e.C.vb:null;D_(e,f,g,a);i_(e.F.root);E_(e,f,g,a);return d});function D_(a,b,c,d){const e=a_(c);b.children.forEach(f=>{const g=f.value.mb;var k=e[g],m=f.value;const p=k?k.value:null;m===p?m.component?(m=d.getContext(m.mb))&&D_(a,f,k,m.children):D_(a,f,k,d):p&&G_(a,k,d);delete e[g]});Object.values(e).forEach(f=>{G_(a,f,d)})}
function E_(a,b,c,d){const e=a_(c);b.children.forEach(f=>{var g=e[f.value.mb],k=f.value,m=g?g.value:null;i_(k);k===m?k.component?(k=UZ(d,k.mb),E_(a,f,g,k.children)):E_(a,f,g,d):k.component?(g=UZ(d,k.mb),m=B_(k.kb),g.D=null,g.C=k,g.xa=null!=m?m:g.xa,g.mb&&m_(g.mb,k,g.xa),E_(a,f,null,g.children)):E_(a,f,null,d);a.D(new PZ(f.value.kb))});b.children.length&&a.D(new NZ(b.value.kb))}
function G_(a,b,c){const d=c.getContext(b.value.mb);c=d&&b.value.component?d.children:c;b=a_(b);for(const e of Object.values(b))G_(a,e,c);d&&(d.mb&&(d.mb.jn(),d.children.C=new Map),d.D=null,d.C=null)}var C_=class{constructor(a,b,c,d){this.hs=a;this.F=b;this.C=c;this.D=d}};const H_=Symbol("INITIAL_VALUE");function I_(){return Io(a=>C(Pj(a.map(b=>C(b,fo(),Ho(H_)))),xj(b=>{for(const c of b)if(!0!==c){if(c===H_)return H_;if(!1===c||jZ(c))return c}return!0}),tk(b=>b!==H_),fo()))};var J_=class{constructor(a){this.path=a;this.C=this.path[this.path.length-1]}},K_=class{constructor(a,b){this.component=a;this.C=b}};function L_(a,b){const c=Symbol(),d=b.get(a,c);return d===c?"function"!==typeof a||null!==Od(a)?b.get(a):a:d}function M_(a,b,c,d,e={canDeactivateChecks:[],canActivateChecks:[]}){const f=a_(b);a.children.forEach(g=>{N_(g,f[g.value.mb],c,d.concat([g.value]),e);delete f[g.value.mb]});Object.entries(f).forEach(([g,k])=>O_(k,c.getContext(g),e));return e}
function N_(a,b,c,d,e={canDeactivateChecks:[],canActivateChecks:[]}){const f=a.value,g=b?b.value:null,k=c?c.getContext(a.value.mb):null;if(g&&f.jb===g.jb){const m=P_(g,f,f.jb.tG);m?e.canActivateChecks.push(new J_(d)):(f.data=g.data,f.C=g.C);f.component?M_(a,b,k?k.children:null,d,e):M_(a,b,c,d,e);m&&k&&k.mb&&k.mb.er&&e.canDeactivateChecks.push(new K_(k.mb.component,g))}else g&&O_(b,k,e),e.canActivateChecks.push(new J_(d)),f.component?M_(a,null,k?k.children:null,d,e):M_(a,null,c,d,e)}
function P_(a,b,c){if("function"===typeof c)return c(a,b);switch(c){case "pathParamsChange":return!FY(a.url,b.url);case "pathParamsOrQueryParamsChange":return!FY(a.url,b.url)||!uY(a.tb,b.tb);case "always":return!0;case "paramsOrQueryParamsChange":return!j_(a,b)||!uY(a.tb,b.tb);default:return!j_(a,b)}}
function O_(a,b,c){const d=a_(a),e=a.value;Object.entries(d).forEach(([f,g])=>{e.component?b?O_(g,b.children.getContext(f),c):O_(g,null,c):O_(g,b,c)});e.component?b&&b.mb&&b.mb.er?c.canDeactivateChecks.push(new K_(b.mb.component,e)):c.canDeactivateChecks.push(new K_(null,e)):c.canDeactivateChecks.push(new K_(null,e))};function Q_(a){return"function"===typeof a}function R_(a){return a instanceof pj||"EmptyError"===(null==a?void 0:a.name)};function S_(a,b){return Tj(c=>{const d=c.qd,e=c.Dh;var f=c.pj;const g=f.canActivateChecks;f=f.canDeactivateChecks;return 0===f.length&&0===g.length?jj(Object.assign({},c,{Pf:!0})):C(T_(f,d,e,a),Tj(k=>k&&"boolean"===typeof k?U_(d,g,a,b):jj(k)),xj(k=>Object.assign({},c,{Pf:k})))})}function T_(a,b,c,d){return C(Sj(a),Tj(e=>V_(e.component,e.C,c,b,d)),oo(e=>!0!==e,!0))}
function U_(a,b,c,d){return C(Sj(b),Tj(e=>{var f=e.C.parent;null!==f&&d&&d(new MZ(f));f=jj(!0);var g=e.C;null!==g&&d&&d(new OZ(g));g=jj(!0);return hk(f,g,W_(a,e.path,c),X_(a,e.C,c))},1),oo(e=>!0!==e,!0))}function X_(a,b,c){var d=b.jb?b.jb.Pu:null;if(!d||0===d.length)return jj(!0);d=d.map(e=>ik(()=>{var f;const g=null!=(f=B_(b))?f:c,k=L_(e,g);f=k&&Q_(k.Pu)?k.Pu(b,a):yg(g,()=>k(b,a));return C(xY(f),oo())}));return C(jj(d),I_())}
function W_(a,b,c){const d=b[b.length-1];b=b.slice(0,b.length-1).reverse().map(e=>{const f=e.jb?e.jb.Qu:null;return f&&0!==f.length?{node:e,pj:f}:null}).filter(e=>null!==e).map(e=>ik(()=>{const f=e.pj.map(g=>{let k;const m=null!=(k=B_(e.node))?k:c,p=L_(g,m);g=p&&Q_(p.Qu)?p.Qu(d,a):yg(m,()=>p(d,a));return C(xY(g),oo())});return C(jj(f),I_())}));return C(jj(b),I_())}
function V_(a,b,c,d,e){var f=b&&b.jb?b.jb.Ru:null;if(!f||0===f.length)return jj(!0);f=f.map(g=>{let k;const m=null!=(k=B_(b))?k:e,p=L_(g,m);g=p&&Q_(p.Ru)?p.Ru(a,b,c,d):yg(m,()=>p(a,b,c,d));return C(xY(g),oo())});return C(jj(f),I_())}function Y_(a,b,c){var d=b.Su;if(void 0===d||0===d.length)return jj(!0);d=d.map(e=>{const f=L_(e,a);e=f&&Q_(f.Su)?f.Su(b,c):yg(a,()=>f(b,c));return xY(e)});return C(jj(d),I_(),Z_())}
function Z_(){return Ui([Po(a=>{if("boolean"!==typeof a)throw t_(a);}),xj(a=>!0===a)])}function $_(a,b,c){var d=b.Tu;if(!d||0===d.length)return jj(!0);d=d.map(e=>{const f=L_(e,a);e=f&&Q_(f.Tu)?f.Tu(b,c):yg(a,()=>f(b,c));return xY(e)});return C(jj(d),I_(),Z_())};var a0=class{constructor(a){this.rf=a||null}},b0=class extends Error{constructor(a){super();this.C=a}};function c0(a){return kj(new a0(a))}function d0(a){let b=[];for(a=a.root;;){b=b.concat(a.segments);if(0===a.C)return jj(b);if(1<a.C||!a.children.primary)return kj(new Zd(4E3,!1));a=a.children.primary}}function e0(a,b,c,d,e){b=f0(a,b,c.root,d,e);return new KY(b,g0(c.tb,a.D.tb),c.Yb)}
function h0(a,b,c,d,e,f){if("string"!==typeof c){const g=c,k=e.tb,m=e.Yb,p=e.jb,t=e.url,w=e.mb,B=e.params,G=e.data,J=e.title;c=yg(f,()=>g({params:B,data:G,tb:k,Yb:m,jb:p,url:t,mb:w,title:J}));if(c instanceof KY)throw new b0(c);}a=e0(a,c,a.C.parse(c),b,d);if("/"===c[0])throw new b0(a);return a}function f0(a,b,c,d,e){const f=i0(c.segments,d,e);let g={};Object.entries(c.children).forEach(([k,m])=>{g[k]=f0(a,b,m,d,e)});return new IY(f,g)}
function g0(a,b){const c={};Object.entries(a).forEach(([d,e])=>{c[d]="string"===typeof e&&":"===e[0]?b[e.substring(1)]:e});return c}function i0(a,b,c){return a.map(d=>{if(":"===d.path[0]){if(d=c[d.path.substring(1)],!d)throw new Zd(4001,!1);}else a:{let e=0;for(const f of b){if(f.path===d.path){b.splice(e);d=f;break a}e++}}return d})}var j0=class{constructor(a,b){this.C=a;this.D=b}};const k0={Nh:!1,Rk:[],Sl:[],parameters:{},so:{}};function l0(a,b,c,d){const e=m0(a,b,c);if(!e.Nh)return jj(e);d=x_(b,d);return C($_(d,b,c),xj(f=>!0===f?e:Object.assign({},k0)))}
function m0(a,b,c){if("**"===b.path)return{Nh:!0,parameters:0<c.length?(0<c.length?c[c.length-1]:null).parameters:{},Rk:c,Sl:[],so:{}};if(""===b.path)return"full"===b.Yw&&(0<a.C||0<c.length)?Object.assign({},k0):{Nh:!0,Rk:[],Sl:c,parameters:{},so:{}};a=(b.lr||tY)(c,a,b);if(!a)return Object.assign({},k0);const d={};var e;Object.entries(null!=(e=a.gx)?e:{}).forEach(([g,k])=>{d[g]=k.path});e=0<a.Qk.length?Object.assign({},d,a.Qk[a.Qk.length-1].parameters):d;let f;return{Nh:!0,Rk:a.Qk,Sl:c.slice(a.Qk.length),
parameters:e,so:null!=(f=a.gx)?f:{}}}function n0(a,b,c,d){if(0<c.length&&o0(a,c,d)){var e=IY;a=new IY(c,a.children);c={};c.primary=a;for(var f of d)""===f.path&&"primary"!==z_(f)&&(d=new IY([],{}),c[z_(f)]=d);c={rf:new e(b,c),xs:[]}}else if(0===c.length&&p0(a,c,d)){b=IY;f=a.segments;var g=a.children;const k={};for(e of d)q0(a,c,e)&&!g[z_(e)]&&(d=new IY([],{}),k[z_(e)]=d);d=Object.assign({},g,k);c={rf:new b(f,d),xs:c}}else c={rf:new IY(a.segments,a.children),xs:c};return c}
function o0(a,b,c){return c.some(d=>q0(a,b,d)&&"primary"!==z_(d))}function p0(a,b,c){return c.some(d=>q0(a,b,d))}function q0(a,b,c){return(0<a.C||0<b.length)&&"full"===c.Yw?!1:""===c.path};class r0{}function s0(a,b,c,d,e,f,g="emptyOnly"){return(new t0(a,b,c,d,e,g,f)).recognize()}function u0(a,b,c,d,e,f){return 0===d.segments.length&&0<d.C?v0(a,b,c,d,f):C(w0(a,b,c,d,d.segments,e,!0,f),xj(g=>g instanceof $Z?[g]:[]))}
function v0(a,b,c,d,e){const f=[];for(const g of Object.keys(d.children))"primary"===g?f.unshift(g):f.push(g);return C(Sj(f),Tj(g=>{const k=d.children[g],m=A_(c,g);return u0(a,b,m,k,g,e)},1),wo((g,k)=>{g.push(...k);return g}),Un(null),so(),Tj(g=>{if(null===g)return c0(d);g=x0(g);y0(g);return jj(g)}))}
function w0(a,b,c,d,e,f,g,k){return C(Sj(c),Tj(m=>{let p;return C(z0(a,null!=(p=m.Ua)?p:b,c,m,d,e,f,g,k),Mn(t=>{if(t instanceof a0)return jj(null);throw t;}))},1),oo(m=>!!m),Mn(m=>{if(R_(m))return 0!==e.length||d.children[f]?c0(d):jj(new r0);throw m;}))}function z0(a,b,c,d,e,f,g,k,m){return(z_(d)===g||"primary"!==g&&q0(e,f,d))&&m0(e,d,f).Nh?void 0===d.Ql?A0(a,b,e,d,f,g,m):a.F&&k?B0(a,b,e,c,d,f,g,m):c0(e):c0(e)}
function A0(a,b,c,d,e,f,g){const k=l0(c,d,e,b);"**"===d.path&&(c.children={});return C(k,Io(m=>{if(!m.Nh)return c0(c);let p;b=null!=(p=d.Ua)?p:b;return C(C0(a,b,d,e),Io(({Nj:t})=>{var w;const B=null!=(w=d.wk)?w:b;w=m.Rk;const G=m.Sl;var J;let N;const aa=new f_(w,m.parameters,Object.freeze(Object.assign({},a.C.tb)),a.C.Yb,d.data||{},z_(d),null!=(N=null!=(J=d.component)?J:d.vk)?N:null,d,d.resolve||{});J=e_(aa,g,a.Bj);aa.params=Object.freeze(J.params);aa.data=Object.freeze(J.data);const {rf:ca,xs:ia}=
n0(c,w,G,t);return 0===ia.length&&0<ca.C?C(v0(a,B,t,ca,aa),xj(O=>new $Z(aa,O))):0===t.length&&0===ia.length?jj(new $Z(aa,[])):C(w0(a,B,t,ca,ia,z_(d)===f?"primary":f,!0,aa),xj(O=>new $Z(aa,O instanceof $Z?[O]:[])))}))}))}
function B0(a,b,c,d,e,f,g,k){const {Nh:m,parameters:p,Rk:t,so:w,Sl:B}=m0(c,e,f);if(!m)return c0(c);"string"===typeof e.Ql&&"/"===e.Ql[0]&&(a.D++,31<a.D&&(a.F=!1));var G;let J;f=new f_(f,p,Object.freeze(Object.assign({},a.C.tb)),a.C.Yb,e.data||{},z_(e),null!=(J=null!=(G=e.component)?G:e.vk)?J:null,e,e.resolve||{});G=e_(f,k,a.Bj);f.params=Object.freeze(G.params);f.data=Object.freeze(G.data);e=h0(a.O,t,e.Ql,w,f,b);return C(d0(e),Tj(N=>w0(a,b,d,c,N.concat(B),g,!1,k)))}
function C0(a,b,c,d){return c.children?jj({Nj:c.children,xa:b}):c.Xn?void 0!==c.Dm?jj({Nj:c.Dm,xa:c.wk}):C(Y_(b,c,d),Tj(e=>e?C(a.G.Xn(b,c),Po(f=>{c.Dm=f.Nj;c.wk=f.xa})):kj(u_(3)))):jj({Nj:[],xa:b})}
var t0=class{constructor(a,b,c,d,e,f,g){this.xa=a;this.G=b;this.J=c;this.config=d;this.C=e;this.Bj=f;this.N=g;this.O=new j0(this.N,this.C);this.D=0;this.F=!0}recognize(){const a=n0(this.C.root,[],[],this.config).rf;return C(this.match(a),xj(({children:b,MD:c})=>{b=new h_(new $Z(c,b));c=kZ(c,this.C.tb,this.C.Yb);c.tb=this.C.tb;b.url=JY(c);return{state:b,qy:c}}))}match(a){const b=new f_([],Object.freeze({}),Object.freeze(Object.assign({},this.C.tb)),this.C.Yb,Object.freeze({}),"primary",this.J,null,
{});return C(u0(this,this.xa,this.config,a,"primary",b),xj(c=>({children:c,MD:b})),Mn(c=>{if(c instanceof b0)return this.C=c.C,this.match(c.C.root);if(c instanceof a0)throw new Zd(4002,`'${c.rf}'`);throw c;}))}};function y0(a){a.sort((b,c)=>"primary"===b.value.mb?-1:"primary"===c.value.mb?1:b.value.mb.localeCompare(c.value.mb))}function D0(a){return(a=a.value.jb)&&""===a.path}
function x0(a){const b=[],c=new Set;for(const d of a)D0(d)?(a=b.find(e=>d.value.jb===e.value.jb),void 0!==a?(a.children.push(...d.children),c.add(a)):b.push(d)):b.push(d);for(const d of c)a=x0(d.children),b.push(new $Z(d.value,a));return b.filter(d=>!c.has(d))};function E0(a,b,c,d,e,f){return Tj(g=>C(s0(a,b,c,d,g.gd,e,f),xj(({state:k,qy:m})=>Object.assign({},g,{qd:k,bc:m}))))};function F0(a,b){return Tj(c=>{const d=c.qd,e=c.pj.canActivateChecks;if(!e.length)return jj(c);const f=new Set(e.map(m=>m.C)),g=new Set;for(const m of f)if(!g.has(m))for(const p of G0(m))g.add(p);let k=0;return C(Sj(g),Tj(m=>{if(f.has(m))return H0(m,d,a,b);m.data=e_(m,m.parent,a).resolve;return jj(void 0)},1),Po(()=>k++),po(),Tj(()=>k===g.size?jj(c):fj))})}function G0(a){const b=a.children.map(c=>G0(c)).flat();return[a,...b]}
function H0(a,b,c,d){const e=a.jb,f=a.D;void 0!==(null==e?void 0:e.title)&&"string"!==typeof e.title&&null!==e.title&&(f[sY]=e.title);return C(I0(f,a,b,d),xj(g=>{a.C=g;a.data=e_(a,a.parent,c).resolve;return null}))}function I0(a,b,c,d){const e=vY(a);if(0===e.length)return jj({});const f={};return C(Sj(e),Tj(g=>C(J0(a[g],b,c,d),oo(),Po(k=>{f[g]=k}))),po(),to(f),Mn(g=>R_(g)?fj:kj(g)))}
function J0(a,b,c,d){let e;const f=null!=(e=B_(b))?e:d,g=L_(a,f);a=g.resolve?g.resolve(b,c):yg(f,()=>g(b,c));return xY(a)};function K0(a){return Io(b=>{const c=a(b);return c?C(Sj(c),xj(()=>b)):jj(b)})};function L0(a){let b;for(a=a.root;void 0!==a;){let c;b=null!=(c=a.data[sY])?c:b;a=a.children.find(d=>"primary"===d.mb)}return b}var M0=class{};M0.K=function(a){return new (a||M0)};M0.pa=Md({ma:M0,ba:()=>lf(N0),ea:"root"});var N0=class extends M0{constructor(a){super();this.title=a}};N0.K=function(a){return new (a||N0)(r(vH))};N0.pa=Md({ma:N0,ba:N0.K,ea:"root"});var O0=new df("",{ea:"root",ba:()=>({})});var P0=new df(""),S0=class{constructor(){this.D=new WeakMap;this.C=new WeakMap;lf(ps)}Al(a){if(this.D.get(a))return this.D.get(a);if(a.vk)return jj(a.vk);this.F&&this.F(a);var b=C(xY(a.Al()),xj(Q0),Po(c=>{this.G&&this.G(a);a.vk=c}),mo(()=>{this.D.delete(a)}));b=C(new uj(b,()=>new cj),sj());this.D.set(a,b);return b}Xn(a,b){if(this.C.get(b))return this.C.get(b);if(b.Dm)return jj({Nj:b.Dm,xa:b.wk});this.F&&this.F(b);a=C(R0(b,a,this.G),mo(()=>{this.C.delete(b)}));a=C(new uj(a,()=>new cj),sj());this.C.set(b,
a);return a}};S0.K=function(a){return new (a||S0)};S0.pa=Md({ma:S0,ba:S0.K,ea:"root"});function R0(a,b,c){return C(xY(a.Xn()),xj(Q0),Tj(d=>d instanceof $h||Array.isArray(d)?jj(d):Sj(Promise.resolve(new pn(d)))),xj(d=>{c&&c(a);let e;Array.isArray(d)||(e=d.create(b).xa,d=e.get(P0,[],{optional:!0,self:!0}).flat());return{Nj:d.map(y_),xa:e}}))}function Q0(a){return a&&"object"===typeof a&&"default"in a?a["default"]:a};var T0=class{};T0.K=function(a){return new (a||T0)};T0.pa=Md({ma:T0,ba:()=>lf(U0),ea:"root"});var U0=class{merge(a){return a}};U0.K=function(a){return new (a||U0)};U0.pa=Md({ma:U0,ba:U0.K,ea:"root"});var V0=new df("");new df("");var W0=new df("");function X0(a,b){const c=++a.Uh;let d;null==(d=a.Yg)||d.next(Object.assign({},a.Yg.value,b,{id:c}))}function Y0(a,b,c){c=new CZ(b.id,JY(b.gd),"",c);a.hc.next(c);b.resolve(!1)}function Z0(a){let b,c;return(null==(b=a.C)?void 0:b.gd.toString())!==(null==(c=a.C)?void 0:c.Sb.toString())}function $0(a){let b,c;return a.N.parse(a.location.path(!0)).toString()!==(null==(b=a.C)?void 0:b.gd.toString())&&!(null==(c=a.C)?0:c.extras.Sj)}
function a1(a,b,c,d){a.Yg=new ej({id:0,Sb:c,wv:c,gd:c,bc:c,vo:c,extras:{},resolve:()=>{},reject:()=>{},promise:Promise.resolve(!0),source:"imperative",Lj:null,Dh:d.kb,qd:null,oq:d,em:null,pj:{canActivateChecks:[],canDeactivateChecks:[]},Pf:null});return C(a.Yg,tk(e=>0!==e.id),xj(e=>Object.assign({},e,{gd:e.vo})),Io(e=>{let f=!1,g=!1;return C(jj(e),Io(k=>{if(a.Uh>e.id)return Y0(a,e,1),fj;a.C=e;a.D={id:k.id,Xq:k.vo,gd:k.gd,Le:k.source,extras:k.extras,BD:a.G?Object.assign({},a.G,{BD:null}):null};const m=
!b.co||Z0(a)||$0(a);let p;const t=null!=(p=k.extras.Dr)?p:b.Dr;return m||"reload"===t?C(jj(k),Io(w=>{let B;const G=null==(B=a.Yg)?void 0:dj(B);a.hc.next(new AZ(w.id,JY(w.gd),w.Lj));let J;return G!==(null==(J=a.Yg)?void 0:dj(J))?fj:Promise.resolve(w)}),E0(a.Of,a.F,a.J,b.config,a.N,a.Bj),Po(w=>{e.qd=w.qd;e.bc=w.bc;a.D=Object.assign({},a.D,{el:w.bc});w=new FZ(w.id,JY(w.gd),JY(w.bc),w.qd);a.hc.next(w)})):(a.hc.next(new DZ(k.id,JY(k.vo))),k.resolve(!1),fj)}),Po(k=>{k=new GZ(k.id,JY(k.gd),JY(k.bc),k.qd);
a.hc.next(k)}),xj(k=>{var m=Object,p=m.assign;var t=k.Dh;const w=k.qd.vb;t=M_(w,t?t.vb:null,a.O,[w.value]);return a.C=e=p.call(m,{},k,{pj:t})}),S_(a.Of,k=>a.hc.next(k)),Po(k=>{if((e.Pf=k.Pf)&&"boolean"!==typeof k.Pf)throw t_(k.Pf);k=new HZ(k.id,JY(k.gd),JY(k.bc),k.qd,!!k.Pf);a.hc.next(k)}),tk(k=>k.Pf?!0:(Y0(a,k,3),!1)),K0(k=>{if(k.pj.canActivateChecks.length)return C(jj(k),Po(m=>{m=new IZ(m.id,JY(m.gd),JY(m.bc),m.qd);a.hc.next(m)}),Io(m=>{let p=!1;return C(jj(m),F0(a.Bj,a.Of),Po({next:()=>p=!0,complete:()=>
{p||Y0(a,m,2)}}))}),Po(m=>{m=new JZ(m.id,JY(m.gd),JY(m.bc),m.qd);a.hc.next(m)}))}),K0(k=>{const m=p=>{const t=[];let w;(null==(w=p.jb)?0:w.Al)&&!p.jb.vk&&t.push(C(a.F.Al(p.jb),Po(B=>{p.component=B}),xj(()=>{})));for(const B of p.children)t.push(...m(B));return t};return C(Pj(m(k.qd.root)),Un(null),fo())}),K0(()=>a.T()),Io(()=>{const k=e.Dh,m=e.qd;let p;const t=null==(p=a.aa)?void 0:p.call(a,a.Of,k.root,m.root);return t?C(Sj(t),xj(()=>e)):jj(e)}),xj(k=>{var m=k.qd;var p=k.oq;p=q_(b.hs,m.vb,p?p.vb:
void 0);m=new c_(p,m);a.C=e=Object.assign({},k,{em:m});a.D.em=m;return e}),Po(()=>{a.hc.next(new QZ)}),F_(a.O,b.hs,k=>a.hc.next(k)),fo(),Po({next:k=>{f=!0;a.G=a.D;a.hc.next(new BZ(k.id,JY(k.gd),JY(k.bc)));let m;if(null!=(m=a.ja)){const p=L0(k.em.kb);void 0!==p&&(m.title.C.title=p||"")}k.resolve(!0)},complete:()=>{f=!0}}),Lo(C(a.P,Po(k=>{throw k;}))),mo(()=>{f||g||Y0(a,e,1);let k;(null==(k=a.C)?void 0:k.id)===e.id&&(a.D=null,a.C=null)}),Mn(k=>{g=!0;if(v_(k))a.hc.next(new CZ(e.id,JY(e.gd),k.message,
k.aB)),v_(k)&&jZ(k.url)?a.hc.next(new RZ(k.url,k.Fl)):e.resolve(!1);else{let m;const p=new EZ(e.id,JY(e.gd),k,null!=(m=e.qd)?m:void 0);try{yg(a.Of,()=>{let w;return null==(w=a.fa)?void 0:w.call(a,p)});a.hc.next(p);const t=b.tn(k);e.resolve(!!t)}catch(t){a.options.qG?e.resolve(!1):e.reject(t)}}return fj}))}))}
var b1=class{get ca(){return 0!==this.Uh}constructor(){this.G=this.C=this.D=null;this.hc=new cj;this.P=new cj;this.F=lf(S0);this.Of=lf(kg);this.N=lf(QY);this.O=lf(SZ);this.location=lf(Et);lf(o_,{optional:!0});this.ja=lf(M0);this.options=lf(O0,{optional:!0})||{};this.Bj=this.options.Bj||"emptyOnly";lf(T0);this.aa=lf(V0,{optional:!0});this.fa=lf(W0,{optional:!0});this.Uh=0;this.T=()=>jj(void 0);this.J=null;this.F.G=a=>this.hc.next(new LZ(a));this.F.F=a=>this.hc.next(new KZ(a))}complete(){let a;null==
(a=this.Yg)||a.complete()}};b1.K=function(a){return new (a||b1)};b1.pa=Md({ma:b1,ba:b1.K,ea:"root"});var c1=class{};c1.K=function(a){return new (a||c1)};c1.pa=Md({ma:c1,ba:()=>lf(d1),ea:"root"});var e1=class{store(){}},d1=class extends e1{};d1.K=(()=>{let a;return function(b){return(a||(a=Mh(d1)))(b||d1)}})();d1.pa=Md({ma:d1,ba:d1.K,ea:"root"});var f1=class{};f1.K=function(a){return new (a||f1)};f1.pa=Md({ma:f1,ba:()=>lf(g1),ea:"root"});function h1(a){return{Ke:a.Ke,Sb:a.Sb,Nd:a.Nd}}function i1(a,b){return a.location.subscribe(c=>{"popstate"===c.type&&b(c.url,c.state)})}function j1(a,b,c){b=JY(b);var d=a.location;d.path()==d.normalize(b+tt(""))||c.extras.Ul?(c=Object.assign({},c.extras.state,k1(a,c.id,a.D)),Bt(a.location,b,c)):(c=Object.assign({},c.extras.state,k1(a,c.id,a.D+1)),At(a.location,b,c))}
function l1(a,b,c=!1){"computed"===a.Mk?(c=a.C-a.D,0!==c?Ct(a.location,c):a.Sb===b.el&&0===c&&(m1(a,b),n1(a))):"replace"===a.Mk&&(c&&m1(a,b),n1(a))}function k1(a,b,c){return"computed"===a.Mk?{Uh:b,Sy:c}:{Uh:b}}function m1(a,b){a.Nd=a.G.Nd;a.Sb=a.G.Sb;let c;a.Ke=a.F.merge(a.Sb,null!=(c=b.el)?c:a.Ke)}function n1(a){Bt(a.location,JY(a.Ke),k1(a,a.J,a.C))}
var g1=class extends f1{constructor(){super(...arguments);this.location=lf(Et);lf(QY);this.options=lf(O0,{optional:!0})||{};this.Mk=this.options.Mk||"replace";this.F=lf(T0);this.Wj=this.options.Wj||"deferred";this.Ke=this.Sb=new KY;this.C=0;this.J=-1;const a=new h_(new $Z(new f_([],{},{},"",{},"primary",null,null,{}),[])),b=new d_(new ej([new NY("",{})]),new ej({}),new ej({}),new ej(""),new ej({}),"primary",null,a.root);b.kb=a.root;this.Nd=new c_(new $Z(b,[]),a);this.G=h1(this)}Lj(){return this.location.getState()}get D(){if("computed"!==
this.Mk)return this.C;let a,b;return null!=(b=null==(a=this.Lj())?void 0:a.Sy)?b:this.C}};g1.K=(()=>{let a;return function(b){return(a||(a=Mh(g1)))(b||g1)}})();g1.pa=Md({ma:g1,ba:g1.K,ea:"root"});function o1(a,b){C(a.hc,tk(c=>c instanceof BZ||c instanceof CZ||c instanceof EZ||c instanceof DZ),xj(c=>c instanceof BZ||c instanceof DZ?0:c instanceof CZ&&(0===c.code||1===c.code)?2:1),tk(c=>2!==c),fo()).subscribe(()=>{b()})};function p1(a){throw a;}var q1={paths:"exact",Yb:"ignored",Fw:"ignored",tb:"exact"},r1={paths:"subset",Yb:"ignored",Fw:"ignored",tb:"subset"};
function s1(a){const b=a.C.hc.subscribe(c=>{try{const e=a.C.C,f=a.C.D;if(null!==e&&null!==f){var d=a.D;if(c instanceof AZ)d.G=h1(d);else if(c instanceof DZ)d.Ke=f.Xq;else if(c instanceof FZ){if("eager"===d.Wj&&!f.extras.Sj){const g=d.F.merge(f.el,f.Xq);j1(d,g,f)}}else c instanceof QZ?(d.Sb=f.el,d.Ke=d.F.merge(f.el,f.Xq),d.Nd=f.em,"deferred"===d.Wj&&(f.extras.Sj||j1(d,d.Ke,f))):c instanceof CZ&&(3===c.code||2===c.code)?l1(d,f):c instanceof EZ?l1(d,f,!0):c instanceof BZ&&(d.J=c.id,d.C=d.D);if(c instanceof
CZ&&0!==c.code&&1!==c.code)a.co=!0;else if(c instanceof BZ)a.co=!0;else if(c instanceof RZ){const g=c.Fl,k=a.J.merge(c.url,e.wv),m=Object.assign({},{info:e.extras.info,Sj:e.extras.Sj,Ul:e.extras.Ul||"eager"===a.Wj||"imperative"!==e.source},g);t1(a,k,"imperative",null,m,{resolve:e.resolve,reject:e.reject,promise:e.promise})}}c instanceof QZ||c instanceof RZ||a.Bb.next(c)}catch(e){a.C.P.next(e)}});a.P.add(b)}
function t1(a,b,c,d,e,f){if(a.O)return Promise.resolve(!1);let g,k;f?(g=f.resolve,k=f.reject,f=f.promise):f=new Promise((p,t)=>{g=p;k=t});const m=a.G.add();o1(a,()=>{queueMicrotask(()=>a.G.remove(m))});X0(a.C,{source:c,Lj:d,Sb:a.Sb,wv:a.Sb,vo:b,extras:e,resolve:g,reject:k,promise:f,Dh:a.Nd.kb,oq:a.Nd});return f.catch(p=>Promise.reject(p))}function u1(a){null!=a.F||(a.F=i1(a.D,(b,c)=>{setTimeout(()=>{v1(a,b,"popstate",c)},0)}))}
function v1(a,b,c,d){const e={Ul:!0},f=(null==d?0:d.Uh)?d:null;d&&(d=Object.assign({},d),delete d.Uh,delete d.Sy,0!==Object.keys(d).length&&(e.state=d));b=w1(a,b);t1(a,b,c,f,e)}function w1(a,b){try{return a.N.parse(b)}catch(c){return a.N.parse("/")}}function x1(a){return Object.entries(a).reduce((b,[c,d])=>{null!==d&&void 0!==d&&(b[c]=d);return b},{})}
function y1(a,b,c={}){const d=c.oG,e=c.tb;var f=c.Yb;f=c.lG?a.Sb.Yb:f;switch(c.px){case "merge":c=Object.assign({},a.Sb.tb,e);break;case "preserve":c=a.Sb.tb;break;default:c=e||null}null!==c&&(c=x1(c));let g;try{g=lZ(d?d.kb:a.Nd.kb.root)}catch(k){if("string"!==typeof b[0]||"/"!==b[0][0])b=[];g=a.Sb.root}return mZ(g,b,c,null!=f?f:null)}function z1(a,b,c={Sj:!1}){b=jZ(b)?b:w1(a,b);b=a.J.merge(b,a.Ke);return t1(a,b,"imperative",null,c)}
var A1=class{get Sb(){return this.D.Sb}get Ke(){return this.D.Ke}get hc(){return this.Bb}get Nd(){return this.D.Nd}constructor(){this.O=!1;this.console=lf(wn);this.D=lf(f1);this.options=lf(O0,{optional:!0})||{};this.G=lf(xn);this.Wj=this.options.Wj||"deferred";this.C=lf(b1);this.N=lf(QY);this.location=lf(Et);this.J=lf(T0);this.Bb=new cj;this.tn=this.options.tn||p1;this.co=!1;this.hs=lf(c1);this.Dr=this.options.Dr||"ignore";let a,b;this.config=null!=(b=null==(a=lf(P0,{optional:!0}))?void 0:a.flat())?
b:[];lf(o_,{optional:!0});this.P=new Ni;this.config=this.config.map(y_);this.co=!1;a1(this.C,this,this.Sb,this.Nd).subscribe({error:c=>{this.console.warn(c)}});s1(this)}get url(){return JY(this.Sb)}va(){this.dispose()}dispose(){this.C.complete();this.F&&(this.F.unsubscribe(),this.F=void 0);this.O=!0;this.P.unsubscribe()}navigate(a,b={Sj:!1}){for(let c=0;c<a.length;c++)if(null==a[c])throw new Zd(4008,!1);return z1(this,y1(this,a,b),b)}isActive(a,b){let c;!0===b?c=Object.assign({},q1):!1===b?c=Object.assign({},
r1):c=b;if(jZ(a))return EY(this.Sb,a,c);a=w1(this,a);return EY(this.Sb,a,c)}};A1.K=function(a){return new (a||A1)};A1.pa=Md({ma:A1,ba:A1.K,ea:"root"});var B1=new df("");function C1(a){return a.Nd.root}new df("",{ea:"root",ba:()=>!1});function D1(){const a=lf(Wh);return b=>{var c=a.get(fp);if(b===c.Kf[0]){b=a.get(A1);var d=a.get(E1);1===a.get(F1)&&(u1(b),b.C.ca||v1(b,b.location.path(!0),"imperative",b.D.Lj()));var e;null==(e=a.get(G1,null,8))||e.vG();var f;null==(f=a.get(B1,null,8))||f.init();c=c.T[0];b.Nd.root.component=c;b.C.J=c;d.closed||(d.next(),d.complete(),d.unsubscribe())}}}const E1=new df("",{ba:()=>new cj}),F1=new df("",{ea:"root",ba:()=>1}),G1=new df("");var H1=new df("ROUTER_FORROOT_GUARD"),I1=class{};I1.K=function(a){return new (a||I1)(r(H1,8))};I1.Ya=Of({type:I1});I1.Xa=Nd({});new df("");function J1(a){const b={};a.zc?b.benchmark="1":(b.data=JSON.stringify({models:a.models,nodeData:a.nodeData,uiState:a.uiState}),b.renderer=a.C,b.show_open_in_new_tab=a.Co?"1":"0");const c=JSON.stringify(b);c!==a.D&&(a.D=c,a.F.navigate([],{tb:b,px:"",Ul:!1}))}function K1(a,b){a.models=b;J1(a)}function L1(a,b){a.uiState=b;J1(a)}
var M1=class{constructor(a){this.F=a;this.models=[];this.nodeData=[];this.D="";this.C="webgl";this.zc=this.Co=!1;a=new URLSearchParams(document.location.search);var b=a.get("data");if(b){let c=b=JSON.parse(b);null!=b.modelUrl&&(c={models:[{url:b.modelUrl}],uiState:{paneStates:[{deepestExpandedGroupNodeIds:b.deepestExpandedGroupNodeIds||[],selectedNodeId:b.selectedNodeId||"",selectedGraphId:b.selectedGraphId||"",selectedCollectionLabel:"",widthFraction:1}]}});this.models=c.models;this.uiState=c.uiState;
this.nodeData=c.nodeData}this.C=a.get("renderer")||"webgl";this.Co="1"===a.get("show_open_in_new_tab");this.zc="1"===a.get("benchmark")}};M1.K=function(a){return new (a||M1)(r(A1))};M1.pa=Md({ma:M1,ba:M1.K,ea:"root"});const N1=["modelPathInput"];
function O1(a){a&1&&(R(0,"div",19),Z(1," Select models from your comupter or enter the file paths directly (up to 10 in total)."),Mq(2,"br"),Z(3," Supported model formats: "),R(4,"a",32),Z(5,"TF"),S(),Z(6," (.pb, .pbtxt, .graphdef), "),R(7,"a",33),Z(8,"TFLite"),S(),Z(9," (.tflite), "),R(10,"a",34),Z(11,"TFJS"),S(),Z(12," (.json), "),R(13,"a",35),Z(14,"JAX"),S(),Z(15," (.pb), "),R(16,"a",36),Z(17,"PyTorch ExportedProgram"),S(),Z(18," (.pt2), "),R(19,"span",37),Z(20," MLIR "),S(),Z(21," (.mlir, .mlirbc). "),
S());if(a&2){a=Y();const b=zr(37);D(19);H("xapInlineDialog",b)("overlaySize",a.Th)("hoverDelayMs",50)}}
function P1(a){a&1&&(R(0,"div",19),Z(1," Enter model CNS/"),R(2,"span",38),Z(3,"TFHub"),S(),Z(4," paths/urls or select models from your computer (up to 10). Supported model format: "),R(5,"span",39),Z(6," TF "),S(),R(7,"span",40),Z(8," TFLite "),S(),R(9,"span",41),Z(10," TFJS "),S(),R(11,"span",42),Z(12," JAX "),S(),R(13,"span",43),Z(14," MLIR "),S(),R(15,"span",44),Z(16," JSON graphs "),S()());if(a&2){a=Y();const b=zr(27),c=zr(29),d=zr(31),e=zr(33),f=zr(35),g=zr(39);D(5);H("xapInlineDialog",b)("overlaySize",
a.Th)("hoverDelayMs",50);D(2);H("xapInlineDialog",c)("overlaySize",a.Th)("hoverDelayMs",50);D(2);H("xapInlineDialog",d)("overlaySize",a.Th)("hoverDelayMs",50);D(2);H("xapInlineDialog",e)("overlaySize",a.Th)("hoverDelayMs",50);D(2);H("xapInlineDialog",f)("overlaySize",a.Th)("hoverDelayMs",50);D(2);H("xapInlineDialog",g)("overlaySize",a.Th)("hoverDelayMs",50)}}
function Q1(a,b){if(a&1){const c=Ug();R(0,"mat-option",23)(1,"div",45);Z(2);R(3,"div",46)(4,"div",47);V("click",function(d){const e=u(c).Cb,f=Y();d.stopPropagation();f.tg.setValue(f.wj[e].path);f.Hw.R.focus();return x()});R(5,"mat-icon");Z(6,"edit");S()();R(7,"div",47);V("click",function(d){const e=u(c).Cb,f=Y();d.stopPropagation();f.wj.splice(e,1);R1(f);PI("model_explorer_model_paths",JSON.stringify(f.wj));return x()});R(8,"mat-icon");Z(9,"delete");S()()()()()}a&2&&(a=b.da,H("value",a),D(2),Br(" ",
a," "))}function S1(a){a&1&&(R(0,"div",29),Z(1," You have selected maximum number of models "),S())}function T1(a){a&1&&Mq(0,"mat-spinner",57)}function U1(a){a&1&&(R(0,"div",63),Z(1),S());a&2&&(a=Y(2).da,D(),Br(" ",a.errorMessage," "))}function V1(a){a&1&&(R(0,"div",62)(1,"mat-icon"),Z(2,"info"),S()(),Ap(3,U1,2,1,"ng-template",null,13,Qr));if(a&2){a=zr(4);const b=Y(3);H("xapInlineDialog",a)("overlaySize",b.LB)("hoverDelayMs",50)}}
function W1(a){if(a&1){const b=Ug();R(0,"div",64,14);V("click",function(){u(b);const c=zr(1),d=Y().da,e=Y(2);return x(X1(e,d,c))});R(2,"div",65);Z(3);S();R(4,"mat-icon");Z(5,"arrow_drop_down");S()()}if(a&2){a=Y().da;const b=Y(2);nq("disabled",b.kd());D(3);Br(" ",Y1(a)," ")}}function Z1(a){a&1&&(R(0,"div",60),Z(1," No supported adapter "),S())}
function $1(a,b){if(a&1){const c=Ug();R(0,"tr")(1,"td")(2,"div",53)(3,"mat-checkbox",54,12);V("change",function(d){const e=u(c).da;Y(2);e.selected=d.checked;return x()});S();R(5,"div",55);V("click",function(){const d=u(c).da,e=zr(4);Y(2);d.selected=!e.checked;return x()});Z(6);S()()();R(7,"td")(8,"div",56);Ap(9,T1,1,0,"mat-spinner",57);R(10,"div",58);Z(11);S();Ap(12,V1,5,3);S()();R(13,"td");Ap(14,W1,6,3,"div",59)(15,Z1,2,0,"div",60);S();R(16,"td")(17,"button",61);V("click",function(){const d=u(c).index,
e=Y(2);e.od.splice(d,1);bn(e.ek);return x()});R(18,"mat-icon");Z(19,"delete");S()()()()}a&2&&(a=b.da,b=Y(2),D(2),nq("disabled",!(0<(a.Gf||[]).length)),D(),H("checked",a.selected)("disabled",!(0<(a.Gf||[]).length)||b.kd()),D(2),nq("disabled",!(0<(a.Gf||[]).length)||b.kd()),D(),Br(" ",a.path," "),D(3),Eq("Converting"===a.status()||"Uploading"===a.status()?9:-1),D(),nq("done",a.status()===b.jz.DONE)("error",b.ze(a)),D(),Br(" ",a2(a)," "),D(),Eq(b.ze(a)?12:-1),D(2),Eq(0<(a.Gf||[]).length?14:15),D(3),
H("disabled",b.kd()))}function b2(a){a&1&&(R(0,"div",30)(1,"table")(2,"thead")(3,"tr")(4,"th",48),Z(5,"Model name"),S(),R(6,"th"),Z(7,"Status"),S(),R(8,"th")(9,"div",49),Z(10," Adapter "),R(11,"div",50)(12,"mat-icon",51),Z(13,"help_outline"),S()()()(),R(14,"th"),Z(15,"Delete"),S()()(),R(16,"tbody"),Ap(17,$1,20,16,"tr",52),S()()());if(a&2){a=Y();const b=zr(43);D(11);H("xapInlineDialog",b)("overlayPositions",a.TA)("hoverDelayMs",100);D(6);H("ngForOf",a.od)("ngForTrackBy",a.AE)}}
function c2(a){if(a&1){const b=Ug();R(0,"div",31)(1,"div",66)(2,"button",67);V("click",function(){u(b);const c=Y();for(const d of c.od)d.selected=!1;return x()});R(3,"mat-icon");Z(4,"close");S()();Z(5);S();R(6,"button",68);V("click",function(){u(b);const c=Y();return x(d2(c))});Z(7," View selected models ");S()()}a&2&&(a=Y(),D(),nq("hide",0===a.ps),D(),H("disabled",a.kd()),D(3),Br(" ",a.ps," selected "),D(),H("disabled",0===a.ps||a.kd()))}
function e2(a){a&1&&(R(0,"div",69)(1,"div"),Z(2," Enter the "),R(3,"b"),Z(4,"CNS path"),S(),Z(5,", "),R(6,"b"),Z(7,"Placer path"),S(),Z(8,", "),R(9,"b")(10,"a",70),Z(11,"Piper path"),S()(),Z(12,", "),R(13,"b"),Z(14,"TFHub path"),S(),Z(15,", or "),R(16,"b"),Z(17,"CNS Viewer url"),S(),Z(18," to a TF model "),R(19,"b"),Z(20,"directory"),S(),Z(21,", or upload the "),R(22,"b"),Z(23,".pb"),S(),Z(24," file from your computer. "),Mq(25,"br")(26,"br"),R(27,"span",71),Z(28,"When using CNS/Placer/TFHub, the model file inside is required to be named "),
R(29,"b"),Z(30,"saved_model.pb"),S(),Z(31,"."),S(),Mq(32,"br")(33,"br"),S(),R(34,"div",72),Z(35," Examples: "),R(36,"ul")(37,"li"),Z(38,"/cns/ok-d/home/odml-tooling-team/mgv/models/coco-ssd-tf/"),S(),R(39,"li"),Z(40,"/tfhub/prod/tfhub.dev/tensorflow/tutorials/spam-detection/1/"),S(),R(41,"li"),Z(42,"google3/path/to/tf/model/on/piper/"),S()()()())}
function f2(a){a&1&&(R(0,"div",69)(1,"div"),Z(2," Enter the "),R(3,"b"),Z(4,"CNS path"),S(),Z(5,", "),R(6,"b"),Z(7,"Placer path"),S(),Z(8,", "),R(9,"b")(10,"a",70),Z(11,"Piper path"),S()(),Z(12,", "),R(13,"b"),Z(14,"TFHub path"),S(),Z(15,", or "),R(16,"b"),Z(17,"CNS Viewer url"),S(),Z(18," to a TFLite model "),R(19,"b"),Z(20,"file"),S(),Z(21,", or upload the "),R(22,"b"),Z(23,".tflite"),S(),Z(24," file from your computer. "),S(),R(25,"div",72),Z(26," Examples: "),R(27,"ul")(28,"li"),Z(29,"/cns/ok-d/home/odml-tooling-team/mgv/models/coco-ssd.tflite"),
S(),R(30,"li"),Z(31,"/tfhub/prod/tfhub.dev/tensorflow/lite-model/tutorials/spam-detection/tflite/1/1.tflite"),S(),R(32,"li"),Z(33,"google3/path/to/tflite/model/on/piper.tflite"),S()()()())}function g2(a){a&1&&(R(0,"div",69)(1,"div"),Z(2," Enter the "),R(3,"b"),Z(4,"url"),S(),Z(5," for the model.json file, or upload it from your computer. "),S(),R(6,"div",72),Z(7," Example: https://tfhub.dev/tensorflow/tfjs-model/ssd_mobilenet_v2/1/default/1/model.json?tfjs-format=file "),S()())}
function h2(a){a&1&&(R(0,"div",69)(1,"div"),Z(2," JAX models must be convereted to TF, TFLite, or TFJS prior to loading in Model Explorer. Check out the "),R(3,"a",73),Z(4,"JAX to TF"),S(),Z(5," or "),R(6,"a",74),Z(7,"JAX to TFLite/TFJS"),S(),Z(8," documentation for conversion instructions. "),S()())}
function i2(a){a&1&&(R(0,"div",69)(1,"div"),Z(2," Enter the "),R(3,"b"),Z(4,"CNS path"),S(),Z(5,", "),R(6,"b"),Z(7,"Placer path"),S(),Z(8,", "),R(9,"b")(10,"a",70),Z(11,"Piper path"),S()(),Z(12,", "),R(13,"b"),Z(14,"TFHub path"),S(),Z(15,", or "),R(16,"b"),Z(17,"CNS Viewer url"),S(),Z(18," to a MLIR model "),R(19,"b"),Z(20,"file"),S(),Z(21,", or upload the "),R(22,"b"),Z(23,".mlir"),S(),Z(24," or "),R(25,"b"),Z(26,".mlirbc"),S(),Z(27," file from your computer. Only support TF, TFL and StableHLO dialect. "),
S(),R(28,"div",72),Z(29," Examples: "),R(30,"ul")(31,"li"),Z(32,"/cns/sandbox/home/yijieyang/mlir/tfl_ssd_debug_info.mlir"),S()()()())}function j2(a){a&1&&(R(0,"div",69)(1,"div"),Z(2," Support TF, TFL and StableHLO dialect. "),S()())}
function k2(a){a&1&&(R(0,"div",69)(1,"div"),Z(2," Enter the "),R(3,"b"),Z(4,"url"),S(),Z(5," for the JSON graphs file (downloadable from "),R(6,"a",75),Z(7,"here"),S(),Z(8,", or convert your graphs into "),R(9,"a",76),Z(10,"this format"),S(),Z(11,"), or upload it from your computer. Using JSON graphs file has a much faster processing time. "),S(),R(12,"div",72),Z(13," Example: https://storage.googleapis.com/tfweb/model-explorer-models/mobilebert_fp32.tflite.json "),S()())}
function l2(a){a&1&&(R(0,"div",77),Z(1," Max upload size: 256 MB. "),Mq(2,"br")(3,"br"),Z(4," Model too large? Try "),R(5,"a",78),Z(6,"converting"),S(),Z(7," it to JSON graphs and upload the JSON file instead. "),S())}
function m2(a){a&1&&(R(0,"div",79)(1,"span",80),Z(2,"Adapters"),S(),Z(3,' transform model files into an intermediate format that Model Explorer can understand and visualize. For certain model types, multiple adapters may exist, offering different transformation approaches. The "default" adapter should cover the majority of common scenarios. '),Mq(4,"br")(5,"br"),Z(6," You can also create your own "),R(7,"span",80),Z(8,"adapter extensions"),S(),Z(9," to add support for more model formats. See "),R(10,
"a",81),Z(11,"this guide"),S(),Z(12," for instructions. "),S())}function X1(a,b,c){let d;const e=null==(d=b.Sc)?void 0:d.id,f=n2(a,c);c=f.attach(a.Ld);c.instance.Sc=b.Sc;c.instance.cq=b.Gf||[];c.instance.he.subscribe(g=>{f.dispose();(null==g?void 0:g.id)!==e&&null!=b.errorMessage&&(b.status.set("Not started"),b.errorMessage=void 0,b.selected=!0);b.Sc=g;bn(a.ek)})}function Y1(a){if(null==a.Sc)return"?";let b;return 0===(a.Gf||[]).indexOf(a.Sc)?"Default":(null==(b=a.Sc)?void 0:b.name)||"?"}
function a2(a){a=a.status();return"Uploading"===a?"Processing":a}function d2(a){a.kd.set(!0);const b=a.od.filter(c=>c.selected);o2(a.J,b).then(()=>{a.kd.set(!1)});L1(a.F);K1(a.F,b.filter(c=>"remote"===c.type||"graphs_json_from_server"===c.type||"file_path"===c.type).map(c=>{let d;return{url:c.path,adapterId:null==(d=c.Sc)?void 0:d.id}}));a.rj.set(b.some(c=>"local"===c.type))}
function R1(a){const b=(a.tg.value||"").toLowerCase();a.Qv=a.wj.filter(c=>c.path.toLowerCase().includes(b)).map(c=>c.path);bn(a.ek)}function p2(){return JSON.parse(window.localStorage.getItem("model_explorer_model_paths")||"[]")}
function q2(a,b){a.od=b.map(c=>{if(c.url.startsWith("graphs://")){var d=pY(a.C,"builtin_json"),e=null==d?[]:[d];return{path:c.url,type:"graphs_json_from_server",status:xr("Not started"),selected:0<e.length,Gf:e,Sc:d}}d=kY(c.url,a.C);(e=pY(a.C,c.converterId||c.adapterId||""))||(e=0<d.length?d[0]:void 0);return{path:c.url,type:"file_path",status:xr("Not started"),selected:0<d.length,Gf:d,Sc:e}});cn(a.ek);a.od.some(c=>c.selected)&&d2(a)}
function r2(a,b){b=b.filter(c=>null==a.od.find(d=>{let e,f,g,k;return d.path===c.path&&(null==(e=d.file)?void 0:e.size)===(null==(f=c.file)?void 0:f.size)&&(null==(g=d.file)?void 0:g.lastModified)===(null==(k=c.file)?void 0:k.lastModified)}));a.od.push(...b);10<a.od.length&&a.od.splice(10);bn(a.ek)}
function s2(a,b){const c=p2();for(const d of b)b=c.findIndex(e=>e.path===d),0<=b?(b=c.splice(b,1))&&0<b.length&&(b[0].ts=Date.now(),c.unshift(b[0])):c.unshift({path:d,ts:Date.now()});50<c.length&&c.splice(50);PI("model_explorer_model_paths",JSON.stringify(c));a.wj=c}
function t2(a){return sa(function*(){yield new Promise(c=>{setTimeout(c)});var b=a.tg.value;null!=b&&(b=b.trim().split(",").filter(c=>""!==c.trim()).map(c=>{const d=kY(c,a.C);return{path:c,type:a.ee?"remote":"file_path",status:xr("Not started"),selected:0<d.length,Gf:d,Sc:0<d.length?d[0]:void 0}}),r2(a,b),s2(a,b.map(c=>c.path)),a.tg.setValue(""),setTimeout(()=>{let c;null==(c=a.MC)||iB(c);a.Hw.R.blur()}))})}
function u2(a,b){const c=[];for(let d=0;d<b.length;d++){const e=b[d],f=kY(e.name,a.C);c.push({path:e.name,type:"local",status:xr("Not started"),selected:0<f.length,file:e,Gf:f,Sc:0<f.length?f[0]:void 0})}r2(a,c)}
function n2(a,b){b=new Au({uc:Ku(Rv(a.D.position(),b),[{cb:"start",eb:"bottom",Na:"start",Ta:"top"}]),Tb:!0,dc:"cdk-overlay-transparent-backdrop",vc:a.D.C.C(),maxWidth:"380px",hb:"graph-selector-panel"});const c=a.D.create(b);a.Ld=new Uv(jY,a.xb);c.dd().subscribe(()=>{c.dispose()});return c}
var v2=class{constructor(a,b,c,d,e,f,g,k){this.ek=a;this.C=b;this.G=c;this.J=e;this.D=f;this.F=g;this.xb=k;this.tg=new vx("");this.od=[];this.wj=[];this.Qv=[];this.jz=fY;this.Th={maxWidth:400,minHeight:0};this.LB={minHeight:0};this.TA=[{cb:"start",eb:"top",Na:"start",Ta:"bottom",offsetY:-4}];this.kd=xr(!1);this.rj=xr(!1);this.Ld=null;C(this.tg.ah,sL(this.G)).subscribe(()=>{R1(this)});this.wj=p2();R1(this)}Kq(a){const b=a.files;if(b){var c=[];for(let d=0;d<b.length;d++){const e=b.item(d);e&&c.push(e)}u2(this,
c);a.value=""}}AE(a,b){let c,d;return`${b.path}_${null==(c=b.file)?void 0:c.size}_${null==(d=b.file)?void 0:d.lastModified}`}ze(a){return"Error"===a.status()}get sq(){if(this.Rq)return!0;if(this.ee){var a=(this.tg.value||"").toLowerCase().trim();return""!==a&&!bY.test(a)&&!a.startsWith("http")||""===a}a=(this.tg.value||"").trim();return""===a||""!==a&&!a.startsWith("/")&&!a.startsWith("~")}get Ad(){return!0}get ee(){return!1}get ps(){return this.od.filter(a=>a.selected).length}get Rq(){return 10===
this.od.length}get QC(){return this.Ad?"Absolute file paths (recommended for large models)":"Model CNS/TFHub paths or urls"}};v2.K=function(a){return new (a||v2)(z(Sr),z(rY),z(yi),z(QI),z("ModelLoaderService"),z(bw),z(M1),z(kp))};
v2.Da=Gf({type:v2,ha:[["model-source-input"]],fb:function(a,b){a&1&&(ur(N1,5),ur(dB,5));if(a&2){let c;vr(c=wr())&&(b.Hw=c.first);vr(c=wr())&&(b.MC=c.first)}},ka:!0,features:[Mr],za:44,Aa:17,Ca:[["auto","matAutocomplete"],["modelPathInput",""],["uploadInput",""],["tfHelp",""],["tfliteHelp",""],["tfjsHelp",""],["jaxHelp",""],["mlirHelp",""],["externalMlirHelp",""],["graphsJsonHelp",""],["uploadHelp",""],["adapterHelp",""],["checkbox",""],["errorInfo",""],["selector",""],[1,"container"],[1,"content"],
[1,"title"],[1,"description-container"],[1,"description"],[1,"source-input-container"],[1,"input-container"],[1,"model-path-autocomplete",3,"optionSelected"],[3,"value"],[3,"keydown.enter","placeholder","formControl","matAutocomplete"],["mat-raised-button","",1,"btn-add-model-path",3,"click","disabled"],[1,"or-label"],["mat-flat-button","","dialogLabel","uploadHelp",1,"btn-upload",3,"click","disabled","xapInlineDialog","xapInlineDialogDisabled","hoverDelayMs"],["type","file","multiple","",1,"upload-input",
3,"change"],[1,"model-count-warning"],[1,"models-table"],[1,"table-action-bar"],["href","https://www.tensorflow.org/","target","_blank"],["href","https://www.tensorflow.org/lite","target","_blank"],["href","https://www.tensorflow.org/js","target","_blank"],["href","https://www.tensorflow.org/guide/jax2tf","target","_blank"],["href","https://github.com/google-ai-edge/model-explorer/wiki/2.-User-Guide#how-to-select-pytorch-models","target","_blank"],["dialogLabel","externalMlirHelp",1,"help-trigger",
"external",3,"xapInlineDialog","overlaySize","hoverDelayMs"],["matTooltip","In the form of /tfhub/...",1,"tfhub"],["dialogLabel","tfHelp",1,"help-trigger",3,"xapInlineDialog","overlaySize","hoverDelayMs"],["dialogLabel","tfliteHelp",1,"help-trigger",3,"xapInlineDialog","overlaySize","hoverDelayMs"],["dialogLabel","tfjsHelp",1,"help-trigger",3,"xapInlineDialog","overlaySize","hoverDelayMs"],["dialogLabel","jaxHelp",1,"help-trigger",3,"xapInlineDialog","overlaySize","hoverDelayMs"],["dialogLabel","mlirHelp",
1,"help-trigger",3,"xapInlineDialog","overlaySize","hoverDelayMs"],["dialogLabel","graphsJsonHelp",1,"help-trigger",3,"xapInlineDialog","overlaySize","hoverDelayMs"],[1,"option-container"],[1,"icons-container"],[1,"mat-icon-container",3,"click"],[1,"model-name-col"],[1,"adapter-header"],["dialogLabel","adapterHelp",1,"icon-container",3,"xapInlineDialog","overlayPositions","hoverDelayMs"],["fontSet","google-material-icons"],[4,"ngFor","ngForOf","ngForTrackBy"],[1,"model-name-container"],[3,"change",
"checked","disabled"],[1,"model-path",3,"click"],[1,"model-status-container"],["color","primary","diameter","16"],[1,"model-status-label-container"],[1,"selected-adapter-container",3,"disabled"],[1,"no-supported-adapter-msg"],["mat-icon-button","",1,"btn-delete-model-item",3,"click","disabled"],["dialogLabel","errorInfo",1,"error-info-container",3,"xapInlineDialog","overlaySize","hoverDelayMs"],[1,"model-explorer-error-info-popup"],[1,"selected-adapter-container",3,"click"],[1,"selected-adapter-label"],
[1,"selection-status"],["mat-icon-button","",1,"btn-deselect-all",3,"click","disabled"],["mat-flat-button","","color","primary",1,"btn-view-models",3,"click","disabled"],[1,"model-explorer-model-format-help-popup"],["href","http://go/google-src","target","_blank"],[1,"info"],[1,"example"],["href","https://g3doc.corp.google.com/third_party/py/jax/experimental/jax2tf/README.md","target","_blank"],["href","https://g3doc.corp.google.com/third_party/tensorflow/google/g3doc/jax_production/g3doc/guides/mobile.md",
"target","_blank"],["href","https://screenshot.googleplex.com/8u9evbbgNDiNmJX","target","_blank"],["href","https://source.corp.google.com/piper///depot/google3/learning/brain/mobile/lite/tooling/model_graph_visualizer/module/common/input_graph.ts?q=symbol:%5CbGraphCollection%5Cb","target","_blank"],[1,"model-explorer-upload-help-popup"],["href","https://source.corp.google.com/piper///depot/google3/third_party/tensorflow/compiler/mlir/lite/experimental/google/tooling/","target","_blank"],[1,"model-explorer-adapter-help-popup"],
[1,"bold"],["href","https://github.com/google-ai-edge/model-explorer/wiki/6.-Develop-Adapter-Extension","target","_blank"]],ta:function(a,b){if(a&1){const c=Ug();R(0,"div",15)(1,"div",16)(2,"div",17);Z(3," Select Models ");S();R(4,"div",18);Ap(5,O1,22,3,"div",19)(6,P1,17,18,"div",19);S();R(7,"div",20)(8,"div",21)(9,"mat-autocomplete",22,0);V("optionSelected",function(d){u(c);b.sq||(Uz(d.Pw),t2(b));return x()});Jq(11,Q1,10,2,"mat-option",23,Hq);S();R(13,"input",24,1);V("keydown.enter",function(){u(c);
return x(!b.sq&&t2(b))});S();R(15,"button",25);V("click",function(){u(c);return x(t2(b))});Z(16," Add ");S()();R(17,"div",26);Z(18,"or");S();R(19,"button",27);V("click",function(){u(c);const d=zr(22);return x(d.click())});Z(20," Select from your computer ");S();R(21,"input",28,2);V("change",function(){u(c);const d=zr(22);return x(b.Kq(d))});S()();Ap(23,S1,2,0,"div",29)(24,b2,18,5,"div",30);S();Ap(25,c2,8,5,"div",31);S();Ap(26,e2,43,0,"ng-template",null,3,Qr)(28,f2,34,0,"ng-template",null,4,Qr)(30,
g2,8,0,"ng-template",null,5,Qr)(32,h2,9,0,"ng-template",null,6,Qr)(34,i2,33,0,"ng-template",null,7,Qr)(36,j2,3,0,"ng-template",null,8,Qr)(38,k2,14,0,"ng-template",null,9,Qr)(40,l2,8,0,"ng-template",null,10,Qr)(42,m2,13,0,"ng-template",null,11,Qr)}if(a&2){a=zr(10);const c=zr(41);D();nq("no-models",0===b.od.length);D(4);Eq(b.Ad?5:-1);D();Eq(b.ee?6:-1);D();nq("reverse",b.Ad);D(4);Lq(b.Qv);D(2);H("placeholder",b.QC)("formControl",b.tg)("matAutocomplete",a);D(2);H("disabled",b.sq||b.kd());D(4);H("disabled",
b.Rq||b.kd())("xapInlineDialog",c)("xapInlineDialogDisabled",b.Ad)("hoverDelayMs",50);D(4);Eq(b.Rq?23:-1);D();Eq(0<b.od.length?24:-1);D();Eq(0<b.od.length?25:-1)}},Ha:[ht,Zs,pB,mw,mA,Vz,dB,BB,AB,vB,yB,OB,DB,IB,AC,wD,vD,sD,RD,hG,AE,fG,iI,hI,Cx,sw,Mw,xx,HG,EG],styles:[".container[_ngcontent-%COMP%]{width:100%;height:100%;-moz-box-sizing:border-box;box-sizing:border-box;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:start;-webkit-align-items:flex-start;-moz-box-align:start;-ms-flex-align:start;align-items:flex-start;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;border:1px solid #dadce0;border-radius:8px;overflow:hidden}.container[_ngcontent-%COMP%]   .content[_ngcontent-%COMP%]{width:100%;-moz-box-sizing:border-box;box-sizing:border-box;padding:24px;padding-bottom:0;overflow:hidden;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column}.container[_ngcontent-%COMP%]   .content.no-models[_ngcontent-%COMP%]{padding-bottom:24px}.container[_ngcontent-%COMP%]   .title[_ngcontent-%COMP%]{color:#1f1f1f;font-size:24px}.container[_ngcontent-%COMP%]   .description-container[_ngcontent-%COMP%]{color:#444746;font-size:14px;margin-top:8px}.container[_ngcontent-%COMP%]   .description-container[_ngcontent-%COMP%]   .description[_ngcontent-%COMP%]{line-height:20px}.container[_ngcontent-%COMP%]   .description-container[_ngcontent-%COMP%]   .description[_ngcontent-%COMP%]   .tfhub[_ngcontent-%COMP%]{display:inline-block;text-decoration:underline;-webkit-text-decoration-color:#aaa;-moz-text-decoration-color:#aaa;text-decoration-color:#aaa;cursor:help}.container[_ngcontent-%COMP%]   .description-container[_ngcontent-%COMP%]   .description[_ngcontent-%COMP%]   .help-trigger[_ngcontent-%COMP%]{display:inline-block;margin-right:6px;text-decoration:underline;-webkit-text-decoration-color:#aaa;-moz-text-decoration-color:#aaa;text-decoration-color:#aaa;cursor:help}.container[_ngcontent-%COMP%]   .description-container[_ngcontent-%COMP%]   .description[_ngcontent-%COMP%]   .help-trigger.external[_ngcontent-%COMP%]{margin-right:0}.container[_ngcontent-%COMP%]   .description-container[_ngcontent-%COMP%]   .description[_ngcontent-%COMP%]   a[_ngcontent-%COMP%]{color:inherit}.container[_ngcontent-%COMP%]   .source-input-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;height:56px;-moz-box-sizing:border-box;box-sizing:border-box;width:100%;margin-top:8px;padding:8px 0;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-flex-shrink:0;-ms-flex-negative:0;flex-shrink:0}.container[_ngcontent-%COMP%]   .source-input-container.reverse[_ngcontent-%COMP%]{-webkit-box-orient:horizontal;-webkit-box-direction:reverse;-webkit-flex-direction:row-reverse;-moz-box-orient:horizontal;-moz-box-direction:reverse;-ms-flex-direction:row-reverse;flex-direction:row-reverse}.container[_ngcontent-%COMP%]   .source-input-container[_ngcontent-%COMP%]   .input-container[_ngcontent-%COMP%]{height:100%;-moz-box-sizing:border-box;box-sizing:border-box;background-color:#f0f4f9;border-radius:99px;-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;padding-left:18px}.container[_ngcontent-%COMP%]   .source-input-container[_ngcontent-%COMP%]   .input-container[_ngcontent-%COMP%]   input[_ngcontent-%COMP%]{height:100%;-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1;border:none;outline:none;background-color:transparent;color:#444746;font-size:14px;margin-right:16px}.container[_ngcontent-%COMP%]   .source-input-container[_ngcontent-%COMP%]   .input-container[_ngcontent-%COMP%]   .btn-add-model-path[_ngcontent-%COMP%]{background-color:#fff;border-radius:99px;color:#0b57d0;height:40px}.container[_ngcontent-%COMP%]   .source-input-container[_ngcontent-%COMP%]   .input-container[_ngcontent-%COMP%]   .btn-add-model-path[disabled][_ngcontent-%COMP%]{background-color:rgba(31,31,31,.12);color:rgba(31,31,31,.38)}.container[_ngcontent-%COMP%]   .source-input-container[_ngcontent-%COMP%]   .or-label[_ngcontent-%COMP%]{font-size:14px;margin:0 20px;color:#5f6368}.container[_ngcontent-%COMP%]   .source-input-container[_ngcontent-%COMP%]   .btn-upload[_ngcontent-%COMP%]{height:100%;background-color:#c2e7ff;border-radius:99px;color:#001d35}.container[_ngcontent-%COMP%]   .source-input-container[_ngcontent-%COMP%]   .btn-upload[disabled][_ngcontent-%COMP%]{background-color:rgba(31,31,31,.12);color:rgba(31,31,31,.38)}.container[_ngcontent-%COMP%]   .source-input-container[_ngcontent-%COMP%]   .upload-input[_ngcontent-%COMP%]{display:none}.container[_ngcontent-%COMP%]   .model-count-warning[_ngcontent-%COMP%]{color:#ab6c17;background-color:#ffefd9;margin:4px 0;width:-webkit-fit-content;width:-moz-fit-content;width:fit-content;padding:2px 9px;border-radius:4px;font-size:13px;font-weight:500}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]{width:100%;color:#1f1f1f;font-size:14px;margin-top:18px;overflow:hidden auto}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]{width:100%;border-spacing:0;border-collapse:collapse;padding:0}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   thead[_ngcontent-%COMP%]{position:-webkit-sticky;position:sticky;top:0;z-index:100;box-shadow:inset 0 -1px 0 #c4c7c5;background-color:#fff}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   thead[_ngcontent-%COMP%]   .model-name-col[_ngcontent-%COMP%]{width:100%}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   thead[_ngcontent-%COMP%]   .adapter-header[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;gap:4px}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   thead[_ngcontent-%COMP%]   .adapter-header[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;cursor:pointer;opacity:.8}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   thead[_ngcontent-%COMP%]   .adapter-header[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]:hover{opacity:1}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   thead[_ngcontent-%COMP%]   .adapter-header[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{font-size:20px;width:20px;height:20px;color:#777}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   th[_ngcontent-%COMP%]{height:49px;text-align:left;padding:0;font-weight:500}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   tr[_ngcontent-%COMP%]{height:49px;text-align:left;padding:0}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   tr[_ngcontent-%COMP%]:not(:last-child){border-bottom:1px solid #c4c7c5}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   td[_ngcontent-%COMP%]{padding:0}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .model-name-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;padding-left:12px;margin-right:12px;word-break:break-all}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .model-name-container.disabled[_ngcontent-%COMP%]{pointer-events:none}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .model-name-container[_ngcontent-%COMP%]   mat-checkbox[_ngcontent-%COMP%]{margin-right:2px}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .model-name-container[_ngcontent-%COMP%]   .model-path[_ngcontent-%COMP%]{cursor:pointer;line-height:16px;padding:8px 0}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .model-name-container[_ngcontent-%COMP%]   .model-path.disabled[_ngcontent-%COMP%]{pointer-events:none}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .selected-adapter-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between;cursor:pointer;height:48px;margin-right:24px;white-space:nowrap}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .selected-adapter-container.disabled[_ngcontent-%COMP%]{opacity:.5;pointer-events:none}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .selected-adapter-container[_ngcontent-%COMP%]:hover   mat-icon[_ngcontent-%COMP%]{opacity:1}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .selected-adapter-container[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{opacity:.7}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .no-supported-adapter-msg[_ngcontent-%COMP%]{color:#ab6c17;background-color:#ffefd9;padding:2px 8px;width:-webkit-fit-content;width:-moz-fit-content;width:fit-content;border-radius:4px;font-size:12px;margin-right:24px;white-space:nowrap;font-weight:500}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .model-status-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;margin-right:32px}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .model-status-container[_ngcontent-%COMP%]   mat-spinner[_ngcontent-%COMP%]{margin-right:4px}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .model-status-container[_ngcontent-%COMP%]   .model-status-label-container[_ngcontent-%COMP%]{font-size:12px;padding:2px 8px;background-color:#f1f3f4;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;color:#3c4043;width:-webkit-fit-content;width:-moz-fit-content;width:fit-content;border-radius:4px;white-space:nowrap;font-weight:500}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .model-status-container[_ngcontent-%COMP%]   .model-status-label-container.done[_ngcontent-%COMP%]{color:#0d652d;background-color:#e6f4ea}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .model-status-container[_ngcontent-%COMP%]   .model-status-label-container.error[_ngcontent-%COMP%]{color:#a50e0e;background-color:#fce8e6}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .model-status-container[_ngcontent-%COMP%]   .error-info-container[_ngcontent-%COMP%]{width:20px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;margin-left:4px;cursor:pointer}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .model-status-container[_ngcontent-%COMP%]   .error-info-container[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{width:20px;height:20px;font-size:20px}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .btn-delete-model-item[_ngcontent-%COMP%]{width:36px;height:36px;padding:0;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .btn-delete-model-item[disabled][_ngcontent-%COMP%]{opacity:.5}.container[_ngcontent-%COMP%]   .models-table[_ngcontent-%COMP%]   table[_ngcontent-%COMP%]   .btn-delete-model-item[_ngcontent-%COMP%]     mat-icon{font-size:20px;width:20px;height:20px;color:#444746}.container[_ngcontent-%COMP%]   .table-action-bar[_ngcontent-%COMP%]{-webkit-flex-shrink:0;-ms-flex-negative:0;flex-shrink:0;height:60px;width:100%;-moz-box-sizing:border-box;box-sizing:border-box;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between;background-color:#f0f4f9;padding:0 24px 0 38px}.container[_ngcontent-%COMP%]   .table-action-bar[_ngcontent-%COMP%]   .selection-status[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;font-size:14px;color:#444746}.container[_ngcontent-%COMP%]   .table-action-bar[_ngcontent-%COMP%]   .selection-status.hide[_ngcontent-%COMP%]{visibility:hidden}.container[_ngcontent-%COMP%]   .table-action-bar[_ngcontent-%COMP%]   .btn-deselect-all[_ngcontent-%COMP%]{width:36px;height:36px;padding:0;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center}.container[_ngcontent-%COMP%]   .table-action-bar[_ngcontent-%COMP%]   .btn-deselect-all[_ngcontent-%COMP%]     mat-icon{font-size:20px;width:20px;height:20px;color:#444746}.container[_ngcontent-%COMP%]   .table-action-bar[_ngcontent-%COMP%]   .btn-view-models[_ngcontent-%COMP%]{height:40px;border-radius:99px}.container[_ngcontent-%COMP%]   .instructions-container[_ngcontent-%COMP%]{font-size:13px;color:#555;margin-top:24px;display:none}.container[_ngcontent-%COMP%]   .instructions-container[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{font-size:18px;width:18px;height:18px;margin-right:4px}.container[_ngcontent-%COMP%]   .instructions-container[_ngcontent-%COMP%]   .instruction-title[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}.container[_ngcontent-%COMP%]   .instructions-container[_ngcontent-%COMP%]   .instruction-title[_ngcontent-%COMP%]   .model-format[_ngcontent-%COMP%]{color:#333;margin-right:4px;margin-left:4px;border:1px solid #ccc;border-radius:4px;padding:2px 4px;cursor:pointer}.container[_ngcontent-%COMP%]   .instructions-container[_ngcontent-%COMP%]   .instruction-title[_ngcontent-%COMP%]   .model-format[_ngcontent-%COMP%]:hover{background-color:#eee}  .model-explorer-upload-help-popup{padding:12px;font-weight:400;color:#555;line-height:14px;font-size:12px;font-family:Google Sans Text,Arial,Helvetica,sans-serif}  .model-explorer-adapter-help-popup{padding:12px;font-weight:400;color:#555;line-height:15px;font-size:12px;font-family:Google Sans Text,Arial,Helvetica,sans-serif}  .model-explorer-adapter-help-popup .bold{font-weight:700}  .model-explorer-model-format-help-popup{padding:12px;font-weight:400;color:#555;line-height:15px;font-size:12px;font-family:Google Sans Text,Arial,Helvetica,sans-serif;letter-spacing:normal}  .model-explorer-model-format-help-popup a{color:inherit}  .model-explorer-model-format-help-popup .example{margin-top:8px;color:#aaa}  .model-explorer-model-format-help-popup .example ul{margin:4px 0;-webkit-padding-start:28px;-moz-padding-start:28px;padding-inline-start:28px}  .model-explorer-model-format-help-popup .example li{margin-top:4px}  .model-explorer-model-format-help-popup .info{display:inline-block;background-color:#ffd1a8}  .model-explorer-error-info-popup{padding:12px;font-weight:400;background-color:#f0f4f9;line-height:14px;font-size:12px;font-family:Google Sans Text,Arial,Helvetica,sans-serif}  .model-path-autocomplete mat-option{min-height:13px;font-size:13px;line-height:13px;padding-top:5px;padding-bottom:5px}  .model-path-autocomplete mat-option:hover .option-container .mat-icon-container{visibility:visible}  .model-path-autocomplete mat-option>span{width:100%}  .model-path-autocomplete mat-option .option-container{width:100%;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between;word-break:break-all}  .model-path-autocomplete mat-option .option-container .icons-container{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;gap:4px;min-width:50px;-webkit-box-pack:end;-webkit-justify-content:flex-end;-moz-box-pack:end;-ms-flex-pack:end;justify-content:flex-end}  .model-path-autocomplete mat-option .option-container .mat-icon-container{visibility:hidden;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;opacity:.6;width:18px}  .model-path-autocomplete mat-option .option-container .mat-icon-container:hover{opacity:1}  .model-path-autocomplete mat-option .option-container .mat-icon-container mat-icon{font-size:18px;width:18px;height:18px}"],
Ra:0});const w2=(a,b)=>b.name;function x2(a){if(a&1){const b=Ug();R(0,"div",11);V("click",function(){u(b);const c=Y().da;return x(c.expanded=!c.expanded)});R(1,"mat-icon");Z(2,"unfold_more");S()()}}function y2(a,b){a&1&&(R(0,"div",3)(1,"div",6),Z(2),S(),R(3,"div",7),Z(4),S(),R(5,"div",8)(6,"div",9),Z(7),S(),Ap(8,x2,3,0,"div",10),S()());a&2&&(a=b.da,D(2),Ar(a.name),D(2),Ar(a.copyright),D(),nq("expanded",a.expanded),D(2),Ar(a.Vn),D(),Eq(a.expanded?-1:8))}
var z2=class{constructor(){this.IC=[{name:"Angular",copyright:"Copyright (c) 2010-2024 Google LLC.",Vn:'The MIT License\n\nCopyright (c) 2010-2024 Google LLC. https://angular.io/license\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the "Software"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in\nall copies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN\nTHE SOFTWARE.'},
{name:"d3 (v5.7.0)",copyright:"Copyright 2010-2017 Mike Bostock",Vn:'Copyright 2010-2017 Mike Bostock\nAll rights reserved.\n\nRedistribution and use in source and binary forms, with or without modification,\nare permitted provided that the following conditions are met:\n\n* Redistributions of source code must retain the above copyright notice, this\n  list of conditions and the following disclaimer.\n\n* Redistributions in binary form must reproduce the above copyright notice,\n  this list of conditions and the following disclaimer in the documentation\n  and/or other materials provided with the distribution.\n\n* Neither the name of the author nor the names of contributors may be used to\n  endorse or promote products derived from this software without specific prior\n  written permission.\n\nTHIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND\nANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED\nWARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE\nDISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR\nANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES\n(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;\nLOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON\nANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT\n(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS\nSOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.'},
{name:"dagre (v0.8.5)",copyright:"Copyright (c) 2012-2014 Chris Pettitt",Vn:'Copyright (c) 2012-2014 Chris Pettitt\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the "Software"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in\nall copies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN\nTHE SOFTWARE.'},
{name:"three.js (r134)",copyright:"Copyright \u00a9 2010-2021 three.js authors",Vn:'The MIT License\n\nCopyright \u00a9 2010-2021 three.js authors\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the "Software"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in\nall copies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN\nTHE SOFTWARE.'}]}};
z2.K=function(a){return new (a||z2)};
z2.Da=Gf({type:z2,ha:[["open-source-libs-dialog"]],ka:!0,features:[Mr],za:9,Aa:0,Ca:[["mat-dialog-title",""],[1,"title"],["mat-dialog-content","",1,"dialog-content"],[1,"lib-container"],["mat-dialog-actions","","align","end"],["mat-button","","mat-dialog-close",""],[1,"name"],[1,"copyright"],[1,"license-container"],[1,"license-content"],[1,"icon-container"],[1,"icon-container",3,"click"]],ta:function(a,b){a&1&&(R(0,"div",0)(1,"div",1),Z(2,"Open source libraries"),S()(),R(3,"div",2),Jq(4,y2,9,6,"div",
3,w2),S(),R(6,"div",4)(7,"button",5),Z(8,"Close"),S()());a&2&&(D(4),Lq(b.IC))},Ha:[ht,BB,AB,vB,zC,sC,uC,wC,vC,iI,hI],styles:["[_ngcontent-%COMP%]:not(mat-icon){font-family:Google Sans Text,Arial,Helvetica,sans-serif!important;letter-spacing:normal!important}.title[_ngcontent-%COMP%]{font-size:24px;font-weight:500;margin-top:-16px}.dialog-content[_ngcontent-%COMP%]{padding-bottom:0}.lib-container[_ngcontent-%COMP%]{margin:18px 0}.lib-container[_ngcontent-%COMP%]   .name[_ngcontent-%COMP%]{font-size:16px;font-weight:700;color:#333}.lib-container[_ngcontent-%COMP%]   .copyright[_ngcontent-%COMP%]{font-size:12px;color:#777}.lib-container[_ngcontent-%COMP%]   .license-container[_ngcontent-%COMP%]{width:520px;font-size:11px;border:1px solid #ccc;margin-top:8px;background-color:#f9f9f9;height:100px;overflow:hidden;position:relative}.lib-container[_ngcontent-%COMP%]   .license-container.expanded[_ngcontent-%COMP%]{height:-webkit-fit-content;height:-moz-fit-content;height:fit-content}.lib-container[_ngcontent-%COMP%]   .license-container[_ngcontent-%COMP%]   .license-content[_ngcontent-%COMP%]{white-space:pre-wrap;line-height:14px;padding:8px;overflow:hidden;height:100%;-moz-box-sizing:border-box;box-sizing:border-box}.lib-container[_ngcontent-%COMP%]   .license-container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]{position:absolute;right:8px;top:8px;cursor:pointer;opacity:.8}.lib-container[_ngcontent-%COMP%]   .license-container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]:hover{opacity:1}"]});var A2={Vy:0,Uo:1,vz:2,0:"BOOLEAN",1:"NUMBER",2:"TEXT_MULTILINE"},B2={label:"Maximum element count for constant tensor values",key:"const_element_count_limit",type:1,defaultValue:16,help:"Controls the number of values extracted from the constant tensors during model processing. Increasing this number may impact performance due to larger payload sizes."},C2={label:"Hide op nodes with labels below (comma separated)",key:"hide_op_nodes_with_labels",type:2,defaultValue:"Const,pseudo_const,pseudo_qconst,ReadVariableOp",
help:"Removes op nodes from model graphs if their label matches any of the labels entered below."},D2={label:"Maximum number of nodes in an artificial layer",key:"artificial_layer_node_count_threshold",type:1,defaultValue:1E3,help:"Controls the maximum number of immediate child nodes displayed under a layer. When the number of child nodes exceeds this limit, Model Explorer automatically groups them into smaller, more manageable artificial layers to improve layout performance and readability."},E2=
{label:"Edge label font size",key:"edge_label_font_size",type:1,defaultValue:7.5},F2={label:"Disallow vertical edge labels",key:"disallow_vertical_edge_labels",type:0,defaultValue:!1,help:"-"},G2=[B2,C2,D2,E2,{label:"Show welcome card",key:"show_welcome_card",type:0,defaultValue:!0},F2];function H2(a,b){return null==a.C[b.key]?!0===b.defaultValue:!0===a.C[b.key]}function I2(a,b){a=a.C[b.key];return null!=a?Number(a):b.defaultValue||0}
function J2(a,b,c){a.C[c]=b;window.localStorage.setItem("model_explorer_settings",JSON.stringify(a.C))}function K2(a,b,c){isNaN(b)||(a.C[c]=b,window.localStorage.setItem("model_explorer_settings",JSON.stringify(a.C)))}function L2(a,b,c){a.C[c]=b;window.localStorage.setItem("model_explorer_settings",JSON.stringify(a.C))}function M2(){return G2.find(a=>"show_welcome_card"===a.key)}
var N2=class{constructor(){const a=window.localStorage.getItem("model_explorer_settings")||"";this.C=""===a?{}:JSON.parse(a)}getStringValue(a){const b=this.C[a.key];return null!=b?b:a.defaultValue||""}};N2.K=function(a){return new (a||N2)};N2.pa=Md({ma:N2,ba:N2.K,ea:"root"});function O2(a){a&1&&(Z(0," When "),R(1,"a",17),Z(2,"View tensor shape on edge"),S(),Z(3," is turned on, Model Explorer will try to position the shape along the edge curve. When the edge is close to vertical, the shape may be hard to read. Enabling this option will force the shapes to be rendered horizontally in such situations to improve readbility. "))}function P2(a){a&1&&Z(0);a&2&&(a=Y(3).da,Br(" ",a.help," "))}
function Q2(a){a&1&&(R(0,"div",16),Ap(1,O2,4,0)(2,P2,1,1),S());a&2&&(a=Y(2).da,D(),Eq("disallow_vertical_edge_labels"===a.key?1:2))}function R2(a){a&1&&(R(0,"div",14)(1,"mat-icon",15),Z(2,"help_outline"),S()(),Ap(3,Q2,3,1,"ng-template",null,0,Qr));if(a&2){a=zr(4);const b=Y(2);H("xapInlineDialog",a)("overlaySize",b.hd)("overlayPositions",b.eC)("hoverDelayMs",100)}}
function S2(a){if(a&1){const b=Ug();Nq(0);R(1,"mat-slide-toggle",18);V("change",function(c){u(b);const d=Y().da,e=Y();return x(J2(e.Pc,c.checked,d.key))});S();Oq()}if(a&2){a=Y().da;const b=Y();D();H("checked",H2(b.Pc,a))}}
function T2(a){if(a&1){const b=Ug();Nq(0);R(1,"input",19,1);V("focus",function(){u(b);const c=zr(2);return x(c.select())})("change",function(){u(b);const c=zr(2),d=Y().da,e=Y();return x(K2(e.Pc,c.valueAsNumber,d.key))});S();R(3,"div",20);V("click",function(){u(b);const c=Y().da,d=Y();K2(d.Pc,c.defaultValue,c.key);return x()});Z(4,"Reset to default");S();Oq()}if(a&2){a=Y().da;const b=Y();D();H("value",I2(b.Pc,a))}}
function U2(a){if(a&1){const b=Ug();Nq(0);R(1,"textarea",19,1);V("focus",function(){u(b);const c=zr(2);return x(c.select())})("change",function(){u(b);const c=zr(2),d=Y().da,e=Y();return x(L2(e.Pc,c.value,d.key))});S();R(3,"div",20);V("click",function(){u(b);const c=Y().da,d=Y();L2(d.Pc,c.defaultValue,c.key);return x()});Z(4,"Reset to default");S();Oq()}if(a&2){a=Y().da;const b=Y();D();H("value",b.Pc.getStringValue(a))}}
function V2(a,b){a&1&&(R(0,"div",10),Nq(1,11),R(2,"div",12),Z(3),Ap(4,R2,5,4),S(),Ap(5,S2,2,1,"ng-container",13)(6,T2,5,1,"ng-container",13)(7,U2,5,1,"ng-container",13),Oq(),S());a&2&&(a=b.da,b=Y(),D(),H("ngSwitch",a.type),D(2),Br(" ",a.label," "),D(),Eq(a.help?4:-1),D(),H("ngSwitchCase",b.Wo.Vy),D(),H("ngSwitchCase",b.Wo.Uo),D(),H("ngSwitchCase",b.Wo.vz))}
var W2=class{constructor(a){this.Pc=a;this.Wo=A2;this.hd={minWidth:0,minHeight:0,maxWidth:340};this.eC=[{cb:"end",eb:"top",Na:"start",Ta:"top",offsetX:4}];this.VA=G2}};W2.K=function(a){return new (a||W2)(z(N2))};
W2.Da=Gf({type:W2,ha:[["settings-dialog"]],ka:!0,features:[Mr],za:11,Aa:1,Ca:[["helpPopup",""],["input",""],["mat-dialog-title",""],[1,"title"],[1,"local-storage-info"],["mat-dialog-content","",1,"dialog-content"],[1,"settings-container"],["class","setting-item",4,"ngFor","ngForOf"],["mat-dialog-actions","","align","end"],["mat-button","","mat-dialog-close",""],[1,"setting-item"],[3,"ngSwitch"],[1,"setting-label"],[4,"ngSwitchCase"],["dialogLabel","help popup",1,"icon-container",3,"xapInlineDialog",
"overlaySize","overlayPositions","hoverDelayMs"],["fontSet","google-material-icons"],[1,"model-explorer-settings-help-popup"],["href","https://github.com/google-ai-edge/model-explorer/wiki/2.-User-Guide#overlay-data-on-nodesedges","target","_blank"],[3,"change","checked"],["type","number",3,"focus","change","value"],[1,"reset",3,"click"]],ta:function(a,b){a&1&&(R(0,"div",2)(1,"div",3),Z(2,"Advanced settings"),S(),R(3,"div",4),Z(4," Settings are stored in current browser's local storage "),S()(),R(5,
"div",5)(6,"div",6),Ap(7,V2,8,6,"div",7),S()(),R(8,"div",8)(9,"button",9),Z(10,"Close"),S()());a&2&&(D(7),H("ngForOf",b.VA))},Ha:[ht,Zs,et,ft,BB,AB,vB,zC,sC,uC,wC,vC,bE,TD,XD,iI,hI,HG,EG],styles:["[_ngcontent-%COMP%]:not(mat-icon){font-family:Google Sans Text,Arial,Helvetica,sans-serif}.title[_ngcontent-%COMP%]{font-size:24px;font-weight:500;margin-top:-16px}.local-storage-info[_ngcontent-%COMP%]{font-size:12px;color:#777;margin-top:6px}.dialog-content[_ngcontent-%COMP%]{padding-bottom:0}.settings-container[_ngcontent-%COMP%]{font-size:13px;color:#000;margin-top:20px;margin-bottom:10px}.setting-item[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;-webkit-box-align:start;-webkit-align-items:flex-start;-moz-box-align:start;-ms-flex-align:start;align-items:flex-start;margin-bottom:24px}.setting-item[_ngcontent-%COMP%]   .setting-label[_ngcontent-%COMP%]{line-height:16px;margin-bottom:4px;width:100%;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between}.setting-item[_ngcontent-%COMP%]   .setting-label[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]{width:16px;height:16px;margin-left:12px;opacity:.4;cursor:pointer}.setting-item[_ngcontent-%COMP%]   .setting-label[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]:hover{opacity:.6}.setting-item[_ngcontent-%COMP%]   .setting-label[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{font-size:16px;width:16px;height:16px}.setting-item[_ngcontent-%COMP%]   input[_ngcontent-%COMP%], .setting-item[_ngcontent-%COMP%]   textarea[_ngcontent-%COMP%]{-moz-box-sizing:border-box;box-sizing:border-box;width:100%}.setting-item[_ngcontent-%COMP%]   input[_ngcontent-%COMP%]::-webkit-inner-spin-button, .setting-item[_ngcontent-%COMP%]   input[_ngcontent-%COMP%]::-webkit-outer-spin-button{-webkit-appearance:none;margin:0}.setting-item[_ngcontent-%COMP%]   .reset[_ngcontent-%COMP%]{color:#999;cursor:pointer;margin-top:2px;text-decoration:underline;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none}.setting-item[_ngcontent-%COMP%]   .reset[_ngcontent-%COMP%]:hover{color:#777}  .model-explorer-settings-help-popup{font-size:12px;padding:8px;background-color:#fff;line-height:normal}  .model-explorer-settings-help-popup .bold{font-weight:700}"]});function X2(a){a&1&&(Nq(0),R(1,"div",6)(2,"a",7)(3,"mat-icon"),Z(4,"open_in_new"),S(),R(5,"span",8),Z(6,"Learn more from GitHub"),S()()(),Oq())}function Y2(a){a&1&&(Nq(0),R(1,"div",6)(2,"a",9)(3,"mat-icon"),Z(4,"open_in_new"),S(),R(5,"span",8),Z(6,"Learn more from g3doc"),S()()(),R(7,"div",6)(8,"a",10)(9,"mat-icon"),Z(10,"group"),S(),R(11,"span",8),Z(12,"Join chat space"),S()()(),Oq())}var Z2=class{get Ad(){return!0}};Z2.K=function(a){return new (a||Z2)};
Z2.Da=Gf({type:Z2,ha:[["welcome-card"]],ka:!0,features:[Mr],za:9,Aa:2,Ca:[[1,"container"],[1,"left"],[1,"title"],[1,"description"],[1,"doc-links-container"],[4,"ngIf"],[1,"doc-link"],["href","https://github.com/google-ai-edge/model-explorer","target","_blank"],[1,"doc-text"],["href","http://go/model-explorer-doc","target","_blank"],["href","https://chat.google.com/room/AAAAxS-rXf0?cls=4","target","_blank"]],ta:function(a,b){a&1&&(R(0,"div",0)(1,"div",1)(2,"div",2),Z(3," Welcome to Model Explorer "),
S(),R(4,"div",3),Z(5," A modern model graph visualizer and debugger "),S(),R(6,"div",4),Ap(7,X2,7,0,"ng-container",5)(8,Y2,13,0,"ng-container",5),S()()());a&2&&(D(7),H("ngIf",b.Ad),D(),H("ngIf",!b.Ad))},Ha:[ht,ct,iI,hI],styles:[".container[_ngcontent-%COMP%]{width:100%;height:100%;-moz-box-sizing:border-box;box-sizing:border-box;border:1px solid #dadce0;border-radius:8px;padding:24px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between}.container[_ngcontent-%COMP%]   .title[_ngcontent-%COMP%]{color:#1f1f1f;font-size:24px}.container[_ngcontent-%COMP%]   .description[_ngcontent-%COMP%]{color:#3c4043;margin-top:8px;font-size:14px}.container[_ngcontent-%COMP%]   .doc-links-container[_ngcontent-%COMP%]{margin-top:24px}.container[_ngcontent-%COMP%]   .doc-link[_ngcontent-%COMP%]{font-size:14px;color:#0b57d0;cursor:pointer;opacity:.8;margin-bottom:8px}.container[_ngcontent-%COMP%]   .doc-link[_ngcontent-%COMP%]   .doc-text[_ngcontent-%COMP%]{color:#0b57d0}.container[_ngcontent-%COMP%]   .doc-link[_ngcontent-%COMP%]   a[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;text-decoration:none}.container[_ngcontent-%COMP%]   .doc-link[_ngcontent-%COMP%]   [_ngcontent-%COMP%]:hover{opacity:1}.container[_ngcontent-%COMP%]   .doc-link[_ngcontent-%COMP%]   [_ngcontent-%COMP%]:hover   .doc-text[_ngcontent-%COMP%]{text-decoration:underline}.container[_ngcontent-%COMP%]   .doc-link[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{font-size:18px;width:18px;height:18px;margin-right:8px;color:#0b57d0}.container[_ngcontent-%COMP%]   .video-container[_ngcontent-%COMP%]{width:430px;height:100%;border-radius:8px;background-color:#f1f1f1;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;color:#999;font-size:13px}"]});var $2=class{};$2.K=function(a){return new (a||$2)};$2.pa=Md({ma:$2,ba:$2.K,ea:"root"});const a3=["modelSourceInput"],b3=["modelGraphVisualizer"],Or=()=>[];function c3(a){a&1&&(R(0,"a",8)(1,"button",20)(2,"mat-icon"),Z(3,"article"),S()()())}function d3(a){a&1&&(R(0,"a",21)(1,"button",22)(2,"mat-icon"),Z(3,"article"),S()()(),R(4,"a",23)(5,"button",24)(6,"mat-icon"),Z(7,"bug_report"),S()()())}function e3(a){a&1&&(R(0,"a",9)(1,"button",25)(2,"mat-icon"),Z(3,"alternate_email"),S()()())}
function f3(a){if(a&1){const b=Ug();R(0,"button",26)(1,"mat-icon");Z(2,"info_outline");S()();R(3,"mat-menu",null,0)(5,"button",27);V("click",function(){u(b);Y().Gh.open(z2,{});return x()});Z(6," Open source libraries ");S();R(7,"div",28);Z(8);S()();Mq(9,"open-in-new-tab-button")}if(a&2){a=zr(4);const b=Y();H("matMenuTriggerFor",a);D(8);Br("v",b.runningVersion(),"")}}
function g3(a){if(a&1){const b=Ug();R(0,"div",11);Mq(1,"welcome-card");R(2,"div",29);V("click",function(){u(b);const c=Y();J2(c.Pc,!1,"show_welcome_card");return x()});R(3,"mat-icon");Z(4,"close");S()()()}}function h3(a){a&1&&(R(0,"div",12),Mq(1,"mat-spinner",30),S())}function i3(a){a&1&&(R(0,"div",13),Mq(1,"model-source-input",null,1),S())}function j3(a){a&1&&(R(0,"a",15)(1,"mat-icon"),Z(2,"bug_report"),S()())}function k3(a){a&1&&Ap(0,j3,3,0,"a",31);a&2&&(a=Y(),H("ngIf",null==a.fe()&&!a.zc))}
function l3(a){a&1&&(R(0,"div",16),Z(1,"The "),R(2,"a",32),Z(3," Google Terms of Service "),S(),Z(4," apply to your use of this offering "),S())}function m3(a){if(a&1){const b=Ug();R(0,"div",34);Mq(1,"div",35);R(2,"div",36)(3,"button",37);V("click",function(){u(b);const c=Y(2);return x(n3(c))});R(4,"mat-icon");Z(5,"share");S()()()()}a&2&&(a=Y(2),D(2),H("matTooltip",a.Kx()),D(),H("disabled",a.rj()))}
function o3(a){if(a&1){const b=Ug();R(0,"model-graph-visualizer",33,2);V("titleClicked",function(){u(b);const c=Y();return x(p3(c))})("modelGraphProcessed",function(c){u(b);const d=Y();if(d.Mg&&0<d.Mg.length&&!d.tx){let e;null==(e=d.PC)||VX(e,d.Mg,c.modelGraph);d.tx=!0}return x()})("uiStateChanged",function(c){u(b);const d=Y();L1(d.fk,c);return x()})("remoteNodeDataPathsChanged",function(c){u(b);var d=Y().fk;d.nodeData=c;J1(d);return x()});Ap(2,m3,6,2,"div",34);S()}a&2&&(a=Y(),H("graphCollections",
a.zc?Nr():a.fe())("benchmark",a.zc)("config",a.qB)("initialUiState",a.Ce)("nodeDataSources",a.Mg),D(2),Eq(a.ee?2:-1))}
function n3(a){null!=window.gtag&&window.gtag("event","share_clicked",Object.assign({},void 0));const b=window.location.href,c=new URLSearchParams;c.append("url",b);c.append("contains_no_pii","true");fetch("https://shortn.corp.google.com/",{mode:"cors",credentials:"include",method:"POST",headers:{Accept:"application/json","Content-Type":"application/x-www-form-urlencoded"},body:c.toString()}).then(d=>{d.json().then(e=>{navigator.clipboard.writeText(e.url);a.G.open("Sharable URL copied to clipboard",
"Close",{duration:5E3})})}).catch(d=>{console.error(d);navigator.clipboard.writeText(b);a.G.open("Sharable URL copied to clipboard","Close",{duration:5E3})})}function p3(a,b=!1){b?a.F.navigate(["/"]).then(()=>{window.location.reload()}):window.history.back()}function q3(a){a.fe.set(void 0);a.Ce=void 0;setTimeout(()=>{a.F.navigate([],{tb:{},px:"",Ul:!0})})}
var r3=class{constructor(a,b,c,d,e,f,g,k,m,p,t){this.Gh=a;this.D=b;this.J=d;this.N=e;this.C=f;this.F=g;this.Pc=k;this.G=m;this.fk=t;this.Bw=this.D.kd;this.fe=this.J.fe;this.runningVersion=Bs(()=>this.N.info().runningVersion);this.zc=this.Zk=!1;this.Mg=[];this.rj=xr(!1);this.Kx=xr("");this.tx=!1;this.Ce=this.fk.uiState;Js(()=>{this.D.kd()||setTimeout(()=>{const w=this.fk.models;null!=w&&0<w.length&&q2(this.bo,w)})});Js(()=>{null!=this.fe()&&window.history.pushState({ts:Date.now()},"")});this.zc="1"===
(new URLSearchParams(document.location.search)).get("benchmark");this.Mg=this.fk.nodeData||[]}pd(){this.bo&&(this.rj=this.bo.rj,this.Kx=Bs(()=>this.rj()?"Share is not available for uploaded models":"Share"))}Lq(a){null==this.fe()&&(this.Zk=!0);a.preventDefault()}Mq(a){a.preventDefault();this.Zk=!1;const b=[];let c;if(null==(c=a.dataTransfer)?0:c.items)[...a.dataTransfer.items].forEach(d=>{"file"===d.kind&&(d=d.getAsFile())&&b.push(d)});else{let d;b.push(...((null==(d=a.dataTransfer)?void 0:d.files)||
[]))}u2(this.bo,b)}get Ad(){return!0}get ee(){return!1}get mE(){const a=M2();return a?H2(this.Pc,a):!0}get qB(){const a="1"===this.C.kb.tb.show_tflite_consts;return{nodeLabelsToHide:this.Pc.getStringValue(C2).split(",").map(b=>b.trim()).filter(b=>""!==b&&(!a||a&&"pseudo_const"!==b&&"pseudo_qconst"!==b)),artificialLayerNodeCountThreshold:I2(this.Pc,D2),edgeLabelFontSize:I2(this.Pc,E2),maxConstValueCount:I2(this.Pc,B2),disallowVerticalEdgeLabels:H2(this.Pc,F2)}}};
r3.K=function(a){return new (a||r3)(z(pC),z(oY),z($2),z("ModelLoaderService"),z(DS),z(d_),z(A1),z(N2),z(vE),z(zM),z(M1))};
r3.Da=Gf({type:r3,ha:[["home-page"]],fb:function(a,b){a&1&&(ur(a3,5),ur(b3,5));if(a&2){let c;vr(c=wr())&&(b.bo=c.first);vr(c=wr())&&(b.PC=c.first)}},Ia:function(a,b){a&1&&V("popstate",function(){return q3(b)},!1,Bi)},ka:!0,features:[Mr],za:25,Aa:14,Ca:[["menu","matMenu"],["modelSourceInput",""],["modelGraphVisualizer",""],[1,"container",3,"dragover","dragleave","dragend","drop"],[1,"title"],[1,"name",3,"click"],[1,"icons-container"],["mat-icon-button","","aria-label","settings","matTooltip","Advanced settings",
3,"click"],["href","http://go/model-explorer-doc","target","_blank"],["href","https://moma.corp.google.com/team/1963115384013","target","_blank"],[1,"content"],[1,"welcome-card-container"],[1,"loading-adapter-extension-container"],[1,"model-source-input-container"],[1,"padding"],["href","https://b.corp.google.com/issues/new?component=1338731&template=1850894","target","_blank","matTooltip","File bug or feature request",1,"bug-report"],[1,"utos"],[3,"graphCollections","benchmark","config","initialUiState",
"nodeDataSources","titleClicked","modelGraphProcessed","uiStateChanged","remoteNodeDataPathsChanged",4,"ngIf"],[1,"dragover-overlay"],[1,"msg"],"mat-icon-button  aria-label doc matTooltip g3doc".split(" "),["href","https://github.com/google-ai-edge/model-explorer","target","_blank"],"mat-icon-button  aria-label doc matTooltip GitHub".split(" "),["href","https://github.com/google-ai-edge/model-explorer/issues","target","_blank"],"mat-icon-button;;aria-label;bug;matTooltip;File issue or feature request".split(";"),
"mat-icon-button;;aria-label;contact;matTooltip;Contact us".split(";"),["mat-icon-button","","aria-label","about","matTooltip","About",3,"matMenuTriggerFor"],["mat-menu-item","",3,"click"],[1,"model-explorer-version"],["matTooltip","Dismiss",1,"icon-container",3,"click"],["color","primary","diameter","24"],["href","https://b.corp.google.com/issues/new?component=1338731&template=1850894","class","bug-report","target","_blank","matTooltip","File bug or feature request",4,"ngIf"],["href","https://policies.google.com/terms?hl=en-US#toc-intro",
"target","_blank"],[3,"titleClicked","modelGraphProcessed","uiStateChanged","remoteNodeDataPathsChanged","graphCollections","benchmark","config","initialUiState","nodeDataSources"],[1,"btns-container"],[1,"divider"],[1,"share-button-container",3,"matTooltip"],["mat-icon-button","","aria-label","share",3,"click","disabled"]],ta:function(a,b){a&1&&(R(0,"div",3),V("dragover",function(c){return b.Lq(c)})("dragleave",function(){b.Zk=!1})("dragend",function(){b.Zk=!1})("drop",function(c){return b.Mq(c)}),
R(1,"div",4)(2,"div",5),V("click",function(){return p3(b,!0)}),Mq(3,"me-logo"),Z(4," Model Explorer "),Mq(5,"new-version-chip"),S(),R(6,"div",6)(7,"button",7),V("click",function(){b.Gh.open(W2,{})}),R(8,"mat-icon"),Z(9,"settings"),S()(),Ap(10,c3,4,0,"a",8)(11,d3,8,0)(12,e3,4,0,"a",9)(13,f3,10,2),S()(),R(14,"div",10),Ap(15,g3,5,0,"div",11)(16,h3,2,0,"div",12)(17,i3,3,0,"div",13),Mq(18,"div",14),S(),Ap(19,k3,1,1,"a",15)(20,l3,5,0,"div",16)(21,o3,3,7,"model-graph-visualizer",17),R(22,"div",18)(23,"div",
19),Z(24,"Drop to add model files"),S()()());a&2&&(nq("dragover",b.Zk),D(),nq("hide",null!=b.fe()||b.zc),D(9),Eq(b.ee?10:11),D(2),Eq(b.ee?12:-1),D(),Eq(b.Ad?13:-1),D(),nq("hide",null!=b.fe()||b.zc),D(),Eq(b.mE?15:-1),D(),Eq(b.Bw()?16:17),D(3),Eq(b.ee?19:-1),D(),Eq(null==b.fe()&&b.Ad&&!b.zc?20:-1),D(),H("ngIf",null!=b.fe()||b.zc))},Ha:[ht,ct,hG,AE,fG,BB,AB,yB,$C,CC,QC,JC,YC,wD,vD,sD,yE,WJ,xC,iI,hI,WX,v2,ES,kI,Z2],styles:[".container[_ngcontent-%COMP%]{width:100%;height:100%;overflow:hidden;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;position:relative}.container.dragover[_ngcontent-%COMP%]   .dragover-overlay[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex}.container[_ngcontent-%COMP%]   .dragover-overlay[_ngcontent-%COMP%]{display:none;position:absolute;left:0;top:0;width:100%;height:100%;background-color:rgba(0,0,0,.1);pointer-events:none;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:end;-webkit-justify-content:flex-end;-moz-box-pack:end;-ms-flex-pack:end;justify-content:flex-end;z-index:500}.container[_ngcontent-%COMP%]   .dragover-overlay[_ngcontent-%COMP%]   .msg[_ngcontent-%COMP%]{margin-bottom:16px;color:#fff;padding:4px 12px;border-radius:99px;background-color:#4285f4}@-webkit-keyframes _ngcontent-%COMP%_rotate{0%{-webkit-transform:rotate(0deg);transform:rotate(0deg)}30%{-webkit-transform:rotate(180deg);transform:rotate(180deg)}60%{-webkit-transform:rotate(1turn);transform:rotate(1turn)}to{-webkit-transform:rotate(1turn);transform:rotate(1turn)}}@keyframes _ngcontent-%COMP%_rotate{0%{-webkit-transform:rotate(0deg);transform:rotate(0deg)}30%{-webkit-transform:rotate(180deg);transform:rotate(180deg)}60%{-webkit-transform:rotate(1turn);transform:rotate(1turn)}to{-webkit-transform:rotate(1turn);transform:rotate(1turn)}}@-webkit-keyframes _ngcontent-%COMP%_goUp{0%{-webkit-transform:translateY(10%);transform:translateY(10%);opacity:0}30%{-webkit-transform:translate(0);transform:translate(0);opacity:1}to{-webkit-transform:translateY(-25%);transform:translateY(-25%);opacity:0}}@keyframes _ngcontent-%COMP%_goUp{0%{-webkit-transform:translateY(10%);transform:translateY(10%);opacity:0}30%{-webkit-transform:translate(0);transform:translate(0);opacity:1}to{-webkit-transform:translateY(-25%);transform:translateY(-25%);opacity:0}}.container[_ngcontent-%COMP%]   .title[_ngcontent-%COMP%]{padding:8px 12px;-moz-box-sizing:border-box;box-sizing:border-box;height:48px;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:justify;-webkit-justify-content:space-between;-moz-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between;cursor:pointer;color:#444746;border-bottom:1px solid transparent}.container[_ngcontent-%COMP%]   .title.hide[_ngcontent-%COMP%]{display:none}.container[_ngcontent-%COMP%]   .title[_ngcontent-%COMP%]   .name[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;font-size:20px}.container[_ngcontent-%COMP%]   .title[_ngcontent-%COMP%]   .name[_ngcontent-%COMP%]   me-logo[_ngcontent-%COMP%]{margin-right:6px}.container[_ngcontent-%COMP%]   .title[_ngcontent-%COMP%]   .name[_ngcontent-%COMP%]   new-version-chip[_ngcontent-%COMP%]{margin-left:16px}.container[_ngcontent-%COMP%]   .title[_ngcontent-%COMP%]   .icons-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}.container[_ngcontent-%COMP%]   .title[_ngcontent-%COMP%]   .icons-container[_ngcontent-%COMP%]   a[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;color:#000}.container[_ngcontent-%COMP%]   .title[_ngcontent-%COMP%]   .icons-container[_ngcontent-%COMP%]   open-in-new-tab-button[_ngcontent-%COMP%]{margin-left:8px}.container[_ngcontent-%COMP%]   .content[_ngcontent-%COMP%]{-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:start;-webkit-justify-content:flex-start;-moz-box-pack:start;-ms-flex-pack:start;justify-content:flex-start;position:relative;margin-top:16px;overflow:hidden}.container[_ngcontent-%COMP%]   .content.hide[_ngcontent-%COMP%]{display:none}.container[_ngcontent-%COMP%]   .content[_ngcontent-%COMP%]   .welcome-card-container[_ngcontent-%COMP%]{width:1016px;-webkit-flex-shrink:0;-ms-flex-negative:0;flex-shrink:0;position:relative}.container[_ngcontent-%COMP%]   .content[_ngcontent-%COMP%]   .welcome-card-container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]{position:absolute;top:6px;right:6px;cursor:pointer;opacity:.5}.container[_ngcontent-%COMP%]   .content[_ngcontent-%COMP%]   .welcome-card-container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]:hover{opacity:.8}.container[_ngcontent-%COMP%]   .content[_ngcontent-%COMP%]   .welcome-card-container[_ngcontent-%COMP%]   .icon-container[_ngcontent-%COMP%]   mat-icon[_ngcontent-%COMP%]{font-size:16px;width:16px;height:16px}.container[_ngcontent-%COMP%]   .content[_ngcontent-%COMP%]   .loading-adapter-extension-container[_ngcontent-%COMP%]{margin-top:20px}.container[_ngcontent-%COMP%]   .content[_ngcontent-%COMP%]   .model-source-input-container[_ngcontent-%COMP%]{width:1016px;margin-top:20px;overflow:hidden;z-index:100;background-color:#fff}.container[_ngcontent-%COMP%]   .content[_ngcontent-%COMP%]   .padding[_ngcontent-%COMP%]{width:100%;-webkit-box-flex:1;-webkit-flex-grow:1;-moz-box-flex:1;-ms-flex-positive:1;flex-grow:1;min-height:12px}.container[_ngcontent-%COMP%]   .content[_ngcontent-%COMP%]   .cover[_ngcontent-%COMP%]{position:absolute;top:0;left:0;width:100%;height:100%;z-index:5000;color:#333;background-color:hsla(0,0%,100%,.9);-webkit-backdrop-filter:blur(10px);backdrop-filter:blur(10px);display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-webkit-flex-direction:column;-moz-box-orient:vertical;-moz-box-direction:normal;-ms-flex-direction:column;flex-direction:column;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center}.container[_ngcontent-%COMP%]   .content[_ngcontent-%COMP%]   .cover[_ngcontent-%COMP%]   .loading-subtitle[_ngcontent-%COMP%]{font-size:14px;color:#999;margin-top:20px}.container[_ngcontent-%COMP%]   .content[_ngcontent-%COMP%]   .loading[_ngcontent-%COMP%]   .wait-icon[_ngcontent-%COMP%]{display:inline-block;margin:0 8px;-webkit-animation:_ngcontent-%COMP%_rotate 2s ease-in-out 0s infinite;animation:_ngcontent-%COMP%_rotate 2s ease-in-out 0s infinite}.container[_ngcontent-%COMP%]   .content[_ngcontent-%COMP%]   .uploading[_ngcontent-%COMP%]   .upload-icon[_ngcontent-%COMP%]{display:inline-block;margin:0 8px;-webkit-animation:_ngcontent-%COMP%_goUp 1s ease-in-out 0s infinite;animation:_ngcontent-%COMP%_goUp 1s ease-in-out 0s infinite}.container[_ngcontent-%COMP%]   .utos[_ngcontent-%COMP%]{position:absolute;right:12px;bottom:12px;font-size:12px;color:#999}.container[_ngcontent-%COMP%]   .utos[_ngcontent-%COMP%]   a[_ngcontent-%COMP%]{color:#999}.container[_ngcontent-%COMP%]   model-graph-visualizer[_ngcontent-%COMP%]{width:100%;height:100%}.container[_ngcontent-%COMP%]   .bug-report[_ngcontent-%COMP%]{position:absolute;bottom:8px;left:8px;display:inline-block;display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center;color:#000;opacity:.5}.container[_ngcontent-%COMP%]   .bug-report[_ngcontent-%COMP%]:hover{opacity:.8}.container[_ngcontent-%COMP%]   .btns-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center}.container[_ngcontent-%COMP%]   .btns-container[_ngcontent-%COMP%]   .share-button-container[_ngcontent-%COMP%]{display:-webkit-box;display:-webkit-flex;display:-moz-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-moz-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-webkit-justify-content:center;-moz-box-pack:center;-ms-flex-pack:center;justify-content:center}.container[_ngcontent-%COMP%]   .btns-container[_ngcontent-%COMP%]   .divider[_ngcontent-%COMP%]{border-left:1px solid #ccc;height:28px;-moz-box-sizing:border-box;box-sizing:border-box;margin-left:8px;margin-right:8px}  .model-explorer-version{font-size:12px;text-align:right;padding:8px 16px 0;color:#999}"]});var s3=class{};s3.K=function(a){return new (a||s3)};s3.Da=Gf({type:s3,ha:[["app-root"]],ka:!0,features:[Mr],za:1,Aa:0,ta:function(a){a&1&&Mq(0,"home-page")},Ha:[r3,I1]});function o2(a,b){return sa(function*(){var c=[];for(const d of b)"local"!==d.type&&"graphs_json_from_server"!==d.type&&"file_path"!==d.type||c.push(t3(a,d));c=yield Promise.all(c);for(const d of b)null!=d.errorMessage&&(d.errorMessage=II(d.errorMessage));null==c.find(d=>0===d.length)&&a.fe.set(c.flat())})}
function t3(a,b){return sa(function*(){b.status.set("Converting");var c=[];if("file_path"===b.type){var d;switch(null==(d=b.Sc)?void 0:d.id){case "builtin_json":try{const f=yield u3(b.path),g=b.path.split("/").pop()||"untitled",k=JSON.parse(f),m=dY(g,k);if(m.error)throw Error(`Failed to process file: ${m.error})`);m.graphCollections&&(c=m.graphCollections);b.status.set("Done")}catch(f){b.selected=!1,b.status.set("Error"),b.errorMessage=f}break;default:c=b.path;var e=c.split("/").pop()||"untitled";
c=yield v3(a,b,c,e,!1)}}else switch(d=b.file,null==(e=b.Sc)?void 0:e.id){case "builtin_json":try{c="graphs_json_from_server"===b.type?yield w3(b.path):yield cY(d),b.status.set("Done")}catch(k){b.selected=!1,b.status.set("Error"),b.errorMessage=k}break;default:b.status.set("Uploading");const {path:f,error:g}=yield x3(d);if(g)return b.selected=!1,b.status.set("Error"),b.errorMessage=g,[];c=yield v3(a,b,f,d.name,!0)}return c})}
function u3(a){return sa(function*(){const b=yield(yield fetch(`${"/api/v1/read_text_file"}?path=${a}`)).json();if(b.error)throw Error(`Failed to read file: ${b.error}`);return b.content})}
function v3(a,b,c,d,e){return sa(function*(){let f=[];b.status.set("Converting");var g,k=(null==(g=b.Sc)?void 0:g.id)||"";g=a.Pc;const m={};for(const w of G2)switch(w.type){case 0:m[w.key]=H2(g,w);break;case 1:m[w.key]=I2(g,w)}const {iB:p,Sw:t}=yield nY({cmdId:"convert",extensionId:k,modelPath:c,settings:m,deleteAfterConversion:e});if(k=(null==p?void 0:p.error)||t)return b.selected=!1,b.status.set("Error"),b.errorMessage=k,[];p&&(f=y3(p,d));b.status.set("Done");return f})}
function w3(a){return sa(function*(){var b=a.replace("graphs://","").split("/");const c=b[0];b=yield(yield fetch(`${"/api/v1/load_graphs_json"}?graph_index=${Number(b[1])}`)).json();return y3(b,c)})}function x3(a){return sa(function*(){var b=new FormData;b.append("file",a,a.name);b=yield fetch("/apipost/v1/upload",{method:"POST",body:b});if(b.ok)return{path:JSON.parse(yield b.text()).path};console.error(yield b.text());return{path:"",error:"Failed to upload model. Check console for details"}})}
function y3(a,b){return a.graphs?[{label:b,graphs:a.graphs}]:a.graphCollections?a.graphCollections.map(c=>({label:""===c.label?b:`${b} (${c.label})`,graphs:c.graphs})):[]}var z3=class{constructor(a){this.Pc=a;this.fe=xr()}};z3.K=function(a){return new (a||z3)(r(N2),r(oY))};z3.pa=Md({ma:z3,ba:z3.K,ea:"root"});var A3=class{};function B3(){return new Zd(3008,!1)};function C3(a){if("number"==typeof a)return a;a=a.match(/^(-?[\.\d]+)(m?s)/);return!a||2>a.length?0:D3(parseFloat(a[1]),a[2])}function D3(a,b){switch(b){case "s":return 1E3*a;default:return a}}
function E3(a,b,c){if(a.hasOwnProperty("duration"))b=a;else a:{var d=/^(-?[\.\d]+)(m?s)(?:\s+(-?[\.\d]+)(m?s))?(?:\s+([-a-z]+(?:\(.+?\))?))?$/i;let e=0,f="";if("string"===typeof a){d=a.match(d);if(null===d){b.push(new Zd(3E3,!1));b={duration:0,delay:0,easing:""};break a}a=D3(parseFloat(d[1]),d[2]);const g=d[3];null!=g&&(e=D3(parseFloat(g),d[4]));(d=d[5])&&(f=d)}c||(c=!1,d=b.length,0>a&&(b.push(new Zd(3100,!1)),c=!0),0>e&&(b.push(new Zd(3101,!1)),c=!0),c&&b.splice(d,0,new Zd(3E3,!1)));b={duration:a,
delay:e,easing:f}}return b}function F3(a){return a.length?a[0]instanceof Map?a:a.map(b=>new Map(Object.entries(b))):[]}function G3(a,b,c){b.forEach((d,e)=>{const f=H3(e);c&&!c.has(e)&&c.set(e,a.style[f]);a.style[f]=d})}function I3(a,b){b.forEach((c,d)=>{d=H3(d);a.style[d]=""})}function J3(a){if(Array.isArray(a)){var b;1==a.length?b=a[0]:b={type:2,steps:a,options:null};a=b}return a}
function K3(a,b,c){const d=b.params||{};a=L3(a);a.length&&a.forEach(e=>{d.hasOwnProperty(e)||c.push(new Zd(3001,!1))})}const M3=RegExp("{{\\s*(.+?)\\s*}}","g");function L3(a){let b=[];if("string"===typeof a){let c;for(;c=M3.exec(a);)b.push(c[1]);M3.lastIndex=0}return b}function N3(a,b,c){const d=`${a}`,e=d.replace(M3,(f,g)=>{g=b[g];null==g&&(c.push(new Zd(3003,!1)),g="");return g.toString()});return e==d?a:e}const O3=/-+([a-z0-9])/g;function H3(a){return a.replace(O3,(...b)=>b[1].toUpperCase())}
function P3(a,b,c){if(c.size&&b.length){let d=b[0],e=[];c.forEach((f,g)=>{d.has(g)||e.push(g);d.set(g,f)});if(e.length)for(c=1;c<b.length;c++){let f=b[c];e.forEach(g=>f.set(g,Q3(a,g)))}}return b}
function R3(a,b,c){switch(b.type){case 7:return a.Ny(b,c);case 0:return a.Ps(b,c);case 1:return a.Qs(b,c);case 2:return a.Ly(b,c);case 3:return a.Jy(b,c);case 4:return a.Gy(b,c);case 5:return a.Jo(b,c);case 6:return a.mm(b,c);case 8:return a.Ko(b,c);case 9:return a.Hy(b,c);case 10:return a.Iy(b,c);case 11:return a.Ky(b,c);case 12:return a.My(b,c);default:throw new Zd(3004,!1);}}function Q3(a,b){return window.getComputedStyle(a)[b]};const S3=new Set("width height minWidth minHeight maxWidth maxHeight left top bottom right fontSize outlineWidth outlineOffset paddingTop paddingLeft paddingBottom paddingRight marginTop marginLeft marginBottom marginRight borderRadius borderWidth borderTopWidth borderLeftWidth borderRightWidth borderBottomWidth textIndent perspective".split(" "));
function T3(a,b,c){let d="";const e=b.toString().trim();S3.has(a)&&0!==b&&"0"!==b&&("number"===typeof b?d="px":(a=b.match(/^[+-]?[\d\.]+([a-z]*)$/))&&0==a[1].length&&c.push(new Zd(3005,!1)));return e+d}var U3=class extends A3{};function V3(a){switch(a.length){case 0:return new fz;case 1:return a[0];default:return new jz(a)}}function W3(a,b=new Map,c=new Map){const d=[],e=[];let f=-1,g=null;a.forEach(k=>{const m=k.get("offset"),p=m==f,t=p&&g||new Map;k.forEach((w,B)=>{let G=B;if("offset"!==B)switch(G=H3(G),w){case "!":w=b.get(B);break;case "*":w=c.get(B);break;default:w=T3(G,w,d)}t.set(G,w)});p||e.push(t);g=t;f=m});if(d.length)throw new Zd(3502,!1);return e}
function X3(a,b,c,d){switch(b){case "start":a.onStart(()=>d(c&&Y3(c,"start",a)));break;case "done":a.Kd(()=>d(c&&Y3(c,"done",a)));break;case "destroy":a.ob(()=>d(c&&Y3(c,"destroy",a)))}}function Y3(a,b,c){const d=c.totalTime;b=Z3(a.element,a.Qc,a.zg,a.me,b||a.uD,void 0==d?a.totalTime:d,c.disabled?!0:!1);a=a._data;null!=a&&(b._data=a);return b}function Z3(a,b,c,d,e="",f=0,g){return{element:a,Qc:b,zg:c,me:d,uD:e,totalTime:f,disabled:!!g}}function $3(a,b,c){let d=a.get(b);d||a.set(b,d=c);return d}
function a4(a){const b=a.indexOf(":");return[a.substring(1,b),a.slice(b+1)]}const b4="undefined"===typeof document?null:document.documentElement;function c4(a){a=a.parentNode||a.host||null;return a===b4?null:a}function d4(a,b){for(;b;){if(b===a)return!0;b=c4(b)}return!1};var e4=class{};function f4(a,b){const c=[];"string"==typeof a?a.split(/\s*,\s*/).forEach(d=>{a:{if(":"==d[0]&&(d=g4(d,b),"function"==typeof d)){c.push(d);d=void 0;break a}var e=d.match(/^(\*|[-\w]+)\s*(<?[=-]>)\s*(\*|[-\w]+)$/);if(null==e||4>e.length)b.push(new Zd(3015,!1)),d=c;else{d=e[1];var f=e[2];e=e[3];c.push(h4(d,e));"<"!=f[0]||"*"==d&&"*"==e||c.push(h4(e,d));d=void 0}}return d}):c.push(a);return c}
function g4(a,b){switch(a){case ":enter":return"void => *";case ":leave":return"* => void";case ":increment":return(c,d)=>parseFloat(d)>parseFloat(c);case ":decrement":return(c,d)=>parseFloat(d)<parseFloat(c);default:return b.push(new Zd(3016,!1)),"* => *"}}const i4=new Set(["true","1"]),j4=new Set(["false","0"]);
function h4(a,b){const c=i4.has(a)||j4.has(a),d=i4.has(b)||j4.has(b);return(e,f)=>{let g="*"==a||a==e,k="*"==b||b==f;!g&&c&&"boolean"===typeof e&&(g=e?i4.has(a):j4.has(a));!k&&d&&"boolean"===typeof f&&(k=f?i4.has(b):j4.has(b));return g&&k}};const k4=RegExp("s*:selfs*,?","g");
var q4=class{D(a,b){b=new l4(b);this.F(b);return R3(this,J3(a),b)}F(a){a.D="";a.G=new Map;a.G.set("",new Map);a.C=0}Ny(a,b){let c=b.Kg=0,d=b.bf=0;const e=[],f=[];"@"==a.name.charAt(0)&&b.errors.push(new Zd(3006,!1));a.yB.forEach(g=>{this.F(b);if(0==g.type){var k=g.name;k.toString().split(/\s*,\s*/).forEach(m=>{g.name=m;e.push(this.Ps(g,b))});g.name=k}else 1==g.type?(k=this.Qs(g,b),c+=k.Kg,d+=k.bf,f.push(k)):b.errors.push(new Zd(3007,!1))});return{type:7,name:a.name,ki:e,Yg:f,Kg:c,bf:d,options:null}}Ps(a,
b){const c=this.mm(a.styles,b),d=a.options&&a.options.params||null;if(c.nB){const e=new Set,f=d||{};c.styles.forEach(g=>{g instanceof Map&&g.forEach(k=>{L3(k).forEach(m=>{f.hasOwnProperty(m)||e.add(m)})})});e.size&&b.errors.push(B3([...e.values()]))}return{type:0,name:a.name,style:c,options:d?{params:d}:null}}Qs(a,b){b.Kg=0;b.bf=0;const c=R3(this,J3(a.animation),b);return{type:1,Ew:f4(a.MB,b.errors),animation:c,Kg:b.Kg,bf:b.bf,options:m4(a.options)}}Ly(a,b){return{type:2,steps:a.steps.map(c=>R3(this,
c,b)),options:m4(a.options)}}Jy(a,b){const c=b.C;let d=0;const e=a.steps.map(f=>{b.C=c;f=R3(this,f,b);d=Math.max(d,b.C);return f});b.C=d;return{type:3,steps:e,options:m4(a.options)}}Gy(a,b){const c=n4(a.dg,b.errors);b.F=c;var d=a.styles?a.styles:$y({});if(5==d.type)a=this.Jo(d,b);else{d=a.styles;a=!1;d||(a=!0,d={},c.easing&&(d.easing=c.easing),d=$y(d));b.C+=c.duration+c.delay;const e=this.mm(d,b);e.vC=a;a=e}b.F=null;return{type:4,dg:c,style:a,options:null}}mm(a,b){a=this.C(a,b);this.G(a,b);return a}C(a,
b){const c=[],d=Array.isArray(a.styles)?a.styles:[a.styles];for(let g of d)"string"===typeof g?"*"===g?c.push(g):b.errors.push(new Zd(3002,!1)):c.push(new Map(Object.entries(g)));let e=!1,f=null;c.forEach(g=>{if(g instanceof Map&&(g.has("easing")&&(f=g.get("easing"),g.delete("easing")),!e))for(let k of g.values())if(0<=k.toString().indexOf("{{")){e=!0;break}});return{type:6,styles:c,easing:f,offset:a.offset,nB:e,options:null}}G(a,b){const c=b.F;let d=b.C,e=b.C;c&&0<e&&(e-=c.duration+c.delay);a.styles.forEach(f=>
{"string"!==typeof f&&f.forEach((g,k)=>{const m=b.G.get(b.D),p=m.get(k);let t=!0;p&&(e!=d&&e>=p.startTime&&d<=p.endTime&&(b.errors.push(new Zd(3010,!1)),t=!1),e=p.startTime);t&&m.set(k,{startTime:e,endTime:d});b.options&&K3(g,b.options,b.errors)})})}Jo(a,b){const c={type:5,styles:[],options:null};if(!b.F)return b.errors.push(new Zd(3011,!1)),c;let d=0;const e=[];let f=!1,g=!1,k=0;const m=a.steps.map(J=>{J=this.C(J,b);let N=null!=J.offset?J.offset:o4(J.styles),aa=0;null!=N&&(d++,aa=J.offset=N);g=g||
0>aa||1<aa;f=f||aa<k;k=aa;e.push(aa);return J});g&&b.errors.push(new Zd(3012,!1));f&&b.errors.push(new Zd(3200,!1));a=a.steps.length;let p=0;0<d&&d<a?b.errors.push(new Zd(3202,!1)):0==d&&(p=1/(a-1));const t=a-1,w=b.C,B=b.F,G=B.duration;m.forEach((J,N)=>{N=0<p?N==t?1:p*N:e[N];const aa=N*G;b.C=w+B.delay+aa;B.duration=aa;this.G(J,b);J.offset=N;c.styles.push(J)});return c}Ko(a,b){return{type:8,animation:R3(this,J3(a.animation),b),options:m4(a.options)}}Hy(a,b){b.bf++;return{type:9,options:m4(a.options)}}Iy(a,
b){return{type:10,animation:this.Ko(a.animation,b),options:m4(a.options)}}Ky(a,b){const c=b.D,d=a.options||{};b.Kg++;b.J=a;const [e,f]=p4(a.Wl);b.D=c.length?c+" "+e:e;$3(b.G,b.D,new Map);const g=R3(this,J3(a.animation),b);b.J=null;b.D=c;return{type:11,Wl:e,limit:d.limit||0,optional:!!d.optional,mC:f,animation:g,eG:a.Wl,options:m4(a.options)}}My(a,b){b.J||b.errors.push(new Zd(3013,!1));const c="full"===a.dg?{duration:0,delay:0,easing:"full"}:E3(a.dg,b.errors,!0);return{type:12,animation:R3(this,J3(a.animation),
b),dg:c,options:null}}};function p4(a){const b=a.split(/\s*,\s*/).find(c=>":self"==c)?!0:!1;b&&(a=a.replace(k4,""));a=a.replace(/@\*/g,".ng-trigger").replace(/@\w+/g,c=>".ng-trigger-"+c.slice(1)).replace(/:animating/g,".ng-animating");return[a,b]}var l4=class{constructor(a){this.errors=a;this.bf=this.Kg=0;this.F=this.D=this.J=null;this.C=0;this.G=new Map;this.options=null}};
function o4(a){if("string"==typeof a)return null;let b=null;Array.isArray(a)?a.forEach(c=>{c instanceof Map&&c.has("offset")&&(b=parseFloat(c.get("offset")),c.delete("offset"))}):a instanceof Map&&a.has("offset")&&(b=parseFloat(a.get("offset")),a.delete("offset"));return b}
function n4(a,b){if(a.hasOwnProperty("duration"))return a;if("number"==typeof a)return{duration:E3(a,b).duration,delay:0,easing:""};if(a.split(/\s+/).some(c=>"{"==c.charAt(0)&&"{"==c.charAt(1)))return b={duration:0,delay:0,easing:"",dynamic:!0},b.strValue=a,b;a=E3(a,b);return{duration:a.duration,delay:a.delay,easing:a.easing}}function m4(a){if(a){if(a=Object.assign({},a),a.params){var b=a;var c=(c=a.params)?Object.assign({},c):null;b.params=c}}else a={};return a};function r4(a,b,c,d,e,f,g=null,k=!1){return{type:1,element:a,Bg:b,Fj:c,ai:d,duration:e,delay:f,totalTime:e+f,easing:g,tE:k}};var s4=class{constructor(){this.C=new Map}get(a){return this.C.get(a)||[]}append(a,b){let c=this.C.get(a);c||this.C.set(a,c=[]);c.push(...b)}has(a){return this.C.has(a)}clear(){this.C.clear()}};const t4=RegExp(":enter","g"),u4=RegExp(":leave","g");function v4(a,b,c,d,e,f=new Map,g=new Map,k,m,p=[]){return w4(a,b,c,d,e,f,g,k,m,p)}
function w4(a,b,c,d,e,f,g,k,m,p=[]){var t=new x4;m=m||new s4;a=new y4(a,b,m,d,e,p,[]);a.options=k;d=k.delay?C3(k.delay):0;z4(a.C,d);A4(a.C,[f],null,a.errors,k);R3(t,c,a);c=a.sf.filter(w=>w.sa());if(c.length&&g.size){let w;for(f=c.length-1;0<=f;f--)if(t=c[f],t.element===b){w=t;break}w&&w.na===w.C&&A4(w,[g],null,a.errors,k)}return c.length?c.map(w=>w.ja()):[r4(b,[],[],[],0,d,"",!1)]}
var x4=class{Ny(){}Ps(){}Qs(){}Hy(a,b){var c=b.ca.get(b.element);if(c){const d=B4(b,a.options),e=b.C.D;c=this.D(c,d,d.options);e!=c&&C4(b,c)}b.D=a}Iy(a,b){const c=B4(b,a.options);C4(c);this.C([a.options,a.animation.options],b,c);this.Ko(a.animation,c);C4(b,c.C.D);b.D=a}C(a,b,c){for(const d of a){let e;if(a=null==(e=d)?void 0:e.delay){let f,g;a="number"===typeof a?a:C3(N3(a,null!=(g=null==(f=d)?void 0:f.params)?g:{},b.errors));D4(c,a)}}}D(a,b,c){let d=b.C.D;const e=null!=c.duration?C3(c.duration):
null,f=null!=c.delay?C3(c.delay):null;0!==e&&a.forEach(g=>{const k={duration:null!=e?e:g.duration,delay:b.C.D+(null!=f?f:0)+g.delay,easing:""};g=new E4(b.J,g.element,g.Bg,g.Fj,g.ai,k,g.sE);b.sf.push(g);d=Math.max(d,k.duration+k.delay)});return d}Ko(a,b){F4(b,a.options,!0);R3(this,a.animation,b);b.D=a}Ly(a,b){const c=b.N;let d=b;const e=a.options;e&&(e.params||e.delay)&&(d=B4(b,e),C4(d),null!=e.delay&&(6==d.D.type&&(G4(d.C),d.D=H4),D4(d,C3(e.delay))));a.steps.length&&(a.steps.forEach(f=>R3(this,f,
d)),I4(d.C),d.N>c&&C4(d));b.D=a}Jy(a,b){const c=[];let d=b.C.D;const e=a.options&&a.options.delay?C3(a.options.delay):0;a.steps.forEach(f=>{const g=B4(b,a.options);e&&D4(g,e);R3(this,f,g);d=Math.max(d,g.C.D);c.push(g.C)});c.forEach(f=>J4(b.C,f));C4(b,d);b.D=a}F(a,b){return a.dynamic?(a=a.strValue,a=b.params?N3(a,b.params,b.errors):a,E3(a,b.errors)):{duration:a.duration,delay:a.delay,easing:a.easing}}Gy(a,b){const c=b.F=this.F(a.dg,b),d=b.C;c.delay&&(K4(b.C,b.C.duration+c.delay),G4(d));const e=a.style;
5==e.type?this.Jo(e,b):(K4(b.C,b.C.duration+c.duration),this.mm(e,b),I4(d));b.F=null;b.D=a}mm(a,b){const c=b.C;var d=b.F;!d&&0<c.C.size&&(c.duration+=1,c.T());d=d&&d.easing||a.easing;if(a.vC){d&&c.aa.set("easing",d);for(let [e,f]of c.F)d=e,c.P.set(d,f||"*"),c.C.set(d,"*");c.na=c.C}else A4(c,a.styles,d,b.errors,b.options);b.D=a}Jo(a,b){const c=b.F,d=b.C.duration,e=c.duration,f=B4(b).C;f.easing=c.easing;a.styles.forEach(g=>{K4(f,(g.offset||0)*e);A4(f,g.styles,g.easing,b.errors,b.options);I4(f)});J4(b.C,
f);C4(b,d+e);b.D=a}Ky(a,b){var c=b.C.D;const d=a.options||{},e=d.delay?C3(d.delay):0;e&&(6===b.D.type||0==c&&0<b.C.C.size)&&(G4(b.C),b.D=H4);let f=c;c=L4(b,a.Wl,a.limit,a.mC,d.optional?!0:!1,b.errors);b.G=c.length;let g=null;c.forEach((k,m)=>{b.ce=m;m=B4(b,a.options,k);e&&D4(m,e);k===b.element&&(g=m.C);R3(this,a.animation,m);I4(m.C);f=Math.max(f,m.C.D)});b.ce=0;b.G=0;C4(b,f);g&&(J4(b.C,g),G4(b.C));b.D=a}My(a,b){const c=b.aa,d=b.C;var e=a.dg,f=Math.abs(e.duration);const g=f*(b.G-1);f*=b.ce;switch(0>
e.duration?"reverse":e.easing){case "reverse":f=g-f;break;case "full":f=c.T}e=b.C;f&&z4(e,f);e=e.D;R3(this,a.animation,b);b.D=a;c.T=d.D-e+(d.startTime-c.C.startTime)}};const H4={};function B4(a,b=null,c){c=c||a.element;c=new y4(a.J,c,a.ca,a.O,a.P,a.errors,a.sf,a.C.fork(c,0));c.D=a.D;c.F=a.F;c.options=a.fa();F4(c,b);c.ce=a.ce;c.G=a.G;c.aa=a;a.N++;return c}function C4(a,b){a.D=H4;a.C=a.C.fork(a.element,b);a.sf.push(a.C)}function D4(a,b){0<b&&z4(a.C,b)}
function F4(a,b,c){if(b){var d=a.options;null!=b.duration&&(d.duration=C3(b.duration));null!=b.delay&&(d.delay=C3(b.delay));var e=b.params;if(e){let f=d.params;f||(f=a.options.params={});Object.keys(e).forEach(g=>{c&&f.hasOwnProperty(g)||(f[g]=N3(e[g],f,a.errors))})}}}
function L4(a,b,c,d,e,f){let g=[];d&&g.push(a.element);0<b.length&&(b=b.replace(t4,"."+a.O),b=b.replace(u4,"."+a.P),a=a.J.query(a.element,b,1!=c),0!==c&&(a=0>c?a.slice(a.length+c,a.length):a.slice(0,c)),g.push(...a));e||0!=g.length||f.push(new Zd(3014,!1));return g}
var y4=class{constructor(a,b,c,d,e,f,g,k){this.J=a;this.element=b;this.ca=c;this.O=d;this.P=e;this.errors=f;this.sf=g;this.F=this.aa=null;this.D=H4;this.N=0;this.options={};this.T=this.G=this.ce=0;this.C=k||new M4(this.J,b,0);g.push(this.C)}get params(){return this.options.params}fa(){const a={};if(this.options){const b=this.options.params;if(b){const c=a.params={};Object.keys(b).forEach(d=>{c[d]=b[d]})}}return a}};
function z4(a,b){const c=1===a.G.size&&a.N.size;a.duration||c?(K4(a,a.D+b),c&&G4(a)):a.startTime+=b}function A4(a,b,c,d,e){c&&a.aa.set("easing",c);c=e&&e.params||{};b=N4(b,a.F);for(let [f,g]of b){b=f;e=N3(g,c,d);a.N.set(b,e);if(!a.J.has(b)){let k;a.P.set(b,null!=(k=a.F.get(b))?k:"*")}a.fa(b,e)}}function G4(a){for(let [b,c]of a.J){let d=b,e=c;a.N.set(d,e);a.fa(d,e)}}function I4(a){0!=a.N.size&&(a.N.forEach((b,c)=>{a.C.set(c,b)}),a.N.clear(),a.J.forEach((b,c)=>{a.C.has(c)||a.C.set(c,b)}))}
function J4(a,b){b.ca.forEach((c,d)=>{const e=a.ca.get(d);(!e||c.time>e.time)&&a.fa(d,c.value)})}function K4(a,b){I4(a);a.duration=b;a.T()}
var M4=class{constructor(a,b,c,d){this.ua=a;this.element=b;this.startTime=c;this.O=d;this.duration=0;this.easing=null;this.aa=new Map;this.C=new Map;this.G=new Map;this.ca=new Map;this.J=new Map;this.N=new Map;this.P=new Map;this.na=null;this.O||(this.O=new Map);this.F=this.O.get(b);this.F||(this.F=this.J,this.O.set(b,this.J));this.T()}sa(){switch(this.G.size){case 0:return!1;case 1:return 0<this.C.size;default:return!0}}get D(){return this.startTime+this.duration}fork(a,b){I4(this);return new M4(this.ua,
a,b||this.D,this.O)}T(){this.C&&(this.aa=this.C);this.C=this.G.get(this.duration);this.C||(this.C=new Map,this.G.set(this.duration,this.C))}fa(a,b){this.J.set(a,b);this.F.set(a,b);this.ca.set(a,{time:this.D,value:b})}get properties(){const a=[];for(let b in this.C)a.push(b);return a}ja(){I4(this);const a=new Set,b=new Set,c=1===this.G.size&&0===this.duration;let d=[];this.G.forEach((g,k)=>{g=new Map([...this.P,...g]);g.forEach((m,p)=>{"!"===m?a.add(p):"*"===m&&b.add(p)});c||g.set("offset",k/this.duration);
d.push(g)});const e=[...a.values()],f=[...b.values()];if(c){const g=d[0],k=new Map(g);g.set("offset",0);k.set("offset",1);d=[g,k]}return r4(this.element,d,e,f,this.duration,this.startTime,this.easing,!1)}};
class E4 extends M4{constructor(a,b,c,d,e,f,g=!1){super(a,b,f.delay);this.Bg=c;this.Fj=d;this.ai=e;this.wa=g;this.dg={duration:f.duration,delay:f.delay,easing:f.easing}}sa(){return 1<this.Bg.length}ja(){let a=this.Bg,{delay:b,duration:c,easing:d}=this.dg;if(this.wa&&b){const g=[],k=c+b;var e=b/k,f=new Map(a[0]);f.set("offset",0);g.push(f);f=new Map(a[0]);f.set("offset",O4(e));g.push(f);e=a.length-1;for(f=1;f<=e;f++){let m=new Map(a[f]);const p=m.get("offset");m.set("offset",O4((b+p*c)/k));g.push(m)}c=
k;b=0;d="";a=g}return r4(this.element,a,this.Fj,this.ai,c,b,d,!0)}}function O4(a){const b=Math.pow(10,2);return Math.round(a*b)/b}function N4(a,b){const c=new Map;let d;a.forEach(e=>{if("*"===e){null!=d||(d=b.keys());for(let f of d)c.set(f,"*")}else for(let [f,g]of e)c.set(f,g)});return c};function P4(a,b,c,d,e,f,g,k,m,p,t,w,B){return{type:0,element:a,Qc:b,ww:e,zg:c,Cn:f,me:d,gm:g,sf:k,CD:m,Fj:p,ai:t,totalTime:w,errors:B}};const Q4={};function R4(a,b,c,d){let e=a.F.get("*");void 0!==b&&(e=a.F.get(null==b?void 0:b.toString())||e);return e?S4(e,c,d):new Map}
var V4=class{constructor(a,b,c){this.G=a;this.C=b;this.F=c}match(a,b,c,d){return T4(this.C.Ew,a,b,c,d)}D(a,b,c,d,e,f,g,k,m,p){const t=[];var w=this.C.options&&this.C.options.params||Q4;g=R4(this,c,g&&g.params||Q4,t);const B=k&&k.params||Q4;k=R4(this,d,B,t);const G=new Set,J=new Map,N=new Map,aa="void"===d;let ca;w={params:U4(B,w),delay:null==(ca=this.C.options)?void 0:ca.delay};a=p?[]:v4(a,b,this.C.animation,e,f,g,k,w,m,t);let ia=0;a.forEach(O=>{ia=Math.max(O.duration+O.delay,ia)});if(t.length)return P4(b,
this.G,c,d,aa,g,k,[],[],J,N,ia,t);a.forEach(O=>{const T=O.element,Q=$3(J,T,new Set);O.Fj.forEach(F=>Q.add(F));const la=$3(N,T,new Set);O.ai.forEach(F=>la.add(F));T!==b&&G.add(T)});return P4(b,this.G,c,d,aa,g,k,a,[...G.values()],J,N,ia)}};function T4(a,b,c,d,e){return a.some(f=>f(b,c,d,e))}function U4(a,b){const c=Object.assign({},b);Object.entries(a).forEach(([d,e])=>{null!=e&&(c[d]=e)});return c}
function S4(a,b,c){const d=new Map,e=U4(b,a.C);a.styles.styles.forEach(f=>{"string"!==typeof f&&f.forEach((g,k)=>{g&&(g=N3(g,e,c));const m=H3(k);g=T3(m,g,c);d.set(k,g)})});return d}var W4=class{constructor(a,b){this.styles=a;this.C=b}};function X4(a,b,c,d,e){return a.F.find(f=>f.match(b,c,d,e))||null}var $4=class{constructor(a,b){this.name=a;this.C=b;this.F=[];this.ki=new Map;b.ki.forEach(c=>{this.ki.set(c.name,new W4(c.style,c.options&&c.options.params||{}))});Y4(this.ki,"true","1");Y4(this.ki,"false","0");b.Yg.forEach(c=>{this.F.push(new V4(a,c,this.ki))});this.D=Z4(a,this.ki)}};function Z4(a,b){return new V4(a,{type:1,animation:{type:2,steps:[],options:null},Ew:[()=>!0],options:null,Kg:0,bf:0},b)}
function Y4(a,b,c){a.has(b)?a.has(c)||a.set(c,a.get(b)):a.has(c)&&a.set(b,a.get(c))};const a5=new s4;
var b5=class{constructor(a){this.G=a;this.J=new Map;this.F=new Map;this.C=[]}register(a,b){const c=[];b=(new q4).D(b,c,[]);if(c.length)throw new Zd(3503,!1);this.J.set(a,b)}create(a,b,c={}){var d=[];const e=this.J.get(a),f=new Map;e?(b=v4(this.G,b,e,"ng-enter","ng-leave",new Map,new Map,c,a5,d),b.forEach(g=>{const k=$3(f,g.element,new Map);g.ai.forEach(m=>k.set(m,null))})):(d.push(new Zd(3300,!1)),b=[]);if(d.length)throw new Zd(3504,!1);f.forEach((g,k)=>{g.forEach((m,p)=>{g.set(p,Q3(k,p))})});d=b.map(g=>
{var k=f.get(g.element);const m=g.element;k=W3(g.Bg,new Map,k);return this.G.animate(m,k,g.duration,g.delay,g.easing,[],!0)});d=V3(d);this.F.set(a,d);d.ob(()=>this.destroy(a));this.C.push(d);return d}destroy(a){const b=this.D(a);b.destroy();this.F.delete(a);a=this.C.indexOf(b);0<=a&&this.C.splice(a,1)}D(a){a=this.F.get(a);if(!a)throw new Zd(3301,!1);return a}Gd(a,b,c,d){b=Z3(b,"","","");X3(this.D(a),c,b,d);return()=>{}}};const c5=[],d5={lc:"",hi:!1,ss:!1,Gn:!1,bs:!1},e5={lc:"",ss:!1,hi:!1,Gn:!1,bs:!0};function f5(a,b){const c=b.params;if(c){const d=a.options.params;Object.keys(c).forEach(e=>{null==d[e]&&(d[e]=c[e])})}}class g5{get params(){return this.options.params}constructor(a,b){this.lc=void 0===b?"":b;var c=(b=a&&a.hasOwnProperty("value"))?a.value:a;this.value=null!=c?c:null;b?(a=Object.assign({},a),this.options=(delete a.value,a)):this.options={};this.options.params||(this.options.params={})}}const h5=new g5("void");
function i5(a,b){a.C.C.delete(b);a.G.delete(b);const c=a.C.N.get(b);c&&(c.forEach(d=>d.destroy()),a.C.N.delete(b))}function j5(a,b,c,d,e){const f=a.C.C.get(b),g=new Map;if(f){const k=[];f.forEach((m,p)=>{g.set(p,m.value);a.D.has(p)&&(m=a.Le(b,p,"void",e))&&k.push(m)});if(k.length)return k5(a.C,a.id,b,!0,c,g),d&&V3(k).Kd(()=>l5(a.C,b)),!0}return!1}
function m5(a,b){const c=a.G.get(b),d=a.C.C.get(b);if(c&&d){const e=new Set;c.forEach(f=>{f=f.name;if(!e.has(f)){e.add(f);var g=a.D.get(f).D,k=d.get(f)||h5,m=new g5("void"),p=new n5(a.id,f,b);a.C.na++;a.J.push({element:b,Qc:f,transition:g,zg:k,me:m,Yh:p,qw:!0})}})}}
function o5(a,b){const c=[];a.J.forEach(d=>{const e=d.Yh;if(!e.C){var f=d.element,g=a.G.get(f);g&&g.forEach(k=>{if(k.name==d.Qc){const m=Z3(f,d.Qc,d.zg.value,d.me.value);m._data=b;X3(d.Yh,k.Xh,m,k.callback)}});e.O?p5(a.C,()=>{e.destroy()}):c.push(d)}});a.J=[];return c.sort((d,e)=>{const f=d.transition.C.bf,g=e.transition.C.bf;return 0==f||0==g?f-g:d4(d.element,e.element)?1:-1})}
class q5{constructor(a,b,c){this.id=a;this.df=b;this.C=c;this.F=[];this.D=new Map;this.J=[];this.G=new Map;this.N="ng-tns-"+a;r5(b,this.N)}Gd(a,b,c,d){if(!this.D.has(b))throw new Zd(3302,!1);if(null==c||0==c.length)throw new Zd(3303,!1);if("start"!=c&&"done"!=c)throw new Zd(3400,!1);const e=$3(this.G,a,[]),f={name:b,Xh:c,callback:d};e.push(f);const g=$3(this.C.C,a,new Map);g.has(b)||(r5(a,"ng-trigger"),r5(a,"ng-trigger-"+b),g.set(b,h5));return()=>{p5(this.C,()=>{const k=e.indexOf(f);0<=k&&e.splice(k,
1);this.D.has(b)||g.delete(b)})}}register(a,b){if(this.D.has(a))return!1;this.D.set(a,b);return!0}P(a){a=this.D.get(a);if(!a)throw new Zd(3401,!1);return a}Le(a,b,c,d){d=void 0===d?!0:d;const e=this.P(b),f=new n5(this.id,b,a);var g=this.C.C.get(a);g||(r5(a,"ng-trigger"),r5(a,"ng-trigger-"+b),this.C.C.set(a,g=new Map));let k=g.get(b);const m=new g5(c,this.id);c&&c.hasOwnProperty("value")||!k||f5(m,k.options);g.set(b,m);k||(k=h5);if("void"!==m.value&&k.value===m.value){if(!s5(k.params,m.params)){d=
[];const t=R4(e.D,k.value,k.params,d),w=R4(e.D,m.value,m.params,d);if(d.length)throw new Zd(3402,!1);p5(this.C,()=>{I3(a,t);G3(a,w)})}}else{c=$3(this.C.N,a,[]);c.forEach(t=>{t.lc==this.id&&t.Qc==b&&t.Md&&t.destroy()});g=X4(e,k.value,m.value,a,m.params);var p=!1;if(!g){if(!d)return;g=e.D;p=!0}this.C.na++;this.J.push({element:a,Qc:b,transition:g,zg:k,me:m,Yh:f,qw:p});p||(r5(a,"ng-animate-queued"),f.onStart(()=>{t5(a,"ng-animate-queued")}));f.Kd(()=>{var t=this.F.indexOf(f);0<=t&&this.F.splice(t,1);
if(t=this.C.N.get(a)){let w=t.indexOf(f);0<=w&&t.splice(w,1)}});this.F.push(f);c.push(f);return f}}O(a,b){const c=this.C.F.query(a,".ng-trigger",!0);c.forEach(d=>{if(!d.__ng_removed){var e=u5(this.C,d);e.size?e.forEach(f=>j5(f,d,b,!1,!0)):i5(this,d)}});v5(this.C,()=>c.forEach(d=>i5(this,d)))}removeNode(a,b){const c=this.C;a.childElementCount&&this.O(a,b);if(!j5(this,a,b,!0)){var d=!1;if(c.Ja){var e=c.ca.length?c.T.get(a):[];if(e&&e.length)d=!0;else for(e=a;e=e.parentNode;)if(c.C.get(e)){d=!0;break}}m5(this,
a);d?k5(c,this.id,a,!1,b):(d=a.__ng_removed,d&&d!==d5||(p5(c,()=>i5(this,a)),w5(c,a),c.wa(a,b)))}}destroy(a){this.F.forEach(b=>b.destroy());this.O(this.df,a)}}function p5(a,b){a.ua.push(b)}function u5(a,b){const c=new Set;if(b=a.C.get(b))for(let d of b.values())d.lc&&(b=a.J(d.lc))&&c.add(b);return c}function v5(a,b){a.fa.push(b)}function k5(a,b,c,d,e,f){a.D.push(c);c.__ng_removed={lc:b,hi:e,Gn:d,bs:!1,Sr:f}}
function l5(a,b){const c=b.__ng_removed;if(c&&c.hi){b.__ng_removed=d5;if(c.lc){w5(a,b);const e=a.J(c.lc);e&&i5(e,b)}a.wa(b,c.hi)}let d;(null==(d=b.classList)?0:d.contains("ng-animate-disabled"))&&x5(a,b,!1);a.F.query(b,".ng-animate-disabled",!0).forEach(e=>{x5(a,e,!1)})}function w5(a,b){let c=a.F.query(b,".ng-trigger",!0);c.forEach(d=>y5(a,d));0!=a.T.size&&(c=a.F.query(b,".ng-animating",!0),c.forEach(d=>z5(a,d)))}
function A5(a,b,c,d){if(B5(c)){var e=c.__ng_removed;e&&e.hi&&(e.hi=!1,e.ss=!0,e=a.D.indexOf(c),0<=e&&a.D.splice(e,1));b&&(b=a.J(b))&&r5(c,b.N);d&&a.G.push(c)}}function x5(a,b,c){c?a.P.has(b)||(a.P.add(b),r5(b,"ng-animate-disabled")):a.P.has(b)&&(a.P.delete(b),t5(b,"ng-animate-disabled"))}function y5(a,b){(a=a.N.get(b))&&a.forEach(c=>{c.Md?c.O=!0:c.destroy()})}function z5(a,b){(a=a.T.get(b))&&a.forEach(c=>c.finish())}
function C5(a,b,c,d){return 0<c.length?a.F.animate(b.element,c,b.duration,b.delay,b.easing,d):new fz(b.duration,b.delay)}
var K5=class{wa(a,b){this.Pa(a,b)}constructor(a,b){this.sa=a;this.F=b;this.ca=[];this.ja=new Map;this.N=new Map;this.T=new Map;this.C=new Map;this.P=new Set;this.na=this.Ja=0;this.aa={};this.O=[];this.ua=[];this.fa=[];this.Ea=new Map;this.G=[];this.D=[];this.Pa=()=>{}}Sa(a,b){const c=this.O,d=this.Ea;if(0<=c.length-1){let e=!1,f=c4(b);for(;f;){const g=d.get(f);if(g){c.splice(c.indexOf(g)+1,0,a);e=!0;break}f=c4(f)}e||c.unshift(a)}else c.push(a);d.set(b,a);return a}register(a,b){var c=this.aa[a];c||
(c=new q5(a,b,this),this.sa&&d4(this.sa,b)?this.Sa(c,b):(this.ja.set(b,c),this.G.push(b)),c=this.aa[a]=c);return c}destroy(a,b){a&&(p5(this,()=>{}),v5(this,()=>{const c=this.J(a);this.Ea.delete(c.df);const d=this.O.indexOf(c);0<=d&&this.O.splice(d,1);c.destroy(b);delete this.aa[a]}))}J(a){return this.aa[a]}Le(a,b,c,d){return B5(b)&&(a=this.J(a))?(a.Le(b,c,d),!0):!1}removeNode(a,b,c){if(B5(b)){var d=a?this.J(a):null;d?d.removeNode(b,c):k5(this,a,b,!1,c);(d=this.Ea.get(b))&&d.id!==a&&d.removeNode(b,
c)}else this.wa(b,c)}Gd(a,b,c,d,e){return B5(b)?this.J(a).Gd(b,c,d,e):()=>{}}lb(a,b,c,d,e){return a.transition.D(this.F,a.element,a.zg.value,a.me.value,c,d,a.zg.options,a.me.options,b,e)}flush(a){a=void 0===a?-1:a;let b=[];this.ja.size&&(this.ja.forEach((d,e)=>this.Sa(d,e)),this.ja.clear());if(this.Ja&&this.G.length)for(var c=0;c<this.G.length;c++)r5(this.G[c],"ng-star-inserted");if(this.O.length&&(this.na||this.D.length)){c=[];try{b=this.sb(c,a)}finally{for(a=0;a<c.length;a++)c[a]()}}else for(a=
0;a<this.D.length;a++)l5(this,this.D[a]);this.na=0;this.G.length=0;this.D.length=0;this.ua.forEach(d=>d());this.ua=[];if(this.fa.length){const d=this.fa;this.fa=[];b.length?V3(b).Kd(()=>{d.forEach(e=>e())}):d.forEach(e=>e())}}sb(a,b){const c=new s4,d=[],e=new Map,f=[],g=new Map,k=new Map,m=new Map,p=new Set;this.P.forEach(qa=>{p.add(qa);qa=this.F.query(qa,".ng-animate-queued",!0);for(let U=0;U<qa.length;U++)p.add(qa[U])});const t=this.sa;var w=Array.from(this.C.keys());const B=D5(w,this.G),G=new Map;
let J=0;B.forEach((qa,U)=>{const ea="ng-enter"+J++;G.set(U,ea);qa.forEach(na=>r5(na,ea))});const N=[],aa=new Set;var ca=new Set;for(var ia=0;ia<this.D.length;ia++){var O=this.D[ia];const qa=O.__ng_removed;qa&&qa.hi&&(N.push(O),aa.add(O),qa.Gn?this.F.query(O,".ng-star-inserted",!0).forEach(U=>aa.add(U)):ca.add(O))}const T=new Map,Q=D5(w,Array.from(aa));Q.forEach((qa,U)=>{const ea="ng-leave"+J++;T.set(U,ea);qa.forEach(na=>r5(na,ea))});a.push(()=>{B.forEach((qa,U)=>{const ea=G.get(U);qa.forEach(na=>
t5(na,ea))});Q.forEach((qa,U)=>{const ea=T.get(U);qa.forEach(na=>t5(na,ea))});N.forEach(qa=>{l5(this,qa)})});const la=[],F=[];for(a=this.O.length-1;0<=a;a--)o5(this.O[a],b).forEach(qa=>{const U=qa.Yh,ea=qa.element;la.push(U);if(this.G.length){var na=ea.__ng_removed;if(na&&na.ss){if(na.Sr&&na.Sr.has(qa.Qc)){na=na.Sr.get(qa.Qc);var Ja=this.C.get(qa.element);if(Ja&&Ja.has(qa.Qc)){var cb=Ja.get(qa.Qc);cb.value=na;Ja.set(qa.Qc,cb)}}U.destroy();return}}na=!t||!d4(t,ea);Ja=T.get(ea);cb=G.get(ea);const Pa=
this.lb(qa,c,cb,Ja,na);if(Pa.errors&&Pa.errors.length)F.push(Pa);else if(na)U.onStart(()=>I3(ea,Pa.Cn)),U.ob(()=>G3(ea,Pa.gm)),d.push(U);else if(qa.qw)U.onStart(()=>I3(ea,Pa.Cn)),U.ob(()=>G3(ea,Pa.gm)),d.push(U);else{var ub=[];Pa.sf.forEach(pa=>{pa.sE=!0;this.P.has(pa.element)||ub.push(pa)});Pa.sf=ub;c.append(ea,Pa.sf);f.push({kw:Pa,Yh:U,element:ea});Pa.CD.forEach(pa=>$3(g,pa,[]).push(U));Pa.Fj.forEach((pa,Ha)=>{if(pa.size){let Ua=k.get(Ha);Ua||k.set(Ha,Ua=new Set);pa.forEach((kb,xb)=>Ua.add(xb))}});
Pa.ai.forEach((pa,Ha)=>{let Ua=m.get(Ha);Ua||m.set(Ha,Ua=new Set);pa.forEach((kb,xb)=>Ua.add(xb))})}});if(F.length){const qa=[];F.forEach(()=>{qa.push(new Zd(3505,!1))});la.forEach(U=>U.destroy());throw new Zd(3402,!1);}const M=new Map,W=new Map;f.forEach(qa=>{const U=qa.element;c.has(U)&&(W.set(U,U),this.bb(qa.Yh.lc,qa.kw,M))});d.forEach(qa=>{const U=qa.element;this.Va(U,!1,qa.lc,qa.Qc,null).forEach(ea=>{$3(M,U,[]).push(ea);ea.destroy()})});const Aa=N.filter(qa=>E5(qa,k,m)),pb=new Map;F5(pb,ca,m).forEach(qa=>
{E5(qa,k,m)&&Aa.push(qa)});const qb=new Map;B.forEach(qa=>{F5(qb,new Set(qa),k)});Aa.forEach(qa=>{const U=pb.get(qa),ea=qb.get(qa);let na,Ja;pb.set(qa,new Map([...(null!=(na=null==U?void 0:U.entries())?na:[]),...(null!=(Ja=null==ea?void 0:ea.entries())?Ja:[])]))});const Ia=[],Vb=[],Ma={};f.forEach(qa=>{const U=qa.element,ea=qa.Yh,na=qa.kw;if(c.has(U))if(p.has(U))ea.ob(()=>G3(U,na.gm)),ea.disabled=!0,ea.totalTime=na.totalTime,d.push(ea);else{var Ja=Ma;if(1<W.size){qa=U;const cb=[];for(;qa=qa.parentNode;){const Pa=
W.get(qa);if(Pa){Ja=Pa;break}cb.push(qa)}cb.forEach(Pa=>W.set(Pa,Ja))}qa=this.gb(ea.lc,na,M,e,qb,pb);G5(ea,qa);Ja===Ma?Ia.push(ea):((qa=this.N.get(Ja))&&qa.length&&(ea.F=V3(qa)),d.push(ea))}else I3(U,na.Cn),ea.ob(()=>G3(U,na.gm)),Vb.push(ea),p.has(U)&&d.push(ea)});Vb.forEach(qa=>{var U=e.get(qa.element);U&&U.length&&(U=V3(U),G5(qa,U))});d.forEach(qa=>{qa.F?H5(qa,qa.F):qa.destroy()});for(b=0;b<N.length;b++)if(ca=N[b],a=ca.__ng_removed,t5(ca,"ng-leave"),!a||!a.Gn){a=[];if(g.size)for((w=g.get(ca))&&
w.length&&a.push(...w),w=this.F.query(ca,".ng-animating",!0),ia=0;ia<w.length;ia++)(O=g.get(w[ia]))&&O.length&&a.push(...O);a=a.filter(qa=>!qa.C);a.length?I5(this,ca,a):l5(this,ca)}N.length=0;Ia.forEach(qa=>{this.ca.push(qa);qa.Kd(()=>{qa.destroy();this.ca.splice(this.ca.indexOf(qa),1)});qa.play()});return Ia}Va(a,b,c,d,e){let f=[];if(b)(e=this.T.get(a))&&(f=e);else if(a=this.N.get(a)){const g=!e||"void"==e;a.forEach(k=>{k.Md||(g||k.Qc==d)&&f.push(k)})}if(c||d)f=f.filter(g=>c&&c!=g.lc||d&&d!=g.Qc?
!1:!0);return f}bb(a,b,c){var d=b.Qc;const e=b.element;a=b.ww?void 0:a;d=b.ww?void 0:d;for(const f of b.sf){const g=f.element,k=g!==e,m=$3(c,g,[]);this.Va(g,k,a,d,b.me).forEach(p=>{const t=p.D;t.Jk&&t.Jk();p.destroy();m.push(p)})}I3(e,b.Cn)}gb(a,b,c,d,e,f){const g=b.Qc,k=b.element,m=[],p=new Set,t=new Set,w=b.sf.map(G=>{const J=G.element;p.add(J);var N=J.__ng_removed;if(N&&N.bs)return new fz(G.duration,G.delay);N=J!==k;var aa=J5((c.get(J)||c5).map(O=>O.D)).filter(O=>O.element?O.element===J:!1),ca=
e.get(J);const ia=f.get(J);ca=W3(G.Bg,ca,ia);aa=C5(this,G,ca,aa);G.tE&&d&&t.add(J);N&&(G=new n5(a,g,J),G5(G,aa),m.push(G));return aa});m.forEach(G=>{$3(this.T,G.element,[]).push(G);G.Kd(()=>{var J=this.T,N=G.element;let aa=J.get(N);aa&&(aa.length&&aa.splice(aa.indexOf(G),1),0==aa.length&&J.delete(N));return aa})});p.forEach(G=>r5(G,"ng-animating"));const B=V3(w);B.ob(()=>{p.forEach(G=>t5(G,"ng-animating"));G3(k,b.gm)});t.forEach(G=>{$3(d,G,[]).push(B)});return B}};
function G5(a,b){a.N||(a.D=b,a.J.forEach((c,d)=>{c.forEach(e=>X3(b,d,void 0,e))}),a.J.clear(),a.N=!0,a.totalTime=b.totalTime,a.Md=!1)}function H5(a,b){const c=a.D;if(c.oi)b.onStart(()=>c.oi("start"));b.Kd(()=>a.finish());b.ob(()=>a.destroy())}
var n5=class{constructor(a,b,c){this.lc=a;this.Qc=b;this.element=c;this.D=new fz;this.N=!1;this.J=new Map;this.C=!1;this.F=null;this.disabled=this.O=!1;this.Md=!0;this.totalTime=0}G(a,b){$3(this.J,a,[]).push(b)}Kd(a){this.Md&&this.G("done",a);this.D.Kd(a)}onStart(a){this.Md&&this.G("start",a);this.D.onStart(a)}ob(a){this.Md&&this.G("destroy",a);this.D.ob(a)}init(){this.D.init()}Qf(){return this.Md?!1:this.D.Qf()}play(){!this.Md&&this.D.play()}pause(){!this.Md&&this.D.pause()}le(){!this.Md&&this.D.le()}finish(){this.D.finish()}destroy(){this.C=
!0;this.D.destroy()}reset(){!this.Md&&this.D.reset()}setPosition(a){this.Md||this.D.setPosition(a)}oi(a){const b=this.D;b.oi&&b.oi(a)}};function B5(a){return a&&1===a.nodeType}function L5(a,b){const c=a.style.display;a.style.display=null!=b?b:"none";return c}
function F5(a,b,c){const d=[];b.forEach(g=>d.push(L5(g)));const e=[];c.forEach((g,k)=>{const m=new Map;g.forEach(p=>{const t=Q3(k,p);m.set(p,t);t&&0!=t.length||(k.__ng_removed=e5,e.push(k))});a.set(k,m)});let f=0;b.forEach(g=>L5(g,d[f++]));return e}
function D5(a,b){function c(g){if(!g)return 1;let k=f.get(g);if(k)return k;const m=g.parentNode;d.has(m)?k=m:e.has(m)?k=1:k=c(m);f.set(g,k);return k}const d=new Map;a.forEach(g=>d.set(g,[]));if(0==b.length)return d;const e=new Set(b),f=new Map;b.forEach(g=>{const k=c(g);1!==k&&d.get(k).push(g)});return d}function r5(a,b){let c;null==(c=a.classList)||c.add(b)}function t5(a,b){let c;null==(c=a.classList)||c.remove(b)}function I5(a,b,c){V3(c).Kd(()=>l5(a,b))}
function J5(a){const b=[];M5(a,b);return b}function M5(a,b){for(let c=0;c<a.length;c++){const d=a[c];d instanceof jz?M5(d.C,b):b.push(d)}}function s5(a,b){const c=Object.keys(a);var d=Object.keys(b);if(c.length!=d.length)return!1;for(d=0;d<c.length;d++){const e=c[d];if(!b.hasOwnProperty(e)||a[e]!==b[e])return!1}return!0}function E5(a,b,c){const d=c.get(a);if(!d)return!1;let e=b.get(a);e?d.forEach(f=>e.add(f)):b.set(a,d);c.delete(a);return!0};function N5(a,b){v5(a.C,b)}
var O5=class{constructor(a,b){this.G={};this.D=()=>{};this.C=new K5(a.body,b);this.F=new b5(b);this.C.Pa=(c,d)=>{this.D(c,d)}}register(a,b){this.C.register(a,b)}destroy(a,b){this.C.destroy(a,b)}process(a,b,c,d){if("@"==c.charAt(0)){const [f,g]=a4(c);a=this.F;c=f;var e=g;if("register"==e)a.register(c,d[0]);else if("create"==e)a.create(c,b,d[0]||{});else switch(b=a.D(c),e){case "play":b.play();break;case "pause":b.pause();break;case "reset":b.reset();break;case "restart":b.le();break;case "finish":b.finish();
break;case "init":b.init();break;case "setPosition":b.setPosition(parseFloat(d[0]));break;case "destroy":a.destroy(c)}}else this.C.Le(a,b,c,d)}Gd(a,b,c,d,e){if("@"==c.charAt(0)){const [f,g]=a4(c);return this.F.Gd(f,b,g,e)}return this.C.Gd(a,b,c,d,e)}flush(a=-1){this.C.flush(a)}};function P5(a,b){let c=null,d=null;Array.isArray(b)&&b.length?(c=Q5(b[0]),1<b.length&&(d=Q5(b[b.length-1]))):b instanceof Map&&(c=Q5(b));return c||d?new R5(a,c,d):null}
var R5=class{constructor(a,b,c){this.C=a;this.F=b;this.D=c;this.pc=0;let d=S5.get(a);d||S5.set(a,d=new Map);this.G=d}start(){1>this.pc&&(this.F&&G3(this.C,this.F,this.G),this.pc=1)}finish(){this.start();2>this.pc&&(G3(this.C,this.G),this.D&&(G3(this.C,this.D),this.D=null),this.pc=1)}destroy(){this.finish();3>this.pc&&(S5.delete(this.C),this.F&&(I3(this.C,this.F),this.D=null),this.D&&(I3(this.C,this.D),this.D=null),G3(this.C,this.G),this.pc=3)}},S5=new WeakMap;
function Q5(a){let b=null;a.forEach((c,d)=>{if("display"===d||"position"===d)b=b||new Map,b.set(d,c)});return b};function T5(a){a.N||(a.N=!0,a.G.forEach(b=>b()),a.G=[])}function U5(a){if(!a.ca){a.ca=!0;var b=a.Bg;a.C=a.Ea(a.element,b,a.options);a.ua=b.length?b[b.length-1]:new Map;var c=()=>T5(a);a.C.addEventListener("finish",c);a.ob(()=>{a.C.removeEventListener("finish",c)})}}
var V5=class{constructor(a,b,c,d){this.element=a;this.Bg=b;this.options=c;this.D=d;this.G=[];this.J=[];this.O=[];this.Ga=this.P=this.N=this.ca=!1;this.fa=[];this.ja=[];this.time=0;this.F=null;this.Dh=new Map;this.na=c.duration;this.T=c.delay||0;this.time=this.na+this.T}init(){U5(this);this.wa()}wa(){this.T?this.aa():this.C.pause()}sa(a){const b=[];a.forEach(c=>{b.push(Object.fromEntries(c))});return b}Ea(a,b,c){return a.animate(this.sa(b),c)}onStart(a){this.ja.push(a);this.J.push(a)}Kd(a){this.fa.push(a);
this.G.push(a)}ob(a){this.O.push(a)}play(){U5(this);this.Qf()||(this.J.forEach(a=>a()),this.J=[],this.P=!0,this.D&&this.D.start());this.C.play()}pause(){this.init();this.C.pause()}finish(){this.init();this.D&&this.D.finish();T5(this);this.C.finish()}reset(){this.aa();this.P=this.N=this.Ga=!1;this.J=this.ja;this.G=this.fa}aa(){this.C&&this.C.cancel()}le(){this.reset();this.play()}Qf(){return this.P}destroy(){this.Ga||(this.Ga=!0,this.aa(),T5(this),this.D&&this.D.destroy(),this.O.forEach(a=>a()),this.O=
[])}setPosition(a){void 0===this.C&&this.init();this.C.currentTime=a*this.time}get totalTime(){return this.T+this.na}Jk(){const a=new Map;this.Qf()&&this.ua.forEach((b,c)=>{"offset"!==c&&a.set(c,this.N?b:Q3(this.element,c))});this.Dh=a}oi(a){a="start"===a?this.J:this.G;a.forEach(b=>b());a.length=0}};var W5=class{query(a,b,c){a=c?Array.from(a.querySelectorAll(b)):(a=a.querySelector(b))?[a]:[];return a}animate(a,b,c,d,e,f=[]){const g={duration:c,delay:d,fill:0==d?"both":"forwards"};e&&(g.easing=e);const k=new Map;e=f.filter(p=>p instanceof V5);0!==c&&0!==d||e.forEach(p=>{p.Dh.forEach((t,w)=>k.set(w,t))});let m=F3(b).map(p=>new Map(p));m=P3(a,m,k);b=P5(a,m);return new V5(a,m,g,b)}};var X5=class{constructor(a,b,c,d){this.lc=a;this.rb=b;this.C=c;this.D=d}get data(){return this.rb.data}Xk(a){let b,c;null==(c=(b=this.rb).Xk)||c.call(b,a)}destroy(){this.C.destroy(this.lc,this.rb);N5(this.C,()=>{queueMicrotask(()=>{this.rb.destroy()})});let a;null==(a=this.D)||a.call(this)}createElement(a,b){return this.rb.createElement(a,b)}createComment(a){return this.rb.createComment(a)}lq(a){return this.rb.lq(a)}appendChild(a,b){this.rb.appendChild(a,b);A5(this.C.C,this.lc,b,!1)}insertBefore(a,
b,c,d=!0){this.rb.insertBefore(a,b,c);A5(this.C.C,this.lc,b,d)}removeChild(a,b){this.C.C.removeNode(this.lc,b,this.rb)}ns(a,b){return this.rb.ns(a,b)}parentNode(a){return this.rb.parentNode(a)}nextSibling(a){return this.rb.nextSibling(a)}setAttribute(a,b,c,d){this.rb.setAttribute(a,b,c,d)}removeAttribute(a,b,c){this.rb.removeAttribute(a,b,c)}Ri(a,b){this.rb.Ri(a,b)}wo(a,b){this.rb.wo(a,b)}setStyle(a,b,c,d){this.rb.setStyle(a,b,c,d)}Zr(a,b,c){this.rb.Zr(a,b,c)}setProperty(a,b,c){"@"==b.charAt(0)&&
"@.disabled"==b?x5(this.C.C,a,!!c):this.rb.setProperty(a,b,c)}setValue(a,b){this.rb.setValue(a,b)}Gd(a,b,c){return this.rb.Gd(a,b,c)}},a6=class extends X5{constructor(a,b,c,d){super(b,c,d);this.ba=a;this.lc=b}setProperty(a,b,c){"@"==b.charAt(0)?"."==b.charAt(1)&&"@.disabled"==b?x5(this.C.C,a,void 0===c?!0:!!c):this.C.process(this.lc,a,b.slice(1),c):this.rb.setProperty(a,b,c)}Gd(a,b,c){if("@"==b.charAt(0)){a=Y5(a);b=b.slice(1);let d="";"@"!=b.charAt(0)&&([b,d]=Z5(b));return this.C.Gd(this.lc,a,b,d,
e=>{$5(this.ba,e._data||-1,c,e)})}return this.rb.Gd(a,b,c)}};function Y5(a){switch(a){case "body":return document.body;case "document":return document;case "window":return window;default:return a}}function Z5(a){const b=a.indexOf(".");return[a.substring(0,b),a.slice(b+1)]};function $5(a,b,c,d){if(0<=b&&b<a.F)a.yb.run(()=>c(d));else{var e=a.G;0==e.length&&queueMicrotask(()=>{a.yb.run(()=>{e.forEach(f=>{const [g,k]=f;g(k)});a.G=[]})});e.push([c,d])}}
var b6=class{constructor(a,b,c){this.rb=a;this.C=b;this.yb=c;this.J=0;this.F=1;this.G=[];this.N=new Map;this.D=0;b.D=(d,e)=>{const f=null==e?void 0:e.parentNode(d);f&&e.removeChild(f,d)}}Uk(a,b){const c=this.rb.Uk(a,b);let d;if(!a||!(null==b?0:null==(d=b.data)?0:d.animation)){const k=this.N;a=k.get(c);a||(a=new X5("",c,this.C,()=>k.delete(c)),k.set(c,a));return a}const e=b.id,f=b.id+"-"+this.J;this.J++;this.C.register(f,a);const g=k=>{if(Array.isArray(k))k.forEach(g);else{var m=this.C,p=k.name,t=
e+"-"+p,w=m.G[t];if(!w){w=[];k=(new q4).D(k,w,[]);if(w.length)throw new Zd(3404,!1);w=new $4(p,k);m.G[t]=w}m=m.C;(t=m.aa[f])&&t.register(p,w)&&m.Ja++}};b.data.animation.forEach(g);return new a6(this,f,c,this.C)}Vm(){this.D++;this.rb.Vm&&this.rb.Vm()}O(){queueMicrotask(()=>{this.F++})}end(){this.D--;0==this.D&&Mk(this.yb,()=>{this.O();this.C.flush(this.F)});this.rb.end&&this.rb.end()}};var c6=class extends O5{constructor(a,b,c){super(a,b,c)}va(){this.flush()}};c6.K=function(a){return new (a||c6)(r(jt),r(e4),r(A3))};c6.pa=Md({ma:c6,ba:c6.K});const d6=[{oa:A3,Fc:function(){return new U3}},{oa:O5,Ec:c6},{oa:ui,Fc:function(a,b,c){return new b6(a,b,c)},Ac:[eH,O5,Qk]}];var e6=[{oa:e4,Fc:()=>new W5},{oa:mi,nb:"BrowserAnimations"},...d6];[...d6];var f6=class{};f6.K=function(a){return new (a||f6)};f6.Ya=Of({type:f6});f6.Xa=Nd({Wa:e6,imports:[TH]});var g6={Wa:[Yf([f6]),function(a,...b){return{Zj:[{oa:P0,Eb:!0,nb:a},[],{oa:d_,Fc:C1,Ac:[A1]},{oa:Zo,Eb:!0,Fc:D1},b.map(c=>c.Zj)]}}([]),{oa:"ModelLoaderService",Ec:z3}]},h6=Object,i6=h6.assign,j6,k6={Wa:[(null==g6?void 0:g6.Wa)||[],SH]};let l6;j6={XA:[...tH,...(null!=(l6=null==k6?void 0:k6.Wa)?l6:[])],wD:qH};
(function(a){try{const b=a.LD,c=a.XA,d=ys(a.wD),e=[ks(),...(c||[])],f=(new qn({Wa:e,parent:d,yv:"",zx:!1})).xa,g=f.get(Qk);return g.run(()=>{pg(f);const k=f.get(wi,null);let m;Mk(g,()=>{m=g.onError.subscribe({next:w=>{k.handleError(w)}})});const p=()=>f.destroy(),t=d.get(qs);t.add(p);f.ob(()=>{m.unsubscribe();t.delete(p)});return ap(k,g,()=>{const w=f.get(vn);un(w);return w.F.then(()=>{f.get(os,"en-US");const B=f.get(fp);void 0!==b&&B.Kk(b);return B})})})}catch(b){return Promise.reject(b)}})(i6.call(h6,
{},{LD:s3},j6)).catch(a=>{console.error(a)});
